#include <iomanip>
#include <iostream>
#include <list>
#include <stdio.h>
#include <fstream>


#include "MCS_FileLocations.h"
#include "MCS_GeographicCoordinates.h"
#include "MCS_MSULevel0.h"
#include "MCS_MSULevel1A.h"
#include "MCS_Satellite.h"
#include "MCS_StringExtensions.h"
#include "MCS_Triple.h"

#define APPEND_MODE 1
#define INV_OUTPUT 1
#define SWATH_OUTPUT 1

int main()
{
	using namespace mcs;
	using namespace mcs::geo;
	using namespace mcs::sat;

	typedef Dataset< MSU<SwathData,msu::Level0,msu::v2> >				Level0DatasetType;
	
//	typedef Dataset< MSU<SwathData,msu::Level1A<msu::Linear>,msu::v2> >	Level1ADatasetType;
	typedef Dataset< MSU<SwathData,msu::Level1A<msu::Mo1995>,msu::v2> >	Level1ADatasetType;
//	typedef Dataset< MSU<SwathData,msu::Level1A<msu::Mo2001>,msu::v2> >	Level1ADatasetType;
		
	Int32	sat_start,sat_end;
		
	sat_start 	= 9;
	sat_end 	= 9;
	
	
	if (sat_start < 1 || sat_end > 9 || sat_end < sat_start) return -1;
	
	for (Int32 i=sat_start;i<=sat_end;++i)
	{
		std::string	sat_name;
		
		switch (i)
		{
			case	1	:	sat_name = "TIROS-N"; break;
			case	2	:	sat_name = "NOAA-06"; break;
			case	3	:	sat_name = "NOAA-07"; break;
			case	4	:	sat_name = "NOAA-08"; break;
			case	5	:	sat_name = "NOAA-09"; break;
			case	6	:	sat_name = "NOAA-10"; break;
			case	7	:	sat_name = "NOAA-11"; break;
			case	8	:	sat_name = "NOAA-12"; break;
			case	9	:	sat_name = "NOAA-14"; break;
		}
		
		SatelliteID			sat_ID(sat_name);
		
		Level0DatasetType	ds(sat_ID);
		Level1ADatasetType	ds_out(sat_ID);
		
		std::ofstream	inv;
		
		
		
		
				
		#if INV_OUTPUT
		const std::string	directory = FileLocations["MSU Level 1A"]+sat_name+"\\"+Level1ADatasetType::CalibrationType::tag()+"\\";
		//ifs_inv.close();	
		inv.open((directory+sat_name+"_L1A.inv").c_str(),std::ios::binary | std::ios::out | std::ios::app);	
		inv.seekp(-28,std::ios::end);
		#endif // INV_OUTPUT
		
		OrbitIterator<Level0DatasetType>	orbdat(ds);
		
		typedef std::list< Swath<Level1ADatasetType> >	SwathStackType;
		
		// Open a file that contains the orbit numbers to process
		
		const std::string	lev0_directory = FileLocations["MSU Level 0"]+sat_name+"\\";
		const std::string   rev_list_file = lev0_directory+"Channel_02_revs_to_process.txt";

		FILE *rev_list_f;
		
		if ((rev_list_f = fopen((rev_list_file).c_str(),"r")) == NULL){
			std::cout << "can't open rev list file";
			std::exit(1);
			}
		 
		Int32 orbit_n;
		
 		// Loop until no orbit number is present
 		
		while(fscanf(rev_list_f,"%d",&orbit_n) == 1)
		{
			Float32	min_ht = _plus_Inf_,
					max_ht = _minus_Inf_;
					
			FixedVector<Float32,4>	TA_sum(0.0),
									TA_wt(0.0);
			
			SwathStackType	swath_data;
					
					
			// set the orbit number to orbit_n	
			OrbitIterator<Level0DatasetType>::iterator it=orbdat.number(orbit_n);
			std::cout << " Processing Orbit " << it().value() << std::endl;

			try
			{
				const Orbit<Level0DatasetType>&	orb(*it);

				for (Orbit<Level0DatasetType>::iterator sit=orb.begin();sit!=orb.end();++sit)
				{
					try
					{
						if (!sit->passedQC()) continue;
						
						//std::cout << sit->T_DickeLoad() << std::endl;
						
						//std::cout << "Read swath " << sit->swathNumber() << std::endl;
						
						// remove bad days in NOAA-06
						static const SatelliteID	NOAA_06("NOAA-06");
						
						if (sat_ID == NOAA_06)
							if (sit->epoch().day() == Day(1980,3,3) || sit->epoch().day() == Day(1980,3,4)) continue;
							
						const Date	ep = sit->epoch();
						
						Swath<Level1ADatasetType>	sw_L1A(*sit);
						
						if (!sw_L1A.passedQC()) continue;
						
						if (sw_L1A.height() < min_ht) min_ht = sw_L1A.height();
						if (sw_L1A.height() > max_ht) max_ht = sw_L1A.height();
						
						for (Int32 ch=msu::CH_1;ch<=msu::CH_4;++ch)
						{
							if (isfinite(sw_L1A.TA(ch,msu::SUBTRACK)))
							{
								TA_sum[ch-1] += sw_L1A.TA(ch,msu::SUBTRACK);
								TA_wt[ch-1] += 1;
							}
						}
						
						// set NOAA-09 Ch 2 data to NaN after data gets bad
						static const SatelliteID	NOAA_09("NOAA-09");
						
						if (sat_ID == NOAA_09)
							if (sw_L1A.epoch().day() > Day(1987,3,8)) 
								for (Index<1> fovidx=sw_L1A.TA().first(1);fovidx<=sw_L1A.TA().last(1);++fovidx)
									sw_L1A.TA(msu::CH_2,fovidx,_NaN_);
						
						swath_data.push_back(sw_L1A);
					}
					catch (Orbit<Level0DatasetType>::BadSwathErr& ex)
					{
						continue;
					}
					catch (Orbit<Level0DatasetType>::MissingSwathErr& ex)
					{
						continue;
					}
				}
				
				if (swath_data.size() <= 0) throw Orbit<Level0DatasetType>::BadOrbitErr();
			}
			catch (Orbit<Level0DatasetType>::BadOrbitErr& ex)
			{
				std::cerr << "Orbit " << it().value() << " bad..." << ex.message() << std::endl;
				continue;
			}
			catch (Orbit<Level0DatasetType>::MissingOrbitErr& ex)
			{
				std::cerr << "Orbit " << it().value() << " missing..." << ex.message() << std::endl;
				continue;
			}
			catch (...)
			{
				continue;
			}
			
			const Int32	orbit_num = it->orbitNumber();
					
			#if SWATH_OUTPUT || INV_OUTPUT 
			// create output stream
			std::stringstream	sstr;
			
			const Int32		hi_rev = static_cast<Int32>(std::ceil(orbit_num/5000.0))*5000,
							lo_rev = hi_rev-4999;
			
			sstr << FileLocations["MSU Level 1A"] 
				 << sat_name << "\\"
				 << Level1ADatasetType::CalibrationType::tag() << "_V02_1" << "\\"
				 << "r" << std::setw(5) << std::setfill('0') << lo_rev 
				 << "_" << std::setw(5) << std::setfill('0') << hi_rev 
				 << "\\";
				 
			static std::string	last_directory_created = "";
				 
			if (sstr.str() != last_directory_created)
			{
				//std::system(("mkdir "+sstr.str()).c_str()); 
			
				last_directory_created = sstr.str();
			}
				 
			sstr << sat_name << "_r" << std::setw(5) << std::setfill('0') << orbit_num << ".L1A";
			#endif // SWATH_OUTPUT || INV_OUTPUT

			#if SWATH_OUTPUT
			std::ofstream	ofs(sstr.str().c_str(),std::ios::binary | std::ios::out | std::ios::trunc);
			
			// write header
			ofs.write(reinterpret_cast<const char*>(sat_name.c_str()),8*sizeof(char));						// 8 bytes
			ofs.write(reinterpret_cast<const char*>(&orbit_num),sizeof(Int32));								// 4 bytes
			
			const Int32		nscan = swath_data.size();
			
			ofs.write(reinterpret_cast<const char*>(&nscan),sizeof(Int32));									// 4 bytes
			
			const Int32		swath_num_start = swath_data.begin()->swathNumber(),
							swath_num_end = swath_data.rbegin()->swathNumber();

			ofs.write(reinterpret_cast<const char*>(&swath_num_start),sizeof(Int32));						// 4 bytes
			ofs.write(reinterpret_cast<const char*>(&swath_num_end),sizeof(Int32));							// 4 bytes
			
			const Float64	rd2000_start = toRD2000(swath_data.begin()->epoch()),
							rd2000_end = toRD2000(swath_data.rbegin()->epoch());
			
			ofs.write(reinterpret_cast<const char*>(&rd2000_start),sizeof(Float64));						// 8 bytes
			ofs.write(reinterpret_cast<const char*>(&rd2000_end),sizeof(Float64));							// 8 bytes
	
			// pad header to 348 bytes
			static const Vector<Uint8>	fill(IndexRange<1>(308),Uint8(0));
			
			fill.put(ofs);
			ofs.flush();

			for (SwathStackType::const_iterator sit=swath_data.begin();sit!=swath_data.end();++sit)
				sit->write(ofs);
			
			ofs.flush();
			ofs.close();
			
			std::system(("D:\\GNU\\gzip.exe -f1 "+sstr.str()).c_str());
			#endif // SWATH_OUTPUT

			#ifdef INV_OUTPUT
			inv.write(reinterpret_cast<const char*>(&orbit_num),sizeof(Int32));
			
			// output for start and end of orbit
			for (Int32 i=1;i<=2;++i)
			{
				SwathStackType::const_iterator	sit;
				
				if (i==1)
				{
					sit = swath_data.begin();
				}
				else
				{
					sit = swath_data.end();
					--sit;
				}
					
				const Int32	swath_num = sit->swathNumber();
				
				const Date	ep = sit->epoch();
				
				const std::pair<Float32,Float32>	loc(sit->latitude(6),sit->longitude(6));
				
				std::cout << sat_name << "\t"
						  << orbit_num << "\t" 
						  << swath_num << "\t" << ep << "\t"
						  << loc.first << "\t" << loc.second << "\t"
						  << min_ht << "\t" << max_ht << "\t"
						  << TA_sum/TA_wt << std::endl;
					 
				const Float64	rd2000 = toRD2000(ep);
				
				inv.write(reinterpret_cast<const char*>(&swath_num),sizeof(Int32));
				inv.write(reinterpret_cast<const char*>(&rd2000),sizeof(Float64));
			}

			inv.flush();
			#endif // INV_OUTPUT
			
			swath_data.clear();
		}
		
		#if INV_OUTPUT
		inv.close();
		#endif // INV_OUTPUT
	}

	return 0;
}	
