#include "MCS_Debug.h"

#include "MCS_Date.h"
#include "MCS_MATLAB_Engine.h"
#include "MCS_RootFinder.h"
#include "MCS_Satellite.h"
	
using namespace std;
using namespace mcs;

// plot of satellite orbit
int main(void)
{
	string	satellite_name;
	
	cout << "Satellite? ";
	cin >> satellite_name;
	
	const Satellite	sat(toUpper(satellite_name));
	
	const Date	ep0(1999,164),
				ep1(1999,171);
	Date		ep(ep0);
	
	ep.time(Time(20,0,0));
	
	MATLAB_Engine	engine;
	
	engine.evaluate("clear;figure(1);clf;sphere(30);axis equal;hold on;");

	Int32	num = 0;
	
	const FixedVector<Float64,3>	refPos(sat.position(ep).toCartesian(ep));
	
	while (ep <= ep1)
	{
		using namespace geography;
		
		const FixedVector<Float64,3>	pos(sat.position(ep).toCartesian(ep));
		
		const Float64	dist = std::sqrt(norm(pos-refPos));
		
		static const Int32	NUM = 50;
		
		const Int32	nummod = num++ % NUM + 1;
		
		if (nummod == NUM)
			//cout << num << "\t" << ep << "\t" << pos << endl;
			cout << num << "\t" << ep << "\t" << toRD2000(ep) << "\t" << sat.location(ep) << endl;
		
		ep.addSeconds(30);
		
		engine.putValue("ep",toRD2000(ep));
		// there's a discontinuity in the plot of dist due to a jump in orbital period for TRMM...interesting
		//engine.putValue("ep",toRD2000(ep)/(sat.period(ep)/Time::secondsInDay()));
		engine.putValue("lat",sat.location(ep).latitude());
		engine.putValue("dist",dist);
		engine.putValue("ht",sat.location(ep).h());
		engine.putValue("px",pos[0]);
		engine.putValue("py",pos[1]);
		engine.putValue("pz",pos[2]);
		
		engine.putValue("num",num);
		engine.putValue("nummod",nummod);	
		engine.evaluate("t(num)=ep;");
		engine.evaluate("l(num)=lat;");
		engine.evaluate("d(num)=dist;");
		engine.evaluate("h(num)=ht;");
		engine.evaluate("p(nummod,:)=[px py pz]/6371;");
		
		if (nummod == NUM)
		{
			engine.evaluate("figure(1);plot3(p(:,1),p(:,2),p(:,3),'b.');");
			//engine.evaluate("figure(2);plot(t,d,'r.');");
			engine.evaluate("figure(2);plot(t,h,'r.');");
		}
		
		FixedVector<Float64,3>	posP;
		
		try
		{
			//const Date	epP = equatorCrossingTime(sat,ep);
			const Date	epP = crossingTime(sat,ep,55);
			
			posP = sat.position(epP).toCartesian(epP);
		}
		catch (numerics::NumericException& ex)
		{
			cout << ex.code() << "\t" << num << endl;
			
			posP = FixedVector<Float64,3>(_NaN_);
		}

		engine.putValue("ppx",posP[0]);
		engine.putValue("ppy",posP[1]);
		engine.putValue("ppz",posP[2]);
		
		engine.evaluate("pp(nummod,:)=[ppx ppy ppz]/6371;");
		
		if (nummod == NUM)
		{
			engine.evaluate("figure(1);plot3(pp(:,1),pp(:,2),pp(:,3),'k*');");
		}
	}
	
	engine.evaluate("whos");
	
	return 0;
}

/*
// plot of satellite orbit
int main(void)
{
	string		satellite_name;
	
	cout << "Satellite? ";
	cin >> satellite_name;
	
	const Satellite	sat(satellite_name);
	
	const Date	ep0(1999,164),
				ep1(1999,171);
	Date		ep(ep0);
	
	ep.time(Time(20,0,0));
	
	MATLAB_Engine	engine;
	
	engine.evaluate("figure(1);clf;sphere(30);axis equal;hold on;");

	Int32	num = 0;
	
	while (ep <= ep1)
	{
		using namespace geography;
		
		//const Inertial<Float64>			Ipos(sat.position(ep)),
		//								Ivel(sat.velocity(ep));
		//const FixedVector<Float64,3>	pos(Ipos.toCartesian(ep)),
		//								vel(Ivel.toCartesian(ep));
		const FixedVector<Float64,3>	pos(sat.position(ep).toCartesian(ep)),
										vel(sat.velocity(ep).toCartesian(ep));
										
		static const Int32	NUM = 100;
		
		const Int32	nummod = num++ % NUM + 1;
		
		if (nummod == NUM)
			cout << num << "\t" << ep << "\t" << pos << "\t" << vel << endl;
		
		ep.addSeconds(30);
		
		engine.putValue("ep",ep.toRD2000());
		engine.putValue("px",pos[0]);
		engine.putValue("py",pos[1]);
		engine.putValue("pz",pos[2]);
		engine.putValue("vx",vel[0]);
		engine.putValue("vy",vel[1]);
		engine.putValue("vz",vel[2]);
		
		engine.putValue("num",nummod);	
		engine.evaluate("t(num)=ep;");
		engine.evaluate("p(num,:)=[px py pz]/6371;");
		engine.evaluate("v(num,:)=[vx vy vz];");
		
		if (nummod == NUM)
			engine.evaluate("plot3(p(:,1),p(:,2),p(:,3),'b.');");
	}
	
	engine.evaluate("whos");
	
	return 0;
}
*/