#include <fstream>
#include <iostream>

#include "MCS_DateUtilities.h"
#include "MCS_Satellite.h"
#include "MCS_StringExtensions.h"

int main()
{	
	using namespace mcs;

	for (Int32 i=1;i<=9;++i)
	{
		std::string	sat_name;
		
		switch (i)
		{
			case	1	:	sat_name = "TIROS-N"; break;
			case	2	:	sat_name = "NOAA-06"; break;
			case	3	:	sat_name = "NOAA-07"; break;
			case	4	:	sat_name = "NOAA-08"; break;
			case	5	:	sat_name = "NOAA-09"; break;
			case	6	:	sat_name = "NOAA-10"; break;
			case	7	:	sat_name = "NOAA-11"; break;
			case	8	:	sat_name = "NOAA-12"; break;
			case	9	:	sat_name = "NOAA-14"; break;
		}
					
		Satellite	sat(toUpper(sat_name));
		
		std::ofstream	ofs((sat_name+".LECT").c_str(),std::ios::out | std::ios::trunc);
		
		const typename Satellite::TLEInventoryType&	TLE_inventory = sat.getTLEInventory();
		
		for (Date ep(TLE_inventory.begin()->epoch());ep<=TLE_inventory.rbegin()->epoch();ep.addDays(1))
		{
			try
			{
				// guess at bracketing dates
				Date	xing_ep = equatorCrossingTime(sat,ep);
					
				Int32	iter = 0;
					
				// try to get ascending crossing
				while (!sat.ascending(xing_ep))
				{
					static const Int32	MAX_ITER = 5;
					
					if (iter >= MAX_ITER)
					{
						std::cerr << "Could not get ascending node crossing...";
						std::cerr << shortYear(xing_ep.year()) << " " 
								  << xing_ep.calendarDay() << " "
								  << xing_ep.ticks()/Time::secondsInDay() << "\t"
								  << xing_ep << std::endl;
								  
						throw Exception();
					}
					
					const Float64	del = 0.5*(sat.period(xing_ep)/Time::secondsInDay()),
									dt = toRD2000(xing_ep)-toRD2000(ep) >= 0 ? -del : +del;
									
					//ep = Date(xing_ep.julian()+dt);
					xing_ep = equatorCrossingTime(sat,xing_ep+dt);

					++iter;
				}
				
				geo::Geodetic<Float64>	loc = sat.location(xing_ep);
				
				const Float64	xing_RD = toRD2000(xing_ep);
				Float32			xing_lat = loc.latitude(),
								xing_lon = loc.longitude();
								
				ofs << std::setw(4) << xing_ep.year() << "\t" 
					<< std::setw(3) << xing_ep.calendarDay() << "\t"
					<< geo::localTime(xing_ep.time(),xing_lon).ticks()/Time::secondsInDay() << "\t"
					<< std::endl;
			}
			catch (Exception& ex)
			{
				std::cerr << ex.message() << "\t" << ep << std::endl;
				std::cerr << sat.TLEbefore(ep) << std::endl;
				std::cerr << sat.TLEafter(ep) << std::endl;
				continue;
			}
			catch (...)
			{
				std::cerr << "Unknown exception" << std::endl;
				continue;
			}
		}

		ofs.close();
	}
	
	return 0;
}
