#include "MCS_Debug.h"

#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <iomanip>
#include <iostream>
#include <sstream>
#include <string>

#include "MCS_Constants.h"
#include "MCS_Date.h"
#include "MCS_OrbitalPropagator.h"
#include "MCS_Typedefs.h"

using namespace std;
using namespace mcs;
using namespace geography;

// performance test the SGP/SGP4/SDP4 algorithms
int main(void)
{
	//const bool	performance_test = true;
	//const Int32	num_steps = 1000000;
	const bool	performance_test = false;
	const Int32	num_steps = 20;
	
	cout.setf(ios::scientific,ios::floatfield);
	cout.width(18);
	cout.precision(12);
	
	//! READ IN MEAN ELEMENTS FROM 2 CARD T(TRANS) OR G(INTERN) FORMAT
	// for testing SGP/SGP4/SGP8
	//const Date		start_date(Day(1980,275));
	//const string	line1 = "1 88888U 98030A   80275.98708465  .00073094  13844-3  66816-4 0     8",
	//				line2 = "2 88888  72.8435 115.9689 0086731  52.6988 110.5714 16.05824518   105";

	// for testing SDP4/SDP8
	//const Date		start_date(Day(1997,1,17),Time(14,40,0));
	//const string	line1 = "1 19220U 88051F   97079.11485035 +.00000856 +00000-0 +43713-3 0 07948",
	//				line2 = "2 19220 009.9715 304.0187 5734345 315.9233 010.5502 04.37227961086454";		

	const Date		start_date(Day(1995,1,10),Time(21,40));
	const string	line1 = "1 23243U 92021N   95016.23792613 +.00000374 +00000-0 +34706-2 0 00450",
					line2 = "2 23243 003.2770 358.1095 7290859 272.4086 014.1114 02.14568415009855";

	const TLE	tle(line1,line2);
	
	//OrbitalPropagator<KeplerModel>		orbit(tle);
	//OrbitalPropagator<SGPModel>			orbit(tle);
	OrbitalPropagator<SGP4SDP4Model>	orbit(tle);
	
	const Float64	t_start = (start_date.julian()-tle.epoch().julian())*Time::secondsInDay(),
					t_finish = t_start+10*60,
					t_step = (t_finish-t_start)/num_steps;
					
	//! PERIOD GE 225 MINUTES IS DEEP SPACE
	if (orbit.period() >= 225*Time::secondsInMinute())
		cout << "SHOULD USE DEEP SPACE EPHEMERIS" << endl;
	else		
		cout << "SHOULD USE NEAR EARTH EPHEMERIS" << endl;

	Float64	dt = t_start;
	Int32	counter = 0;
	
	const Float64	start_sec = Time::now().ticks();
	
	while (dt <= t_finish)
	{
		orbit.propagate(dt);
		
		if (!performance_test || counter % 50000 == 0)
		{
			const Date	curr_epoch(tle.epoch().julian()+dt/Time::secondsInDay());

			const Geodetic<Float64>	obs(38.43662,toEWLongitude(-122.711913),0);

			cout << setw(9) << setprecision(4)
				 << curr_epoch << "\t"
			//cout << orbit.position() << "\t" << orbit.velocity() << "\t";
				 << orbit.location() << "\t"
				 << look(curr_epoch,obs,orbit.location()) << "\t"
				 << endl;
		}
				
		dt += t_step;
		++counter;
	}
	
	const Float64	end_sec = Time::now().ticks();
	
	cout << endl
		 << "Completed " << counter << " iterations in " << end_sec-start_sec << " seconds "
		 << "at a rate of " << counter/(end_sec-start_sec) << " iterations/s"
		 << endl;
		 
	return 0;
}

/*
	SDP4 performance with level 2 optimization for PIII, with inline

SHOULD USE DEEP SPACE EPHEMERIS
   Friday January 17, 1997 14:39:6.0000e+01     alt: 8.8070e+03 lon: 1.4050e+01W lat: 6.0833e+00S
   Friday January 17, 1997 14:40:1.5000e+01     alt: 8.8517e+03 lon: 1.3872e+01W lat: 6.0495e+00S
   Friday January 17, 1997 14:40:3.0000e+01     alt: 8.8964e+03 lon: 1.3696e+01W lat: 6.0159e+00S
   Friday January 17, 1997 14:40:4.5000e+01     alt: 8.9410e+03 lon: 1.3521e+01W lat: 5.9823e+00S
   Friday January 17, 1997 14:41:6.7949e-05     alt: 8.9854e+03 lon: 1.3347e+01W lat: 5.9488e+00S
   Friday January 17, 1997 14:41:1.5000e+01     alt: 9.0297e+03 lon: 1.3175e+01W lat: 5.9154e+00S
   Friday January 17, 1997 14:41:3.0000e+01     alt: 9.0739e+03 lon: 1.3004e+01W lat: 5.8821e+00S
   Friday January 17, 1997 14:41:4.5000e+01     alt: 9.1180e+03 lon: 1.2834e+01W lat: 5.8489e+00S
   Friday January 17, 1997 14:42:1.4484e-04     alt: 9.1620e+03 lon: 1.2666e+01W lat: 5.8157e+00S
   Friday January 17, 1997 14:42:1.5000e+01     alt: 9.2058e+03 lon: 1.2499e+01W lat: 5.7827e+00S
   Friday January 17, 1997 14:42:3.0000e+01     alt: 9.2495e+03 lon: 1.2333e+01W lat: 5.7498e+00S
   Friday January 17, 1997 14:42:4.5000e+01     alt: 9.2931e+03 lon: 1.2169e+01W lat: 5.7169e+00S
   Friday January 17, 1997 14:43:2.6196e-04     alt: 9.3366e+03 lon: 1.2006e+01W lat: 5.6842e+00S
   Friday January 17, 1997 14:43:1.5000e+01     alt: 9.3800e+03 lon: 1.1844e+01W lat: 5.6515e+00S
   Friday January 17, 1997 14:43:3.0000e+01     alt: 9.4232e+03 lon: 1.1684e+01W lat: 5.6190e+00S
   Friday January 17, 1997 14:43:4.5000e+01     alt: 9.4664e+03 lon: 1.1524e+01W lat: 5.5865e+00S
   Friday January 17, 1997 14:44:3.3885e-04     alt: 9.5094e+03 lon: 1.1366e+01W lat: 5.5542e+00S
   Friday January 17, 1997 14:44:1.5000e+01     alt: 9.5523e+03 lon: 1.1210e+01W lat: 5.5219e+00S
   Friday January 17, 1997 14:44:3.0000e+01     alt: 9.5951e+03 lon: 1.1054e+01W lat: 5.4897e+00S
   Friday January 17, 1997 14:44:4.5000e+01     alt: 9.6377e+03 lon: 1.0899e+01W lat: 5.4576e+00S
   Friday January 17, 1997 14:45:4.1574e-04     alt: 9.6803e+03 lon: 1.0746e+01W lat: 5.4256e+00S
   Friday January 17, 1997 14:45:1.5000e+01     alt: 9.7227e+03 lon: 1.0594e+01W lat: 5.3937e+00S
   Friday January 17, 1997 14:45:3.0000e+01     alt: 9.7650e+03 lon: 1.0443e+01W lat: 5.3619e+00S
   Friday January 17, 1997 14:45:4.5000e+01     alt: 9.8072e+03 lon: 1.0293e+01W lat: 5.3302e+00S

Completed 1199999 iterations in 1.0000e+01 seconds at a rate of 1.2000e+05 iterations/s

2         0      1440       360
1 88888U 98030A   80275.98708465  .00073094  13844-3  66816-4 0     8
2 88888  72.8435 115.9689 0086731  52.6988 110.5714 16.05824518   105
*/
