#include "MCS_AtmosphericProfile.h"
#include "MCS_BoundaryConditions.h"
#include "MCS_Field.h"
#include "MCS_Grid.h"
#include "MCS_Profile.h"
#include "MCS_Satellite.h"
#include "MCS_Sequence.h"
#include "MCS_StandardAtmosphere.h"
#include "MCS_TLE.h"
#include "MCS_WaterVapor.h"

#include "MCS_MATLAB_Engine.h"

#include "MCS_BinaryFormatter.h"
#include "MCS_TextFormatter.h"
#include "MCS_XMLFormatter.h"

using namespace mcs;

int main(void)
{
	Float32	dat[] = { 1.0, 2.0, 3.0, 4.0, 5.0 };
	
	db::XMLFormatter::Writer	wrt(std::cout);
	
	wrt.openElement("Stuff");
	
	wrt.attribute("chunky",7);
	wrt.attribute("creamy",3.4);
	
	wrt.openElement("PB&J");
	wrt.closeElement();
	
	wrt.openElement("Chippy");
	
	wrt.binary(true);
	wrt.data(dat,5);
	wrt.binary(false);
	
	wrt.closeElement();
	
	wrt.closeElement();
	/*
	db::XMLReader	rdr(std::cin);
	
	Int32	a1;
	Float32	a2;
	Float32	v1[5];
	
	rdr.openElement("Stuff");
	
	rdr.readAttribute("chunky",a1);
	rdr.readAttribute("creamy",a2);
	
	rdr.openElement("PB&J");
	rdr.closeElement();
	
	rdr.openElement("Chippy");
	
	rdr.binary(true);
	rdr.readData(v1,5);
	rdr.binary(false);
	
	rdr.closeElement();
	
	rdr.closeElement();
	*/
	
	db::XMLFormatter	fmt;
	
	MATLAB_Engine	engine;
	
	try
	{
		const StandardAtmosphere<Float64>::vector_type::range_type	rng(101);
		
		StandardAtmosphere<Float64>	atm(rng,0,40000),
									atm2(rng,41000,100000);
									
		atm.merge(atm2,"pressure",std::greater<Float64>());
		
		atm.insert("cloud",randomCloudLayer(atm,0.3));
		
		atm.insert("hypsometric height",hypsometricHeight(atm));
		atm.insert("hypsometric pressure",hypsometricPressure(atm));
		
		atm.serialize(fmt,std::cout);
		
		const Float64	lapse = lapseRate(atm);
		
		engine.put("T",atm["temperature"]);
		engine.put("P",atm["pressure"]);
		engine.put("Z",atm["height"]);
		engine.put("C",atm["cloud"]);
		engine.put("PV",atm["vapor pressure"]);
		engine.put("RH",atm["relative humidity"]);
		engine.put("hZ",atm["hypsometric height"]);
		engine.put("hP",atm["hypsometric pressure"]);
		
		Satellite	sat("NOAA-14");
		
		std::cout << "Done testing" << std::endl;
	}
	catch (Exception& ex)
	{
		std::cout << ex.message() << std::endl;
	}
	catch (...)
	{
		std::cout << "Unknown exception..." << std::endl;
	}
	
	return 0;
}