#ifndef MCS_BOUNDARYCONDITIONS_H
#define MCS_BOUNDARYCONDITIONS_H

#include "MCS_Exceptions.h"
#include "MCS_IndexRange.h"
#include "MCS_Index.h"

namespace mcs {

// fixed boundary conditions
template<Uint8 D> 
class FixedBoundary
{ 
	public:
		static Index<D> mapIndex(const Index<D>& idx,const IndexRange<D>& rng);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		FixedBoundary<D>();
		FixedBoundary<D>(const FixedBoundary<D>&);
		FixedBoundary<D>& operator=(const FixedBoundary<D>&);
};

// periodic boundary conditions
template<Uint8 D> 
class PeriodicBoundary
{ 
	public:
		static Index<D> mapIndex(const Index<D>& idx,const IndexRange<D>& rng);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		PeriodicBoundary<D>();
		PeriodicBoundary<D>(const PeriodicBoundary<D>&);
		PeriodicBoundary<D>& operator=(const PeriodicBoundary<D>&);
};

// even mirror boundary conditions
template<Uint8 D> 
class EvenMirrorBoundary
{ 
	public:
		static Index<D> mapIndex(const Index<D>& idx,const IndexRange<D>& rng);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		EvenMirrorBoundary<D>();
		EvenMirrorBoundary<D>(const EvenMirrorBoundary<D>&);
		EvenMirrorBoundary<D>& operator=(const EvenMirrorBoundary<D>&);
};

// odd mirror boundary conditions
template<Uint8 D> 
class OddMirrorBoundary
{ 
	public:
		static Index<D> mapIndex(const Index<D>& idx,const IndexRange<D>& rng);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		OddMirrorBoundary<D>();
		OddMirrorBoundary<D>(const OddMirrorBoundary<D>&);
		OddMirrorBoundary<D>& operator=(const OddMirrorBoundary<D>&);
};

// cylindrical boundary conditions
template<Uint8 D> 
class CylindricalBoundary
{ 
	public:
		static Index<D> mapIndex(const Index<D>& idx,const IndexRange<D>& rng);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		CylindricalBoundary<D>();
		CylindricalBoundary<D>(const CylindricalBoundary<D>&);
		CylindricalBoundary<D>& operator=(const CylindricalBoundary<D>&);
};

// spherical boundary conditions
template<Uint8 D> 
class SphericalBoundary
{ 
	public:
		static Index<D> mapIndex(const Index<D>& idx,const IndexRange<D>& rng);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		SphericalBoundary<D>();
		SphericalBoundary<D>(const SphericalBoundary<D>&);
		SphericalBoundary<D>& operator=(const SphericalBoundary<D>&);
};

} // namespace mcs

#endif // MCS_BOUNDARYCONDITIONS_H