#include <cmath>
#include <complex>
#include <string>

#include "MCS_Ensure.h"
#include "MCS_MathExtensions.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T,Uint8 N> const EnsureNumeric<T> FixedMatrix<T,N>::dummy_;

// constructor
template<class T,Uint8 N> inline FixedMatrix<T,N>::FixedMatrix()
{
	set();
}

template<class T,Uint8 N> inline FixedMatrix<T,N>::FixedMatrix(const T& val)
{
	set(val);
}

template<class T,Uint8 N> inline FixedMatrix<T,N>::FixedMatrix(const T* data)
{
	set(data);
}

template<class T,Uint8 N> inline FixedMatrix<T,N>::FixedMatrix(const std::string& str_init)
{	
	set(str_init);
}

template<class T,Uint8 N> inline FixedMatrix<T,N>::FixedMatrix(const FixedMatrix<T,N>& source)
{
	set(source);
}	
	
// destructor
template<class T,Uint8 N> inline FixedMatrix<T,N>::~FixedMatrix()
{ }

// assignment
template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator=(const T& val)
{
	 set(val); 
	 return (*this); 
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator=(const std::string& str_init)
{ 
	set(str_init); 
	return (*this); 
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator=(const FixedMatrix<T,N>& source)
{
	if (this == &source) return *this;

	set(source);	
	
	return *this;
}		

// element assignors
// initialize with value
template<class T,Uint8 N> inline void FixedMatrix<T,N>::set()
{ 
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] = T();
}

template<class T,Uint8 N> inline void FixedMatrix<T,N>::set(const T& val)
{ 
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] = val;
}

// initialize with array
template<class T,Uint8 N> inline void FixedMatrix<T,N>::set(const T* data)
{ 
	for (Int32 i=first(),k=0;i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] = data[k++];
}

template<class T,Uint8 N> inline void FixedMatrix<T,N>::set(const std::string& str_init)
{	
	std::stringstream	temp(str_init);
	
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			temp >> mat_data_[i][j];
}

template<class T,Uint8 N> inline void FixedMatrix<T,N>::set(const FixedMatrix<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] = source.mat_data_[i][j];
}

// arithmetic operators
// scalar member assign/equals operators
template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator+=(const T& val)
{
	if (val == T(0)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] += val;
	
	return *this;
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator-=(const T& val)
{
	if (val == T(0)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] -= val;
	
	return *this;
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator*=(const T& val)
{
	if (val == T(1)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] *= val;
	
	return *this;
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator/=(const T& val)
{
	if (val == T(1)) return *this;
		
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] /= val;
	
	return *this;
}

// FixedMatrix member assign/equals operators
template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator+=(const FixedMatrix<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] += source.mat_data_[i][j];
		
	return *this;
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator-=(const FixedMatrix<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] -= source.mat_data_[i][j];
	
	return *this;
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator*=(const FixedMatrix<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] *= source.mat_data_[i][j];
	
	return *this;
}

template<class T,Uint8 N> inline FixedMatrix<T,N>& FixedMatrix<T,N>::operator/=(const FixedMatrix<T,N>& source)
{
	for (Int32 i=first();i<=last();i++)
		for (Int32 j=first();j<=last();j++)
			mat_data_[i][j] /= source.mat_data_[i][j];
	
	return *this;
}

} // namespace mcs
