#ifndef MCS_FIXEDMATRIXFUNCTIONS_H
#define MCS_FIXEDMATRIXFUNCTIONS_H

#include "MCS_FixedMatrix.h"
#include "MCS_MathExtensions.h"
#include "MCS_Promote.h"

namespace mcs {

// in place
// arithmetic functions
template<class T,Uint8 N> const FixedMatrix<T,N>& Abs(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Ceil(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Exp(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Floor(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Log(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Pow(FixedMatrix<T,N>& source,const T& expon);
template<class T,Uint8 N> const FixedMatrix<T,N>& Round(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sqr(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sqrt(FixedMatrix<T,N>& source);

// trigonometric functions
template<class T,Uint8 N> const FixedMatrix<T,N>& Acos(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Asin(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Atan(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Cos(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sin(FixedMatrix<T,N>& source); 
template<class T,Uint8 N> const FixedMatrix<T,N>& Tan(FixedMatrix<T,N>& source);

// hyperbolic functions
template<class T,Uint8 N> const FixedMatrix<T,N>& Acosh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Asinh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Atanh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Cosh(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Sinh(FixedMatrix<T,N>& source); 
template<class T,Uint8 N> const FixedMatrix<T,N>& Tanh(FixedMatrix<T,N>& source);

// complex vectors
template<class T,Uint8 N> const FixedMatrix<T,N>& Conj(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Imag(FixedMatrix<T,N>& source);
template<class T,Uint8 N> const FixedMatrix<T,N>& Real(FixedMatrix<T,N>& source);

// not in place 
// arithmetic functions
template<class T,Uint8 N> FixedMatrix<T,N> abs(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> ceil(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> exp(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> floor(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> log(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> pow(const FixedMatrix<T,N>& source,const T& expon);
template<class T,Uint8 N> FixedMatrix<T,N> round(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sqr(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sqrt(const FixedMatrix<T,N>& source);

// trigonometric functions
template<class T,Uint8 N> FixedMatrix<T,N> acos(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> asin(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> atan(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> atan2(const FixedMatrix<T,N>& y,const FixedMatrix<T,N>& x);
template<class T,Uint8 N> FixedMatrix<T,N> cos(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sin(const FixedMatrix<T,N>& source); 
template<class T,Uint8 N> FixedMatrix<T,N> tan(const FixedMatrix<T,N>& source);

// hyperbolic functions
template<class T,Uint8 N> FixedMatrix<T,N> acosh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> asinh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> atanh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> cosh(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> sinh(const FixedMatrix<T,N>& source); 
template<class T,Uint8 N> FixedMatrix<T,N> tanh(const FixedMatrix<T,N>& source);
 
// complex vectors
template<class T,Uint8 N> FixedMatrix<T,N> conj(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<typename DemoteComplex<T>::demote_t,N> imag(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<typename DemoteComplex<T>::demote_t,N> real(const FixedMatrix<T,N>& source);

// miscellaneous
template<class T,Uint8 N> FixedMatrix<T,N> bound(const FixedMatrix<T,N>& x,const T& min_val,const T& max_val);
template<class T,Uint8 N> FixedMatrix<Binary,N> even(const FixedMatrix<T,N>& x);
template<class T,Uint8 N> FixedMatrix<Binary,N> odd(const FixedMatrix<T,N>& x);
template<class T,Uint8 N> FixedMatrix<T,N> mod(const FixedMatrix<T,N>& x,const T& y);
template<class T,Uint8 N> FixedMatrix<T,N> mod(const FixedMatrix<T,N>& x,const FixedMatrix<T,N>& y);
template<class T,Uint8 N> FixedMatrix<T,N> quotient(const FixedMatrix<T,N>& x,const T& y);
template<class T,Uint8 N> FixedMatrix<T,N> quotient(const FixedMatrix<T,N>& x,const FixedMatrix<T,N>& y);

// vector specific
template<class T,Uint8 N> bool all(const FixedMatrix<T,N>&);
template<class T,Uint8 N> bool any(const FixedMatrix<T,N>&);

template<class T,Uint8 N> T sum(const FixedMatrix<T,N>& source);

// in place
// arithmetic functions
template<class T,Uint8 N> inline const FixedMatrix<T,N>& Abs(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::abs(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Ceil(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::ceil(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Exp(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::exp(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Floor(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::floor(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Log(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::log(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Pow(FixedMatrix<T,N>& source,const T& expon)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::pow(source(i,j),expon);
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Round(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::round(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Sqr(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) *= source(i,j);
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Sqrt(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::sqrt(source(i,j));
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T,Uint8 N> inline const FixedMatrix<T,N>& Conj(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = mcs::conj(source(i,j));
		
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Imag(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = mcs::imag(source(i,j));
		
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Real(FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = mcs::real(source(i,j));
		
	return source;
}

// trigonometric functions
template<class T,Uint8 N> inline const FixedMatrix<T,N>& Sin(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::sin(source(i,j));

	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Cos(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::cos(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Tan(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::tan(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Asin(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::asin(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Acos(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::acos(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Atan(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::atan(source(i,j));
	
	return source;
}

// hyperbolic functions
template<class T,Uint8 N> inline const FixedMatrix<T,N>& Sinh(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::sinh(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Cosh(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::cosh(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Tanh(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::tanh(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Asinh(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::asinh(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Acosh(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::acosh(source(i,j));
	
	return source;
}

template<class T,Uint8 N> inline const FixedMatrix<T,N>& Atanh(FixedMatrix<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			source(i,j) = std::atanh(source(i,j));

	return source;
}

// not in place
// arithmetic functions
template<class T,Uint8 N> inline FixedMatrix<T,N> abs(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Abs(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> ceil(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Ceil(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> exp(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Exp(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> floor(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Floor(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> log(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Log(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> pow(const FixedMatrix<T,N>& source,const T& expon)
{
	FixedMatrix<T,N>	tmp(source);
	
	Pow(tmp,expon);
	
	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> round(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Round(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> sqr(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Sqr(tmp);

	return tmp;
}


template<class T,Uint8 N> inline FixedMatrix<T,N> sqrt(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Sqrt(tmp);

	return tmp;
}

// complex vectors
template<class T,Uint8 N> inline FixedMatrix<T,N> conj(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Conj(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<typename DemoteComplex<T>::demote_t,N> imag(const FixedMatrix<T,N>& source)
{
	FixedMatrix<typename DemoteComplex<T>::demote_t,N>	tmp(source);

	Imag(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> imag(const FixedMatrix< std::complex<T>,N>& source)
{
	FixedMatrix<T,N>	tmp;

	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = source(i,j).imag();

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<typename DemoteComplex<T>::demote_t,N> real(const FixedMatrix<T,N>& source)
{
	FixedMatrix<typename DemoteComplex<T>::demote_t,N>	tmp(source);

	Real(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> real(const FixedMatrix< std::complex<T>,N>& source)
{
	FixedMatrix<T,N>	tmp;

	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = source(i,j).real();

	return tmp;
}

// trigonometric functions
template<class T,Uint8 N> inline FixedMatrix<T,N> sin(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Sin(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> cos(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Cos(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> tan(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Tan(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> asin(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Asin(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> acos(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Acos(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> atan(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Atan(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> atan2(const FixedMatrix<T,N>& y,const FixedMatrix<T,N>& x)
{
	static const EnsureFloat<T>	dummy_;
		
	FixedMatrix<T,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = std::atan2(y(i,j),x(i,j));
	
	return tmp;
}

// hyperbolic functions
template<class T,Uint8 N> inline FixedMatrix<T,N> sinh(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Sinh(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> cosh(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Cosh(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> tanh(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Tanh(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> asinh(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Asinh(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> acosh(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Acosh(tmp);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> atanh(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N>	tmp(source);

	Atanh(tmp);

	return tmp;
}

// miscellaneous
template<class T,Uint8 N> inline FixedMatrix<T,N> bound(const FixedMatrix<T,N>& x,const T& min_val,const T& max_val)
{
	FixedMatrix<T,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::bound(x(i,j),min_val,max_val);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<Binary,N> even(const FixedMatrix<T,N>& x)
{
	FixedMatrix<Binary,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::even(x(i,j));

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<Binary,N> odd(const FixedMatrix<T,N>& x)
{
	FixedMatrix<Binary,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::odd(x(i,j));

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> mod(const FixedMatrix<T,N>& x,const T& y)
{
	FixedMatrix<T,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::mod(x(i,j),y);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> mod(const FixedMatrix<T,N>& x,const FixedMatrix<T,N>& y)
{
	FixedMatrix<T,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::mod(x(i,j),y(i,j));

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> quotient(const FixedMatrix<T,N>& x,const T& y)
{
	FixedMatrix<T,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::quotient(x(i,j),y);

	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> quotient(const FixedMatrix<T,N>& x,const FixedMatrix<T,N>& y)
{
	FixedMatrix<T,N>	tmp;
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			tmp(i,j) = mcs::quotient(x(i,j),y(i,j));

	return tmp;
}

// vector specific
template<class T,Uint8 N> inline bool all(const FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			if (!source(i,j)) return false;
		
	return true;
}

template<class T,Uint8 N> inline bool any(const FixedMatrix<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			if (source(i,j)) return true;
		
	return false;
}

template<class T,Uint8 N> inline T sum(const FixedMatrix<T,N>& source)
{
	T	sum = static_cast<T>(0);
	
	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 j=source.first();j<=source.last();j++)
			sum += source(i,j);
		
	return sum;
}

} // namespace mcs

//#include "MCS_FixedMatrix1Functions.h"
//#include "MCS_FixedMatrix2Functions.h"
//#include "MCS_FixedMatrix3Functions.h"
//#include "MCS_FixedMatrix4Functions.h"

#endif // MCS_FIXEDMATRIXFUNCTIONS_H
