#ifndef MCS_FIXEDMATRIXOPERATORS_H
#define MCS_FIXEDMATRIXOPERATORS_H

#include "MCS_FixedVector.h"
#include "MCS_FixedMatrix.h"

namespace mcs {

// matrix non-member operators
template<class T,Uint8 N> FixedMatrix<T,N> operator+(const T& lscal,const FixedMatrix<T,N>& rvec);
template<class T,Uint8 N> FixedMatrix<T,N> operator-(const T& lscal,const FixedMatrix<T,N>& rvec);
template<class T,Uint8 N> FixedMatrix<T,N> operator*(const T& lscal,const FixedMatrix<T,N>& rvec);
template<class T,Uint8 N> FixedMatrix<T,N> operator/(const T& lscal,const FixedMatrix<T,N>& rvec);

template<class T,Uint8 N> FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& lvec,const T& rscal);
template<class T,Uint8 N> FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& lvec,const T& rscal);
template<class T,Uint8 N> FixedMatrix<T,N> operator*(const FixedMatrix<T,N>& lvec,const T& rscal);
template<class T,Uint8 N> FixedMatrix<T,N> operator/(const FixedMatrix<T,N>& lvec,const T& rscal);

template<class T,Uint8 N> typename FixedMatrix<T,N>::Mask operator&&(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> typename FixedMatrix<T,N>::Mask operator||(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);

template<class T,Uint8 N> FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> FixedMatrix<T,N> operator*(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);
template<class T,Uint8 N> FixedMatrix<T,N> operator/(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs);

template<class T,Uint8 N> FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& source);
template<class T,Uint8 N> FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& source);

template<class T,Uint8 N> FixedMatrix<T,N> operator!(const FixedMatrix<T,N>& source);

template<class T,Uint8 N> FixedVector<T,N> operator*(const FixedVector<T,N>& lvec,const FixedMatrix<T,N>& rmat);
template<class T,Uint8 N> FixedVector<T,N> operator*(const FixedMatrix<T,N>& lmat,const FixedVector<T,N>& rvec);

// scalar op Matrix
template<class T,Uint8 N> inline FixedMatrix<T,N> operator+(const T& lscal,const FixedMatrix<T,N>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (FixedMatrix<T,N>(lscal) += rvec);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator-(const T& lscal,const FixedMatrix<T,N>& rvec)
{
	if (lscal == static_cast<T>(0)) return -rvec;
		
	return (FixedMatrix<T,N>(lscal) -= rvec);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator*(const T& lscal,const FixedMatrix<T,N>& rvec)
{
	if (lscal == static_cast<T>(1)) return rvec;
	if (lscal == static_cast<T>(0)) return FixedMatrix<T,N>(static_cast<T>(0));
	
	return (FixedMatrix<T,N>(lscal) *= rvec);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator/(const T& lscal,const FixedMatrix<T,N>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (FixedMatrix<T,N>(lscal) /= rvec);
}

// Matrix op scalar
template<class T,Uint8 N> inline FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (FixedMatrix<T,N>(lvec) += rscal);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (FixedMatrix<T,N>(lvec) -= rscal);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator*(const FixedMatrix<T,N>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
	if (rscal == static_cast<T>(0)) return FixedMatrix<T,N>(static_cast<T>(0));	

	return (FixedMatrix<T,N>(lvec) *= rscal);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator/(const FixedMatrix<T,N>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
		
	return (FixedMatrix<T,N>(lvec) /= rscal);
}

// Matrix op Matrix
template<class T,Uint8 N> inline typename FixedMatrix<T,N>::Mask operator&&(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs)
{
	FixedMatrix<T,N>::Mask tmp;
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		for (Int32 j=lhs.first();j=lhs.last();j++)
			tmp(i,j) = (lhs(i,j) && rhs(i,j));
		
	return tmp;
}

template<class T,Uint8 N> inline typename FixedMatrix<T,N>::Mask operator||(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs)
{
	FixedMatrix<T,N>::Mask tmp;
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		for (Int32 j=lhs.first();j=lhs.last();j++)
			tmp(i,j) = (lhs(i,j) || rhs(i,j));
		
	return tmp;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs)
{
	return (FixedMatrix<T,N>(lhs) += rhs);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs)
{
	return (FixedMatrix<T,N>(lhs) -= rhs);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator*(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs)
{
	return (FixedMatrix<T,N>(lhs) *= rhs);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator/(const FixedMatrix<T,N>& lhs,const FixedMatrix<T,N>& rhs)
{
	return (FixedMatrix<T,N>(lhs) /= rhs);
}

// unary operators
template<class T,Uint8 N> inline FixedMatrix<T,N> operator+(const FixedMatrix<T,N>& source)
{
	return source;
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator-(const FixedMatrix<T,N>& source)
{
	return (FixedMatrix<T,N>(static_cast<T>(0)) -= source);
}

template<class T,Uint8 N> inline FixedMatrix<T,N> operator!(const FixedMatrix<T,N>& source)
{
	FixedMatrix<T,N> tmp(source);
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		for (Int32 j=lhs.first();j=lhs.last();j++)
			tmp(i,j) = !source(i,j);
		
	return tmp;
}

// matrix-vector multiplication
template<class T,Uint8 N> inline FixedVector<T,N> operator*(const FixedVector<T,N>& lvec,const FixedMatrix<T,N>& rmat)
{
	FixedVector<T,N>	tmp(T(0));
	
	for (Int32 i=rmat.first();i<=rmat.last();i++)
		for (Int32 j=rmat.first();j<=rmat.last();j++)
			tmp[i] += lvec[i]*rmat(i,j);
			
	return tmp;
}

template<class T,Uint8 N> inline FixedVector<T,N> operator*(const FixedMatrix<T,N>& lmat,const FixedVector<T,N>& rvec)
{
	FixedVector<T,N>	tmp(T(0));
	
	for (Int32 i=lmat.first();i<=lmat.last();i++)
		for (Int32 j=lmat.first();j<=lmat.last();j++)
			tmp[i] += lmat(i,j)*rvec[j];
			
	return tmp;
}

} // namespace mcs

//#include "MCS_FixedMatrix1Operators.h"
//#include "MCS_FixedMatrix2Operators.h"
//#include "MCS_FixedMatrix3Operators.h"
//#include "MCS_FixedMatrix4Operators.h"

#endif // MCS_FIXEDMATRIXOPERATORS_H