#ifndef MCS_FIXEDVECTOR1_H
#define MCS_FIXEDVECTOR1_H

#include "MCS_FixedVector.h"

namespace mcs {

template<class T> class FixedVector<T,1> 
{
	public:	
		typedef FixedVector<Binary,1>	Mask;
		
		// ctor/dtor		
		FixedVector();
		FixedVector(const T&);
		explicit FixedVector(const T*);
		FixedVector(const std::string&);
		FixedVector(const FixedVector<T,1>&);
		~FixedVector();
		
		// range accessors
				bool isValid() const			{ return true; }
							 	
				Int32 elem() const				{ return 1; }
				 	
				Int32 first() const				{ return 0; }
				Int32 last() const				{ return 1-1; }
				Int32 dim() const				{ return 1; }			
				Int32 offset() const			{ return 0; }
				
		// assignment
				FixedVector<T,1>& operator=(T);
				FixedVector<T,1>& operator=(const std::string&);
				FixedVector<T,1>& operator=(const FixedVector<T,1>&);
				 
		// element accessors		
				T& 					operator[](const Int32& idx)		{ return vec_data_[idx]; }
		const 	T& 					operator[](const Int32& idx) const	{ return vec_data_[idx]; }
			 	
				T& 					rank(const Int32& idx)				{ return vec_data_[idx]; }
		const 	T& 					rank(const Int32& idx) const		{ return vec_data_[idx]; }
			 	
		// arithmetic operations		
			 	FixedVector<T,1>& 	operator+=(T);		
			 	FixedVector<T,1>& 	operator-=(T);
			 	FixedVector<T,1>& 	operator*=(T);
			 	FixedVector<T,1>& 	operator/=(T);
			 	FixedVector<T,1>& 	operator%=(T);
			 	FixedVector<T,1>& 	operator&=(T);
			 	FixedVector<T,1>& 	operator|=(T);
			 	FixedVector<T,1>& 	operator^=(T);
			 	FixedVector<T,1>& 	operator<<=(T);
			 	FixedVector<T,1>& 	operator>>=(T);

			 	FixedVector<T,1>& 	operator+=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator-=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator*=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator/=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator%=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator&=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator|=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator^=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator<<=(const FixedVector<T,1>&);
			 	FixedVector<T,1>& 	operator>>=(const FixedVector<T,1>&);
			 	
		// unique to FixedVector<T,1>
		// type conversion to base
		operator T() const 						{ return vec_data_[0]; }
									
		// increment/decrement
		FixedVector<T,1>& operator--()			{ --((*this)[0]); return *this; }
		const FixedVector<T,1> operator--(int)	{ const FixedVector<T,1> tmp(*this); ((*this)[0])--; return tmp; }
		FixedVector<T,1>& operator++()			{ ++((*this)[0]); return *this; }
		const FixedVector<T,1> operator++(int)	{ const FixedVector<T,1> tmp(*this); ((*this)[0])++; return tmp; }
		
	private:
		static const EnsureNumeric<T>	dummy_;
		
		// element assignors			
				void			set();
				void			set(const T&);
				void			set(const T*);
				void			set(const std::string&);
				void			set(const FixedVector<T,1>& source);	
									
		T		vec_data_[1];
};

// implementation
#include "MCS_FixedVector1.cpp"

} // namespace mcs

#endif // MCS_FIXEDVECTOR1_H
