#ifndef MCS_FIXEDVECTOR1FUNCTIONS_H
#define MCS_FIXEDVECTOR1FUNCTIONS_H

#include "MCS_FixedVector1.h"

namespace mcs {

// in place
// arithmetic functions
template<class T> inline const FixedVector<T,1>& Abs(FixedVector<T,1>& source)
{
	source[0] = std::abs(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Ceil(FixedVector<T,1>& source)
{
	source[0] = std::ceil(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Exp(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::exp(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Floor(FixedVector<T,1>& source)
{
	source[0] = std::floor(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Log(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::log(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Pow(FixedVector<T,1>& source,T expon)
{
	source[0] = std::pow(source[0],expon);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Round(FixedVector<T,1>& source)
{
	source[0] = std::round(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Sqr(FixedVector<T,1>& source)
{
	source[0] = mcs::sqr(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Sqrt(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sqrt(source[0]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> inline const FixedVector<T,1>& Conj(FixedVector<T,1>& source)
{
	source[0] = mcs::conj(source[0]);
		
	return source;
}

template<class T> inline const FixedVector<T,1>& Imag(FixedVector<T,1>& source)
{
	source[0] = mcs::imag(source[0]);
		
	return source;
}

template<class T> inline const FixedVector<T,1>& Real(FixedVector<T,1>& source)
{
	source[0] = mcs::real(source[0]);
		
	return source;
}

// trigonometric functions
template<class T> inline const FixedVector<T,1>& Sin(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sin(source[0]);

	return source;
}

template<class T> inline const FixedVector<T,1>& Cos(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::cos(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Tan(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::tan(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Asin(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::asin(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Acos(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::acos(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Atan(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::atan(source[0]);
	
	return source;
}

// hyperbolic functions
template<class T> inline const FixedVector<T,1>& Sinh(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sinh(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Cosh(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::cosh(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Tanh(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::tanh(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Asinh(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::asinh(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Acosh(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::acosh(source[0]);
	
	return source;
}

template<class T> inline const FixedVector<T,1>& Atanh(FixedVector<T,1>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::atanh(source[0]);

	return source;
}

// not in place
template<class T> inline FixedVector<T,1> imag(const FixedVector< std::complex<T>,1>& source)
{
	return FixedVector<T,1>(source[0].imag());
}

template<class T> inline FixedVector<T,1> real(const FixedVector< std::complex<T>,1>& source)
{
	return FixedVector<T,1>(source[0].real());
}

template<class T> inline FixedVector<T,1> atan2(const FixedVector<T,1>& y,const FixedVector<T,1>& x)
{
	static const EnsureFloat<T>	dummy_;
		
	FixedVector<T,1>	tmp;
	
	tmp[0] = std::atan2(y[0],x[0]);
	
	return tmp;
}

// miscellaneous
template<class T> inline FixedVector<T,1> bound(const FixedVector<T,1>& x,T min_val,T max_val)
{
	FixedVector<T,1>	tmp;
	
	tmp[0] = mcs::bound(x[0],min_val,max_val);

	return tmp;
}

template<class T> inline FixedVector<Binary,1> even(const FixedVector<T,1>& x)
{
	FixedVector<Binary,1>	tmp;
	
	tmp[0] = mcs::even(x[0]);

	return tmp;
}

template<class T> inline FixedVector<Binary,1> odd(const FixedVector<T,1>& x)
{
	FixedVector<Binary,1>	tmp;
	
	tmp[0] = mcs::odd(x[0]);

	return tmp;
}

template<class T> inline FixedVector<T,1> mod(const FixedVector<T,1>& x,T y)
{
	FixedVector<T,1>	tmp;
	
	tmp[0] = mcs::mod(x[0],y);

	return tmp;
}

template<class T> inline FixedVector<T,1> mod(const FixedVector<T,1>& x,const FixedVector<T,1>& y)
{
	FixedVector<T,1>	tmp;
	
	tmp[0] = mcs::mod(x[0],y[0]);

	return tmp;
}

template<class T> inline FixedVector<T,1> quotient(const FixedVector<T,1>& x,T y)
{
	FixedVector<T,1>	tmp;
	
	tmp[0] = mcs::quotient(x[0],y);

	return tmp;
}

template<class T> inline FixedVector<T,1> quotient(const FixedVector<T,1>& x,const FixedVector<T,1>& y)
{
	FixedVector<T,1>	tmp;
	
	tmp[0] = mcs::quotient(x[0],y[0]);

	return tmp;
}

// vector specific
template<class T> inline bool all(const FixedVector<T,1>& source)
{
	if (!source[0]) 
		return false;
		
	return true;
}

template<class T> inline bool any(const FixedVector<T,1>& source)
{
	if (source[0]) 
		return true;
		
	return false;
}

template<class T> inline T product(const FixedVector<T,1>& source)
{
	return (source[0]);
}

template<class T> inline T sum(const FixedVector<T,1>& source)
{
	return (source[0]);
}

} // namespace mcs

#endif // MCS_FIXEDVECTOR1FUNCTIONS_H
