#ifndef MCS_FIXEDVECTOR2_H
#define MCS_FIXEDVECTOR2_H

#include "MCS_FixedVector.h"

namespace mcs {

template<class T> class FixedVector<T,2> 
{
	public:	
		typedef FixedVector<Binary,2>	Mask;
		
		// ctor/dtor		
		FixedVector();
		explicit FixedVector(const T&);
		explicit FixedVector(const T&,const T&);
		explicit FixedVector(const T*);
		FixedVector(const std::string&);
		FixedVector(const FixedVector<T,2>&);
		~FixedVector();
		
		// range accessors
				bool isValid() const			{ return true; }
							 	
				Int32 elem() const				{ return 2; }
				 	
				Int32 first() const				{ return 0; }
				Int32 last() const				{ return 2-1; }
				Int32 dim() const				{ return 2; }	
				Int32 offset() const			{ return 0; }
				
		// assignment
				FixedVector<T,2>& operator=(T);
				FixedVector<T,2>& operator=(const std::string&);
				FixedVector<T,2>& operator=(const FixedVector<T,2>&);
				 
		// element accessors		
				T& 					operator[](const Int32& idx)		{ return vec_data_[idx]; }
		const 	T& 					operator[](const Int32& idx) const	{ return vec_data_[idx]; }
			 	
		// arithmetic operations		
			 	FixedVector<T,2>& 	operator+=(T);		
			 	FixedVector<T,2>& 	operator-=(T);
			 	FixedVector<T,2>& 	operator*=(T);
			 	FixedVector<T,2>& 	operator/=(T);

			 	FixedVector<T,2>& 	operator+=(const FixedVector<T,2>&);
			 	FixedVector<T,2>& 	operator-=(const FixedVector<T,2>&);
			 	FixedVector<T,2>& 	operator*=(const FixedVector<T,2>&);
			 	FixedVector<T,2>& 	operator/=(const FixedVector<T,2>&);
				
	private:
		static const EnsureNumeric<T>	dummy_;
		
		// element assignors			
				void			set();
				void			set(const T&);
				void			set(const T&,const T&);
				void			set(const T*);
				void			set(const std::string&);
				void			set(const FixedVector<T,2>& source);	
									
		T		vec_data_[2];
};

// implementation
#include "MCS_FixedVector2.cpp"

} // namespace mcs

#endif // MCS_FIXEDVECTOR2_H
