#ifndef MCS_FIXEDVECTOR2FUNCTIONS_H
#define MCS_FIXEDVECTOR2FUNCTIONS_H

#include "MCS_FixedVector2.h"

namespace mcs {

// in place
// arithmetic functions
template<class T> inline const FixedVector<T,2>& Abs(FixedVector<T,2>& source)
{
	source[0] = std::abs(source[0]);
	source[1] = std::abs(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Ceil(FixedVector<T,2>& source)
{
	source[0] = std::ceil(source[0]);
	source[1] = std::ceil(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Exp(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::exp(source[0]);
	source[1] = std::exp(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Floor(FixedVector<T,2>& source)
{
	source[0] = std::floor(source[0]);
	source[1] = std::floor(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Log(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::log(source[0]);
	source[1] = std::log(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Pow(FixedVector<T,2>& source,T expon)
{
	source[0] = std::pow(source[0],expon);
	source[1] = std::pow(source[1],expon);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Round(FixedVector<T,2>& source)
{
	source[0] = std::round(source[0]);
	source[1] = std::round(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Sqr(FixedVector<T,2>& source)
{
	source[0] = mcs::sqr(source[0]);
	source[1] = mcs::sqr(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Sqrt(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sqrt(source[0]);
	source[1] = std::sqrt(source[1]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> inline const FixedVector<T,2>& Conj(FixedVector<T,2>& source)
{
	source[0] = mcs::conj(source[0]);
	source[1] = mcs::conj(source[1]);
		
	return source;
}

template<class T> inline const FixedVector<T,2>& Imag(FixedVector<T,2>& source)
{
	source[0] = mcs::imag(source[0]);
	source[1] = mcs::imag(source[1]);
		
	return source;
}

template<class T> inline const FixedVector<T,2>& Real(FixedVector<T,2>& source)
{
	source[0] = mcs::real(source[0]);
	source[1] = mcs::real(source[1]);
		
	return source;
}

// trigonometric functions
template<class T> inline const FixedVector<T,2>& Sin(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sin(source[0]);
	source[1] = std::sin(source[1]);

	return source;
}

template<class T> inline const FixedVector<T,2>& Cos(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::cos(source[0]);
	source[1] = std::cos(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Tan(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::tan(source[0]);
	source[1] = std::tan(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Asin(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::asin(source[0]);
	source[1] = std::asin(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Acos(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::acos(source[0]);
	source[1] = std::acos(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Atan(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::atan(source[0]);
	source[1] = std::atan(source[1]);
	
	return source;
}

// hyperbolic functions
template<class T> inline const FixedVector<T,2>& Sinh(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sinh(source[0]);
	source[1] = std::sinh(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Cosh(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::cosh(source[0]);
	source[1] = std::cosh(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Tanh(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::tanh(source[0]);
	source[1] = std::tanh(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Asinh(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::asinh(source[0]);
	source[1] = std::asinh(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Acosh(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::acosh(source[0]);
	source[1] = std::acosh(source[1]);
	
	return source;
}

template<class T> inline const FixedVector<T,2>& Atanh(FixedVector<T,2>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::atanh(source[0]);
	source[1] = std::atanh(source[1]);

	return source;
}

// not in place
template<class T> inline FixedVector<T,2> imag(const FixedVector< std::complex<T>,2>& source)
{
	return FixedVector<T,2>(source[0].imag(),source[1].imag());
}

template<class T> inline FixedVector<T,2> real(const FixedVector< std::complex<T>,2>& source)
{
	return FixedVector<T,2>(source[0].real(),source[1].real());
}

template<class T> inline FixedVector<T,2> atan2(const FixedVector<T,2>& y,const FixedVector<T,2>& x)
{
	static const EnsureFloat<T>	dummy_;
		
	FixedVector<T,2>	tmp;
	
	tmp[0] = std::atan2(y[0],x[0]);
	tmp[1] = std::atan2(y[1],x[1]);
	
	return tmp;
}

// miscellaneous
template<class T> inline FixedVector<T,2> bound(const FixedVector<T,2>& x,T min_val,T max_val)
{
	FixedVector<T,2>	tmp;
	
	tmp[0] = mcs::bound(x[0],min_val,max_val);
	tmp[1] = mcs::bound(x[1],min_val,max_val);

	return tmp;
}

template<class T> inline FixedVector<Binary,2> even(const FixedVector<T,2>& x)
{
	FixedVector<Binary,2>	tmp;
	
	tmp[0] = mcs::even(x[0]);
	tmp[1] = mcs::even(x[1]);

	return tmp;
}

template<class T> inline FixedVector<Binary,2> odd(const FixedVector<T,2>& x)
{
	FixedVector<Binary,2>	tmp;
	
	tmp[0] = mcs::odd(x[0]);
	tmp[1] = mcs::odd(x[1]);

	return tmp;
}

template<class T> inline FixedVector<T,2> mod(const FixedVector<T,2>& x,T y)
{
	FixedVector<T,2>	tmp;
	
	tmp[0] = mcs::mod(x[0],y);
	tmp[1] = mcs::mod(x[1],y);

	return tmp;
}

template<class T> inline FixedVector<T,2> mod(const FixedVector<T,2>& x,const FixedVector<T,2>& y)
{
	FixedVector<T,2>	tmp;
	
	tmp[0] = mcs::mod(x[0],y[0]);
	tmp[1] = mcs::mod(x[1],y[1]);

	return tmp;
}

template<class T> inline FixedVector<T,2> quotient(const FixedVector<T,2>& x,T y)
{
	FixedVector<T,2>	tmp;
	
	tmp[0] = mcs::quotient(x[0],y);
	tmp[1] = mcs::quotient(x[1],y);

	return tmp;
}

template<class T> inline FixedVector<T,2> quotient(const FixedVector<T,2>& x,const FixedVector<T,2>& y)
{
	FixedVector<T,2>	tmp;
	
	tmp[0] = mcs::quotient(x[0],y[0]);
	tmp[1] = mcs::quotient(x[1],y[1]);

	return tmp;
}

// vector specific
template<class T> inline bool all(const FixedVector<T,2>& source)
{
	if (!source[0] || !source[1]) 
		return false;
		
	return true;
}

template<class T> inline bool any(const FixedVector<T,2>& source)
{
	if (source[0] || source[1]) 
		return true;
		
	return false;
}

template<class T> inline T product(const FixedVector<T,2>& source)
{
	return (source[0]*source[1]);
}

template<class T> inline T sum(const FixedVector<T,2>& source)
{
	return (source[0]+source[1]);
}

} // namespace mcs

#endif // MCS_FIXEDVECTOR2FUNCTIONS_H
