#ifndef MCS_FIXEDVECTOR4FUNCTIONS_H
#define MCS_FIXEDVECTOR4FUNCTIONS_H

#include "MCS_FixedVector4.h"

namespace mcs {

// in place
// arithmetic functions
template<class T> inline const FixedVector<T,4>& Abs(FixedVector<T,4>& source)
{
	source[0] = std::abs(source[0]);
	source[1] = std::abs(source[1]);
	source[2] = std::abs(source[2]);
	source[3] = std::abs(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Ceil(FixedVector<T,4>& source)
{
	source[0] = std::ceil(source[0]);
	source[1] = std::ceil(source[1]);
	source[2] = std::ceil(source[2]);
	source[3] = std::ceil(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Exp(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::exp(source[0]);
	source[1] = std::exp(source[1]);
	source[2] = std::exp(source[2]);
	source[3] = std::exp(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Floor(FixedVector<T,4>& source)
{
	source[0] = std::floor(source[0]);
	source[1] = std::floor(source[1]);
	source[2] = std::floor(source[2]);
	source[3] = std::floor(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Log(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::log(source[0]);
	source[1] = std::log(source[1]);
	source[2] = std::log(source[2]);
	source[3] = std::log(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Pow(FixedVector<T,4>& source,T expon)
{
	source[0] = std::pow(source[0],expon);
	source[1] = std::pow(source[1],expon);
	source[2] = std::pow(source[2],expon);
	source[3] = std::pow(source[3],expon);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Round(FixedVector<T,4>& source)
{
	source[0] = std::round(source[0]);
	source[1] = std::round(source[1]);
	source[2] = std::round(source[2]);
	source[3] = std::round(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Sqr(FixedVector<T,4>& source)
{
	source[0] = mcs::sqr(source[0]);
	source[1] = mcs::sqr(source[1]);
	source[2] = mcs::sqr(source[2]);
	source[3] = mcs::sqr(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Sqrt(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sqrt(source[0]);
	source[1] = std::sqrt(source[1]);
	source[2] = std::sqrt(source[2]);
	source[3] = std::sqrt(source[3]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> inline const FixedVector<T,4>& Conj(FixedVector<T,4>& source)
{
	source[0] = mcs::conj(source[0]);
	source[1] = mcs::conj(source[1]);
	source[2] = mcs::conj(source[2]);
	source[3] = mcs::conj(source[3]);
		
	return source;
}

template<class T> inline const FixedVector<T,4>& Imag(FixedVector<T,4>& source)
{
	source[0] = mcs::imag(source[0]);
	source[1] = mcs::imag(source[1]);
	source[2] = mcs::imag(source[2]);
	source[3] = mcs::imag(source[3]);
		
	return source;
}

template<class T> inline const FixedVector<T,4>& Real(FixedVector<T,4>& source)
{
	source[0] = mcs::real(source[0]);
	source[1] = mcs::real(source[1]);
	source[2] = mcs::real(source[2]);
	source[3] = mcs::real(source[3]);
		
	return source;
}

// trigonometric functions
template<class T> inline const FixedVector<T,4>& Sin(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sin(source[0]);
	source[1] = std::sin(source[1]);
	source[2] = std::sin(source[2]);
	source[3] = std::sin(source[3]);

	return source;
}

template<class T> inline const FixedVector<T,4>& Cos(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::cos(source[0]);
	source[1] = std::cos(source[1]);
	source[2] = std::cos(source[2]);
	source[3] = std::cos(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Tan(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::tan(source[0]);
	source[1] = std::tan(source[1]);
	source[2] = std::tan(source[2]);
	source[3] = std::tan(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Asin(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::asin(source[0]);
	source[1] = std::asin(source[1]);
	source[2] = std::asin(source[2]);
	source[3] = std::asin(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Acos(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::acos(source[0]);
	source[1] = std::acos(source[1]);
	source[2] = std::acos(source[2]);
	source[3] = std::acos(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Atan(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::atan(source[0]);
	source[1] = std::atan(source[1]);
	source[2] = std::atan(source[2]);
	source[3] = std::atan(source[3]);
	
	return source;
}

// hyperbolic functions
template<class T> inline const FixedVector<T,4>& Sinh(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::sinh(source[0]);
	source[1] = std::sinh(source[1]);
	source[2] = std::sinh(source[2]);
	source[3] = std::sinh(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Cosh(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::cosh(source[0]);
	source[1] = std::cosh(source[1]);
	source[2] = std::cosh(source[2]);
	source[3] = std::cosh(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Tanh(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::tanh(source[0]);
	source[1] = std::tanh(source[1]);
	source[2] = std::tanh(source[2]);
	source[3] = std::tanh(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Asinh(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::asinh(source[0]);
	source[1] = std::asinh(source[1]);
	source[2] = std::asinh(source[2]);
	source[3] = std::asinh(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Acosh(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::acosh(source[0]);
	source[1] = std::acosh(source[1]);
	source[2] = std::acosh(source[2]);
	source[3] = std::acosh(source[3]);
	
	return source;
}

template<class T> inline const FixedVector<T,4>& Atanh(FixedVector<T,4>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	source[0] = std::atanh(source[0]);
	source[1] = std::atanh(source[1]);
	source[2] = std::atanh(source[2]);
	source[3] = std::atanh(source[3]);

	return source;
}

// not in place
template<class T> inline FixedVector<T,4> imag(const FixedVector< std::complex<T>,4>& source)
{
	return FixedVector<T,4>(source[0].imag(),source[1].imag(),source[2].imag(),source[3].imag());
}

template<class T> inline FixedVector<T,4> real(const FixedVector< std::complex<T>,4>& source)
{
	return FixedVector<T,4>(source[0].real(),source[1].real(),source[2].real(),source[3].real());
}

template<class T> inline FixedVector<T,4> atan2(const FixedVector<T,4>& y,const FixedVector<T,4>& x)
{
	static const EnsureFloat<T>	dummy_;
		
	FixedVector<T,4>	tmp;
	
	tmp[0] = std::atan2(y[0],x[0]);
	tmp[1] = std::atan2(y[1],x[1]);
	tmp[2] = std::atan2(y[2],x[2]);
	tmp[3] = std::atan2(y[3],x[3]);
	
	return tmp;
}

// miscellaneous
template<class T> inline FixedVector<T,4> bound(const FixedVector<T,4>& x,T min_val,T max_val)
{
	FixedVector<T,4>	tmp;
	
	tmp[0] = mcs::bound(x[0],min_val,max_val);
	tmp[1] = mcs::bound(x[1],min_val,max_val);
	tmp[2] = mcs::bound(x[2],min_val,max_val);
	tmp[3] = mcs::bound(x[3],min_val,max_val);

	return tmp;
}

template<class T> inline FixedVector<Binary,4> even(const FixedVector<T,4>& x)
{
	FixedVector<Binary,4>	tmp;
	
	tmp[0] = mcs::even(x[0]);
	tmp[1] = mcs::even(x[1]);
	tmp[2] = mcs::even(x[2]);
	tmp[3] = mcs::even(x[3]);

	return tmp;
}

template<class T> inline FixedVector<Binary,4> odd(const FixedVector<T,4>& x)
{
	FixedVector<Binary,4>	tmp;
	
	tmp[0] = mcs::odd(x[0]);
	tmp[1] = mcs::odd(x[1]);
	tmp[2] = mcs::odd(x[2]);
	tmp[3] = mcs::odd(x[3]);

	return tmp;
}

template<class T> inline FixedVector<T,4> mod(const FixedVector<T,4>& x,T y)
{
	FixedVector<T,4>	tmp;
	
	tmp[0] = mcs::mod(x[0],y);
	tmp[1] = mcs::mod(x[1],y);
	tmp[2] = mcs::mod(x[2],y);
	tmp[3] = mcs::mod(x[3],y);

	return tmp;
}

template<class T> inline FixedVector<T,4> mod(const FixedVector<T,4>& x,const FixedVector<T,4>& y)
{
	FixedVector<T,4>	tmp;
	
	tmp[0] = mcs::mod(x[0],y[0]);
	tmp[1] = mcs::mod(x[1],y[1]);
	tmp[2] = mcs::mod(x[2],y[2]);
	tmp[3] = mcs::mod(x[3],y[3]);

	return tmp;
}

template<class T> inline FixedVector<T,4> quotient(const FixedVector<T,4>& x,T y)
{
	FixedVector<T,4>	tmp;

	tmp[0] = mcs::quotient(x[0],y);
	tmp[1] = mcs::quotient(x[1],y);
	tmp[2] = mcs::quotient(x[2],y);
	tmp[3] = mcs::quotient(x[3],y);

	return tmp;
}

template<class T> inline FixedVector<T,4> quotient(const FixedVector<T,4>& x,const FixedVector<T,4>& y)
{
	FixedVector<T,4>	tmp;

	tmp[0] = mcs::quotient(x[0],y[0]);
	tmp[1] = mcs::quotient(x[1],y[1]);
	tmp[2] = mcs::quotient(x[2],y[2]);
	tmp[3] = mcs::quotient(x[3],y[3]);

	return tmp;
}

// vector specific
template<class T> inline bool all(const FixedVector<T,4>& source)
{
	if (!source[0] || !source[1] || !source[2] || !source[3]) 
		return false;
		
	return true;
}

template<class T> inline bool any(const FixedVector<T,4>& source)
{
	if (source[0] || source[1] || source[2] || source[3]) 
		return true;
		
	return false;
}

template<class T> inline T product(const FixedVector<T,4>& source)
{
	return (source[0]*source[1]*source[2]*source[3]);
}

template<class T> inline T sum(const FixedVector<T,4>& source)
{
	return (source[0]+source[1]+source[2]+source[3]);
}

} // namespace mcs

#endif // MCS_FIXEDVECTOR4FUNCTIONS_H
