#ifndef MCS_FIXEDVECTOR6_H
#define MCS_FIXEDVECTOR6_H

#include "MCS_FixedVector.h"

namespace mcs {

template<class T> class FixedVector<T,6> 
{
	public:	
		typedef FixedVector<Binary,6>	Mask;
		
		// ctor/dtor		
		FixedVector();
		explicit FixedVector(const T&);
		explicit FixedVector(const T&,const T&,const T&,const T&,const T&,const T&);
		explicit FixedVector(const T*);
		FixedVector(const std::string&);
		FixedVector(const FixedVector<T,6>&);
		~FixedVector();
		
		// range accessors
				bool isValid() const			{ return true; }
							
				Int32 elem() const				{ return 6; }
				 	
				Int32 first() const				{ return 0; }
				Int32 last() const				{ return 6-1; }
				Int32 dim() const				{ return 6; }	
				Int32 offset() const			{ return 0; }
				
		// assignment
				FixedVector<T,6>& operator=(T);
				FixedVector<T,6>& operator=(const std::string&);
				FixedVector<T,6>& operator=(const FixedVector<T,6>&);
				 
		// element accessors		
				T& 					operator[](const Int32& idx)		{ return vec_data_[idx]; }
		const 	T& 					operator[](const Int32& idx) const	{ return vec_data_[idx]; }
			 	
		// arithmetic operations		
			 	FixedVector<T,6>& 	operator+=(T);		
			 	FixedVector<T,6>& 	operator-=(T);
			 	FixedVector<T,6>& 	operator*=(T);
			 	FixedVector<T,6>& 	operator/=(T);

			 	FixedVector<T,6>& 	operator+=(const FixedVector<T,6>&);
			 	FixedVector<T,6>& 	operator-=(const FixedVector<T,6>&);
			 	FixedVector<T,6>& 	operator*=(const FixedVector<T,6>&);
			 	FixedVector<T,6>& 	operator/=(const FixedVector<T,6>&);
				
	private:
		static const EnsureNumeric<T>	dummy_;
		
		// element assignors			
				void			set();
				void			set(const T&);
				void			set(const T&,const T&,const T&,const T&,const T&,const T&);
				void			set(const T*);
				void			set(const std::string&);
				void			set(const FixedVector<T,6>& source);	
									
		T		vec_data_[6];
};

// implementation
#include "MCS_FixedVector6.cpp"

} // namespace mcs

#endif // MCS_FIXEDVECTOR6_H
