#include "MCS_FixedVectorFunctions.h"
#include "MCS_MathExtensions.h"
#include "MCS_Promote.h"

namespace mcs {

// in place
// arithmetic functions
template<class T,Uint8 N> const FixedVector<T,N>& Abs(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::abs(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Ceil(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::ceil(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Exp(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::exp(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Floor(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::floor(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Log(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::log(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Pow(FixedVector<T,N>& source,T expon)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::pow(source[i],expon);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Round(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::round(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Sqr(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] *= source[i];
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Sqrt(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sqrt(source[i]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T,Uint8 N> const FixedVector<T,N>& Conj(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::conj(source[i]);
		
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Imag(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::imag(source[i]);
		
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Real(FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::real(source[i]);
		
	return source;
}

// trigonometric functions
template<class T,Uint8 N> const FixedVector<T,N>& Sin(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sin(source[i]);

	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Cos(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cos(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Tan(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tan(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Asin(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asin(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Acos(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acos(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Atan(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atan(source[i]);
	
	return source;
}

// hyperbolic functions
template<class T,Uint8 N> const FixedVector<T,N>& Sinh(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sinh(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Cosh(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cosh(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Tanh(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tanh(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Asinh(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asinh(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Acosh(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acosh(source[i]);
	
	return source;
}

template<class T,Uint8 N> const FixedVector<T,N>& Atanh(FixedVector<T,N>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atanh(source[i]);

	return source;
}

// not in place
// arithmetic functions
template<class T,Uint8 N> FixedVector<T,N> abs(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Abs(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> ceil(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Ceil(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> exp(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Exp(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> floor(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Floor(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> log(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Log(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> pow(const FixedVector<T,N>& source,T expon)
{
	FixedVector<T,N>	tmp(source);
	
	Pow(tmp,expon);
	
	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> round(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Round(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> sqr(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Sqr(tmp);

	return tmp;
}


template<class T,Uint8 N> FixedVector<T,N> sqrt(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Sqrt(tmp);

	return tmp;
}

// complex vectors
template<class T,Uint8 N> FixedVector<T,N> conj(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Conj(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<typename DemoteComplex<T>::demote_t,N> imag(const FixedVector<T,N>& source)
{
	FixedVector<typename DemoteComplex<T>::demote_t,N>	tmp;

	for (Int32 i=0;i<N;i++)
		tmp[i] = imag(source[i]);

	return tmp;
}

template<class T,Uint8 N> FixedVector<typename DemoteComplex<T>::demote_t,N> real(const FixedVector<T,N>& source)
{
	FixedVector<typename DemoteComplex<T>::demote_t,N>	tmp;

	for (Int32 i=0;i<N;i++)
		tmp[i] = real(source[i]);

	return tmp;
}

// trigonometric functions
template<class T,Uint8 N> FixedVector<T,N> sin(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Sin(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> cos(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Cos(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> tan(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Tan(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> asin(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Asin(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> acos(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Acos(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> atan(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Atan(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> atan2(const FixedVector<T,N>& y,const FixedVector<T,N>& x)
{
	static const EnsureFloat<T>	dummy_;
		
	FixedVector<T,N>	tmp;
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::atan2(y[i],x[i]);
	
	return tmp;
}

// hyperbolic functions
template<class T,Uint8 N> FixedVector<T,N> sinh(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Sinh(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> cosh(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Cosh(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> tanh(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Tanh(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> asinh(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Asinh(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> acosh(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Acosh(tmp);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> atanh(const FixedVector<T,N>& source)
{
	FixedVector<T,N>	tmp(source);

	Atanh(tmp);

	return tmp;
}

// miscellaneous
template<class T,Uint8 N> FixedVector<T,N> bound(const FixedVector<T,N>& x,T min_val,T max_val)
{
	FixedVector<T,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::bound(x[i],min_val,max_val);

	return tmp;
}

template<class T,Uint8 N> FixedVector<Binary,N> even(const FixedVector<T,N>& x)
{
	FixedVector<Binary,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::even(x[i]);

	return tmp;
}

template<class T,Uint8 N> FixedVector<Binary,N> odd(const FixedVector<T,N>& x)
{
	FixedVector<Binary,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::odd(x[i]);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> mod(const FixedVector<T,N>& x,T y)
{
	FixedVector<T,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::mod(x[i],y);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> mod(const FixedVector<T,N>& x,const FixedVector<T,N>& y)
{
	FixedVector<T,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::mod(x[i],y[i]);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> quotient(const FixedVector<T,N>& x,T y)
{
	FixedVector<T,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::quotient(x[i],y);

	return tmp;
}

template<class T,Uint8 N> FixedVector<T,N> quotient(const FixedVector<T,N>& x,const FixedVector<T,N>& y)
{
	FixedVector<T,N>	tmp;
	
	for (Int32 i=0;i<N;i++)
		tmp[i] = mcs::quotient(x[i],y[i]);

	return tmp;
}

// vector specific
template<class T,Uint8 N> bool all(const FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (!source[i]) return false;
		
	return true;
}

template<class T,Uint8 N> bool any(const FixedVector<T,N>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (source[i]) return true;
		
	return false;
}

template<class T,Uint8 N> const typename FixedVector<T,N>::Mask is_finite(const FixedVector<T,N>& source)
{
	typename FixedVector<T,N>::Mask	tmp;
	
	for (Int32 i=source.first();i<=source.last();++i)
		tmp[i] = isfinite(source[i]);
		
	return tmp;
}

template<class T,Uint8 N> inline T dot(const FixedVector<T,N>& lhs,const FixedVector<T,N>& rhs)
{
	return sum(lhs*rhs);
}
	
template<class T,Uint8 N> inline T mag(const FixedVector<T,N>& source)
{
	return std::sqrt(norm(source));
}

template<class T,Uint8 N> inline T norm(const FixedVector<T,N>& source)
{
	return sum(sqr(source));
}

template<class T,Uint8 N> T product(const FixedVector<T,N>& source)
{
	T	pd(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		pd *= source[i];
		
	return pd;
}

template<class T,Uint8 N> T sum(const FixedVector<T,N>& source)
{
	T	sm(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		sm += source[i];
		
	return sm;
}

template<class T,Uint8 N> inline FixedVector<T,N> unit(const FixedVector<T,N>& source)
{
	return source/mag(source);
}

} // namespace mcs

