#ifndef MCS_FIXEDVECTORFUNCTIONS_H
#define MCS_FIXEDVECTORFUNCTIONS_H

#include "MCS_FixedVector.h"
#include "MCS_MathExtensions.h"
#include "MCS_Promote.h"

namespace mcs {

// in place
// arithmetic functions
template<class T,Uint8 N> const FixedVector<T,N>& Abs(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Ceil(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Exp(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Floor(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Log(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Pow(FixedVector<T,N>& source,T expon);
template<class T,Uint8 N> const FixedVector<T,N>& Round(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Sqr(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Sqrt(FixedVector<T,N>& source);

// trigonometric functions
template<class T,Uint8 N> const FixedVector<T,N>& Acos(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Asin(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Atan(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Cos(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Sin(FixedVector<T,N>& source); 
template<class T,Uint8 N> const FixedVector<T,N>& Tan(FixedVector<T,N>& source);

// hyperbolic functions
template<class T,Uint8 N> const FixedVector<T,N>& Acosh(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Asinh(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Atanh(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Cosh(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Sinh(FixedVector<T,N>& source); 
template<class T,Uint8 N> const FixedVector<T,N>& Tanh(FixedVector<T,N>& source);

// complex vectors
template<class T,Uint8 N> const FixedVector<T,N>& Conj(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Imag(FixedVector<T,N>& source);
template<class T,Uint8 N> const FixedVector<T,N>& Real(FixedVector<T,N>& source);

// not in place 
// arithmetic functions
template<class T,Uint8 N> FixedVector<T,N> abs(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> ceil(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> exp(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> floor(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> log(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> pow(const FixedVector<T,N>& source,T expon);
template<class T,Uint8 N> FixedVector<T,N> round(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> sqr(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> sqrt(const FixedVector<T,N>& source);

// trigonometric functions
template<class T,Uint8 N> FixedVector<T,N> acos(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> asin(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> atan(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> atan2(const FixedVector<T,N>& y,const FixedVector<T,N>& x);
template<class T,Uint8 N> FixedVector<T,N> cos(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> sin(const FixedVector<T,N>& source); 
template<class T,Uint8 N> FixedVector<T,N> tan(const FixedVector<T,N>& source);

// hyperbolic functions
template<class T,Uint8 N> FixedVector<T,N> acosh(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> asinh(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> atanh(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> cosh(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<T,N> sinh(const FixedVector<T,N>& source); 
template<class T,Uint8 N> FixedVector<T,N> tanh(const FixedVector<T,N>& source);
 
// complex vectors
template<class T,Uint8 N> FixedVector<T,N> conj(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<typename DemoteComplex<T>::demote_t,N> imag(const FixedVector<T,N>& source);
template<class T,Uint8 N> FixedVector<typename DemoteComplex<T>::demote_t,N> real(const FixedVector<T,N>& source);

// miscellaneous
template<class T,Uint8 N> FixedVector<T,N> bound(const FixedVector<T,N>& x,T min_val,T max_val);
template<class T,Uint8 N> FixedVector<Binary,N> even(const FixedVector<T,N>& x);
template<class T,Uint8 N> FixedVector<Binary,N> odd(const FixedVector<T,N>& x);
template<class T,Uint8 N> FixedVector<T,N> mod(const FixedVector<T,N>& x,T y);
template<class T,Uint8 N> FixedVector<T,N> mod(const FixedVector<T,N>& x,const FixedVector<T,N>& y);
template<class T,Uint8 N> FixedVector<T,N> quotient(const FixedVector<T,N>& x,T y);
template<class T,Uint8 N> FixedVector<T,N> quotient(const FixedVector<T,N>& x,const FixedVector<T,N>& y);

// boolean
template<class T,Uint8 N> bool all(const FixedVector<T,N>&);
template<class T,Uint8 N> bool any(const FixedVector<T,N>&);

template<class T,Uint8 N> const typename FixedVector<T,N>::Mask is_finite(const FixedVector<T,N>& source);

// vector specific
template<class T,Uint8 N> T dot(const FixedVector<T,N>& lhs,const FixedVector<T,N>& rhs);
template<class T,Uint8 N> T mag(const FixedVector<T,N>& source);
template<class T,Uint8 N> T norm(const FixedVector<T,N>& source);	
template<class T,Uint8 N> T product(const FixedVector<T,N>& source);		
template<class T,Uint8 N> T sum(const FixedVector<T,N>& source);

template<class T,Uint8 N> FixedVector<T,N> unit(const FixedVector<T,N>& source);

// cross product
//	- only defined for FixedVector<T,3>
template<class T,Uint8 N> FixedVector<T,N> cross(const FixedVector<T,N>& lhs,const FixedVector<T,N>& rhs);

} // namespace mcs

#include "MCS_FixedVector1Functions.h"
#include "MCS_FixedVector2Functions.h"
#include "MCS_FixedVector3Functions.h"
#include "MCS_FixedVector4Functions.h"

// implementation
#include "MCS_FixedVectorFunctions.cpp"

#endif // MCS_FIXEDVECTORFUNCTIONS_H
