#ifndef MCS_FIXEDVECTORIO_H
#define MCS_FIXEDVECTORIO_H

#include <iosfwd>

#include "MCS_FixedVector.h"
#include "MCS_IO.h"

namespace mcs {

// IO
template<class T,Uint8 N> std::istream& operator>>(std::istream& is,FixedVector<T,N>& dest);
template<class T,Uint8 N> std::ostream& operator<<(std::ostream& os,const FixedVector<T,N>& source);

template<class T,Uint8 N> void read(std::istream&,FixedVector<T,N>& dest);
template<class T,Uint8 N> void write(std::ostream&,const FixedVector<T,N>& source);

// stream input
template<class T,Uint8 N> 
inline std::istream& operator>>(std::istream& is,FixedVector<T,N>& dest)
{ 
	for (Int32 i=dest.first();i<=dest.last();i++)
		is >> dest[i];
	
	return is; 
}

// stream output
template<class T,Uint8 N> 
inline std::ostream& operator<<(std::ostream& os,const FixedVector<T,N>& source)
{
	// allow this to be changed
	static const Int32 	MAX_OUT_ELEM = 10;
	
	os << "[ ";

	if (source.dim() > MAX_OUT_ELEM)
	{
		for (Int32 i=source.first();i<=source.first()+MAX_OUT_ELEM-1;i++)
			os << source[i] << " ";
			
		os << "... " << source[source.last()] << " ";
	}
	else
	{
		for (Int32 i=source.first();i<=source.last()-1;i++)
			os << source[i] << "  ";
			
		os << source[source.last()] << " ";
	}

	os << "]";
	
	return os;
}

template<class T,Uint8 N>
inline void read(FixedVector<T,N>& dest,std::istream& is)
{
	is.read(reinterpret_cast<char*>(&dest),sizeof(dest));
}

template<class T,Uint8 N>
inline void write(const FixedVector<T,N>& source,std::ostream& os)
{
	os.write(reinterpret_cast<const char*>(&source),sizeof(source));
}

} // namespace mcs

#endif // MCS_FIXEDVECTORIO_H

