#ifndef MCS_FIXEDVECTORSTATISTICS_H
#define MCS_FIXEDVECTORSTATISTICS_H

#include "MCS_Promote.h"
#include "MCS_FixedVector.h"

namespace mcs {

namespace statistics {

namespace normal {

template<class T,Uint8 D> T 				max(const FixedVector<T,D>& source);
template<class T,Uint8 D> FixedVector<T,D> 	max(const FixedVector<T,D>& lhs,const T& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	max(const T& lhs,const FixedVector<T,D>& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	max(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs);

template<class T,Uint8 D> T					min(const FixedVector<T,D>& source);
template<class T,Uint8 D> FixedVector<T,D> 	min(const FixedVector<T,D>& lhs,const T& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	min(const T& lhs,const FixedVector<T,D>& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	min(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs);

template<class T,Uint8 D> T					product(const FixedVector<T,D>& source);
template<class T,Uint8 D> T					sum(const FixedVector<T,D>& source); 		

template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	mean(const FixedVector<T,D>& source);		
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	standardDeviation(const FixedVector<T,D>& source);
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	variance(const FixedVector<T,D>& source);
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	weightedMean(const FixedVector<T,D>& source,const FixedVector<T,D>& weight);

// min/max
template<class T,Uint8 D> T max(const FixedVector<T,D>& source)
{
	T max_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		max_val = std::max(source[i],max_val);
		
	return max_val;
}

template<class T,Uint8 D> FixedVector<T,D> max(const FixedVector<T,D>& lhs,const T& rhs)
{
	FixedVector<T,D> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(lhs[i],rhs);
	
	return tmp;
}

template<class T,Uint8 D> FixedVector<T,D> max(const T& lhs,const FixedVector<T,D>& rhs)
{
	FixedVector<T,D> tmp(rhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(rhs[i],lhs);
	
	return tmp;
}

template<class T,Uint8 D> FixedVector<T,D> max(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs)
{
	FixedVector<T,D> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(lhs[i],rhs[i]);
	
	return tmp;
}

template<class T,Uint8 D> T min(const FixedVector<T,D>& source)
{
	T min_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		min_val = std::min(source[i],min_val);
		
	return min_val;
}

template<class T,Uint8 D> FixedVector<T,D> min(const FixedVector<T,D>& lhs,const T& rhs)
{
	FixedVector<T,D> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs[i],rhs);
	
	return tmp;
}

template<class T,Uint8 D> FixedVector<T,D> min(const T& lhs,const FixedVector<T,D>& rhs)
{
	FixedVector<T,D> tmp(rhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs,rhs[i]);
	
	return tmp;
}

template<class T,Uint8 D> FixedVector<T,D> min(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs)
{		
	FixedVector<T,D> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs[i],rhs[i]);
	
	return tmp;
}

// product of all elements
/*
template<class T,Uint8 D> T product(const FixedVector<T,D>& source)
{
	return mcs::prod(source);
}
*/
// sum of all elements
template<class T,Uint8 D> T sum(const FixedVector<T,D>& source)
{
	return mcs::sum(source);
}

// other statistics
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t mean(const FixedVector<T,D>& source)
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);
	
	for (Int32 i=source.first();i<=source.last();tsum += source[i++]) { };
			
	return tsum/source.dim();
}

// standard deviation
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t standardDeviation(const FixedVector<T,D>& source)
{ 
	return std::sqrt(variance(source)); 
}

// corrected two-pass calculation (NR pp. 613)
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t variance(const FixedVector<T,D>& source)
{ 
	if (D < 2) return PromoteFloat<T>::promote_t(_NaN_);
		
	typename PromoteFloat<T>::promote_t xbar = mean(source),
										sum1(0),
										sum2(0);
	
	for (Int32 i=source.first();i<=source.last();i++)
	{
		const typename PromoteFloat<T>::promote_t	tmp(source[i]-xbar);
		
		sum2 += tmp*tmp;
		sum1 += tmp;
	}
		
	return (sum2 - sum1*sum1/source.dim())/(source.dim()-1); 
}

template<class T,Uint8 D> typename PromoteFloat<T>::promote_t weightedMean(const FixedVector<T,D>& source,const FixedVector<T,D>& weight) 
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		tsum += source[i]*weight[i];
		norm += weight[i];
	}
				
	return tsum/norm;
}

} // namespace normal

namespace ignoreNaN {

template<class T,Uint8 D> T 				max(const FixedVector<T,D>& source);
template<class T,Uint8 D> FixedVector<T,D> 	max(const FixedVector<T,D>& lhs,const T& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	max(const T& lhs,const FixedVector<T,D>& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	max(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs);

template<class T,Uint8 D> T 				min(const FixedVector<T,D>& source);
template<class T,Uint8 D> FixedVector<T,D> 	min(const FixedVector<T,D>& lhs,const T& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	min(const T& lhs,const FixedVector<T,D>& rhs);
template<class T,Uint8 D> FixedVector<T,D> 	min(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs);

template<class T,Uint8 D> T					product(const FixedVector<T,D>& source);
template<class T,Uint8 D> T 				sum(const FixedVector<T,D>& source); 
		
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	mean(const FixedVector<T,D>& source);		
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	standardDeviation(const FixedVector<T,D>& source);
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	variance(const FixedVector<T,D>& source);
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t	weightedMean(const FixedVector<T,D>& source,const FixedVector<T,D>& weight);

// min/max
template<class T,Uint8 D> T max(const FixedVector<T,D>& source)
{
	T max_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		if (isfinite(source[i]))
			max_val = std::max(source[i],max_val);
		
	return max_val;
}

template<class T,Uint8 D> FixedVector<T,D> max(const FixedVector<T,D>& lhs,const T& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T,Uint8 D> FixedVector<T,D> max(const T& lhs,const FixedVector<T,D>& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T,Uint8 D> FixedVector<T,D> max(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T,Uint8 D> T min(const FixedVector<T,D>& source)
{
	T min_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		if (isfinite(source[i]))
			min_val = std::min(source[i],min_val);
		
	return min_val;
}

template<class T,Uint8 D> FixedVector<T,D> min(const FixedVector<T,D>& lhs,const T& rhs)
{
	return normal::min(lhs,rhs);
}

template<class T,Uint8 D> FixedVector<T,D> min(const T& lhs,const FixedVector<T,D>& rhs)
{
	return normal::min(lhs,rhs);
}

template<class T,Uint8 D> FixedVector<T,D> min(const FixedVector<T,D>& lhs,const FixedVector<T,D>& rhs)
{
	return normal::min(lhs,rhs);
}

// product of all elements
template<class T,Uint8 D> T product(const FixedVector<T,D>& source)
{
	T tprod(1);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i])) 
			tprod *= source[i];
				
	return tprod;
}

// sum of all elements
template<class T,Uint8 D> T sum(const FixedVector<T,D>& source)
{
	T tsum(0);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i])) 
			tsum += source[i];
				
	return tsum;
}

// other statistics
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t mean(const FixedVector<T,D>& source)
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		if (isfinite(source[i]))
		{
			tsum += source[i];
			norm += 1;
		}
	}
		
	return tsum/norm;
}

// standard deviation
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t standardDeviation(const FixedVector<T,D>& source)
{ 
	return std::sqrt(variance(source)); 
}

// corrected two-pass calculation (NR pp. 613)
template<class T,Uint8 D> typename PromoteFloat<T>::promote_t variance(const FixedVector<T,D>& source)
{ 
	if (D < 2) return PromoteFloat<T>::promote_t(_NaN_);
		
	typename PromoteFloat<T>::promote_t xbar(mean(source)),
										sum1(0),
										sum2(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		const typename PromoteFloat<T>::promote_t tmp(source[i]-xbar);
		
		if (isfinite(tmp))
		{
			sum2 += tmp*tmp;
			sum1 += tmp;
			norm += 1;
		}
	}
	
	return (sum2 - sum1*sum1/norm)/(norm-1); 
}

template<class T,Uint8 D> typename PromoteFloat<T>::promote_t weightedMean(const FixedVector<T,D>& source,const FixedVector<T,D>& weight) 
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i]) && isfinite(weight[i]))
		{
			tsum += source[i]*weight[i];
			norm += weight[i];
		}

	return tsum/norm;
}

} // namespace ignoreNaN

} // namespace statistics

} // namespace mcs

#endif // MCS_FIXEDVECTORSTATISTICS_H
