#ifndef MCS_BRACKET_H
#define MCS_BRACKET_H

#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"

namespace mcs {

// bracket a point in D dimensions
template<class T,Uint8 D> 
class Bracket
{	
	public:
		Bracket(const FixedVector<T,D>&,const FixedVector<T,D>&);
		
		const FixedVector<T,D>& lo() const	{ return lo_; }
		const FixedVector<T,D>& hi() const	{ return hi_; }
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		FixedVector<T,D>	lo_,hi_; 
};

// implementation
template<class T,Uint8 D> 
inline Bracket<T,D>::Bracket(const FixedVector<T,D>& lo,const FixedVector<T,D>& hi) :
	lo_(lo),
	hi_(hi)
{
	//if (any(lo_ >= hi_)) throw Err();
}

template<class T> 
inline FixedVector<T,1> left(const Bracket<T,1>& brak)
{
	return FixedVector<T,1>(brak.lo());
}

template<class T> 
inline FixedVector<T,1> right(const Bracket<T,1>& brak)
{
	return FixedVector<T,1>(brak.hi());
}

template<class T> 
inline FixedVector<T,2> lowerLeft(const Bracket<T,2>& brak)
{
	return FixedVector<T,2>(brak.lo()[0],brak.lo()[1]);
}

template<class T> 
inline FixedVector<T,2> lowerRight(const Bracket<T,2>& brak)
{
	return FixedVector<T,2>(brak.hi()[0],brak.lo()[1]);
}

template<class T> 
inline FixedVector<T,2> upperRight(const Bracket<T,2>& brak)
{
	return FixedVector<T,2>(brak.hi()[0],brak.hi()[1]);
}

template<class T> 
inline FixedVector<T,2> upperLeft(const Bracket<T,2>& brak)
{
	return FixedVector<T,2>(brak.lo()[0],brak.hi()[1]);
}

} // namespace mcs

#endif // MCS_BRACKET_H