#ifndef MCS_BYTESWAP_H
#define MCS_BYTESWAP_H

#include "MCS_Typedefs.h"

namespace mcs {

// utility routine to swap between big-endian and little-endian

template<class T,class U>
inline T memoryAs(const U* ptr)
{ return *(reinterpret_cast<const T*>(ptr)); }

// mutating
template<class T> 
inline void byteSwap(T&)
{ return; }

// non-mutating
template<class T> 
inline T byteSwap(const T& source)
{ 
	T	temp(source);
	
	byteSwap(temp);
	
	return temp;
}

// specializations
template<> 
inline void byteSwap(Int8&)
{ return; }

template<> 
inline void byteSwap(Uint8&)
{ return; }

template<> 
inline void byteSwap(Int16& source)
{ 
	char*	temp = reinterpret_cast<char *>(&source);
	
	std::swap(temp[0],temp[1]);
	
	return;
}

template<> 
inline void byteSwap(Uint16& source)
{ 
	char*	temp = reinterpret_cast<char *>(&source);
	
	std::swap(temp[0],temp[1]);
	
	return;
}

template<> 
inline void byteSwap(Int32& source)
{ 
	char*	temp = reinterpret_cast<char *>(&source);
	
	std::swap(temp[0],temp[3]);
	std::swap(temp[1],temp[2]);
	
	return;
}

template<> 
inline void byteSwap(Uint32& source)
{ 
	char*	temp = reinterpret_cast<char *>(&source);
	
	std::swap(temp[0],temp[3]);
	std::swap(temp[1],temp[2]);
	
	return;
}

template<> 
inline void byteSwap(Float32& source)
{ 
	char*	temp = reinterpret_cast<char *>(&source);
	
	std::swap(temp[0],temp[3]);
	std::swap(temp[1],temp[2]);
	
	return;
}

template<> 
inline void byteSwap(Float64& source)
{ 
	char*	temp = reinterpret_cast<char *>(&source);
	
	std::swap(temp[0],temp[7]);
	std::swap(temp[1],temp[6]);
	std::swap(temp[2],temp[5]);
	std::swap(temp[3],temp[4]);
	
	return;
}

template<> 
inline void byteSwap(Complex32& source)
{
	Float32	*reptr = reinterpret_cast<Float32*>(&source),
			*imptr = reinterpret_cast<Float32*>(&source)+sizeof(Float32);
	
	byteSwap(*reptr);
	byteSwap(*imptr);
}

template<> 
inline void byteSwap(Complex64& source)
{
	Float64	*reptr = reinterpret_cast<Float64*>(&source),
			*imptr = reinterpret_cast<Float64*>(&source)+sizeof(Float64);
	
	byteSwap(*reptr);
	byteSwap(*imptr);
}

inline void byteSwap(char* data,Int32 num)
{	
	for (Int32 i=0;i<num/2;i++)
		std::swap(data[i],data[num-i-1]);
}

} // namespace mcs

#endif // MCS_BYTESWAP_H