#ifndef MCS_CONSTANTS_H
#define MCS_CONSTANTS_H

#include "MCS_MathExtensions.h"
#include "MCS_Typedefs.h"

namespace mcs {

namespace constants {

namespace physical {

static const Float64 	// radiation constants
						_c_				= 2.99792458e8,		// velocity of light (m/s)
						_h_				= 6.6260755e-34,	// Planck constant (J s)
						_h_bar_			= _h_/_two_pi_,		// Dirac constant (J s)
						_k_B_			= 1.380658e-23,		// Boltzmann constant (J/K)
						_c_rad_1_		= 1.1910427e-16,	// first radiation constant (W m^2/sr)
						_c_rad_2_ 		= 1.4387752e-2,		// second radiation constant (m K)
						_sigma_SB_		= 5.67051e-8,		// Stefan-Boltzmann constant (W/m^2/K^4)
						// gas constants
						_N_A_			= 6.0221367e23,		// Avogadro's number (/mol)
						_R_gas_			= _N_A_*_k_B_,		// Ideal gas constant (J/mol/K)
						_atm_			= 1.01325e5,		// standard atmosphere (Pa)
						_M_W_air_		= 2.8966e-2,		// molecular weight of dry air (kg/mol)
						_M_W_water_		= 1.8015324e-2,		// molecular weight of water (kg/mol)
						_c_p_air_		= 1.005e3,			// heat capacity of dry air (J/kg/K)
						_c_p_water_		= 1.85e3,			// heat capacity of water (J/kg/K)
						_l_c_			= 2.5e6,			// latent heat of condensation (J/kg) @ 273K
						_T_MP_			= 273.16,			// melting point of water (K)
						_T_BP_			= _T_MP_ + 100.0,	// boiling point of water (K)
						// astronomical constants
						_G_				= 6.67259e-11,		// gravitational constant (N m^2/kg^2)
						_g_0_			= 9.80665,			// Earth's gravitational acceleration (m/s^2)
						_m_E_			= 5.97370e24,		// Earth's mass (kg)
						_r_E_			= 6.371009e6,		// Earth's mean radius (m)
						_r_Ep_			= 6.3567523142e6,	// Earth's polar radius (m)
						_r_Ee_			= 6.378137e6,		// Earth's equatorial radius (m)
						_oblateness_	= 1.0-_r_Ep_/_r_Ee_,// Earth's oblateness
						_S_sol_			= 1.368e3,			// solar constant (W/m^2)
						_r_sol_			= 6.9595e8,			// solar radius (visible disk) (m)
						_d_sol_			= 1.4956e11,		// mean Earth-sun distance (m)
						_T_BC_			= 2.73,				// cosmological background temperature (K)
						// conversions
						_mbar_p_Pa_		= 1.0e-2,			// millibars per Pa
						_naper_p_dB_	= 0.2302585094e-3,	// naper/cm per dB/km
						_degC_K_		= _T_MP_,			// convert Celsius to Kelvin
						// meteorological constants
						_eps_			= 0.622,
						_gamma_d_		= _g_0_/_c_p_air_,	// dry adiabatic lapse rate
						_P_0_			= 1013.25;			// surface pressure (mbar)
						
} // namespace physical

} // namespace constants

} // namespace mcs

#endif // MCS_CONSTANTS_H