#ifndef MCS_DATEITERATOR_H
#define MCS_DATEITERATOR_H

#include "MCS_Exceptions.h"
#include "MCS_IntIndexed.h"
#include "MCS_Iterator.h"

namespace mcs {

class HalfDayIndexed : public IntIndexed
{
	public:
		HalfDayIndexed();
		HalfDayIndexed(const Date& ep);
		HalfDayIndexed(const HalfDayIndexed&);
		~HalfDayIndexed() { }
		
		HalfDayIndexed& operator=(const HalfDayIndexed&);
};

inline HalfDayIndexed::HalfDayIndexed() :
	IntIndexed(halfDayNumber(Date()))
{ }

inline HalfDayIndexed::HalfDayIndexed(const Date& ep) :
	IntIndexed(halfDayNumber(ep))
{ }

inline HalfDayIndexed::HalfDayIndexed(const HalfDayIndexed& source) :
	IntIndexed(source)
{ }

inline HalfDayIndexed& HalfDayIndexed::operator=(const HalfDayIndexed& source)
{
	if (this == &source) return *this;
	
	IntIndexed::operator=(source);
	
	return *this; 
}

class FixedDayIndexed : public IntIndexed
{
	public:
		FixedDayIndexed();
		FixedDayIndexed(const Date& ep);
		FixedDayIndexed(const FixedDayIndexed&);
		~FixedDayIndexed() { }
		
		FixedDayIndexed& operator=(const FixedDayIndexed&);
};

inline FixedDayIndexed::FixedDayIndexed() :
	IntIndexed(Date().fixed())
{ }

inline FixedDayIndexed::FixedDayIndexed(const Date& ep) :
	IntIndexed(ep.fixed())
{ }

inline FixedDayIndexed::FixedDayIndexed(const FixedDayIndexed& source) :
	IntIndexed(source)
{ }

inline FixedDayIndexed& FixedDayIndexed::operator=(const FixedDayIndexed& source)
{
	if (this == &source) return *this;
	
	IntIndexed::operator=(source);
	
	return *this; 
}

} // namespace mcs

#endif // MCS_DATEITERATOR_H