#ifndef MCS_DEREFERENCER_H
#define MCS_DEREFERENCER_H

#include "MCS_Exceptions.h"

namespace mcs {

// Iterator dereferencing class
//
//	Handles all details involved in dereferencing iterator
//
// 		class should be specialized for user defined classes with 
//			particular requirements
template<class P,class Imp> class Dereferencer
{
	public:
		Dereferencer(const Imp& imp = Imp());
		Dereferencer(const Dereferencer<P,Imp>& source);
		~Dereferencer() { }
		
		P* operator->()		{ return ptr_.operator->(); }
		
		const P& dereference(const Imp& imp);
		
	private:
		bool				initialized_;
		Imp					imp_;			
		std::auto_ptr<P>	ptr_;
};

template<class P,class Imp>
inline Dereferencer<P,Imp>::Dereferencer(const Imp& imp) :
	initialized_(false),
	imp_(imp),
	ptr_()
{ }

template<class P,class Imp>
inline Dereferencer<P,Imp>::Dereferencer(const Dereferencer<P,Imp>& source) :
	initialized_(false),
	imp_(source.imp_),
	ptr_()
{ }

template<class P,class Imp>
inline const P& Dereferencer<P,Imp>::dereference(const Imp& imp) 
{
	// if new imp, initialize ptr_, otherwise just return existing ptr_
	if (!initialized_ || imp_ != imp) 
	{
		ptr_ = std::auto_ptr<P>(new P(imp));
		imp_ = imp;
		initialized_ = true;
	}
	
	return *ptr_;
}

} // namespace mcs

#endif // MCS_DEREFERENCER_H