#ifndef MCS_EXCEPTIONS_H
#define MCS_EXCEPTIONS_H

#include <sstream>
#include <string>

#include "MCS_Typedefs.h"

#define __DECLARE_EXCEPTION_CLASS__(cls,der) \
class cls : \
	public der \
{ \
	public: \
		cls() : der() { } \
		cls(const Int32& codeval) : der(codeval) { } \
		cls(const std::string& mesg) : der(mesg) { } \
		cls(const Int32& codeval,const std::string& mesg) : der(codeval,mesg) { } \
		~cls() { } \
};

namespace mcs {

class Exception
{
	public:
		Exception();
		Exception(const Int32& codeval);
		Exception(const std::string& mesg);
		Exception(const Int32& codeval,const std::string& mesg);
		virtual ~Exception();
		
		Int32 code() const				{ return code_; }
		std::string message() const 	{ return name()+message_; }
		std::string name() const		{ return typeid(*this).name(); }
	
	private:
		void init();
		
		Int32			code_;
		std::string		message_; 
};

inline Exception::Exception() :
	code_(0),
	message_("")
{ 
	init();
}

inline Exception::Exception(const Int32& codeval) : 
	code_(codeval),
	message_("")
{ 
	init();
}

inline Exception::Exception(const std::string& mesg) : 
	code_(0),
	message_(mesg)
{ 
	init();
}

inline Exception::Exception(const Int32& codeval,const std::string& mesg) : 
	code_(codeval),
	message_(mesg)
{ 
	init();
}

inline Exception::~Exception()
{ }

inline void Exception::init()
{
	std::stringstream	sstr;
	
	sstr << " exception thrown : "
		 << "code ("
		 << code_
		 << "), "
		 << message_;
	
	message_ = sstr.str();;
}

} // namespace mcs

#endif // MCS_EXCEPTIONS_H