#ifndef MCS_FUNCTOR_H
#define MCS_FUNCTOR_H

#include "MCS_FixedVector.h"

namespace mcs {

// FunctorN takes N arguments of arbitrary type and
//	returns a single value of another type
template<class Ret> 
class Functor0
{
	public:
		virtual ~Functor0() { }
		
		virtual Ret operator()(void) const = 0;
};

template<class Arg1,class Ret> 
class Functor1
{
	public:
		virtual ~Functor1() { }
		
		virtual Ret operator()(Arg1) const = 0;
};

template<class Arg1,class Arg2,class Ret> 
class Functor2
{
	public:
		virtual ~Functor2() { }
		
		virtual Ret operator()(Arg1,Arg2) const = 0;
};

template<class Arg1,class Arg2,class Arg3,class Ret> 
class Functor3
{
	public:
		virtual ~Functor3() { }
		
		virtual Ret operator()(Arg1,Arg2,Arg3) const = 0;
};

template<class Arg1,class Arg2,class Arg3,class Arg4,class Ret> 
class Functor4
{
	public:
		virtual ~Functor4() { }
		
		virtual Ret operator()(Arg1,Arg2,Arg3,Arg4) const = 0;
};

template<class Arg,Uint8 NArg,class Ret,Uint8 NRet> 
class NaryFunctor
{
	public:
		virtual ~NaryFunctor() { }
		
		virtual FixedVector<Ret,NRet> operator()(const FixedVector<Arg,NArg>&) const = 0;
};

// wrapper classes to convert normal function pointer to functor
template<class Ret> 
class Func0 : public Functor0<Ret>
{
	public:
		typedef Ret (*FunctionPtr)(void);
		
		Func0(FunctionPtr);
		~Func0() { }
		
		FunctionPtr function() const	{ return func_; }
		
		Ret operator()(void) const		{ return (*func_)(); }
		
	private:
		FunctionPtr	func_;
};

template<class Ret> 
inline Func0<Ret>::Func0(FunctionPtr func) :
	func_(func)
{ }
		
template<class Arg1,class Ret> 
class Func1 : public Functor1<Arg1,Ret>
{
	public:
		typedef Ret (*FunctionPtr)(Arg1);
		
		Func1(FunctionPtr);
		~Func1() { }
		
		FunctionPtr function() const	{ return func_; }
		
		Ret operator()(Arg1 arg1) const	{ return (*func_)(arg1); }
		
	private:
		FunctionPtr	func_;
};

template<class Arg1,class Ret> 
inline Func1<Arg1,Ret>::Func1(FunctionPtr func) :
	func_(func)
{ }

template<class Arg1,class Arg2,class Ret> 
class Func2 : public Functor2<Arg1,Arg2,Ret>
{
	public:
		typedef Ret (*FunctionPtr)(Arg1,Arg2);
		
		Func2(FunctionPtr);
		~Func2() { }
		
		FunctionPtr function() const	{ return func_; }
		
		Ret operator()(Arg1 arg1,Arg2 arg2) const	{ return (*func_)(arg1,arg2); }
		
	private:
		FunctionPtr	func_;
};

template<class Arg1,class Arg2,class Ret> 
inline Func2<Arg1,Arg2,Ret>::Func2(FunctionPtr func) :
	func_(func)
{ }

template<class Arg1,class Arg2,class Arg3,class Ret> 
class Func3 : public Functor3<Arg1,Arg2,Arg3,Ret>
{
	public:
		typedef Ret (*FunctionPtr)(Arg1,Arg2,Arg3);
		
		Func3(FunctionPtr);
		~Func3() { }
		
		FunctionPtr function() const	{ return func_; }
		
		Ret operator()(Arg1 arg1,Arg2 arg2,Arg3 arg3) const	{ return (*func_)(arg1,arg2,arg3); }
		
	private:
		FunctionPtr	func_;
};

template<class Arg1,class Arg2,class Arg3,class Ret> 
inline Func3<Arg1,Arg2,Arg3,Ret>::Func3(FunctionPtr func) :
	func_(func)
{ }

template<class Arg1,class Arg2,class Arg3,class Arg4,class Ret> 
class Func4 : public Functor4<Arg1,Arg2,Arg3,Arg4,Ret>
{
	public:
		typedef Ret (*FunctionPtr)(Arg1,Arg2,Arg3,Arg4);
		
		Func4(FunctionPtr);
		~Func4() { }
		
		FunctionPtr function() const	{ return func_; }
		
		Ret operator()(Arg1 arg1,Arg2& arg2,Arg3 arg3,Arg4 arg4) const	{ return (*func_)(arg1,arg2,arg3,arg4); }
		
	private:
		FunctionPtr	func_;
};

template<class Arg1,class Arg2,class Arg3,class Arg4,class Ret> 
inline Func4<Arg1,Arg2,Arg3,Arg4,Ret>::Func4(FunctionPtr func) :
	func_(func)
{ }

} // namespace mcs

#endif // MCS_FUNCTOR_H