#ifndef MCS_INDEX_H
#define MCS_INDEX_H

#include "MCS_FixedVector.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<Uint8 D> 
class Index : public FixedVector<Int32,D>
{
	public:
		Index() : FixedVector<Int32,D>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,D>(idx) { }
		explicit Index(const std::string& str) : FixedVector<Int32,D>(str) { }
		Index(const FixedVector<Int32,D>& source) : FixedVector<Int32,D>(source) { }
		Index(const Index<D>& source) : FixedVector<Int32,D>(source) { }
		~Index() { }
		
		Index<D>& operator=(Int32);
		Index<D>& operator=(const std::string&);
		Index<D>& operator=(const FixedVector<Int32,D>&);
		Index<D>& operator=(const Index<D>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// specializations
// vector index -> behaves like Int32
template<> 
class Index<1> : public FixedVector<Int32,1>
{
	public:
		Index() : FixedVector<Int32,1>() { }
		Index(Int32 idx) : FixedVector<Int32,1>(idx) { }
		explicit Index(const std::string& str) : FixedVector<Int32,1>(str) { }
		Index(const FixedVector<Int32,1>& source) : FixedVector<Int32,1>(source) { }
		Index(const Index<1>& source) : FixedVector<Int32,1>(source) { }
		~Index() { }
		
		Index<1>& operator=(const Int32&);
		Index<1>& operator=(const std::string&);
		Index<1>& operator=(const FixedVector<Int32,1>&);
		Index<1>& operator=(const Index<1>&);
		
//		Int32& rank(Uint8 d) 					{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const				{ return (*this)[d]; }
		
		// unique to Index<1>
//		operator Int32() const 					{ return (*this)[0]; }
		
		// increment/decrement
		Index<1>& operator--()					{ --((*this)[0]); return *this; }
		const Index<1> operator--(int)			{ const Index<1> tmp(*this); ((*this)[0])--; return tmp; }
		Index<1>& operator++()					{ ++((*this)[0]); return *this; }
		const Index<1> operator++(int)			{ const Index<1> tmp(*this); ((*this)[0])++; return tmp; }
};

// matrix index 
template<> 
class Index<2> : public FixedVector<Int32,2>
{
	public:
		Index() : FixedVector<Int32,2>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,2>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2) : FixedVector<Int32,2>(idx1,idx2) { }
		explicit Index(const std::string& str) : FixedVector<Int32,2>(str) { }
		Index(const FixedVector<Int32,2>& source) : FixedVector<Int32,2>(source) { }
	 	Index(const Index<2>& source) : FixedVector<Int32,2>(source) { }
		~Index() { }
		
		Index<2>& operator=(Int32);
		Index<2>& operator=(const std::string&);
		Index<2>& operator=(const FixedVector<Int32,2>&);
		Index<2>& operator=(const Index<2>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 3 tensor index 
template<> 
class Index<3> : public FixedVector<Int32,3>
{
	public:
		Index() : FixedVector<Int32,3>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,3>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3) : FixedVector<Int32,3>(idx1,idx2,idx3) { }
		explicit Index(const std::string& str) : FixedVector<Int32,3>(str) { }
		Index(const FixedVector<Int32,3>& source) : FixedVector<Int32,3>(source) { }
		Index(const Index<3>& source) : FixedVector<Int32,3>(source) { }
		~Index() { }
		
		Index<3>& operator=(Int32);
		Index<3>& operator=(const std::string&);
		Index<3>& operator=(const FixedVector<Int32,3>&);
		Index<3>& operator=(const Index<3>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 4 tensor index 
template<> 
class Index<4> : public FixedVector<Int32,4>
{
	public:
		Index() : FixedVector<Int32,4>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,4>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4) : FixedVector<Int32,4>(idx1,idx2,idx3,idx4) { }
		explicit Index(const std::string& str) : FixedVector<Int32,4>(str) { }
		Index(const FixedVector<Int32,4>& source) : FixedVector<Int32,4>(source) { }
		Index(const Index<4>& source) : FixedVector<Int32,4>(source) { }
		~Index() { }
		
		Index<4>& operator=(Int32);
		Index<4>& operator=(const std::string&);
		Index<4>& operator=(const FixedVector<Int32,4>&);
		Index<4>& operator=(const Index<4>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 5 tensor index 
template<> 
class Index<5> : public FixedVector<Int32,5>
{
	public:
		Index() : FixedVector<Int32,5>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,5>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4,Int32 idx5) : FixedVector<Int32,5>(idx1,idx2,idx3,idx4,idx5) { }
		explicit Index(const std::string& str) : FixedVector<Int32,5>(str) { }
		Index(const FixedVector<Int32,5>& source) : FixedVector<Int32,5>(source) { }
		Index(const Index<5>& source) : FixedVector<Int32,5>(source) { }
		~Index() { }
		
		Index<5>& operator=(Int32);
		Index<5>& operator=(const std::string&);
		Index<5>& operator=(const FixedVector<Int32,5>&);
		Index<5>& operator=(const Index<5>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 6 tensor index 
template<> 
class Index<6> : public FixedVector<Int32,6>
{
	public:
		Index() : FixedVector<Int32,6>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,6>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4,Int32 idx5,Int32 idx6) : 
			FixedVector<Int32,6>(idx1,idx2,idx3,idx4,idx5,idx6) { }
		explicit Index(const std::string& str) : FixedVector<Int32,6>(str) { }
		Index(const FixedVector<Int32,6>& source) : FixedVector<Int32,6>(source) { }
		Index(const Index<6>& source) : FixedVector<Int32,6>(source) { }
		~Index() { }
		
		Index<6>& operator=(Int32);
		Index<6>& operator=(const std::string&);
		Index<6>& operator=(const FixedVector<Int32,6>&);
		Index<6>& operator=(const Index<6>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 7 tensor index 
template<> 
class Index<7> : public FixedVector<Int32,7>
{
	public:
		Index() : FixedVector<Int32,7>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,7>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4,Int32 idx5,Int32 idx6,Int32 idx7) : 
			FixedVector<Int32,7>(idx1,idx2,idx3,idx4,idx5,idx6,idx7) { }
		explicit Index(const std::string& str) : FixedVector<Int32,7>(str) { }
		Index(const FixedVector<Int32,7>& source) : FixedVector<Int32,7>(source) { }
		Index(const Index<7>& source) : FixedVector<Int32,7>(source) { }
		~Index() { }
		
		Index<7>& operator=(Int32);
		Index<7>& operator=(const std::string&);
		Index<7>& operator=(const FixedVector<Int32,7>&);
		Index<7>& operator=(const Index<7>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 8 tensor index 
template<> 
class Index<8> : public FixedVector<Int32,8>
{
	public:
		Index() : FixedVector<Int32,8>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,8>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4,Int32 idx5,Int32 idx6,Int32 idx7,Int32 idx8) : 
			FixedVector<Int32,8>(idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8) { }
		explicit Index(const std::string& str) : FixedVector<Int32,8>(str) { }
		Index(const FixedVector<Int32,8>& source) : FixedVector<Int32,8>(source) { }
		Index(const Index<8>& source) : FixedVector<Int32,8>(source) { }
		~Index() { }
		
		Index<8>& operator=(Int32);
		Index<8>& operator=(const std::string&);
		Index<8>& operator=(const FixedVector<Int32,8>&);
		Index<8>& operator=(const Index<8>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 9 tensor index 
template<> 
class Index<9> : public FixedVector<Int32,9>
{
	public:
		Index() : FixedVector<Int32,9>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,9>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4,Int32 idx5,Int32 idx6,Int32 idx7,Int32 idx8,Int32 idx9) : 
			FixedVector<Int32,9>(idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8,idx9) { }
		explicit Index(const std::string& str) : FixedVector<Int32,9>(str) { }
		Index(const FixedVector<Int32,9>& source) : FixedVector<Int32,9>(source) { }
		Index(const Index<9>& source) : FixedVector<Int32,9>(source) { }
		~Index() { }
		
		Index<9>& operator=(Int32);
		Index<9>& operator=(const std::string&);
		Index<9>& operator=(const FixedVector<Int32,9>&);
		Index<9>& operator=(const Index<9>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// rank 10 tensor index 
template<> 
class Index<10> : public FixedVector<Int32,10>
{
	public:
		Index() : FixedVector<Int32,10>() { }
		explicit Index(Int32 idx) : FixedVector<Int32,10>(idx) { }
		explicit Index(Int32 idx1,Int32 idx2,Int32 idx3,Int32 idx4,Int32 idx5,Int32 idx6,Int32 idx7,Int32 idx8,Int32 idx9,Int32 idx10) : 
			FixedVector<Int32,10>(idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8,idx9,idx10) { }
		explicit Index(const std::string& str) : FixedVector<Int32,10>(str) { }
		Index(const FixedVector<Int32,10>& source) : FixedVector<Int32,10>(source) { }
		Index(const Index<10>& source) : FixedVector<Int32,10>(source) { }
		~Index() { }
		
		Index<10>& operator=(Int32);
		Index<10>& operator=(const std::string&);
		Index<10>& operator=(const FixedVector<Int32,10>&);
		Index<10>& operator=(const Index<10>&);
		
//		Int32& rank(Uint8 d) 		{ return (*this)[d]; }
//		Int32 rank(Uint8 d) const	{ return (*this)[d]; }
};

// implementation
#include "MCS_Index.cpp"

} // namespace mcs

#endif // MCS_INDEX_H