#ifndef MCS_INTINDEXED_H
#define MCS_INTINDEXED_H

namespace mcs {

// base class for integer indexed iterator implementation classes
class IntIndexed
{
	public:
		IntIndexed() : val_(0) { }
		explicit IntIndexed(const Int32& val) : val_(val) { }
		IntIndexed(const IntIndexed& source) : val_(source.val_) { }
		~IntIndexed() { }
				
		IntIndexed& operator=(const IntIndexed&);
		
		bool		operator==(const IntIndexed& it) const		{ return val_ == it.val_; }
		bool		operator!=(const IntIndexed& it) const		{ return val_ != it.val_; }
		bool		operator<(const IntIndexed& it) const		{ return val_ < it.val_; }
		bool		operator<=(const IntIndexed& it) const		{ return val_ <= it.val_; }
		bool		operator>(const IntIndexed& it) const		{ return val_ > it.val_; }
		bool		operator>=(const IntIndexed& it) const		{ return val_ >= it.val_; }
		
		IntIndexed&	operator+=(const Int32& num)				{ val_ += num; return *this; }
		IntIndexed&	operator-=(const Int32& num)				{ val_ -= num; return *this; }
		
		Int32		operator-(IntIndexed& it) const				{ return val_-it.val_; }
		Int32		operator-(const IntIndexed& it) const		{ return val_-it.val_; }
		
		IntIndexed&	operator++() 								{ ++val_; return *this; }
		IntIndexed&	operator--()								{ --val_; return *this; }
		const IntIndexed	operator++(int)						{ IntIndexed tmp(*this); ++val_; return tmp; }
		const IntIndexed	operator--(int)						{ IntIndexed tmp(*this); --val_; return tmp; }
		
		IntIndexed	operator+(const Int32& num)	const			{ IntIndexed tmp(*this); tmp += num; return tmp; }
		IntIndexed	operator-(const Int32& num)	const			{ IntIndexed tmp(*this); tmp -= num; return tmp; }
		
		const Int32& value() const								{ return val_; }
		
	private:		
		Int32	val_;
};

inline IntIndexed& IntIndexed::operator=(const IntIndexed& it)
{
	if (this == &it) return *this;
	
	val_ = it.val_;
	
	return *this;
}

} // namespace mcs

#endif // MCS_INTINDEXED_H