#ifndef MCS_ORDERED_H
#define MCS_ORDERED_H

#include "MCS_Date.h"
#include "MCS_Typedefs.h"

namespace mcs {

template<class T> 
class Ordered
{
	public:
		typedef T	OrdinalType;
		
		// constructor/destructor
		Ordered(const T& val = T()) : value_(val) { }
		virtual ~Ordered() { }
				
		const T& value() const		{ return value_; }
		
		void value(const T& val)	{ value_ = val; }
		
	protected:	
		T	value_;
};

// specialize for date ordered types, providing epoch() call as a convenience
template<> 
class Ordered<Date>
{
	public:
		typedef Date	OrdinalType;
		
		// constructor/destructor
		Ordered(const Date& val = Date()) : value_(val) { }
		virtual ~Ordered() { }
				
		const Date& value() const		{ return value_; }
		const Date& epoch() const		{ return value_; }
		
		void value(const Date& val)		{ value_ = val; }
		void epoch(const Date& val)		{ value_ = val; }
	
	protected:	
		Date	value_;
};

// comparison operators
template<class T> inline bool operator==(const Ordered<T>& lhs,const T& rhs) 			{ return (lhs.value()==rhs); }
template<class T> inline bool operator!=(const Ordered<T>& lhs,const T& rhs) 			{ return (lhs.value()!=rhs); }
template<class T> inline bool operator<(const Ordered<T>& lhs,const T& rhs) 			{ return (lhs.value()<rhs);  }
template<class T> inline bool operator<=(const Ordered<T>& lhs,const T& rhs) 			{ return (lhs.value()<=rhs); }
template<class T> inline bool operator>(const Ordered<T>& lhs,const T& rhs) 			{ return (lhs.value()>rhs);  }
template<class T> inline bool operator>=(const Ordered<T>& lhs,const T& rhs) 			{ return (lhs.value()>=rhs); }
			  		  
template<class T> inline bool operator==(const T& lhs,const Ordered<T>& rhs) 			{ return (lhs==rhs.value()); }
template<class T> inline bool operator!=(const T& lhs,const Ordered<T>& rhs) 			{ return (lhs!=rhs.value()); }
template<class T> inline bool operator<(const T& lhs,const Ordered<T>& rhs) 			{ return (lhs<rhs.value());  }
template<class T> inline bool operator<=(const T& lhs,const Ordered<T>& rhs) 			{ return (lhs<=rhs.value()); }
template<class T> inline bool operator>(const T& lhs,const Ordered<T>& rhs) 			{ return (lhs>rhs.value());  }
template<class T> inline bool operator>=(const T& lhs,const Ordered<T>& rhs) 			{ return (lhs>=rhs.value()); }
			  		  
template<class T> inline bool operator==(const Ordered<T>& lhs,const Ordered<T>& rhs) 	{ return (lhs.value()==rhs.value()); }
template<class T> inline bool operator!=(const Ordered<T>& lhs,const Ordered<T>& rhs)	{ return (lhs.value()!=rhs.value()); }
template<class T> inline bool operator<(const Ordered<T>& lhs,const Ordered<T>& rhs) 	{ return (lhs.value()<rhs.value());  }
template<class T> inline bool operator<=(const Ordered<T>& lhs,const Ordered<T>& rhs) 	{ return (lhs.value()<=rhs.value()); }
template<class T> inline bool operator>(const Ordered<T>& lhs,const Ordered<T>& rhs) 	{ return (lhs.value()>rhs.value());  }
template<class T> inline bool operator>=(const Ordered<T>& lhs,const Ordered<T>& rhs) 	{ return (lhs.value()>=rhs.value()); }

} // namespace mcs

#endif // MCS_ORDERED_H