#ifndef MCS_SINGLETON_H
#define MCS_SINGLETON_H

#include <memory>

#include "MCS_Tag.h"

namespace mcs {

// read-write access to contained data member
template<class T> class Singleton
{
	public:
		static T& instance();
		
	private:
		Singleton();
		Singleton(const Singleton<T>&);
		Singleton<T>& operator=(const Singleton<T>&);
				
		virtual ~Singleton();
		
		T& instanceData() { return *data_ptr_; }
		
		std::auto_ptr<T>	data_ptr_;
};

template<class T>
inline T& Singleton<T>::instance()
{
	static Singleton<T>	instance;
	
	return instance.instanceData();
}

template<class T>
inline Singleton<T>::Singleton()
{
	data_ptr_ = std::auto_ptr<T>(new T());
}

template<class T>
inline Singleton<T>::~Singleton()
{ }

// read-only access to contained data member
//	- initialization must occur in default constructor
template<class T> class ConstSingleton
{
	public:
		static const T& instance();
		
	private:
		ConstSingleton();
		ConstSingleton(const ConstSingleton<T>&);
		ConstSingleton<T>& operator=(const ConstSingleton<T>&);
				
		virtual ~ConstSingleton();
		
		const T& instanceData() { return *data_ptr_; }
		
		std::auto_ptr<T>	data_ptr_;
};

template<class T>
inline const T& ConstSingleton<T>::instance()
{
	static ConstSingleton<T>	instance;
	
	return instance.instanceData();
}

template<class T>
inline ConstSingleton<T>::ConstSingleton()
{
	data_ptr_ = std::auto_ptr<T>(new T());
}

template<class T>
inline ConstSingleton<T>::~ConstSingleton()
{ }

} // namespace mcs

#endif //MCS_SINGLETON_H