#include "MCS_EarthMap.h"
#include "MCS_Field.h"
#include "MCS_Grid.h"
#include "MCS_IndexRange.h"

namespace mcs {

namespace geo {

template<class T> 
inline CenteredEarthGrid<T>::CenteredEarthGrid(const IndexRange<2>& rng) :
	UniformGrid<T,2,SphericalBoundary<2> >(IndexRange<2>(rng),
				  					  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(-(90.0-0.5*180.0/rng.dim(0)),0.5*360.0/rng.dim(1)),
				   					  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(90.0-0.5*180.0/rng.dim(0),360.0-0.5*360.0/rng.dim(1)))
{ }

template<class T> 
inline CornerEarthGrid<T>::CornerEarthGrid(const IndexRange<2>& rng) :
	UniformGrid<T,2,SphericalBoundary<2> >(IndexRange<2>(rng),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(-90.0,0.0),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(90.0,360.0))
{ }

template<class T> 
inline EdgeEarthGrid<T>::EdgeEarthGrid(const IndexRange<2>& rng) :
	UniformGrid<T,2,SphericalBoundary<2> >(IndexRange<2>(rng),
						 				   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(-90.0,0.5*360.0/rng.dim(1)),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(90.0,360.0-0.5*360.0/rng.dim(1)))
{ }

template<class T> 
inline CCM3Grid<T>::CCM3Grid(const IndexRange<2>& rng) :
	UniformGrid<T,2,SphericalBoundary<2> >(IndexRange<2>(rng),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(-(90.0-0.5*180.0/rng.dim(0)),0.0),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(90.0-0.5*180.0/rng.dim(0),360.0-360.0/rng.dim(1)))
{ }

template<class T> 
inline NCEPGrid<T>::NCEPGrid(const IndexRange<2>& rng) :
	UniformGrid<T,2,SphericalBoundary<2> >(IndexRange<2>(rng),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(-90,0.0),
						 			  	   typename UniformGrid<T,2,SphericalBoundary<2> >::CoordinateType(90,360.0-360.0/rng.dim(1)))
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>::EarthMap() : 
	Field<T,M,B>()
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>::EarthMap(const Matrix<T>& data) : 
	Field<T,M,B>(CenteredEarthGrid<M>(data.range()),data)
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>::EarthMap(const IndexRange<2>& rng,const T& val) : 
	Field<T,M,B>(CenteredEarthGrid<M>(rng),val)
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>::EarthMap(const GridType& grid,const T& val) : 
	Field<T,M,B>(grid,val)
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>::EarthMap(const GridType& grid,const Matrix<T>& data) : 
	Field<T,M,B>(grid,data)
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>::EarthMap(const EarthMap<T,M,B>& map) : 
	Field<T,M,B>(map)
{ }

template<class T,class M,class B> 
inline EarthMap<T,M,B>& EarthMap<T,M,B>::operator=(const T& val)
{
	Field<T,M,B>::operator=(val);

	return (*this);
}

template<class T,class M,class B> 
inline EarthMap<T,M,B>& EarthMap<T,M,B>::operator=(const Matrix<T>& source)
{
	if (source.range() != (*this).range()) throw Err();
	
	Field<T,M,B>::operator=(source);
	
	return (*this);
}

template<class T,class M,class B> 
inline EarthMap<T,M,B>& EarthMap<T,M,B>::operator=(const EarthMap<T,M,B>& source)
{
	if (this == &source) return *this;
	
	Field<T,M,B>::operator=(source);
	
	return (*this);
}

} // namespace geo

} // namespace mcs
