#include <fstream>
#include <iostream>
#include <string>

#include "MCS_FileLocations.h"
#include "MCS_Geography.h"

namespace mcs {

namespace geo {

// WGS 1984 earth geoid
CenteredSurfaceMapF32& getLoResGeoidData()
{
	static const IndexRange<2>	rng(Index<2>(180,360));

	static bool geoid_read = false;
	
	static CenteredSurfaceMapF32	data(rng);	
	
	if (!geoid_read)
	{
		static const std::string	filename = FileLocations["geoid"]+"WGS84GEOID_L.MAT.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		geoid_read = true;
	}
	
	return data;
}

CenteredSurfaceMapF32& getMedResGeoidData()
{
	static const IndexRange<2>	rng(Index<2>(720,1440));

	static bool geoid_read = false;
	
	static CenteredSurfaceMapF32	data(rng);	
	
	if (!geoid_read)
	{
		static const std::string	filename = FileLocations["geoid"]+"WGS84GEOID_M.MAT.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		geoid_read = true;
	}
	
	return data;
}

// USGS Land Use/Land Cover System land cover type
CenteredSurfaceMapI8& getLoResLandCoverData()
{
	static const IndexRange<2>	rng(Index<2>(225,450));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapI8	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["land cover"]+"LandCover_L.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapI8& getMedResLandCoverData()
{
	static const IndexRange<2>	rng(Index<2>(675,1350));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapI8	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["land cover"]+"LandCover_M.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapI8& getHiResLandCoverData()
{
	static const IndexRange<2>	rng(Index<2>(2700,5400));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapI8	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["land cover"]+"LandCover_H.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapI8& getHi2ResLandCoverData()
{
	static const IndexRange<2>	rng(Index<2>(5400,10800));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapI8	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["land cover"]+"LandCover_H2.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapI8& getHi3ResLandCoverData()
{
	static const IndexRange<2>	rng(Index<2>(10800,21600));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapI8	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["land cover"]+"LandCover_H3.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapI8& getHi4ResLandCoverData()
{
	static const IndexRange<2>	rng(Index<2>(21600,43200));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapI8	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["land cover"]+"USGS_LandCover.mcs";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

// land fraction data 
CenteredSurfaceMapF32& getLoResLandFractionData()
{
	static const IndexRange<2>	rng(Index<2>(225,450));
	
	static bool land_frac_read = false;
	
	static CenteredSurfaceMapF32	data(rng);	
	
	if (!land_frac_read)
	{
		static const std::string	filename = FileLocations["land fraction"]+"LANDFRACTION_L.MAT.MCS";

		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		land_frac_read = true;
	}
	
	return data;
}

CenteredSurfaceMapF32& getMedResLandFractionData()
{
	static const IndexRange<2>	rng(Index<2>(675,1350));
	
	static bool land_frac_read = false;
	
	static CenteredSurfaceMapF32	data(rng);	
	
	if (!land_frac_read)
	{
		static const std::string	filename = FileLocations["land fraction"]+"LANDFRACTION_M.MAT.MCS";

		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		land_frac_read = true;
	}
	
	return data;
}

CenteredSurfaceMapF32& getHiResLandFractionData()
{
	static const IndexRange<2>	rng(Index<2>(2700,5400));
	
	static bool land_frac_read = false;
	
	static CenteredSurfaceMapF32	data(rng);	
	
	if (!land_frac_read)
	{
		static const std::string	filename = FileLocations["land fraction"]+"LANDFRACTION_H.MAT.MCS";

		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		land_frac_read = true;
	}
	
	return data;
}

// GTOPO topographic altitude above geoid
CenteredSurfaceMapF32& getLoResTopographyData()
{
	static const IndexRange<2>	rng(Index<2>(225,450));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapF32	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["topography"]+"GTOPO_L.MAT.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapF32& getMedResTopographyData()
{
	static const IndexRange<2>	rng(Index<2>(675,1350));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapF32	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["topography"]+"GTOPO_M.MAT.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapF32& getHiResTopographyData()
{
	static const IndexRange<2>	rng(Index<2>(2700,5400));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapF32	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["topography"]+"GTOPO_H.MAT.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

CenteredSurfaceMapF32& getHi2ResTopographyData()
{
	static const IndexRange<2>	rng(Index<2>(5400,10800));
	
	static bool topo_read = false;
	
	static CenteredSurfaceMapF32	data(rng);
	
	if (!topo_read)
	{
		static const std::string	filename = FileLocations["topography"]+"GTOPO_H2.MAT.MCS";
		
		std::cerr << "Reading..." << filename;
		
		std::ifstream ifs(filename.c_str(),std::ios::binary | std::ios::in);		
		data.read(ifs);
		ifs.close();
		
		std::cerr << "...done" << std::endl;
		
		topo_read = true;
	}
	
	return data;
}

} // namespace geo

} // namespace mcs