#include "MCS_Geoid.h"

namespace mcs {

namespace geo {

Geoid::Geoid(const Float32& res)
{
	setResolution(res);
}

void Geoid::setResolution(const Float32& res)
{ 
	res_ = res;
	
	if (resolution() < (180.0/720))
		throw Err();
	else if (resolution() < (180.0/180))
		data_ptr_ = &getMedResGeoidData();
	else
		data_ptr_ = &getLoResGeoidData();
}

Float32 Geoid::operator()(const GeographicCoordinate<Float32>& coord) const
{
	return data_ptr_->z(coord);
}

CenteredSurfaceMapF32 Geoid::operator()(const CenteredSurfaceMapF32::GridType& grid) const
{
	CenteredSurfaceMapF32	map(grid);
	
	return (*this)(map);
}

CenteredSurfaceMapF32& Geoid::operator()(CenteredSurfaceMapF32& map) const
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = (*this)(map.coordinate(idx));
						
	return map;
}

Float32 Geoid::ellipsoid(const GeographicCoordinate<Float32>& coord) const
{
	// Table 3.3
	static const Float64	SEMIMAJOR_AXIS_ = constants::physical::_r_Ee_,  	// a (m) = 6378137.0 
							SEMIMINOR_AXIS_ = constants::physical::_r_Ep_,		// b (m) = 6356752.3142				
							AXIS_RATIO_ = constants::physical::_oblateness_;	// b/a = 0.996647189335
			
	const Float64	latrad = toRadians(coord.latitude()),
					a2 = std::sqr(SEMIMAJOR_AXIS_),
					b2 = std::sqr(SEMIMINOR_AXIS_),
					as2 = std::sqr(SEMIMAJOR_AXIS_*std::sin(latrad)),
					bc2 = std::sqr(SEMIMINOR_AXIS_*std::cos(latrad));
					
	return std::sqrt((a2*b2)/(as2+bc2));
}							 

} // namespace geo

} // namespace mcs