#include "MCS_LandCoverType.h"

namespace mcs {

namespace geo {

LandCoverType::LandCoverType(const Float32& res)
{
	setResolution(res);
}

void LandCoverType::setResolution(const Float32& res)
{ 
	res_ = res;
	
	if (resolution() < (180.0/21600))
		throw Err();
	else if (resolution() < (180.0/10800))
		data_ptr_ = &getHi4ResLandCoverData();
	else if (resolution() < (180.0/5400))
		data_ptr_ = &getHi3ResLandCoverData();
	else if (resolution() < (180.0/2700))
		data_ptr_ = &getHi2ResLandCoverData();
	else if (resolution() < (180.0/675))
		data_ptr_ = &getHiResLandCoverData();
	else if (resolution() < (180.0/225))
		data_ptr_ = &getMedResLandCoverData();
	else
		data_ptr_ = &getLoResLandCoverData();
}

Float32 LandCoverType::operator()(const GeographicCoordinate<Float32>& coord) const
{
	return data_ptr_->z(data_ptr_->grid().nearest(coord));
}

CenteredSurfaceMapI8 LandCoverType::operator()(const CenteredSurfaceMapI8::GridType& grid) const
{
	CenteredSurfaceMapI8	map(grid);
	
	return (*this)(map);
}

CenteredSurfaceMapI8& LandCoverType::operator()(CenteredSurfaceMapI8& map) const
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = (*this)(map.coordinate(idx));
						
	return map;
}

} // namespace geo

} // namespace mcs