#include "MCS_LandFraction.h"

namespace mcs {

namespace geo {

LandFraction::LandFraction(const Float32& res)
{
	setResolution(res);
}

void LandFraction::setResolution(const Float32& res)
{ 
	res_ = res;
	
	if (resolution() < (180.0/2700))
		throw Err();
	// 0.0667 deg.
	else if (resolution() < (180.0/675))
		data_ptr_ = &getHiResLandFractionData();
	// 0.2667 deg.
	else if (resolution() < (180.0/225))
		data_ptr_ = &getMedResLandFractionData();
	// 0.8000 deg.
	else
		data_ptr_ = &getLoResLandFractionData();
}

Float32 LandFraction::operator()(const GeographicCoordinate<Float32>& coord) const
{
	return data_ptr_->z(coord);
}

CenteredSurfaceMapF32 LandFraction::operator()(const CenteredSurfaceMapF32::GridType& grid) const
{
	CenteredSurfaceMapF32	map(grid);
	
	return (*this)(map);
}

CenteredSurfaceMapF32& LandFraction::operator()(CenteredSurfaceMapF32& map) const
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = (*this)(map.coordinate(idx));
						
	return map;
}

} // namespace geo

} // namespace mcs