#ifndef MCS_LANDMASK_H
#define MCS_LANDMASK_H

#include "MCS_EarthMap.h"
#include "MCS_Exceptions.h"
#include "MCS_Geography.h"

namespace mcs {

namespace geo {

class LandMask
{
	public:
		LandMask(const Float32& res = 0.25,const Float32& thresh = 0.1);
		~LandMask() { }
		
		void resolution(const Float32& res)				{ setResolution(res); }
		const Float32& resolution() const				{ return res_; }
		
		void threshold(const Float32& thresh)			{ setThreshold(thresh); }
		const Float32& threshold() const				{ return thresh_; }
		
		bool					operator()(const GeographicCoordinate<Float32>& coord) const;
		CenteredSurfaceMapF32&	operator()(CenteredSurfaceMapF32& map) const;
	
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,GeographicErr)
		
	private:
		void setResolution(const Float32& res);
		void setThreshold(const Float32& thresh);
		
		Float32					res_,
								thresh_;
		CenteredSurfaceMapF32*	data_ptr_;
};

} // namespace geo

} // namespace mcs

#endif // MCS_LANDMASK_H
