#include "MCS_Topography.h"

namespace mcs {

namespace geo {

Topography::Topography(const Float32& res)
{
	setResolution(res);
}

void Topography::setResolution(const Float32& res)
{ 
	res_ = res;
	
	if (resolution() < (180.0/5400))
		throw Err();
	else if (resolution() < (180.0/2700))
		data_ptr_ = &getHi2ResTopographyData();
	else if (resolution() < (180.0/675))
		data_ptr_ = &getHiResTopographyData();
	else if (resolution() < (180.0/225))
		data_ptr_ = &getMedResTopographyData();
	else
		data_ptr_ = &getLoResTopographyData();
}

Float32 Topography::operator()(const GeographicCoordinate<Float32>& coord) const
{
	const Float32 	val = data_ptr_->z(coord);
	
	return isfinite(val) ? val : 0;
}

CenteredSurfaceMapF32 Topography::operator()(const CenteredSurfaceMapF32::GridType& grid) const
{
	CenteredSurfaceMapF32	map(grid);
	
	return (*this)(map);
}

CenteredSurfaceMapF32& Topography::operator()(CenteredSurfaceMapF32& map) const
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = (*this)(map.coordinate(idx));
						
	return map;
}

} // namespace geo

} // namespace mcs