#ifndef MCS_TOPOGRAPHY_H
#define MCS_TOPOGRAPHY_H

#include "MCS_EarthMap.h"
#include "MCS_Exceptions.h"
#include "MCS_Geography.h"

namespace mcs {

namespace geo {

class Topography
{
	public:
		Topography(const Float32& res = 0.25);
		~Topography() { }
		
		void resolution(const Float32& res)				{ setResolution(res); }
		const Float32& resolution() const				{ return res_; }
		
		Float32					operator()(const GeographicCoordinate<Float32>& coord) const;
		CenteredSurfaceMapF32	operator()(const CenteredSurfaceMapF32::GridType& grid) const;
		CenteredSurfaceMapF32&	operator()(CenteredSurfaceMapF32& map) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,GeographicErr)
		
	private:
		void setResolution(const Float32& res);
		
		Float32					res_;
		CenteredSurfaceMapF32*	data_ptr_;
};

} // namespace geo

} // namespace mcs

#endif // MCS_TOPOGRAPHY_H
