#include "MCS_WaterMask.h"

namespace mcs {

namespace geo {

WaterMask::WaterMask(const Float32& res,const Float32& thresh)
{
	setResolution(res);
	setThreshold(thresh);
}

void WaterMask::setResolution(const Float32& res)
{ 
	res_ = res;
	
	if (resolution() < (180.0/2700))
		throw Err();
	// 0.0667 deg.
	else if (resolution() < (180.0/675))
		data_ptr_ = &getHiResLandFractionData();
	// 0.2667 deg.
	else if (resolution() < (180.0/225))
		data_ptr_ = &getMedResLandFractionData();
	// 0.8000 deg.
	else
		data_ptr_ = &getLoResLandFractionData();
}

void WaterMask::setThreshold(const Float32& thresh)
{ 
	thresh_ = thresh;
}

bool WaterMask::operator()(const GeographicCoordinate<Float32>& coord) const
{
	return 1-data_ptr_->z(coord) > thresh_;
}

CenteredSurfaceMapF32& WaterMask::operator()(CenteredSurfaceMapF32& map) const
{
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			if ((*this)(map.coordinate(idx)))
				map(idx) = _NaN_;
						
	return map;
}

} // namespace geo

} // namespace mcs