#include "MCS_FileLocations.h"
#include "MCS_Geography.h"

namespace mcs {

namespace geo {

Float32 getEllipsoid(const GeographicCoordinate<Float32>& coord)
{
	// Table 3.3
	static const Float64	SEMIMAJOR_AXIS_ = constants::physical::_r_Ee_,  	// a (m) = 6378137.0 
							SEMIMINOR_AXIS_ = constants::physical::_r_Ep_,		// b (m) = 6356752.3142				
							AXIS_RATIO_ = constants::physical::_oblateness_;	// b/a = 0.996647189335
			
	const Float64	latrad = toRadians(coord.latitude()),
					a2 = std::sqr(SEMIMAJOR_AXIS_),
					b2 = std::sqr(SEMIMINOR_AXIS_),
					as2 = std::sqr(SEMIMAJOR_AXIS_*std::sin(latrad)),
					bc2 = std::sqr(SEMIMINOR_AXIS_*std::cos(latrad));
					
	return std::sqrt((a2*b2)/(as2+bc2));
}							 

Float32 getGeoid(const GeographicCoordinate<Float32>& coord)
{		
	const CenteredSurfaceMapF32&	data = getMedResGeoidData();
	
	return data.z(coord);
}

CenteredSurfaceMapF32 getGeoid(const CenteredSurfaceMapF32::GridType& grid)
{
	const CenteredSurfaceMapF32*	dataptr = &getMedResGeoidData();
	CenteredSurfaceMapF32			map(grid);
	
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = dataptr->z(map.coordinate(idx));
		
	return map;
}

CenteredSurfaceMapF32& getGeoid(CenteredSurfaceMapF32& map)
{
	CenteredSurfaceMapF32*		dataptr;
	
	if (any(map.resolution() <= static_cast<CenteredSurfaceMapF32::CoordinateType>(180.0/720.0)))
		dataptr = &getMedResGeoidData();
	else
		dataptr = &getLoResGeoidData();
	
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
			map(idx) = dataptr->z(map.coordinate(idx));
		
	return map;
}

} // namespace geo

} // namespace mcs