#include "MCS_Date.h"
#include "MCS_Geography.h"
#include "MCS_MathExtensions.h"

namespace mcs {

namespace geo {

// calculate insolation at the Earth's surface
//
//		based on An Introduction to Atmospheric Radiation, K.N. Liou, pp. 45-48
//
//		phi		: latitude (deg.)
//		today	: the date
//
//	returns insolation (W/m^2)
//
Float32 getInsolation(const GeographicCoordinate<Float32>& coord,const Date& today)
{
	static const Float64	_S_sol_ = constants::physical::_S_sol_;
							
	const Complex64	frac_year 	= Float64(today.calendarDay())/today.daysInYear(),
					phi_val 	= toRadians(coord.latitude()),
					// solar zenith angle assuming solstice on 21st
					delta		= 23.45*_rad_per_deg_*std::cos(_two_pi_*(frac_year-0.475)),	
					// orbital eccentricity (max. on Jan. 3rd)
					f 			= 1.0 + 0.0344*std::cos(_two_pi_*(frac_year-0.0821)),
					H 			= std::acos(-std::tan(phi_val)*std::tan(delta)),
					tanH 		= std::sqrt(1.0-sqr(std::tan(phi_val))*sqr(std::tan(delta)));

	return ((_S_sol_/4)*sqr(f)*(std::sin(phi_val)*std::sin(delta)*H+std::cos(phi_val)*std::cos(delta)*tanH)).real();
}

} // namespace geo

} // namespace mcs