#include "MCS_Date.h"
#include "MCS_Geography.h"

// Naval Observatory Vector Astrometry Subroutines (NOVAS)
extern "C" 
{
	#include "novascon.h"
	#include "novas.h"
}

namespace mcs {

namespace geo {

// calculates the latitude and longitude of the point of local noon
GeographicCoordinate<Float32> getSun(const Date& today)
{
	static body	NOVAS_SUN = { 0, 10, "Sun" },
				NOVAS_EARTH = { 0, 3, "Earth" };

	Float64	right_ascension,
			declination,
			R,
			equation_of_equinox,
			apparent_sidereal_time,
			t1,t2,t3,t4,
			tjd = today.julian();
		
	virtual_planet(tjd,&NOVAS_SUN,&NOVAS_EARTH,&right_ascension,&declination,&R);
	earthtilt(tjd,&t1,&t2,&equation_of_equinox,&t3,&t4);
	sidereal_time(std::floor(tjd),tjd-std::floor(tjd),equation_of_equinox,&apparent_sidereal_time);
	
	return GeographicCoordinate<Float32>(static_cast<Float32>(declination),
							  			 static_cast<Float32>(std::mod((right_ascension-apparent_sidereal_time)*360.0/24.0,360.0)));
}

} // namespace geo

} // namespace mcs
