#include "MCS_FileLocations.h"
#include "MCS_Geography.h"

namespace mcs {

namespace geo {

// MCS 99/08/19 - return GTOPO topographic height relative to geoid
Float32 getTopography(const GeographicCoordinate<Float32>& coord)
{	
	const CenteredSurfaceMapF32&	data = getMedResTopographyData();
		
	const Float32	val = data.z(coord);
	
	if (val < -500 || val > 9000) throw GeographyErr();
		
	return isfinite(val) ? val : 0;
}

CenteredSurfaceMapF32 getTopography(const CenteredSurfaceMapF32::GridType& grid)
{
	CenteredSurfaceMapF32	map(grid);
	
	getTopography(map);
				
	return map;
}

CenteredSurfaceMapF32& getTopography(CenteredSurfaceMapF32& map)
{	
	CenteredSurfaceMapF32*	dataptr;
	
	const Float32	res = statistics::normal::min(map.resolution());
	
	if (res <= (180.0/5400))
		dataptr = &getHi2ResTopographyData();
	else if (res <= (180.0/2700))
		dataptr = &getHiResTopographyData();
	else if (res <= (180.0/675))
		dataptr = &getMedResTopographyData();
	else
		dataptr = &getLoResTopographyData();
		
	for (Index<2> idx(map.first());idx[0]<=map.last(0);++idx[0])
	{
		for (idx[1]=map.first(1);idx[1]<=map.last(1);++idx[1])
		{
			const Float32	val = dataptr->z(map.coordinate(idx));
			
			map(idx) = isfinite(val) ? val : 0;
		}
	}

	return map;
}

} // namespace geo

} // namespace mcs