#include "MCS_Exceptions.h"
#include "MCS_IO.h"
#include "MCS_Vector.h"

#include "zlib_util.h"

namespace mcs {

namespace gz {

Int32 uncompressedSize(const std::string& filename)
{
	long	num_elem;
	
	uncompressed_size(filename.c_str(),&num_elem);
	
	return num_elem;
}

// smarter memory management
Vector<Uint8>& readGZIP(const std::string& filename,Vector<Uint8>& data)
{
	const Int32	num_elem = uncompressedSize(filename);
	
	return readGZIP(filename,data,num_elem);
}

Vector<Uint8>& readGZIP(const std::string& filename,Vector<Uint8>& data,const Int32& num_elem)
{
	if (num_elem <= 0) throw IOErr("[readGZIP] Bad GZIP file : "+filename);
	
	data = Vector<Uint8>(IndexRange<1>(num_elem));
	
	long 	num_bytes,err;

	GZIP_inflate(filename.c_str(),&data[0],&num_bytes,&err);
	
	if (err == -1 || num_bytes == -1 || num_bytes != num_elem) 
		return (data = Vector<Uint8>());

	return data;
}

} // namespace gz

} // namespace mcs