#ifndef MCS_IO_H
#define MCS_IO_H

#include <fstream>
#include <iostream>
#include <string>

#include "MCS_ByteSwap.h"
#include "MCS_Exceptions.h"

namespace mcs {

//class Err : public Exception { };
__DECLARE_EXCEPTION_CLASS__(IOErr,Exception)

// get filelength
inline std::streamoff fileLength(std::FILE* infile)
{
	// get file length
	if (std::ferror(infile)) throw IOErr();
	
	std::streamoff	pos = std::ftell(infile);
	if (!std::fseek(infile,0,SEEK_END)) throw IOErr();
	
	std::streamoff	len = std::ftell(infile);
	if (!std::fseek(infile,pos,SEEK_SET)) throw IOErr();
	
	return len;
}

inline std::streamoff fileLength(std::ifstream& ifs)
{
	// get file length
	if (!ifs.good()) throw IOErr();
	
	std::streampos	pos = ifs.tellg();
	if (!ifs.seekg(0,std::ios::end)) throw IOErr();
	
	std::streamoff	len = ifs.tellg();
	if (!ifs.seekg(pos)) throw IOErr();
	
	return len;
}

inline std::streamoff fileLength(std::ofstream& ofs)
{
	// get file length
	if (!ofs.good()) throw IOErr();
	
	std::streampos	pos = ofs.tellp();
	if (!ofs.seekp(0,std::ios::end)) throw IOErr();
	
	std::streamoff	len = ofs.tellp();
	if (!ofs.seekp(pos)) throw IOErr();
	
	return len;
}

template<class T> class Vector;
template<class T> class Matrix;

namespace gz {

Int32 uncompressedSize(const std::string& filename);

Vector<Uint8>& readGZIP(const std::string& filename,Vector<Uint8>& data);
Vector<Uint8>& readGZIP(const std::string& filename,Vector<Uint8>& data,const Int32& num_elem);

} // namespace gz

Matrix<Uint8>& readHDF(const std::string& filename,Matrix<Uint8>& data);

} // namespace mcs

#endif // MCS_IO_H