#include "MCS_Debug.h"

#ifdef DEBUG
	#include "MSLHeaders++.h"
#else
	#include "MCS_Headers++.h"
#endif

#include "MCS_MATLAB_Engine.h"

using namespace std;

//#if 0
int main()
{
	MATLAB_Engine	engine;
	
	Vector<Float64>	t(v_range(1,10)," 0.0  1.0  2.0  3.0  4.0  5.0  6.0  7.0  8.0  9.0 ");
	
	engine.putVector("t",t);
	engine.evaluate("d = .5.*(-9.8).*t.^2;");

	engine.evaluate("figure(1);");
	engine.evaluate("plot(t,d);");
	engine.evaluate("title('Position vs. Time for a falling object');");
	engine.evaluate("xlabel('Time (seconds)');");
	engine.evaluate("ylabel('Position (meters)');");

	Matrix<Float64>	m(m_range(1,4,1,5));
	
	for (v_idx i=m.first_x();i<=m.last_x();i++)
		for (v_idx j=m.first_y();j<=m.last_y();j++)
			m[i][j] = i-j;
			
	engine.putMatrix("m",m);
	engine.evaluate("mp = sqrt(abs(m));");
	
	engine.evaluate("figure(2);");
	engine.evaluate("imagesc(m);");
	engine.evaluate("imagesc(mp);");
	
	Matrix<Float64>	mp = engine.get("mp");
	
	char	tmp;
	
	cout << "Hit return to continue\n\n" << endl;
	cin.get(tmp);
	
	cout << "Done with part 1..." << endl;
	
	engine.evaluate("close;");
	 
	while (true) 
	{
	   	string	command;
	   	
	    cout << "Enter a MATLAB command to evaluate.  This command should" << endl
	    	 << "create a variable X.  This program will then determine" << endl
	    	 << "what kind of variable you created." << endl
	    	 << "For example: X = 1:5" << endl
	    	 << ">> ";

	    cin >> command;
	  
	    engine.evaluate(command);
	    
	    cout << engine.buffer() << endl;
	    
	    cout << "Retrieving X..." << endl;
	    
	    try
	    {	    	
	    	cout << "X is class " << engine.className("X") << endl;
	    	
	    	break;
	    }
	    catch (MATLAB_Engine::Err&)
	    {
	    	cout << "Oops! You didn't create a variable X" << endl;
	    }
	}
	
	cout << "Done with part 2..." << endl;
	
	return 0;
}
//#endif

#if 0
int main()
{
	MATLAB_Engine	engine;
	
	try
	{
		#if 0
		GEOG::SurfaceMap	geoid(m_range(360,720)),
							landfrac(m_range(360,720)),
							oceanfrac(m_range(360,720)),
							topo(m_range(360,720));
							
		GEOG::getLandFraction(landfrac);
			
		engine.putMatrix("lf",landfrac);
		engine.evaluate("figure(1);");
		engine.evaluate("imagesc(flipud(lf'));colorbar;");
		
		GEOG::getOceanFraction(oceanfrac);
		
		engine.putMatrix("of",oceanfrac);
		engine.evaluate("figure(1);");
		engine.evaluate("imagesc(flipud(of'));colorbar;");
		
		GEOG::getGeoid(geoid);
		
		engine.putMatrix("geoid",geoid);
		engine.evaluate("figure(2);");
		engine.evaluate("imagesc(flipud(geoid'));colorbar;");
		engine.evaluate("");
		
		GEOG::getTopography(topo);
		
		engine.putMatrix("topo",topo);		
		engine.evaluate("figure(3);");
		engine.evaluate("imagesc(flipud(topo'));colorbar;");
		
		engine.evaluate("figure(4);");
		engine.evaluate("imagesc(flipud((geoid+topo)'));colorbar;");
	
		const GEOG::CenteredEarthGrid<Float32>	newgrid(m_range(180,360));
		Matrix<Float32>							map;
		
		SSMI_Map	ssmi(SatelliteID("DMSP F14"));
		Date		ssmi_date(2000,1,1);
		
		ssmi.readOne(ssmi_date,SSMI_Map::CLOUD_WATER,true);
		map = ssmi.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ssmi.readOne(ssmi_date,SSMI_Map::COLUMN_VAPOR,true);
		map = ssmi.resize(newgrid);
		
		engine.putMatrix("data",map);
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ssmi.readOne(ssmi_date,SSMI_Map::OBSERVATION_TIME,true);
		map = ssmi.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ssmi.readOne(ssmi_date,SSMI_Map::RAIN_RATE,true);
		map = ssmi.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ssmi.readOne(ssmi_date,SSMI_Map::WIND_SPEED,true);
		map = ssmi.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	

		NCEP_SurfaceMap	ncep_surf(Date(1999,1,1));
		
		ncep_surf.EDR(NCEP_SurfaceMap::COLUMN_VAPOR);
		ncep_surf.read();
		map = ncep_surf.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ncep_surf.EDR(NCEP_SurfaceMap::TEMPERATURE);
		ncep_surf.read();
		map = ncep_surf.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ncep_surf.EDR(NCEP_SurfaceMap::WIND_U);
		ncep_surf.read();
		map = ncep_surf.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		ncep_surf.EDR(NCEP_SurfaceMap::WIND_V);
		ncep_surf.read();
		map = ncep_surf.resize(newgrid);
		
		engine.putMatrix("data",map);	
		engine.evaluate("imagesc(flipud(data'));colorbar;pause(1);");
	
		
		NCEP_ProfileMap	ncep_prof(Date(1998,6,1));
		
		// geopotential height
		ncep_prof.EDR(NCEP_ProfileMap::GEOPOTENTIAL_HEIGHT);

		for (v_idx i=ncep_prof.pressureLevels().first();i<=ncep_prof.pressureLevels().last();i++)
		{	
			cout << "Level " << ncep_prof.pressureLevels()[i] << " mbar" << endl;

			ncep_prof.level(i);
			ncep_prof.read();
			map = ncep_prof.resize(newgrid);
	
			engine.putMatrix("data",map);	
			engine.evaluate("imagesc(flipud(data'));colorbar;");
		}
		
		// relative humidity
		ncep_prof.EDR(NCEP_ProfileMap::RELATIVE_HUMIDITY);
		
		for (v_idx i=ncep_prof.pressureLevels().first();i<=ncep_prof.pressureLevels().last();i++)
		{	
			cout << "Level " << ncep_prof.pressureLevels()[i] << " mbar" << endl;

			ncep_prof.level(i);
			ncep_prof.read();
			map = ncep_prof.resize(newgrid);
	
			engine.putMatrix("data",map);	
			engine.evaluate("imagesc(flipud(data'));colorbar;");
		}
				
		// temperature 
		ncep_prof.EDR(NCEP_ProfileMap::TEMPERATURE);
		
		for (v_idx i=ncep_prof.pressureLevels().first();i<=ncep_prof.pressureLevels().last();i++)
		{	
			cout << "Level " << ncep_prof.pressureLevels()[i] << " mbar" << endl;

			ncep_prof.level(i);
			ncep_prof.read();
			map = ncep_prof.resize(newgrid);
	
			engine.putMatrix("data",map);	
			engine.evaluate("imagesc(flipud(data'));colorbar;");
		}
		
		// ucomponent of wind	
		ncep_prof.EDR(NCEP_ProfileMap::WIND_U);
		
		for (v_idx i=ncep_prof.pressureLevels().first();i<=ncep_prof.pressureLevels().last();i++)
		{	
			cout << "Level " << ncep_prof.pressureLevels()[i] << " mbar" << endl;

			ncep_prof.level(i);
			ncep_prof.read();
			map = ncep_prof.resize(newgrid);
	
			engine.putMatrix("data",map);	
			engine.evaluate("imagesc(flipud(data'));colorbar;");
		}
		
		// v component of wind		
		ncep_prof.EDR(NCEP_ProfileMap::WIND_V);
		
		for (v_idx i=ncep_prof.pressureLevels().first();i<=ncep_prof.pressureLevels().last();i++)
		{	
			cout << "Level " << ncep_prof.pressureLevels()[i] << " mbar" << endl;

			ncep_prof.level(i);
			ncep_prof.read();
			map = ncep_prof.resize(newgrid);
	
			engine.putMatrix("data",map);	
			engine.evaluate("imagesc(flipud(data'));colorbar;");
		}
		#endif
		
		// test SST maps and climatology
		//SST_Map	sst(Date(1981,10,29));
		SST_Map	sst(Date(2001,1,1));
		
		engine.evaluate("i=1;");
		
		while (sst.epoch()<=Date(2002,12,31))
		{
			//sst.next();
			
			sst.epoch(Date(sst.epoch().julian()+7));
			sst.read();
			
			engine.putMatrix("data",sst);
			engine.evaluate("figure(1);imagesc(flipud(data'));colorbar;");
			
			cout << sst.epoch() << endl;
			
			engine.putValue("t",sst.epoch().to_RD_2000());
			engine.evaluate("tim(i)=t;");
			engine.evaluate("zon(i,:)=nanmean(data);");
			engine.evaluate("figure(2);plot(zon(i,:));");
			engine.evaluate("figure(3);plot(tim,nanmean(zon'),'.');");
			engine.evaluate("i=i+1;");
		}
	}
	catch (...)
	{ 
		cout << endl << "Error..." << endl; 
	}
	
	return 0;
}
#endif