#include <cmath>

#include "MCS_Matrix.h"

namespace mcs {

// in-place
// arithmetic functions
template<class T> const Matrix<T>& Abs(Matrix<T>& source)
{
	using namespace mcs;
	
	Abs(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Ceil(Matrix<T>& source)
{
	using namespace mcs;
	
	Ceil(source.asVector());
		
	return source;
}

template<class T> const Matrix<T>& Exp(Matrix<T>& source)
{
	using namespace mcs;
	
	Exp(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Floor(Matrix<T>& source)
{
	using namespace mcs;
	
	Floor(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Log(Matrix<T>& source)
{
	using namespace mcs;
	
	Log(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Pow(Matrix<T>& source,const T& expon)
{
	using namespace mcs;
	
	Pow(source.asVector(),expon);
	
	return source;
}

template<class T> const Matrix<T>& Round(Matrix<T>& source)
{
	using namespace mcs;
	
	Round(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Sqr(Matrix<T>& source)
{
	using namespace mcs;
	
	Sqr(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Sqrt(Matrix<T>& source)
{
	using namespace mcs;
	
	Sqrt(source.asVector());
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> const Matrix<T>& Conj(Matrix<T>& source)
{
	using namespace mcs;
	
	Conj(source.asVector());
		
	return source;
}

template<class T> const Matrix<T>& Imag(Matrix<T>& source)
{
	using namespace mcs;
	
	Imag(source.asVector());
		
	return source;
}

template<class T> const Matrix<T>& Real(Matrix<T>& source)
{
	using namespace mcs;
	
	Real(source.asVector());
		
	return source;
}

// trigonometric functions
template<class T> const Matrix<T>& Sin(Matrix<T>& source)
{
	using namespace mcs;
	
	Sin(source.asVector());

	return source;
}

template<class T> const Matrix<T>& Cos(Matrix<T>& source)
{
	using namespace mcs;
	
	Cos(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Tan(Matrix<T>& source)
{
	using namespace mcs;
	
	Tan(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Asin(Matrix<T>& source)
{
	using namespace mcs;
	
	Asin(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Acos(Matrix<T>& source)
{
	using namespace mcs;
	
	Acos(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Atan(Matrix<T>& source)
{
	using namespace mcs;
	
	Atan(source.asVector());
	
	return source;
}

// hyperbolic functions
template<class T> const Matrix<T>& Sinh(Matrix<T>& source)
{
	using namespace mcs;
	
	Sinh(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Cosh(Matrix<T>& source)
{
	using namespace mcs;
	
	Cosh(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Tanh(Matrix<T>& source)
{
	using namespace mcs;
	
	Tanh(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Asinh(Matrix<T>& source)
{
	using namespace mcs;
	
	Asinh(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Acosh(Matrix<T>& source)
{
	using namespace mcs;
	
	Acosh(source.asVector());
	
	return source;
}

template<class T> const Matrix<T>& Atanh(Matrix<T>& source)
{
	using namespace mcs;
	
	Atanh(source.asVector());

	return source;
}

// not in place
// arithmetic functions
template<class T> Matrix<T> abs(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Abs(tmp);
	
	return tmp;
}

template<class T> Matrix<T> ceil(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Ceil(tmp);
	
	return tmp;
}

template<class T> Matrix<T> exp(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Exp(tmp);
	
	return tmp;
}

template<class T> Matrix<T> floor(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Floor(tmp);
	
	return tmp;
}

template<class T> Matrix<T> log(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Log(tmp);
	
	return tmp;
}

template<class T> Matrix<T> pow(const Matrix<T>& source,const T& expon)
{
	Matrix<T>	tmp(source);
	
	Pow(tmp,expon);
	
	return tmp;
}

template<class T> Matrix<T> round(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Round(tmp);
	
	return tmp;
}

template<class T> Matrix<T> sqr(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Sqr(tmp);
	
	return tmp;
}

template<class T> Matrix<T> sqrt(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Sqrt(tmp);
	
	return tmp;
}

// complex vectors
template<class T> Matrix<T> conj(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Conj(tmp);
	
	return tmp;
}

template<class T> Matrix<typename DemoteComplex<T>::demote_t> imag(const Matrix<T>& source)
{
	Matrix<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Imag(tmp);
	
	return tmp;
}

template<class T> Matrix<T> imag(const Matrix< std::complex<T> >& source)
{
	return Matrix<T>(source.range(),imag(source.asVector()));
}

template<class T> Matrix<typename DemoteComplex<T>::demote_t> real(const Matrix<T>& source)
{
	Matrix<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Real(tmp);
	
	return tmp;
}

template<class T> Matrix<T> real(const Matrix< std::complex<T> >& source)
{
	return Matrix<T>(source.range(),real(source.asVector()));
}

// trigonometric functions
template<class T> Matrix<T> sin(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Sin(tmp);
	
	return tmp;
}

template<class T> Matrix<T> cos(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Cos(tmp);
	
	return tmp;
}

template<class T> Matrix<T> tan(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Tan(tmp);
	
	return tmp;
}

template<class T> Matrix<T> asin(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Asin(tmp);
	
	return tmp;
}

template<class T> Matrix<T> acos(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Acos(tmp);
	
	return tmp;
}

template<class T> Matrix<T> atan(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Atan(tmp);
	
	return tmp;
}

template<class T> Matrix<T> atan2(const Matrix<T>& y,const Matrix<T>& x)
{
	using namespace mcs;
	
	static const EnsureFloat<T>	dummy_;
	
	if (x.range() != y.range()) throw Matrix<T>::RangeErr();

	return Matrix<T>(x.range(),atan2(y.asVector(),x.asVector()));
}

// hyperbolic functions
template<class T> Matrix<T> sinh(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Sinh(tmp);
	
	return tmp;
}

template<class T> Matrix<T> cosh(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Cosh(tmp);
	
	return tmp;
}

template<class T> Matrix<T> tanh(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Tanh(tmp);
	
	return tmp;
}

template<class T> Matrix<T> asinh(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Asinh(tmp);
	
	return tmp;
}

template<class T> Matrix<T> acosh(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Acosh(tmp);
	
	return tmp;
}

template<class T> Matrix<T> atanh(const Matrix<T>& source)
{
	Matrix<T>	tmp(source);
	
	Atanh(tmp);
	
	return tmp;
}

// miscellaneous
template<class T> inline Matrix<T> bound(const Matrix<T>& x,const T& min_val,const T& max_val)
{
	return Matrix<T>(x.range(),bound(x.asVector(),min_val,max_val));
}

template<class T> inline Matrix<Binary> even(const Matrix<T>& x)
{
	return Matrix<Binary>(x.range(),even(x.asVector()));
}

template<class T> inline Matrix<Binary> odd(const Matrix<T>& x)
{
	return Matrix<Binary>(x.range(),odd(x.asVector()));
}

template<class T> inline Matrix<T> mod(const Matrix<T>& x,const T& y)
{
	return Matrix<T>(x.range(),mod(x.asVector(),y));
}

template<class T> inline Matrix<T> mod(const Matrix<T>& x,const Matrix<T>& y)
{
	if (x.range() != y.range()) throw Matrix<T>::SizeMismatchErr();
	
	return Matrix<T>(x.range(),mod(x.asVector(),y.asVector()));
}

template<class T> inline Matrix<T> quotient(const Matrix<T>& x,const T& y)
{
	return Matrix<T>(x.range(),quotient(x.asVector(),y));
}

template<class T> inline Matrix<T> quotient(const Matrix<T>& x,const Matrix<T>& y)
{
	if (x.range() != y.range()) throw Matrix<T>::SizeMismatchErr();
	
	return Matrix<T>(x.range(),quotient(x.asVector(),y.asVector()));
}

// Matrix specific
template<class T> inline bool all(const Matrix<T>& source)
{
	return all(source.asVector());
}

template<class T> inline bool any(const Matrix<T>& source)
{
	return any(source.asVector());
}

// determine if matrix is square
template<class T> inline bool isSquare(const Matrix<T>& source)
{
	return (source.dim(0) == source.dim(1));
}

// product of all elements
template<class T> inline T prod(const Matrix<T>& source)
{
	return prod(source.asVector());
}

// sum of all elements
template<class T> inline T sum(const Matrix<T>& source)
{
	return sum(source.asVector());
}

template<class T> inline T trace(const Matrix<T>& source)
{
	return sum(diagonal(source));
}

// diagonal vector
template<class T> Vector<T> diagonal(const Matrix<T>& source)
{
	if (!isSquare(source)) throw Matrix<T>::IllegalOperationErr();
	
	Vector<T>	diag(source.rows());
	
	for (Int32 i=diag.first();i<=diag.last();i++)
		diag[i] = source(i,i);
		
	return diag;
}

// matrix-matrix product
template<class T> Matrix<T> product(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw Matrix<T>::RangeErr();
			
	Matrix<T> tmp(lhs.range());
	
	for (Int32 i=lhs.first(0);i<=lhs.last(0);i++)
		for (Int32 j=lhs.first(1);j<=lhs.last(1);j++)
			for (Int32 k=rhs.first(1);k<=rhs.last(1);k++)
				tmp(i,k) += lhs(i,j)*rhs(j,k);
		
	return tmp;
}

template<class T> Matrix<T> transpose(const Matrix<T>& source)
{
	const IndexRange<2>	rng(source.range());
	
	Matrix<T> tmp(IndexRange<2>(Index<2>(rng.first(1),rng.first(0)),
								Index<2>(rng.last(1),rng.last(0))));
		
	for (Int32 i=tmp.first(1);i<=tmp.last(0);i++)
		for (Int32 j=tmp.first(1);j<=tmp.last(1);j++)
			tmp(i,j) = source(j,i);

	return tmp;
}

} // namespace mcs
