#ifndef MCS_MATRIXINDIRECTION_H
#define MCS_MATRIXINDIRECTION_H

#include "MCS_Matrix.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

// matrix indirection class
class MatrixIndirection : public Vector< Index<2> >
{
	public:
		explicit MatrixIndirection(const IndexRange<1>&);
		explicit MatrixIndirection(const IndexRange<1>&,const Index<2>&);
		explicit MatrixIndirection(const IndexRange<1>&,const Vector< Int32 >&,const Vector< Int32 >&);
		explicit MatrixIndirection(const Vector< Index<2> >&);
		MatrixIndirection(const MatrixIndirection&);
		~MatrixIndirection();
		
		MatrixIndirection& operator=(const MatrixIndirection&);
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,VectorException)
		__DECLARE_EXCEPTION_CLASS__(IllegalOperationErr,Err)
		__DECLARE_EXCEPTION_CLASS__(InvalidDataErr,Err)
		__DECLARE_EXCEPTION_CLASS__(IOErr,Err)
		__DECLARE_EXCEPTION_CLASS__(MemoryErr,Err)
		__DECLARE_EXCEPTION_CLASS__(RangeErr,Err)
		__DECLARE_EXCEPTION_CLASS__(SizeMismatchErr,Err)
		__DECLARE_EXCEPTION_CLASS__(UnimplementedErr,Err)
};

inline MatrixIndirection::MatrixIndirection(const IndexRange<1>& rng) :
	Vector< Index<2> >(rng)
{ }

inline MatrixIndirection::MatrixIndirection(const IndexRange<1>& rng,const Index<2>& val) :
	Vector< Index<2> >(rng,val)
{ }

inline MatrixIndirection::MatrixIndirection(const IndexRange<1>& rng,const Vector< Int32 >& idx1,const Vector< Int32 >& idx2)
	: Vector< Index<2> >(rng)
{
	if (idx1.range() != rng || idx2.range() != rng) throw SizeMismatchErr();
	
	for (Int32 i=first();i<=last();i++)
		(*this)[i] = Index<2>(idx1[i],idx2[i]);
}
		
inline MatrixIndirection::MatrixIndirection(const Vector< Index<2> >& source) :
	Vector< Index<2> >(source)
{ }

inline MatrixIndirection::MatrixIndirection(const MatrixIndirection& source) :
	Vector< Index<2> >(source)
{ }

inline MatrixIndirection::~MatrixIndirection()
{ }

inline MatrixIndirection& MatrixIndirection::operator=(const MatrixIndirection& source)
{
	if (this == &source) return *this;

	Vector< Index<2> >::operator=(source);
		
	return *this;
}

} // namespace mcs

#endif // MCS_MATRIXINDIRECTION_H