			 	//Vector<T> 	column(Index) const;
			 	//Vector<T> 	diag(Index = 0) const;
			 	//Vector<T> 	row(Index) const;
		
		// modifiers			
			 	NumericMatrix<T>& 	flip_columns();
			 	NumericMatrix<T>& 	flip_rows();
			 	NumericMatrix<T>& 	remove_columns(const Vector<bool>&) const;
			 	NumericMatrix<T>& 	remove_rows(const Vector<bool>&) const;
			 	NumericMatrix<T>&	swap_columns(Matrix::Index,Matrix::Index);
			 	NumericMatrix<T>&	swap_rows(Matrix::Index,Matrix::Index);
			 	
			 	NumericMatrix<bool> 	is_nan() const;
			 	NumericMatrix<bool> 	not_nan() const;
			 	bool 					square() const;
		
		


template<class T> Vector<T> Matrix<T>::column(Index idx) const
{
	Vector<T> tmp(x_range());
	
	for (Index i=rowRange().first();i<=rowRange().last();i++)
		tmp[i] = (*this)[i][idx];
		
	return tmp;
}

// need to implement off-diagonals
template<class T> Vector<T> Matrix<T>::diag(Index idx) const
{
	if (!square() || idx != 0) throw Mat::RangeErr();
	
	Vector<T> tmp(x_range());
	
	for (Index i=rowRange().first();i<=rowRange().last();i++)
		tmp[i] = (*this)[i][i];
		
	return tmp;
}

template<class T> inline Vector<T> Matrix<T>::row(Index idx) const
{ 
	return (*this)[idx];
}

// modifiers
template<class T> Matrix<T>& Matrix<T>::flip_columns()
{
	Index	middle = first_y() - 1 + Index(std::floor(y_dim()/2.0));
	
	for (Index j=first_y(),k=last_y();j<=middle;j++,k--)
		swap_columns(j,k);

	return (*this);
}

template<class T> Matrix<T>& Matrix<T>::flip_rows()
{
	Index	middle = rowRange().first() - 1 + Index(std::floor(x_dim()/2.0));
	
	for (Index i=rowRange().first(),j=rowRange().last();i<=middle;i++,j--) 
		swap_rows(i,j);
		
	return (*this);
}

// untested 98/12/27
// remove columns for which mask is true
template<class T> Matrix<T>& Matrix<T>::remove_columns(const Vector<bool>& mask) const
{
	if (mask.range() != y_range()) throw Mat::RangeErr();
	
	Index num_columns = 0;
	
	for (Index i=mask.first();i<=mask.last();i++)
		if (!mask[i]) num_columns++;
	
	Matrix<T>	tmp(x_range(),v_range(first_y(),first_y()+num_columns-1));
	
	for (Index i=rowRange().first();i<=rowRange().last();i++)
		for (Index j=mask.first(),k=tmp.first_y();j<=mask.last();j++)
			if (!mask[j]) tmp[i][k++] = (*this)[i][j];
			
	return ((*this) = tmp);
}

// untested 98/12/27
// deflate rows, eliminating rows for which mask is false
template<class T> Matrix<T>& Matrix<T>::remove_rows(const Vector<bool>& mask) const
{
	if (mask.range() != x_range()) throw Mat::RangeErr();
	
	Index num_rows = 0;
	
	for (Index i=mask.first();i<=mask.last();i++)
		if (!mask[i]) num_rows++;
	
	Matrix<T>	tmp(v_range(rowRange().first(),rowRange().first()+num_rows-1),y_range());
	
	for (Index i=rowRange().first(),j=tmp.rowRange().first();i<=rowRange().last();i++)
		if (!mask[i]) tmp[j++] = (*this)[i];
		
	return ((*this) = tmp);
}

template<class T> Matrix<T> Matrix<T>::submatrix(const Range& rng) const
{
	if (rowRange().first() > rng.rowRange().first() || rowRange().last() < rng.rowRange().last() ||
	     first_y() > rng.first_y() || last_y() < rng.last_y()) throw Mat::RangeErr();
		
	Matrix<T> tmp(rng);
	
	if (rng == range())
		return (*this);
	else
		for (Index i=tmp.rowRange().first();i<=tmp.rowRange().last();i++)
			tmp[i] = (*this)[i][v_range(tmp.first_y(),tmp.last_y())];
		
	return tmp;
}

template<class T> Matrix<T>& Matrix<T>::swap_columns(Index col_1,Index col_2)
{
	for (Index i=rowRange().first();i<=rowRange().last();i++)
		swap((*this)[i][col_1],(*this)[i][col_2]);
		
	return (*this);
}

template<class T> Matrix<T>& Matrix<T>::swap_rows(Index row_1,Index row_2)
{
	Vector<T>	tmp((*this)[row_1]);
	
	(*this)[row_1] = (*this)[row_2];
	(*this)[row_2] = tmp;
	
	return (*this);
}

template<class T> typename Matrix<T>::Mask Matrix<T>::is_nan() const
{
	Matrix<bool> tmp(range(),false);
	
	for (Mat::Index i=rowRange().first();i<=rowRange().last();i++)
		tmp[i] = (*this)[i].is_nan();
	
	return tmp;
}

template<class T> typename Matrix<T>::Mask Matrix<T>::not_nan() const
{
	Matrix<bool> tmp(range(),false);
	
	for (Mat::Index i=rowRange().first();i<=rowRange().last();i++)
		tmp[i] = (*this)[i].not_nan();
	
	return tmp;
}

template<class T> inline bool Matrix<T>::square() const
{ return (rowRange().first() == first_y() && rowRange().last() == last_y()); }

