#include "MCS_Matrix.h"

namespace mcs {

// logical operators

// Matrix op scalar	
template<class T> inline MatrixMask operator==(const Matrix<T>& lmat,const T& rscal)
{
	return MatrixMask(lmat.range(),lmat.asVector() == rscal);
}

template<class T> inline MatrixMask operator!=(const Matrix<T>& lmat,const T& rscal)
{
	return MatrixMask(lmat.range(),lmat.asVector() != rscal);
}

template<class T> inline MatrixMask operator<(const Matrix<T>& lmat,const T& rscal)
{
	return MatrixMask(lmat.range(),lmat.asVector() < rscal);
}

template<class T> inline MatrixMask operator<=(const Matrix<T>& lmat,const T& rscal)
{
	return MatrixMask(lmat.range(),lmat.asVector() <= rscal);
}

template<class T> inline MatrixMask operator>(const Matrix<T>& lmat,const T& rscal)
{
	return MatrixMask(lmat.range(),lmat.asVector() > rscal);
}

template<class T> inline MatrixMask operator>=(const Matrix<T>& lmat,const T& rscal)
{
	return MatrixMask(lmat.range(),lmat.asVector() >= rscal);
}

// Matrix op Matrix
template<class T> inline MatrixMask operator==(const Matrix<T>& lmat,const Matrix<T>& rmat)
{
	if (lmat.range() != rmat.range()) throw typename Matrix<T>::SizeMismatchErr();
	
	return MatrixMask(lmat.range(),lmat.asVector() == rmat.asVector());
}

template<class T> inline MatrixMask operator!=(const Matrix<T>& lmat,const Matrix<T>& rmat)
{
	if (lmat.range() != rmat.range()) throw typename Matrix<T>::SizeMismatchErr();
	
	return MatrixMask(lmat.range(),lmat.asVector() != rmat.asVector());
}

template<class T> inline MatrixMask operator<(const Matrix<T>& lmat,const Matrix<T>& rmat)
{
	if (lmat.range() != rmat.range()) throw typename Matrix<T>::SizeMismatchErr();
	
	return MatrixMask(lmat.range(),lmat.asVector() < rmat.asVector());
}

template<class T> inline MatrixMask operator<=(const Matrix<T>& lmat,const Matrix<T>& rmat)
{
	if (lmat.range() != rmat.range()) throw typename Matrix<T>::SizeMismatchErr();
	
	return MatrixMask(lmat.range(),lmat.asVector() <= rmat.asVector());
}

template<class T> inline MatrixMask operator>(const Matrix<T>& lmat,const Matrix<T>& rmat)
{
	if (lmat.range() != rmat.range()) throw typename Matrix<T>::SizeMismatchErr();
	
	return MatrixMask(lmat.range(),lmat.asVector() > rmat.asVector());
}

template<class T> inline MatrixMask operator>=(const Matrix<T>& lmat,const Matrix<T>& rmat)
{
	if (lmat.range() != rmat.range()) throw typename Matrix<T>::SizeMismatchErr();
	
	return MatrixMask(lmat.range(),lmat.asVector() >= rmat.asVector());
}

// arithmetic operators

// scalar op Matrix
template<class T> inline  Matrix<T> operator+(const T& lscal,const Matrix<T>& rmat)
{
	if (lscal == static_cast<T>(0)) return rmat;

	return (Matrix<T>(rmat.range(),lscal) += rmat);
}

template<class T> inline  Matrix<T> operator-(const T& lscal,const Matrix<T>& rmat)
{
	if (lscal == static_cast<T>(0)) return rmat;
	/*		
	Matrix<T>	tmp(rmat.range(),lscal);
	
	tmp -= rmat;
	
	return tmp;
	*/
	return (Matrix<T>(rmat.range(),lscal) -= rmat);
}

template<class T> inline  Matrix<T> operator*(const T& lscal,const Matrix<T>& rmat)
{
	if (lscal == static_cast<T>(1)) return rmat;
	if (lscal == static_cast<T>(0)) return Matrix<T>(rmat.range(),static_cast<T>(0));	
	/*		
	Matrix<T>	tmp(rmat.range(),lscal);
	
	tmp *= rmat;
	
	return tmp;
	*/
	return (Matrix<T>(rmat.range(),lscal) *= rmat);
}

template<class T> inline  Matrix<T> operator/(const T& lscal,const Matrix<T>& rmat)
{
	if (lscal == static_cast<T>(1)) return rmat;
	/*	
	Matrix<T>	tmp(rmat.range(),lscal);
	
	tmp /= rmat;
	
	return tmp;
	*/
	return (Matrix<T>(rmat.range(),lscal) /= rmat);
}

// Matrix op scalar
template<class T> inline  Matrix<T> operator+(const Matrix<T>& lmat,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lmat;
	/*
	Matrix<T>	tmp(lmat);
	
	tmp += rscal;
	
	return tmp;
	*/
	return (Matrix<T>(lmat) += rscal);
}

template<class T> inline  Matrix<T> operator-(const Matrix<T>& lmat,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lmat;
	/*		
	Matrix<T>	tmp(lmat);
	
	tmp -= rscal;
	
	return tmp;
	*/
	return (Matrix<T>(lmat) -= rscal);
}

template<class T> inline  Matrix<T> operator*(const Matrix<T>& lmat,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lmat;
	if (rscal == static_cast<T>(0)) return Matrix<T>(lmat.range(),static_cast<T>(0));	
	/*
	Matrix<T>	tmp(lmat);
	
	tmp *= rscal;
	
	return tmp;
	*/
	return (Matrix<T>(lmat) *= rscal);
}

template<class T> inline  Matrix<T> operator/(const Matrix<T>& lmat,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lmat;
	/*		
	Matrix<T>	tmp(lmat);
	
	tmp /= rscal;
	
	return tmp;
	*/
	return (Matrix<T>(lmat)/=rscal);
}

// Matrix op Matrix
template<class T> inline MatrixMask operator&&(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Matrix<T>::SizeMismatchErr();
			
	return MatrixMask(lhs.range(),lhs.asVector() && rhs.asVector());
}

template<class T> inline MatrixMask operator||(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Matrix<T>::SizeMismatchErr();
			
	return MatrixMask(lhs.range(),lhs.asVector() || rhs.asVector());

}

template<class T> inline Matrix<T> operator+(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Matrix<T>::SizeMismatchErr();
			
	return Matrix<T>(lhs.range(),lhs.asVector()+rhs.asVector());
}

template<class T> inline Matrix<T> operator-(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Matrix<T>::SizeMismatchErr();
						
	return Matrix<T>(lhs.range(),lhs.asVector()-rhs.asVector());
}

template<class T> inline Matrix<T> operator*(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Matrix<T>::SizeMismatchErr();
			
	return Matrix<T>(lhs.range(),lhs.asVector()*rhs.asVector());
}

template<class T> inline Matrix<T> operator/(const Matrix<T>& lhs,const Matrix<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Matrix<T>::SizeMismatchErr();
			
	return Matrix<T>(lhs.range(),lhs.asVector()/rhs.asVector());
}

// Vector op Matrix
template<class T> Vector<T> operator*(const Vector<T>& lvec,const Matrix<T>& rmat)
{
	if (lvec.first() != rmat.first(0) || lvec.last() != rmat.last(0)) 
		throw typename Matrix<T>::SizeMismatchErr();
	
	Vector<T> tmp(rmat.range(1));
	
	for (Int32 i=rmat.first(0);i<=rmat.last(0);i++)
		for (Int32 j=rmat.first(1);j<=rmat.last(1);j++)
			tmp[i] += lvec[i]*rmat(i,j);
		
	return tmp;
}

// Matrix op Vector
template<class T> Vector<T> operator*(const Matrix<T>& lmat,const Vector<T>& rvec)
{
	if (rvec.range() != lmat.range(1)) 
		throw typename Matrix<T>::SizeMismatchErr();
	
	Vector<T> tmp(lmat.range(0));
	
	for (Int32 i=lmat.first(0);i<=lmat.last(0);i++)
		for (Int32 j=lmat.first(1);j<=lmat.last(1);j++)
			tmp[j] += lmat(i,j)*rvec[j];
				
	return tmp;
}

// unary operators
template<class T> inline  Matrix<T> operator+(const Matrix<T>& source)
{ 
	return source; 
}

template<class T> inline  Matrix<T> operator-(const Matrix<T>& source)
{
	return Matrix<T>(source.range(),-source.asVector());
}

template<class T> inline Matrix<T> operator!(const Matrix<T>& source)
{
	return Matrix<T>(source.range(),!source.asVector());
}

} // namespace mcs
