#ifndef MCS_MATRIXUTILITIES_H
#define MCS_MATRIXUTILITIES_H

#include "MCS_Matrix.h"

namespace mcs {

// miscellaneous utility functions
template<class T,class F> Matrix<T> apply(const F& func,const Matrix<T>& source);
template<class T,class F> Matrix<T> apply(const F& func,const Matrix<T>& vec1,const Matrix<T>& vec2);

template<class T> Vector<T> column(const Matrix<T>& source,const Int32& col);
template<class T> Vector<T> row(const Matrix<T>& source,const Int32& row);

template<class T> MatrixIndirection find(const Matrix<T>& source);

template<class T> Matrix<T> outer(const Vector<T>& lhs,const Vector<T>& rhs);

template<class T> void swapColumns(Matrix<T>& source,const Int32& col1,const Int32& col2);
template<class T> void swapRows(Matrix<T>& source,const Int32& row1,const Int32& row2);

// miscellaneous operators
template<class T,class F> Matrix<T> apply(const F& func,const Matrix<T>& source)
{
	Matrix<T>	tmp(source.range());
	
	for (Int32 i=source.first();i<=source.last();i++)
		tmp[i] = func(source[i]);
		
	return tmp;
}

template<class T,class F> Matrix<T> apply(const F& func,const Matrix<T>& vec1,const Matrix<T>& vec2)
{
	if (vec1.range() != vec2.range()) throw Matrix<T>::RangeErr();
	
	Matrix<T>	tmp(vec1.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = func(vec1[i],vec2[i]);
		
	return tmp;
}

template<class T> Vector<T> column(const Matrix<T>& source,const Int32& colidx)
{
	Vector<T>	vec(IndexRange<1>(source.first(0),source.last(0)));
	
	for (Int32 i=source.first(0);i<=source.last(0);i++)
		vec[i] = source(i,colidx);
		
	return vec;
}

template<class T> Vector<T> row(const Matrix<T>& source,const Int32& rowidx)
{
	Vector<T>	vec(IndexRange<1>(source.first(1),source.last(1)));
	
	for (Int32 j=source.first(1);j<=source.last(1);j++)
		vec[j] = source(rowidx,j);
		
	return vec;
}
/*
template<class T> MatrixIndirection find(const Matrix<T>& source)
{
	Int32	num_elem = 0;
	
	for (Int32 i=source.first(0);i<=source.last(0);i++)
		for (Int32 j=source.first(1);j<=source.last(1);j++)
			if (source(i,j)) num_elem++;
		
	MatrixIndirection	tmp(IndexRange<1>(num_elem));
	
	for (Int32 i=source.first(0),k=tmp.first();i<=source.last(0);i++)
		for (Int32 j=source.first(1);j<=source.last(1);j++)
			if (source(i,j)) tmp[k++] = Index<2>(i,j);
		
	return tmp;
}
*/
template<class T> Matrix<T> outer(const Vector<T>& lhs,const Vector<T>& rhs)
{
	Matrix<T> tmp(IndexRange<2>(Index<2>(lhs.first(),rhs.first()),Index<2>(lhs.last(),rhs.last())));
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		for (Int32 j=rhs.first();j<=rhs.last();j++)
			tmp(i,j) = lhs[i]*rhs[j];
			
	return tmp;
}

template<class T> void swapColumns(Matrix<T>& source,const Int32& col1,const Int32& col2)
{
	for (Int32 i=source.first(0);i<=source.last(0);++i)
		std::swap(source(i,col1),source(i,col2));
}

template<class T> void swapRows(Matrix<T>& source,const Int32& row1,const Int32& row2)
{
	for (Int32 j=source.first(1);j<=source.last(1);++j)
		std::swap(source(row1,j),source(row2,j));
}

} // namespace mcs

#endif // MCS_MATRIXUTILITIES_H