#ifndef MCS_FOOTPRINTBIN_H
#define MCS_FOOTPRINTBIN_H

#include "MCS_Constants.h"
#include "MCS_Ensure.h"
#include "MCS_FixedVector.h"
#include "MCS_TensorBin.h"

namespace mcs {

template<class T> class Footprint
{
	public:
		Footprint() : x_(_NaN_),y_(_NaN_) { }
		Footprint(const T& x,const T& y) : x_(x),y_(y) { }
		~Footprint() { }
		
		const T&	delta_x() const	{ return x_; }
		const T& 	delta_y() const	{ return y_; }
		
	private:
		static const EnsureFloat<T>	dummy_;
		
		T	x_,y_;	
};

template<class T> const EnsureFloat<T>	Footprint<T>::dummy_;

template<class T> class FootprintBin : 
	public TensorBin<T,2>
{
	public:
		typedef TensorBin<T,2>	BinType;
		
		FootprintBin(const typename BinType::BoundsType& bds) : BinType(bds) { }
		~FootprintBin() { }
		
		void add(const FixedVector<T,2>& x,const T& z,const T& w = T(1))		{ TensorBin<T,2>::add(x,z,w); }
		void add(const FixedVector<T,2>& x,const T& z,const Footprint<T>& fp);

	private:
		static const EnsureFloat<T>	dummy_;		
};

template<class T> const EnsureFloat<T>	FootprintBin<T>::dummy_;

template<class T>
inline void FootprintBin<T>::add(const FixedVector<T,2>& x,const T& z,const Footprint<T>& fp)
{
	const T	dlat = bounds().delta(0),
			footlat = 1.5*fp.delta_x()*360/(2*_pi_*(constants::physical::_r_Ep_/1000.0)),
			latrng = std::max(footlat,dlat),	
			dlon = bounds().delta(1),
			footlon = 1.5*fp.delta_y()*360/(2*_pi_*(constants::physical::_r_Ee_/1000.0)*cosd(x[0])),
			lonrng = std::max(footlon,dlon),
			rho2 = (sqr(latrng)+sqr(lonrng))/2,
			w0 = 2/rho2;		
	
	TensorBin<T,2>::add(x,z,w0);
	
	for (T lat=x[0]-latrng;lat<=x[0]+latrng;lat+=dlat)
	{
		const T	deltalat = lat-x[0];
		
		for (T lon=x[1]-lonrng;lon<=x[1]+lonrng;lon+=dlon)
		{
			const T	deltalon = lon-x[1],
					rad2 = sqr(deltalat)+sqr(deltalon),
					wt = std::exp(-(rad2/rho2))*w0;

			if (wt >= 0.1*w0)				
				TensorBin<T,2>::add(FixedVector<T,2>(lat,mod(lon,static_cast<Float32>(360.0))),z,wt);
		}
	}
	
	return;
}

} // namespace mcs

#endif MCS_FOOTPRINTBIN_H