#ifndef MCS_SEQUENCES_H
#define MCS_SEQUENCES_H

#include "MCS_Vector.h"

namespace mcs {

namespace num {

template<class T> Vector<T> arithmeticSequence(const IndexRange<1>& rng)
{
	Vector<T>	tmp(rng,static_cast<T>(rng.first()));
	
	for (Index<1> i=tmp.first()+1;i<=tmp.last();i++)
		tmp[i] = tmp[i-1]+1;
		
	return tmp;
}

template<class T> Vector<T> arithmeticSequence(const IndexRange<1>& rng,const T& lo,const T& hi)
{
	Vector<T>	tmp(rng,lo);
	
	const T	step = (hi-lo)/(rng.dim()-1);
	
	for (Index<1> i=tmp.first()+1;i<=tmp.last();i++)
		tmp[i] = tmp[i-1]+step;
		
	return tmp;
}

template<class T> Vector<T> geometricSequence(const IndexRange<1>& rng,const T& lo,const T& hi)
{
	Vector<T>	tmp = exp(arithmeticSequence(T(0),std::log(hi-lo+1),rng));
	
	return tmp+lo-T(1);
}

} // namespace num

} // namespace mcs

#endif // MCS_SEQUENCES_H