#ifndef MCS_NRDOUBLEPROTOTYPES_H
#define MCS_NRDOUBLEPROTOTYPES_H

#include <cstdio>

#include "MCS_NumericalRecipes.h"

namespace NR {

namespace DP {

// NR Ch 1 Preliminaries
long julday(int mm, int id, int iyyy);
void flmoon(int n, int nph, long *jd, double *frac);
void caldat(long julian, int *mm, int *id, int *iyyy);

// NR Ch 2 Solution of linear algebraic equations
void gaussj(double **a, int n, double **b, int m);
void ludcmp(double **a, int n, int *indx, double *d);
void lubksb(double **a, int n, int *indx, double b[]);
void tridag(double a[],double b[],double c[],double r[],double u[],unsigned long n);
void banmul(double **a, unsigned long n, int m1, int m2, double x[], double b[]);
void bandec(double **a, unsigned long n, int m1, int m2, double **al,unsigned long indx[], double *d);
void banbks(double **a, unsigned long n, int m1, int m2, double **al,unsigned long indx[], double b[]);
void mprove(double **a, double **alud, int n, int indx[], double b[], double x[]);
void svbksb(double **u, double w[], double **v, int m, int n, double b[], double x[]);
void svdcmp(double **a, int m, int n, double w[], double **v);
double pythag(double a, double b);
void cyclic(double a[], double b[], double c[], double alpha, double beta,double r[], double x[], unsigned long n);
void sprsin(double **a, int n, double thresh, unsigned long nmax, double sa[],unsigned long ija[]);
void sprsax(double sa[], unsigned long ija[], double x[], double b[],unsigned long n);
void sprstx(double sa[], unsigned long ija[], double x[], double b[],unsigned long n);
void sprstp(double sa[], unsigned long ija[], double sb[], unsigned long ijb[]);
void sprspm(double sa[], unsigned long ija[], double sb[], unsigned long ijb[],double sc[], unsigned long ijc[]);
void sprstm(double sa[], unsigned long ija[], double sb[], unsigned long ijb[],double thresh, unsigned long nmax, double sc[], unsigned long ijc[]);
void linbcg(unsigned long n, double b[], double x[], int itol, double tol,int itmax, int *iter, double *err);
double snrm(unsigned long n, double sx[], int itol);
void vander(double x[], double w[], double q[], int n);
void toeplz(double r[], double x[], double y[], int n);
void choldc(double **a, int n, double p[]);
void cholsl(double **a, int n, double p[], double b[], double x[]);
void qrdcmp(double **a, int n, double *c, double *d, int *sing);
void qrsolv(double **a, int n, double c[], double d[], double b[]);
void rsolv(double **a, int n, double d[], double b[]);
void qrupdt(double **r, double **qt, int n, double u[], double v[]);
void rotate(double **r, double **qt, int n, int i, double a, double b);

// NR Ch 3 Interpolation and extrapolation
void polint(double xa[], double ya[], int n, double x, double *y, double *dy);
void ratint(double xa[], double ya[], int n, double x, double *y, double *dy);
void spline(double x[], double y[], int n, double yp1, double ypn, double y2[]);
void splint(double xa[], double ya[], double y2a[], int n, double x, double *y);
void locate(double xx[], unsigned long n, double x, unsigned long *j);
void hunt(double xx[], unsigned long n, double x, unsigned long *jlo);
void polcoe(double x[], double y[], int n, double cof[]);
void polcof(double xa[], double ya[], int n, double cof[]);
void blnint(double y[], double x1l,double x1u, double x2l, double x2u, double x1, double x2, double *ansy);
void polin2(double x1a[], double x2a[], double **ya, int m, int n, double x1,double x2, double *y, double *dy);
void bcucof(double y[], double y1[], double y2[], double y12[], double d1, double d2, double **c);
void bcuint(double y[], double y1[], double y2[], double y12[], double x1l,double x1u, double x2l, double x2u, double x1, double x2, double *ansy,double *ansy1, double *ansy2);
void splie2(double x1a[], double x2a[], double **ya, int m, int n, double **y2a);
void splin2(double x1a[], double x2a[], double **ya, double **y2a, int m, int n,double x1, double x2, double *y);

// NR Ch 4 Integration of functions
double trapzd(double (*func)(double), double a, double b, int n);
double qtrap(double (*func)(double), double a, double b);
double qtrapr(double (*func)(double), double a, double b,double (*choose)(double(*)(double), double, double, int));
double qsimp(double (*func)(double), double a, double b);
double qsimpr(double (*func)(double), double a, double b,double (*choose)(double(*)(double), double, double, int));
double qromb(double (*func)(double), double a, double b);
double qrombr(double (*func)(double), double a, double b,double (*choose)(double(*)(double), double, double, int));
double midpnt(double (*func)(double), double a, double b, int n);
double qromo(double (*func)(double), double a, double b,double (*choose)(double(*)(double), double, double, int));
double midinf(double (*funk)(double), double aa, double bb, int n);
double midsql(double (*funk)(double), double aa, double bb, int n);
double midsqu(double (*funk)(double), double aa, double bb, int n);
double midexp(double (*funk)(double), double aa, double bb, int n);
double qgaus(double (*func)(double), double a, double b);
void gauleg(double x1, double x2, double x[], double w[], int n);
void gaulag(double x[], double w[], int n, double alf);
void gauher(double x[], double w[], int n);
void gaujac(double x[], double w[], int n, double alf, double bet);
void gaucof(int n, double a[], double b[], double amu0, double x[], double w[]);
void orthog(int n, double anu[], double alpha[], double beta[], double a[],double b[]);
double quad3d(double (*func)(double, double, double), double x1, double x2);
double f1(double x);
double f2(double y);
double f3(double z);

// NR Ch 5 Evaluation of functions
void eulsum(double *sum, double term, int jterm, double wksp[]);
void ddpoly(double c[], int nc, double x, double pd[], int nd);
void poldiv(double u[], int n, double v[], int nv, double q[], double r[]);
double ratval(double x, double cof[], int mm, int kk);
double dfridr(double (*func)(double), double x, double h, double *err);
void chebft(double a, double b, double c[], int n, double (*func)(double));
double chebev(double a, double b, double c[], int m, double x);
void chder(double a, double b, double c[], double cder[], int n);
void chint(double a, double b, double c[], double cint[], int n);
void chebpc(double c[], double d[], int n);
void pcshft(double a, double b, double d[], int n);
void pccheb(double d[], double c[], int n);
void pade(double cof[], int n, double *resid);
void ratlsq(double (*fn)(double), double a, double b, int mm, int kk,double cof[], double *dev);

// NR Ch 6 Special functions
double gammln(double xx);
double factrl(int n);
double bico(int n, int k);
double factln(int n);
double beta(double z, double w);
double gammp(double a, double x);
double gammq(double a, double x);
void gser(double *gamser, double a, double x, double *gln);
void gcf(double *gammcf, double a, double x, double *gln);
double erff(double x);
double erffc(double x);
double erfcc(double x);
double expint(int n, double x);
double ei(double x);
double betai(double a, double b, double x);
double betacf(double a, double b, double x);
double bessj0(double x);
double bessy0(double x);
double bessj1(double x);
double bessy1(double x);
double bessy(int n, double x);
double bessj(int n, double x);
double bessi0(double x);
double bessk0(double x);
double bessi1(double x);
double bessk1(double x);
double bessk(int n, double x);
double bessi(int n, double x);
void bessjy(double x, double xnu, double *rj, double *ry, double *rjp, double *ryp);
void beschb(double x, double *gam1, double *gam2, double *gampl, double *gammi);
void bessik(double x, double xnu, double *ri, double *rk, double *rip, double *rkp);
void airy(double x, double *ai, double *bi, double *aip, double *bip);
void sphbes(int n, double x, double *sj, double *sy, double *sjp, double *syp);
double plgndr(int l, int m, double x);
void frenel(double x, double *s, double *c);
void cisi(double x, double *ci, double *si);
double dawson(double x);
double rf(double x, double y, double z);
double rd(double x, double y, double z);
double rj(double x, double y, double z, double p);
double rc(double x, double y);
double ellf(double phi, double ak);
double elle(double phi, double ak);
double ellpi(double phi, double en, double ak);
void sncndn(double uu, double emmc, double *sn, double *cn, double *dn);
dcomplex hypgeo(dcomplex a, dcomplex b, dcomplex c, dcomplex z);
void hypser(dcomplex a, dcomplex b, dcomplex c, dcomplex z, dcomplex *series,dcomplex *deriv);
void hypdrv(double s, double yy[], double dyyds[]);

// NR Ch 7 Random numbers
double ran0(long *idum);
double ran1(long *idum);
double ran2(long *idum);
double ran3(long *idum);
double expdev(long *idum);
double gasdev(long *idum);
double gamdev(int ia, long *idum);
double poidev(double xm, long *idum);
double bnldev(double pp, int n, long *idum);
int irbit1(unsigned long *iseed);
int irbit2(unsigned long *iseed);
void psdes(unsigned long *lword, unsigned long *irword);
double ran4(long *idum);
void sobseq(int *n, double x[]);
void vegas(double regn[], int ndim, double (*fxn)(double [], double), int init,unsigned long ncall, int itmx, int nprn, double *tgral, double *sd,double *chi2a);
void rebin(double rc, int nd, double r[], double xin[], double xi[]);
void miser(double (*func)(double []), double regn[], int ndim, unsigned long npts,double dith, double *ave, double *var);
void ranpt(double pt[], double regn[], int n);

// NR Ch 8 Sorting
void piksrt(int n, double arr[]);
void piksr2(int n, double arr[], double brr[]);
void shell(unsigned long n, double a[]);
void sort(unsigned long n, double arr[]);
void sort2(unsigned long n, double arr[], double brr[]);
void hpsort(unsigned long n, double ra[]);
void indexx(unsigned long n, double arr[], unsigned long indx[]);
void iindexx(unsigned long n, long arr[], unsigned long indx[]);
void sort3(unsigned long n, double ra[], double rb[], double rc[]);
void rank(unsigned long n, unsigned long indx[], unsigned long irank[]);
double select(unsigned long k, unsigned long n, double arr[]);
double selip(unsigned long k, unsigned long n, double arr[]);
void hpsel(unsigned long m, unsigned long n, double arr[], double heap[]);
void eclass(int nf[], int n, int lista[], int listb[], int m);
void eclazz(int nf[], int n, int (*equiv)(int, int));

// NR Ch 9 Root finding and nonlinear sets of equations
int zbrac(double (*func)(double), double *x1, double *x2);
void zbrak(double (*fx)(double), double x1, double x2, int n, double xb1[],double xb2[], int *nb);
double rtbis(double (*func)(double), double x1, double x2, double xacc);
double rtflsp(double (*func)(double), double x1, double x2, double xacc);
double rtsec(double (*func)(double), double x1, double x2, double xacc);
double zriddr(double (*func)(double), double x1, double x2, double xacc);
double zbrent(double (*func)(double), double x1, double x2, double tol);
double rtnewt(void (*funcd)(double, double *, double *), double x1, double x2,double xacc);
double rtsafe(void (*funcd)(double, double *, double *), double x1, double x2,double xacc);
void laguer(dcomplex a[], int m, dcomplex *x, int *its);
void zroots(dcomplex a[], int m, dcomplex roots[], int polish);
void zrhqr(double a[], int m, double rtr[], double rti[]);
void qroot(double p[], int n, double *b, double *c, double eps);
void mnewt(int ntrial, double x[], int n, double tolx, double tolf);
void lnsrch(int n, double xold[], double fold, double g[], double p[], double x[],double *f, double stpmax, int *check, double (*func)(double []));
void newt(double x[], int n, int *check,void (*vecfunc)(int, double [], double []));
void fdjac(int n, double x[], double fvec[], double **df,void (*vecfunc)(int, double [], double []));
double fmin(double x[]);
void broydn(double x[], int n, int *check,void (*vecfunc)(int, double [], double []));

// NR Ch 10 Minimization and maximization of functions
void mnbrak(double *ax, double *bx, double *cx, double *fa, double *fb, double *fc,double (*func)(double));
double golden(double ax, double bx, double cx, double (*f)(double), double tol,double *xmin);
double brent(double ax, double bx, double cx, double (*f)(double), double tol,double *xmin);
double dbrent(double ax, double bx, double cx, double (*f)(double),double (*df)(double), double tol, double *xmin);
void amoeba(double **p, double y[], int ndim, double ftol,double (*funk)(double []), int *nfunk);
double amotry(double **p, double y[], double psum[], int ndim,double (*funk)(double []), int ihi, double fac);
void powell(double p[], double **xi, int n, double ftol, int *iter, double *fret,double (*func)(double []));
void linmin(double p[], double xi[], int n, double *fret, double (*func)(double []));
double f1dim(double x);
void frprmn(double p[], int n, double ftol, int *iter, double *fret,double (*func)(double []), void (*dfunc)(double [], double []));
void dlinmin(double p[], double xi[], int n, double *fret, double (*func)(double []),void (*dfunc)(double [], double []));
double df1dim(double x);
void dfpmin(double p[], int n, double gtol, int *iter, double *fret,double(*func)(double []), void (*dfunc)(double [], double []));
void simplx(double **a, int m, int n, int m1, int m2, int m3, int *icase,int izrov[], int iposv[]);
void simp1(double **a, int mm, int ll[], int nll, int iabf, int *kp,double *bmax);
void simp2(double **a, int n, int l2[], int nl2, int *ip, int kp, double *q1);
void simp3(double **a, int i1, int k1, int ip, int kp);
void anneal(double x[], double y[], int iorder[], int ncity);
double revcst(double x[], double y[], int iorder[], int ncity, int n[]);
void reverse(int iorder[], int ncity, int n[]);
double trncst(double x[], double y[], int iorder[], int ncity, int n[]);
void trnspt(int iorder[], int ncity, int n[]);
int metrop(double de, double t);
void amebsa(double **p, double y[], int ndim, double pb[], double *yb, double ftol,double (*funk)(double []), int *iter, double temptr);
double amotsa(double **p, double y[], double psum[], int ndim, double pb[],double *yb, double (*funk)(double []), int ihi, double *yhi, double fac);

// NR Ch 11 Eigensystems
void jacobi(double **a, int n, double d[], double **v, int *nrot);
void eigsrt(double d[], double **v, int n);
void tred2(double **a, int n, double d[], double e[]);
void tqli(double d[], double e[], int n, double **z);
void balanc(double **a, int n);
void elmhes(double **a, int n);
void hqr(double **a, int n, double wr[], double wi[]);

// NR Ch 12 Fast Fourier transform
void four1(double data[], unsigned long nn, int isign);
void twofft(double data1[], double data2[], double fft1[], double fft2[],unsigned long n);
void realft(double data[], unsigned long n, int isign);
void sinft(double y[], int n);
void cosft1(double y[], int n);
void cosft2(double y[], int n, int isign);
void fourn(double data[], unsigned long nn[], int ndim, int isign);
void rlft3(double ***data, double **speq, unsigned long nn1, unsigned long nn2,unsigned long nn3, int isign);
void fourfs(std::FILE *file[5], unsigned long nn[], int ndim, int isign);
void fourew(std::FILE *file[5], int *na, int *nb, int *nc, int *nd);

// NR Ch 13 Fourier and spectral applications
void convlv(double data[], unsigned long n, double respns[], unsigned long m,int isign, double ans[]);
void correl(double data1[], double data2[], unsigned long n, double ans[]);
void spctrm(std::FILE *fp, double p[], int m, int k, int ovrlap);
void memcof(double data[], int n, int m, double *xms, double d[]);
void fixrts(double d[], int m);
void predic(double data[], int ndata, double d[], int m, double future[],int nfut);
double evlmem(double fdt, double d[], int m, double xms);
void period(double x[], double y[], int n, double ofac, double hifac, double px[],double py[], int np, int *nout, int *jmax, double *prob);
void fasper(double x[], double y[], unsigned long n, double ofac, double hifac,double wk1[], double wk2[], unsigned long nwk, unsigned long *nout,unsigned long *jmax, double *prob);
void spread(double y, double yy[], unsigned long n, double x, int m);
void dftcor(double w, double delta, double a, double b, double endpts[],double *corre, double *corim, double *corfac);
void dftint(double (*func)(double), double a, double b, double w, double *cosint,double *sinint);
void wt1(double a[], unsigned long n, int isign,void (*wtstep)(double [], unsigned long, int));
void daub4(double a[], unsigned long n, int isign);
void pwtset(int n);
void pwt(double a[], unsigned long n, int isign);
void wtn(double a[], unsigned long nn[], int ndim, int isign,void (*wtstep)(double [], unsigned long, int));

// NR 14 Statistical description of data
void moment(double data[], int n, double *ave, double *adev, double *sdev,double *var, double *skew, double *curt);
void ttest(double data1[], unsigned long n1, double data2[], unsigned long n2,double *t, double *prob);
void avevar(double data[], unsigned long n, double *ave, double *var);
void tutest(double data1[], unsigned long n1, double data2[], unsigned long n2,double *t, double *prob);
void tptest(double data1[], double data2[], unsigned long n, double *t,double *prob);
void ftest(double data1[], unsigned long n1, double data2[], unsigned long n2,double *f, double *prob);
void chsone(double bins[], double ebins[], int nbins, int knstrn, double *df,double *chsq, double *prob);
void chstwo(double bins1[], double bins2[], int nbins, int knstrn, double *df,double *chsq, double *prob);
void ksone(double data[], unsigned long n, double (*func)(double), double *d,double *prob);
void kstwo(double data1[], unsigned long n1, double data2[], unsigned long n2,double *d, double *prob);
double probks(double alam);
void cntab1(int **nn, int ni, int nj, double *chisq, double *df, double *prob,double *cramrv, double *ccc);
void cntab2(int **nn, int ni, int nj, double *h, double *hx, double *hy,double *hygx, double *hxgy, double *uygx, double *uxgy, double *uxy);
void pearsn(double x[], double y[], unsigned long n, double *r, double *prob,double *z);
void spear(double data1[], double data2[], unsigned long n, double *d, double *zd,double *probd, double *rs, double *probrs);
void crank(unsigned long n, double w[], double *s);
void kendl1(double data1[], double data2[], unsigned long n, double *tau,double *z, double *prob);
void kendl2(double **tab, int i, int j, double *tau, double *z, double *prob);
void ks2d1s(double x1[], double y1[], unsigned long n1,void (*quadvl)(double, double, double *, double *, double *, double *),double *d1, double *prob);
void quadct(double x, double y, double xx[], double yy[], unsigned long nn,double *fa, double *fb, double *fc, double *fd);
void quadvl(double x, double y, double *fa, double *fb, double *fc, double *fd);
void ks2d2s(double x1[], double y1[], unsigned long n1, double x2[], double y2[],unsigned long n2, double *d, double *prob);
void savgol(double c[], int np, int nl, int nr, int ld, int m);

// NR Ch 15 Modeling of data
void fit(double x[], double y[], int ndata, double sig[], int mwt, double *a,double *b, double *siga, double *sigb, double *chi2, double *q);
void fitexy(double x[], double y[], int ndat, double sigx[], double sigy[],double *a, double *b, double *siga, double *sigb, double *chi2, double *q);
double chixy(double bang);
void lfit(double x[], double y[], double sig[], int ndat, double a[], int ia[],int ma, double **covar, double *chisq, void (*funcs)(double, double [], int));
void covsrt(double **covar, int ma, int ia[], int mfit);
void svdfit(double x[], double y[], double sig[], int ndata, double a[], int ma,double **u, double **v, double w[], double *chisq,void (*funcs)(double, double [], int));
void svdfitm(double **x, double y[], double sig[], int ndata, double a[], int ma,double **u, double **v, double w[], double *chisq);
void svdvar(double **v, int ma, double w[], double **cvm);
void fpoly(double x, double p[], int np);
void fleg(double x, double pl[], int nl);
void mrqmin(double x[], double y[], double sig[], int ndata, double a[], int ia[],int ma, double **covar, double **alpha, double *chisq,void (*funcs)(double, double [], double *, double [], int), double *alamda);
void mrqcof(double x[], double y[], double sig[], int ndata, double a[], int ia[],int ma, double **alpha, double beta[], double *chisq,void (*funcs)(double, double [], double *, double [], int));
void fgauss(double x, double a[], double *y, double dyda[], int na);
void medfit(double x[], double y[], int ndata, double *a, double *b, double *abdev);
double rofunc(double b);

// NR Ch 16 Integration of ordinary differential equations
void rk4(double y[], double dydx[], int n, double x, double h, double yout[],void (*derivs)(double, double [], double []));
void rkdumb(double vstart[], int nvar, double x1, double x2, int nstep,void (*derivs)(double, double [], double []));
void rkqs(double y[], double dydx[], int n, double *x, double htry, double eps,double yscal[], double *hdid, double *hnext,void (*derivs)(double, double [], double []));
void rkck(double y[], double dydx[], int n, double x, double h, double yout[],double yerr[], void (*derivs)(double, double [], double []));
void odeint(double ystart[], int nvar, double x1, double x2, double eps, double h1,double hmin, int *nok, int *nbad,void (*derivs)(double, double [], double []),void (*rkqs)(double [], double [], int, double *, double, double, double [],double *, double *, void (*)(double, double [], double [])));
void mmid(double y[], double dydx[], int nvar, double xs, double htot, int nstep,double yout[], void (*derivs)(double, double[], double[]));
void bsstep(double y[], double dydx[], int nv, double *xx, double htry, double eps,double yscal[], double *hdid, double *hnext,void (*derivs)(double, double [], double []));
void pzextr(int iest, double xest, double yest[], double yz[], double dy[], int nv);
void rzextr(int iest, double xest, double yest[], double yz[], double dy[], int nv);
void stoerm(double y[], double d2y[], int nv, double xs, double htot, int nstep,double yout[], void (*derivs)(double, double [], double []));
void stiff(double y[], double dydx[], int n, double *x, double htry, double eps,double yscal[], double *hdid, double *hnext,void (*derivs)(double, double [], double []));
void jacobn(double x, double y[], double dfdx[], double **dfdy, int n);
void derivs(double x, double y[], double dydx[]);
void simpr(double y[], double dydx[], double dfdx[], double **dfdy, int n,double xs, double htot, int nstep, double yout[],void (*derivs)(double, double [], double []));
void stifbs(double y[], double dydx[], int nv, double *xx, double htry, double eps,double yscal[], double *hdid, double *hnext,void (*derivs)(double, double [], double []));

// NR Ch 17 Two point boundary value problems
void shoot(int n, double v[], double f[]);
void shootf(int n, double v[], double f[]);
void solvde(int itmax, double conv, double slowc, double scalv[], int indexv[],int ne, int nb, int m, double **y, double ***c, double **s);
void bksub(int ne, int nb, int jf, int k1, int k2, double ***c);
void pinvs(int ie1, int ie2, int je1, int jsf, int jc1, int k, double ***c,double **s);
void red(int iz1, int iz2, int jz1, int jz2, int jm1, int jm2, int jmf,int ic1, int jc1, int jcf, int kc, double ***c, double **s);

// NR Ch 18 Integral equations and inverse theory
void fred2(int n, double a, double b, double t[], double f[], double w[],double (*g)(double), double (*ak)(double, double));
double fredin(double x, int n, double a, double b, double t[], double f[],double w[], double (*g)(double), double (*ak)(double, double));
void voltra(int n, int m, double t0, double h, double *t, double **f,double (*g)(int, double), double (*ak)(int, int, double, double));

// NR Ch 19 Partial differential equations
void sor(double **a, double **b, double **c, double **d, double **e,
	double **f, double **u, int jmax, double rjac);
void mglin(double **u, int n, int ncycle);
void rstrct(double **uc, double **uf, int nc);
void interp(double **uf, double **uc, int nf);
void addint(double **uf, double **uc, double **res, int nf);
void slvsml(double **u, double **rhs);
void relax(double **u, double **rhs, int n);
void resid(double **res, double **u, double **rhs, int n);
void copy(double **aout, double **ain, int n);
void fill0(double **u, int n);
void mgfas(double **u, int n, int maxcyc);
void relax2(double **u, double **rhs, int n);
void slvsm2(double **u, double **rhs);
void lop(double **out, double **u, int n);
void matadd(double **a, double **b, double **c, int n);
void matsub(double **a, double **b, double **c, int n);
double anorm2(double **a, int n);

// NR Ch 20 Less numerical algorithms
void machar(int *ibeta, int *it, int *irnd, int *ngrd, int *machep, int *negep,int *iexp, int *minexp, int *maxexp, double *eps, double *epsneg,double *xmin, double *xmax);
unsigned long igray(unsigned long n, int is);
unsigned short icrc1(unsigned short crc, unsigned char onech);
unsigned short icrc(unsigned short crc, unsigned char *bufptr,unsigned long len, short jinit, int jrev);
int decchk(char string[], int n, char *ch);
void hufmak(unsigned long nfreq[], unsigned long nchin, unsigned long *ilong,unsigned long *nlong, huffcode *hcode);
void hufapp(unsigned long index[], unsigned long nprob[], unsigned long n,unsigned long i);
void hufenc(unsigned long ich, unsigned char **codep, unsigned long *lcode,unsigned long *nb, huffcode *hcode);
void hufdec(unsigned long *ich, unsigned char *code, unsigned long lcode,unsigned long *nb, huffcode *hcode);
void arcmak(unsigned long nfreq[], unsigned long nchh, unsigned long nradd,arithcode *acode);
void arcode(unsigned long *ich, unsigned char **codep, unsigned long *lcode,unsigned long *lcd, int isign, arithcode *acode);
void arcsum(unsigned long iin[], unsigned long iout[], unsigned long ja,int nwk, unsigned long nrad, unsigned long nc);
void mpadd(unsigned char w[], unsigned char u[], unsigned char v[], int n);
void mpsub(int *is, unsigned char w[], unsigned char u[], unsigned char v[],int n);
void mpsad(unsigned char w[], unsigned char u[], int n, int iv);
void mpsmu(unsigned char w[], unsigned char u[], int n, int iv);
void mpsdv(unsigned char w[], unsigned char u[], int n, int iv, int *ir);
void mpneg(unsigned char u[], int n);
void mpmov(unsigned char u[], unsigned char v[], int n);
void mplsh(unsigned char u[], int n);
void mpmul(unsigned char w[], unsigned char u[], unsigned char v[], int n,int m);
void mpinv(unsigned char u[], unsigned char v[], int n, int m);
void mpdiv(unsigned char q[], unsigned char r[], unsigned char u[],unsigned char v[], int n, int m);
void mpsqrt(unsigned char w[], unsigned char u[], unsigned char v[], int n,int m);
void mp2dfr(unsigned char a[], unsigned char s[], int n, int *m);
void mppi(int n);

} // namespace DP

} // namespace NR

#endif // MCS_NRDOUBLEPROTOTYPES_H