#ifndef MCS_NRFLOATPROTOTYPES_H
#define MCS_NRFLOATPROTOTYPES_H

#include <cstdio>

#include "MCS_NumericalRecipes.h"

namespace NR {

namespace SP {

// NR Ch 1 Preliminaries
long julday(int mm, int id, int iyyy);
void flmoon(int n, int nph, long *jd, double *frac);
void caldat(long julian, int *mm, int *id, int *iyyy);

// NR Ch 2 Solution of linear algebraic equations
void gaussj(float **a, int n, float **b, int m);
void ludcmp(float **a, int n, int *indx, float *d);
void lubksb(float **a, int n, int *indx, float b[]);
void tridag(float a[],float b[],float c[],float r[],float u[],unsigned long n);
void banmul(float **a, unsigned long n, int m1, int m2, float x[], float b[]);
void bandec(float **a, unsigned long n, int m1, int m2, float **al,unsigned long indx[], float *d);
void banbks(float **a, unsigned long n, int m1, int m2, float **al,unsigned long indx[], float b[]);
void mprove(float **a, float **alud, int n, int indx[], float b[], float x[]);
void svbksb(float **u, float w[], float **v, int m, int n, float b[], float x[]);
void svdcmp(float **a, int m, int n, float w[], float **v);
float pythag(float a, float b);
void cyclic(float a[], float b[], float c[], float alpha, float beta,float r[], float x[], unsigned long n);
void sprsin(float **a, int n, float thresh, unsigned long nmax, float sa[],unsigned long ija[]);
void sprsax(float sa[], unsigned long ija[], float x[], float b[],unsigned long n);
void sprstx(float sa[], unsigned long ija[], float x[], float b[],unsigned long n);
void sprstp(float sa[], unsigned long ija[], float sb[], unsigned long ijb[]);
void sprspm(float sa[], unsigned long ija[], float sb[], unsigned long ijb[],float sc[], unsigned long ijc[]);
void sprstm(float sa[], unsigned long ija[], float sb[], unsigned long ijb[],float thresh, unsigned long nmax, float sc[], unsigned long ijc[]);
void linbcg(unsigned long n, double b[], double x[], int itol, double tol,int itmax, int *iter, double *err);
double snrm(unsigned long n, double sx[], int itol);
void vander(double x[], double w[], double q[], int n);
void toeplz(float r[], float x[], float y[], int n);
void choldc(float **a, int n, float p[]);
void cholsl(float **a, int n, float p[], float b[], float x[]);
void qrdcmp(float **a, int n, float *c, float *d, int *sing);
void qrsolv(float **a, int n, float c[], float d[], float b[]);
void rsolv(float **a, int n, float d[], float b[]);
void qrupdt(float **r, float **qt, int n, float u[], float v[]);
void rotate(float **r, float **qt, int n, int i, float a, float b);

// NR Ch 3 Interpolation and extrapolation
void polint(float xa[], float ya[], int n, float x, float *y, float *dy);
void ratint(float xa[], float ya[], int n, float x, float *y, float *dy);
void spline(float x[], float y[], int n, float yp1, float ypn, float y2[]);
void splint(float xa[], float ya[], float y2a[], int n, float x, float *y);
void locate(float xx[], unsigned long n, float x, unsigned long *j);
void hunt(float xx[], unsigned long n, float x, unsigned long *jlo);
void polcoe(float x[], float y[], int n, float cof[]);
void polcof(float xa[], float ya[], int n, float cof[]);
void blnint(float y[], float x1l,float x1u, float x2l, float x2u, float x1, float x2, float *ansy);
void polin2(float x1a[], float x2a[], float **ya, int m, int n, float x1,float x2, float *y, float *dy);
void bcucof(float y[], float y1[], float y2[], float y12[], float d1, float d2, float **c);
void bcuint(float y[], float y1[], float y2[], float y12[], float x1l,float x1u, float x2l, float x2u, float x1, float x2, float *ansy,float *ansy1, float *ansy2);
void splie2(float x1a[], float x2a[], float **ya, int m, int n, float **y2a);
void splin2(float x1a[], float x2a[], float **ya, float **y2a, int m, int n,float x1, float x2, float *y);

// NR Ch 4 Integration of functions
float trapzd(float (*func)(float), float a, float b, int n);
float qtrap(float (*func)(float), float a, float b);
float qtrapr(float (*func)(float), float a, float b,float (*choose)(float(*)(float), float, float, int));
float qsimp(float (*func)(float), float a, float b);
float qsimpr(float (*func)(float), float a, float b,float (*choose)(float(*)(float), float, float, int));
float qromb(float (*func)(float), float a, float b);
float qrombr(float (*func)(float), float a, float b,float (*choose)(float(*)(float), float, float, int));
float midpnt(float (*func)(float), float a, float b, int n);
float qromo(float (*func)(float), float a, float b,float (*choose)(float(*)(float), float, float, int));
float midinf(float (*funk)(float), float aa, float bb, int n);
float midsql(float (*funk)(float), float aa, float bb, int n);
float midsqu(float (*funk)(float), float aa, float bb, int n);
float midexp(float (*funk)(float), float aa, float bb, int n);
float qgaus(float (*func)(float), float a, float b);
void gauleg(float x1, float x2, float x[], float w[], int n);
void gaulag(float x[], float w[], int n, float alf);
void gauher(float x[], float w[], int n);
void gaujac(float x[], float w[], int n, float alf, float bet);
void gaucof(int n, float a[], float b[], float amu0, float x[], float w[]);
void orthog(int n, float anu[], float alpha[], float beta[], float a[],float b[]);
float quad3d(float (*func)(float, float, float), float x1, float x2);
float f1(float x);
float f2(float y);
float f3(float z);

// NR Ch 5 Evaluation of functions
void eulsum(float *sum, float term, int jterm, float wksp[]);
void ddpoly(float c[], int nc, float x, float pd[], int nd);
void poldiv(float u[], int n, float v[], int nv, float q[], float r[]);
double ratval(double x, double cof[], int mm, int kk);
float dfridr(float (*func)(float), float x, float h, float *err);
void chebft(float a, float b, float c[], int n, float (*func)(float));
float chebev(float a, float b, float c[], int m, float x);
void chder(float a, float b, float c[], float cder[], int n);
void chint(float a, float b, float c[], float cint[], int n);
void chebpc(float c[], float d[], int n);
void pcshft(float a, float b, float d[], int n);
void pccheb(float d[], float c[], int n);
void pade(double cof[], int n, float *resid);
void ratlsq(double (*fn)(double), double a, double b, int mm, int kk,double cof[], double *dev);

// NR Ch 6 Special functions
float gammln(float xx);
float factrl(int n);
float bico(int n, int k);
float factln(int n);
float beta(float z, float w);
float gammp(float a, float x);
float gammq(float a, float x);
void gser(float *gamser, float a, float x, float *gln);
void gcf(float *gammcf, float a, float x, float *gln);
float erff(float x);
float erffc(float x);
float erfcc(float x);
float expint(int n, float x);
float ei(float x);
float betai(float a, float b, float x);
float betacf(float a, float b, float x);
float bessj0(float x);
float bessy0(float x);
float bessj1(float x);
float bessy1(float x);
float bessy(int n, float x);
float bessj(int n, float x);
float bessi0(float x);
float bessk0(float x);
float bessi1(float x);
float bessk1(float x);
float bessk(int n, float x);
float bessi(int n, float x);
void bessjy(float x, float xnu, float *rj, float *ry, float *rjp, float *ryp);
void beschb(double x, double *gam1, double *gam2, double *gampl, double *gammi);
void bessik(float x, float xnu, float *ri, float *rk, float *rip, float *rkp);
void airy(float x, float *ai, float *bi, float *aip, float *bip);
void sphbes(int n, float x, float *sj, float *sy, float *sjp, float *syp);
float plgndr(int l, int m, float x);
void frenel(float x, float *s, float *c);
void cisi(float x, float *ci, float *si);
float dawson(float x);
float rf(float x, float y, float z);
float rd(float x, float y, float z);
float rj(float x, float y, float z, float p);
float rc(float x, float y);
float ellf(float phi, float ak);
float elle(float phi, float ak);
float ellpi(float phi, float en, float ak);
void sncndn(float uu, float emmc, float *sn, float *cn, float *dn);
fcomplex hypgeo(fcomplex a, fcomplex b, fcomplex c, fcomplex z);
void hypser(fcomplex a, fcomplex b, fcomplex c, fcomplex z, fcomplex *series,fcomplex *deriv);
void hypdrv(float s, float yy[], float dyyds[]);

// NR Ch 7 Random numbers
float ran0(long *idum);
float ran1(long *idum);
float ran2(long *idum);
float ran3(long *idum);
float expdev(long *idum);
float gasdev(long *idum);
float gamdev(int ia, long *idum);
float poidev(float xm, long *idum);
float bnldev(float pp, int n, long *idum);
int irbit1(unsigned long *iseed);
int irbit2(unsigned long *iseed);
void psdes(unsigned long *lword, unsigned long *irword);
float ran4(long *idum);
void sobseq(int *n, float x[]);
void vegas(float regn[], int ndim, float (*fxn)(float [], float), int init,unsigned long ncall, int itmx, int nprn, float *tgral, float *sd,float *chi2a);
void rebin(float rc, int nd, float r[], float xin[], float xi[]);
void miser(float (*func)(float []), float regn[], int ndim, unsigned long npts,float dith, float *ave, float *var);
void ranpt(float pt[], float regn[], int n);

// NR Ch 8 Sorting
void piksrt(int n, float arr[]);
void piksr2(int n, float arr[], float brr[]);
void shell(unsigned long n, float a[]);
void sort(unsigned long n, float arr[]);
void sort2(unsigned long n, float arr[], float brr[]);
void hpsort(unsigned long n, float ra[]);
void indexx(unsigned long n, float arr[], unsigned long indx[]);
void iindexx(unsigned long n, long arr[], unsigned long indx[]);
void sort3(unsigned long n, float ra[], float rb[], float rc[]);
void rank(unsigned long n, unsigned long indx[], unsigned long irank[]);
float select(unsigned long k, unsigned long n, float arr[]);
float selip(unsigned long k, unsigned long n, float arr[]);
void hpsel(unsigned long m, unsigned long n, float arr[], float heap[]);
void eclass(int nf[], int n, int lista[], int listb[], int m);
void eclazz(int nf[], int n, int (*equiv)(int, int));

// NR Ch 9 Root finding and nonlinear sets of equations
int zbrac(float (*func)(float), float *x1, float *x2);
void zbrak(float (*fx)(float), float x1, float x2, int n, float xb1[],float xb2[], int *nb);
float rtbis(float (*func)(float), float x1, float x2, float xacc);
float rtflsp(float (*func)(float), float x1, float x2, float xacc);
float rtsec(float (*func)(float), float x1, float x2, float xacc);
float zriddr(float (*func)(float), float x1, float x2, float xacc);
float zbrent(float (*func)(float), float x1, float x2, float tol);
float rtnewt(void (*funcd)(float, float *, float *), float x1, float x2,float xacc);
float rtsafe(void (*funcd)(float, float *, float *), float x1, float x2,float xacc);
void laguer(fcomplex a[], int m, fcomplex *x, int *its);
void zroots(fcomplex a[], int m, fcomplex roots[], int polish);
void zrhqr(float a[], int m, float rtr[], float rti[]);
void qroot(float p[], int n, float *b, float *c, float eps);
void mnewt(int ntrial, float x[], int n, float tolx, float tolf);
void lnsrch(int n, float xold[], float fold, float g[], float p[], float x[],float *f, float stpmax, int *check, float (*func)(float []));
void newt(float x[], int n, int *check,void (*vecfunc)(int, float [], float []));
void fdjac(int n, float x[], float fvec[], float **df,void (*vecfunc)(int, float [], float []));
float fmin(float x[]);
void broydn(float x[], int n, int *check,void (*vecfunc)(int, float [], float []));

// NR Ch 10 Minimization and maximization of functions
void mnbrak(float *ax, float *bx, float *cx, float *fa, float *fb, float *fc,float (*func)(float));
float golden(float ax, float bx, float cx, float (*f)(float), float tol,float *xmin);
float brent(float ax, float bx, float cx, float (*f)(float), float tol,float *xmin);
float dbrent(float ax, float bx, float cx, float (*f)(float),float (*df)(float), float tol, float *xmin);
void amoeba(float **p, float y[], int ndim, float ftol,float (*funk)(float []), int *nfunk);
float amotry(float **p, float y[], float psum[], int ndim,float (*funk)(float []), int ihi, float fac);
void powell(float p[], float **xi, int n, float ftol, int *iter, float *fret,float (*func)(float []));
void linmin(float p[], float xi[], int n, float *fret, float (*func)(float []));
float f1dim(float x);
void frprmn(float p[], int n, float ftol, int *iter, float *fret,float (*func)(float []), void (*dfunc)(float [], float []));
void dlinmin(float p[], float xi[], int n, float *fret, float (*func)(float []),void (*dfunc)(float [], float []));
float df1dim(float x);
void dfpmin(float p[], int n, float gtol, int *iter, float *fret,float(*func)(float []), void (*dfunc)(float [], float []));
void simplx(float **a, int m, int n, int m1, int m2, int m3, int *icase,int izrov[], int iposv[]);
void simp1(float **a, int mm, int ll[], int nll, int iabf, int *kp,float *bmax);
void simp2(float **a, int n, int l2[], int nl2, int *ip, int kp, float *q1);
void simp3(float **a, int i1, int k1, int ip, int kp);
void anneal(float x[], float y[], int iorder[], int ncity);
float revcst(float x[], float y[], int iorder[], int ncity, int n[]);
void reverse(int iorder[], int ncity, int n[]);
float trncst(float x[], float y[], int iorder[], int ncity, int n[]);
void trnspt(int iorder[], int ncity, int n[]);
int metrop(float de, float t);
void amebsa(float **p, float y[], int ndim, float pb[], float *yb, float ftol,float (*funk)(float []), int *iter, float temptr);
float amotsa(float **p, float y[], float psum[], int ndim, float pb[],float *yb, float (*funk)(float []), int ihi, float *yhi, float fac);

// NR Ch 11 Eigensystems
void jacobi(float **a, int n, float d[], float **v, int *nrot);
void eigsrt(float d[], float **v, int n);
void tred2(float **a, int n, float d[], float e[]);
void tqli(float d[], float e[], int n, float **z);
void balanc(float **a, int n);
void elmhes(float **a, int n);
void hqr(float **a, int n, float wr[], float wi[]);

// NR Ch 12 Fast Fourier transform
void four1(float data[], unsigned long nn, int isign);
void twofft(float data1[], float data2[], float fft1[], float fft2[],unsigned long n);
void realft(float data[], unsigned long n, int isign);
void sinft(float y[], int n);
void cosft1(float y[], int n);
void cosft2(float y[], int n, int isign);
void fourn(float data[], unsigned long nn[], int ndim, int isign);
void rlft3(float ***data, float **speq, unsigned long nn1, unsigned long nn2,unsigned long nn3, int isign);
void fourfs(std::FILE *file[5], unsigned long nn[], int ndim, int isign);
void fourew(std::FILE *file[5], int *na, int *nb, int *nc, int *nd);

// NR Ch 13 Fourier and spectral applications
void convlv(float data[], unsigned long n, float respns[], unsigned long m,int isign, float ans[]);
void correl(float data1[], float data2[], unsigned long n, float ans[]);
void spctrm(std::FILE *fp, float p[], int m, int k, int ovrlap);
void memcof(float data[], int n, int m, float *xms, float d[]);
void fixrts(float d[], int m);
void predic(float data[], int ndata, float d[], int m, float future[],int nfut);
float evlmem(float fdt, float d[], int m, float xms);
void period(float x[], float y[], int n, float ofac, float hifac, float px[],float py[], int np, int *nout, int *jmax, float *prob);
void fasper(float x[], float y[], unsigned long n, float ofac, float hifac,float wk1[], float wk2[], unsigned long nwk, unsigned long *nout,unsigned long *jmax, float *prob);
void spread(float y, float yy[], unsigned long n, float x, int m);
void dftcor(float w, float delta, float a, float b, float endpts[],float *corre, float *corim, float *corfac);
void dftint(float (*func)(float), float a, float b, float w, float *cosint,float *sinint);
void wt1(float a[], unsigned long n, int isign,void (*wtstep)(float [], unsigned long, int));
void daub4(float a[], unsigned long n, int isign);
void pwtset(int n);
void pwt(float a[], unsigned long n, int isign);
void wtn(float a[], unsigned long nn[], int ndim, int isign,void (*wtstep)(float [], unsigned long, int));

// NR 14 Statistical description of data
void moment(float data[], int n, float *ave, float *adev, float *sdev,float *var, float *skew, float *curt);
void ttest(float data1[], unsigned long n1, float data2[], unsigned long n2,float *t, float *prob);
void avevar(float data[], unsigned long n, float *ave, float *var);
void tutest(float data1[], unsigned long n1, float data2[], unsigned long n2,float *t, float *prob);
void tptest(float data1[], float data2[], unsigned long n, float *t,float *prob);
void ftest(float data1[], unsigned long n1, float data2[], unsigned long n2,float *f, float *prob);
void chsone(float bins[], float ebins[], int nbins, int knstrn, float *df,float *chsq, float *prob);
void chstwo(float bins1[], float bins2[], int nbins, int knstrn, float *df,float *chsq, float *prob);
void ksone(float data[], unsigned long n, float (*func)(float), float *d,float *prob);
void kstwo(float data1[], unsigned long n1, float data2[], unsigned long n2,float *d, float *prob);
float probks(float alam);
void cntab1(int **nn, int ni, int nj, float *chisq, float *df, float *prob,float *cramrv, float *ccc);
void cntab2(int **nn, int ni, int nj, float *h, float *hx, float *hy,float *hygx, float *hxgy, float *uygx, float *uxgy, float *uxy);
void pearsn(float x[], float y[], unsigned long n, float *r, float *prob,float *z);
void spear(float data1[], float data2[], unsigned long n, float *d, float *zd,float *probd, float *rs, float *probrs);
void crank(unsigned long n, float w[], float *s);
void kendl1(float data1[], float data2[], unsigned long n, float *tau,float *z, float *prob);
void kendl2(float **tab, int i, int j, float *tau, float *z, float *prob);
void ks2d1s(float x1[], float y1[], unsigned long n1,void (*quadvl)(float, float, float *, float *, float *, float *),float *d1, float *prob);
void quadct(float x, float y, float xx[], float yy[], unsigned long nn,float *fa, float *fb, float *fc, float *fd);
void quadvl(float x, float y, float *fa, float *fb, float *fc, float *fd);
void ks2d2s(float x1[], float y1[], unsigned long n1, float x2[], float y2[],unsigned long n2, float *d, float *prob);
void savgol(float c[], int np, int nl, int nr, int ld, int m);

// NR Ch 15 Modeling of data
void fit(float x[], float y[], int ndata, float sig[], int mwt, float *a,float *b, float *siga, float *sigb, float *chi2, float *q);
void fitexy(float x[], float y[], int ndat, float sigx[], float sigy[],float *a, float *b, float *siga, float *sigb, float *chi2, float *q);
float chixy(float bang);
void lfit(float x[], float y[], float sig[], int ndat, float a[], int ia[],int ma, float **covar, float *chisq, void (*funcs)(float, float [], int));
void covsrt(float **covar, int ma, int ia[], int mfit);
void svdfit(float x[], float y[], float sig[], int ndata, float a[], int ma,float **u, float **v, float w[], float *chisq,void (*funcs)(float, float [], int));
void svdfitm(float **x, float y[], float sig[], int ndata, float a[], int ma,float **u, float **v, float w[], float *chisq);
void svdvar(float **v, int ma, float w[], float **cvm);
void fpoly(float x, float p[], int np);
void fleg(float x, float pl[], int nl);
void mrqmin(float x[], float y[], float sig[], int ndata, float a[], int ia[],int ma, float **covar, float **alpha, float *chisq,void (*funcs)(float, float [], float *, float [], int), float *alamda);
void mrqcof(float x[], float y[], float sig[], int ndata, float a[], int ia[],int ma, float **alpha, float beta[], float *chisq,void (*funcs)(float, float [], float *, float [], int));
void fgauss(float x, float a[], float *y, float dyda[], int na);
void medfit(float x[], float y[], int ndata, float *a, float *b, float *abdev);
float rofunc(float b);

// NR Ch 16 Integration of ordinary differential equations
void rk4(float y[], float dydx[], int n, float x, float h, float yout[],void (*derivs)(float, float [], float []));
void rkdumb(float vstart[], int nvar, float x1, float x2, int nstep,void (*derivs)(float, float [], float []));
void rkqs(float y[], float dydx[], int n, float *x, float htry, float eps,float yscal[], float *hdid, float *hnext,void (*derivs)(float, float [], float []));
void rkck(float y[], float dydx[], int n, float x, float h, float yout[],float yerr[], void (*derivs)(float, float [], float []));
void odeint(float ystart[], int nvar, float x1, float x2, float eps, float h1,float hmin, int *nok, int *nbad,void (*derivs)(float, float [], float []),void (*rkqs)(float [], float [], int, float *, float, float, float [],float *, float *, void (*)(float, float [], float [])));
void mmid(float y[], float dydx[], int nvar, float xs, float htot, int nstep,float yout[], void (*derivs)(float, float[], float[]));
void bsstep(float y[], float dydx[], int nv, float *xx, float htry, float eps,float yscal[], float *hdid, float *hnext,void (*derivs)(float, float [], float []));
void pzextr(int iest, float xest, float yest[], float yz[], float dy[], int nv);
void rzextr(int iest, float xest, float yest[], float yz[], float dy[], int nv);
void stoerm(float y[], float d2y[], int nv, float xs, float htot, int nstep,float yout[], void (*derivs)(float, float [], float []));
void stiff(float y[], float dydx[], int n, float *x, float htry, float eps,float yscal[], float *hdid, float *hnext,void (*derivs)(float, float [], float []));
void jacobn(float x, float y[], float dfdx[], float **dfdy, int n);
void derivs(float x, float y[], float dydx[]);
void simpr(float y[], float dydx[], float dfdx[], float **dfdy, int n,float xs, float htot, int nstep, float yout[],void (*derivs)(float, float [], float []));
void stifbs(float y[], float dydx[], int nv, float *xx, float htry, float eps,float yscal[], float *hdid, float *hnext,void (*derivs)(float, float [], float []));

// NR Ch 17 Two point boundary value problems
void shoot(int n, float v[], float f[]);
void shootf(int n, float v[], float f[]);
void solvde(int itmax, float conv, float slowc, float scalv[], int indexv[],int ne, int nb, int m, float **y, float ***c, float **s);
void bksub(int ne, int nb, int jf, int k1, int k2, float ***c);
void pinvs(int ie1, int ie2, int je1, int jsf, int jc1, int k, float ***c,float **s);
void red(int iz1, int iz2, int jz1, int jz2, int jm1, int jm2, int jmf,int ic1, int jc1, int jcf, int kc, float ***c, float **s);

// NR Ch 18 Integral equations and inverse theory
void fred2(int n, float a, float b, float t[], float f[], float w[],float (*g)(float), float (*ak)(float, float));
float fredin(float x, int n, float a, float b, float t[], float f[],float w[], float (*g)(float), float (*ak)(float, float));
void voltra(int n, int m, float t0, float h, float *t, float **f,float (*g)(int, float), float (*ak)(int, int, float, float));

// NR Ch 19 Partial differential equations
void sor(double **a, double **b, double **c, double **d, double **e, double **f, double **u, int jmax, double rjac);
void mglin(double **u, int n, int ncycle);
void rstrct(double **uc, double **uf, int nc);
void interp(double **uf, double **uc, int nf);
void addint(double **uf, double **uc, double **res, int nf);
void slvsml(double **u, double **rhs);
void relax(double **u, double **rhs, int n);
void resid(double **res, double **u, double **rhs, int n);
void copy(double **aout, double **ain, int n);
void fill0(double **u, int n);
void mgfas(double **u, int n, int maxcyc);
void relax2(double **u, double **rhs, int n);
void slvsm2(double **u, double **rhs);
void lop(double **out, double **u, int n);
void matadd(double **a, double **b, double **c, int n);
void matsub(double **a, double **b, double **c, int n);
double anorm2(double **a, int n);

// NR Ch 20 Less numerical algorithms
void machar(int *ibeta, int *it, int *irnd, int *ngrd, int *machep, int *negep,int *iexp, int *minexp, int *maxexp, float *eps, float *epsneg,float *xmin, float *xmax);
unsigned long igray(unsigned long n, int is);
unsigned short icrc1(unsigned short crc, unsigned char onech);
unsigned short icrc(unsigned short crc, unsigned char *bufptr,unsigned long len, short jinit, int jrev);
int decchk(char string[], int n, char *ch);
void hufmak(unsigned long nfreq[], unsigned long nchin, unsigned long *ilong,unsigned long *nlong, huffcode *hcode);
void hufapp(unsigned long index[], unsigned long nprob[], unsigned long n,unsigned long i);
void hufenc(unsigned long ich, unsigned char **codep, unsigned long *lcode,unsigned long *nb, huffcode *hcode);
void hufdec(unsigned long *ich, unsigned char *code, unsigned long lcode,unsigned long *nb, huffcode *hcode);
void arcmak(unsigned long nfreq[], unsigned long nchh, unsigned long nradd,arithcode *acode);
void arcode(unsigned long *ich, unsigned char **codep, unsigned long *lcode,unsigned long *lcd, int isign, arithcode *acode);
void arcsum(unsigned long iin[], unsigned long iout[], unsigned long ja,int nwk, unsigned long nrad, unsigned long nc);
void mpadd(unsigned char w[], unsigned char u[], unsigned char v[], int n);
void mpsub(int *is, unsigned char w[], unsigned char u[], unsigned char v[],int n);
void mpsad(unsigned char w[], unsigned char u[], int n, int iv);
void mpsmu(unsigned char w[], unsigned char u[], int n, int iv);
void mpsdv(unsigned char w[], unsigned char u[], int n, int iv, int *ir);
void mpneg(unsigned char u[], int n);
void mpmov(unsigned char u[], unsigned char v[], int n);
void mplsh(unsigned char u[], int n);
void mpmul(unsigned char w[], unsigned char u[], unsigned char v[], int n,int m);
void mpinv(unsigned char u[], unsigned char v[], int n, int m);
void mpdiv(unsigned char q[], unsigned char r[], unsigned char u[],unsigned char v[], int n, int m);
void mpsqrt(unsigned char w[], unsigned char u[], unsigned char v[], int n,int m);
void mp2dfr(unsigned char a[], unsigned char s[], int n, int *m);
void mppi(int n);

} // namespace SP

} // namespace NR

#endif // MCS_NRFLOATPROTOTYPES_H