#ifndef MCS_NRWRAPPERSDOUBLE_H
#define MCS_NRWRAPPERSDOUBLE_H

#include "MCS_NRWrappers.h"

namespace NR {

// single precision specialization
template<> class FN<double>
{
	public:
		// NR Ch 2 Solution of linear algebraic equations
		typedef void (*_gaussj)(double**,int,double**,int);
		typedef void (*_ludcmp)(double**,int,int*,double*);
		typedef void (*_lubksb)(double**,int,int*,double*);
		typedef void (*_tridag)(double*,double*,double*,double*,double*,unsigned long);
		typedef void (*_banmul)(double**,unsigned long,int,int,double*,double*);
		typedef void (*_bandec)(double**,unsigned long,int,int,double**,unsigned long*,double*);
		typedef void (*_banbks)(double**,unsigned long,int,int,double**,unsigned long*,double*);
		typedef void (*_mprove)(double**,double**,int,int*,double*,double*);
		typedef void (*_svbksb)(double**,double*,double**,int,int,double*,double*);
		typedef void (*_svdcmp)(double**,int,int,double*,double**);
		typedef double (*_pythag)(double,double);
		typedef void (*_cyclic)(double*,double*,double*,double,double,double*,double*,unsigned long);
		typedef void (*_sprsin)(double**,int,double,unsigned long,double*,unsigned long*);
		typedef void (*_sprsax)(double*,unsigned long*,double*,double*,unsigned long);
		typedef void (*_sprstx)(double*,unsigned long*,double*,double*,unsigned long);
		typedef void (*_sprstp)(double*,unsigned long*,double*,unsigned long*);
		typedef void (*_sprspm)(double*,unsigned long*,double*,unsigned long*,double*,unsigned long*);
		typedef void (*_sprstm)(double*,unsigned long*,double*,unsigned long*,double,unsigned long,double*,unsigned long*);
		typedef void (*_linbcg)(unsigned long,double*,double*,int,double,int,int*,double*);
		typedef double (*_snrm)(unsigned long,double*,int);
		typedef void (*_vander)(double*,double*,double*,int);
		typedef void (*_toeplz)(double*,double*,double*,int);
		typedef void (*_choldc)(double**,int,double*);
		typedef void (*_cholsl)(double**,int,double*,double*,double*);
		typedef void (*_qrdcmp)(double**,int,double*,double*,int*);
		typedef void (*_qrsolv)(double**,int,double*,double*,double*);
		typedef void (*_rsolv)(double**,int,double*,double*);
		typedef void (*_qrupdt)(double**,double**,int,double*,double*);
		typedef void (*_rotate)(double**,double**,int,int,double,double);
		
		// NR Ch 3 Interpolation and extrapolation
		typedef void (*_polint)(double*,double*,int,double,double*,double*);
		typedef void (*_ratint)(double*,double*,int,double,double*,double*);
		typedef void (*_spline)(double*,double*,int,double,double,double*);
		typedef void (*_splint)(double*,double*,double*,int,double,double*);
		typedef void (*_locate)(double*,unsigned long,double,unsigned long*);
		typedef void (*_hunt)(double*,unsigned long,double,unsigned long*);
		typedef void (*_polcoe)(double*,double*,int,double*);
		typedef void (*_polcof)(double*,double*,int,double*);
		typedef void (*_blnint)(double*,double,double,double,double,double,double,double*);
		typedef void (*_polin2)(double*,double*,double**,int,int,double,double,double*,double*);
		typedef void (*_bcucof)(double*,double*,double*,double*,double,double,double**);
		typedef void (*_bcuint)(double*,double*,double*,double*,double,double,double,double,double,double,double*,double*,double*);
		typedef void (*_splie2)(double*,double*,double**,int,int,double**);
		typedef void (*_splin2)(double*,double*,double**,double**,int,int,double,double,double*);
		
		// NR Ch 4 Integration of functions
		typedef double (*_trapzd)(double (*)(double),double,double,int);
		typedef double (*_qtrap)(double (*)(double),double,double);
		typedef double (*_qtrapr)(double (*)(double),double,double,double (*)(double(*)(double),double,double,int));
		typedef double (*_qsimp)(double (*)(double),double,double);
		typedef double (*_qsimpr)(double (*)(double),double,double,double (*)(double(*)(double),double,double,int));
		typedef double (*_qromb)(double (*)(double),double,double);
		typedef double (*_qrombr)(double (*)(double),double,double,double (*)(double(*)(double),double,double,int));
		typedef double (*_midpnt)(double (*)(double),double,double,int);
		typedef double (*_qromo)(double (*)(double),double,double,double (*)(double(*)(double),double,double,int));
		typedef double (*_midinf)(double (*)(double),double,double,int);
		typedef double (*_midsql)(double (*)(double),double,double,int);
		typedef double (*_midsqu)(double (*)(double),double,double,int);
		typedef double (*_midexp)(double (*)(double),double,double,int);
		typedef double (*_qgaus)(double (*)(double),double,double);
		typedef void (*_gauleg)(double,double,double*,double*,int);
		typedef void (*_gaulag)(double*,double*,int,double);
		typedef void (*_gauher)(double*,double*,int);
		typedef void (*_gaujac)(double*,double*,int,double,double);
		typedef void (*_gaucof)(int,double*,double*,double,double*,double*);
		typedef void (*_orthog)(int,double*,double*,double*,double*,double*);
		typedef double (*_quad3d)(double (*)(double,double,double),double,double);
		typedef double (*_f1)(double);
		typedef double (*_f2)(double);
		typedef double (*_f3)(double);

		// NR Ch 5 Evaluation of functions
		typedef void (*_eulsum)(double*,double,int,double*);
		typedef void (*_ddpoly)(double*,int,double,double*,int nd);
		typedef void (*_poldiv)(double*,int,double*,int,double*,double*);
		typedef double (*_ratval)(double,double*,int,int);
		typedef double (*_dfridr)(double (*)(double),double,double,double*);
		typedef void (*_chebft)(double,double,double*,int,double (*)(double));
		typedef double (*_chebev)(double,double,double*,int,double);
		typedef void (*_chder)(double,double,double*,double*,int n);
		typedef void (*_chint)(double,double,double*,double*,int n);
		typedef void (*_chebpc)(double*,double*,int n);
		typedef void (*_pcshft)(double,double,double*,int n);
		typedef void (*_pccheb)(double*,double*,int n);
		typedef void (*_pade)(double*,int,double*);
		typedef void (*_ratlsq)(double (*)(double),double,double,int,int,double*,double *dev);

		// NR Ch 6 Special functions
		typedef double (*_gammln)(double xx);
		typedef double (*_factrl)(int n);
		typedef double (*_bico)(int,int);
		typedef double (*_factln)(int n);
		typedef double (*_beta)(double,double);
		typedef double (*_gammp)(double,double);
		typedef double (*_gammq)(double,double);
		typedef void (*_gser)(double*,double,double,double*);
		typedef void (*_gcf)(double*,double,double,double*);
		typedef double (*_erff)(double x);
		typedef double (*_erffc)(double x);
		typedef double (*_erfcc)(double x);
		typedef double (*_expint)(int,double);
		typedef double (*_ei)(double x);
		typedef double (*_betai)(double,double,double);
		typedef double (*_betacf)(double,double,double);
		typedef double (*_bessj0)(double x);
		typedef double (*_bessy0)(double x);
		typedef double (*_bessj1)(double x);
		typedef double (*_bessy1)(double x);
		typedef double (*_bessy)(int,double);
		typedef double (*_bessj)(int,double);
		typedef double (*_bessi0)(double x);
		typedef double (*_bessk0)(double x);
		typedef double (*_bessi1)(double x);
		typedef double (*_bessk1)(double x);
		typedef double (*_bessk)(int,double);
		typedef double (*_bessi)(int,double);
		typedef void (*_bessjy)(double,double,double*,double*,double*,double *ryp);
		typedef void (*_beschb)(double,double*,double*,double*,double *gammi);
		typedef void (*_bessik)(double,double,double*,double*,double*,double *rkp);
		typedef void (*_airy)(double,double*,double*,double*,double *bip);
		typedef void (*_sphbes)(int,double,double*,double*,double*,double *syp);
		typedef double (*_plgndr)(int,int,double);
		typedef void (*_frenel)(double,double*,double *c);
		typedef void (*_cisi)(double,double*,double *si);
		typedef double (*_dawson)(double x);
		typedef double (*_rf)(double,double,double);
		typedef double (*_rd)(double,double,double);
		typedef double (*_rj)(double,double,double,double);
		typedef double (*_rc)(double,double);
		typedef double (*_ellf)(double,double);
		typedef double (*_elle)(double,double);
		typedef double (*_ellpi)(double,double,double);
		typedef void (*_sncndn)(double,double,double*,double*,double *dn);
		typedef dcomplex (*_hypgeo)(dcomplex,dcomplex,dcomplex,dcomplex);
		typedef void (*_hypser)(dcomplex,dcomplex,dcomplex,dcomplex,dcomplex*,dcomplex *deriv);
		typedef void (*_hypdrv)(double,double*,double*);

		// NR Ch 7 Random numbers
		typedef double (*_ran0)(long *idum);
		typedef double (*_ran1)(long *idum);
		typedef double (*_ran2)(long *idum);
		typedef double (*_ran3)(long *idum);
		typedef double (*_expdev)(long *idum);
		typedef double (*_gasdev)(long *idum);
		typedef double (*_gamdev)(int,long*);
		typedef double (*_poidev)(double,long*);
		typedef double (*_bnldev)(double,int,long*);
		typedef int (*_irbit1)(unsigned long *iseed);
		typedef int (*_irbit2)(unsigned long *iseed);
		typedef void (*_psdes)(unsigned long*,unsigned long *irword);
		typedef double (*_ran4)(long *idum);
		typedef void (*_sobseq)(int*,double*);
		typedef void (*_vegas)(double*,int,double (*)(double*,double),int,unsigned long,int,int,double*,double*,double *chi2a);
		typedef void (*_rebin)(double,int,double*,double*,double*);
		typedef void (*_miser)(double (*)(double*),double*,int,unsigned long,double,double*,double *var);
		typedef void (*_ranpt)(double*,double*,int n);

		// NR Ch 8 Sorting
		typedef void (*_piksrt)(int,double*);
		typedef void (*_piksr2)(int,double*,double*);
		typedef void (*_shell)(unsigned long,double*);
		typedef void (*_sort)(unsigned long,double*);
		typedef void (*_sort2)(unsigned long,double*,double*);
		typedef void (*_hpsort)(unsigned long,double*);
		typedef void (*_indexx)(unsigned long,double*,unsigned long*);
		typedef void (*_iindexx)(unsigned long,long*,unsigned long*);
		typedef void (*_sort3)(unsigned long,double*,double*,double*);
		typedef void (*_rank)(unsigned long,unsigned long*,unsigned long*);
		typedef double (*_select)(unsigned long,unsigned long,double*);
		typedef double (*_selip)(unsigned long,unsigned long,double*);
		typedef void (*_hpsel)(unsigned long,unsigned long,double*,double*);
		typedef void (*_eclass)(int*,int,int*,int*,int m);
		typedef void (*_eclazz)(int*,int,int (*)(int,int));

		// NR Ch 9 Root finding and nonlinear sets of equations
		typedef int (*_zbrac)(double (*)(double),double*,double *x2);
		typedef void (*_zbrak)(double (*)(double),double,double,int,double*,double*,int *nb);
		typedef double (*_rtbis)(double (*)(double),double,double,double);
		typedef double (*_rtflsp)(double (*)(double),double,double,double);
		typedef double (*_rtsec)(double (*)(double),double,double,double);
		typedef double (*_zriddr)(double (*)(double),double,double,double);
		typedef double (*_zbrent)(double (*)(double),double,double,double);
		typedef double (*_rtnewt)(void (*)(double,double *,double *),double,double,double xacc);
		typedef double (*_rtsafe)(void (*)(double,double *,double *),double,double,double xacc);
		typedef void (*_laguer)(dcomplex*,int,dcomplex*,int *its);
		typedef void (*_zroots)(dcomplex*,int,dcomplex*,int polish);
		typedef void (*_zrhqr)(double*,int,double*,double*);
		typedef void (*_qroot)(double*,int,double*,double*,double eps);
		typedef void (*_mnewt)(int,double*,int,double,double);
		typedef void (*_lnsrch)(int,double*,double,double*,double*,double*,double*,double,int*,double (*)(double*));
		typedef void (*_newt)(double*,int,int*,void (*)(int,double*,double*));
		typedef void (*_fdjac)(int,double*,double*,double**,void (*)(int,double*,double*));
		typedef double (*_fmin)(double*);
		typedef void (*_broydn)(double*,int,int*,void (*)(int,double*,double*));

		// NR Ch 10 Minimization and maximization of functions
		typedef void (*_mnbrak)(double*,double*,double*,double*,double*,double*,double (*)(double));
		typedef double (*_golden)(double,double,double,double (*)(double),double,double *xmin);
		typedef double (*_brent)(double,double,double,double (*)(double),double,double *xmin);
		typedef double (*_dbrent)(double,double,double,double (*)(double),double (*)(double),double,double*);
		typedef void (*_amoeba)(double**,double*,int,double,double (*)(double*),int*);
		typedef double (*_amotry)(double**,double*,double*,int,double (*)(double*),int,double);
		typedef void (*_powell)(double*,double**,int,double,int*,double*,double (*)(double*));
		typedef void (*_linmin)(double*,double*,int,double*,double (*)(double*));
		typedef double (*_f1dim)(double x);
		typedef void (*_frprmn)(double*,int,double,int*,double*,double (*)(double*),void (*)(double*,double*));
		typedef void (*_dlinmin)(double*,double*,int,double*,double (*)(double*),void (*)(double*,double*));
		typedef double (*_df1dim)(double x);
		typedef void (*_dfpmin)(double*,int,double,int*,double*,double(*func)(double*),void (*)(double*,double*));
		typedef void (*_simplx)(double**,int,int,int,int,int,int*,int*,int*);
		typedef void (*_simp1)(double**,int,int*,int,int,int*,double *bmax);
		typedef void (*_simp2)(double**,int,int*,int,int*,int,double*);
		typedef void (*_simp3)(double**,int,int,int,int);
		typedef void (*_anneal)(double*,double*,int*,int ncity);
		typedef double (*_revcst)(double*,double*,int*,int,int*);
		typedef void (*_reverse)(int*,int,int*);
		typedef double (*_trncst)(double*,double*,int*,int,int*);
		typedef void (*_trnspt)(int*,int,int*);
		typedef int (*_metrop)(double,double);
		typedef void (*_amebsa)(double**,double*,int,double*,double*,double,double (*)(double*),int*,double temptr);
		typedef double (*_amotsa)(double**,double*,double*,int,double*,double*,double (*)(double*),int,double*,double fac);

		// NR Ch 11 Eigensystems
		typedef void (*_jacobi)(double**,int,double*,double**,int *nrot);
		typedef void (*_eigsrt)(double*,double**,int n);
		typedef void (*_tred2)(double**,int,double*,double*);
		typedef void (*_tqli)(double*,double*,int,double**);
		typedef void (*_balanc)(double**,int n);
		typedef void (*_elmhes)(double**,int n);
		typedef void (*_hqr)(double**,int,double*,double*);

		// NR Ch 12 Fast Fourier transform
		typedef void (*_four1)(double*,unsigned long,int);
		typedef void (*_twofft)(double*,double*,double*,double*,unsigned long n);
		typedef void (*_realft)(double*,unsigned long,int);
		typedef void (*_sinft)(double*,int n);
		typedef void (*_cosft1)(double*,int n);
		typedef void (*_cosft2)(double*,int,int);
		typedef void (*_fourn)(double*,unsigned long*,int,int);
		typedef void (*_rlft3)(double***,double**,unsigned long,unsigned long,unsigned long,int);
		typedef void (*_fourfs)(std::FILE *file[5],unsigned long*,int,int);
		typedef void (*_fourew)(std::FILE *file[5],int*,int*,int*,int *nd);

		// NR Ch 13 Fourier and spectral applications
		typedef void (*_convlv)(double*,unsigned long,double*,unsigned long,int,double*);
		typedef void (*_correl)(double*,double*,unsigned long,double*);
		typedef void (*_spctrm)(std::FILE *fp,double*,int,int,int);
		typedef void (*_memcof)(double*,int,int,double*,double*);
		typedef void (*_fixrts)(double*,int m);
		typedef void (*_predic)(double*,int,double*,int,double*,int nfut);
		typedef double (*_evlmem)(double,double*,int,double);
		typedef void (*_period)(double*,double*,int,double,double,double*,double*,int,int*,int*,double *prob);
		typedef void (*_fasper)(double*,double*,unsigned long,double,double,double*,double*,unsigned long,unsigned long*,unsigned long*,double *prob);
		typedef void (*_spread)(double,double*,unsigned long,double,int);
		typedef void (*_dftcor)(double,double,double,double,double*,double*,double*,double *corfac);
		typedef void (*_dftint)(double (*)(double),double,double,double,double*,double *sinint);
		typedef void (*_wt1)(double*,unsigned long,int,void (*)(double*,unsigned long,int));
		typedef void (*_daub4)(double*,unsigned long,int);
		typedef void (*_pwtset)(int n);
		typedef void (*_pwt)(double*,unsigned long,int);
		typedef void (*_wtn)(double*,unsigned long*,int,int,void (*)(double*,unsigned long,int));

		// NR 14 Statistical description of data
		typedef void (*_moment)(double*,int,double*,double*,double*,double*,double*,double *curt);
		typedef void (*_ttest)(double*,unsigned long,double*,unsigned long,double*,double *prob);
		typedef void (*_avevar)(double*,unsigned long,double*,double *var);
		typedef void (*_tutest)(double*,unsigned long,double*,unsigned long,double*,double *prob);
		typedef void (*_tptest)(double*,double*,unsigned long,double*,double *prob);
		typedef void (*_ftest)(double*,unsigned long,double*,unsigned long,double*,double *prob);
		typedef void (*_chsone)(double*,double*,int,int,double*,double*,double *prob);
		typedef void (*_chstwo)(double*,double*,int,int,double*,double*,double *prob);
		typedef void (*_ksone)(double*,unsigned long,double (*)(double),double*,double *prob);
		typedef void (*_kstwo)(double*,unsigned long,double*,unsigned long,double*,double *prob);
		typedef double (*_probks)(double alam);
		typedef void (*_cntab1)(int**,int,int,double*,double*,double*,double*,double *ccc);
		typedef void (*_cntab2)(int**,int,int,double*,double*,double*,double*,double*,double*,double*,double *uxy);
		typedef void (*_pearsn)(double*,double*,unsigned long,double*,double*,double *z);
		typedef void (*_spear)(double*,double*,unsigned long,double*,double*,double*,double*,double *probrs);
		typedef void (*_crank)(unsigned long,double*,double *s);
		typedef void (*_kendl1)(double*,double*,unsigned long,double*,double*,double *prob);
		typedef void (*_kendl2)(double**,int,int,double*,double*,double *prob);
		typedef void (*_ks2d1s)(double*,double*,unsigned long,void (*)(double,double,double *,double *,double *,double *),double*,double *prob);
		typedef void (*_quadct)(double,double,double*,double*,unsigned long,double*,double*,double*,double *fd);
		typedef void (*_quadvl)(double,double,double*,double*,double*,double *fd);
		typedef void (*_ks2d2s)(double*,double*,unsigned long,double*,double*,unsigned long,double*,double *prob);
		typedef void (*_savgol)(double*,int,int,int,int,int);

		// NR Ch 15 Modeling of data
		typedef void (*_fit)(double*,double*,int,double*,int,double*,double*,double*,double*,double*,double *q);
		typedef void (*_fitexy)(double*,double*,int,double*,double*,double*,double*,double*,double*,double*,double *q);
		typedef double (*_chixy)(double bang);
		typedef void (*_lfit)(double*,double*,double*,int,double*,int*,int,double**,double*,void (*)(double,double*,int));
		typedef void (*_covsrt)(double**,int,int*,int mfit);
		typedef void (*_svdfit)(double*,double*,double*,int,double*,int,double**,double**,double*,double*,void (*)(double,double*,int));
		typedef void (*_svdfitm)(double**,double*,double*,int,double*,int,double**,double**,double*,double*);
		typedef void (*_svdvar)(double**,int,double*,double**);
		typedef void (*_fpoly)(double,double*,int np);
		typedef void (*_fleg)(double,double*,int nl);
		typedef void (*_mrqmin)(double*,double*,double*,int,double*,int*,int,double**,double**,double*,void (*)(double,double*,double *,double*,int),double*);
		typedef void (*_mrqcof)(double*,double*,double*,int,double*,int*,int,double**,double*,double*,void (*)(double,double*,double *,double*,int));
		typedef void (*_fgauss)(double,double*,double*,double*,int na);
		typedef void (*_medfit)(double*,double*,int,double*,double*,double *abdev);
		typedef double (*_rofunc)(double b);

		// NR Ch 16 Integration of ordinary differential equations
		typedef void (*_rk4)(double*,double*,int,double,double,double*,void (*)(double,double*,double*));
		typedef void (*_rkdumb)(double*,int,double,double,int,void (*)(double,double*,double*));
		typedef void (*_rkqs)(double*,double*,int,double*,double,double,double*,double*,double*,void (*)(double,double*,double*));
		typedef void (*_rkck)(double*,double*,int,double,double,double*,double*,void (*)(double,double*,double*));
		typedef void (*_odeint)(double*,int,double,double,double,double,double,int*,int*,void (*)(double,double*,double*),void (*)(double*,double*,int,double *,double,double,double*,double *,double *,void (*)(double,double*,double*)));
		typedef void (*_mmid)(double*,double*,int,double,double,int,double*,void (*)(double,double[],double[]));
		typedef void (*_bsstep)(double*,double*,int,double*,double,double,double*,double*,double*,void (*)(double,double*,double*));
		typedef void (*_pzextr)(int,double,double*,double*,double*,int nv);
		typedef void (*_rzextr)(int,double,double*,double*,double*,int nv);
		typedef void (*_stoerm)(double*,double*,int,double,double,int,double*,void (*)(double,double*,double*));
		typedef void (*_stiff)(double*,double*,int,double*,double,double,double*,double*,double*,void (*)(double,double*,double*));
		typedef void (*_jacobn)(double,double*,double*,double**,int n);
		typedef void (*_derivs)(double,double*,double*);
		typedef void (*_simpr)(double*,double*,double*,double**,int,double,double,int,double*,void (*)(double,double*,double*));
		typedef void (*_stifbs)(double*,double*,int,double*,double,double,double*,double*,double*,void (*)(double,double*,double*));

		// NR Ch 17 Two point boundary value problems
		typedef void (*_shoot)(int,double*,double*);
		typedef void (*_shootf)(int,double*,double*);
		typedef void (*_solvde)(int,double,double,double*,int*,int,int,int,double**,double***,double**);
		typedef void (*_bksub)(int,int,int,int,int,double***);
		typedef void (*_pinvs)(int,int,int,int,int,int,double***,double**);
		typedef void (*_red)(int,int,int,int,int,int,int,int,int,int,int,double***,double**);

		// NR Ch 18 Integral equations and inverse theory
		typedef void (*_fred2)(int,double,double,double*,double*,double*,double (*)(double),double (*)(double,double));
		typedef double (*_fredin)(double,int,double,double,double*,double*,double*,double (*)(double),double (*)(double,double));
		typedef void (*_voltra)(int,int,double,double,double*,double**,double (*)(int,double),double (*)(int,int,double,double));

		// NR Ch 19 Partial differential equations
		typedef void (*_sor)(double**,double**,double**,double**,double**,double**,double**,int,double);
		typedef void (*_mglin)(double**,int,int);
		typedef void (*_rstrct)(double**,double**,int nc);
		typedef void (*_interp)(double**,double**,int nf);
		typedef void (*_addint)(double**,double**,double**,int nf);
		typedef void (*_slvsml)(double**,double**);
		typedef void (*_relax)(double**,double**,int n);
		typedef void (*_resid)(double**,double**,double**,int n);
		typedef void (*_copy)(double**,double**,int n);
		typedef void (*_fill0)(double**,int n);
		typedef void (*_mgfas)(double**,int,int);
		typedef void (*_relax2)(double**,double**,int n);
		typedef void (*_slvsm2)(double**,double**);
		typedef void (*_lop)(double**,double**,int n);
		typedef void (*_matadd)(double**,double**,double**,int n);
		typedef void (*_matsub)(double**,double**,double**,int n);
		typedef double (*_anorm2)(double**,int n);

		// NR Ch 20 Less numerical algorithms
		typedef void (*_machar)(int*,int*,int*,int*,int*,int*,int*,int*,int*,double*,double*,double*,double *xmax);
		typedef unsigned long (*_igray)(unsigned long,int);
		typedef unsigned short (*_icrc1)(unsigned short,unsigned char onech);
		typedef unsigned short (*_icrc)(unsigned short,unsigned char*,unsigned long,short,int);
		typedef int (*_decchk)(char*,int,char*);
		typedef void (*_hufmak)(unsigned long*,unsigned long,unsigned long*,unsigned long*,huffcode *hcode);
		typedef void (*_hufapp)(unsigned long*,unsigned long*,unsigned long,unsigned long i);
		typedef void (*_hufenc)(unsigned long,unsigned char**,unsigned long*,unsigned long*,huffcode *hcode);
		typedef void (*_hufdec)(unsigned long*,unsigned char*,unsigned long,unsigned long*,huffcode *hcode);
		typedef void (*_arcmak)(unsigned long*,unsigned long,unsigned long,arithcode *acode);
		typedef void (*_arcode)(unsigned long*,unsigned char**,unsigned long*,unsigned long*,int,arithcode*);
		typedef void (*_arcsum)(unsigned long*,unsigned long*,unsigned long,int,unsigned long,unsigned long nc);
		typedef void (*_mpadd)(unsigned char*,unsigned char*,unsigned char*,int n);
		typedef void (*_mpsub)(int*,unsigned char*,unsigned char*,unsigned char*,int n);
		typedef void (*_mpsad)(unsigned char*,unsigned char*,int,int);
		typedef void (*_mpsmu)(unsigned char*,unsigned char*,int,int);
		typedef void (*_mpsdv)(unsigned char*,unsigned char*,int,int,int*);
		typedef void (*_mpneg)(unsigned char*,int n);
		typedef void (*_mpmov)(unsigned char*,unsigned char*,int n);
		typedef void (*_mplsh)(unsigned char*,int n);
		typedef void (*_mpmul)(unsigned char*,unsigned char*,unsigned char*,int,int m);
		typedef void (*_mpinv)(unsigned char*,unsigned char*,int,int);
		typedef void (*_mpdiv)(unsigned char*,unsigned char*,unsigned char*,unsigned char*,int,int);
		typedef void (*_mpsqrt)(unsigned char*,unsigned char*,unsigned char*,int,int m);
		typedef void (*_mp2dfr)(unsigned char*,unsigned char*,int,int*);
		typedef void (*_mppi)(int n);

		// static variable declarations
		// NR Ch 2 Solution of linear algebraic equations
		static const _gaussj gaussj;
		static const _ludcmp ludcmp;
		static const _lubksb lubksb;
		static const _tridag tridag;
		static const _banmul banmul;
		static const _bandec bandec;
		static const _banbks banbks;
		static const _mprove mprove;
		static const _svbksb svbksb;
		static const _svdcmp svdcmp;
		static const _pythag pythag;
		static const _cyclic cyclic;
		static const _sprsin sprsin;
		static const _sprsax sprsax;
		static const _sprstx sprstx;
		static const _sprstp sprstp;
		static const _sprspm sprspm;
		static const _sprstm sprstm;
		//static const _linbcg linbcg;
		static const _snrm snrm;
		static const _vander vander;
		static const _toeplz toeplz;
		static const _choldc choldc;
		static const _cholsl cholsl;
		static const _qrdcmp qrdcmp;
		static const _qrsolv qrsolv;
		static const _rsolv rsolv;
		static const _qrupdt qrupdt;
		static const _rotate rotate;
		
		// NR Ch 3 Interpolation and extrapolation
		static const _polint polint;
		static const _ratint ratint;
		static const _spline spline;
		static const _splint splint;
		static const _locate locate;
		static const _hunt hunt;
		static const _polcoe polcoe;
		static const _polcof polcof;
		static const _blnint blnint;
		static const _polin2 polin2;
		static const _bcucof bcucof;
		static const _bcuint bcuint;
		static const _splie2 splie2;
		static const _splin2 splin2;

		// NR Ch 4 Integration of functions
		static const _trapzd trapzd;
		static const _qtrap qtrap;
		static const _qtrapr qtrapr;
		static const _qsimp qsimp;
		static const _qsimpr qsimpr;
		static const _qromb qromb;
		static const _qrombr qrombr;
		static const _midpnt midpnt;
		static const _qromo qromo;
		static const _midinf midinf;
		static const _midsql midsql;
		static const _midsqu midsqu;
		static const _midexp midexp;
		static const _qgaus qgaus;
		static const _gauleg gauleg;
		static const _gaulag gaulag;
		static const _gauher gauher;
		static const _gaujac gaujac;
		static const _gaucof gaucof;
		static const _orthog orthog;
		static const _quad3d quad3d;
		static const _f1 f1;
		static const _f2 f2;
		static const _f3 f3;
		
		// NR Ch 5 Evaluation of functions
		static const _eulsum eulsum;
		static const _ddpoly ddpoly;
		static const _poldiv poldiv;
		static const _ratval ratval;
		static const _dfridr dfridr;
		static const _chebft chebft;
		static const _chebev chebev;
		static const _chder chder;
		static const _chint chint;
		static const _chebpc chebpc;
		static const _pcshft pcshft;
		static const _pccheb pccheb;
		static const _pade pade;
		static const _ratlsq ratlsq;

		// NR Ch 6 Special functions
		static const _gammln gammln;
		static const _factrl factrl;
		static const _bico bico;
		static const _factln factln;
		static const _beta beta;
		static const _gammp gammp;
		static const _gammq gammq;
		static const _gser gser;
		static const _gcf gcf;
		static const _erff erff;
		static const _erffc erffc;
		static const _erfcc erfcc;
		static const _expint expint;
		static const _ei ei;
		static const _betai betai;
		static const _betacf betacf;
		static const _bessj0 bessj0;
		static const _bessy0 bessy0;
		static const _bessj1 bessj1;
		static const _bessy1 bessy1;
		static const _bessy bessy;
		static const _bessj bessj;
		static const _bessi0 bessi0;
		static const _bessk0 bessk0;
		static const _bessi1 bessi1;
		static const _bessk1 bessk1;
		static const _bessk bessk;
		static const _bessi bessi;
		static const _bessjy bessjy;
		static const _beschb beschb;
		static const _bessik bessik;
		static const _airy airy;
		static const _sphbes sphbes;
		static const _plgndr plgndr;
		static const _frenel frenel;
		static const _cisi cisi;
		static const _dawson dawson;
		static const _rf rf;
		static const _rd rd;
		static const _rj rj;
		static const _rc rc;
		static const _ellf ellf;
		static const _elle elle;
		static const _ellpi ellpi;
		static const _sncndn sncndn;
		static const _hypgeo hypgeo;
		static const _hypser hypser;
		static const _hypdrv hypdrv;

		// NR Ch 7 Random numbers
		static const _ran0 ran0;
		static const _ran1 ran1;
		static const _ran2 ran2;
		static const _ran3 ran3;
		static const _expdev expdev;
		static const _gasdev gasdev;
		static const _gamdev gamdev;
		static const _poidev poidev;
		static const _bnldev bnldev;
		static const _irbit1 irbit1;
		static const _irbit2 irbit2;
		static const _psdes psdes;
		static const _ran4 ran4;
		static const _sobseq sobseq;
		static const _vegas vegas;
		static const _rebin rebin;
		static const _miser miser;
		static const _ranpt ranpt;

		// NR Ch 8 Sorting
		static const _piksrt piksrt;
		static const _piksr2 piksr2;
		static const _shell shell;
		static const _sort sort;
		static const _sort2 sort2;
		static const _hpsort hpsort;
		static const _indexx indexx;
		static const _iindexx iindexx;
		static const _sort3 sort3;
		static const _rank rank;
		static const _select select;
		static const _selip selip;
		static const _hpsel hpsel;
		static const _eclass eclass;
		static const _eclazz eclazz;

		// NR Ch 9 Root finding and nonlinear sets of equations
		static const _zbrac zbrac;
		static const _zbrak zbrak;
		static const _rtbis rtbis;
		static const _rtflsp rtflsp;
		static const _rtsec rtsec;
		static const _zriddr zriddr;
		static const _zbrent zbrent;
		static const _rtnewt rtnewt;
		static const _rtsafe rtsafe;
		static const _laguer laguer;
		static const _zroots zroots;
		static const _zrhqr zrhqr;
		static const _qroot qroot;
		static const _mnewt mnewt;
		static const _lnsrch lnsrch;
		static const _newt newt;
		static const _fdjac fdjac;
		static const _fmin fmin;
		static const _broydn broydn;

		// NR Ch 10 Minimization and maximization of functions
		static const _mnbrak mnbrak;
		static const _golden golden;
		static const _brent brent;
		static const _dbrent dbrent;
		static const _amoeba amoeba;
		static const _amotry amotry;
		static const _powell powell;
		static const _linmin linmin;
		static const _f1dim f1dim;
		static const _frprmn frprmn;
		static const _dlinmin dlinmin;
		static const _df1dim df1dim;
		static const _dfpmin dfpmin;
		static const _simplx simplx;
		static const _simp1 simp1;
		static const _simp2 simp2;
		static const _simp3 simp3;
		static const _anneal anneal;
		static const _revcst revcst;
		static const _reverse reverse;
		static const _trncst trncst;
		static const _trnspt trnspt;
		static const _metrop metrop;
		static const _amebsa amebsa;
		static const _amotsa amotsa;

		// NR Ch 11 Eigensystems
		static const _jacobi jacobi;
		static const _eigsrt eigsrt;
		static const _tred2 tred2;
		static const _tqli tqli;
		static const _balanc balanc;
		static const _elmhes elmhes;
		static const _hqr hqr;

		// NR Ch 12 Fast Fourier transform
		static const _four1 four1;
		static const _twofft twofft;
		static const _realft realft;
		static const _sinft sinft;
		static const _cosft1 cosft1;
		static const _cosft2 cosft2;
		static const _fourn fourn;
		static const _rlft3 rlft3;
		static const _fourfs fourfs;
		static const _fourew fourew;

		// NR Ch 13 Fourier and spectral applications
		static const _convlv convlv;
		static const _correl correl;
		static const _spctrm spctrm;
		static const _memcof memcof;
		static const _fixrts fixrts;
		static const _predic predic;
		static const _evlmem evlmem;
		static const _period period;
		static const _fasper fasper;
		static const _spread spread;
		static const _dftcor dftcor;
		static const _dftint dftint;
		static const _wt1 wt1;
		static const _daub4 daub4;
		static const _pwtset pwtset;
		static const _pwt pwt;
		static const _wtn wtn;

		// NR 14 Statistical description of data
		static const _moment moment;
		static const _ttest ttest;
		static const _avevar avevar;
		static const _tutest tutest;
		static const _tptest tptest;
		static const _ftest ftest;
		static const _chsone chsone;
		static const _chstwo chstwo;
		static const _ksone ksone;
		static const _kstwo kstwo;
		static const _probks probks;
		static const _cntab1 cntab1;
		static const _cntab2 cntab2;
		static const _pearsn pearsn;
		static const _spear spear;
		static const _crank crank;
		static const _kendl1 kendl1;
		static const _kendl2 kendl2;
		static const _ks2d1s ks2d1s;
		static const _quadct quadct;
		static const _quadvl quadvl;
		static const _ks2d2s ks2d2s;
		static const _savgol savgol;

		// NR Ch 15 Modeling of data
		static const _fit fit;
		static const _fitexy fitexy;
		static const _chixy chixy;
		static const _lfit lfit;
		static const _covsrt covsrt;
		static const _svdfit svdfit;
		static const _svdfitm svdfitm;
		static const _svdvar svdvar;
		static const _fpoly fpoly;
		static const _fleg fleg;
		static const _mrqmin mrqmin;
		static const _mrqcof mrqcof;
		static const _fgauss fgauss;
		static const _medfit medfit;
		static const _rofunc rofunc;

		// NR Ch 16 Integration of ordinary differential equations
		static const _rk4 rk4;
		static const _rkdumb rkdumb;
		static const _rkqs rkqs;
		static const _rkck rkck;
		static const _odeint odeint;
		static const _mmid mmid;
		static const _bsstep bsstep;
		static const _pzextr pzextr;
		static const _rzextr rzextr;
		static const _stoerm stoerm;
		static const _stiff stiff;
		static const _jacobn jacobn;
		static const _derivs derivs;
		static const _simpr simpr;
		static const _stifbs stifbs;

		// NR Ch 17 Two point boundary value problems
		static const _shoot shoot;
		static const _shootf shootf;
		static const _solvde solvde;
		static const _bksub bksub;
		static const _pinvs pinvs;
		static const _red red;

		// NR Ch 18 Integral equations and inverse theory
		static const _fred2 fred2;
		static const _fredin fredin;
		static const _voltra voltra;

		// NR Ch 19 Partial differential equations
		static const _sor sor;
		static const _mglin mglin;
		static const _rstrct rstrct;
		static const _interp interp;
		static const _addint addint;
		static const _slvsml slvsml;
		static const _relax relax;
		static const _resid resid;
		static const _copy copy;
		static const _fill0 fill0;
		static const _mgfas mgfas;
		static const _relax2 relax2;
		static const _slvsm2 slvsm2;
		static const _lop lop;
		static const _matadd matadd;
		static const _matsub matsub;
		static const _anorm2 anorm2;

		// NR Ch 20 Less numerical algorithms
		static const _machar machar;
		static const _igray igray;
		static const _icrc1 icrc1;
		static const _icrc icrc;
		static const _decchk decchk;
		static const _hufmak hufmak;
		static const _hufapp hufapp;
		static const _hufenc hufenc;
		static const _hufdec hufdec;
		static const _arcmak arcmak;
		static const _arcode arcode;
		static const _arcsum arcsum;
		static const _mpadd mpadd;
		static const _mpsub mpsub;
		static const _mpsad mpsad;
		static const _mpsmu mpsmu;
		static const _mpsdv mpsdv;
		static const _mpneg mpneg;
		static const _mpmov mpmov;
		static const _mplsh mplsh;
		static const _mpmul mpmul;
		static const _mpinv mpinv;
		static const _mpdiv mpdiv;
		static const _mpsqrt mpsqrt;
		static const _mp2dfr mp2dfr;
		static const _mppi mppi;

	private:
		FN();
};

} // namespace NR

#endif // MCS_NRWRAPPERSDOUBLE_H