#ifndef MCS_NRWRAPPERSFLOAT_H
#define MCS_NRWRAPPERSFLOAT_H

#include "MCS_NRWrappers.h"

namespace NR {

// single precision specialization
template<> class FN<float>
{
	public:
		// NR Ch 2 Solution of linear algebraic equations
		typedef void (*_gaussj)(float**,int,float**,int);
		typedef void (*_ludcmp)(float**,int,int*,float*);
		typedef void (*_lubksb)(float**,int,int*,float*);
		typedef void (*_tridag)(float*,float*,float*,float*,float*,unsigned long);
		typedef void (*_banmul)(float**,unsigned long,int,int,float*,float*);
		typedef void (*_bandec)(float**,unsigned long,int,int,float**,unsigned long*,float*);
		typedef void (*_banbks)(float**,unsigned long,int,int,float**,unsigned long*,float*);
		typedef void (*_mprove)(float**,float**,int,int*,float*,float*);
		typedef void (*_svbksb)(float**,float*,float**,int,int,float*,float*);
		typedef void (*_svdcmp)(float**,int,int,float*,float**);
		typedef float (*_pythag)(float,float);
		typedef void (*_cyclic)(float*,float*,float*,float,float,float*,float*,unsigned long);
		typedef void (*_sprsin)(float**,int,float,unsigned long,float*,unsigned long*);
		typedef void (*_sprsax)(float*,unsigned long*,float*,float*,unsigned long);
		typedef void (*_sprstx)(float*,unsigned long*,float*,float*,unsigned long);
		typedef void (*_sprstp)(float*,unsigned long*,float*,unsigned long*);
		typedef void (*_sprspm)(float*,unsigned long*,float*,unsigned long*,float*,unsigned long*);
		typedef void (*_sprstm)(float*,unsigned long*,float*,unsigned long*,float,unsigned long,float*,unsigned long*);
		typedef void (*_linbcg)(unsigned long,double*,double*,int,double,int,int*,double*);
		typedef double (*_snrm)(unsigned long,double*,int);
		typedef void (*_vander)(double*,double*,double*,int);
		typedef void (*_toeplz)(float*,float*,float*,int);
		typedef void (*_choldc)(float**,int,float*);
		typedef void (*_cholsl)(float**,int,float*,float*,float*);
		typedef void (*_qrdcmp)(float**,int,float*,float*,int*);
		typedef void (*_qrsolv)(float**,int,float*,float*,float*);
		typedef void (*_rsolv)(float**,int,float*,float*);
		typedef void (*_qrupdt)(float**,float**,int,float*,float*);
		typedef void (*_rotate)(float**,float**,int,int,float,float);
		
		// NR Ch 3 Interpolation and extrapolation
		typedef void (*_polint)(float*,float*,int,float,float*,float*);
		typedef void (*_ratint)(float*,float*,int,float,float*,float*);
		typedef void (*_spline)(float*,float*,int,float,float,float*);
		typedef void (*_splint)(float*,float*,float*,int,float,float*);
		typedef void (*_locate)(float*,unsigned long,float,unsigned long*);
		typedef void (*_hunt)(float*,unsigned long,float,unsigned long*);
		typedef void (*_polcoe)(float*,float*,int,float*);
		typedef void (*_polcof)(float*,float*,int,float*);
		typedef void (*_blnint)(float*,float,float,float,float,float,float,float*);
		typedef void (*_polin2)(float*,float*,float**,int,int,float,float,float*,float*);
		typedef void (*_bcucof)(float*,float*,float*,float*,float,float,float**);
		typedef void (*_bcuint)(float*,float*,float*,float*,float,float,float,float,float,float,float*,float*,float*);
		typedef void (*_splie2)(float*,float*,float**,int,int,float**);
		typedef void (*_splin2)(float*,float*,float**,float**,int,int,float,float,float*);
		
		// NR Ch 4 Integration of functions
		typedef float (*_trapzd)(float (*)(float),float,float,int);
		typedef float (*_qtrap)(float (*)(float),float,float);
		typedef float (*_qtrapr)(float (*)(float),float,float,float (*)(float(*)(float),float,float,int));
		typedef float (*_qsimp)(float (*)(float),float,float);
		typedef float (*_qsimpr)(float (*)(float),float,float,float (*)(float(*)(float),float,float,int));
		typedef float (*_qromb)(float (*)(float),float,float);
		typedef float (*_qrombr)(float (*)(float),float,float,float (*)(float(*)(float),float,float,int));
		typedef float (*_midpnt)(float (*)(float),float,float,int);
		typedef float (*_qromo)(float (*)(float),float,float,float (*)(float(*)(float),float,float,int));
		typedef float (*_midinf)(float (*)(float),float,float,int);
		typedef float (*_midsql)(float (*)(float),float,float,int);
		typedef float (*_midsqu)(float (*)(float),float,float,int);
		typedef float (*_midexp)(float (*)(float),float,float,int);
		typedef float (*_qgaus)(float (*)(float),float,float);
		typedef void (*_gauleg)(float,float,float*,float*,int);
		typedef void (*_gaulag)(float*,float*,int,float);
		typedef void (*_gauher)(float*,float*,int);
		typedef void (*_gaujac)(float*,float*,int,float,float);
		typedef void (*_gaucof)(int,float*,float*,float,float*,float*);
		typedef void (*_orthog)(int,float*,float*,float*,float*,float*);
		typedef float (*_quad3d)(float (*)(float,float,float),float,float);
		typedef float (*_f1)(float);
		typedef float (*_f2)(float);
		typedef float (*_f3)(float);

		// NR Ch 5 Evaluation of functions
		typedef void (*_eulsum)(float*,float,int,float*);
		typedef void (*_ddpoly)(float*,int,float,float*,int nd);
		typedef void (*_poldiv)(float*,int,float*,int,float*,float*);
		typedef double (*_ratval)(double,double*,int,int);
		typedef float (*_dfridr)(float (*)(float),float,float,float*);
		typedef void (*_chebft)(float,float,float*,int,float (*)(float));
		typedef float (*_chebev)(float,float,float*,int,float);
		typedef void (*_chder)(float,float,float*,float*,int n);
		typedef void (*_chint)(float,float,float*,float*,int n);
		typedef void (*_chebpc)(float*,float*,int n);
		typedef void (*_pcshft)(float,float,float*,int n);
		typedef void (*_pccheb)(float*,float*,int n);
		typedef void (*_pade)(double*,int,float*);
		typedef void (*_ratlsq)(double (*)(double),double,double,int,int,double*,double *dev);

		// NR Ch 6 Special functions
		typedef float (*_gammln)(float xx);
		typedef float (*_factrl)(int n);
		typedef float (*_bico)(int,int);
		typedef float (*_factln)(int n);
		typedef float (*_beta)(float,float);
		typedef float (*_gammp)(float,float);
		typedef float (*_gammq)(float,float);
		typedef void (*_gser)(float*,float,float,float*);
		typedef void (*_gcf)(float*,float,float,float*);
		typedef float (*_erff)(float x);
		typedef float (*_erffc)(float x);
		typedef float (*_erfcc)(float x);
		typedef float (*_expint)(int,float);
		typedef float (*_ei)(float x);
		typedef float (*_betai)(float,float,float);
		typedef float (*_betacf)(float,float,float);
		typedef float (*_bessj0)(float x);
		typedef float (*_bessy0)(float x);
		typedef float (*_bessj1)(float x);
		typedef float (*_bessy1)(float x);
		typedef float (*_bessy)(int,float);
		typedef float (*_bessj)(int,float);
		typedef float (*_bessi0)(float x);
		typedef float (*_bessk0)(float x);
		typedef float (*_bessi1)(float x);
		typedef float (*_bessk1)(float x);
		typedef float (*_bessk)(int,float);
		typedef float (*_bessi)(int,float);
		typedef void (*_bessjy)(float,float,float*,float*,float*,float *ryp);
		typedef void (*_beschb)(double,double*,double*,double*,double *gammi);
		typedef void (*_bessik)(float,float,float*,float*,float*,float *rkp);
		typedef void (*_airy)(float,float*,float*,float*,float *bip);
		typedef void (*_sphbes)(int,float,float*,float*,float*,float *syp);
		typedef float (*_plgndr)(int,int,float);
		typedef void (*_frenel)(float,float*,float *c);
		typedef void (*_cisi)(float,float*,float *si);
		typedef float (*_dawson)(float x);
		typedef float (*_rf)(float,float,float);
		typedef float (*_rd)(float,float,float);
		typedef float (*_rj)(float,float,float,float);
		typedef float (*_rc)(float,float);
		typedef float (*_ellf)(float,float);
		typedef float (*_elle)(float,float);
		typedef float (*_ellpi)(float,float,float);
		typedef void (*_sncndn)(float,float,float*,float*,float *dn);
		typedef fcomplex (*_hypgeo)(fcomplex,fcomplex,fcomplex,fcomplex);
		typedef void (*_hypser)(fcomplex,fcomplex,fcomplex,fcomplex,fcomplex*,fcomplex *deriv);
		typedef void (*_hypdrv)(float,float*,float*);

		// NR Ch 7 Random numbers
		typedef float (*_ran0)(long *idum);
		typedef float (*_ran1)(long *idum);
		typedef float (*_ran2)(long *idum);
		typedef float (*_ran3)(long *idum);
		typedef float (*_expdev)(long *idum);
		typedef float (*_gasdev)(long *idum);
		typedef float (*_gamdev)(int,long*);
		typedef float (*_poidev)(float,long*);
		typedef float (*_bnldev)(float,int,long*);
		typedef int (*_irbit1)(unsigned long *iseed);
		typedef int (*_irbit2)(unsigned long *iseed);
		typedef void (*_psdes)(unsigned long*,unsigned long *irword);
		typedef float (*_ran4)(long *idum);
		typedef void (*_sobseq)(int*,float*);
		typedef void (*_vegas)(float*,int,float (*)(float*,float),int,unsigned long,int,int,float*,float*,float *chi2a);
		typedef void (*_rebin)(float,int,float*,float*,float*);
		typedef void (*_miser)(float (*)(float*),float*,int,unsigned long,float,float*,float *var);
		typedef void (*_ranpt)(float*,float*,int n);

		// NR Ch 8 Sorting
		typedef void (*_piksrt)(int,float*);
		typedef void (*_piksr2)(int,float*,float*);
		typedef void (*_shell)(unsigned long,float*);
		typedef void (*_sort)(unsigned long,float*);
		typedef void (*_sort2)(unsigned long,float*,float*);
		typedef void (*_hpsort)(unsigned long,float*);
		typedef void (*_indexx)(unsigned long,float*,unsigned long*);
		typedef void (*_iindexx)(unsigned long,long*,unsigned long*);
		typedef void (*_sort3)(unsigned long,float*,float*,float*);
		typedef void (*_rank)(unsigned long,unsigned long*,unsigned long*);
		typedef float (*_select)(unsigned long,unsigned long,float*);
		typedef float (*_selip)(unsigned long,unsigned long,float*);
		typedef void (*_hpsel)(unsigned long,unsigned long,float*,float*);
		typedef void (*_eclass)(int*,int,int*,int*,int m);
		typedef void (*_eclazz)(int*,int,int (*)(int,int));

		// NR Ch 9 Root finding and nonlinear sets of equations
		typedef int (*_zbrac)(float (*)(float),float*,float *x2);
		typedef void (*_zbrak)(float (*)(float),float,float,int,float*,float*,int *nb);
		typedef float (*_rtbis)(float (*)(float),float,float,float);
		typedef float (*_rtflsp)(float (*)(float),float,float,float);
		typedef float (*_rtsec)(float (*)(float),float,float,float);
		typedef float (*_zriddr)(float (*)(float),float,float,float);
		typedef float (*_zbrent)(float (*)(float),float,float,float);
		typedef float (*_rtnewt)(void (*)(float,float *,float *),float,float,float xacc);
		typedef float (*_rtsafe)(void (*)(float,float *,float *),float,float,float xacc);
		typedef void (*_laguer)(fcomplex*,int,fcomplex*,int *its);
		typedef void (*_zroots)(fcomplex*,int,fcomplex*,int polish);
		typedef void (*_zrhqr)(float*,int,float*,float*);
		typedef void (*_qroot)(float*,int,float*,float*,float eps);
		typedef void (*_mnewt)(int,float*,int,float,float);
		typedef void (*_lnsrch)(int,float*,float,float*,float*,float*,float*,float,int*,float (*)(float*));
		typedef void (*_newt)(float*,int,int*,void (*)(int,float*,float*));
		typedef void (*_fdjac)(int,float*,float*,float**,void (*)(int,float*,float*));
		typedef float (*_fmin)(float*);
		typedef void (*_broydn)(float*,int,int*,void (*)(int,float*,float*));

		// NR Ch 10 Minimization and maximization of functions
		typedef void (*_mnbrak)(float*,float*,float*,float*,float*,float*,float (*)(float));
		typedef float (*_golden)(float,float,float,float (*)(float),float,float *xmin);
		typedef float (*_brent)(float,float,float,float (*)(float),float,float *xmin);
		typedef float (*_dbrent)(float,float,float,float (*)(float),float (*)(float),float,float*);
		typedef void (*_amoeba)(float**,float*,int,float,float (*)(float*),int*);
		typedef float (*_amotry)(float**,float*,float*,int,float (*)(float*),int,float);
		typedef void (*_powell)(float*,float**,int,float,int*,float*,float (*)(float*));
		typedef void (*_linmin)(float*,float*,int,float*,float (*)(float*));
		typedef float (*_f1dim)(float x);
		typedef void (*_frprmn)(float*,int,float,int*,float*,float (*)(float*),void (*)(float*,float*));
		typedef void (*_dlinmin)(float*,float*,int,float*,float (*)(float*),void (*)(float*,float*));
		typedef float (*_df1dim)(float x);
		typedef void (*_dfpmin)(float*,int,float,int*,float*,float(*func)(float*),void (*)(float*,float*));
		typedef void (*_simplx)(float**,int,int,int,int,int,int*,int*,int*);
		typedef void (*_simp1)(float**,int,int*,int,int,int*,float *bmax);
		typedef void (*_simp2)(float**,int,int*,int,int*,int,float*);
		typedef void (*_simp3)(float**,int,int,int,int);
		typedef void (*_anneal)(float*,float*,int*,int ncity);
		typedef float (*_revcst)(float*,float*,int*,int,int*);
		typedef void (*_reverse)(int*,int,int*);
		typedef float (*_trncst)(float*,float*,int*,int,int*);
		typedef void (*_trnspt)(int*,int,int*);
		typedef int (*_metrop)(float,float);
		typedef void (*_amebsa)(float**,float*,int,float*,float*,float,float (*)(float*),int*,float temptr);
		typedef float (*_amotsa)(float**,float*,float*,int,float*,float*,float (*)(float*),int,float*,float fac);

		// NR Ch 11 Eigensystems
		typedef void (*_jacobi)(float**,int,float*,float**,int *nrot);
		typedef void (*_eigsrt)(float*,float**,int n);
		typedef void (*_tred2)(float**,int,float*,float*);
		typedef void (*_tqli)(float*,float*,int,float**);
		typedef void (*_balanc)(float**,int n);
		typedef void (*_elmhes)(float**,int n);
		typedef void (*_hqr)(float**,int,float*,float*);

		// NR Ch 12 Fast Fourier transform
		typedef void (*_four1)(float*,unsigned long,int);
		typedef void (*_twofft)(float*,float*,float*,float*,unsigned long n);
		typedef void (*_realft)(float*,unsigned long,int);
		typedef void (*_sinft)(float*,int n);
		typedef void (*_cosft1)(float*,int n);
		typedef void (*_cosft2)(float*,int,int);
		typedef void (*_fourn)(float*,unsigned long*,int,int);
		typedef void (*_rlft3)(float***,float**,unsigned long,unsigned long,unsigned long,int);
		typedef void (*_fourfs)(std::FILE *file[5],unsigned long*,int,int);
		typedef void (*_fourew)(std::FILE *file[5],int*,int*,int*,int *nd);

		// NR Ch 13 Fourier and spectral applications
		typedef void (*_convlv)(float*,unsigned long,float*,unsigned long,int,float*);
		typedef void (*_correl)(float*,float*,unsigned long,float*);
		typedef void (*_spctrm)(std::FILE *fp,float*,int,int,int);
		typedef void (*_memcof)(float*,int,int,float*,float*);
		typedef void (*_fixrts)(float*,int m);
		typedef void (*_predic)(float*,int,float*,int,float*,int nfut);
		typedef float (*_evlmem)(float,float*,int,float);
		typedef void (*_period)(float*,float*,int,float,float,float*,float*,int,int*,int*,float *prob);
		typedef void (*_fasper)(float*,float*,unsigned long,float,float,float*,float*,unsigned long,unsigned long*,unsigned long*,float *prob);
		typedef void (*_spread)(float,float*,unsigned long,float,int);
		typedef void (*_dftcor)(float,float,float,float,float*,float*,float*,float *corfac);
		typedef void (*_dftint)(float (*)(float),float,float,float,float*,float *sinint);
		typedef void (*_wt1)(float*,unsigned long,int,void (*)(float*,unsigned long,int));
		typedef void (*_daub4)(float*,unsigned long,int);
		typedef void (*_pwtset)(int n);
		typedef void (*_pwt)(float*,unsigned long,int);
		typedef void (*_wtn)(float*,unsigned long*,int,int,void (*)(float*,unsigned long,int));

		// NR 14 Statistical description of data
		typedef void (*_moment)(float*,int,float*,float*,float*,float*,float*,float *curt);
		typedef void (*_ttest)(float*,unsigned long,float*,unsigned long,float*,float *prob);
		typedef void (*_avevar)(float*,unsigned long,float*,float *var);
		typedef void (*_tutest)(float*,unsigned long,float*,unsigned long,float*,float *prob);
		typedef void (*_tptest)(float*,float*,unsigned long,float*,float *prob);
		typedef void (*_ftest)(float*,unsigned long,float*,unsigned long,float*,float *prob);
		typedef void (*_chsone)(float*,float*,int,int,float*,float*,float *prob);
		typedef void (*_chstwo)(float*,float*,int,int,float*,float*,float *prob);
		typedef void (*_ksone)(float*,unsigned long,float (*)(float),float*,float *prob);
		typedef void (*_kstwo)(float*,unsigned long,float*,unsigned long,float*,float *prob);
		typedef float (*_probks)(float alam);
		typedef void (*_cntab1)(int**,int,int,float*,float*,float*,float*,float *ccc);
		typedef void (*_cntab2)(int**,int,int,float*,float*,float*,float*,float*,float*,float*,float *uxy);
		typedef void (*_pearsn)(float*,float*,unsigned long,float*,float*,float *z);
		typedef void (*_spear)(float*,float*,unsigned long,float*,float*,float*,float*,float *probrs);
		typedef void (*_crank)(unsigned long,float*,float *s);
		typedef void (*_kendl1)(float*,float*,unsigned long,float*,float*,float *prob);
		typedef void (*_kendl2)(float**,int,int,float*,float*,float *prob);
		typedef void (*_ks2d1s)(float*,float*,unsigned long,void (*)(float,float,float *,float *,float *,float *),float*,float *prob);
		typedef void (*_quadct)(float,float,float*,float*,unsigned long,float*,float*,float*,float *fd);
		typedef void (*_quadvl)(float,float,float*,float*,float*,float *fd);
		typedef void (*_ks2d2s)(float*,float*,unsigned long,float*,float*,unsigned long,float*,float *prob);
		typedef void (*_savgol)(float*,int,int,int,int,int);

		// NR Ch 15 Modeling of data
		typedef void (*_fit)(float*,float*,int,float*,int,float*,float*,float*,float*,float*,float *q);
		typedef void (*_fitexy)(float*,float*,int,float*,float*,float*,float*,float*,float*,float*,float *q);
		typedef float (*_chixy)(float bang);
		typedef void (*_lfit)(float*,float*,float*,int,float*,int*,int,float**,float*,void (*)(float,float*,int));
		typedef void (*_covsrt)(float**,int,int*,int mfit);
		typedef void (*_svdfit)(float*,float*,float*,int,float*,int,float**,float**,float*,float*,void (*)(float,float*,int));
		typedef void (*_svdfitm)(float**,float*,float*,int,float*,int,float**,float**,float*,float*);
		typedef void (*_svdvar)(float**,int,float*,float**);
		typedef void (*_fpoly)(float,float*,int np);
		typedef void (*_fleg)(float,float*,int nl);
		typedef void (*_mrqmin)(float*,float*,float*,int,float*,int*,int,float**,float**,float*,void (*)(float,float*,float *,float*,int),float*);
		typedef void (*_mrqcof)(float*,float*,float*,int,float*,int*,int,float**,float*,float*,void (*)(float,float*,float *,float*,int));
		typedef void (*_fgauss)(float,float*,float*,float*,int na);
		typedef void (*_medfit)(float*,float*,int,float*,float*,float *abdev);
		typedef float (*_rofunc)(float b);

		// NR Ch 16 Integration of ordinary differential equations
		typedef void (*_rk4)(float*,float*,int,float,float,float*,void (*)(float,float*,float*));
		typedef void (*_rkdumb)(float*,int,float,float,int,void (*)(float,float*,float*));
		typedef void (*_rkqs)(float*,float*,int,float*,float,float,float*,float*,float*,void (*)(float,float*,float*));
		typedef void (*_rkck)(float*,float*,int,float,float,float*,float*,void (*)(float,float*,float*));
		typedef void (*_odeint)(float*,int,float,float,float,float,float,int*,int*,void (*)(float,float*,float*),void (*)(float*,float*,int,float *,float,float,float*,float *,float *,void (*)(float,float*,float*)));
		typedef void (*_mmid)(float*,float*,int,float,float,int,float*,void (*)(float,float[],float[]));
		typedef void (*_bsstep)(float*,float*,int,float*,float,float,float*,float*,float*,void (*)(float,float*,float*));
		typedef void (*_pzextr)(int,float,float*,float*,float*,int nv);
		typedef void (*_rzextr)(int,float,float*,float*,float*,int nv);
		typedef void (*_stoerm)(float*,float*,int,float,float,int,float*,void (*)(float,float*,float*));
		typedef void (*_stiff)(float*,float*,int,float*,float,float,float*,float*,float*,void (*)(float,float*,float*));
		typedef void (*_jacobn)(float,float*,float*,float**,int n);
		typedef void (*_derivs)(float,float*,float*);
		typedef void (*_simpr)(float*,float*,float*,float**,int,float,float,int,float*,void (*)(float,float*,float*));
		typedef void (*_stifbs)(float*,float*,int,float*,float,float,float*,float*,float*,void (*)(float,float*,float*));

		// NR Ch 17 Two point boundary value problems
		typedef void (*_shoot)(int,float*,float*);
		typedef void (*_shootf)(int,float*,float*);
		typedef void (*_solvde)(int,float,float,float*,int*,int,int,int,float**,float***,float**);
		typedef void (*_bksub)(int,int,int,int,int,float***);
		typedef void (*_pinvs)(int,int,int,int,int,int,float***,float**);
		typedef void (*_red)(int,int,int,int,int,int,int,int,int,int,int,float***,float**);

		// NR Ch 18 Integral equations and inverse theory
		typedef void (*_fred2)(int,float,float,float*,float*,float*,float (*)(float),float (*)(float,float));
		typedef float (*_fredin)(float,int,float,float,float*,float*,float*,float (*)(float),float (*)(float,float));
		typedef void (*_voltra)(int,int,float,float,float*,float**,float (*)(int,float),float (*)(int,int,float,float));

		// NR Ch 19 Partial differential equations
		typedef void (*_sor)(double**,double**,double**,double**,double**,double**,double**,int,double);
		typedef void (*_mglin)(double**,int,int);
		typedef void (*_rstrct)(double**,double**,int nc);
		typedef void (*_interp)(double**,double**,int nf);
		typedef void (*_addint)(double**,double**,double**,int nf);
		typedef void (*_slvsml)(double**,double**);
		typedef void (*_relax)(double**,double**,int n);
		typedef void (*_resid)(double**,double**,double**,int n);
		typedef void (*_copy)(double**,double**,int n);
		typedef void (*_fill0)(double**,int n);
		typedef void (*_mgfas)(double**,int,int);
		typedef void (*_relax2)(double**,double**,int n);
		typedef void (*_slvsm2)(double**,double**);
		typedef void (*_lop)(double**,double**,int n);
		typedef void (*_matadd)(double**,double**,double**,int n);
		typedef void (*_matsub)(double**,double**,double**,int n);
		typedef double (*_anorm2)(double**,int n);

		// NR Ch 20 Less numerical algorithms
		typedef void (*_machar)(int*,int*,int*,int*,int*,int*,int*,int*,int*,float*,float*,float*,float *xmax);
		typedef unsigned long (*_igray)(unsigned long,int);
		typedef unsigned short (*_icrc1)(unsigned short,unsigned char onech);
		typedef unsigned short (*_icrc)(unsigned short,unsigned char*,unsigned long,short,int);
		typedef int (*_decchk)(char*,int,char*);
		typedef void (*_hufmak)(unsigned long*,unsigned long,unsigned long*,unsigned long*,huffcode *hcode);
		typedef void (*_hufapp)(unsigned long*,unsigned long*,unsigned long,unsigned long i);
		typedef void (*_hufenc)(unsigned long,unsigned char**,unsigned long*,unsigned long*,huffcode *hcode);
		typedef void (*_hufdec)(unsigned long*,unsigned char*,unsigned long,unsigned long*,huffcode *hcode);
		typedef void (*_arcmak)(unsigned long*,unsigned long,unsigned long,arithcode *acode);
		typedef void (*_arcode)(unsigned long*,unsigned char**,unsigned long*,unsigned long*,int,arithcode*);
		typedef void (*_arcsum)(unsigned long*,unsigned long*,unsigned long,int,unsigned long,unsigned long nc);
		typedef void (*_mpadd)(unsigned char*,unsigned char*,unsigned char*,int n);
		typedef void (*_mpsub)(int*,unsigned char*,unsigned char*,unsigned char*,int n);
		typedef void (*_mpsad)(unsigned char*,unsigned char*,int,int);
		typedef void (*_mpsmu)(unsigned char*,unsigned char*,int,int);
		typedef void (*_mpsdv)(unsigned char*,unsigned char*,int,int,int*);
		typedef void (*_mpneg)(unsigned char*,int n);
		typedef void (*_mpmov)(unsigned char*,unsigned char*,int n);
		typedef void (*_mplsh)(unsigned char*,int n);
		typedef void (*_mpmul)(unsigned char*,unsigned char*,unsigned char*,int,int m);
		typedef void (*_mpinv)(unsigned char*,unsigned char*,int,int);
		typedef void (*_mpdiv)(unsigned char*,unsigned char*,unsigned char*,unsigned char*,int,int);
		typedef void (*_mpsqrt)(unsigned char*,unsigned char*,unsigned char*,int,int m);
		typedef void (*_mp2dfr)(unsigned char*,unsigned char*,int,int*);
		typedef void (*_mppi)(int n);

		// static variable declarations
		// NR Ch 2 Solution of linear algebraic equations
		static const _gaussj gaussj;
		static const _ludcmp ludcmp;
		static const _lubksb lubksb;
		static const _tridag tridag;
		static const _banmul banmul;
		static const _bandec bandec;
		static const _banbks banbks;
		static const _mprove mprove;
		static const _svbksb svbksb;
		static const _svdcmp svdcmp;
		static const _pythag pythag;
		static const _cyclic cyclic;
		static const _sprsin sprsin;
		static const _sprsax sprsax;
		static const _sprstx sprstx;
		static const _sprstp sprstp;
		static const _sprspm sprspm;
		static const _sprstm sprstm;
		//static const _linbcg linbcg;
		static const _snrm snrm;
		static const _vander vander;
		static const _toeplz toeplz;
		static const _choldc choldc;
		static const _cholsl cholsl;
		static const _qrdcmp qrdcmp;
		static const _qrsolv qrsolv;
		static const _rsolv rsolv;
		static const _qrupdt qrupdt;
		static const _rotate rotate;
		
		// NR Ch 3 Interpolation and extrapolation
		static const _polint polint;
		static const _ratint ratint;
		static const _spline spline;
		static const _splint splint;
		static const _locate locate;
		static const _hunt hunt;
		static const _polcoe polcoe;
		static const _polcof polcof;
		static const _blnint blnint;
		static const _polin2 polin2;
		static const _bcucof bcucof;
		static const _bcuint bcuint;
		static const _splie2 splie2;
		static const _splin2 splin2;

		// NR Ch 4 Integration of functions
		static const _trapzd trapzd;
		static const _qtrap qtrap;
		static const _qtrapr qtrapr;
		static const _qsimp qsimp;
		static const _qsimpr qsimpr;
		static const _qromb qromb;
		static const _qrombr qrombr;
		static const _midpnt midpnt;
		static const _qromo qromo;
		static const _midinf midinf;
		static const _midsql midsql;
		static const _midsqu midsqu;
		static const _midexp midexp;
		static const _qgaus qgaus;
		static const _gauleg gauleg;
		static const _gaulag gaulag;
		static const _gauher gauher;
		static const _gaujac gaujac;
		static const _gaucof gaucof;
		static const _orthog orthog;
		static const _quad3d quad3d;
		static const _f1 f1;
		static const _f2 f2;
		static const _f3 f3;
		
		// NR Ch 5 Evaluation of functions
		static const _eulsum eulsum;
		static const _ddpoly ddpoly;
		static const _poldiv poldiv;
		static const _ratval ratval;
		static const _dfridr dfridr;
		static const _chebft chebft;
		static const _chebev chebev;
		static const _chder chder;
		static const _chint chint;
		static const _chebpc chebpc;
		static const _pcshft pcshft;
		static const _pccheb pccheb;
		static const _pade pade;
		static const _ratlsq ratlsq;

		// NR Ch 6 Special functions
		static const _gammln gammln;
		static const _factrl factrl;
		static const _bico bico;
		static const _factln factln;
		static const _beta beta;
		static const _gammp gammp;
		static const _gammq gammq;
		static const _gser gser;
		static const _gcf gcf;
		static const _erff erff;
		static const _erffc erffc;
		static const _erfcc erfcc;
		static const _expint expint;
		static const _ei ei;
		static const _betai betai;
		static const _betacf betacf;
		static const _bessj0 bessj0;
		static const _bessy0 bessy0;
		static const _bessj1 bessj1;
		static const _bessy1 bessy1;
		static const _bessy bessy;
		static const _bessj bessj;
		static const _bessi0 bessi0;
		static const _bessk0 bessk0;
		static const _bessi1 bessi1;
		static const _bessk1 bessk1;
		static const _bessk bessk;
		static const _bessi bessi;
		static const _bessjy bessjy;
		static const _beschb beschb;
		static const _bessik bessik;
		static const _airy airy;
		static const _sphbes sphbes;
		static const _plgndr plgndr;
		static const _frenel frenel;
		static const _cisi cisi;
		static const _dawson dawson;
		static const _rf rf;
		static const _rd rd;
		static const _rj rj;
		static const _rc rc;
		static const _ellf ellf;
		static const _elle elle;
		static const _ellpi ellpi;
		static const _sncndn sncndn;
		static const _hypgeo hypgeo;
		static const _hypser hypser;
		static const _hypdrv hypdrv;

		// NR Ch 7 Random numbers
		static const _ran0 ran0;
		static const _ran1 ran1;
		static const _ran2 ran2;
		static const _ran3 ran3;
		static const _expdev expdev;
		static const _gasdev gasdev;
		static const _gamdev gamdev;
		static const _poidev poidev;
		static const _bnldev bnldev;
		static const _irbit1 irbit1;
		static const _irbit2 irbit2;
		static const _psdes psdes;
		static const _ran4 ran4;
		static const _sobseq sobseq;
		static const _vegas vegas;
		static const _rebin rebin;
		static const _miser miser;
		static const _ranpt ranpt;

		// NR Ch 8 Sorting
		static const _piksrt piksrt;
		static const _piksr2 piksr2;
		static const _shell shell;
		static const _sort sort;
		static const _sort2 sort2;
		static const _hpsort hpsort;
		static const _indexx indexx;
		static const _iindexx iindexx;
		static const _sort3 sort3;
		static const _rank rank;
		static const _select select;
		static const _selip selip;
		static const _hpsel hpsel;
		static const _eclass eclass;
		static const _eclazz eclazz;

		// NR Ch 9 Root finding and nonlinear sets of equations
		static const _zbrac zbrac;
		static const _zbrak zbrak;
		static const _rtbis rtbis;
		static const _rtflsp rtflsp;
		static const _rtsec rtsec;
		static const _zriddr zriddr;
		static const _zbrent zbrent;
		static const _rtnewt rtnewt;
		static const _rtsafe rtsafe;
		static const _laguer laguer;
		static const _zroots zroots;
		static const _zrhqr zrhqr;
		static const _qroot qroot;
		static const _mnewt mnewt;
		static const _lnsrch lnsrch;
		static const _newt newt;
		static const _fdjac fdjac;
		static const _fmin fmin;
		static const _broydn broydn;

		// NR Ch 10 Minimization and maximization of functions
		static const _mnbrak mnbrak;
		static const _golden golden;
		static const _brent brent;
		static const _dbrent dbrent;
		static const _amoeba amoeba;
		static const _amotry amotry;
		static const _powell powell;
		static const _linmin linmin;
		static const _f1dim f1dim;
		static const _frprmn frprmn;
		static const _dlinmin dlinmin;
		static const _df1dim df1dim;
		static const _dfpmin dfpmin;
		static const _simplx simplx;
		static const _simp1 simp1;
		static const _simp2 simp2;
		static const _simp3 simp3;
		static const _anneal anneal;
		static const _revcst revcst;
		static const _reverse reverse;
		static const _trncst trncst;
		static const _trnspt trnspt;
		static const _metrop metrop;
		static const _amebsa amebsa;
		static const _amotsa amotsa;

		// NR Ch 11 Eigensystems
		static const _jacobi jacobi;
		static const _eigsrt eigsrt;
		static const _tred2 tred2;
		static const _tqli tqli;
		static const _balanc balanc;
		static const _elmhes elmhes;
		static const _hqr hqr;

		// NR Ch 12 Fast Fourier transform
		static const _four1 four1;
		static const _twofft twofft;
		static const _realft realft;
		static const _sinft sinft;
		static const _cosft1 cosft1;
		static const _cosft2 cosft2;
		static const _fourn fourn;
		static const _rlft3 rlft3;
		static const _fourfs fourfs;
		static const _fourew fourew;

		// NR Ch 13 Fourier and spectral applications
		static const _convlv convlv;
		static const _correl correl;
		static const _spctrm spctrm;
		static const _memcof memcof;
		static const _fixrts fixrts;
		static const _predic predic;
		static const _evlmem evlmem;
		static const _period period;
		static const _fasper fasper;
		static const _spread spread;
		static const _dftcor dftcor;
		static const _dftint dftint;
		static const _wt1 wt1;
		static const _daub4 daub4;
		static const _pwtset pwtset;
		static const _pwt pwt;
		static const _wtn wtn;

		// NR 14 Statistical description of data
		static const _moment moment;
		static const _ttest ttest;
		static const _avevar avevar;
		static const _tutest tutest;
		static const _tptest tptest;
		static const _ftest ftest;
		static const _chsone chsone;
		static const _chstwo chstwo;
		static const _ksone ksone;
		static const _kstwo kstwo;
		static const _probks probks;
		static const _cntab1 cntab1;
		static const _cntab2 cntab2;
		static const _pearsn pearsn;
		static const _spear spear;
		static const _crank crank;
		static const _kendl1 kendl1;
		static const _kendl2 kendl2;
		static const _ks2d1s ks2d1s;
		static const _quadct quadct;
		static const _quadvl quadvl;
		static const _ks2d2s ks2d2s;
		static const _savgol savgol;

		// NR Ch 15 Modeling of data
		static const _fit fit;
		static const _fitexy fitexy;
		static const _chixy chixy;
		static const _lfit lfit;
		static const _covsrt covsrt;
		static const _svdfit svdfit;
		static const _svdfitm svdfitm;
		static const _svdvar svdvar;
		static const _fpoly fpoly;
		static const _fleg fleg;
		static const _mrqmin mrqmin;
		static const _mrqcof mrqcof;
		static const _fgauss fgauss;
		static const _medfit medfit;
		static const _rofunc rofunc;

		// NR Ch 16 Integration of ordinary differential equations
		static const _rk4 rk4;
		static const _rkdumb rkdumb;
		static const _rkqs rkqs;
		static const _rkck rkck;
		static const _odeint odeint;
		static const _mmid mmid;
		static const _bsstep bsstep;
		static const _pzextr pzextr;
		static const _rzextr rzextr;
		static const _stoerm stoerm;
		static const _stiff stiff;
		static const _jacobn jacobn;
		static const _derivs derivs;
		static const _simpr simpr;
		static const _stifbs stifbs;

		// NR Ch 17 Two point boundary value problems
		static const _shoot shoot;
		static const _shootf shootf;
		static const _solvde solvde;
		static const _bksub bksub;
		static const _pinvs pinvs;
		static const _red red;

		// NR Ch 18 Integral equations and inverse theory
		static const _fred2 fred2;
		static const _fredin fredin;
		static const _voltra voltra;

		// NR Ch 19 Partial differential equations
		static const _sor sor;
		static const _mglin mglin;
		static const _rstrct rstrct;
		static const _interp interp;
		static const _addint addint;
		static const _slvsml slvsml;
		static const _relax relax;
		static const _resid resid;
		static const _copy copy;
		static const _fill0 fill0;
		static const _mgfas mgfas;
		static const _relax2 relax2;
		static const _slvsm2 slvsm2;
		static const _lop lop;
		static const _matadd matadd;
		static const _matsub matsub;
		static const _anorm2 anorm2;

		// NR Ch 20 Less numerical algorithms
		static const _machar machar;
		static const _igray igray;
		static const _icrc1 icrc1;
		static const _icrc icrc;
		static const _decchk decchk;
		static const _hufmak hufmak;
		static const _hufapp hufapp;
		static const _hufenc hufenc;
		static const _hufdec hufdec;
		static const _arcmak arcmak;
		static const _arcode arcode;
		static const _arcsum arcsum;
		static const _mpadd mpadd;
		static const _mpsub mpsub;
		static const _mpsad mpsad;
		static const _mpsmu mpsmu;
		static const _mpsdv mpsdv;
		static const _mpneg mpneg;
		static const _mpmov mpmov;
		static const _mplsh mplsh;
		static const _mpmul mpmul;
		static const _mpinv mpinv;
		static const _mpdiv mpdiv;
		static const _mpsqrt mpsqrt;
		static const _mp2dfr mp2dfr;
		static const _mppi mppi;

	private:
		FN();
};

} // namespace NR

#endif // MCS_NRWRAPPERSFLOAT_H