#ifndef MCS_NUMERICALRECIPES_H
#define MCS_NUMERICALRECIPES_H

#include <cmath>
#include <complex>

#include "MCS_NumericExceptions.h"

namespace NR {

class Exception : public mcs::num::NumericException { };

// NRUTIL.h
template<class T> inline T fabs(const T& val) 	{ return std::abs(val); }
template<class T> inline T sqr(const T& val)	{ return val*val; }

template<class T> inline void shft(T& a,T& b,const T& c)
{
	a = b;
	b = c;
}

template<class T> inline void shft(T& a,T& b,T& c,const T& d)
{
	a = b;
	b = c;
	c = d;
}

//#define LOBYTE(x) 		((unsigned char) ((x) & 0xff))
//#define HIBYTE(x) 		((unsigned char) ((x) >> 8 & 0xff))

#define SQR(a) 			(sqr(a))
#define DSQR(a) 		(sqr(a))
#define DMAX(a,b) 		(std::max(a,b))
#define DMIN(a,b) 		(std::min(a,b))
#define FMAX(a,b) 		(std::max(a,b))
#define FMIN(a,b) 		(std::min(a,b))
#define LMAX(a,b) 		(std::max(a,b))
#define LMIN(a,b) 		(std::min(a,b))
#define IMAX(a,b) 		(std::max(a,b))
#define IMIN(a,b) 		(std::min(a,b))

#define SIGN(a,b) 		((b) >= 0.0 ? fabs(a) : -fabs(a))

#define SHFT2(a,b,c)	(shft(a,b,c))
#define SHFT3(a,b,c,d)	(shft(a,b,c,d))
#define SWAP(a,b)		(std::swap(a,b))

// NR.h defines
struct fcomplex { float r,i; };
struct dcomplex { double r,i; };

struct wavefilt { int ncof,ioff,joff; float *cc,*cr; };
struct dwavefilt { int ncof,ioff,joff; double *cc,*cr; };

struct arithcode { unsigned long *ilob,*iupb,*ncumfq,jdif,nc,minint,nch,ncum,nrad; };
struct huffcode { unsigned long *icod,*ncod,*left,*right,nch,nodemax; };

fcomplex Cadd(fcomplex a, fcomplex b);
fcomplex Csub(fcomplex a, fcomplex b);
fcomplex Cmul(fcomplex a, fcomplex b);
fcomplex Complex(float re, float im);
fcomplex Conjg(fcomplex z);
fcomplex Cdiv(fcomplex a, fcomplex b);
float Cabs(fcomplex z);
fcomplex Csqrt(fcomplex z);
fcomplex RCmul(float x, fcomplex a);

dcomplex Cadd(dcomplex a, dcomplex b);
dcomplex Csub(dcomplex a, dcomplex b);
dcomplex Cmul(dcomplex a, dcomplex b);
dcomplex Complex(double re, double im);
dcomplex Conjg(dcomplex z);
dcomplex Cdiv(dcomplex a, dcomplex b);
double Cabs(dcomplex z);
dcomplex Csqrt(dcomplex z);
dcomplex RCmul(double x, dcomplex a);

// function prototypes
void nrerror(char error_text[]);
void nrmessage(char message_text[]);
void nrwarning(char warning_text[]);
float *vector(long nl, long nh);
int *ivector(long nl, long nh);
unsigned char *cvector(long nl, long nh);
unsigned long *lvector(long nl, long nh);
double *dvector(long nl, long nh);
float **matrix(long nrl, long nrh, long ncl, long nch);
double **dmatrix(long nrl, long nrh, long ncl, long nch);
int **imatrix(long nrl, long nrh, long ncl, long nch);
float **submatrix(float **a, long oldrl, long oldrh, long oldcl, long oldch,
	long newrl, long newcl);
float **convert_matrix(float *a, long nrl, long nrh, long ncl, long nch);
float ***f3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh);
void free_vector(float *v, long nl, long nh);
void free_ivector(int *v, long nl, long nh);
void free_cvector(unsigned char *v, long nl, long nh);
void free_lvector(unsigned long *v, long nl, long nh);
void free_dvector(double *v, long nl, long nh);
void free_matrix(float **m, long nrl, long nrh, long ncl, long nch);
void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch);
void free_imatrix(int **m, long nrl, long nrh, long ncl, long nch);
void free_submatrix(float **b, long nrl, long nrh, long ncl, long nch);
void free_convert_matrix(float **b, long nrl, long nrh, long ncl, long nch);
void free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch, long ndl, long ndh);

void arcmak(unsigned long nfreq[], unsigned long nchh, unsigned long nradd,arithcode *acode);
void arcode(unsigned long *ich, unsigned char **codep, unsigned long *lcode,unsigned long *lcd, int isign, arithcode *acode);
void arcsum(unsigned long iin[], unsigned long iout[], unsigned long ja,int nwk, unsigned long nrad, unsigned long nc);
void caldat(long julian, int *mm, int *id, int *iyyy);
void eclass(int nf[], int n, int lista[], int listb[], int m);
void eclazz(int nf[], int n, int (*equiv)(int, int));
void hufapp(unsigned long index[], unsigned long nprob[], unsigned long n,unsigned long i);
void hufdec(unsigned long *ich, unsigned char *code, unsigned long lcode,unsigned long *nb, huffcode *hcode);
void hufenc(unsigned long ich, unsigned char **codep, unsigned long *lcode,unsigned long *nb, huffcode *hcode);
void hufmak(unsigned long nfreq[], unsigned long nchin, unsigned long *ilong,unsigned long *nlong, huffcode *hcode);
unsigned short icrc(unsigned short crc, unsigned char *bufptr,unsigned long len, short jinit, int jrev);
unsigned short icrc1(unsigned short crc, unsigned char onech);
void iindexx(unsigned long n, long arr[], unsigned long indx[]);
unsigned long igray(unsigned long n, int is);
int irbit1(unsigned long *iseed);
int irbit2(unsigned long *iseed);
long julday(int mm, int id, int iyyy);
void rank(unsigned long n, unsigned long indx[], unsigned long irank[]);

} // namespace NR

#include "MCS_NRFloatPrototypes.h"
#include "MCS_NRDoublePrototypes.h"

#endif // MCS_NUMERICALRECIPES_H