#include "MCS_NumericalRecipes.h"
	
namespace NR {

using namespace std;

// NR 1.1 Program organization and control structures
#define IGREG (15+31L*(10+12L*1582))

long julday(int mm, int id, int iyyy)
{
	void nrerror(char error_text[]);
	long jul;
	int ja,jy=iyyy,jm;

	if (jy == 0) nrerror("julday: there is no year zero.");
	if (jy < 0) ++jy;
	if (mm > 2) {
		jm=mm+1;
	} else {
		--jy;
		jm=mm+13;
	}
	jul = (long) (floor(365.25*jy)+floor(30.6001*jm)+id+1720995);
	if (id+31L*(mm+12L*iyyy) >= IGREG) {
		ja=(int)(0.01*jy);
		jul += 2-ja+(int) (0.25*ja);
	}
	return jul;
}
#undef IGREG

#define RAD (3.14159265/180.0)

void flmoon(int n, int nph, long *jd, float *frac)
{
	void nrerror(char error_text[]);
	int i;
	float am,as,c,t,t2,xtra;

	c=n+nph/4.0;
	t=c/1236.85;
	t2=t*t;
	as=359.2242+29.105356*c;
	am=306.0253+385.816918*c+0.010730*t2;
	*jd=2415020+28L*n+7L*nph;
	xtra=0.75933+1.53058868*c+((1.178e-4)-(1.55e-7)*t)*t2;
	if (nph == 0 || nph == 2)
		xtra += (0.1734-3.93e-4*t)*sin(RAD*as)-0.4068*sin(RAD*am);
	else if (nph == 1 || nph == 3)
		xtra += (0.1721-4.0e-4*t)*sin(RAD*as)-0.6280*sin(RAD*am);
	else nrerror("nph is unknown in flmoon");
	i=(int)(xtra >= 0.0 ? floor(xtra) : ceil(xtra-1.0));
	*jd += i;
	*frac=xtra-i;
}
#undef RAD

#define IGREG 2299161

void caldat(long julian, int *mm, int *id, int *iyyy)
{
	long ja,jalpha,jb,jc,jd,je;

	if (julian >= IGREG) {
		jalpha=(long)(((float) (julian-1867216)-0.25)/36524.25);
		ja=julian+1+jalpha-(long) (0.25*jalpha);
	} else
		ja=julian;
	jb=ja+1524;
	jc=(long)(6680.0+((float) (jb-2439870)-122.1)/365.25);
	jd=(long)(365*jc+(0.25*jc));
	je=(long)((jb-jd)/30.6001);
	*id=jb-jd-(long) (30.6001*je);
	*mm=je-1;
	if (*mm > 12) *mm -= 12;
	*iyyy=jc-4715;
	if (*mm > 2) --(*iyyy);
	if (*iyyy <= 0) --(*iyyy);
}
#undef IGREG

} // namespace NR