#include "MCS_NumericalRecipes.h"
	
namespace NR {

namespace DP {

using namespace std;

// NR 18.1 Fredholm equations of the second kind
void fred2(int n, double a, double b, double t[], double f[], double w[],
	double (*g)(double), double (*ak)(double, double))
{
	int i,j,*indx;
	double d,**omk;

	indx=ivector(1,n);
	omk=dmatrix(1,n,1,n);
	gauleg(a,b,t,w,n);
	for (i=1;i<=n;i++) {
		for (j=1;j<=n;j++)
			omk[i][j]=(double)(i == j)-(*ak)(t[i],t[j])*w[j];
		f[i]=(*g)(t[i]);
	}
	ludcmp(omk,n,indx,&d);
	lubksb(omk,n,indx,f);
	free_dmatrix(omk,1,n,1,n);
	free_ivector(indx,1,n);
}

double fredin(double x, int n, double a, double b, double t[], double f[],
	double w[], double (*g)(double), double (*ak)(double, double))
{
	int i;
	double sum=0.0;

	for (i=1;i<=n;i++) sum += (*ak)(x,t[i])*w[i]*f[i];
	return (*g)(x)+sum;
}

// NR 18.2 Volterra equations
void voltra(int n, int m, double t0, double h, double *t, double **f,
	double (*g)(int, double), double (*ak)(int, int, double, double))
{
	int i,j,k,l,*indx;
	double d,sum,**a,*b;

	indx=ivector(1,m);
	a=dmatrix(1,m,1,m);
	b=dvector(1,m);
	t[1]=t0;
	for (k=1;k<=m;k++) f[k][1]=(*g)(k,t[1]);
	for (i=2;i<=n;i++) {
		t[i]=t[i-1]+h;
		for (k=1;k<=m;k++) {
			sum=(*g)(k,t[i]);
			for (l=1;l<=m;l++) {
				sum += 0.5*h*(*ak)(k,l,t[i],t[1])*f[l][1];
				for (j=2;j<i;j++)
					sum += h*(*ak)(k,l,t[i],t[j])*f[l][j];
				a[k][l]=(k == l)-0.5*h*(*ak)(k,l,t[i],t[i]);
			}
			b[k]=sum;
		}
		ludcmp(a,m,indx,&d);
		lubksb(a,m,indx,b);
		for (k=1;k<=m;k++) f[k][i]=b[k];
	}
	free_dvector(b,1,m);
	free_dmatrix(a,1,m,1,m);
	free_ivector(indx,1,m);
}

} // namespace DP

} // namespace NR