#include "MCS_NumericalRecipes.h"
	
namespace NR {

namespace DP {

using namespace std;

// NR 20.1 Diagnosing machine parameters
void machar(int *ibeta, int *it, int *irnd, int *ngrd, int *machep, int *negep,
	int *iexp, int *minexp, int *maxexp, double *eps, double *epsneg,
	double *xmin, double *xmax)
{
	#define CONV(i) ((double)(i))

	int i,itemp,iz,j,k,mx,nxres;
	double a,b,beta,betah,betain,one,t,temp,temp1,tempa,two,y,z,zero;

	one=CONV(1);
	two=one+one;
	zero=one-one;
	a=one;
	do {
		a += a;
		temp=a+one;
		temp1=temp-a;
	} while (temp1-one == zero);
	b=one;
	do {
		b += b;
		temp=a+b;
		itemp=(int)(temp-a);
	} while (itemp == 0);
	*ibeta=itemp;
	beta=CONV(*ibeta);
	*it=0;
	b=one;
	do {
		++(*it);
		b *= beta;
		temp=b+one;
		temp1=temp-b;
	} while (temp1-one == zero);
	*irnd=0;
	betah=beta/two;
	temp=a+betah;
	if (temp-a != zero) *irnd=1;
	tempa=a+beta;
	temp=tempa+betah;
	if (*irnd == 0 && temp-tempa != zero) *irnd=2;
	*negep=(*it)+3;
	betain=one/beta;
	a=one;
	for (i=1;i<=(*negep);i++) a *= betain;
	b=a;
	for (;;) {
		temp=one-a;
		if (temp-one != zero) break;
		a *= beta;
		--(*negep);
	}
	*negep = -(*negep);
	*epsneg=a;
	*machep = -(*it)-3;
	a=b;
	for (;;) {
		temp=one+a;
		if (temp-one != zero) break;
		a *= beta;
		++(*machep);
	}
	*eps=a;
	*ngrd=0;
	temp=one+(*eps);
	if (*irnd == 0 && temp*one-one != zero) *ngrd=1;
	i=0;
	k=1;
	z=betain;
	t=one+(*eps);
	nxres=0;
	for (;;) {
		y=z;
		z=y*y;
		a=z*one;
		temp=z*t;
		if (a+a == zero || fabs(z) >= y) break;
		temp1=temp*betain;
		if (temp1*beta == z) break;
		++i;
		k += k;
	}
	if (*ibeta != 10) {
		*iexp=i+1;
		mx=k+k;
	} else {
		*iexp=2;
		iz=(*ibeta);
		while (k >= iz) {
			iz *= *ibeta;
			++(*iexp);
		}
		mx=iz+iz-1;
	}
	for (;;) {
		*xmin=y;
		y *= betain;
		a=y*one;
		temp=y*t;
		if (a+a != zero && fabs(y) < *xmin) {
			++k;
			temp1=temp*betain;
			if (temp1*beta == y && temp != y) {
				nxres=3;
				*xmin=y;
				break;
			}
		}
		else break;
	}
	*minexp = -k;
	if (mx <= k+k-3 && *ibeta != 10) {
		mx += mx;
		++(*iexp);
	}
	*maxexp=mx+(*minexp);
	*irnd += nxres;
	if (*irnd >= 2) *maxexp -= 2;
	i=(*maxexp)+(*minexp);
	if (*ibeta == 2 && !i) --(*maxexp);
	if (i > 20) --(*maxexp);
	if (a != y) *maxexp -= 2;
	*xmax=one-(*epsneg);
	if ((*xmax)*one != *xmax) *xmax=one-beta*(*epsneg);
	*xmax /= (*xmin*beta*beta*beta);
	i=(*maxexp)+(*minexp)+3;
	for (j=1;j<=i;j++) {
		if (*ibeta == 2) *xmax += *xmax;
		else *xmax *= beta;
	}

	#undef CONV
}

// NR 20.2 Gray codes
unsigned long igray(unsigned long n, int is)
{
	int ish;
	unsigned long ans,idiv;

	if (is >= 0)
		return n ^ (n >> 1);
	ish=1;
	ans=n;
	for (;;) {
		ans ^= (idiv=ans >> ish);
		if (idiv <= 1 || ish == 16) return ans;
		ish <<= 1;
	}
}

// NR 20.3 Cyclic redundancy and other checksums
unsigned short icrc1(unsigned short crc, unsigned char onech)
{
	int i;
	unsigned short ans=(crc ^ onech << 8);

	for (i=0;i<8;i++) {
		if (ans & 0x8000)
			ans = (ans <<= 1) ^ 4129;
		else
			ans <<= 1;
	}
	return ans;
}

unsigned short icrc(unsigned short crc, unsigned char *bufptr,
	unsigned long len, short jinit, int jrev)
{
	static unsigned short icrctb[256],init=0;
	static unsigned char rchr[256];
	unsigned short j,cword=crc;
	static unsigned char it[16]={0,8,4,12,2,10,6,14,1,9,5,13,3,11,7,15};

	if (!init) {
		init=1;
		for (j=0;j<=255;j++) {
			icrctb[j]=icrc1(j << 8,(unsigned char)0);
			rchr[j]=(unsigned char)(it[j & 0xF] << 4 | it[j >> 4]);
		}
	}
	if (jinit >= 0) cword=((unsigned char) jinit) | (((unsigned char) jinit) << 8);
	else if (jrev < 0) cword=rchr[HIBYTE(cword)] | rchr[LOBYTE(cword)] << 8;
	for (j=1;j<=len;j++)
		cword=icrctb[(jrev < 0 ? rchr[bufptr[j]] :
			bufptr[j]) ^ HIBYTE(cword)] ^ LOBYTE(cword) << 8;
	return (jrev >= 0 ? cword : rchr[HIBYTE(cword)] | rchr[LOBYTE(cword)] << 8);
}

int decchk(char string[], int n, char *ch)
{
	char c;
	int j,k=0,m=0;
	static int ip[10][8]={0,1,5,8,9,4,2,7,1,5, 8,9,4,2,7,0,2,7,0,1,
		5,8,9,4,3,6,3,6,3,6, 3,6,4,2,7,0,1,5,8,9, 5,8,9,4,2,7,0,1,6,3,
		6,3,6,3,6,3,7,0,1,5, 8,9,4,2,8,9,4,2,7,0, 1,5,9,4,2,7,0,1,5,8};
	static int ij[10][10]={0,1,2,3,4,5,6,7,8,9, 1,2,3,4,0,6,7,8,9,5,
		2,3,4,0,1,7,8,9,5,6, 3,4,0,1,2,8,9,5,6,7, 4,0,1,2,3,9,5,6,7,8,
		5,9,8,7,6,0,4,3,2,1, 6,5,9,8,7,1,0,4,3,2, 7,6,5,9,8,2,1,0,4,3,
		8,7,6,5,9,3,2,1,0,4, 9,8,7,6,5,4,3,2,1,0};

	for (j=0;j<n;j++) {
		c=string[j];
		if (c >= 48 && c <= 57)
			k=ij[k][ip[(c+2) % 10][7 & m++]];
	}
	for (j=0;j<=9;j++)
		if (!ij[k][ip[j][m & 7]]) break;
	*ch=j+48;
	return k==0;
}

// NR 20.4 Huffman coding and compression of data
void hufmak(unsigned long nfreq[], unsigned long nchin, unsigned long *ilong,
	unsigned long *nlong, huffcode *hcode)
{
	int ibit;
	long node,*up;
	unsigned long j,k,*index,n,nused,*nprob;
	static unsigned long setbit[32]={0x1L,0x2L,0x4L,0x8L,0x10L,0x20L,
		0x40L,0x80L,0x100L,0x200L,0x400L,0x800L,0x1000L,0x2000L,
		0x4000L,0x8000L,0x10000L,0x20000L,0x40000L,0x80000L,0x100000L,
		0x200000L,0x400000L,0x800000L,0x1000000L,0x2000000L,0x4000000L,
		0x8000000L,0x10000000L,0x20000000L,0x40000000L,0x80000000L};

	hcode->nch=nchin;
	index=lvector(1,(long)(2*hcode->nch-1));
	up=(long *)lvector(1,(long)(2*hcode->nch-1));
	nprob=lvector(1,(long)(2*hcode->nch-1));
	for (nused=0,j=1;j<=hcode->nch;j++) {
		nprob[j]=nfreq[j];
		hcode->icod[j]=hcode->ncod[j]=0;
		if (nfreq[j]) index[++nused]=j;
	}
	for (j=nused;j>=1;j--) hufapp(index,nprob,nused,j);
	k=hcode->nch;
	while (nused > 1) {
		node=index[1];
		index[1]=index[nused--];
		hufapp(index,nprob,nused,1);
		nprob[++k]=nprob[index[1]]+nprob[node];
		hcode->left[k]=node;
		hcode->right[k]=index[1];
		up[index[1]] = -k;
		up[node]=index[1]=k;
		hufapp(index,nprob,nused,1);
	}
	up[hcode->nodemax=k]=0;
	for (j=1;j<=hcode->nch;j++) {
		if (nprob[j]) {
			for (n=0,ibit=0,node=up[j];node;node=up[node],ibit++) {
				if (node < 0) {
					n |= setbit[ibit];
					node = -node;
				}
			}
			hcode->icod[j]=n;
			hcode->ncod[j]=ibit;
		}
	}
	*nlong=0;
	for (j=1;j<=hcode->nch;j++) {
		if (hcode->ncod[j] > *nlong) {
			*nlong=hcode->ncod[j];
			*ilong=j-1;
		}
	}
	free_lvector(nprob,1,(long)(2*hcode->nch-1));
	free_lvector((unsigned long *)up,1,(long)(2*hcode->nch-1));
	free_lvector(index,1,(long)(2*hcode->nch-1));
}

void hufapp(unsigned long index[], unsigned long nprob[], unsigned long n,
	unsigned long i)
{
	unsigned long j,k;

	k=index[i];
	while (i <= (n>>1)) {
		if ((j = i << 1) < n && nprob[index[j]] > nprob[index[j+1]]) j++;
		if (nprob[k] <= nprob[index[j]]) break;
		index[i]=index[j];
		i=j;
	}
	index[i]=k;
}

void hufenc(unsigned long ich, unsigned char **codep, unsigned long *lcode,
	unsigned long *nb, huffcode *hcode)
{
	int l,n;
	unsigned long k,nc;
	static unsigned long setbit[32]={0x1L,0x2L,0x4L,0x8L,0x10L,0x20L,
		0x40L,0x80L,0x100L,0x200L,0x400L,0x800L,0x1000L,0x2000L,
		0x4000L,0x8000L,0x10000L,0x20000L,0x40000L,0x80000L,0x100000L,
		0x200000L,0x400000L,0x800000L,0x1000000L,0x2000000L,0x4000000L,
		0x8000000L,0x10000000L,0x20000000L,0x40000000L,0x80000000L};

	k=ich+1;
	if (k > hcode->nch || k < 1) nrerror("ich out of range in hufenc.");
	for (n=hcode->ncod[k]-1;n>=0;n--,++(*nb)) {
		nc=(*nb >> 3);
		if (++nc >= *lcode) {
			nrmessage("Reached the end of the 'code' array.\n");
			nrmessage("Attempting to expand its size.\n");
			*lcode *= 1.5;
			if ((*codep=(unsigned char *)realloc(*codep,
				(unsigned)(*lcode*sizeof(unsigned char)))) == NULL) {
				nrerror("Size expansion failed.");
			}
		}
		l=(*nb) & 7;
		if (!l) (*codep)[nc]=0;
		if (hcode->icod[k] & setbit[n]) (*codep)[nc] |= setbit[l];
	}
}

void hufdec(unsigned long *ich, unsigned char *code, unsigned long lcode,
	unsigned long *nb, huffcode *hcode)
{
	long nc,node;
	static unsigned char setbit[8]={0x1,0x2,0x4,0x8,0x10,0x20,0x40,0x80};

	node=hcode->nodemax;
	for (;;) {
		nc=(*nb >> 3);
		if (++nc > lcode) {
			*ich=hcode->nch;
			return;
		}
		node=(code[nc] & setbit[7 & (*nb)++] ?
			hcode->right[node] : hcode->left[node]);
		if (node <= hcode->nch) {
			*ich=node-1;
			return;
		}
	}
}

// NR 20.5 Arithmetic coding
void arcmak(unsigned long nfreq[], unsigned long nchh, unsigned long nradd,
	arithcode *acode)
{
	#define MC 512
	#ifdef ULONG_MAX
	#define MAXINT (ULONG_MAX >> 1)
	#else
	#define MAXINT 2147483647
	#endif

	unsigned long j;

	if (nchh > MC) nrerror("input radix may not exceed MC in arcmak.");
	if (nradd > 256) nrerror("output radix may not exceed 256 in arcmak.");

	acode->minint=MAXINT/nradd;
	acode->nch=nchh;
	acode->nrad=nradd;
	acode->ncumfq[1]=0;
	for (j=2;j<=acode->nch+1;j++)
		acode->ncumfq[j]=acode->ncumfq[j-1]+IMAX(nfreq[j-1],(unsigned long)(1));
	acode->ncum=acode->ncumfq[acode->nch+2]=acode->ncumfq[acode->nch+1]+1;

	#undef MC
	#undef MAXINT
}

void arcode(unsigned long *ich, unsigned char **codep, unsigned long *lcode,
	unsigned long *lcd, int isign, arithcode *acode)
{
	#define NWK 20
	#define JTRY(j,k,m) ((long)((((double)(k))*((double)(j)))/((double)(m))))

	int j,k;
	unsigned long ihi,ja,jh,jl,m;

	if (!isign) {
		acode->jdif=acode->nrad-1;
		for (j=NWK;j>=1;j--) {
			acode->iupb[j]=acode->nrad-1;
			acode->ilob[j]=0;
			acode->nc=j;
			if (acode->jdif > acode->minint) return;
			acode->jdif=(acode->jdif+1)*acode->nrad-1;
		}
		nrerror("NWK too small in arcode.");
	} else {
		if (isign > 0) {
			if (*ich > acode->nch) nrerror("bad ich in arcode.");
		}
		else {
			ja=(*codep)[*lcd]-acode->ilob[acode->nc];
			for (j=acode->nc+1;j<=NWK;j++) {
				ja *= acode->nrad;
				ja += ((*codep)[*lcd+j-acode->nc]-acode->ilob[j]);
			}
			ihi=acode->nch+1;
			*ich=0;
			while (ihi-(*ich) > 1) {
				m=(*ich+ihi)>>1;
				if (ja >= JTRY(acode->jdif,acode->ncumfq[m+1],acode->ncum))
					*ich=m;
				else ihi=m;
			}
			if (*ich == acode->nch) return;
		}
		jh=JTRY(acode->jdif,acode->ncumfq[*ich+2],acode->ncum);
		jl=JTRY(acode->jdif,acode->ncumfq[*ich+1],acode->ncum);
		acode->jdif=jh-jl;
		arcsum(acode->ilob,acode->iupb,jh,NWK,acode->nrad,acode->nc);
		arcsum(acode->ilob,acode->ilob,jl,NWK,acode->nrad,acode->nc);
		for (j=acode->nc;j<=NWK;j++) {
			if (*ich != acode->nch && acode->iupb[j] != acode->ilob[j]) break;
			if (*lcd > *lcode) {
				nrmessage("Reached the end of the 'code' array.\n");
				nrmessage("Attempting to expand its size.\n");
				*lcode *= 1.5;
				if ((*codep=(unsigned char *)realloc(*codep,
					(unsigned)(*lcode*sizeof(unsigned char)))) == NULL) {
					nrerror("Size expansion failed");
				}
			}
			if (isign > 0) (*codep)[*lcd]=(unsigned char)acode->ilob[j];
			++(*lcd);
		}
		if (j > NWK) return;
		acode->nc=j;
		for(j=0;acode->jdif<acode->minint;j++)
			acode->jdif *= acode->nrad;
		if (acode->nc-j < 1) nrerror("NWK too small in arcode.");
		if (j) {
			for (k=acode->nc;k<=NWK;k++) {
				acode->iupb[k-j]=acode->iupb[k];
				acode->ilob[k-j]=acode->ilob[k];
			}
		}
		acode->nc -= j;
		for (k=NWK-j+1;k<=NWK;k++) acode->iupb[k]=acode->ilob[k]=0;
	}
	return;

	#undef NWK
	#undef JTRY
}

void arcsum(unsigned long iin[], unsigned long iout[], unsigned long ja,
	int nwk, unsigned long nrad, unsigned long nc)
{
	int j,karry=0;
	unsigned long jtmp;

	for (j=nwk;j>nc;j--) {
		jtmp=ja;
		ja /= nrad;
		iout[j]=iin[j]+(jtmp-ja*nrad)+karry;
		if (iout[j] >= nrad) {
			iout[j] -= nrad;
			karry=1;
		} else karry=0;
	}
	iout[nc]=iin[nc]+ja+karry;
}

// NR 20.6 Arithmetic at arbitrary precision
void mpadd(unsigned char w[], unsigned char u[], unsigned char v[], int n)
{
	int j;
	unsigned short ireg=0;

	for (j=n;j>=1;j--) {
		ireg=u[j]+v[j]+HIBYTE(ireg);
		w[j+1]=LOBYTE(ireg);
	}
	w[1]=HIBYTE(ireg);
}

void mpsub(int *is, unsigned char w[], unsigned char u[], unsigned char v[],
	int n)
{
	int j;
	unsigned short ireg=256;

	for (j=n;j>=1;j--) {
		ireg=255+u[j]-v[j]+HIBYTE(ireg);
		w[j]=LOBYTE(ireg);
	}
	*is=HIBYTE(ireg)-1;
}

void mpsad(unsigned char w[], unsigned char u[], int n, int iv)
{
	int j;
	unsigned short ireg;

	ireg=256*iv;
	for (j=n;j>=1;j--) {
		ireg=u[j]+HIBYTE(ireg);
		w[j+1]=LOBYTE(ireg);
	}
	w[1]=HIBYTE(ireg);
}

void mpsmu(unsigned char w[], unsigned char u[], int n, int iv)
{
	int j;
	unsigned short ireg=0;

	for (j=n;j>=1;j--) {
		ireg=u[j]*iv+HIBYTE(ireg);
		w[j+1]=LOBYTE(ireg);
	}
	w[1]=HIBYTE(ireg);
}

void mpsdv(unsigned char w[], unsigned char u[], int n, int iv, int *ir)
{
	int i,j;

	*ir=0;
	for (j=1;j<=n;j++) {
		i=256*(*ir)+u[j];
		w[j]=(unsigned char) (i/iv);
		*ir=i % iv;
	}
}

void mpneg(unsigned char u[], int n)
{
	int j;
	unsigned short ireg=256;

	for (j=n;j>=1;j--) {
		ireg=255-u[j]+HIBYTE(ireg);
		u[j]=LOBYTE(ireg);
	}
}

void mpmov(unsigned char u[], unsigned char v[], int n)
{
	int j;

	for (j=1;j<=n;j++) u[j]=v[j];
}

void mplsh(unsigned char u[], int n)
{
	int j;

	for (j=1;j<=n;j++) u[j]=u[j+1];
}

void mpmul(unsigned char w[], unsigned char u[], unsigned char v[], int n,
	int m)
{
	#define RX 256.0

	int j,mn,nn=1;
	double cy,t,*a,*b;

	mn=IMAX(m,n);
	while (nn < mn) nn <<= 1;
	nn <<= 1;
	a=dvector(1,nn);
	b=dvector(1,nn);
	for (j=1;j<=n;j++)
		a[j]=(double)u[j];
	for (j=n+1;j<=nn;j++) a[j]=0.0;
	for (j=1;j<=m;j++)
		b[j]=(double)v[j];
	for (j=m+1;j<=nn;j++) b[j]=0.0;
	realft(a,nn,1);
	realft(b,nn,1);
	b[1] *= a[1];
	b[2] *= a[2];
	for (j=3;j<=nn;j+=2) {
		b[j]=(t=b[j])*a[j]-b[j+1]*a[j+1];
		b[j+1]=t*a[j+1]+b[j+1]*a[j];
	}
	realft(b,nn,-1);
	cy=0.0;
	for (j=nn;j>=1;j--) {
		t=b[j]/(nn>>1)+cy+0.5;
		b[j]=t;
		while (b[j] >= RX) b[j] -= RX;
		cy=(int) (t/RX);
	}
	if (cy >= RX) nrerror("cannot happen in fftmul");
	w[1]=(unsigned char) cy;
	for (j=2;j<=n+m;j++)
		w[j]=(unsigned char) b[j-1];
	free_dvector(b,1,nn);
	free_dvector(a,1,nn);

	#undef RX
}

void mpinv(unsigned char u[], unsigned char v[], int n, int m)
{
	#define MF 4
	#define BI (1.0/256)

	unsigned char *rr,*s;
	int i,j,maxmn,mm;
	double fu,fv;

	maxmn=IMAX(n,m);
	rr=cvector(1,1+(maxmn<<1));
	s=cvector(1,maxmn);
	mm=IMIN(MF,m);
	fv=(double) v[mm];
	for (j=mm-1;j>=1;j--) {
		fv *= BI;
		fv += v[j];
	}
	fu=1.0/fv;
	for (j=1;j<=n;j++) {
		i=(int) fu;
		u[j]=(unsigned char) i;
		fu=256.0*(fu-i);
	}
	for (;;) {
		mpmul(rr,u,v,n,m);
		mpmov(s,&rr[1],n);
		mpneg(s,n);
		s[1] -= 254;
		mpmul(rr,s,u,n,n);
		mpmov(u,&rr[1],n);
		for (j=2;j<n;j++)
			if (s[j]) break;
		if (j==n) {
			free_cvector(s,1,maxmn);
			free_cvector(rr,1,1+(maxmn<<1));
			return;
		}
	}

	#undef MF
	#undef BI
}

void mpdiv(unsigned char q[], unsigned char r[], unsigned char u[],
	unsigned char v[], int n, int m)
{
	#define MACC 3

	int is;
	unsigned char *rr,*s;

	rr=cvector(1,(n+MACC)<<1);
	s=cvector(1,n+MACC);
	mpinv(s,v,n-m+MACC,m);
	mpmul(rr,s,u,n-m+MACC,n);
	mpmov(q,&rr[1],n-m+1);
	mpmul(rr,q,v,n-m+1,m);
	mpsub(&is,&rr[1],u,&rr[1],n);
	if (is) nrerror("MACC too small in mpdiv");
	mpmov(r,&rr[n-m+1],m);
	free_cvector(s,1,n+MACC);
	free_cvector(rr,1,(n+MACC)<<1);

	#undef MACC
}

void mpsqrt(unsigned char w[], unsigned char u[], unsigned char v[], int n,
	int m)
{
	#define MF 3
	#define BI (1.0/256)

	int i,ir,j,mm;
	double fu,fv;
	unsigned char *r,*s;

	r=cvector(1,n<<1);
	s=cvector(1,n<<1);
	mm=IMIN(m,MF);
	fv=(double) v[mm];
	for (j=mm-1;j>=1;j--) {
		fv *= BI;
		fv += v[j];
	}
	fu=1.0/sqrt(fv);
	for (j=1;j<=n;j++) {
		i=(int) fu;
		u[j]=(unsigned char) i;
		fu=256.0*(fu-i);
	}
	for (;;) {
		mpmul(r,u,u,n,n);
		mplsh(r,n);
		mpmul(s,r,v,n,m);
		mplsh(s,n);
		mpneg(s,n);
		s[1] -= 253;
		mpsdv(s,s,n,2,&ir);
		for (j=2;j<n;j++) {
			if (s[j]) {
				mpmul(r,s,u,n,n);
				mpmov(u,&r[1],n);
				break;
			}
		}
		if (j<n) continue;
		mpmul(r,u,v,n,m);
		mpmov(w,&r[1],n);
		free_cvector(s,1,n<<1);
		free_cvector(r,1,n<<1);
		return;
	}
	#undef MF
	#undef BI
}

void mp2dfr(unsigned char a[], unsigned char s[], int n, int *m)
{
	#define IAZ 48

	int j;

	*m=(int) (2.408*n);
	for (j=1;j<=(*m);j++) {
		mpsmu(a,a,n,10);
		s[j]=a[1]+IAZ;
		mplsh(a,n);
	}

	#undef IAZ
}

void mppi(int n)
{
	#define IAOFF 48

	int ir,j,m;
	unsigned char mm,*x,*y,*sx,*sxi,*t,*s,*pi;

	x=cvector(1,n+1);
	y=cvector(1,n<<1);
	sx=cvector(1,n);
	sxi=cvector(1,n);
	t=cvector(1,n<<1);
	s=cvector(1,3*n);
	pi=cvector(1,n+1);
	t[1]=2;
	for (j=2;j<=n;j++) t[j]=0;
	mpsqrt(x,x,t,n,n);
	mpadd(pi,t,x,n);
	mplsh(pi,n);
	mpsqrt(sx,sxi,x,n,n);
	mpmov(y,sx,n);
	for (;;) {
		mpadd(x,sx,sxi,n);
		mpsdv(x,&x[1],n,2,&ir);
		mpsqrt(sx,sxi,x,n,n);
		mpmul(t,y,sx,n,n);
		mpadd(&t[1],&t[1],sxi,n);
		x[1]++;
		y[1]++;
		mpinv(s,y,n,n);
		mpmul(y,&t[2],s,n,n);
		mplsh(y,n);
		mpmul(t,x,s,n,n);
		mm=t[2]-1;
		j=t[n+1]-mm;
		if (j > 1 || j < -1) {
			for (j=3;j<=n;j++) {
				if (t[j] != mm) {
					mpmul(s,pi,&t[1],n,n);
					mpmov(pi,&s[1],n);
					break;
				}
			}
			if (j <= n) continue;
		}
		nrmessage("pi=\n");
		s[1]=pi[1]+IAOFF;
		s[2]='.';
		m=mm;
		mp2dfr(&pi[1],&s[2],n-1,&m);
		s[m+3]=0;
		fprintf(stderr," %64s\n",&s[1]);
		free_cvector(pi,1,n+1);
		free_cvector(s,1,3*n);
		free_cvector(t,1,n<<1);
		free_cvector(sxi,1,n);
		free_cvector(sx,1,n);
		free_cvector(y,1,n<<1);
		free_cvector(x,1,n+1);
		return;
	}

	#undef IAOFF
}

} // namespace DP

} // namespace NR