#include "MCS_Matrix.h"
#include "MCS_NRLegacy.h"
#include "MCS_Vector.h"

using namespace mcs::numerics;
using namespace NR::legacy;

VF32 Linear_Regression(const VF32& x,const VF32& y)
{ 
	Regression<Float32>	rData;
	
	return Polynomial_Regression(x,y,rData,1); 
}

VF64 Linear_Regression(const VF64& x,const VF64& y)
{ 
	Regression<Float64>	rData;
	
	return Polynomial_Regression(x,y,rData,1); 
}

VF32 Linear_Regression(const VF32& x,const VF32& y,Regression<Float32>& rData)
{ 
	return Polynomial_Regression(x,y,rData,1); 
}

VF64 Linear_Regression(const VF64& x,const VF64& y,Regression<Float64>& rData)
{ 
	return Polynomial_Regression(x,y,rData,1); 
}

VF32 Quadratic_Regression(const VF32& x,const VF32& y)
{ 
	Regression<Float32>	rData;
	
	return Polynomial_Regression(x,y,rData,2); 
}

VF64 Quadratic_Regression(const VF64& x,const VF64& y)
{ 
	Regression<Float64>	rData;
	
	return Polynomial_Regression(x,y,rData,2); 
}

VF32 Quadratic_Regression(const VF32& x,const VF32& y,Regression<Float32>& rData)
{ 
	return Polynomial_Regression(x,y,rData,2); 
}

VF64 Quadratic_Regression(const VF64& x,const VF64& y,Regression<Float64>& rData)
{ 
	return Polynomial_Regression(x,y,rData,2); 
}

VF32 Polynomial_Regression(const VF32& x,const VF32& y,Int32 order)
{ 
	Regression<Float32>	rData;
	
	return Polynomial_Regression(x,y,rData,order); 
}

VF64 Polynomial_Regression(const VF64& x,const VF64& y,Int32 order)
{ 
	Regression<Float64>	rData;
	
	return Polynomial_Regression(x,y,rData,order); 
}

VF32 Polynomial_Regression(const VF32& x,const VF32& y,Regression<Float32>& rData,Int32 order)
{	
	if (x.range() != y.range() || order < 1) throw legacy::legacy::Err();
	
	try 
	{ 
		rData = Regression<Float32>(order+1);
		
		Matrix<Float32>	covar;
		Vector<Float32>	coef(rData.coefficients().range());
		Float32			chisq;
	
		rData.fit_function(Poly_Fit);
	
		Matrix<Float32>	u,v;
		Vector<Float32>	w;
		
		rData.fit(svdfit(x,y,Vector<Float32>(x.range(),1),coef,u,v,w,chisq,Poly_Fit));
		rData.coefficients(coef);
		rData.residual(y-rData.fit());
		rData.chiSquare(chisq);
		
		svdvar(v,w,covar);
		rData.covariance(covar);
		
		return rData.coefficients();
	}
	catch (VM::Err&) 
	{ 
		throw legacy::Err(); 
	}
}

VF64 Polynomial_Regression(const VF64& x,const VF64& y,Regression<Float64>& rData,Int32 order)
{	
	if (x.range() != y.range() || order < 1) throw legacy::Err();
	
	try 
	{ 
		rData = Regression<Float64>(order+1);
		
		rData.fit_function(Poly_Fit);
	
		Matrix<Float64>	covar;
		Vector<Float64>	coef(rData.coefficients().range());
		Float64			chisq;
	
		Matrix<Float64>	u,v;
		Vector<Float64>	w;
		
		rData.fit(svdfit(x,y,Vector<Float64>(x.range(),1),coef,u,v,w,chisq,Poly_Fit));
		rData.coefficients(coef);
		rData.residual(y-rData.fit());
		rData.chiSquare(chisq);
		
		svdvar(v,w,covar);
		rData.covariance(covar);
		
		return rData.coefficients();
	}
	catch (VM::Err&) 
	{ 
		throw legacy::Err(); 
	}
}

