#ifndef MCS_NRLEGACY_H
#define MCS_NRLEGACY_H

#include "MCS_Matrix.h"
#include "MCS_Regression.h"
#include "MCS_Vector.h"

namespace NR {

namespace legacy {

typedef mcs::Vector<bool>		VB;
typedef mcs::Vector<Int8>		VI8;
typedef mcs::Vector<Uint8>		VUI8;
typedef mcs::Vector<Int16>		VI16;
typedef mcs::Vector<Uint16>		VUI16;
typedef mcs::Vector<Int32>		VI32;
typedef mcs::Vector<Uint32>		VUI32;
typedef mcs::Vector<Float32>	VF32;
typedef mcs::Vector<Float64>	VF64;
typedef mcs::Vector<Complex32>	VC32;
typedef mcs::Vector<Complex64>	VC64;

typedef mcs::Matrix<bool>		MB;
typedef mcs::Matrix<Int8>		MI8;
typedef mcs::Matrix<Uint8>		MUI8;
typedef mcs::Matrix<Int16>		MI16;
typedef mcs::Matrix<Uint16>		MUI16;
typedef mcs::Matrix<Int32>		MI32;
typedef mcs::Matrix<Uint32>		MUI32;
typedef mcs::Matrix<Float32>	MF32;
typedef mcs::Matrix<Float64>	MF64;
typedef mcs::Matrix<Complex32>	MC32;
typedef mcs::Matrix<Complex64>	MC64;

typedef mcs::numerics::Regression<Float32>	RF32;
typedef mcs::numerics::Regression<Float64>	RF64;

// Numerical Recipes error class
class Err 
{ 
	public: 
		Err(const std::string message = "") 
		{ if (message != "") std::cerr << "NR : " << message << std::endl; }
		
		Err(const char* file,int line)
		{ std::cerr << "NR error in " << file << ", line: " << line << std::endl; }
};

// other numerical algorithms			
VF32					Detrend(const VF32& x,const VF32& y);
VF64					Detrend(const VF64& x,const VF64& y);

VF32					Gaussian_Smooth(const VF32& y,Float32 sigma);	
VF64					Gaussian_Smooth(const VF64& y,Float64 sigma);	

VF32				 	Linear_Regression(const VF32& x,const VF32& y);
VF64				 	Linear_Regression(const VF64& x,const VF64& y);
VF32				 	Linear_Regression(const VF32& x,const VF32& y,RF32& rData);
VF64				 	Linear_Regression(const VF64& x,const VF64& y,RF64& rData);

VB			 			Median_Filter(const VF32& data,Int32 nsigma);
VB			 			Median_Filter(const VF64& data,Int32 nsigma);

void 					Poly_Fit(Float32 x_val,VF32& function_val);
void 					Poly_Fit(Float64 x_val,VF64& function_val);

VF32					Polynomial_Regression(const VF32& x,const VF32& y,Int32 order);
VF64					Polynomial_Regression(const VF64& x,const VF64& y,Int32 order);
VF32					Polynomial_Regression(const VF32& x,const VF32& y,RF32& rData,Int32 order);
VF64					Polynomial_Regression(const VF64& x,const VF64& y,RF64& rData,Int32 order);

VF32					Quadratic_Regression(const VF32& x,const VF32& y);
VF64					Quadratic_Regression(const VF64& x,const VF64& y);
VF32					Quadratic_Regression(const VF32& x,const VF32& y,RF32& rData);
VF64					Quadratic_Regression(const VF64& x,const VF64& y,RF64& rData);

VF32 					Time_Series_Fit(const VF32& x,const VF32& y,VF32& fit,Float32& chisq,Int32 order);
VF64 					Time_Series_Fit(const VF64& x,const VF64& y,VF64& fit,Float64& chisq,Int32 order);
VF32					Time_Series_Fit(const VF32& x,const VF32& y,const VF32& s,VF32& fit,Float32& chisq,Int32 order);
VF64					Time_Series_Fit(const VF64& x,const VF64& y,const VF64& s,VF64& fit,Float64& chisq,Int32 order);

VF32					Time_Series_Regression(const VF32& x,const VF32& y,RF32& rData,Int32 order = 2);
VF64					Time_Series_Regression(const VF64& x,const VF64& y,RF64& rData,Int32 order = 2);
VF32					Time_Series_Regression(const VF32& x,const VF32& y,const VF32& s,RF32& rData,Int32 order = 2);
VF64					Time_Series_Regression(const VF64& x,const VF64& y,const VF64& s,RF64& rData,Int32 order = 2);

void 					Trig_Fit(Float32 x_val,VF32& function_val);
void 					Trig_Fit(Float64 x_val,VF64& function_val);

VF32 					Trig_Fit_Param(const VF32& coef);
VF64 					Trig_Fit_Param(const VF64& coef);
					  
Float32 				RAOB_Integrate(const VF32& x,const VF32& y,const std::string& algorithm = "Linear");   
Float64 				RAOB_Integrate(const VF64& x,const VF64& y,const std::string& algorithm = "Linear");   
									   
Float32 		 		Exp_Integrate(const VF32& x,const VF32& y);
Float64 		 		Exp_Integrate(const VF64& x,const VF64& y);

Float32 		 		Lin_Integrate(const VF32& x,const VF32& y);
Float64 		 		Lin_Integrate(const VF64& x,const VF64& y);

Float32 		 		Mix_Integrate(const VF32& x,const VF32& y);
Float64 		 		Mix_Integrate(const VF64& x,const VF64& y);

} // namespace legacy

} // namespace NR

#endif // MCS_NRLEGACY_H