//#define DEBUG
#ifndef DEBUG
	#include "MCS_Headers++.h"
#else
	#include <MSLHeaders++.h>
	
	#include "MCS_Vector.h"
	#include "MCS_Matrix.h"
	
	#include "NR.h"
#endif

using namespace NR;

Float32 NR::RAOB_Integrate(const VF32& x,const VF32& y,const string& algorithm)
{
	if (algorithm == "Linear")
		return Lin_Integrate(x,y);
	else if (algorithm == "Exponential")
		return Exp_Integrate(x,y);
	else if (algorithm == "Mixed")
		return Mix_Integrate(x,y);
	else
		return NAN;
}
		
Float32 NR::Lin_Integrate(const VF32& x,const VF32& y)
{
	if (x.range() != y.range()) throw Err(__FILE__,__LINE__);
	
	float	sum = 0;
	
	try
	{
		for (int idx=x.first()+1;idx<=x.last();idx++)
			sum += 0.5*(y[idx] + y[idx-1])*(x[idx]-x[idx-1]);
		
		return sum;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

Float32 NR::Exp_Integrate(const VF32& x,const VF32& y)
{
	if (x.range() != y.range()) throw Err(__FILE__,__LINE__);
	
	float	sum = 0;
	
	try
	{
		for (int idx=x.first()+1;idx<=x.last();idx++)
			sum += 0.5*sqrt(y[idx]*y[idx-1])*(x[idx]-x[idx-1]);
		
		return sum;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

Float32 NR::Mix_Integrate(const VF32& x,const VF32& y)
{
	if (x.range() != y.range()) throw Err(__FILE__,__LINE__);
	
	float	sum = 0;
	
	try
	{
		for (int idx=x.first()+1;idx<=x.last();idx++)
			sum += 0.25*(y[idx] + y[idx-1] + 2*sqrt(y[idx]*y[idx-1]))*(x[idx]-x[idx-1]);
		
		return sum;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}


