//#define DEBUG
//#define DEBUG
#ifndef DEBUG
	#include "MCS_Headers++.h"
#else
	#include <MSLHeaders++.h>
	
	#include "MCS_Vector.h"
	#include "MCS_Matrix.h"
	
	#include "NR.h"
#endif

using namespace NR;

// single precision
VF32 NR::Time_Series_Fit(const VF32& x,const VF32& y,
						 VF32& fit,Float32& chisq,Int32 order)
{
	if (x.range() != y.range())
		throw Err(__FILE__,__LINE__);
	
	try
	{
		Vector<float>	s(x.range());
		
		// determine weighting coefficients by spacing between data points
		s[s.first()] = std::sqrt(fabs(x[x.first()+1]-x[x.first()]));
		s[s.last()] = std::sqrt(fabs(x[x.last()]-x[x.last()-1]));
		for (v_idx i=s.first()+1;i<=s.last()-1;i++)
			s[i] = std::sqrt(0.5*(fabs(x[i+1]-x[i])+fabs(x[i]-x[i-1])));
		s /= mean(s);
									
		return Time_Series_Fit(x,y,s,fit,chisq,order);
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

VF32 NR::Time_Series_Fit(const VF32& x,const VF32& y,const VF32& s,
						 VF32& fit,Float32& chisq,Int32 order)
{
	if (x.range() != y.range() || x.range() != s.range())
		throw Err(__FILE__,__LINE__);
		
	try
	{
		double			chisq_tmp;
		Vector<double>	fit_tmp;
		
		Vector<float> 	coef(Time_Series_Fit(x.type_cast(Float64()),y.type_cast(Float64()),s.type_cast(Float64()),fit_tmp,chisq_tmp,order).type_cast(Float32()));
		
		fit = fit_tmp.type_cast(Float32());
		chisq = static_cast<Float32>(chisq_tmp);
		
		return coef;
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

// double precision
VF64 NR::Time_Series_Fit(const VF64& x,const VF64& y,
						 VF64& fit,Float64& chisq,Int32 order)
{
	if (x.range() != y.range())
		throw Err(__FILE__,__LINE__);
	
	try
	{
		Vector<double>	s(x.range());
		
		// determine weighting coefficients by spacing between data points
		s[s.first()] = std::sqrt(fabs(x[x.first()+1]-x[x.first()]));
		s[s.last()] = std::sqrt(fabs(x[x.last()]-x[x.last()-1]));
		for (v_idx i=s.first()+1;i<=s.last()-1;i++)
			s[i] = std::sqrt(0.5*(fabs(x[i+1]-x[i])+fabs(x[i]-x[i-1])));
		s /= mean(s);
									
		return (Time_Series_Fit(x,y,s,fit,chisq,order));
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}

VF64 NR::Time_Series_Fit(const VF64& x,const VF64& y,const VF64& s,
						 VF64& fit,Float64& chisq,Int32 order)
{
	if (x.range() != y.range() || x.range() != s.range())		
		throw Err(__FILE__,__LINE__);

	try
	{
		Vector<bool> 	mask = (x.is_nan() || y.is_nan());
		Vector<double>	xd(x),yd(y),sd(s);
						
		xd.remove(mask);
		yd.remove(mask);
		sd.remove(mask);
							
		sd /= mean(sd);			
		
		int 	num_fit_param = 2 + 2*order;
		
		if (xd.dim() < 4*num_fit_param)
			throw Err(__FILE__,__LINE__);
	
		Vector<double>	coef(v_range(1,num_fit_param));
		Vector<bool>	flag(v_range(1,num_fit_param),true);
		Matrix<double>	covar(m_range(1,num_fit_param,1,num_fit_param));
		
		Matrix<double>	u,v;
		Vector<double>	w;
	
		// calculate regression coefficients
		fit = svdfit(xd,yd,sd,coef,u,v,w,chisq,Trig_Fit);
		svdvar(v,w,covar);
		
		//Q = sqrt(1-chisq/sum(sqr((yd-mean(yd))/sd)));				// r^2
		//R = stddev(yd-fit);
					
		return Trig_Fit_Param(coef);
	}
	catch (VM::Err&)
	{
		throw Err(__FILE__,__LINE__);
	}
}
