//#define DEBUG
#ifndef DEBUG
	#include "MCS_Headers++.h"
#else
	#include <MSLHeaders++.h>
	
	#include "MCS_Vector.h"
	#include "MCS_Matrix.h"
	
	#include "NR.h"
#endif

using namespace NR;
using namespace Constants::Mathematical;

// single precision
void NR::Trig_Fit(Float32 x_val,VF32& function_val)
{
	function_val[function_val.first()] = 1;				// constant term
	function_val[function_val.first()+1] = x_val;		// linear term
	
	for (v_idx i=function_val.first()+2;i<=function_val.last();i+=2)
	{
		function_val[i]   = sin(_TWO_PI_*float((i-1)/2)*x_val);	
		function_val[i+1] = cos(_TWO_PI_*float((i-1)/2)*x_val);
	}
}

// convert from 
//	c1+c2*x+c3*sin(w*x)+c4*cos(w*x)+c5*sin(2*w*x)+c6*cos(2*w*x)
// to
//	d1+d2*x+d3*sin(w*x+d4)+d5*sin(2*w*x+d6)
VF32 NR::Trig_Fit_Param(const VF32& coef)
{
	Vector<float> tmp(coef);
	
	for (v_idx i=tmp.first()+2;i<=tmp.last();i+=2)
	{
		float	amp,phase;
		
		phase = atan2(coef[i+1],coef[i]);
		amp = coef[i]/cos(phase);
		
		if (amp < 0.0)
		{
			phase = (phase < _PI_ ? phase + _PI_ : phase - _PI_);
			amp = fabs(amp);
		}
		
		if (phase < 0) phase = phase + _TWO_PI_;
		
		phase *= 180.0/_PI_;
		
		tmp[i] = amp;
		tmp[i+1] = phase;
	}
	
	return tmp;
}
					
// double precision
void NR::Trig_Fit(Float64 x_val,VF64& function_val)
{
	function_val[function_val.first()] = 1;				// constant term
	function_val[function_val.first()+1] = x_val;		// linear term
	
	for (v_idx i=function_val.first()+2;i<=function_val.last();i+=2)
	{
		function_val[i]   = sin(_TWO_PI_*double((i-1)/2)*x_val);	
		function_val[i+1] = cos(_TWO_PI_*double((i-1)/2)*x_val);
	}
}

VF64 NR::Trig_Fit_Param(const VF64& coef)
{
	Vector<double> tmp(coef);
	
	for (v_idx i=tmp.first()+2;i<=tmp.last();i+=2)
	{
		double	amp,phase;
		
		phase = atan2(coef[i+1],coef[i]);
		amp = coef[i]/cos(phase);
		
		if (amp < 0.0)
		{
			phase = (phase < _PI_ ? phase + _PI_ : phase - _PI_);
			amp = fabs(amp);
		}
		
		if (phase < 0) phase = phase + _TWO_PI_;
		
		phase *= 180.0/_PI_;
		
		tmp[i] = amp;
		tmp[i+1] = phase;
	}
	
	return tmp;
}
					
