#include "MCS_OrbitalPropagator.h"

namespace mcs {

/*
* DEEP SPACE 31 OCT 80
SUBROUTINE DEEP
COMMON/E1/XMO,XNODEO,OMEGAO,EO,XINCL,XNO,XNDT2O,XNDD6O,BSTAR,X,Y,Z,XDOT,YDOT,ZDOT,EPOCH,DS50
COMMON/C1/CK2,CK4,E6A,QOMS2T,S,TOTHRD,XJ3,XKE,XKMPER,XMNPDA,AE
COMMON/C2/DE2RA,PI,PIO2,TWOPI,X3PIO2
DOUBLE PRECISION EPOCH, DS50
DOUBLE PRECISION DAY,PREEP,XNODCE,ATIME,DELT,SAVTSN,STEP2,STEPN,STEPP
DATA ZNS, C1SS, ZES/ 1.19459E-5, 2.9864797E-6, .01675/
DATA ZNL, C1L, ZEL/ 1.5835218E-4, 4.7968065E-7, .05490/
DATA ZCOSIS, ZSINIS, ZSINGS/ .91744867, .39785416, -.98088458/
DATA ZCOSGS, ZCOSHS, ZSINHS/ .1945905, 1.0, 0.0/
DATA Q22,Q31,Q33/1.7891679E-6,2.1460748E-6,2.2123015E-7/
DATA G22,G32/5.7686396,0.95240898/
DATA G44,G52/1.8014998,1.0508330/
DATA G54/4.4108898/
DATA ROOT22,ROOT32/1.7891679E-6,3.7393792E-7/
DATA ROOT44,ROOT52/7.3636953E-9,1.1428639E-7/
DATA ROOT54/2.1765803E-9/
DATA THDT/4.3752691E-3/
*/
const Float64	SGP4SDP4Model::SDP4::DeepSpace::C1L = 4.7968065E-7,
				SGP4SDP4Model::SDP4::DeepSpace::C1SS = 2.9864797E-6, 
				SGP4SDP4Model::SDP4::DeepSpace::G22 = 5.7686396,
				SGP4SDP4Model::SDP4::DeepSpace::G32 = 0.95240898,
				SGP4SDP4Model::SDP4::DeepSpace::G44 = 1.8014998,
				SGP4SDP4Model::SDP4::DeepSpace::G52 = 1.0508330,
				SGP4SDP4Model::SDP4::DeepSpace::G54 = 4.4108898,
				SGP4SDP4Model::SDP4::DeepSpace::Q22 = 1.7891679E-6,
				SGP4SDP4Model::SDP4::DeepSpace::Q31 = 2.1460748E-6,
				SGP4SDP4Model::SDP4::DeepSpace::Q33 = 2.2123015E-7,	
				SGP4SDP4Model::SDP4::DeepSpace::ROOT22 = 1.7891679E-6,
				SGP4SDP4Model::SDP4::DeepSpace::ROOT32 = 3.7393792E-7,
				SGP4SDP4Model::SDP4::DeepSpace::ROOT44 = 7.3636953E-9,
				SGP4SDP4Model::SDP4::DeepSpace::ROOT52 = 1.1428639E-7,
				SGP4SDP4Model::SDP4::DeepSpace::ROOT54 = 2.1765803E-9,	
				SGP4SDP4Model::SDP4::DeepSpace::THDT = 4.3752691E-3,
				SGP4SDP4Model::SDP4::DeepSpace::ZES = .01675,
				SGP4SDP4Model::SDP4::DeepSpace::ZNL = 1.5835218E-4,
				SGP4SDP4Model::SDP4::DeepSpace::ZNS = 1.19459E-5, 
				SGP4SDP4Model::SDP4::DeepSpace::ZEL = .05490,
				SGP4SDP4Model::SDP4::DeepSpace::ZCOSGS = .1945905,
				SGP4SDP4Model::SDP4::DeepSpace::ZSINGS = -.98088458,
				SGP4SDP4Model::SDP4::DeepSpace::ZCOSIS = .91744867,
				SGP4SDP4Model::SDP4::DeepSpace::ZSINIS = .39785416,
				SGP4SDP4Model::SDP4::DeepSpace::ZCOSHS = 1.0,
				SGP4SDP4Model::SDP4::DeepSpace::ZSINHS = 0.0;

SGP4SDP4Model::SDP4::DeepSpace::DeepSpace(const SGP4SDP4Model::SDP4& sdp4) : 
	sdp4_(sdp4)
{
	iresfl = isynfl = false;
	
	atime = delt = 0;
	d2201 = d2211 = d3210 = d3222 = d4410 = d4422 = d5220 = d5232 = d5421 = d5433 = 0;
	del1 = del2 = del3 = 0;
	ee2 = e3 = 0;
	fasx2 = fasx4 = fasx6 = 0;
	omegaq = 0;
	pe = pinc = pl = 0;
	savtsn = 0;
	stepn = stepp = step2 = 0;
	se2 = se3 = 0;
	si2 = si3 = 0;
	sh2 = sh3 = 0;
	sgh2 = sgh3 = sgh4 = 0;
	shs = shl = sghs = sghl = 0;
	sl2 = sl3 = sl4 = 0;
	sse = ssg = ssh = ssi = ssl = 0;
	xi2 = xi3 = 0;
	xh2 = xh3 = 0;
	xl2 = xl3 = xl4 = 0;
	xgh2 = xgh3 = xgh4 = 0;
	xfact = 0;
	xlamo = 0;
	xli = 0;
	xldot = 0;
	xni = 0;
	xnddt = 0;
	xndot = 0;
	xnq = 0;
	zm = zmol = zmos = 0;
}

//* ENTRANCE FOR DEEP SPACE INITIALIZATION
//ENTRY DPINIT(EQSQ,SINIQ,COSIQ,RTEQSQ,AO,COSQ2,SINOMO,COSOMO,BSQ,XLLDOT,OMGDT,XNODOT,XNODP)
void SGP4SDP4Model::SDP4::DeepSpace::initialize()
{
	//THGR=THETAG(EPOCH)
	const Float64	eqsq = sdp4_.eo2,
					siniq = sdp4_.sinio,
					cosiq = sdp4_.cosio,
					ao = sdp4_.aopp_,
					cosq2 = sdp4_.cosio2,
					sinomo = std::sin(sdp4_.wo_),
					cosomo = std::cos(sdp4_.wo_),
					xlldot = sdp4_.Mdot,
					omgdot = sdp4_.wdot,
					xnodot = sdp4_.Wdot,
					xnodp = sdp4_.nopp_,
					xnodeo = sdp4_.Wo_,
					omegao = sdp4_.wo_,
					thgr = sdp4_.thetag_,
					betao = sdp4_.betao,
					betao2 = sdp4_.betao2;
	
	//EQ = EO
	//XNQ = XNODP
	//AQNV = 1./AO
	//XQNCL = XINCL
	//XMAO=XMO
	//XPIDOT=OMGDT+XNODOT
	//SINQ = SIN(XNODEO)
	//COSQ = COS(XNODEO)
	//OMEGAQ = OMEGAO
	const Float64	eq = sdp4_.eo_,
					xnq = xnodp,
					aqnv = 1/ao,
					xqncl = sdp4_.io_,
					xmao = sdp4_.Mo_,
					xpidot = omgdot+xnodot,
					sinq = std::sin(xnodeo),
					cosq = std::cos(xnodeo),
					omegaq = omegao;
	
	//* INITIALIZE LUNAR SOLAR TERMS
	//5 
	//DAY=DS50+18261.5D0
	//IF (DAY.EQ.PREEP) GO TO 10
	//PREEP = DAY
	//XNODCE=4.5236020-9.2422029E-4*DAY
	//STEM=DSIN (XNODCE)
	//CTEM=DCOS (XNODCE)
	//ZCOSIL=.91375164-.03568096*CTEM
	//ZSINIL=SQRT (1.-ZCOSIL*ZCOSIL)
	//ZSINHL= .089683511*STEM/ZSINIL
	//ZCOSHL=SQRT (1.-ZSINHL*ZSINHL)
	//C=4.7199672+.22997150*DAY
	//GAM=5.8351514+.0019443680*DAY
	//ZMOL = FMOD2P(C-GAM)
	//ZX= .39785416*STEM/ZSINIL
	//ZY= ZCOSHL*CTEM+0.91744867*ZSINHL*STEM
	//ZX=ACTAN(ZX,ZY)
	//ZX=GAM+ZX-XNODCE
	//ZCOSGL=COS (ZX)
	//ZSINGL=SIN (ZX)
	//ZMOS=6.2565837D0+.017201977D0*DAY
	//ZMOS=FMOD2P(ZMOS)
	static Float64	prev_epoch = 0.0;
	//const Float64	day = (sdp4_.model_.epoch().julian()-Date(1950,1,1).julian()) + 18261.5;
	// error in FORTRAN version?  day number looks to be a day off
	const Float64	day = (sdp4_.model_.epoch().julian()-Date(1950,1,1).julian()) + 18262.5;
	
	Float64	zcosgl,zsingl,zcosil,zsinil,zcoshl,zsinhl;
	
	if (day != prev_epoch)
	{
		const Float64	xnodce = 4.5236020 - 9.2422029E-4*day,
						stem = std::sin(xnodce),
						ctem = std::cos(xnodce);
		
		zcosil = .91375164-.03568096*ctem;
		zsinil = std::sqrt(1-std::sqr(zcosil));
		zsinhl = .089683511*stem/zsinil;
		zcoshl = std::sqrt(1-std::sqr(zsinhl));
		
		const Float64	c = 4.7199672 + 0.22997150*day,
						gam = 5.8351514 + 0.0019443680*day;
		
		zmol = std::mod(c-gam,_two_pi_);
		
		const Float64	zy = zcoshl*ctem+0.91744867*zsinhl*stem,
						zx = atan2mod2pi(0.39785416*stem/zsinil,zy)+gam-xnodce;
		
		zcosgl = std::cos(zx);
		zsingl = std::sin(zx);
		
		zmos = std::mod(6.2565837 + 0.017201977*day,_two_pi_);

		prev_epoch = day;
	}
	
	Float64	cc,zn,ze,zmo,xnoi;
	Float64	se,si,sl,sh,sgh;
	Float64	zcosg,zsing,zcosi,zsini,zcosh,zsinh;
	
	//* DO SOLAR TERMS
	//10 
	//LS = 0
	//SAVTSN=1.D20
	//ZCOSG=ZCOSGS
	//ZSING=ZSINGS
	//ZCOSI=ZCOSIS
	//ZSINI=ZSINIS
	//ZCOSH=COSQ
	//ZSINH=SINQ
	//CC=C1SS
	//ZN=ZNS
	//ZE=ZES
	//ZMO=ZMOS
	//XNOI=1./XNQ
	//ASSIGN 30 TO LS
	savtsn = 1.0E20;
	zcosg = ZCOSGS;
	zsing = ZSINGS;
	zcosi = ZCOSIS;
	zsini = ZSINIS;
	zcosh = cosq;
	zsinh = sinq;
	cc = C1SS;
	zn = ZNS;
	ze = ZES;
	zmo = zmos;
	xnoi = 1/xnq;
	
	for (Int32 i=1;i<=2;++i)
	{
		const Float64	//20 
						//A1=ZCOSG*ZCOSH+ZSING*ZCOSI*ZSINH
						//A3=-ZSING*ZCOSH+ZCOSG*ZCOSI*ZSINH
						//A7=-ZCOSG*ZSINH+ZSING*ZCOSI*ZCOSH
						//A8=ZSING*ZSINI
						//A9=ZSING*ZSINH+ZCOSG*ZCOSI*ZCOSH
						a1 = zcosg*zcosh+zsing*zcosi*zsinh,
						a3 = -zsing*zcosh+zcosg*zcosi*zsinh,
						a7 = -zcosg*zsinh+zsing*zcosi*zcosh,
						a8 = zsing*zsini,
						a9 = zsing*zsinh+zcosg*zcosi*zcosh,
						//A10=ZCOSG*ZSINI
						//A2= COSIQ*A7+ SINIQ*A8
						//A4= COSIQ*A9+ SINIQ*A10
						//A5=- SINIQ*A7+ COSIQ*A8
						//A6=- SINIQ*A9+ COSIQ*A10
						a10 = zcosg*zsini,
						a2 = cosiq*a7+siniq*a8,
						a4 = cosiq*a9+siniq*a10,
						a5 = -siniq*a7+cosiq*a8,
						a6 = -siniq*a9+cosiq*a10,
						//X1=A1*COSOMO+A2*SINOMO
						//X2=A3*COSOMO+A4*SINOMO
						//X3=-A1*SINOMO+A2*COSOMO
						//X4=-A3*SINOMO+A4*COSOMO
						//X5=A5*SINOMO
						//X6=A6*SINOMO
						//X7=A5*COSOMO
						//X8=A6*COSOMO
						x1 = a1*cosomo+a2*sinomo,
						x2 = a3*cosomo+a4*sinomo,
						x3 = -a1*sinomo+a2*cosomo,
						x4 = -a3*sinomo+a4*cosomo,
						x5 = a5*sinomo,
						x6 = a6*sinomo,
						x7 = a5*cosomo,
						x8 = a6*cosomo;
							
		Float64	z1,z2,z3,
				z11,z12,z13,
				z21,z22,z23,
				z31,z32,z33;
				
		//Z31=12.*X1*X1-3.*X3*X3
		//Z32=24.*X1*X2-6.*X3*X4
		//Z33=12.*X2*X2-3.*X4*X4
		z31 = 12*std::sqr(x1)-3*std::sqr(x3);
		z32 = 24*x1*x2-6*x3*x4;
		z33 = 12*std::sqr(x2)-3*std::sqr(x4);	
			
		//Z1=3.*(A1*A1+A2*A2)+Z31*EQSQ
		//Z2=6.*(A1*A3+A2*A4)+Z32*EQSQ
		//Z3=3.*(A3*A3+A4*A4)+Z33*EQSQ
		z1 = 3*(std::sqr(a1)+std::sqr(a2))+z31*eqsq;
		z2 = 6*(a1*a3+a2*a4)+z32*eqsq;
		z3 = 3*(std::sqr(a3)+std::sqr(a4))+z33*eqsq;
		
		//Z11=-6.*A1*A5+EQSQ *(-24.*X1*X7-6.*X3*X5)
		//Z12=-6.*(A1*A6+A3*A5)+EQSQ *(-24.*(X2*X7+X1*X8)-6.*(X3*X6+X4*X5))
		//Z13=-6.*A3*A6+EQSQ *(-24.*X2*X8-6.*X4*X6)
		z11 = -6*a1*a5+eqsq*(-24*x1*x7-6*x3*x5);
		z12 = -6*(a1*a6+a3*a5)+eqsq*(-24*(x2*x7+x1*x8)-6*(x3*x6+x4*x5));
		z13 = -6*a3*a6+eqsq*(-24*x2*x8-6*x4*x6);
		
		//Z21=6.*A2*A5+EQSQ *(24.*X1*X5-6.*X3*X7)
		//Z22=6.*(A4*A5+A2*A6)+EQSQ *(24.*(X2*X5+X1*X6)-6.*(X4*X7+X3*X8))
		//Z23=6.*A4*A6+EQSQ *(24.*X2*X6-6.*X4*X8)
		z21 = 6*a2*a5+eqsq*(24*x1*x5-6*x3*x7);
		z22 = 6*(a4*a5+a2*a6)+eqsq*(24*(x2*x5+x1*x6)-6*(x4*x7+x3*x8));	
		z23 = 6*a4*a6+eqsq*(24*x2*x6-6*x4*x8);
						
		//Z1=Z1+Z1+BETAO2*Z31
		//Z2=Z2+Z2+BETAO2*Z32
		//Z3=Z3+Z3+BETAO2*Z33
		z1 = 2*z1+betao2*z31,	
		z2 = 2*z2+betao2*z32,	
		z3 = 2*z3+betao2*z33;	
	
		const Float64	//S3=CC*XNOI
						//S2=-.5*S3/BETAO
						//S4=S3*BETAO
						//S1=-15.*EQ*S4
						//S5=X1*X3+X2*X4
						//S6=X2*X3+X1*X4
						//S7=X2*X4-X1*X3
						s3 = cc*xnoi,
						s2 = -0.5*s3/betao,
						s4 = s3*betao,
						s1 = -15*eq*s4,
						s5 = x1*x3+x2*x4,
						s6 = x2*x3+x1*x4,
						s7 = x2*x4-x1*x3;
			
		//SE=S1*ZN*S5
		//SI=S2*ZN*(Z11+Z13)
		//SL=-ZN*S3*(Z1+Z3-14.-6.*EQSQ)
		//SGH=S4*ZN*(Z31+Z33-6.)		
		se = s1*zn*s5;
		si = s2*zn*(z11+z13);
		sl = -zn*s3*(z1+z3-14-6*eqsq);
		sgh = s4*zn*(z31+z33-6);
		
		//SH=-ZN*S2*(Z21+Z23)
		//IF(XQNCL.LT.5.2359877E-2) SH=0.0
		sh = (xqncl < 5.2359877E-2) ? 0.0 : -zn*s2*(z21+z23);
		
		//EE2=2.*S1*S6
		//E3=2.*S1*S7
		ee2 = 2*s1*s6;
		e3 = 2*s1*s7;
		
		//XI2=2.*S2*Z12
		//XI3=2.*S2*(Z13-Z11)
		xi2 = 2*s2*z12;
		xi3 = 2*s2*(z13-z11);	
		
		//XL2=-2.*S3*Z2
		//XL3=-2.*S3*(Z3-Z1)
		//XL4=-2.*S3*(-21.-9.*EQSQ)*ZE
		xl2 = -2*s3*z2;
		xl3 = -2*s3*(z3-z1);
		xl4 = -2*s3*(-21-9*eqsq)*ze;
		
		//XGH2=2.*S4*Z32
		//XGH3=2.*S4*(Z33-Z31)
		//XGH4=-18.*S4*ZE
		xgh2 = 2*s4*z32;
		xgh3 = 2*s4*(z33-z31);
		xgh4 = -18*s4*ze;
		
		//XH2=-2.*S2*Z22
		//XH3=-2.*S2*(Z23-Z21)
		xh2 = -2*s2*z22;
		xh3 = -2*s2*(z23-z21);

		//GO TO LS
		if (i==2) break;
	
		//* DO LUNAR TERMS
		//30
		//SSE = SE
		//SSI=SI
		//SSL=SL
		//SSH=SH/SINIQ
		//SSG=SGH-COSIQ*SSH
		sse = se;
		ssi = si;
		ssl = sl;
		ssh = sh/siniq;
		ssg = sgh-cosiq*ssh;
		
		//SE2=EE2
		//SI2=XI2
		//SL2=XL2
		//SGH2=XGH2
		//SH2=XH2
		se2 = ee2;
		si2 = xi2;
		sl2 = xl2;
		
		sgh2 = xgh2;
		sh2 = xh2;
		
		//SE3=E3
		//SI3=XI3
		//SL3=XL3
		//SGH3=XGH3
		//SH3=XH3
		se3 = e3;
		si3 = xi3;
		sl3 = xl3;
		sgh3 = xgh3;
		sh3 = xh3;
		
		//SL4=XL4
		//SGH4=XGH4
		sl4 = xl4;
		sgh4 = xgh4;
			
		//ZCOSG=ZCOSGL
		//ZSING=ZSINGL
		//ZCOSI=ZCOSIL
		//ZSINI=ZSINIL
		//ZCOSH=ZCOSHL*COSQ+ZSINHL*SINQ
		//ZSINH=SINQ*ZCOSHL-COSQ*ZSINHL
		//ZN=ZNL
		//CC=C1L
		//ZE=ZEL
		//ZMO=ZMOL
		zcosg = zcosgl;
		zsing = zsingl;
		zcosi = zcosil;
		zsini = zsinil;
		zcosh = zcoshl*cosq+zsinhl*sinq;
		zsinh = sinq*zcoshl-cosq*zsinhl;
		zn = ZNL;
		cc = C1L;
		ze = ZEL;
		zmo = zmol;

		//ASSIGN 40 TO LS
		//GO TO 20
	}
	
	//40 
	//SSE = SSE+SE
	//SSI = SSI+SI
	//SSL = SSL+SL
	//SSG = SSG+SGH-COSIQ/SINIQ*SH
	//SSH = SSH+SH/SINIQ
	sse += se;
	ssi += si;
	ssl += sl;
	ssg += sgh-cosiq/siniq*sh;
	ssh += sh/siniq;
	
	//* GEOPOTENTIAL RESONANCE INITIALIZATION FOR 12 HOUR ORBITS
	//IRESFL=0
	//ISYNFL=0
	iresfl = false;
	isynfl = false;
			
	bool	do_geopotential_resonance,
			do_synchronous_resonance;
			
	//IF(XNQ.LT.(.0052359877).AND.XNQ.GT.(.0034906585)) GO TO 70
	if (xnq < 0.0052359877 && xnq > 0.0034906585)
	{
		do_geopotential_resonance = false;
		do_synchronous_resonance = true;
	}
	else
	{
		do_geopotential_resonance = true;
		do_synchronous_resonance = false;
	}
	
	Float64	g200,g201,g211,
			g300,g310,g322,
			g410,g422,
			g520,g521,g532,g533;
			
	Float64	f220,f221,
			f311,f321,f322,f330,
			f441,f442,
			f522,f523,f542,f543;
				
	Float64	bfact;
	
	if (do_geopotential_resonance)
	{
		//IF (XNQ.LT.(8.26E-3) .OR. XNQ.GT.(9.24E-3)) RETURN
		//IF (EQ.LT.0.5) RETURN
		if (xnq < 8.26E-3 || xnq > 9.24E-3 || eq < 0.5) return;
		
		//IRESFL =1
		iresfl = true;
		
		//EOC=EQ*EQSQ
		const Float64	eoc = eq*eqsq;
		
		//G201=-.306-(EQ-.64)*.440
		g201 = -0.306-(eq - 0.64)*0.44;
		
		//IF(EQ.GT.(.65)) GO TO 45
		if (eq <= 0.65)
		{
			//G211=3.616-13.247*EQ+16.290*EQSQ
			//G310=-19.302+117.390*EQ-228.419*EQSQ+156.591*EOC
			//G322=-18.9068+109.7927*EQ-214.6334*EQSQ+146.5816*EOC
			//G410=-41.122+242.694*EQ-471.094*EQSQ+313.953*EOC
			//G422=-146.407+841.880*EQ-1629.014*EQSQ+1083.435*EOC
			//G520=-532.114+3017.977*EQ-5740*EQSQ+3708.276*EOC
			g211 = 3.616-13.247*eq+16.290*eqsq;
			g310 = -19.302+117.390*eq-228.419*eqsq+156.591*eoc;
			g322 = -18.9068+109.7927*eq-214.6334*eqsq+146.5816*eoc;
			g410 = -41.122+242.694*eq-471.094*eqsq+313.953*eoc;
			g422 = -146.407+841.880*eq-1629.014*eqsq+1083.435*eoc;
			g520 = -532.114+3017.977*eq-5740*eqsq+3708.276*eoc;
			
			//GO TO 55
		}
		else
		{
			//45 
			//G211=-72.099+331.819*EQ-508.738*EQSQ+266.724*EOC
			//G310=-346.844+1582.851*EQ-2415.925*EQSQ+1246.113*EOC
			//G322=-342.585+1554.908*EQ-2366.899*EQSQ+1215.972*EOC
			//G410=-1052.797+4758.686*EQ-7193.992*EQSQ+3651.957*EOC
			//G422=-3581.69+16178.11*EQ-24462.77*EQSQ+12422.52*EOC	
			g211 = -72.099+331.819*eq-508.738*eqsq+266.724*eoc;
			g310 = -346.844+1582.851*eq-2415.925*eqsq+1246.113*eoc;
			g322 = -342.585+1554.908*eq-2366.899*eqsq+1215.972*eoc;
			g410 = -1052.797+4758.686*eq-7193.992*eqsq+3651.957*eoc;
			g422 = -3581.69+16178.11*eq-24462.77*eqsq+12422.52*eoc;	
				
			//IF(EQ.GT.(.715)) GO TO 50	
			if (eq > 0.715)
				//50 
				//G520=-5149.66+29936.92*EQ-54087.36*EQSQ+31324.56*EOC
				g520 = -5149.66+29936.92*eq-54087.36*eqsq+31324.56*eoc;
			else
				//G520=1464.74-4664.75*EQ+3763.64*EQSQ	
				g520 = 1464.74-4664.75*eq+3763.64*eqsq;	
					
			//GO TO 55
		}	
		
		//55 
		//IF(EQ.GE.(.7)) GO TO 60
		if (eq < 0.7)
		{
			//G533=-919.2277+4988.61*EQ-9064.77*EQSQ+5542.21*EOC
			//G521 = -822.71072+4568.6173*EQ-8491.4146*EQSQ+5337.524*EOC
			//G532 = -853.666+4690.25*EQ-8624.77*EQSQ+5341.4*EOC		
			g533 = -919.2277+4988.61*eq-9064.77*eqsq+5542.21*eoc;
			g521 = -822.71072+4568.6173*eq-8491.4146*eqsq+5337.524*eoc;
			g532 = -853.666+4690.25*eq-8624.77*eqsq+5341.4*eoc;		

			//GO TO 65
		}
		else
		{
			//60 
			//G533=-37995.78+161616.52*EQ-229838.2*EQSQ+109377.94*EOC
			//G521 = -51752.104+218913.95*EQ-309468.16*EQSQ+146349.42*EOC
			//G532 = -40023.88+170470.89*EQ-242699.48*EQSQ+115605.82*EOC
			g533 = -37995.78+161616.52*eq-229838.2*eqsq+109377.94*eoc;
			g521 = -51752.104+218913.95*eq-309468.16*eqsq+146349.42*eoc;
			g532 = -40023.88+170470.89*eq-242699.48*eqsq+115605.82*eoc;
		}
			
		//65 
		//SINI2=SINIQ*SINIQ
		//F220=.75*(1.+2.*COSIQ+COSQ2)
		//F221=1.5*SINI2
		//F321=1.875*SINIQ*(1.-2.*COSIQ-3.*COSQ2)
		//F322=-1.875*SINIQ*(1.+2.*COSIQ-3.*COSQ2)
		//F441=35.*SINI2*F220
		//F442=39.3750*SINI2*SINI2
		//F522=9.84375*SINIQ*(SINI2*(1.-2.*COSIQ-5.*COSQ2)+.33333333*(-2.+4.*COSIQ+6.*COSQ2))
		//F523 = SINIQ*(4.92187512*SINI2*(-2.-4.*COSIQ+10.*COSQ2)+6.56250012*(1.+2.*COSIQ-3.*COSQ2))
		//F542 = 29.53125*SINIQ*(2.-8.*COSIQ+COSQ2*(-12.+8.*COSIQ+10.*COSQ2))
		//F543=29.53125*SINIQ*(-2.-8.*COSIQ+COSQ2*(12.+8.*COSIQ-10.*COSQ2))
		const Float64	sini2 = std::sqr(siniq);
		
		f220 = 0.75*(1+2*cosiq+cosq2);
		f221 = 1.5*sini2;
		f321 = 1.875*siniq*(1-2*cosiq-3*cosq2);
		f322 = -1.875*siniq*(1+2*cosiq-3*cosq2);
		f441 = 35*sini2*f220;
		f442 = 39.3750*sini2*sini2;
		f522 = 9.84375*siniq*(sini2*(1-2*cosiq-5*cosq2)+0.33333333*(-2+4*cosiq+6*cosq2));
		f523 = siniq*(4.92187512*sini2*(-2-4*cosiq+10.*cosq2)+6.56250012*(1+2*cosiq-3*cosq2));
		f542 = 29.53125*siniq*(2-8*cosiq+cosq2*(-12+8*cosiq+10*cosq2));
		f543 = 29.53125*siniq*(-2-8*cosiq+cosq2*(12+8*cosiq-10*cosq2));

		//XNO2=XNQ*XNQ
		//AINV2=AQNV*AQNV
		const Float64	xno2 = std::sqr(xnq),
						ainv2 = std::sqr(aqnv);
		
		//TEMP1 = 3.*XNO2*AINV2
		//TEMP = TEMP1*ROOT22
		//D2201 = TEMP*F220*G201
		//D2211 = TEMP*F221*G211
		Float64	temp1,temp;
		
		temp1 = 3*xno2*ainv2;
		temp = temp1*ROOT22;
		d2201 = temp*f220*g201;
		d2211 = temp*f221*g211;
		
		//TEMP1 = TEMP1*AQNV
		//TEMP = TEMP1*ROOT32
		//D3210 = TEMP*F321*G310
		//D3222 = TEMP*F322*G322
		temp1 *= aqnv;
		temp = temp1*ROOT32;
		d3210 = temp*f321*g310;
		d3222 = temp*f322*g322;
		
		//TEMP1 = TEMP1*AQNV
		//TEMP = 2.*TEMP1*ROOT44
		//D4410 = TEMP*F441*G410
		//D4422 = TEMP*F442*G422
		temp1 *= aqnv;
		temp = 2*temp1*ROOT44;
		d4410 = temp*f441*g410;
		d4422 = temp*f442*g422;
		
		//TEMP1 = TEMP1*AQNV
		//TEMP = TEMP1*ROOT52
		//D5220 = TEMP*F522*G520
		//D5232 = TEMP*F523*G532
		temp1 *= aqnv;
		temp = temp1*ROOT52;
		d5220 = temp*f522*g520;
		d5232 = temp*f523*g532;
		
		//TEMP = 2.*TEMP1*ROOT54
		//D5421 = TEMP*F542*G521
		//D5433 = TEMP*F543*G533
		temp = 2*temp1*ROOT54;
		d5421 = temp*f542*g521;
		d5433 = temp*f543*g533;
		
		//XLAMO = XMAO+XNODEO+XNODEO-THGR-THGR
		//BFACT = XMDOT+XNODOT+XNODOT-THDT-THDT
		//BFACT = BFACT+SSL+SSH+SSH
		xlamo = xmao+2*xnodeo-2*thgr;
		bfact = xlldot+2*xnodot-2*THDT+ssl+2*ssh;
		
		//GO TO 80
		do_synchronous_resonance = false;
	}
	
	//* SYNCHRONOUS RESONANCE TERMS INITIALIZATION
	if (do_synchronous_resonance)
	{
		//70 
		//IRESFL=1
		//ISYNFL=1
		iresfl = true;
		isynfl = true;
		
		//G200=1.0+EQSQ*(-2.5+.8125*EQSQ)
		//G310=1.0+2.0*EQSQ
		//G300=1.0+EQSQ*(-6.0+6.60937*EQSQ)
		g200 = 1.0+eqsq*(-2.5+0.8125*eqsq);
		g310 = 1.0+2.0*eqsq;
		g300 = 1.0+eqsq*(-6.0+6.60937*eqsq);

		//F220=.75*(1.+COSIQ)*(1.+COSIQ)
		//F311=.9375*SINIQ*SINIQ*(1.+3.*COSIQ)-.75*(1.+COSIQ)
		//F330=1.+COSIQ
		//F330=1.875*F330*F330*F330
		f220 = 0.75*(1+cosiq)*(1+cosiq);
		f311 = 0.9375*std::sqr(siniq)*(1+3*cosiq)-0.75*(1+cosiq);
		f330 = 1+cosiq;
		f330 = 1.875*f330*f330*f330;

		//DEL1=3.*XNQ*XNQ*AQNV*AQNV
		//DEL2=2.*DEL1*F220*G200*Q22
		//DEL3=3.*DEL1*F330*G300*Q33*AQNV
		del1 = 3*std::sqr(xnq)*std::sqr(aqnv);
		del2 = 2*del1*f220*g200*Q22;
		del3 = 3*del1*f330*g300*Q33*aqnv;
		
		//DEL1=DEL1*F311*G310*Q31*AQNV
		del1 *= f311*g310*Q31*aqnv;

		//FASX2=.13130908
		//FASX4=2.8843198
		//FASX6=.37448087
		fasx2 = 0.13130908;
		fasx4 = 2.8843198;
		fasx6 = 0.37448087;

		//XLAMO=XMAO+XNODEO+OMEGAO-THGR
		//BFACT = XMDOT+XPIDOT-THDT
		//BFACT=BFACT+SSL+SSG+SSH
		xlamo = xmao+xnodeo+omegao-thgr;
		bfact = xlldot+xpidot-THDT+ssl+ssg+ssh;
	}
	
	//80 
	//XFACT=BFACT-XNQ
	xfact = bfact-xnq;
	
	//C INITIALIZE INTEGRATOR
	//XLI=XLAMO
	//XNI=XNQ
	xli = xlamo;
	xni = xnq;
	
	//ATIME=0.D0
	//STEPP=720.D0
	//STEPN=-720.D0
	//STEP2 = 259200.D0
	atime = 0;
	stepn = -720;
	stepp = 720;
	step2 = 259200;
	
	//RETURN
	return;
}

//* ENTRANCE FOR DEEP SPACE SECULAR EFFECTS
//ENTRY DPSEC(XLL,OMGASM,XNODES,EM,XINC,XN,T)
void SGP4SDP4Model::SDP4::DeepSpace::secular(Float64& M_DF,Float64& w_DF,Float64& W_DF,Float64& e_DF,Float64& i_DF,
											 const Float64& dt)
{
	Float64		&xll = M_DF,
				&omgasm = w_DF,
				&xnodes = W_DF,
				&em = e_DF,
				&xinc = i_DF;
	
	const Float64	eo = sdp4_.eo_,
					xincl = sdp4_.io_,
					thgr = sdp4_.thetag_,
					omgdt = sdp4_.wdot;;
	
	//XLL=XLL+SSL*T
	//OMGASM=OMGASM+SSG*T
	//XNODES=XNODES+SSH*T
	//EM=EO+SSE*T
	//XINC=XINCL+SSI*T
	xll += ssl*dt;
	omgasm += ssg*dt;
	xnodes += ssh*dt;
	em = eo+sse*dt;
	xinc = xincl+ssi*dt;
	
	//IF(XINC .GE. 0.) GO TO 90
	if (xinc < 0)
	{
		//XINC = -XINC
		//XNODES = XNODES + PI
		//OMGASM = OMGASM - PI
		xinc = -xinc;
		xnodes += _pi_;
		omgasm -= _pi_;
	}
	
	//90 
	//IF(IRESFL .EQ. 0) RETURN
	if (!iresfl) return;

	Int32	iret = 0,
			iretn = 0;
		
	//100 
	//IF (ATIME.EQ.0.D0) GO TO 170
	//IF(T.GE.(0.D0).AND.ATIME.LT.(0.D0)) GO TO 170
	//IF(T.LT.(0.D0).AND.ATIME.GE.(0.D0)) GO TO 170
	label_100:	
	
	if (atime == 0 || (dt >= 0 && atime < 0) || (dt < 0 && atime >= 0))
		goto label_170;
		 
	//105 
	//IF(DABS(T).GE.DABS(ATIME)) GO TO 120
	//DELT=STEPP
	//IF (T.GE.0.D0) DELT = STEPN
	if (std::abs(dt) < std::abs(atime))
	{ 
		delt = (dt >= 0) ? stepn : stepp;

		//110 
		//ASSIGN 100 TO IRET
		//GO TO 160
		iret = 100;
		
		goto label_160;
	}
	else
	{
		//120 
		//DELT=STEPN
		//IF (T.GT.0.D0) DELT = STEPP
		delt = (dt <= 0) ? delt = stepn : stepp;
	}
	
	while (true)
	{
		label_125:
		
		//125 
		//IF (DABS(T-ATIME).LT.STEPP) GO TO 130
		//ASSIGN 125 TO IRET
		//GO TO 160
		if (std::abs(dt-atime) < stepp) 
			goto label_130;
		
		iret = 125;
		
		goto label_160;
		
		//130 
		//FT = T-ATIME
		//ASSIGN 140 TO IRETN
		//GO TO 150
		label_130:
	
		const Float64	ft = dt-atime;
		iretn = 140;
		
		goto label_150;
		
		//140 
		//XN = XNI+XNDOT*FT+XNDDT*FT*FT*0.5
		//XL = XLI+XLDOT*FT+XNDOT*FT*FT*0.5
		//TEMP = -XNODES+THGR+T*THDT
		//XLL = XL-OMGASM+TEMP
		//IF (ISYNFL.EQ.0) XLL = XL+TEMP+TEMP
		//RETURN
		label_140:
	
		const Float64	xn = xni + xndot*ft + xnddt*std::sqr(ft)/2,
						xl = xli + xldot*ft + xndot*std::sqr(ft)/2,
						temp = -xnodes+thgr+dt*THDT;
						
		xll = xl-omgasm+temp;
		
		if (!isynfl)
			xll = xl+2*temp;
			
		return;
		
		//C DOT TERMS CALCULATED
		//150 
		//IF (ISYNFL.EQ.0) GO TO 152
		//XNDOT=DEL1*SIN (XLI-FASX2)+DEL2*SIN (2.*(XLI-FASX4))+DEL3*SIN (3.*(XLI-FASX6))
		//XNDDT = DEL1*COS(XLI-FASX2)+2.*DEL2*COS(2.*(XLI-FASX4))+3.*DEL3*COS(3.*(XLI-FASX6))
		//GO TO 154
		label_150:
	
		if (isynfl)
		{
			//152 
			//XOMI = OMEGAQ+OMGDT*ATIME
			//X2OMI = XOMI+XOMI
			//X2LI = XLI+XLI
			//XNDOT = D2201*SIN(X2OMI+XLI-G22)
			//* +D2211*SIN(XLI-G22)
			//* +D3210*SIN(XOMI+XLI-G32)
			//* +D3222*SIN(-XOMI+XLI-G32)
			//* +D4410*SIN(X2OMI+X2LI-G44)
			//* +D4422*SIN(X2LI-G44)
			//* +D5220*SIN(XOMI+XLI-G52)
			//* +D5232*SIN(-XOMI+XLI-G52)
			//* +D5421*SIN(XOMI+X2LI-G54)
			//* +D5433*SIN(-XOMI+X2LI-G54)
			//XNDDT = D2201*COS(X2OMI+XLI-G22)
			//* +D2211*COS(XLI-G22)
			//* +D3210*COS(XOMI+XLI-G32)
			//* +D3222*COS(-XOMI+XLI-G32)
			//* +D5220*COS(XOMI+XLI-G52)
			//* +D5232*COS(-XOMI+XLI-G52)
			//* +2.*(D4410*COS(X2OMI+X2LI-G44)
			//* +D4422*COS(X2LI-G44)
			//* +D5421*COS(XOMI+X2LI-G54)
			//* +D5433*COS(-XOMI+X2LI-G54))
			const Float64	xomi = omegaq+omgdt*atime,
							x2omi = 2*xomi,
							x2li = 2*xli;
			
			xndot = d2201*std::sin(x2omi+xli-G22)+
					d2211*std::sin(xli-G22)+
					d3210*std::sin(xomi+xli-G32)+
					d3222*std::sin(-xomi+xli-G32)+
					d4410*std::sin(x2omi+x2li-G44)+
					d4422*std::sin(x2li-G44)+
					d5220*std::sin(xomi+xli-G52)+
					d5232*std::sin(-xomi+xli-G52)+
					d5421*std::sin(xomi+x2li-G54)+
					d5433*std::sin(-xomi+x2li-G54);

			xnddt = d2201*std::cos(x2omi+xli-G22)+
					d2211*std::cos(xli-G22)+
					d3210*std::cos(xomi+xli-G32)+
					d3222*std::cos(-xomi+xli-G32)+
					d5220*std::cos(xomi+xli-G52)+
					d5232*std::cos(-xomi+xli-G52)+
					d4410*std::cos(x2omi+x2li-G44)*2+
					d4422*std::cos(x2li-G44)+
					d5421*std::cos(xomi+x2li-G54)+
					d5433*std::cos(-xomi+x2li-G54);
		}
		else
		{
			xndot = del1*std::sin(xli-fasx2)+del2*std::sin(2*(xli-fasx4))+del3*std::sin(3*(xli-fasx6));
			xnddt = del1*std::cos(xli-fasx2)+2*del2*std::cos(2*(xli-fasx4))+3*del3*std::cos(3*(xli-fasx6));
		}
		
		//154 
		//XLDOT=XNI+XFACT
		//XNDDT = XNDDT*XLDOT
		//GO TO IRETN
		xldot = xni+xfact;
		xnddt *= xldot;
		
		// does FORTRAN computed goto zero value of label after call? i.e. is iretn set back to 0? probably not...
		switch (iretn)
		{
			case 140 : goto label_140; break;
			case 165 : goto label_165; break;
			default  : break;
		};
				
		//C INTEGRATOR
		//160 
		//ASSIGN 165 TO IRETN
		//GO TO 150
		label_160:
		
		iretn = 165;
		
		goto label_150;
		
		//165 
		//XLI = XLI+XLDOT*DELT+XNDOT*STEP2
		//XNI = XNI+XNDOT*DELT+XNDDT*STEP2
		//ATIME=ATIME+DELT
		//GO TO IRET
		label_165:
		
		xli += xldot*delt+xndot*step2;
		xni += xndot*delt+xnddt*step2;
		
		atime += delt;
		
		// does FORTRAN computed goto zero value of label after call? i.e. is iretn set back to 0? probably not...
		switch (iret)
		{
			case 100 : goto label_100; break;
			case 125 : goto label_125; break;
			default  : break;
		};
		
		//C EPOCH RESTART
		//170 
		//IF (T.GE.0.D0) GO TO 175
		//DELT=STEPN
		//GO TO 180
		//180 
		//ATIME = 0.D0
		//XNI=XNQ
		//XLI=XLAMO
		//GO TO 125
		label_170:
		
		delt = (dt < 0) ? stepn : stepp;
		
		atime = 0;
		xni = xnq;
		xli = xlamo;
		
		// unnecessary due to while loop
		//goto label_125;
	}
	
	return;
}

//* ENTRANCES FOR LUNAR-SOLAR PERIODICS
//ENTRY DPPER(EM,XINC,OMGASM,XNODES,XLL)
void SGP4SDP4Model::SDP4::DeepSpace::periodic(Float64& M_DS,Float64& w_DS,Float64& W_DS,Float64& e_DS,Float64& i_DS,
											  const Float64& dt)
{
	Float64		&xll = M_DS,
				&omgasm = w_DS,
				&xnodes = W_DS,
				&em = e_DS,
				&xinc = i_DS;
				
	const Float64	siniq = sdp4_.sinio,
					cosiq = sdp4_.cosio,
					xqncl = sdp4_.io_;
				
	//SINIS = SIN(XINC)
	//COSIS = COS(XINC)
	const Float64	sinis = std::sin(xinc),
					cosis = std::cos(xinc);
					
	//IF (DABS(SAVTSN-T).LT.(30.D0)) GO TO 210
	if (std::abs(savtsn-dt) >= 30)
	{
		//SAVTSN=T
		//ZM=ZMOS+ZNS*T
		savtsn = dt;
		zm = zmos+ZNS*dt;
		
		//205 
		//ZF=ZM+2.*ZES*SIN (ZM)
		//SINZF=SIN (ZF)
		//F2=.5*SINZF*SINZF-.25
		//F3=-.5*SINZF*COS (ZF)
		Float64	zf,f2,f3,sinzf;
		
		zf = zm+2*ZES*std::sin(zm);
		sinzf = std::sin(zf);
		f2 = 0.5*std::sqr(sinzf)-0.25;
		f3 = -0.5*sinzf*std::cos(zf);
		
		//SES=SE2*F2+SE3*F3
		//SIS=SI2*F2+SI3*F3
		//SLS=SL2*F2+SL3*F3+SL4*SINZF
		//SGHS=SGH2*F2+SGH3*F3+SGH4*SINZF
		//SHS=SH2*F2+SH3*F3
		const Float64	ses = se2*f2+se3*f3,
						sis = si2*f2+si3*f3,
						sls = sl2*f2+sl3*f3+sl4*sinzf;
						
		sghs = sgh2*f2+sgh3*f3+sgh4*sinzf;
		shs = sh2*f2+sh3*f3;
		
		//ZM=ZMOL+ZNL*T
		Float64	zm;
		
		zm = zmol+ZNL*dt;
	
		//ZF=ZM+2.*ZEL*SIN (ZM)
		//SINZF=SIN (ZF)
		//F2=.5*SINZF*SINZF-.25
		//F3=-.5*SINZF*COS (ZF)
		zf = zm+2*ZEL*std::sin(zm);
		sinzf = std::sin(zf);
		f2 = 0.5*std::sqr(sinzf)-0.25;
		f3 = -0.5*sinzf*std::cos(zf);
		
		//SEL=EE2*F2+E3*F3
		//SIL=XI2*F2+XI3*F3
		//SLL=XL2*F2+XL3*F3+XL4*SINZF
		//SGHL=XGH2*F2+XGH3*F3+XGH4*SINZF
		//SHL=XH2*F2+XH3*F3
		const Float64	sel = ee2*f2+e3*f3,
						sil = xi2*f2+xi3*f3,
						sll = xl2*f2+xl3*f3+xl4*sinzf;
						
		sghl = xgh2*f2+xgh3*f3+xgh4*sinzf;
		shl = xh2*f2+xh3*f3;
		
		//PE=SES+SEL
		//PINC=SIS+SIL
		//PL=SLS+SLL
		pe = ses+sel;
		pinc = sis+sil;
		pl = sls+sll;
	}

	//210 
	//PGH=SGHS+SGHL
	//PH=SHS+SHL
	//XINC = XINC+PINC
	//EM = EM+PE
	Float64	pgh = sghs+sghl,
			ph = shs+shl;
					
	xinc += pinc;
	em += pe;
	
	//IF(XQNCL.LT.(.2)) GO TO 220
	//GO TO 218
	if (xqncl >= 0.2)
	{
		//C APPLY PERIODICS DIRECTLY
		//218 
		//PH=PH/SINIQ
		//PGH=PGH-COSIQ*PH
		//OMGASM=OMGASM+PGH
		//XNODES=XNODES+PH
		//XLL = XLL+PL
		//GO TO 230
		
		///////////////////////////////////////////////////////////////////////////
		// ********************************************************************* //
		// MCS 2001/11/09	-> possible error using cosiq/siniq; try cosis/sinis //
		//ph /= siniq;                                                           //
		//pgh -= cosiq*ph;                                                       //
		// ********************************************************************* //
		///////////////////////////////////////////////////////////////////////////
		ph /= sinis;
		pgh -= cosis*ph;
		
		omgasm += pgh;
		xnodes += ph;
		xll += pl;
	}
	else
	{
		//C APPLY PERIODICS WITH LYDDANE MODIFICATION
		//220 
		//SINOK=SIN(XNODES)
		//COSOK=COS(XNODES)
		const Float64	sinok = std::sin(xnodes),
						cosok = std::cos(xnodes);
						
		//ALFDP=SINIS*SINOK
		//BETDP=SINIS*COSOK
		Float64	alfdp = sinis*sinok,
				betdp = sinis*cosok;
		
		//DALF=PH*COSOK+PINC*COSIS*SINOK
		//DBET=-PH*SINOK+PINC*COSIS*COSOK
		const Float64	dalf = ph*cosok+pinc*cosis*sinok,
						dbet = -ph*sinok+pinc*cosis*cosok;
						
		//ALFDP=ALFDP+DALF
		//BETDP=BETDP+DBET
		alfdp += dalf;
		betdp += dbet;
		
		//DLS=PL+PGH-PINC*XNODES*SINIS
		//XLS = XLL+OMGASM+COSIS*XNODES
		//XLS=XLS+DLS
		const Float64	dls = pl+pgh-pinc*xnodes*sinis,
						xls = xll+omgasm+cosis*xnodes+dls;
						
		//XNODES=ACTAN(ALFDP,BETDP)
		//XLL = XLL+PL
		//OMGASM = XLS-XLL-COS(XINC)*XNODES
		xnodes = atan2mod2pi(alfdp,betdp);
		xll += pl;
		omgasm = xls-xll-std::cos(xinc)*xnodes;
	}
	
	//230 
	//CONTINUE
	//RETURN
	//END
	return;
}

} // namespace mcs