#include "MCS_SatelliteID.h"
#include "MCS_Singleton.h"

namespace mcs {

SatelliteID::SatelliteDirectory::SatelliteDirectory() :
	TwoWayDirectory<Int32,std::string>()
{
	add(0,"UNSPECIFIED");
	
	add(18123,"DMSP-F08");
	add(18822,"DMSP-F09");
	add(20978,"DMSP-F10");
	add(21798,"DMSP-F11");
	add(23233,"DMSP-F12");
	add(23533,"DMSP-F13");
	add(24753,"DMSP-F14");
	add(25991,"DMSP-F15");
	
	add(8366,"GOES-1");
	add(10061,"GOES-2");
	add(10953,"GOES-3");
	add(11964,"GOES-4");
	add(12472,"GOES-5");
	add(14050,"GOES-6");
	add(17561,"GOES-7");
	add(23051,"GOES-8");
	add(23581,"GOES-9");
	add(24786,"GOES-10");
	add(26352,"GOES-11");
	add(26871,"GOES-12");
	
	add(11060,"TIROS-N");
	add(11416,"NOAA-06");
	add(12553,"NOAA-07");
	add(13923,"NOAA-08");
	add(15427,"NOAA-09");
	add(16969,"NOAA-10");
	add(19531,"NOAA-11");
	add(21263,"NOAA-12");
	add(22739,"NOAA-13");
	add(23455,"NOAA-14");
	add(25338,"NOAA-15");
	add(26536,"NOAA-16");
	
	add(25063,"TRMM");
}

const SatelliteID::SatelliteDirectory& SatelliteID::directory_(ConstSingleton<SatelliteDirectory>::instance());

SatelliteID::SatelliteID() :
	name_(name(0)),
	ID_(0)
{ }

SatelliteID::SatelliteID(const Int32& satID) :
	name_(name(satID)),
	ID_(satID)
{ }

SatelliteID::SatelliteID(const std::string& sat_name) :
	name_(sat_name),
	ID_(number(sat_name))
{ }

SatelliteID::SatelliteID(const SatelliteID& source) :
	name_(source.name_),
	ID_(source.ID_)
{ }

std::string SatelliteID::name(const Int32& satID)
{
	return directory_[satID];
}

Int32 SatelliteID::number(const std::string& sat_name)
{
	return directory_[sat_name];
}

} // namespace mcs