#ifndef MCS_SATELLITEID_H
#define MCS_SATELLITEID_H

#include <string>

#include "MCS_Exceptions.h"
#include "MCS_TwoWayDirectory.h"
#include "MCS_Typedefs.h"

namespace mcs {

class SatelliteID
{
	public:
		SatelliteID();
		explicit SatelliteID(const Int32&);
		SatelliteID(const std::string&);
		SatelliteID(const SatelliteID&);
		~SatelliteID() { }

		Int32		number() const		{ return ID_; }
		std::string name() const		{ return name_; }
			
		static std::string	name(const Int32& sat_ID);
		static Int32		number(const std::string& sat_name);
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)

	private:
		class SatelliteDirectory : public TwoWayDirectory<Int32,std::string>
		{
			public:
				SatelliteDirectory();
		};

		Int32			ID_;
		std::string		name_;
		
		static const SatelliteDirectory& 	directory_;
};

inline bool operator==(const SatelliteID& lhs,const SatelliteID& rhs)
{
	return lhs.number() == rhs.number();
}

inline bool operator!=(const SatelliteID& lhs,const SatelliteID& rhs)
{
	return lhs.number() != rhs.number();
}

} // namespace mcs

#endif // MCS_SATELLITEID_H
