#ifndef MCS_TLE_H
#define MCS_TLE_H

#include <iosfwd>
#include <string>

#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Exceptions.h"
#include "MCS_Ordered.h"
#include "MCS_Typedefs.h"

namespace mcs {

class TLE : public Ordered<Date>
{
	public:
		// constructor/destructor
		TLE();	
		explicit TLE(const std::string&,const std::string&);	// string formatted TLE
		~TLE() { }
				
		// accessors
		const Int32& satelliteNumber() const		{ return satellite_number_; }
		
		Float64 dno_dt() const						{ return nodover2_; }
		Float64 d2no_dt2() const					{ return noddover6_; }
		Float64 bStarDrag() const					{ return bstar_; }
		Float64 eccentricity() const				{ return eo_; }
		Float64 meanMotion() const					{ return no_; }
		Float64 meanAnomaly() const					{ return Mo_; }
		Float64 inclinationAngle() const 			{ return io_; }
		Float64 longitudeOfAscendingNode() const	{ return Wo_; }
		Float64 argumentOfPerigee() const			{ return wo_; }
		
		bool interpolated() const					{ return revolution_number_ == -1; }
		
		void set_parameters(const Date& epoch,Float64 nodover2,Float64 noddover6,Float64 bstar,
		 					Float64 io,Float64 Wo,Float64 eo,Float64 wo,Float64 Mo,Float64 no);

		void print(std::ostream& os) const;
		
		//class Err : public Exception { };
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
				
	private:
		static const Int32	_TLE_STRING_LENGTH_;
	
		std::string	line1_,line2_;
		Int32		satellite_number_,
					launch_year_,
					launch_number_,
					element_number_,
					revolution_number_;
		char		classification_,
					ephemeris_type_,
					checksum1_,
					checksum2_;
		std::string	launch_component_;
		Float64		nodover2_,			// n./2
					noddover6_,			// n:/6
					bstar_,				// b_star
					eo_,				// mean eccentricity
					no_,				// mean motion
					Mo_,				// mean anomaly
					io_,				// mean inclination angle
					Wo_,				// mean longitude of ascending node
					wo_;				// mean argument of perigee
};

std::ostream& operator<<(std::ostream& ostr,const TLE& source);
std::istream& operator>>(std::istream& istr,TLE& dest);

} // namespace mcs

#endif // MCS_TLE_H