#ifndef MCS_BINARYFORMATTER_H
#define MCS_BINARYFORMATTER_H

#include "MCS_ByteSwap.h"

namespace mcs {

namespace db {

class BinaryReader
{
	public:
		BinaryReader(std::istream& is) : is_(is) { }
		~BinaryReader() { }
		
		// required methods
		void binary(const bool) { }

		void openElement(const std::string&) { }
		void closeElement() { }
		
		template<class T> void readAttribute(const std::string& attribute,T& val);
		template<class T> void readData(T& val);
		template<class T> void readData(T vec[],Int32 sz);
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		std::istream& 	is_;
};

template<class T>
inline void BinaryReader::readAttribute(const std::string&,T& val)
{
	is_.read(reinterpret_cast<char*>(&val),sizeof(T));
}

template<class T>
inline void BinaryReader::readData(T& val)
{
	is_.read(reinterpret_cast<char*>(&val),sizeof(T));
}

template<class T>
inline void BinaryReader::readData(T vec[],Int32 sz)
{
	is_.read(reinterpret_cast<char*>(&vec[0]),sz*sizeof(T));
}

class BinaryWriter
{
	public:
		BinaryWriter(std::ostream& os) : os_(os) { }
		~BinaryWriter() { }
		
		// required methods
		void binary(const bool) { }

		void openElement(const std::string&) { }
		void closeElement() { }
		
		template<class T> void writeAttribute(const std::string& attribute,const T& val);
		template<class T> void writeData(const T& val);
		template<class T> void writeData(const T vec[],Int32 sz);
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		std::ostream& 	os_;
};

template<class T>
inline void BinaryWriter::writeAttribute(const std::string&,const T& val)
{
	os_.write(reinterpret_cast<const char*>(&val),sizeof(T));
}

template<class T>
inline void BinaryWriter::writeData(const T& val)
{
	os_.write(reinterpret_cast<const char*>(&val),sizeof(T));
}

template<class T>
inline void BinaryWriter::writeData(const T vec[],Int32 sz)
{
	os_.write(reinterpret_cast<const char*>(&vec[0]),sz*sizeof(T));
}

} // namespace db

} // namespace mcs

#endif // MCS_BINARYFORMATTER_H