#ifndef MCS_PERSISTENCE_H
#define MCS_PERSISTENCE_H

#include <fstream>
#include <iostream>
#include <string>
#include <vector>

namespace mcs {

// serialize
template<class T>
class Serializer
{
	public:
		static void serialize(const T&,std::ostream&);
		static void deserialize(T&,std::istream&);
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
};

template<class T>
inline void Serializer<T>::serialize(const T& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	item.serialize(os);
}

template<class T>
inline void Serializer<T>::deserialize(T& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	//if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	item.deserialize(is);
}

template<>
inline void Serializer<Int8>::serialize(const Int8& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Int8));
}

template<>
inline void Serializer<Int8>::deserialize(Int8& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Int8));
}

template<>
inline void Serializer<Uint8>::serialize(const Uint8& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Uint8));
}

template<>
inline void Serializer<Uint8>::deserialize(Uint8& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Uint8));
}

template<>
inline void Serializer<Int16>::serialize(const Int16& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Int16));
}

template<>
inline void Serializer<Int16>::deserialize(Int16& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Int16));
}

template<>
inline void Serializer<Uint16>::serialize(const Uint16& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Uint16));
}

template<>
inline void Serializer<Uint16>::deserialize(Uint16& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Uint16));
}

template<>
inline void Serializer<Int32>::serialize(const Int32& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Int32));
}

template<>
inline void Serializer<Int32>::deserialize(Int32& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Int32));
}

template<>
inline void Serializer<Uint32>::serialize(const Uint32& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Uint32));
}

template<>
inline void Serializer<Uint32>::deserialize(Uint32& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Uint32));
}

template<>
inline void Serializer<Float32>::serialize(const Float32& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Float32));
}

template<>
inline void Serializer<Float32>::deserialize(Float32& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Float32));
}

template<>
inline void Serializer<Float64>::serialize(const Float64& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	os.write(reinterpret_cast<const char*>(&item),sizeof(Float64));
}

template<>
inline void Serializer<Float64>::deserialize(Float64& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	is.read(reinterpret_cast<char*>(&item),sizeof(Float64));
}

// STL containers
template<class T>
class Serializer< std::vector<T> >
{
	public:
		static void serialize(const std::vector<T>&,std::ostream& os);
		static void deserialize(std::vector<T>&,std::istream& is);
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
};

template<class T>
inline void Serializer< std::vector<T> >::serialize(const std::vector<T>& item,std::ostream& os)
{
	os << typeid(item).name() << std::endl;
	
	const Int32	sz = item.size();
				
	os.write(reinterpret_cast<const char*>(&sz),sizeof(Int32));
	
	for (typename std::vector<T>::const_iterator it=item.begin();it!=item.end();++it)
		os.write(reinterpret_cast<const char*>(&(*it)),sizeof(T));
}

template<class T>
inline void Serializer< std::vector<T> >::deserialize(std::vector<T>& item,std::istream& is)
{
	std::string	name;
	
	std::getline(is,name);
	
	if (typeid(item).name() != name) throw Err("Tried to deserialize "+name);
	
	Int32 sz;
				
	is.read(reinterpret_cast<char*>(&sz),sizeof(Int32));
	
	item.reserve(sz);
	
	T	val;
	
	for (Int32 i=0;i<sz;++i)
	{
		is.read(reinterpret_cast<char*>(&val),sizeof(T));
		item.push_back(val);
	}
}

class Database
{
	public:
		Database(const std::string&);
		~Database();
		
		template<class T> void store(const T& item);
		template<class T> void restore(T& item);
		
		class Err { };
		
	private:
		Database(const Database&);
		Database& operator=(const Database&);
		
		template<class T> void serialize(const T& item);
		template<class T> void deserialize(T& item);
		
		std::string		filename_;
		std::ifstream	ifs_;
		std::ofstream	ofs_;
		bool			open_for_read_,
						open_for_write_;
};

inline Database::Database(const std::string& filename) :
	filename_(filename),
	open_for_read_(false),
	open_for_write_(false)
{ }

inline Database::~Database()
{
	ifs_.close();
	ofs_.close();
}

template<class T>
inline void Database::store(const T& item)
{
	if (!open_for_write_)
	{
		if (open_for_read_) 
		{
			ifs_.close();
			open_for_read_ = false;
		}
		
		//ofs_.open(filename_.c_str(),std::ios::binary | std::ios::out | std::ios::app);
		ofs_.open(filename_.c_str(),std::ios::binary | std::ios::out | std::ios::trunc);

		if (ofs_.good())
		{
			open_for_write_ = true;
		}
		else
		{
			throw Err();
		}
	}
	
	Serializer<T>::serialize(item,ofs_);
	
	ofs_.flush();
}

template<class T>
inline void Database::restore(T& item)
{
	if (!open_for_read_)
	{
		if (open_for_write_) 
		{
			ofs_.close();
			open_for_write_ = false;
		}
		
		ifs_.open(filename_.c_str(),std::ios::binary | std::ios::in);

		if (ifs_.good())
		{
			open_for_read_ = true;
		}
		else
		{
			throw Err();
		}
	}
	
	Serializer<T>::deserialize(item,ifs_);
}

} // namespace mcs

#endif // MCS_PERSISTENCE_H