#ifndef MCS_TEXTFORMATTER_H
#define MCS_TEXTFORMATTER_H

#include "MCS_ByteSwap.h"

namespace mcs {

namespace db {

class TextReader
{
	public:
		TextReader(std::istream& is);
		~TextReader() { }
		
		void delimiter(const char c)	{ delimiter_ = c; }	
		
		// required methods
		void binary(const bool)			{ }

		void openElement(const std::string& tag);
		void closeElement();
		
		template<class T> void readAttribute(const std::string& attribute,T& val);
		template<class T> void readData(T& val);
		template<class T> void readData(T vec[],Int32 sz);
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		std::istream& 	is_;
		char			delimiter_;
};

inline TextReader::TextReader(std::istream& is) : 
	is_(is),
	delimiter_(' ')
{ }

inline void TextReader::openElement(const std::string& tag)
{
	std::string	line;
	
	is_ >> line;

	if (line != tag) throw Err();	
}

inline void TextReader::closeElement()
{ }

template<class T>
inline void TextReader::readAttribute(const std::string& attribute,T& val)
{
	std::string	line;
	
	is_ >> line;

	if (line != attribute) throw Err();	
	
	is_ >> val;
}

template<class T>
inline void TextReader::readData(T& val)
{
	is_ >> val;
}

template<class T>
inline void TextReader::readData(T vec[],Int32 sz)
{
	for (Int32 i=0;i<sz;++i)
		is_ >> vec[i];
}

class TextWriter
{
	public:
		TextWriter(std::ostream& os);
		~TextWriter() { }
		
		void delimiter(const char c)	{ delimiter_ = c; }
		
		// required methods
		void binary(const bool)			{ }

		void openElement(const std::string& tag);
		void closeElement();
		
		template<class T> void writeAttribute(const std::string& attribute,const T& val);
		template<class T> void writeData(const T& val);
		template<class T> void writeData(const T vec[],Int32 sz);
				
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		std::ostream& 	os_;
		char			delimiter_;
};

inline TextWriter::TextWriter(std::ostream& os) : 
	os_(os),
	delimiter_(' ')
{ }

inline void TextWriter::openElement(const std::string& tag)
{
	os_ << tag << std::endl;
}

inline void TextWriter::closeElement()
{ }

template<class T>
inline void TextWriter::writeAttribute(const std::string& attribute,const T& val)
{
	os_ << attribute << delimiter_ << val << std::endl;
}

template<class T>
inline void TextWriter::writeData(const T& val)
{
	os_ << val << std::endl;
}

template<class T>
inline void TextWriter::writeData(const T vec[],Int32 sz)
{
	for (Int32 i=0;i<sz;++i)
		os_ << vec[i] << delimiter_;
			
	os_ << std::endl;
}

} // namespace db

} // namespace mcs

#endif // MCS_TEXTFORMATTER_H