#ifndef MCS_FETCHCONTAINER_H
#define MCS_FETCHCONTAINER_H

#include "MCS_FetchIterator.h"

namespace mcs {

template<class DS> class FetchContainer
{
	public:
		typedef FetchContainer<DS>					this_type;
		typedef DS									dataset_type;
		
		typedef typename dataset_type::value_type	value_type;
		typedef std::ptrdiff_t						difference_type;
		typedef value_type*							pointer;
		typedef const value_type*					const_pointer;
		typedef value_type&							reference;
		typedef const value_type&					const_reference;
		
		typedef typename value_type::iterator_value_type	iterator_value_type;
			
		FetchContainer(const dataset_type&);
		~FetchContainer() { }
		
		value_type& fetch(iterator_value_type iterator_value) const;
		
		typedef fetch_iterator<this_type>			iterator;
		typedef const_fetch_iterator<this_type>		const_iterator;
		
		typedef typename boost::reverse_iterator_generator<iterator>::type			reverse_iterator;
		typedef typename boost::reverse_iterator_generator<const_iterator>::type	const_reverse_iterator;
		
		iterator begin()											{ return iterator(*this,dataset_.beginIteratorValue()); }
		iterator end()												{ return iterator(*this,dataset_.endIteratorValue()); }
		
		const_iterator begin() const								{ return const_iterator(*this,dataset_.beginIteratorValue()); }
		const_iterator end() const									{ return const_iterator(*this,dataset_.endIteratorValue()); }
		
		reverse_iterator rbegin()									{ return reverse_iterator(end()); }
		reverse_iterator rend()										{ return reverse_iterator(begin()); }
		
		const_reverse_iterator rbegin() const						{ return const_reverse_iterator(end()); }
		const_reverse_iterator rend() const							{ return const_reverse_iterator(begin()); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	private:
		FetchContainer(const this_type&);
		this_type& operator=(const this_type&);
		
		dataset_type				dataset_;
		mutable value_type			element_;
		mutable iterator_value_type	last_fetched_;
};

template<class DS>
inline FetchContainer<DS>::FetchContainer(const dataset_type& dataset) :
	dataset_(dataset),
	element_(),
	last_fetched_(dataset_.endIteratorValue())
{ }

template<class DS>
inline typename FetchContainer<DS>::value_type& 
FetchContainer<DS>::fetch(iterator_value_type iterator_value) const	
{ 
	if (iterator_value != last_fetched_) 
	{
		dataset_.fetch(element_,iterator_value); 
		last_fetched_ = iterator_value;
	}
		
	return element_;
}

} // namespace mcs

#endif // MCS_FETCHCONTAINER_H