#ifndef MCS_SSMIEDRMAP_H
#define MCS_SSMIEDRMAP_H

#include "MCS_Cache.h"
#include "MCS_Date.h"
#include "MCS_DateUtilities.h"
#include "MCS_Exceptions.h"
#include "MCS_SatelliteID.h"
#include "MCS_Tensor.h"
#include "MCS_Trace.h"
#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

template<class DS> class Map : 
	public Tensor<Float32,2>
{
	public:
		typedef Map<DS>	this_type;
		typedef DS		dataset_type;
		
		typedef typename dataset_type::iterator_value_type	iterator_value_type;
		
		Map() : Tensor<Float32,2>(dataset_type::range())							{ epoch_ = Date(); }
		Map(iterator_value_type iter) : Tensor<Float32,2>(dataset_type::range())	{ epoch_ = dataset_type::dateFromIterator(iter); }
		
		const Date& epoch() const													{ return epoch_; }
		iterator_value_type interval() const										{ return dataset_type::iteratorFromDate(epoch_); }
		
	private:
		Map(const this_type&);
		//this_type& operator=(const this_type&);
		
		Date	epoch_;
};

class SSMIEDRMap
{
	public:
		typedef Map<SSMIEDRMap>	value_type;
		typedef Int32				iterator_value_type;		// half day number
		
		SSMIEDRMap(const SatelliteID& satID,const std::string& EDR);
		~SSMIEDRMap() { }
		
		static IndexRange<2> range()									{ return IndexRange<2>(Index<2>(LAT_RANGE,LON_RANGE)); }
		
		static iterator_value_type iteratorFromDate(const Date& ep)		{ return halfDayNumber(ep); }
		static Date dateFromIterator(iterator_value_type iter)			{ return std::even(iter) ? Date(Day(iter/2),Time(6,0,0)) : Date(Day(iter/2),Time(18,0,0)); }

		iterator_value_type beginIteratorValue() const 					{ return begin_iterator_value_; }
		iterator_value_type endIteratorValue() const					{ return end_iterator_value_; }		

		void fetch(value_type& element,iterator_value_type iterator_value) const;
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		enum { CLOUD_WATER = 0, COLUMN_VAPOR, OBSERVATION_TIME, RAIN_RATE, WIND_SPEED };
		
		static const Int32 	LAT_RANGE,
							LON_RANGE,
							MAP_SIZE,
							NUM_EDRS; 
							
		static const FixedVector<Int32,5>	MAP_OFFSETS;
		static const FixedVector<Float32,5>	EDR_SCALE_FACTORS,
											EDR_OFFSETS;
							
		static Cache< iterator_value_type,Vector<Uint8> >	cache_;
		
		std::string getEDRFilename(const Date& ep) const;
		
		SatelliteID			satID_;
		std::string			satname_;
		Int32				satnum_,
							EDR_;
		std::string			EDR_directory_;
		Date				begin_date_,
							end_date_;
		iterator_value_type	begin_iterator_value_,
							end_iterator_value_;
};

} // namespace mcs

#endif // MCS_SSMIEDRMAP_H
