#ifndef MCS_BASICSWATHDATA_H
#define MCS_BASICSWATHDATA_H

#include "MCS_Exceptions.h"
#include "MCS_Matrix.h"
#include "MCS_Typedefs.h"

namespace mcs {

namespace sat {

//	policy classes for swath data types
template<class T>
class BasicSwathData
{
	public:
		const IndexRange<1>& channelRange() const								{ return data_.range().rank(1); }
		
		// accessors
		const Matrix<T>& data() const											{ return data_; }
		const T& data(const Index<2>& idx) const								{ return data_(idx); }
		const T& data(Int32 chidx,Int32 fovidx) const							{ return data_(chidx,fovidx); }
		
		// mutators
		void data(const T& val)													{ data_ = val; }
		void data(const Matrix<T>& dt)											{ if (dt.range() == data_.range()) data_ = dt; else throw Err(); }
		void data(const Index<2>& idx,const T& val)								{ data_(idx) = val; }
		void data(Int32 chidx,Int32 fovidx,const T& val)						{ data_(chidx,fovidx) = val; }
		
		class Err : public Exception { };
		
	protected:
		BasicSwathData(const Int32& num_chan,const Int32& num_fovs);
		BasicSwathData(const BasicSwathData<T>&);
		virtual ~BasicSwathData() { }
		
		Matrix<T>	data_;
		
	private:
		BasicSwathData<T>& operator=(const BasicSwathData<T>&);
};

template<class T>
inline BasicSwathData<T>::BasicSwathData(const Int32& num_chan,const Int32& num_fovs) :
	data_(IndexRange<2>(Index<2>(1,1),Index<2>(num_chan,num_fovs)),_NaN_)
{ }

template<class T>
inline BasicSwathData<T>::BasicSwathData(const BasicSwathData<T>& source) :
	data_(source.data_)
{ }

// 	policy class encapsulating implementation specific swath data
//
//		SatelliteSwathData specializations should be templatized
//			on a specific Dataset<Inst,Lev,Ver>
//
template<class DS> class SatelliteSwathData { };

} // namespace sat

} // namespace mcs

#endif // MCS_BASICSWATHDATA_H