#include <boost/iterator_adaptors.hpp>

#include "MCS_FetchIterator.h"

namespace mcs {

// DS class must have static members as follows:
//
//		const Date& beginDate()
//		const Date& endDate()
//
template<class TT,class DS,class F> class TensorSeries
{
	public:
		typedef TensorSeries<TT>	this_type;
		typedef TT					tensor_type;
		typedef DS					dataset_type;
		typedef F					interval_functor_type;
		
		typedef Int32				iterator_value_type;
		
		typedef tensor_type			value_type;
		typedef std::ptrdiff_t		difference_type;
		typedef tensor_type*		pointer;
		typedef const tensor_type*	const_pointer;
		typedef tensor_type&		reference;
		typedef const tensor_type&	const_reference;
			
		virtual ~TensorSeries() { }
		
		const Date& epoch() const	{ return epoch_; }
		Int32 interval() const		{ return interval_functor_(epoch_); }
		
		value_type& fetch(iterator_value_type interval_number);
		
		typedef reference_iterator<this_type,iterator_value_type>			iterator;
		typedef const_reference_iterator<this_type,iterator_value_type>		const_iterator;
		
		typedef typename boost::reverse_iterator_generator<iterator>::type			reverse_iterator;
		typedef typename boost::reverse_iterator_generator<const_iterator>::type	const_reverse_iterator;
		
		iterator begin()							{ return iterator(*this,interval_functor_(dataset_type::beginDate())); }
		iterator end()								{ return iterator(*this,interval_functor_(dataset_type::endDate())); }
		
		const_iterator begin() const				{ return const_iterator(*this,interval_functor_(dataset_type::beginDate())); }
		const_iterator end() const					{ return const_iterator(*this,interval_functor_(dataset_type::endDate())); }
		
		reverse_iterator rbegin()					{ return const_reverse_iterator(end()); }
		reverse_iterator rend()						{ return const_reverse_iterator(begin()); }
		
		const_reverse_iterator rbegin() const		{ return const_reverse_iterator(end()); }
		const_reverse_iterator rend() const			{ return const_reverse_iterator(begin()); }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
	
	protected:
		tensor_type				tensor_;
		interval_functor_type	interval_functor_;	
		Date					epoch_;
		
	private:
		TensorSeries(const this_type&);
		this_type& operator=(const this_type&);
};

} // namespace mcs
