#ifndef MCS_MAPSERIES_H
#define MCS_MAPSERIES_H

#include "MCS_EarthMap.h"
#include "MCS_Metadata.h"
#include "MCS_OrderedSeries.h"

namespace mcs {

template<class MT = geo::EarthMap<Float32> > class MapSeries : 
	public OrderedSeries<MT,Date>
{
	public:
		typedef MT					MapType;
		typedef MT::CoordinateType	CoordinateType;
		typedef MT::GridType		GridType;
		typedef MT::OrdinateType	OrdinateType;
		typedef MT::ValueType		ValueType;
		
		MapSeries(const std::string&);
		virtual ~MapSeries();
		
		Metadata& metadata()										{ return meta_; }
		const Metadata& metadata() const							{ return meta_; }
		
		//std::string& metadata(const std::string& str)				{ return meta_[str]; }
		const std::string& metadata(const std::string& str)	const	{ return meta_[str]; }
		
		const Date& find(const Date&);
		
		const Date& first();
		const Date& last();
		
		const Date& next(const Int32& = 1);
		const Date& prev(const Int32& = 1);
		
		const MapType& currentValue();
		
		const Date& epoch() const	{ return value(); }
		
		class Err { };
		
	protected:	
		virtual Date findNearestInterval(const Date&) const = 0;
		virtual Date incrementInterval(Uint32 num) const = 0;
		virtual void read() = 0;
		
		void epoch(const Date& ep)	{ value(ep); }
		
		void setEpoch(const Date&);
	
		Metadata	meta_;
		MapType		curr_;
};

template<class MT> inline MapSeries<MT>::MapSeries(const std::string& metainit) :
	meta_(metainit)
{ }

template<class MT> inline MapSeries<MT>::~MapSeries()
{ }

template<class MT> inline void MapSeries<MT>::setEpoch(const Date& ep)
{
	if (ep < begin() || ep > end()) throw Err();
	
	epoch(ep);
}

template<class MT> inline const Date& MapSeries<MT>::find(const Date& ep)
{
	setEpoch(findNearestInterval(ep));
	
	return epoch();
}

template<class MT> inline const Date& MapSeries<MT>::first()
{
	return find(begin());
}

template<class MT> inline const Date& MapSeries<MT>::last()
{
	return find(end());
}

template<class MT> inline const Date& MapSeries<MT>::next(const Int32& num)
{
	return find(incrementInterval(num));
}
	
template<class MT> inline const Date& MapSeries<MT>::prev(const Int32& num)
{
	return find(incrementInterval(-num));
}
	
template<class MT> inline const typename MapSeries<MT>::MapType& MapSeries<MT>::currentValue()
{
	read();
	
	return curr_;
}

} // namespace mcs

#endif // MCS_MAPSERIES_H