#ifndef MCS_ORDEREDSERIES_H
#define MCS_ORDEREDSERIES_H

#include "MCS_Date.h"
#include "MCS_EarthMap.h"
#include "MCS_Ordered.h"

namespace mcs {

// series of T's, ordered by D
template<class T,class D> class OrderedSeries : public Ordered<D>
{
	public:
		virtual ~OrderedSeries() { }
		
		// beginning and ending ordinal values in series
		virtual D begin() const = 0;
		virtual D end() const = 0;
		
		// current ordinal value
		const D& curr() const					{ return value(); }
		
		// set current data pointer to element in series with ordinal nearest to argument
		virtual const D& find(const D&) = 0;
		
		// set current data pointer to first/last element in series
		virtual const D& first() = 0;
		virtual const D& last() = 0;
		
		// set current data pointer to element in series offset by increment/decrement
		virtual const D& prev(const Int32& = 1) = 0;
		virtual const D& next(const Int32& = 1) = 0;
		
		// access current data pointer
		virtual const T& currentValue() = 0;
};

template<class T,class D> T interpolate(OrderedSeries<T,D>& series,const D& x)
{	
	const D	x1(series.find(val));
	
	if (x1 > x)
	{
		const D	x2(series.prev());
		const T	val2(series.currentValue());
		
		series.next();
		
		const ScalarValueType<T>::scalar_t	fac = (x-x2)/(x1-x2);
		
		return (1-fac)*val2+fac*series.currentValue();
	}
	else if (x1 < x)
	{
		const D	x2(series.next());
		const T	val2(series.currentValue());
		
		series.prev();
				
		const ScalarValueType<T>::scalar_t	fac = (x-x1)/(x2-x1);
		
		return (1-fac)*series.currentValue()+fac*val2;
	}
	else
	{
		return series.currentValue();
	}	
}

template<class T> T interpolate(OrderedSeries<T,Date>& series,const Date& ep)
{	
	const Date	ep1(series.find(ep));
	
	if (ep1 > ep)
	{
		const Date	ep2(series.prev());
		const T		val2(series.currentValue());
		
		series.next();
		
		const ScalarValueType<T>::scalar_t	fac = (ep.julian()-ep2.julian())/(ep1.julian()-ep2.julian());
		
		return (1-fac)*val2+fac*series.currentValue();
	}
	else if (ep1 < ep)
	{
		const Date	ep2(series.next());
		const T		val2(series.currentValue());
		
		series.prev();
				
		const ScalarValueType<T>::scalar_t	fac = (ep.julian()-ep1.julian())/(ep2.julian()-ep1.julian());
		
		return (1-fac)*series.currentValue()+fac*val2;
	}
	else
	{
		return series.currentValue();
	}	
}
/*
template<class T> class OrderedSeries : public TimeOrdered
{
	public:
		virtual ~OrderedSeries() { }
		
		virtual Date begin() const = 0;
		virtual Date end() const = 0;
		
		// set epoch
		virtual const Date& first() = 0;
		virtual const Date& last() = 0;
		virtual const Date& find(const Date&) = 0;
		virtual const Date& next(Uint32 = 1) = 0;
		virtual const Date& prev(Uint32 = 1) = 0;
		
		// access data
		virtual const T& curr() = 0;
};

template<class T> T interpolate(OrderedSeries<T>& series,const Date& ep)
{	
	const Date	ep1(series.find(ep));
	
	if (ep1 > ep)
	{
		const Date	ep2(series.prev());
		const T		val2(series.curr());
		
		series.next();
		
		const ScalarValueType<T>::scalar_t	fac = (ep.julian()-ep2.julian())/(ep1.julian()-ep2.julian());
		
		return (1-fac)*val2+fac*series.curr();
	}
	else if (ep1 < ep)
	{
		const Date	ep2(series.next());
		const T		val2(series.curr());
		
		series.prev();
				
		const ScalarValueType<T>::scalar_t	fac = (ep.julian()-ep1.julian())/(ep2.julian()-ep1.julian());
		
		return (1-fac)*series.curr()+fac*val2;
	}
	else
	{
		return series.curr();
	}	
}
*/
} // namespace mcs

#endif // MCS_ORDEREDSERIES_H