#ifndef MCS_REFERENCEITERATOR_H
#define MCS_REFERENCEITERATOR_H

#include <iterator>

namespace mcs {

template<class T,class V>
class reference_iterator : 
	public std::iterator<std::random_access_iterator_tag,T,V,T*,T&>
{
	public:
		typedef reference_iterator<T,V>	this_type;
		typedef V						iterator_value_type;
		
		typedef T						value_type;
		typedef V						difference_type;
		typedef value_type&				reference;
		typedef const value_type&		const_reference;
		typedef value_type*				pointer;
		typedef const value_type*		const_pointer;
		
		reference_iterator(value_type& ref,iterator_value_type iter) : ref_(ref),iter_(iter) { }
		reference_iterator(const this_type& source) : ref_(source.ref_),iter_(source.iter_) { }
		~reference_iterator() { }
				
		this_type& operator=(const this_type& it)								{ if (this == &it) return *this; ref_=it.ref_; iter_=it.iter_; return *this; }
		
		bool operator==(const this_type& it) const								{ return iter_ == it.iter_; }
		bool operator!=(const this_type& it) const								{ return iter_ != it.iter_; }
		bool operator<(const this_type& it) const								{ return iter_ < it.iter_; }
		bool operator<=(const this_type& it) const								{ return iter_ <= it.iter_; }
		bool operator>(const this_type& it) const								{ return iter_ > it.iter_; }
		bool operator>=(const this_type& it) const								{ return iter_ >= it.iter_; }
		
		this_type& operator+=(iterator_value_type num)							{ iter_ += num; return *this; }
		this_type& operator-=(iterator_value_type num)							{ iter_ -= num; return *this; }
		
		difference_type operator-(const this_type& it) const					{ return iter_-it.iter_; }
		
		this_type& operator++() 												{ ++iter_; return *this; }
		this_type& operator--()													{ --iter_; return *this; }
		const this_type operator++(int)											{ this_type tmp(*this); ++iter_; return tmp; }
		const this_type operator--(int)											{ this_type tmp(*this); --iter_; return tmp; }
		
		this_type operator+(iterator_value_type num) const						{ this_type tmp(*this); tmp += num; return tmp; }
		this_type operator-(iterator_value_type num) const						{ this_type tmp(*this); tmp -= num; return tmp; }
		
		reference operator*()													{ return ref_.get(iter_); }
		reference operator[](iterator_value_type num)							{ iter_ = num; return ref_.get(iter_); }
		
		pointer operator->()													{ return &ref_.get(iter_); }
		
		value_type& object() const												{ return ref_; }
		iterator_value_type value() const										{ return iter_; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		value_type&				ref_;
		iterator_value_type		iter_;
};

template<class T,class V>
class const_reference_iterator : 
	public std::iterator<std::random_access_iterator_tag,T,V,const T*,const T&>
{
	public:
		typedef const_reference_iterator<T,V>	this_type;
		typedef V								iterator_value_type;
		
		typedef T								value_type;
		typedef V								difference_type;
		typedef value_type&						reference;
		typedef const value_type&				const_reference;
		typedef value_type*						pointer;
		typedef const value_type*				const_pointer;
		
		typedef reference_iterator<T,V>			reference_iterator_type;
				
		const_reference_iterator(const value_type& ref,iterator_value_type iter) : ref_(const_cast<value_type&>(ref)),iter_(iter) { }
		const_reference_iterator(const reference_iterator_type& it) : ref_(it.object()),iter_(it.value()) { }
		const_reference_iterator(const this_type& source) : ref_(source.ref_),iter_(source.iter_) { }
		~const_reference_iterator() { }
				
		this_type& operator=(const reference_iterator_type& it)					{ const_cast<reference_type&>(ref_)=it.object();iter_=it.value(); }
		this_type& operator=(const this_type& it)								{ if (this == &it) return *this; const_cast<value_type&>(ref_)=it.ref_; iter_=it.iter_; return *this; }
		
		bool operator==(const this_type& it) const								{ return iter_ == it.iter_; }
		bool operator!=(const this_type& it) const								{ return iter_ != it.iter_; }
		bool operator<(const this_type& it) const								{ return iter_ < it.iter_; }
		bool operator<=(const this_type& it) const								{ return iter_ <= it.iter_; }
		bool operator>(const this_type& it) const								{ return iter_ > it.iter_; }
		bool operator>=(const this_type& it) const								{ return iter_ >= it.iter_; }
		
		this_type&	operator+=(iterator_value_type num)							{ iter_ += num; return *this; }
		this_type&	operator-=(iterator_value_type num)							{ iter_ -= num; return *this; }
		
		difference_type operator-(const this_type& it) const					{ return iter_-it.iter_; }
		
		this_type&	operator++() 												{ ++iter_; return *this; }
		this_type&	operator--()												{ --iter_; return *this; }
		const this_type operator++(int)											{ this_type tmp(*this); ++iter_; return tmp; }
		const this_type operator--(int)											{ this_type tmp(*this); --iter_; return tmp; }
		
		this_type operator+(iterator_value_type num) const						{ this_type tmp(*this); tmp += num; return tmp; }
		this_type operator-(iterator_value_type num) const						{ this_type tmp(*this); tmp -= num; return tmp; }
		
		const_reference operator*()	const										{ return ref_.get(iter_); }
		const_reference operator[](const iterator_value_type& num)				{ iter_ = num; return ref_.get(iter_); }
		
		const_pointer operator->() const										{ return &ref_.get(iter_); }
		
		const value_type& object() const										{ return ref_; }
		iterator_value_type value() const										{ return iter_; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		const value_type&		ref_;
		iterator_value_type		iter_;
};

} // namespace mcs

#endif // MCS_REFERENCEITERATOR_H