#ifndef MCS_SATELLITEDATASET_H
#define MCS_SATELLITEDATASET_H

namespace mcs {

namespace sat {

// tags for different data set types
class MapData { };
class SwathData { };
class TimeSeriesData { };

// dataset policy class encapsulating static information on dataset 
//
//		Inst will normally be a placeholder like SSMI<SwathData>,
//			encapsulating information on the platform, 
//			specific instrument, and data type
//
//		Lev will normally be a placeholder Level0/Level1A etc...
//			representing the processing level of the data, along
//			with information on calibration algorithms and other
//			processing-specific data
//
//		Ver will normally be a placeholder v1/v2 etc..
//			representing the data version number
//
template<class OT> class Dataset;

template<class DS> class Inventory;

template<class DS> class MapIterator;
template<class DS> class OrbitIterator;

template<class DS> class Map;
template<class DS> class Orbit;

//	top-level swath data class with policies for metadata, geolocation, and swath data
//
//		class Swath : 	public Ordered<Date>,
//						public SatelliteMetadata<DS>,
//						public SatelliteGeolocation<DS>,
//						public SatelliteSwathData<DS> { };
template<class DS> class Swath;

} // namespace sat

} // namespace mcs

#endif // MCS_SATELLITEDATASET_H