#ifndef MCS_SATELLITEMAPITERATOR_H
#define MCS_SATELLITEMAPITERATOR_H

#include "MCS_Exceptions.h"
#include "MCS_Iterator.h"
#include "MCS_SatelliteDataset.h"

namespace mcs {

namespace sat {

// should be specialized for individual datasets
template<class DS> class MapIteratorImp;

} // namespace sat

// specialize Dereferencer for Map data
template<class DS> 
class Dereferencer< sat::Map<DS>,sat::MapIteratorImp<DS> >
{
	public:
		typedef sat::Map<DS>	MapType;
		
		Dereferencer();
		explicit Dereferencer(const sat::MapIteratorImp<DS>& imp);
		Dereferencer(const Dereferencer< MapType,sat::MapIteratorImp<DS> >& source);
		~Dereferencer() { }
		
		MapType* operator->()		{ return ptr_.operator->(); }
		
		const MapType& dereference(const sat::MapIteratorImp<DS>& imp);
		
	private:
		bool					initialized_;
		sat::MapIteratorImp<DS>	imp_;			
		std::auto_ptr<MapType>	ptr_;
};

template<class DS>
inline Dereferencer< sat::Map<DS>,sat::MapIteratorImp<DS> >::Dereferencer() :
	initialized_(false),
	imp_(),
	ptr_()
{ }

template<class DS>
inline Dereferencer< sat::Map<DS>,sat::MapIteratorImp<DS> >::Dereferencer(const sat::MapIteratorImp<DS>& imp) :
	initialized_(false),
	imp_(imp),
	ptr_()
{ }

template<class DS>
inline Dereferencer< sat::Map<DS>,sat::MapIteratorImp<DS> >::Dereferencer(const Dereferencer<sat::Map<DS>,sat::MapIteratorImp<DS> >& source) :
	initialized_(false),	
	imp_(source.imp_),
	ptr_()
{ }

template<class DS>
inline const sat::Map<DS>& Dereferencer< sat::Map<DS>,sat::MapIteratorImp<DS> >::dereference(const sat::MapIteratorImp<DS> & imp) 
{
	// if new imp, get new data, otherwise just return existing
	if (initialized_ == false || imp_ != imp) 
	{
		imp_ = imp;

		if (initialized_ == false) 
		{
			ptr_ = std::auto_ptr<MapType>(new MapType());
			initialized_ = true;
		}
		
		ptr_->getMap(imp_);
	}
	
	return *ptr_;
}

} // namespace mcs

#endif // MCS_SATELLITEMAPITERATOR_H