#ifndef MCS_SATELLITEORBITITERATOR_H
#define MCS_SATELLITEORBITITERATOR_H

#include "MCS_Exceptions.h"
#include "MCS_Iterator.h"
#include "MCS_SatelliteDataset.h"
#include "MCS_SatelliteOrbit.h"

namespace mcs {

namespace sat {

// should be specialized for individual datasets
template<class DS> class OrbitIteratorImp { };

} // namespace sat

// specialize Dereferencer for Orbit data
template<class DS> 
class Dereferencer< sat::Orbit<DS>,sat::OrbitIteratorImp<DS> >
{
	public:
		typedef sat::Orbit<DS>	OrbitType;
		
		Dereferencer();
		explicit Dereferencer(const sat::OrbitIteratorImp<DS>&);
		Dereferencer(const Dereferencer< OrbitType,sat::OrbitIteratorImp<DS> >& source);
		~Dereferencer() { }
		
		OrbitType* operator->()		{ return ptr_.operator->(); }
		
		const OrbitType& dereference(const sat::OrbitIteratorImp<DS>& imp);
		
	private:
		bool						initialized_;
		sat::OrbitIteratorImp<DS>	imp_;			
		std::auto_ptr<OrbitType>	ptr_;
};

template<class DS>
inline Dereferencer< sat::Orbit<DS>,sat::OrbitIteratorImp<DS> >::Dereferencer() :
	initialized_(false),
	imp_(),
	ptr_()
{ }

template<class DS>
inline Dereferencer< sat::Orbit<DS>,sat::OrbitIteratorImp<DS> >::Dereferencer(const sat::OrbitIteratorImp<DS>& imp) :
	initialized_(false),
	imp_(imp),
	ptr_()
{ }

template<class DS>
inline Dereferencer< sat::Orbit<DS>,sat::OrbitIteratorImp<DS> >::Dereferencer(const Dereferencer<OrbitType,sat::OrbitIteratorImp<DS> >& source) :
	initialized_(false),	
	imp_(source.imp_),
	ptr_()
{ }

template<class DS>
inline const sat::Orbit<DS>& Dereferencer< sat::Orbit<DS>,sat::OrbitIteratorImp<DS> >::dereference(const sat::OrbitIteratorImp<DS> & imp) 
{
	// if new imp, get new data, otherwise just return existing
	if (initialized_ == false || imp_ != imp) 
	{
		imp_ = imp;

		if (initialized_ == false) 
		{
			ptr_ = std::auto_ptr<OrbitType>(new OrbitType());
			initialized_ = true;
		}
		
		ptr_->getOrbit(imp_);
	}
	
	return *ptr_;
}

} // namespace mcs

#endif // MCS_SATELLITEORBITITERATOR_H