#ifndef MCS_SATELLITESWATHITERATOR_H
#define MCS_SATELLITESWATHITERATOR_H

#include "MCS_Exceptions.h"
#include "MCS_Iterator.h"
#include "MCS_SatelliteDataset.h"

namespace mcs {

namespace sat {

// should be specialized for individual datasets
template<class DS> class SwathIteratorImp { };

} // namespace sat

// specialize Dereferencer for Swath data
template<class DS> 
class Dereferencer< sat::Swath<DS>,sat::SwathIteratorImp<DS> >
{
	public:
		typedef sat::Swath<DS>	SwathType;
		
		Dereferencer();
		explicit Dereferencer(const sat::SwathIteratorImp<DS>&);
		Dereferencer(const Dereferencer< SwathType,sat::SwathIteratorImp<DS> >& source);
		~Dereferencer() { }
		
		SwathType* operator->()		{ return ptr_.operator->(); }
		
		const SwathType& dereference(const sat::SwathIteratorImp<DS>& imp);
		
	private:
		bool						initialized_;
		sat::SwathIteratorImp<DS> 	imp_;			
		std::auto_ptr<SwathType>	ptr_;
};

template<class DS>
inline Dereferencer< sat::Swath<DS>,sat::SwathIteratorImp<DS> >::Dereferencer() :
	initialized_(false),
	imp_(),
	ptr_()
{ }

template<class DS>
inline Dereferencer< sat::Swath<DS>,sat::SwathIteratorImp<DS> >::Dereferencer(const sat::SwathIteratorImp<DS>& imp) :
	initialized_(false),
	imp_(imp),
	ptr_()
{ }

template<class DS>
inline Dereferencer< sat::Swath<DS>,sat::SwathIteratorImp<DS> >::Dereferencer(const Dereferencer<sat::Swath<DS>,sat::SwathIteratorImp<DS> >& source) :
	initialized_(false),
	imp_(source.imp_),
	ptr_()
{ }

template<class DS>
inline const sat::Swath<DS>& Dereferencer< sat::Swath<DS>,sat::SwathIteratorImp<DS> >::dereference(const sat::SwathIteratorImp<DS>& imp) 
{
	// if new imp, get new data, otherwise just return existing
	if (initialized_ == false || imp_ != imp) 
	{
		imp_ = imp;

		if (initialized_ == false) 
		{
			ptr_ = std::auto_ptr<SwathType>(new SwathType(imp_.dataset()));
			initialized_ = true;
		}
		
		imp_.orbit().getSwath(*ptr_,imp_);
	}
	
	return *ptr_;
}

} // namespace mcs

#endif // MCS_SATELLITESWATHITERATOR_H