// MSU Level 0 SatelliteMetadata
template<class Ver> 
inline SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >::SatelliteMetadata(const DatasetType& ds) :
	Dataset< MSU<SwathData,msu::Level0,Ver> >(ds),
	QC_(0xFFFFFFFF),
	FOV_QC_(IndexRange<1>(1,msu::NUM_SCAN_POSITIONS),0xFF),
	asc_(false),
	swath_number_(-1),
	local_zenith_angle_(_NaN_),
	earth_location_delta_(_NaN_),
	set_coef_(-1),
	serial_number_(-1),
	E_cal_lo_(_NaN_),
	F_cal_hi_(_NaN_),
	minus15_plus5_V_(_NaN_),
	plus5_V_(_NaN_),
	E_zero_(_NaN_),
	T_target1_(_NaN_,_NaN_),
	T_target2_(_NaN_,_NaN_),
	T1_cal_(_NaN_,_NaN_),
	T2_cal_(_NaN_,_NaN_),
	other_temp_(_NaN_,_NaN_),
	xtal1_(_NaN_,_NaN_),
	xtal2_(_NaN_,_NaN_),
	xtal3_(_NaN_,_NaN_),
	xtal4_(_NaN_,_NaN_),
	antenna_bearing_temp_(_NaN_,_NaN_),
	motor_temp_(_NaN_,_NaN_),
	RF_chassis_temp_(_NaN_,_NaN_),
	space_view_(_NaN_),
	blackbody_view_(_NaN_),
	scan_to_spot1_view_(_NaN_),
	LO_temp_(_NaN_),
	Dicke_load_temp_(_NaN_),
	prog_temp_(_NaN_)
{ }

template<class Ver> 
inline SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >::SatelliteMetadata(const SatelliteMetadata<DatasetType>& source) :
	Dataset< MSU<SwathData,msu::Level0,Ver> >(source),
	QC_(source.QC_),
	FOV_QC_(source.FOV_QC_),
	asc_(source.asc_),
	swath_number_(source.swath_number_),
	local_zenith_angle_(source.local_zenith_angle_),
	earth_location_delta_(source.earth_location_delta_),
	set_coef_(source.set_coef_),
	serial_number_(source.serial_number_),
	E_cal_lo_(source.E_cal_lo_),
	F_cal_hi_(source.F_cal_hi_),
	minus15_plus5_V_(source.minus15_plus5_V_),
	plus5_V_(source.plus5_V_),
	E_zero_(source.E_zero_),
	T_target1_(source.T_target1_),
	T_target2_(source.T_target2_),
	T1_cal_(source.T1_cal_),
	T2_cal_(source.T2_cal_),
	other_temp_(source.other_temp_),
	xtal1_(source.xtal1_),
	xtal2_(source.xtal2_),
	xtal3_(source.xtal3_),
	xtal4_(source.xtal4_),
	antenna_bearing_temp_(source.antenna_bearing_temp_),
	motor_temp_(source.motor_temp_),
	RF_chassis_temp_(source.RF_chassis_temp_),
	space_view_(source.space_view_),
	blackbody_view_(source.blackbody_view_),
	scan_to_spot1_view_(source.scan_to_spot1_view_),
	LO_temp_(source.LO_temp_),
	Dicke_load_temp_(source.Dicke_load_temp_),
	prog_temp_(source.prog_temp_)
{ }

template<class Ver> 
inline bool SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >::passedQC() const
{
	return !(QC().test(NO_EARTH_LOCATION) || QC().test(FATAL) || QC().test(BAD) || (QC().test(BAD_TARGET1_T) && QC().test(BAD_TARGET2_T)));
}

template<class Ver> 
inline bool SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >::passedFOV_QC(Int32 idx) const
{
	return !(FOV_QC(idx).test(BAD_FOV) || FOV_QC(idx).test(SCAN_SEQ));
}

template<class Ver> 
inline void SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >::QCConsistencyCheck()
{
	if (!passedQC()) QC(BAD) = true;
	
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
	{
		if (QC(BAD) || QC(FOVFlag(i)) == true || !passedFOV_QC(i))
		{
			QC(FOVFlag(i)) = true;
			FOV_QC(i,BAD_FOV) = true;
		}
	}
}

// MSU Level 0 Swath
template<class Ver>
inline Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >::Swath(const DatasetType& ds) :
	Ordered<Date>(),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >(ds),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level0,Ver> > >(),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level0,Ver> > >()
{ }

template<class Ver>
inline Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >::Swath(const Swath<DatasetType>& source) :
	Ordered<Date>(source),
	SatelliteMetadata< Dataset< MSU<SwathData,msu::Level0,Ver> > >(source),
	SatelliteGeolocation< Dataset< MSU<SwathData,msu::Level0,Ver> > >(source),
	SatelliteSwathData< Dataset< MSU<SwathData,msu::Level0,Ver> > >(source)
{ }

// read MSU Level 0 data
template<class Ver>
inline void Swath< Dataset<MSU< SwathData,msu::Level0,Ver> > >::get(const Vector<Uint8>& swath_data)
{
	using namespace msu;
	
	// added swath number at beginning of swaths
	Vector<Uint8>::Offset	offs(swath_data,-4);
	
	swath_number_ = *(reinterpret_cast<const Int32*>(&swath_data[-4]));
	
// <-- Ordered<Date>
	// get scan epoch
	Int16	lyear,idayjl;
	Float32	secdy;
	
	lyear = *(reinterpret_cast<const Int16*>(&swath_data[0]));
	idayjl = *(reinterpret_cast<const Int16*>(&swath_data[2]));
	secdy = *(reinterpret_cast<const Float32*>(&swath_data[4]));
	
	epoch(Date(lyear,idayjl).time(Time(secdy)));
// --> Ordered<Date>
	
// <-- SatelliteGeolocation
	// height in km wrt geoid
	ht_ = *(reinterpret_cast<const Float32*>(&swath_data[8]));	
			
	local_zenith_angle_ = *(reinterpret_cast<const Float32*>(&swath_data[12]));
	
	const IndexRange<1>	lrng(1,NUM_FOVS);
	
	lat_ = Vector<Float32>(lrng,reinterpret_cast<const Float32*>(&swath_data[16]));
	lon_ = Vector<Float32>(lrng,reinterpret_cast<const Float32*>(&swath_data[60]));
// --> SatelliteGeolocation
	
// <-- SatelliteSwathData
	// note this is transposed relative to counts
	const Matrix<Int16>	tempCounts(IndexRange<2>(Index<2>(1,1),Index<2>(NUM_SCAN_POSITIONS,NUM_CHANNELS+3)),reinterpret_cast<const Int16*>(&swath_data[104]));
	
	// check that counts are in range
	for (Int32 i=counts().first(0);i<=counts().last(0);++i)
		for (Int32 j=counts().first(1);j<=counts().last(1);++j)
			counts(i,j,Check::Int(tempCounts(j,i+3)));
// --> SatelliteSwathData

// <-- SatelliteMetadata
	set_coef_ = *(reinterpret_cast<const Int16*>(&swath_data[300]));
	
	// difference between reported epoch and actual epoch for reported geolocation, in milliseconds
	earth_location_delta_ = *(reinterpret_cast<const Float32*>(&swath_data[302]));
		
	// binary representation from NOAA POD Guide, Table 4.3.2.1-4
	serial_number_ = tempCounts(1,1);			
	T1_cal_.first = tempCounts(1,2);						
	T2_cal_.first = tempCounts(1,3);
	
	E_cal_lo_ = tempCounts(2,1);				
	T1_cal_.second = tempCounts(2,2);					
	T2_cal_.second = tempCounts(2,3);
	
	F_cal_hi_ = tempCounts(3,1);				
	other_temp_.first = tempCounts(3,2);					
	other_temp_.second = tempCounts(3,3);
	
	xtal1_.first = tempCounts(4,1);			
	LO_temp_[0] = tempCounts(4,2);						
	LO_temp_[1] = tempCounts(4,3);
	
	xtal1_.second = tempCounts(5,1);			
	LO_temp_[2] = tempCounts(5,2);						
	LO_temp_[3] = tempCounts(5,3);
	
	xtal2_.first = tempCounts(6,1);			
	Dicke_load_temp_[0] = tempCounts(6,2);				
	Dicke_load_temp_[1] = tempCounts(6,3);
	
	xtal2_.second = tempCounts(7,1);			
	Dicke_load_temp_[2] = tempCounts(7,2);				
	Dicke_load_temp_[3] = tempCounts(7,3);
	
	xtal3_.first = tempCounts(8,1);			
	T_target1_.first = tempCounts(8,2);					
	T_target1_.second = tempCounts(8,3);
	
	xtal3_.second = tempCounts(9,1);			
	T_target2_.first = tempCounts(9,2);					
	T_target2_.second = tempCounts(9,3);
	
	xtal4_.first = tempCounts(10,1);			
	antenna_bearing_temp_.first = tempCounts(10,2);		
	antenna_bearing_temp_.second = tempCounts(10,3);

	xtal4_.second = tempCounts(11,1);		
	motor_temp_.first = tempCounts(11,2);				
	motor_temp_.second = tempCounts(11,3);
	
	minus15_plus5_V_ = tempCounts(12,1);		
	RF_chassis_temp_.first = tempCounts(12,2);			
	RF_chassis_temp_.second = tempCounts(12,3);

	space_view_[0] = tempCounts(12,4);		
	space_view_[1] = tempCounts(12,5);					
	space_view_[2] = tempCounts(12,6);
	space_view_[3] = tempCounts(12,7);
	
	plus5_V_ = tempCounts(13,1);				
	prog_temp_[0] = tempCounts(13,2);					
	prog_temp_[1] = tempCounts(13,3);

	blackbody_view_[0] = tempCounts(13,4);		
	blackbody_view_[1] = tempCounts(13,5);					
	blackbody_view_[2] = tempCounts(13,6);
	blackbody_view_[3] = tempCounts(13,7);
	
	E_zero_ = tempCounts(14,1);					
	prog_temp_[2] = tempCounts(14,2);					
	prog_temp_[3] = tempCounts(14,3);
	
	scan_to_spot1_view_[0] = tempCounts(14,4);		
	scan_to_spot1_view_[1] = tempCounts(14,5);					
	scan_to_spot1_view_[2] = tempCounts(14,6);
	scan_to_spot1_view_[3] = tempCounts(14,7);
// --> SatelliteMetadata

// <-- QCData initialization
	// read Level 0 QC flags
	QC() = std::bitset<32>(*(reinterpret_cast<const Int16*>(&swath_data[306])));
	
	for (Int32 i=FOV_QC().first();i<=FOV_QC().last();++i)
		FOV_QC(i) = *(reinterpret_cast<const Int16*>(&swath_data[306+(i-FOV_QC().first())*sizeof(Int16)]));
// --> QCData initialization
	
	// verify geolocation data
	checkGeolocation();		
	
	// validate counts and compute instrument and target temperatures
	calibrateInstrumentTemperatures();
	
	// make QC flags consistent
	QCConsistencyCheck();
}

template<class Ver> 
inline void Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >::checkGeolocation()
{
	using namespace geo;
	using namespace msu;
	
	// check for zeros in geolocation and height
	if ((latitude(SUBTRACK) == 0.0 && longitude(SUBTRACK) == 0.0) || height() <= 0)
	{
		QC(NO_EARTH_LOCATION) = true;
		
		height(_NaN_);
		
		for (Int32 i=latitude().first();i<=latitude().last();i++)
		{
			QC(FOVFlag(i)) = true;
			FOV_QC(i,BAD_FOV) = true;
			
			latitude(i,_NaN_);
			longitude(i,_NaN_);
			incidenceAngle(i,_NaN_);
		}
		
		return;
	} 
	
	// convert longitudes to 0-360 degrees, Pacific ocean centered
	for (Int32 i=longitude().first();i<=longitude().last();++i)
		longitude(i,fromEWLongitude(longitude(i)));
	
	// use TLE data to verify geolocation
	const Satellite	sat(satelliteID());
						
	// find out if ascending or descending
	//asc_ = isAscending(*this);
	asc_ = sat.ascending(epoch());
	
	QC(ASCENDING) = asc_;
	
	// flag if interpolated TLE is used
	if (sat.interpolated(epoch())) QC(TLE_INTERPOLATED) = true;

	// recompute height from TLE
	const GeographicCoordinate<Float64>	subtrack_coord(latitude(SUBTRACK),longitude(SUBTRACK));
	const Geodetic<Float64>				msu_loc(subtrack_coord,height()),
										tle_loc(sat.location(epoch()));
										
	// compare predicted and reported geolocation
	static const Float64	ANG_TOL = 1.0;

	const Float64	sep = std::abs(std::acosd(cosTheta(msu_loc,tle_loc)));
							
	if (sep > ANG_TOL) QC(SUSPECT_GEOLOCATION) = true;
	
	// set new orbit height
	if (height() - tle_loc.h() > 2.0) QC(SUSPECT_HEIGHT) = true;
	
	height(tle_loc.h());
	
	// get coordinates of satellite											
	const Geodetic<Float64>	sat_loc(subtrack_coord,height());
	const Inertial<Float64>	sat_pos(sat_loc.toInertial(epoch()));
	
	// compute incidence check and nadir angles 
	Int32	num_bad_geoloc = 0;
	
	for (Int32 i=latitude().first();i<=latitude().last();i++)
	{
		try
		{
			if (latitude(i) < -90.0 || latitude(i) > 90.0 || longitude(i) < 0.0 || longitude(i) > 360.0) throw Err();
			
			const GeographicCoordinate<Float64>	offnadir_coord(latitude(i),longitude(i));
			
			// compute incidence angles, assuming nominal nadir angle
			incidenceAngle(i,earthIncidenceAngle(offnadir_coord,static_cast<Float64>(height()),NADIR_ANGLES[i]));
														
			if (i != SUBTRACK)
			{
				const Geodetic<Float64>	footprint_loc(offnadir_coord,0.0);
				const Inertial<Float64>	footprint_pos(footprint_loc.toInertial(epoch()));
				
				// check that nadir angle is consistent
				const Inertial<Float64>	delta = sat_pos-footprint_pos;		
				const Float64			a2 = dot(delta,delta),
										b2 = dot(sat_pos,sat_pos),
										c2 = dot(footprint_pos,footprint_pos),
										theta_nadir = std::abs(std::acosd((a2+b2-c2)/(2*std::sqrt(a2*b2))));
				
				static const Float64	NADIR_ANGLE_TOLERANCE = 1.0;
					
				if (std::abs(theta_nadir-NADIR_ANGLES[i]) > NADIR_ANGLE_TOLERANCE) throw Err();
			}
			
			if (!isfinite(latitude(i)) || !isfinite(longitude(i))) throw Err();
		}
		catch (Err&)
		{
			num_bad_geoloc++;
			
			QC(FOVFlag(i)) = true;
			FOV_QC(i,BAD_FOV) = true;
			
			latitude(i,_NaN_);
			longitude(i,_NaN_);
			incidenceAngle(i,_NaN_);
		}
	}
	
	if (num_bad_geoloc == NUM_FOVS) QC(BAD_GEOLOCATION) = true;
}

template<class Ver>
inline void Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >::calibrateInstrumentTemperatures()
{
	// calibrated at higher precision in calibrateTargetTemperatures()
	T1_cal_.first = Check::Float(T1_cal_.first);
	T2_cal_.first = Check::Float(T2_cal_.first);
	T1_cal_.second = Check::Float(T1_cal_.second);
	T2_cal_.second = Check::Float(T2_cal_.second);
	
	T_target1_.first = Check::Float(T_target1_.first);
	T_target1_.second = Check::Float(T_target1_.second);
	T_target2_.first = Check::Float(T_target2_.first);
	T_target2_.second = Check::Float(T_target2_.second);
	
	// calibrated by calibration algorithm in Level 1A data
	space_view_[0] = Check::Float(space_view_[0]);
	space_view_[1] = Check::Float(space_view_[1]);
	space_view_[2] = Check::Float(space_view_[2]);
	space_view_[3] = Check::Float(space_view_[3]);
	
	blackbody_view_[0] = Check::Float(blackbody_view_[0]);
	blackbody_view_[1] = Check::Float(blackbody_view_[1]);
	blackbody_view_[2] = Check::Float(blackbody_view_[2]);
	blackbody_view_[3] = Check::Float(blackbody_view_[3]);
	
	scan_to_spot1_view_[0] = Check::Float(scan_to_spot1_view_[0]);
	scan_to_spot1_view_[1] = Check::Float(scan_to_spot1_view_[1]);
	scan_to_spot1_view_[2] = Check::Float(scan_to_spot1_view_[2]);
	scan_to_spot1_view_[3] = Check::Float(scan_to_spot1_view_[3]);
	
	// calibrated at lower precision
	other_temp_.first = Check::Float(other_temp_.first);	
	other_temp_.second = Check::Float(other_temp_.second);
	
	LO_temp_[0] = Check::Float(LO_temp_[0]);
	LO_temp_[1] = Check::Float(LO_temp_[1]);
	LO_temp_[2] = Check::Float(LO_temp_[2]);
	LO_temp_[3] = Check::Float(LO_temp_[3]);
	
	Dicke_load_temp_[0] = Check::Float(Dicke_load_temp_[0]);
	Dicke_load_temp_[1] = Check::Float(Dicke_load_temp_[1]);
	Dicke_load_temp_[2] = Check::Float(Dicke_load_temp_[2]);
	Dicke_load_temp_[3] = Check::Float(Dicke_load_temp_[3]);
	
	antenna_bearing_temp_.first = Check::Float(antenna_bearing_temp_.first);
	antenna_bearing_temp_.second = Check::Float(antenna_bearing_temp_.second);
	
	motor_temp_.first = Check::Float(motor_temp_.first);
	motor_temp_.second = Check::Float(motor_temp_.second);
	
	RF_chassis_temp_.first = Check::Float(RF_chassis_temp_.first);
	RF_chassis_temp_.second = Check::Float(RF_chassis_temp_.second);
	
	prog_temp_[0] = Check::Float(prog_temp_[0]);		
	prog_temp_[1] = Check::Float(prog_temp_[1]);
	prog_temp_[2] = Check::Float(prog_temp_[2]);	
	prog_temp_[3] = Check::Float(prog_temp_[3]);

	xtal1_.first = Check::Float(xtal1_.first);
	xtal1_.second = Check::Float(xtal1_.second);
	xtal2_.first = Check::Float(xtal2_.first);
	xtal2_.second = Check::Float(xtal2_.second);
	xtal3_.first = Check::Float(xtal3_.first);
	xtal3_.second = Check::Float(xtal3_.second);
	xtal4_.first = Check::Float(xtal4_.first);
	xtal4_.second = Check::Float(xtal4_.second);
	
	// unclear if these are temperatures...
	E_cal_lo_ = Check::Float(E_cal_lo_);				
	F_cal_hi_ = Check::Float(F_cal_hi_);				

	minus15_plus5_V_ = Check::Float(minus15_plus5_V_);		
	plus5_V_ = Check::Float(plus5_V_);				
	E_zero_ = Check::Float(E_zero_);				

	calibrateDickeLoadTemperatures();
	calibrateTargetTemperatures();
}

template<class Ver>
inline void Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >::calibrateDickeLoadTemperatures()
{
	using namespace mcs::sat::msu;
	
	// PRT count to resistance coefficients
	// 	PRT count to resistance terms k_0 and k_1 are the same for all
	//
	// 	satellites (they were not listed at all for NOAA-13).                              
    static const Float32	k_0 = 495.6,
    						k_1 = 107.8;
    								
	static const Int32	NUM_COEF = 3;
	
	// conversion factor for counts -> temperature from Mo email
	static const Tensor<Float32,3>	
		Dicke_coef(IndexRange<3>(Index<3>(1,1,1),Index<3>(NUM_SATELLITES,NUM_CHANNELS,NUM_COEF)),
			// C** TIROS-N CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 31.61686     0.4622471  	5.212674E-05 "              // MA96 TN Dicke #1 by fitting J Robel Book
			" 27.03480     0.4894769  	3.035319E-05 "              // MA97 TN Dicke #2
			" 33.07723     0.4554204  	5.395613E-05 "              // MA98 TN Dicke #3
			" 31.27395     0.4635345  	4.399543E-05 "              // MA99 TN Dicke #4
			// C** NOAA-6/A CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 30.51720     0.4709382    0.4284148E-04 "             // DICK 1
			" 29.27830     0.4697792    0.3824758E-04 "             // DICK 2
			" 29.29220     0.4759644    0.3829380E-04  "            // DICK 3
			" 29.34510     0.4741300    0.3917353E-04 "             // DICK 4
			// C** NOAA-7/C CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.89400     0.4666685    0.3894647E-04 "              // DICK 1
			" 29.86350     0.4672467    0.3905715E-04 "              // DICK 2
			" 29.27980     0.4701030    0.3870216E-04 "              // DICK 3
			" 29.63960     0.4680572    0.3902504E-04 "              // DICK 4
			// C** NOAA-8/E CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.11870     0.4685492    0.3833274E-04 "              // DICK 1
			" 29.38060     0.4693267    0.3868259E-04 "              // DICK 2
			" 29.25180     0.4705251    0.3885678E-04 "              // DICK 3
			" 29.33650     0.4687460    0.3865337E-04 "              // DICK 4
			// C** NOAA-9/F CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.20200     0.4747215    0.3912485E-04 "              // DICK 1
			" 29.14420     0.4714782    0.3865367E-04 "              // DICK 2
			" 29.29510     0.4735796    0.3928483E-04 "              // DICK 3
			" 29.43370     0.4712602    0.3928737E-04 "              // DICK 4
			// C** NOAA-10/G CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.28590     0.4690913    0.3862824E-04 "              // DICK 1
			" 29.20870     0.4684114    0.3853098E-04 "              // DICK 2
			" 28.91820     0.4688982    0.3817668E-04 "              // DICK 3
			" 29.33290     0.4670492    0.3840239E-04 "              // DICK 4
			// C** NOAA-11/H CPIDS (CALIBRATION PARAMETERs: c0, c1, & C2) ******
			" 29.43870     0.4702864    0.3887238E-04 "              // DICK 1
			" 29.34310     0.4705370    0.3869064E-04 "              // DICK 2
			" 29.37450     0.4709740    0.3882238E-04 "              // DICK 3
			" 29.35430     0.4698934    0.3860885E-04 "              // DICK 4
			// C * NOAA-12/D CPIDS (CALIBRATION PARAMETER: c0, c1, & c2) ******
			" 28.42180     0.4707687    0.3651578E-04 "              // DICK 1
			" 28.47470     0.4716112    0.3652036E-04 "              // DICK 2
			" 27.95760     0.4726148    0.3601286E-04 "              // DICK 3
			" 27.99380     0.4745809    0.3664646E-04 "              // DICK 4
			// C  ****** NOAA-14/J CPIDS (CALIBRATION PARAMETER : c0, c1, & c2) ******
			" 30.05680     0.4683650    0.4011457E-04 "              // DICK 1
			" 29.94440     0.4679420    0.4004929E-04 "              // DICK 2
			" 29.89410     0.4675090    0.3990558E-04 "              // DICK 3
			" 29.67860     0.4678200    0.3993970E-04 ");            // DICK 4
			
	// convert counts to resistance (Spencer, Christy, and Grody, J. Clim. October 1990, Eq. A2)
	const Int32		sat_num = satelliteNumber();
	
	const Float32	denom_1 = T1_cal_.second-T1_cal_.first,
					denom_2 = T2_cal_.second-T2_cal_.first,
					res_1 = k_0+k_1*(T_DickeLoad()[0]-T1_cal_.first)/denom_1,
					res_2 = k_0+k_1*(T_DickeLoad()[1]-T1_cal_.first)/denom_1,
					res_3 = k_0+k_1*(T_DickeLoad()[2]-T2_cal_.first)/denom_2,
					res_4 = k_0+k_1*(T_DickeLoad()[3]-T2_cal_.first)/denom_2;
	
	// convert resistance to temperature (Eq. A3)
	//	Target 1 is for MSU Ch. 1&2
	//	Target 2 is for MSU Ch. 3&4
	Dicke_load_temp_[0] = Dicke_coef(Index<3>(sat_num,msu::CH_1,1))+res_1*(Dicke_coef(Index<3>(sat_num,msu::CH_1,2))+res_1*Dicke_coef(Index<3>(sat_num,msu::CH_1,3)));
	Dicke_load_temp_[1] = Dicke_coef(Index<3>(sat_num,msu::CH_2,1))+res_1*(Dicke_coef(Index<3>(sat_num,msu::CH_2,2))+res_1*Dicke_coef(Index<3>(sat_num,msu::CH_2,3)));
	Dicke_load_temp_[2] = Dicke_coef(Index<3>(sat_num,msu::CH_3,1))+res_1*(Dicke_coef(Index<3>(sat_num,msu::CH_3,2))+res_1*Dicke_coef(Index<3>(sat_num,msu::CH_3,3)));
	Dicke_load_temp_[3] = Dicke_coef(Index<3>(sat_num,msu::CH_4,1))+res_1*(Dicke_coef(Index<3>(sat_num,msu::CH_4,2))+res_1*Dicke_coef(Index<3>(sat_num,msu::CH_4,3)));
}

template<class Ver>
inline void Swath< Dataset< MSU<SwathData,msu::Level0,Ver> > >::calibrateTargetTemperatures()
{
	using namespace mcs::sat::msu;
	
	// PRT count to resistance coefficients
	// 	PRT count to resistance terms k_0 and k_1 are the same for all
	//
	// 	satellites (they were not listed at all for NOAA-13).                              
    static const Float32	k_0 = 495.6,
    						k_1 = 107.8;
    								
	// PRT resistance to temperature coefficients
	//	variously from NOAA Technical Memorandum NESS 107 Rev. 1 (http://www.ntis.gov) 
	// 	and updates on http://psbsgi1.nesdis.noaa.gov/EBB/ml/nic000.html
    static const Vector<Float32>	e0_1A(IndexRange<1>(1,NUM_SATELLITES),
    										" 29.1000  29.3271  28.9760  29.48570  29.429400 "
	      									" 29.2589  29.1014  29.3970  29.003197 "),
									e1_1A(IndexRange<1>(1,NUM_SATELLITES),
	  										" .4259600  .4254270  .4269400  .4247172  .4283514  "
	  										" .4245799  .4242637  .4256561  .4293630  "),
									e2_1A(IndexRange<1>(1,NUM_SATELLITES),
	  										" .3187000E-04  .3165629E-04  .3173700E-04  .3209600E-04  .3223351E-04  "
	  										" .3169871E-04  .3162073E-04  .3238697E-04  .3144484E-04  "),
	  										
									e0_2A(IndexRange<1>(1,NUM_SATELLITES),
	  										" 27.5500  29.3083  28.99500  29.312300  29.263900  "
	  										" 29.4633  28.9481  28.45270  28.514892  "),
									e1_2A(IndexRange<1>(1,NUM_SATELLITES),
	  										" .4278700  .4260801  .4262700  .4326844  .4338180  "
	  										" .4250778  .4277704  .4293800  .4280370  "),
									e2_2A(IndexRange<1>(1,NUM_SATELLITES),
	  										" .3185000E-04  .3149910E-04  .3177500E-04  .3282283E-04  .3334592E-04  "
	  										" .3192864E-04  .3185110E-04  .2974601E-04  .3028803E-04  "),
									
									e0_1B(IndexRange<1>(1,NUM_SATELLITES),
	  										" 29.0500  29.2923  28.9450  29.34710  29.541600  "
	  										" 29.2350  28.9854  28.1854  28.455605  "),
									e1_1B(IndexRange<1>(1,NUM_SATELLITES),
	  										" .4271000  .4261320  .4262800  .4247339  .4248521  "
	  										" .4289328  .4269018  .4302196  .4284300  "),
									e2_1B(IndexRange<1>(1,NUM_SATELLITES),
	  										" .3206000E-04  .3152410E-04  .3160300E-04  .3165123E-04  .3188156E-04  "
	  										" .3232162E-04  .3170732E-04  .2920356E-04  .3021258E-04  "),
									
									e0_2B(IndexRange<1>(1,NUM_SATELLITES),
	  										" 28.9500  29.3731  28.9640  27.97190  29.577800  "
	  										" 29.4680  29.0424  28.9588  29.113998  "),
									e1_2B(IndexRange<1>(1,NUM_SATELLITES),
	  										" .4283600  .4255765  .4249800  .4285466  .4243732  "
	  										" .4275379  .4274570  .4276377  .4257680  "),
									e2_2B(IndexRange<1>(1,NUM_SATELLITES),
	  										" .3175000E-04  .3176020E-04  .3138900E-04  .3095837E-04  .3264424E-04  "
	  										" .3221571E-04  .3191209E-04  .3033068E-04  .3159801E-04  "), 
                                    prt_err_1A(IndexRange<1>(1,NUM_SATELLITES),
											"0.000  0.370  0.000  0.000  0.000  0.000  0.000  0.000  0.000"),
									prt_err_2A(IndexRange<1>(1,NUM_SATELLITES),
											"-1.036 0.405  0.000  0.000  0.000  0.000  0.000  0.000  0.000"),
									prt_err_1B(IndexRange<1>(1,NUM_SATELLITES),
											"-0.440 0.370  0.000  0.000  0.000  0.000  0.000  0.000  0.000"),
									prt_err_2B(IndexRange<1>(1,NUM_SATELLITES),
                							"0.000  0.389  0.000  0.000  0.000  0.000  0.000  0.000  0.000");
																				

	/*
	// conversion factor for counts -> temperature from Mo email
	static const Tensor<Float32,3>	
		Cal_coef(IndexRange<3>(Index<3>(1,1,1),Index<3>(NUM_SATELLITES,NUM_CHANNELS,NUM_COEF)),					
			// C** TIROS-N CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.10        0.42596       3.187e-05 "                 // MG87 TN Cal Tar 1A  (From NESS107)
			" 27.55        0.42787       3.185e-05 "                 // MG87 TN Cal Tar 1B
			" 29.05        0.42710       3.206e-05 "                 // MG87 TN Cal Tar 2A
			" 28.95        0.42836       3.175e-05 "                 // MG87 TN Cal Tar 2B
			// C** NOAA-6/A CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.3271      0.425427      0.3165629E-04 "             // CAL 1A
			" 29.2923      0.426132      0.315241E-04 "              // CAL 1B
			" 29.3083      0.4260801     0.314991E-04 "              // CAL 2A
			" 29.3731      0.4255765     0.317602E-04 "              // CAL 2B
			// C** NOAA-7/C CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 28.9757      0.4269366    0.3173672E-04 "              // CAL 1A
			" 28.9449      0.4262771    0.3160302E-04 "              // CAL 1B
			" 28.9950      0.4262736    0.3177479E-04 "              // CAL 2A
			" 28.9640      0.4249752    0.3138887E-04 "              // CAL 2B
			// C** NOAA-8/E CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.4857      0.4247172    0.3209060E-04 "              // CAL 1A
			" 29.3471      0.4247339    0.3165123E-04 "              // CAL 1B
			" 29.3123      0.4326844    0.3282283E-04 "              // CAL 2A
			" 27.9719      0.4285466    0.3095837E-04 "              // CAL 2B
			// C** NOAA-9/F CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.4294      0.4283514    0.3223351E-04 "              // CAL 1A
			" 29.5416      0.4248521    0.3188156E-04 "              // CAL 1B
			" 29.2639      0.4338180    0.3334592E-04 "              // CAL 2A
			" 29.5778      0.4243732    0.3264424E-04 "              // CAL 2B
			// C** NOAA-10/G CPIDS (CALIBRATION PARAMETERs : c0, c1, & c2) ******
			" 29.2589      0.4245799    0.3169871E-04 "              // CAL 1A
			" 29.2350      0.4289328    0.3232162E-04 "              // CAL 1B
			" 29.4633      0.4250778    0.3192864E-04 "              // CAL 2A
			" 29.4680      0.4275379    0.3221571E-04 "              // CAL 2B
			// C** NOAA-11/H CPIDS (CALIBRATION PARAMETERs: c0, c1, & C2) ******
			" 29.1014      0.4242637    0.3162073E-04 "              // CAL 1A
			" 28.9854      0.4269018    0.3170732E-04 "              // CAL 1B
			" 28.9481      0.4277704    0.3185110E-04 "              // CAL 2A
			" 29.0424      0.4274570    0.3191209E-04 "              // CAL 2B
			// C * NOAA-12/D CPIDS (CALIBRATION PARAMETER: c0, c1, & c2) ******
			" 29.3970      0.4256561    0.3238697D-04 "              // CAL 1A
			" 28.1854      0.4302196    0.2920356D-04 "              // CAL 1B
			" 28.4527      0.4293800    0.2974601D-04 "              // CAL 2A
			" 28.9588      0.4276377    0.3033068D-04 "              // CAL 2B
			// C  ****** NOAA-14/J CPIDS (CALIBRATION PARAMETER : c0, c1, & c2) ******

			" 29.0032      0.4293630    0.3144484E-04 "              // CAL 1A
			" 28.4556      0.4284300    0.3021258E-04 "              // CAL 1B
			" 28.5149      0.4280370    0.3028803E-04 "              // CAL 2A
			" 29.1140      0.4257680    0.3159801E-04 ");            // CAL 2B	
	*/

	// convert counts to resistance (Spencer, Christy, and Grody, J. Clim. October 1990, Eq. A2)
	const Int32		sat_num = satelliteNumber();
	
	const Float32	denom_1 = T1_cal_.second-T1_cal_.first,
					denom_2 = T2_cal_.second-T2_cal_.first,
					res_1A = k_0+k_1*(T_target1_.first-T1_cal_.first)/denom_1,
					//res_1B = k_0+k_1*(T_target1_.second-T1_cal_.first)/denom_1,   // old, incorrect version
					//res_2A = k_0+k_1*(T_target2_.first-T2_cal_.first)/denom_2,    // old, incorrect version
					res_1B = k_0+k_1*(T_target1_.second-T2_cal_.first)/denom_2, // new, correct version
					res_2A = k_0+k_1*(T_target2_.first-T1_cal_.first)/denom_1,  // new, correct version
					res_2B = k_0+k_1*(T_target2_.second-T2_cal_.first)/denom_2;
	
	// convert resistance to temperature (Eq. A3)
	//	Target 1 is for MSU Ch. 1&2
	//	Target 2 is for MSU Ch. 3&4
	T_target1_.first = e0_1A[sat_num]+res_1A*(e1_1A[sat_num]+res_1A*e2_1A[sat_num]);// - prt_err_1A[sat_num] ;
	T_target1_.second = e0_1B[sat_num]+res_1B*(e1_1B[sat_num]+res_1B*e2_1B[sat_num]);//- prt_err_1B[sat_num] ;
	T_target2_.first = e0_2A[sat_num]+res_2A*(e1_2A[sat_num]+res_2A*e2_2A[sat_num]);// - prt_err_2A[sat_num] ;
	T_target2_.second = e0_2B[sat_num]+res_2B*(e1_2B[sat_num]+res_2B*e2_2B[sat_num]);//- prt_err_2B[sat_num] ;
	
	const Float32 TT1 = T_target1_.first,
				  TT2 = T_target1_.second;
	
	static const Float32	MIN_CAL_COUNTS = 500.0;
	
	if (denom_1 <= MIN_CAL_COUNTS || !isfinite(denom_1)) 
	{
		T_target1_.first = T_target1_.second = _NaN_;
		
		QC(BAD_TARGET1_T) = true;
	}
	
	if (denom_2 <= MIN_CAL_COUNTS || !isfinite(denom_1)) 
	{
		T_target2_.first = T_target2_.second = _NaN_;
		
		QC(BAD_TARGET2_T) = true;
	}
}
