#ifndef MCS_MSUORBITDATA_H
#define MCS_MSUORBITDATA_H

#include "MCS_Exceptions.h"
#include "MCS_IntIndexed.h"
#include "MCS_MSUDataset.h"
#include "MCS_SatelliteOrbitIterator.h"

namespace mcs {

namespace sat {

template<class Lev,class Ver>
class OrbitIteratorImp< Dataset< MSU<SwathData,Lev,Ver> > > : public IntIndexed
{
	public:
		typedef	Dataset< MSU<SwathData,Lev,Ver> >	DatasetType;
		
		OrbitIteratorImp(const DatasetType& ds,const Int32& orbit_num) : IntIndexed(orbit_num),ds_(ds) { }
		OrbitIteratorImp(const OrbitIteratorImp& source) : IntIndexed(source),ds_(source.ds_) { }
		~OrbitIteratorImp() { }
		
		const DatasetType& dataset() const	{ return ds_; }
		
		const Int32& orbitNumber() const	{ return value(); }
		
	private:
		DatasetType	ds_;
};

// MSU dataset, iterator iterates over orbits
template<class Lev,class Ver> 
class OrbitIterator< Dataset< MSU<SwathData,Lev,Ver> > > : public Dataset< MSU<SwathData,Lev,Ver> >
{ 
	public:
		typedef Dataset< MSU<SwathData,Lev,Ver> >							DatasetType;				
		typedef OrbitIteratorImp<DatasetType>								IteratorImpType;	
				
		typedef Iterator<Orbit<DatasetType>,IteratorImpType>				iterator;
		typedef ReverseIterator<Orbit<DatasetType>,IteratorImpType>			reverse_iterator;
		typedef ConstIterator<Orbit<DatasetType>,IteratorImpType>			const_iterator;
		typedef ConstReverseIterator<Orbit<DatasetType>,IteratorImpType>	const_reverse_iterator;
		
		OrbitIterator(const SatelliteID& sat_ID) : DatasetType(sat_ID) { }
		OrbitIterator(const DatasetType& ds) : DatasetType(ds) { }
		~OrbitIterator() { }
		
		const_iterator begin() const							{ return const_iterator(IteratorImpType(*this,this->firstOrbit())); }
		const_iterator date(const Date& ep) const;
		const_iterator number(const Int32& num) const			{ return const_iterator(IteratorImpType(*this,num)); }
		const_iterator end() const								{ return const_iterator(IteratorImpType(*this,this->lastOrbit()+1)); }
		
		const_reverse_iterator rbegin() const					{ return const_reverse_iterator(IteratorImpType(*this,this->lastOrbit())); }
		const_reverse_iterator rdate(const Date& ep) const;
		const_reverse_iterator rnumber(const Int32& num) const	{ return const_reverse_iterator(IteratorImpType(*this,num)); }
		const_reverse_iterator rend() const						{ return const_reverse_iterator(IteratorImpType(*this,this->firstOrbit()-1)); }

	private:
		class InInventoryDateRange : public Functor1<const typename DatasetType::InventoryType::value_type&,bool>
		{
			public:
				InInventoryDateRange(const Date& ep) : rd2000_(toRD2000(ep)) { }
				~InInventoryDateRange() { }
				
				bool operator()(const typename DatasetType::InventoryType::value_type& val) const { return rd2000_ >= val.firstRD2000() && rd2000_ <= val.lastRD2000(); }
			
			private:
				Float64	rd2000_;
		};
};

template<class Lev,class Ver> 
inline typename OrbitIterator< Dataset< MSU<SwathData,Lev,Ver> > >::const_iterator OrbitIterator< Dataset< MSU<SwathData,Lev,Ver> > >::date(const Date& ep) const
{
	const InInventoryDateRange	func(ep);
	
	typename DatasetType::InventoryType::const_iterator	it = std::find_if(this->inventory().begin(),this->inventory().end(),func);
	
	return const_iterator(IteratorImpType(*this,it->orbitNumber()));
}

template<class Lev,class Ver> 
inline typename OrbitIterator< Dataset< MSU<SwathData,Lev,Ver> > >::const_reverse_iterator OrbitIterator< Dataset< MSU<SwathData,Lev,Ver> > >::rdate(const Date& ep) const
{
	const InInventoryDateRange	func(ep);
	
	typename DatasetType::InventoryType::const_reverse_iterator	it = std::find_if(this->inventory().rbegin(),this->inventory().rend(),func);
	
	return const_reverse_iterator(IteratorImpType(*this,it->orbitNumber()));
}

} // namespace sat

} // namespace mcs

#endif // MCS_MSUORBITDATA_H