#ifndef MCS_MSUSERIES_H
#define MCS_MSUSERIES_H

#include "MCS_Array.h"
#include "MCS_MapSeries.h"

namespace mcs {

//	Metadata :	 	version		c1
//								d2
//								d3
//					channel		2LT
//								2
//								4
//					type		TB-annual-cycle
//								TB-anomaly
//								TB
//
class MSUSeries : public MapSeries<>
{
	public:
		MSUSeries(const std::string&);
		MSUSeries(const std::string&,const Date&);
		~MSUSeries();
		
		Date begin() const;
		Date end() const;
		
	private:
		Date findNearestInterval(const Date&) const;
		Date incrementInterval(Uint32 num) const;
		void read();
		
		void addMGAnnualCycle();
		void addMGAnomaly();
		void addMGClimatology();
		
		// internal bookkeeping
		std::ifstream	anifs_;
		Metadata		last_meta_;
		Date			last_epoch_;
};

//	Metadata :	 	channel		1..4
//					FOV			-5..5
//
class MSUClimatology : public MapSeries<>
{
	public:
		MSUClimatology(const std::string&);
		MSUClimatology(const std::string&,const Date&);
		~MSUClimatology();
		
		Date begin() const;
		Date end() const;
		
	private:
		Date findNearestInterval(const Date&) const;
		Date incrementInterval(Uint32 num) const;
		void read();
		
		// cache data
		Array<Matrix<Float32>,2>	climatologyData_;
		bool						initialized_;
		
		// internal bookkeeping
		Metadata			last_meta_;
		Date				last_epoch_;
};

} // namespace mcs

#endif // MCS_MSUSERIES_H