#ifndef MCS_MSUSWATHDATA_H
#define MCS_MSUSWATHDATA_H

#include "MCS_BasicSwathData.h"
#include "MCS_MSUDataset.h"
#include "MCS_MSUOrbit.h"
#include "MCS_SatelliteSwathIterator.h"

namespace mcs {

namespace sat {

template<class Lev,class Ver>
class SwathIteratorImp< Dataset< MSU<SwathData,Lev,Ver> > > : public IntIndexed
{
	public:
		typedef Dataset< MSU<SwathData,Lev,Ver> >	DatasetType;
		typedef	Orbit<DatasetType>					OrbitType;
		
		SwathIteratorImp(const OrbitType& orb,const Int32& swath_num) : IntIndexed(swath_num),orbptr_(&orb) { }
		SwathIteratorImp(const SwathIteratorImp& source) : IntIndexed(source),orbptr_(source.orbptr_) { }
		~SwathIteratorImp() { }
		
		const DatasetType& dataset() const	{ return orbit().dataset(); }
		const OrbitType& orbit() const		{ return *orbptr_; }
		
		const Int32& swathNumber() const	{ return value(); }
		
	private:
		const OrbitType*	orbptr_;
};

// MSU SatelliteSwathData specialization
template<class Lev,class Ver> 
class SatelliteSwathData< Dataset< MSU<SwathData,Lev,Ver> > > : 
	public BasicSwathData<typename Dataset< MSU<SwathData,Lev,Ver> >::SwathDataType>
{
	public:
		typedef Dataset< MSU<SwathData,Lev,Ver> >	DatasetType;
		
	protected:
		SatelliteSwathData();
		SatelliteSwathData(const SatelliteSwathData<DatasetType>& source);
		virtual ~SatelliteSwathData() { }
					
	private:
		SatelliteSwathData<DatasetType>& operator=(const SatelliteSwathData<DatasetType>&);
};

template<class Lev,class Ver> 
inline SatelliteSwathData< Dataset< MSU<SwathData,Lev,Ver> > >::SatelliteSwathData() : 
	BasicSwathData<typename Dataset< MSU<SwathData,Lev,Ver> >::SwathDataType>(DatasetType::numberOfChannels(),DatasetType::numberOfFOVs()) 
{ }

template<class Lev,class Ver> 
inline SatelliteSwathData< Dataset< MSU<SwathData,Lev,Ver> > >::SatelliteSwathData(const SatelliteSwathData<DatasetType>& source) : 
	BasicSwathData<typename Dataset< MSU<SwathData,Lev,Ver> >::SwathDataType>(source) 
{ }

} // namespace sat

} // namespace mcs

#endif // MCS_MSUSWATHDATA_H
