template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor() :
	Imp()
{ }

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor(const RangeType& rng) :
	Imp(rng)
{ }

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor(const RangeType& rng,const ElementType& val) :
	Imp(rng,val)
{ }

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor(const RangeType& rng,const std::string& str) :
	Imp(rng)
{ 
	set(str);
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor(const RangeType& rng,ElementType* ptr) :
	Imp(rng,ptr)
{ }

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor(const Imp& imp) :
	Imp(imp)
{ }

template<class T,Uint8 R,class Imp>
template<class Imp2>
inline Tensor<T,R,Imp>::Tensor(Tensor<T,R,Imp2>& source) :
	Imp(source)
{ }

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>::Tensor(const TensorType& source) :
	Imp(source)
{ }

template<class T,Uint8 R,class Imp>
inline void Tensor<T,R,Imp>::set(const ElementType& val)
{
	ImplementationType::set(val);
}

template<class T,Uint8 R,class Imp>
inline void Tensor<T,R,Imp>::set(const std::string& str)
{
	std::stringstream 	sstr;	
	
	sstr << str;
	
	iterator	it=begin();
	
	while (sstr && it!=end()) 
	{
		sstr >> *it;
		++it;
	}
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const ElementType& val)
{
	set(val);
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const std::string& str)
{
	set(str);
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::ElementType& Tensor<T,R,Imp>::operator()(const IndexType& idx)					
{
	return (*this)[offsetFromIndex(idx)]; 
}
 
template<class T,Uint8 R,class Imp>
inline const typename Tensor<T,R,Imp>::ElementType& Tensor<T,R,Imp>::operator()(const IndexType& idx) const		
{ 
	return (*this)[offsetFromIndex(idx)]; 
}
		
template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const TensorType& source)
{
	//if (this == &source) return *this;
	ImplementationType::AccessorType::CheckingPolicyType::compare(this->range(),source.range());
	
	ImplementationType::operator=(source);
	
	return *this;
}

#ifdef USE_PETE_OPERATORS
// PETE copy constructor
template<class T,Uint8 R,class Imp>
template<class RHS>
inline Tensor<T,R,Imp>::Tensor(const Expression<RHS>& rhs)
{
	// ensure that ranges in expression are commensurate with allocated value
	if (forEach(rhs,RangeLeaf<R>(range()),AndCombine()))
	{
	}
}

// PETE assignment
template<class T,Uint8 R,class Imp>
template<class RHS>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const Expression<RHS>& rhs)
{
	// ensure that ranges in expression are commensurate
	if (forEach(rhs,RangeLeaf<R>(range()),AndCombine()))
	{
		for (typename TensorType::iterator it=begin();it!=end();++it)
			*it = forEach(rhs,EvalTensorOffset<T,R,Imp>(it.offset()),OpCombine());

		// iterator evaluation seems slower
		//for (typename TensorType::iterator it=begin();it!=end();++it)
		//	*it = forEach(rhs,EvalTensorIterator<T,R,Imp>(it),OpCombine());
	}
	else
	{
		throw RangeErr();
	}
		
	return *this;
}
#endif

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const MaskSliceType& source)
{
	typename MaskSliceType::const_iterator rit=source.begin();
	
	iterator	it=begin();
	
	for (;it!=end();++it,++rit) *it = *rit;
		
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const RangeSliceType& source)
{
	for (typename RangeSliceType::const_iterator rit=source.begin();rit!=source.end();++rit)
		(*this)(source.indexFromOffset(rit.offset())) = *rit;
		
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator=(const IndirectionSliceType& source)
{
	for (typename IndirectionSliceType::const_iterator rit=source.begin();rit!=source.end();++rit)
		(*this)(source.indexFromOffset(rit.offset())) = *rit;
				
	return *this;
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::MaskSliceType Tensor<T,R,Imp>::operator()(const typename Tensor<T,R,Imp>::MaskType& msk)
{ 
	MaskSliceImplementation<TensorType>	imp(*this,msk);
	
	return MaskSliceType(imp); 
}
																					
template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::RangeSliceType Tensor<T,R,Imp>::operator()(const RangeType& rng)
{ 
	RangeSliceImplementation<TensorType>	imp(*this,rng);
	
	return RangeSliceType(imp); 
}
																					
template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::IndirectionSliceType Tensor<T,R,Imp>::operator()(const TensorIndirection<R>& ind)
{ 
	IndirectionSliceImplementation<TensorType>	imp(*this,ind);
	
	return IndirectionSliceType(imp); 
}
																					
template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator+=(const ElementType& val)
{
	for (iterator it=begin();it!=end();++it) *it += val;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator-=(const ElementType& val)
{
	for (iterator it=begin();it!=end();++it) *it -= val;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator*=(const ElementType& val)
{
	for (iterator it=begin();it!=end();++it) *it *= val;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator/=(const ElementType& val)
{
	for (iterator it=begin();it!=end();++it) *it /= val;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator+=(const TensorType& source)
{
	//if (this->range() != source.range()) throw RangeErr();
	ImplementationType::AccessorType::CheckingPolicyType::compare(this->range(),source.range());

	if (this->range() == RangeType())
		return *this = source;
		
	const_iterator	sit = source.begin();
	
	for (iterator it=begin();it!=end();++it,++sit) *it += *sit;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator-=(const TensorType& source)
{
	//if (this->range() != source.range()) throw RangeErr();
	ImplementationType::AccessorType::CheckingPolicyType::compare(this->range(),source.range());
	
	if (this->range() == RangeType())
		return *this = -source;
		
	const_iterator	sit = source.begin();
	
	for (iterator it=begin();it!=end();++it,++sit) *it -= *sit;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator*=(const TensorType& source)
{
	//if (this->range() != source.range()) throw RangeErr();
	ImplementationType::AccessorType::CheckingPolicyType::compare(this->range(),source.range());
	
	if (this->range() == RangeType())
		return *this = Tensor<T,R,Imp>(source.range());
		
	const_iterator	sit = source.begin();
	
	for (iterator it=begin();it!=end();++it,++sit) *it *= *sit;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline Tensor<T,R,Imp>& Tensor<T,R,Imp>::operator/=(const TensorType& source)
{
	//if (this->range() != source.range()) throw RangeErr();
	ImplementationType::AccessorType::CheckingPolicyType::compare(this->range(),source.range());
	
	if (this->range() == RangeType())
		return *this = Tensor<T,R,Imp>(source.range());
		
	const_iterator	sit = source.begin();
	
	for (iterator it=begin();it!=end();++it,++sit) *it /= *sit;
	
	return *this;
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::const_iterator Tensor<T,R,Imp>::begin() const
{ 
	return const_iterator(*this,0); 
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::const_iterator Tensor<T,R,Imp>::end() const
{ 
	return const_iterator(*this,this->size()); 
}
	 
template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::const_reverse_iterator Tensor<T,R,Imp>::rbegin() const
{ 
	return const_reverse_iterator(*this,this->size()-1); 
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::const_reverse_iterator Tensor<T,R,Imp>::rend() const
{ 
	return const_reverse_iterator(*this,-1); 
}
	 	
template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::iterator Tensor<T,R,Imp>::begin() 
{ 
	return iterator(*this,0); 
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::iterator Tensor<T,R,Imp>::end() 
{ 
	return iterator(*this,this->size()); 
}
	 
template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::reverse_iterator Tensor<T,R,Imp>::rbegin() 
{ 
	return reverse_iterator(*this,this->size()-1); 
}

template<class T,Uint8 R,class Imp>
inline typename Tensor<T,R,Imp>::reverse_iterator Tensor<T,R,Imp>::rend() 
{ 
	return reverse_iterator(*this,-1); 
}

template<class T,Uint8 R,class Imp>
inline void Tensor<T,R,Imp>::deserialize(std::istream& is)			
{ 
	Int32	flags;
	
	is.read(reinterpret_cast<char*>(&flags),sizeof(flags));
	
	if (flags & 0x000F != TypeFlags<T>::flags().to_ulong()) throw IOErr();
	if (flags & 0x00F0 != Rank << 8) throw IOErr();
	
	ImplementationType::deserialize(is); 
}

template<class T,Uint8 R,class Imp>
inline void Tensor<T,R,Imp>::serialize(std::ostream& os) const		
{ 
	const Int32	flags = TypeFlags<T>::flags().to_ulong() | Int32(Rank) << 8;
	
	os.write(reinterpret_cast<const char*>(&flags),sizeof(flags));
	
	ImplementationType::serialize(os); 
}
	 	
