#ifndef MCS_TENSORBIN_H
#define MCS_TENSORBIN_H

#include <vector>

#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"
#include "MCS_Tensor.h"
#include "MCS_TensorBounds.h"
#include "MCS_Typedefs.h"

namespace mcs {

// bin class
template<class T,Uint8 R,class BT = Bounds<T,R,Simple<Regular> > > 
class TensorBin
{
	public:
		typedef typename BT	BoundsType;
		
		TensorBin(const BoundsType& bds);
		~TensorBin() { }
		
		void add(const FixedVector<T,R>& x,T z);
		void add(const FixedVector<T,R>& x,T z,T w);
		template<class F> void add(const FixedVector<T,R>& x,T z,const F& f);
		
		void clear();
		
		const BoundsType& bounds() const		{ return bounds_; }
		
		const Int32& OOB() const				{ return oob_; }
		
		const Tensor<T,R>& bins() const			{ return bins_; }
		const Tensor<T,R>& weights() const		{ return weights_; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		BoundsType	bounds_;
		Tensor<T,R>	bins_,
					weights_;
		Int32		oob_;
};

// implementation
template<class T,Uint8 R,class BT> 
inline TensorBin<T,R,BT>::TensorBin(const BoundsType& bds) : 
	bounds_(bds),
	bins_(bds.range()),
	weights_(bds.range()),
	oob_(0)
{ }

template<class T,Uint8 R,class BT> 
inline void TensorBin<T,R,BT>::add(const FixedVector<T,R>& x,T z)
{
	add(x,z,T(1));
}

template<class T,Uint8 R,class BT> 
inline void TensorBin<T,R,BT>::add(const FixedVector<T,R>& x,T z,T w)
{
	if (!isfinite(z) || !isfinite(w)) 
	{
		oob_++;
		return;
	}
	
	try
	{
		const typename BoundsType::IndexType	idx(bounds_.find(x));
		
		bins_(idx) += z*w;
		weights_(idx) += w;
	}
	catch (Exception&)
	{
		oob_++;
	}
}

template<class T,Uint8 R,class BT> 
template<class F>
inline void TensorBin<T,R,BT>::add(const FixedVector<T,R>& x,T z,const F& f)
{
	if (!isfinite(z)) 
	{
		oob_++;
		return;
	}
	
	try
	{
		f(x,z,*this);
	}
	catch (Exception&)
	{
		oob_++;
	}
}

template<class T,Uint8 R,class BT> 
inline void TensorBin<T,R,BT>::clear()
{
	// clear histogram
	bins_ = 0;
	weights_ = 0;
	oob_ = 0;
}

} // namespace mcs

#endif // MCS_TENSORBIN_H