#ifndef MCS_TENSORHISTOGRAM_H
#define MCS_TENSORHISTOGRAM_H

#include <vector>

#include "MCS_Exceptions.h"
#include "MCS_FixedVector.h"
#include "MCS_Tensor.h"
#include "MCS_TensorBounds.h"
#include "MCS_Typedefs.h"

namespace mcs {

// histogram class
template<class T,Uint8 R,class BT = Bounds<T,R,Simple<Regular> > > 
class TensorHistogram
{
	public:
		typedef typename BT	BoundsType;
		
		TensorHistogram(const BoundsType& bds);
		~TensorHistogram() { }
		
		void add(const FixedVector<T,R>& x);
		
		void clear();
		
		const BoundsType& bounds() const			{ return bounds_; }
		
		const Int32& OOB() const					{ return oob_; }
		
		const Tensor<Int32,R>& histogram() const	{ return hist_; }
		
		__DECLARE_EXCEPTION_CLASS__(Err,Exception)
		
	private:
		BoundsType		bounds_;
		Tensor<Int32,R>	hist_;
		Int32			oob_;
};

// implementation
template<class T,Uint8 R,class BT> 
inline TensorHistogram<T,R,BT>::TensorHistogram(const BoundsType& bds) : 
	bounds_(bds),
	hist_(bds.range()),
	oob_(0)
{ }

template<class T,Uint8 R,class BT> 
inline void TensorHistogram<T,R,BT>::add(const FixedVector<T,R>& x)
{
	try
	{
		const typename BoundsType::IndexType	idx(bounds_.find(x));
		
		hist_(idx)++;
	}
	catch (Exception&)
	{
		oob_++;
	}
}

template<class T,Uint8 R,class BT> 
inline void TensorHistogram<T,R,BT>::clear()
{
	// clear histogram
	hist_ = 0;
	oob_ = 0;
}

} // namespace mcs

#endif // MCS_TENSORHISTOGRAM_H