template<class T,Uint8 R,class Imp>
class Tensor<T,R,Imp>::iterator : public std::iterator<std::random_access_iterator_tag,T,Int32>
{
	public:
		typedef Tensor<T,R,Imp>						TensorType;
		typedef typename TensorType::ElementType	ElementType;
		typedef typename TensorType::IndexType		IndexType;
		
		iterator(TensorType& ref,const Int32& offs) : ref_(ref),iter_(offs) { }
		iterator(const iterator& source) : ref_(source.ref_),iter_(source.iter_) { }
		~iterator() { }
				
		iterator& operator=(const iterator&);
		iterator& operator=(const typename TensorType::IndexType&);
		
		bool operator==(const iterator& it) const								{ return iter_ == it.iter_; }
		bool operator!=(const iterator& it) const								{ return iter_ != it.iter_; }
		bool operator<(const iterator& it) const								{ return iter_ < it.iter_; }
		bool operator<=(const iterator& it) const								{ return iter_ <= it.iter_; }
		bool operator>(const iterator& it) const								{ return iter_ > it.iter_; }
		bool operator>=(const iterator& it) const								{ return iter_ >= it.iter_; }
		
		iterator&	operator+=(const Int32& num)								{ iter_ += num; return *this; }
		iterator&	operator-=(const Int32& num)								{ iter_ -= num; return *this; }
		
		Int32	operator-(const iterator& it) const								{ return iter_-it.iter_; }
		
		iterator&	operator++() 												{ ++iter_; return *this; }
		iterator&	operator--()												{ --iter_; return *this; }
		const iterator operator++(int)											{ iterator tmp(*this); ++iter_; return tmp; }
		const iterator operator--(int)											{ iterator tmp(*this); --iter_; return tmp; }
		
		iterator operator+(const Int32& num) const								{ iterator tmp(*this); tmp += num; return tmp; }
		iterator operator-(const Int32& num) const								{ iterator tmp(*this); tmp -= num; return tmp; }
		
		ElementType& operator*()												{ return ref_.elementFromIterator(iter_); }
		ElementType& operator[](const Int32& num)								{ iter_ = num; return ref_.elementFromIterator(iter_); }
		
		ElementType* operator->()												{ return ref_.elementFromIterator(iter_).operator->(); }
		
		const IndexType index() const											{ return ref_.indexFromOffset(ref_.iteratorToOffset(iter_)); }
		const Int32& offset() const												{ return iter_; }
		const TensorType& tensor() const										{ return ref_; }
		
		class Err : public Exception { };
		
	private:
		friend class TensorType::const_iterator;
		
		TensorType&	ref_;
		Int32		iter_;
};

template<class T,Uint8 R,class Imp>
class Tensor<T,R,Imp>::const_iterator : public std::iterator<std::random_access_iterator_tag,T,Int32>
{
	public:
		typedef Tensor<T,R,Imp>						TensorType;
		typedef typename TensorType::ElementType	ElementType;
		typedef typename TensorType::IndexType		IndexType;
		
		const_iterator(const TensorType& ref,const Int32& offs) : ref_(ref),iter_(offs) { }
		const_iterator(const typename TensorType::iterator& source) : ref_(const_cast<const TensorType&>(source.tensor())),iter_(source.offset()) { }
		const_iterator(const const_iterator& source) : ref_(source.ref_),iter_(source.iter_) { }
		~const_iterator() { }
				
		const_iterator& operator=(const const_iterator&);
		const_iterator& operator=(const typename TensorType::IndexType&);
		
		bool operator==(const const_iterator& it) const							{ return iter_ == it.iter_; }
		bool operator!=(const const_iterator& it) const							{ return iter_ != it.iter_; }
		bool operator<(const const_iterator& it) const							{ return iter_ < it.iter_; }
		bool operator<=(const const_iterator& it) const							{ return iter_ <= it.iter_; }
		bool operator>(const const_iterator& it) const							{ return iter_ > it.iter_; }
		bool operator>=(const const_iterator& it) const							{ return iter_ >= it.iter_; }
		
		const_iterator&	operator+=(const Int32& num)							{ iter_ += num; return *this; }
		const_iterator&	operator-=(const Int32& num)							{ iter_ -= num; return *this; }
		
		Int32	operator-(const const_iterator& it) const						{ return iter_-it.iter_; }
		
		const_iterator&	operator++() 											{ ++iter_; return *this; }
		const_iterator&	operator--()											{ --iter_; return *this; }
		const const_iterator operator++(int)									{ const_iterator tmp(*this); ++iter_; return tmp; }
		const const_iterator operator--(int)									{ const_iterator tmp(*this); --iter_; return tmp; }
		
		const_iterator operator+(const Int32& num) const						{ const_iterator tmp(*this); tmp += num; return tmp; }
		const_iterator operator-(const Int32& num) const						{ const_iterator tmp(*this); tmp -= num; return tmp; }
		
		const ElementType&	operator*()											{ return ref_.elementFromIterator(iter_); }
		const ElementType&	operator[](const Int32& num)						{ iter_ = num; return ref_.elementFromIterator(iter_); }
		
		const ElementType*	operator->()										{ return ref_.elementFromIterator(iter_).operator->(); }
		
		const IndexType index() const											{ return ref_.indexFromOffset(ref_.iteratorToOffset(iter_)); }
		const Int32& offset() const												{ return iter_; }
		const TensorType& tensor() const										{ return ref_; }
		
		class Err : public Exception { };
		
	private:
		const TensorType&	ref_;
		Int32				iter_;
};

template<class T,Uint8 R,class Imp>
class Tensor<T,R,Imp>::reverse_iterator : public std::iterator<std::random_access_iterator_tag,T,Int32>
{
	public:
		typedef Tensor<T,R,Imp>						TensorType;
		typedef typename TensorType::ElementType	ElementType;
		typedef typename TensorType::IndexType		IndexType;
		
		reverse_iterator(TensorType& ref,const Int32& offs) : ref_(ref),iter_(offs) { }
		reverse_iterator(const reverse_iterator& source) : ref_(source.ref_),iter_(source.iter_) { }
		~reverse_iterator() { }
				
		reverse_iterator& operator=(const const_reverse_iterator&);
		reverse_iterator& operator=(const typename TensorType::IndexType&);
		
		bool operator==(const reverse_iterator& it) const						{ return iter_ == it.iter_; }
		bool operator!=(const reverse_iterator& it) const						{ return iter_ != it.iter_; }
		bool operator<(const reverse_iterator& it) const						{ return iter_ > it.iter_; }
		bool operator<=(const reverse_iterator& it) const						{ return iter_ >= it.iter_; }
		bool operator>(const reverse_iterator& it) const						{ return iter_ < it.iter_; }
		bool operator>=(const reverse_iterator& it) const						{ return iter_ <= it.iter_; }
		
		reverse_iterator&	operator+=(const Int32& num)						{ iter_ -= num; return *this; }
		reverse_iterator&	operator-=(const Int32& num)						{ iter_ += num; return *this; }
		
		Int32	operator-(const reverse_iterator& it) const						{ return it.iter_-iter_; }
		
		reverse_iterator&	operator++() 										{ --iter_; return *this; }
		reverse_iterator&	operator--()										{ ++iter_; return *this; }
		const reverse_iterator operator++(int)									{ reverse_iterator tmp(*this); --iter_; return tmp; }
		const reverse_iterator operator--(int)									{ reverse_iterator tmp(*this); ++iter_; return tmp; }
		
		reverse_iterator operator+(const Int32& num) const						{ reverse_iterator tmp(*this); tmp += num; return tmp; }
		reverse_iterator operator-(const Int32& num) const						{ reverse_iterator tmp(*this); tmp -= num; return tmp; }
		
		ElementType&	operator*()												{ return ref_.elementFromIterator(iter_); }
		ElementType&	operator[](const Int32& num)							{ iter_ = num; return ref_.elementFromIterator(iter_); }
		
		ElementType*	operator->()											{ return ref_.elementFromIterator(iter_).operator->(); }
		
		const IndexType index() const											{ return ref_.indexFromOffset(ref_.iteratorToOffset(iter_)); }
		const Int32& offset() const												{ return iter_; }
		const TensorType& tensor() const										{ return ref_; }
		
		class Err : public Exception { };
		
	private:		
		friend class TensorType::const_reverse_iterator;
		
		const TensorType&	ref_;
		Int32				iter_;
};

template<class T,Uint8 R,class Imp>
class Tensor<T,R,Imp>::const_reverse_iterator : public std::iterator<std::random_access_iterator_tag,T,Int32>
{
	public:
		typedef Tensor<T,R,Imp>						TensorType;
		typedef typename TensorType::ElementType	ElementType;
		typedef typename TensorType::IndexType		IndexType;
		
		const_reverse_iterator(const TensorType& ref,const Int32& offs) : ref_(ref),iter_(offs) { }
		const_reverse_iterator(const typename TensorType::reverse_iterator& source) : ref_(const_cast<const TensorType&>(source.tensor())),iter_(source.offset()) { }
		const_reverse_iterator(const const_reverse_iterator& source) : ref_(source.ref_),iter_(source.iter_) { }
		~const_reverse_iterator() { }
				
		const_reverse_iterator& operator=(const const_reverse_iterator&);
		const_reverse_iterator& operator=(const typename TensorType::IndexType&);
		
		bool operator==(const const_reverse_iterator& it) const					{ return iter_ == it.iter_; }
		bool operator!=(const const_reverse_iterator& it) const					{ return iter_ != it.iter_; }
		bool operator<(const const_reverse_iterator& it) const					{ return iter_ > it.iter_; }
		bool operator<=(const const_reverse_iterator& it) const					{ return iter_ >= it.iter_; }
		bool operator>(const const_reverse_iterator& it) const					{ return iter_ < it.iter_; }
		bool operator>=(const const_reverse_iterator& it) const					{ return iter_ <= it.iter_; }
		
		const_reverse_iterator&	operator+=(const Int32& num)					{ iter_ -= num; return *this; }
		const_reverse_iterator&	operator-=(const Int32& num)					{ iter_ += num; return *this; }
		
		Int32	operator-(const const_reverse_iterator& it) const				{ return it.iter_-iter_; }
		
		const_reverse_iterator&	operator++() 									{ --iter_; return *this; }
		const_reverse_iterator&	operator--()									{ ++iter_; return *this; }
		const const_reverse_iterator operator++(int)							{ const_reverse_iterator tmp(*this); --iter_; return tmp; }
		const const_reverse_iterator operator--(int)							{ const_reverse_iterator tmp(*this); ++iter_; return tmp; }
		
		const_reverse_iterator operator+(const Int32& num) const				{ const_reverse_iterator tmp(*this); tmp += num; return tmp; }
		const_reverse_iterator operator-(const Int32& num) const				{ const_reverse_iterator tmp(*this); tmp -= num; return tmp; }
		
		const ElementType&	operator*()											{ return ref_.elementFromIterator(iter_); }
		const ElementType&	operator[](const Int32& num)						{ iter_ = num; return ref_.elementFromIterator(iter_); }
		
		const ElementType*	operator->()										{ return ref_.elementFromIterator(iter_).operator->(); }
		
		const IndexType index() const											{ return ref_.indexFromOffset(ref_.iteratorToOffset(iter_)); }
		const Int32& offset() const												{ return iter_; }
		const TensorType& tensor() const										{ return ref_; }
		
		class Err : public Exception { };
		
	private:		
		const TensorType&	ref_;
		Int32				iter_;
};
