// constructor
// zero offset vector
template<class T> inline Vector<T>::Vector() : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{
	alloc(IndexRange<1>());
}

template<class T> inline Vector<T>::Vector(const IndexRange<1>& rng) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{
	alloc(rng);
	set(T());
}

template<class T> inline Vector<T>::Vector(const IndexRange<1>& rng,const T& val) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{
	alloc(rng);
	set(val);
}

template<class T> inline Vector<T>::Vector(const IndexRange<1>& rng,const T* ptr) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{
	alloc(rng);
	set(ptr);
}

template<class T> inline Vector<T>::Vector(const IndexRange<1>& rng,const std::string& str_init) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{	
	alloc(rng);
	set(str_init);
}
	
template<class T> inline Vector<T>::Vector(const VectorIndirectionSlice<T>& ind) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{	
	alloc(ind.ind_.range());
	set(ind);
}

template<class T> inline Vector<T>::Vector(const VectorMaskSlice<T>& msk) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{	
	alloc(IndexRange<1>(msk.vec_.first(),msk.vec_.first()+msk.vec_.elem()-1));
	set(msk);
}

template<class T> inline Vector<T>::Vector(const VectorRangeSlice<T>& rng) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{	
	alloc(rng.rng_);
	set(rng);
}

// copy constructor
template<class T> inline Vector<T>::Vector(const Vector<T>& source) : 
	rng_(),valid_(false),vec_data_(NULL),vec_ptr_(NULL)
{
	realloc(source.range());
	set(source.dataPointer());	
}

// destructor
template<class T> inline Vector<T>::~Vector()
{ 
	dealloc(); 
}

// arbitrary offset vector
template<class T> inline void Vector<T>::alloc(const IndexRange<1>& rng) 
{
	if (all(rng.dim() >= 1))
	{
		rng_ = rng;
		
		try 
		{ 
			vec_ptr_ = new T[static_cast<Int32>(rng_.elem())]; 
		}
		catch (...) 
		{ 
			rng_ = IndexRange<1>();  
			valid_ = false;
			vec_ptr_ = vec_data_ = NULL;
			throw MemoryErr(); 
		}
		
		// need to set valid first to avoid debug crash
		valid_ = true;
		vec_data_ = vec_ptr_-first();
	}
	else
	{
		rng_ = IndexRange<1>();
		valid_ = false;
		vec_ptr_ = vec_data_ = NULL;
	}
}

// deallocator
template<class T> inline void Vector<T>::dealloc()
{
	rng_ = IndexRange<1>();
	valid_ = false;	
	
	if (vec_ptr_ != NULL)
		delete[] vec_ptr_; 

	vec_ptr_ = vec_data_ = NULL;
}

// reallocator
template<class T> inline void Vector<T>::realloc(const IndexRange<1>& rng)
{
	// check to see if new range is same as old
	if (rng == rng_)
	{
		return;
	}
	// just change range if new dimension is same as old
	else if (all(rng.dim() == rng_.dim()))
	{
		range(rng);
		return;
	}
	// otherwise actually reallocate
	else
	{
		dealloc();
		alloc(rng);
		set(T());
		return;
	}
}

// assignment
template<class T> inline Vector<T>& Vector<T>::operator=(const T& val)
{ 	
	set(val); 
	return (*this); 
}

template<class T> inline Vector<T>& Vector<T>::operator=(const std::string& str_init)
{ 
	set(str_init); 
	return *this; 
}

template<class T> inline Vector<T>& Vector<T>::operator=(const IndexRange<1>& rng)
{ 
	realloc(rng); 
	return *this; 
}

template<class T> inline Vector<T>& Vector<T>::operator=(const Vector<T>& source)
{
	if (this == &source) return *this;
	
	realloc(source.rng_);
	set(source.dataPointer());
	
	return *this;
}		

template<class T> inline Vector<T>& Vector<T>::operator=(const VectorIndirectionSlice<T>& ind)
{
	if (this == &(ind.vec_)) throw IllegalOperationErr();
	
	realloc(ind.ind_.range());
	set(ind);
	
	return *this;
}		

template<class T> inline Vector<T>& Vector<T>::operator=(const VectorMaskSlice<T>& msk)
{
	if (this == &(msk.vec_)) throw IllegalOperationErr();
	
	realloc(IndexRange<1>(msk.vec_.first(),msk.vec_.first()+msk.msk_.trueElements()-1));
	set(msk);
	
	return *this;
}		

template<class T> inline Vector<T>& Vector<T>::operator=(const VectorRangeSlice<T>& rng)
{
	if (this == &(rng.vec_)) throw IllegalOperationErr();
	
	realloc(rng.rng_);
	set(rng);
	
	return *this;
}		

// element assignors
// initialize with value
template<class T> void Vector<T>::set(const T& val)
{ 
	for (Int32 i=first();i<=last();(*this)[i++]=val) { };
}

// initialize with array
template<class T> void Vector<T>::set(const T* ptr)
{ 
	if (ptr == NULL && isValid()) throw MemoryErr();
	
	if (!isValid())
	{
		return;
	}
	else
	{
		//for (Int32 i=first(),j=0;i<=last();(*this)[i++] = data[j++]) { }; 
		STD_(memcpy(reinterpret_cast<char*>(dataPointer()),reinterpret_cast<char*>(const_cast<T*>(ptr)),elem()*sizeof(T)));
	}
}

template<class T> void Vector<T>::set(const std::string& str_init)
{	
	std::stringstream 	temp1,temp2;	
	T					temp_val;
	Uint32 				temp_idx = 0;
	
	temp1 << str_init;
	
	// count number of elements in initializer string
	while (temp1 >> temp_val) temp_idx++;

	if (!isValid()) realloc(IndexRange<1>(temp_idx));
	
	if (temp_idx != dim()) throw SizeMismatchErr();
	
	temp_idx = first();
	temp2 << str_init;
	
	while (temp2 >> temp_val) 
		(*this)[temp_idx++] = temp_val;
}

template<class T> void Vector<T>::set(const VectorIndirectionSlice<T>& ind)
{
	if (isValid() && range() == ind.ind_.range())
	{
		for (Int32 i=first();i<=last();i++)
			(*this)[i] = ind.vec_[ind.ind_[i]];
	}
}

template<class T> void Vector<T>::set(const VectorMaskSlice<T>& msk)
{
	if (isValid() && elem() == msk.msk_.trueElements())
	{
		for (Int32 i=msk.vec_.first(),ii=first();i<=msk.vec_.last();i++)
			if (msk.vec_[i])
				(*this)[ii++] = msk.vec_[i];
	}
}

template<class T> inline void Vector<T>::set(const VectorRangeSlice<T>& rng)
{
	(*this) = rng.vec_alias_;
}

// pretend Vector is located at ptr, with specified size
// 	- only allowed on default allocated, invalid Vectors
//	- N.B. need to call unplace before destructor is reached or aliased memory can be released accidentally
template<class T> inline void Vector<T>::place(T* ptr,const IndexRange<1>& rng)
{
	if (isValid() || ptr == NULL || rng.elem() <= 0) throw IllegalOperationErr();
	
	valid_ = true;
	rng_ = rng;
	vec_ptr_ = ptr;
	vec_data_ = vec_ptr_-first();
}

template<class T> inline void Vector<T>::unplace()
{
	valid_ = false;
	rng_ = IndexRange<1>();
	vec_ptr_ = vec_data_ = NULL;
}

// safe clone of data located at pointer
//	- does not realloc
template<class T> inline void Vector<T>::clone(const T* ptr)
{
	set(ptr);
}

// range assignors
template<class T> inline const Vector<T>& Vector<T>::offset(const Int32& offs) const 
{ 
	range(IndexRange<1>(offs,offs+elem()-1)); 
	
	return (*this); 
}

template<class T> inline const Vector<T>& Vector<T>::range(const IndexRange<1>& rng) const
{
	if (rng.dim() != dim()) throw SizeMismatchErr();
	
	rng_ = rng;
	vec_data_ = vec_ptr_ - first();
	
	return (*this);
}

// element accessors
template<class T> inline T& Vector<T>::operator[](Int32 idx)
#if VECTOR_RANGE_CHECKING
#if CERR_DEBUG
{ 
	if (idx < first() || idx > last()) 
		std::cerr << "Access error [" << idx << "] in " << __FILE__ << ", line: " << __LINE__ << std::endl; 
		
	return vec_data_[idx]; 
}
#elif THROW_DEBUG
{ 
	if (idx < first() || idx > last()) throw RangeErr(); 
	
	return vec_data_[idx]; 
}
#else
{ 
	return vec_data_[idx]; 
}
#endif
#else
{ 
	return vec_data_[idx]; }
#endif

template<class T> inline const T& Vector<T>::operator[](Int32 idx) const
#if VECTOR_RANGE_CHECKING
#if CERR_DEBUG
{ 
	if (idx < first() || idx > last()) 
		std::cerr << "Access error [" << idx << "] in " << __FILE__ << ", line: " << __LINE__ << std::endl; 
		
	return vec_data_[idx]; 
}
#elif THROW_DEBUG
{ 
	if (idx < first() || idx > last()) throw RangeErr(); 
	
	return vec_data_[idx]; 
}
#else
{ 
	return vec_data_[idx]; 
}
#endif
#else
{ 
	return vec_data_[idx]; 
}
#endif

template<class T> inline VectorIndirectionSlice<T> Vector<T>::operator[](const VectorIndirection& ind)
{
	return VectorIndirectionSlice<T>((*this),ind);
}

template<class T> inline VectorMaskSlice<T> Vector<T>::operator[](const VectorMask& msk)
{
	return VectorMaskSlice<T>((*this),msk);
}

template<class T> inline VectorRangeSlice<T> Vector<T>::operator[](const IndexRange<1>& rng)
{
	return VectorRangeSlice<T>((*this),rng);
}

template<class T> inline const VectorIndirectionSlice<T> Vector<T>::operator[](const VectorIndirection& ind) const
{
	return VectorIndirectionSlice<T>((*this),ind);
}

template<class T> inline const VectorMaskSlice<T> Vector<T>::operator[](const VectorMask& msk) const
{
	return VectorMaskSlice<T>((*this),msk);
}

template<class T> inline const VectorRangeSlice<T> Vector<T>::operator[](const IndexRange<1>& rng) const
{
	return VectorRangeSlice<T>((*this),rng);
}

// arithmetic operators
// scalar member assign/equals operators
template<class T> Vector<T>& Vector<T>::operator+=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	for (Int32 i=first();i<=last();(*this)[i++]+=val) { };
	
	return *this;
}

template<class T> Vector<T>& Vector<T>::operator-=(const T& val)
{
	if (val == static_cast<T>(0)) return *this;
		
	for (Int32 i=first();i<=last();(*this)[i++]-=val) { };
	
	return *this;
}

template<class T> Vector<T>& Vector<T>::operator*=(const T& val)
{
	if (val == static_cast<T>(1)) return *this;
		
	for (Int32 i=first();i<=last();(*this)[i++]*=val) { };
	
	return *this;
}

template<class T> inline Vector<T>& Vector<T>::operator/=(const T& val)
{
	(*this) *= static_cast<T>(1)/val;
	
	return *this;
}

// Vector member assign/equals operators
template<class T> Vector<T>& Vector<T>::operator+=(const Vector<T>& source)
{
	if (!isValid()) return (*this = source);
	
	if (source.range() != range()) throw RangeErr();

	for (Int32 i=first();i<=last();i++) 
		(*this)[i] += source[i];
	
	return *this;
}

template<class T> Vector<T>& Vector<T>::operator-=(const Vector<T>& source)
{
	if (!isValid()) return (*this = -source);
	
	if (source.range() != range()) throw RangeErr();

	for (Int32 i=first();i<=last();i++) 
		(*this)[i] -= source[i];
	
	return *this;
}

template<class T> Vector<T>& Vector<T>::operator*=(const Vector<T>& source)
{
	if (!isValid()) return (*this = Vector<T>(source.range()));
	
	if (source.range() != range()) throw RangeErr();

	for (Int32 i=first();i<=last();i++) 
		(*this)[i] *= source[i];
	
	return *this;
}

template<class T> Vector<T>& Vector<T>::operator/=(const Vector<T>& source)
{
	if (!isValid()) return (*this = Vector<T>(source.range()));
	
	if (source.range() != range()) throw RangeErr();

	for (Int32 i=first();i<=last();i++) 
		(*this)[i] /= source[i];
	
	return *this;
}
