#ifndef MCS_VECTORFUNCTIONS_H
#define MCS_VECTORFUNCTIONS_H

#include "MCS_Ensure.h"
#include "MCS_MathExtensions.h"
#include "MCS_Promote.h"
#include "MCS_Vector.h"

namespace mcs {

// in place
// arithmetic functions
template<class T> const Vector<T>& Abs(Vector<T>& source);
template<class T> const Vector<T>& Ceil(Vector<T>& source);
template<class T> const Vector<T>& Exp(Vector<T>& source);
template<class T> const Vector<T>& Floor(Vector<T>& source);
template<class T> const Vector<T>& Log(Vector<T>& source);
template<class T> const Vector<T>& Pow(Vector<T>& source,const T& expon);
template<class T> const Vector<T>& Round(Vector<T>& source);
template<class T> const Vector<T>& Sqr(Vector<T>& source);
template<class T> const Vector<T>& Sqrt(Vector<T>& source);

// trigonometric functions
template<class T> const Vector<T>& Acos(Vector<T>& source);
template<class T> const Vector<T>& Asin(Vector<T>& source);
template<class T> const Vector<T>& Atan(Vector<T>& source);
template<class T> const Vector<T>& Cos(Vector<T>& source);
template<class T> const Vector<T>& Sin(Vector<T>& source); 
template<class T> const Vector<T>& Tan(Vector<T>& source);

// hyperbolic functions
template<class T> const Vector<T>& Acosh(Vector<T>& source);
template<class T> const Vector<T>& Asinh(Vector<T>& source);
template<class T> const Vector<T>& Atanh(Vector<T>& source);
template<class T> const Vector<T>& Cosh(Vector<T>& source);
template<class T> const Vector<T>& Sinh(Vector<T>& source); 
template<class T> const Vector<T>& Tanh(Vector<T>& source);

// complex vectors
template<class T> const Vector<T>& Conj(Vector<T>& source);
template<class T> const Vector<T>& Imag(Vector<T>& source);
template<class T> const Vector<T>& Real(Vector<T>& source);

// not in place 
// arithmetic functions
template<class T> Vector<T> abs(const Vector<T>& source);
template<class T> Vector<T> ceil(const Vector<T>& source);
template<class T> Vector<T> exp(const Vector<T>& source);
template<class T> Vector<T> floor(const Vector<T>& source);
template<class T> Vector<T> log(const Vector<T>& source);
template<class T> Vector<T> pow(const Vector<T>& source,const T& expon);
template<class T> Vector<T> round(const Vector<T>& source);
template<class T> Vector<T> sqr(const Vector<T>& source);
template<class T> Vector<T> sqrt(const Vector<T>& source);

// trigonometric functions
template<class T> Vector<T> acos(const Vector<T>& source);
template<class T> Vector<T> asin(const Vector<T>& source);
template<class T> Vector<T> atan(const Vector<T>& source);
template<class T> Vector<T> atan2(const Vector<T>& y,const Vector<T>& x);
template<class T> Vector<T> cos(const Vector<T>& source);
template<class T> Vector<T> sin(const Vector<T>& source); 
template<class T> Vector<T> tan(const Vector<T>& source);

// hyperbolic functions
template<class T> Vector<T> acosh(const Vector<T>& source);
template<class T> Vector<T> asinh(const Vector<T>& source);
template<class T> Vector<T> atanh(const Vector<T>& source);
template<class T> Vector<T> cosh(const Vector<T>& source);
template<class T> Vector<T> sinh(const Vector<T>& source); 
template<class T> Vector<T> tanh(const Vector<T>& source);
 
// complex vectors
template<class T> Vector<T> conj(const Vector<T>& source);
template<class T> Vector<typename DemoteComplex<T>::demote_t> imag(const Vector<T>& source);
template<class T> Vector<typename DemoteComplex<T>::demote_t> real(const Vector<T>& source);

// miscellaneous
template<class T> Vector<T> bound(const Vector<T>& x,const T& min_val,const T& max_val);
template<class T> Vector<Binary> even(const Vector<T>& x);
template<class T> Vector<Binary> odd(const Vector<T>& x);
template<class T> Vector<T> mod(const Vector<T>& x,const T& y);
template<class T> Vector<T> mod(const Vector<T>& x,const Vector<T>& y);
template<class T> Vector<T> quotient(const Vector<T>& x,const T& y);
template<class T> Vector<T> quotient(const Vector<T>& x,const Vector<T>& y);

// vector specific
template<class T> bool all(const Vector<T>&);
template<class T> bool any(const Vector<T>&);

template<class T> T dot(const Vector<T>& lhs,const Vector<T>& rhs);
template<class T> T mag(const Vector<T>& source);
template<class T> T norm(const Vector<T>& source);	
template<class T> T prod(const Vector<T>& source);		
template<class T> T sum(const Vector<T>& source);

// in place
// arithmetic functions
template<class T> const Vector<T>& Abs(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::abs(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Ceil(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::ceil(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Exp(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::exp(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Floor(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::floor(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Log(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::log(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Pow(Vector<T>& source,const T& expon)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::pow(source[i],expon);
	
	return source;
}

template<class T> const Vector<T>& Round(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::round(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Sqr(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] *= source[i];
	
	return source;
}

template<class T> const Vector<T>& Sqrt(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sqrt(source[i]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> const Vector<T>& Conj(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::conj(source[i]);
		
	return source;
}

template<class T> const Vector<T>& Imag(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::imag(source[i]);
		
	return source;
}

template<class T> const Vector<T>& Real(Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::real(source[i]);
		
	return source;
}

// trigonometric functions
template<class T> const Vector<T>& Sin(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sin(source[i]);

	return source;
}

template<class T> const Vector<T>& Cos(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cos(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Tan(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tan(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Asin(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asin(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Acos(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acos(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Atan(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atan(source[i]);
	
	return source;
}

// hyperbolic functions
template<class T> const Vector<T>& Sinh(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sinh(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Cosh(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cosh(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Tanh(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tanh(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Asinh(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asinh(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Acosh(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acosh(source[i]);
	
	return source;
}

template<class T> const Vector<T>& Atanh(Vector<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atanh(source[i]);

	return source;
}

// not in place
// arithmetic functions
template<class T> inline Vector<T> abs(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Abs(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> ceil(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Ceil(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> exp(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Exp(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> floor(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Floor(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> log(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Log(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> pow(const Vector<T>& source,const T& expon)
{
	Vector<T>	tmp(source);
	
	Pow(tmp,expon);
	
	return tmp;
}

template<class T> inline Vector<T> round(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Round(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> sqr(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Sqr(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> sqrt(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Sqrt(tmp);
	
	return tmp;
}

// complex vectors
template<class T> inline Vector<T> conj(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Conj(tmp);
	
	return tmp;
}

template<class T> inline Vector<typename DemoteComplex<T>::demote_t> imag(const Vector<T>& source)
{
	Vector<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Imag(tmp);
	
	return tmp;
}

template<class T> Vector<T> imag(const Vector< std::complex<T> >& source)
{
	Vector<T>	tmp(source.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = source[i].imag();
	
	return tmp;
}

template<class T> inline Vector<typename DemoteComplex<T>::demote_t> real(const Vector<T>& source)
{
	Vector<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Real(tmp);
	
	return tmp;
}

template<class T> Vector<T> real(const Vector< std::complex<T> >& source)
{
	Vector<T>	tmp(source.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = source[i].real();
	
	return tmp;
}

// trigonometric functions
template<class T> inline Vector<T> sin(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Sin(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> cos(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Cos(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> tan(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Tan(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> asin(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Asin(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> acos(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Acos(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> atan(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Atan(tmp);
	
	return tmp;
}

template<class T> Vector<T> atan2(const Vector<T>& y,const Vector<T>& x)
{
	static const EnsureFloat<T>	dummy_;
	
	if (x.range() != y.range()) throw typename Vector<T>::SizeMismatchErr();

	Vector<T>	tmp(y.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::atan2(y[i],x[i]);
		
	return tmp;
}

// hyperbolic functions
template<class T> inline Vector<T> sinh(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Sinh(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> cosh(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Cosh(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> tanh(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Tanh(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> asinh(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Asinh(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> acosh(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Acosh(tmp);
	
	return tmp;
}

template<class T> inline Vector<T> atanh(const Vector<T>& source)
{
	Vector<T>	tmp(source);
	
	Atanh(tmp);
	
	return tmp;
}

// miscellaneous
template<class T> Vector<T> bound(const Vector<T>& x,const T& min_val,const T& max_val)
{
	Vector<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::bound(x[i],min_val,max_val);

	return tmp;
}

template<class T> Vector<Binary> even(const Vector<T>& x)
{
	Vector<Binary>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::even(x[i]);

	return tmp;
}

template<class T> Vector<Binary> odd(const Vector<T>& x)
{
	Vector<Binary>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::odd(x[i]);

	return tmp;
}

template<class T> Vector<T> mod(const Vector<T>& x,const T& y)
{
	Vector<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::mod(x[i],y);

	return tmp;
}

template<class T> Vector<T> mod(const Vector<T>& x,const Vector<T>& y)
{
	if (x.range() != y.range()) throw typename Vector<T>::SizeMismatchErr();
	
	Vector<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::mod(x[i],y[i]);

	return tmp;
}

template<class T> Vector<T> quotient(const Vector<T>& x,const T& y)
{
	Vector<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::quotient(x[i],y);

	return tmp;
}

template<class T> Vector<T> quotient(const Vector<T>& x,const Vector<T>& y)
{
	if (x.range() != y.range()) throw typename Vector<T>::SizeMismatchErr();
	
	Vector<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::quotient(x[i],y[i]);

	return tmp;
}

// vector specific
template<class T> bool all(const Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (!source[i]) return false;
		
	return true;
}

template<class T> bool any(const Vector<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (source[i]) return true;
		
	return false;
}

// dot product
template<class T> inline T dot(const Vector<T>& lhs,const Vector<T>& rhs)
{	
	return sum(lhs*rhs);
}

// vector magnitude	
template<class T> inline T mag(const Vector<T>& source)
{
	return std::sqrt(norm(source));
}

// vector norm
template<class T> inline T norm(const Vector<T>& source)
{
	return sum(sqr(source));
}

// product of all elements
template<class T> T product(const Vector<T>& source)
{
	T tprod(1);
	
	for (Int32 i=source.first();i<=source.last();tprod *= source[i++]) { };
	
	return tprod;
}

// sum of all elements
template<class T> T sum(const Vector<T>& source)
{
	T tsum(0);
	
	for (Int32 i=source.first();i<=source.last();tsum += source[i++]) { };
	
	return tsum;
}

} // namespace mcs

#ifdef __INTEL__
	#if USE_INTEL_SPL
		#include "MCS_VectorFunctions_IntelSpecialized.h"
	#endif
#endif

#endif // MCS_VECTORFUNCTIONS_H
