#ifndef MCS_VECTORFUNCTIONSINTELSPECIALIZED_H
#define MCS_VECTORFUNCTIONSINTELSPECIALIZED_H

#include "MCS_Constants.h"
#include "MCS_Vector.h"

#include "MCS_IntelNSP.h"

#if USE_INTEL_SPL

namespace mcs {

//! optimized implementations using Intel SPL

// in place
template<> inline const Vector<Float32>& Abs(Vector<Float32>& source)
{
	if (source.isValid()) 
		nspsbAbs1(reinterpret_cast<Float32*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float64>& Abs(Vector<Float64>& source)
{
	if (source.isValid()) 
		nspdbAbs1(reinterpret_cast<Float64*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float32>& Exp(Vector<Float32>& source) 
{
	if (source.isValid()) 
		nspsbExp1(reinterpret_cast<Float32*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float64>& Exp(Vector<Float64>& source) 
{
	if (source.isValid()) 
		nspdbExp1(reinterpret_cast<Float64*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float32>& Log(Vector<Float32>& source) 
{
	if (source.isValid()) 
		nspsbLn1(reinterpret_cast<Float32*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float64>& Log(Vector<Float64>& source) 
{
	if (source.isValid()) 
		nspdbLn1(reinterpret_cast<Float64*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float32>& Sqr(Vector<Float32>& source)  
{
	if (source.isValid()) 
		nspsbSqr1(reinterpret_cast<Float32*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float64>& Sqr(Vector<Float64>& source)  
{
	if (source.isValid()) 
		nspdbSqr1(reinterpret_cast<Float64*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Complex32>& Sqr(Vector<Complex32>& source)
{
	if (source.isValid()) 
		nspcbSqr1(reinterpret_cast<SCplx*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Complex64>& Sqr(Vector<Complex64>& source)
{
	if (source.isValid()) 
		nspzbSqr1(reinterpret_cast<DCplx*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float32>& Sqrt(Vector<Float32>& source) 
{
	if (source.isValid()) 
		nspsbSqrt1(reinterpret_cast<Float32*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Float64>& Sqrt(Vector<Float64>& source) 
{
	if (source.isValid()) 
		nspdbSqrt1(reinterpret_cast<Float64*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Complex32>& Sqrt(Vector<Complex32>& source) 
{
	if (source.isValid()) 
		nspcbSqrt1(reinterpret_cast<SCplx*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

template<> inline const Vector<Complex64>& Sqrt(Vector<Complex64>& source) 
{
	if (source.isValid()) 
		nspzbSqrt1(reinterpret_cast<DCplx*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}
	
template<> inline const Vector<Complex32>& Conj(Vector<Complex32>& source)
{ 
	if (source.isValid()) 
		nspcbConj1(reinterpret_cast<SCplx*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;	
}

template<> inline const Vector<Complex64>& Conj(Vector<Complex64>& source)
{ 
	if (source.isValid()) 
		nspzbConj1(reinterpret_cast<DCplx*>(source.dataPointer()),static_cast<int>(source.dim()));
	
	return source;
}

// not in place
template<> inline Vector<Float32> abs(const Vector<Float32>& source)
{
	Vector<Float32>	tmp(source);
	
	if (source.isValid()) 
		nspsbAbs1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float64> abs(const Vector<Float64>& source)
{
	Vector<Float64>	tmp(source.range());
	
	if (source.isValid()) 
		nspdbAbs2(source.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float32> exp(const Vector<Float32>& source)
{
	Vector<Float32> tmp(source);
	
	if (source.isValid()) 
		nspsbExp1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float64> exp(const Vector<Float64>& source)
{
	Vector<Float64> tmp(source);
	
	if (source.isValid()) 
		nspdbExp1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float32> log(const Vector<Float32>& source)
{
	Vector<Float32> tmp(source);
	
	if (source.isValid()) 
		nspsbLn1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float64> log(const Vector<Float64>& source)
{
	Vector<Float64> tmp(source);
	
	if (source.isValid()) 
		nspdbLn1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float32> sqr(const Vector<Float32>& source)
{
	Vector<Float32> tmp(source);
	
	if (source.isValid()) 
		nspsbSqr1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float64> sqr(const Vector<Float64>& source)
{
	Vector<Float64> tmp(source);
	
	if (source.isValid()) 
		nspdbSqr1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Complex32> sqr(const Vector<Complex32>& source)
{
	Vector<Complex32> tmp(source);
	
	if (source.isValid()) 
		nspcbSqr1(reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Complex64> sqr(const Vector<Complex64>& source)
{
	Vector<Complex64> tmp(source);
	
	if (source.isValid()) 
		nspzbSqr1(reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Float32> sqrt(const Vector<Float32>& source)
{
	Vector<Float32> tmp(source);
	
	if (source.isValid()) 
		nspsbSqrt1(tmp.dataPointer(),static_cast<int>(tmp.dim()));

	return tmp;
}

template<> inline Vector<Float64> sqrt(const Vector<Float64>& source)
{
	Vector<Float64> tmp(source);
	
	if (source.isValid()) 
		nspdbSqrt1(tmp.dataPointer(),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Complex32> sqrt(const Vector<Complex32>& source)
{
	Vector<Complex32> tmp(source);
	
	if (source.isValid()) 
		nspcbSqrt1(reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Complex64> sqrt(const Vector<Complex64>& source)
{
	Vector<Complex64> tmp(source);
	
	if (source.isValid()) 
		nspzbSqrt1(reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Complex32> conj(const Vector<Complex32>& source)
{
	Vector<Complex32>	tmp(source);

	if (source.isValid()) nspcbConj1(reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Vector<Complex64> conj(const Vector<Complex64>& source)
{
	Vector<Complex64>	tmp(source);
	
	if (source.isValid()) nspzbConj1(reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
	
	return tmp;
}

template<> inline Float32 dot(const Vector<Float32>& lhs,const Vector<Float32>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Float32>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return nspsDotProd(lhs.dataPointer(),rhs.dataPointer(),static_cast<int>(lhs.dim()));
	else
		return _NaN_;
}

template<> inline Float64 dot(const Vector<Float64>& lhs,const Vector<Float64>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Float64>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return nspdDotProd(lhs.dataPointer(),rhs.dataPointer(),static_cast<int>(lhs.dim()));
	else
		return _NaN_;
}

template<> inline Complex32 dot(const Vector<Complex32>& lhs,const Vector<Complex32>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Complex32>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return scplx(nspcDotProd(reinterpret_cast<SCplx*>(lhs.dataPointer()),
								 reinterpret_cast<SCplx*>(rhs.dataPointer()),static_cast<int>(lhs.dim())));
	else
		return _NaN_;
}

template<> inline Complex64 dot(const Vector<Complex64>& lhs,const Vector<Complex64>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Complex64>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return dcplx(nspzDotProd(reinterpret_cast<DCplx*>(lhs.dataPointer()),
								 reinterpret_cast<DCplx*>(rhs.dataPointer()),static_cast<int>(lhs.dim())));
	else
		return _NaN_;
}

inline Complex32 dot(const Vector<Float32>& lhs,const Vector<Complex32>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Complex32>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return scplx(nspscDotProd(lhs.dataPointer(),
								  reinterpret_cast<SCplx*>(rhs.dataPointer()),static_cast<int>(lhs.dim())));
	else
		return _NaN_;
}

inline Complex32 dot(const Vector<Complex32>& lhs,const Vector<Float32>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Complex32>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return scplx(nspcsDotProd(reinterpret_cast<SCplx*>(lhs.dataPointer()),
								  rhs.dataPointer(),static_cast<int>(lhs.dim())));
	else
		return _NaN_;
}

inline Complex64 dot(const Vector<Float64>& lhs,const Vector<Complex64>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Complex64>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return dcplx(nspdzDotProd(lhs.dataPointer(),
								  reinterpret_cast<DCplx*>(rhs.dataPointer()),static_cast<int>(lhs.dim())));
	else
		return _NaN_;
}

inline Complex64 dot(const Vector<Complex64>& lhs,const Vector<Float64>& rhs)
{	
	if (lhs.range() != rhs.range()) throw typename Vector<Complex64>::SizeMismatchErr();
	
	if (lhs.isValid() && rhs.isValid()) 
		return dcplx(nspzdDotProd(reinterpret_cast<DCplx*>(lhs.dataPointer()),
								  rhs.dataPointer(),static_cast<int>(lhs.dim())));
	else
		return _NaN_;
}

} // namespace mcs

#endif // USE_INTEL_SPL

#endif // MCS_VECTORFUNCTIONSINTELSPECIALIZED_H