#ifndef MCS_VECTORINDIRECTIONSLICEFUNCTIONS_H
#define MCS_VECTORINDIRECTIONSLICEFUNCTIONS_H

#include "MCS_Ensure.h"
#include "MCS_MathExtensions.h"
#include "MCS_Promote.h"
#include "MCS_VectorIndirectionSlice.h"

namespace mcs {

// in place
// arithmetic functions
template<class T> const VectorIndirectionSlice<T>& Abs(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Ceil(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Exp(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Floor(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Log(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Pow(VectorIndirectionSlice<T>& source,const T& expon);
template<class T> const VectorIndirectionSlice<T>& Round(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Sqr(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Sqrt(VectorIndirectionSlice<T>& source);

// trigonometric functions
template<class T> const VectorIndirectionSlice<T>& Acos(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Asin(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Atan(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Cos(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Sin(VectorIndirectionSlice<T>& source); 
template<class T> const VectorIndirectionSlice<T>& Tan(VectorIndirectionSlice<T>& source);

// hyperbolic functions
template<class T> const VectorIndirectionSlice<T>& Acosh(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Asinh(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Atanh(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Cosh(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Sinh(VectorIndirectionSlice<T>& source); 
template<class T> const VectorIndirectionSlice<T>& Tanh(VectorIndirectionSlice<T>& source);

// complex vectors
template<class T> const VectorIndirectionSlice<T>& Conj(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Imag(VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T>& Real(VectorIndirectionSlice<T>& source);

// not in place 
// arithmetic functions
template<class T> VectorIndirectionSlice<T> abs(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> ceil(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> exp(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> floor(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> log(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> pow(const VectorIndirectionSlice<T>& source,const T& expon);
template<class T> VectorIndirectionSlice<T> round(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> sqr(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> sqrt(const VectorIndirectionSlice<T>& source);

// trigonometric functions
template<class T> VectorIndirectionSlice<T> acos(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> asin(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> atan(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> atan2(const VectorIndirectionSlice<T>& y,const VectorIndirectionSlice<T>& x);
template<class T> VectorIndirectionSlice<T> cos(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> sin(const VectorIndirectionSlice<T>& source); 
template<class T> VectorIndirectionSlice<T> tan(const VectorIndirectionSlice<T>& source);

// hyperbolic functions
template<class T> VectorIndirectionSlice<T> acosh(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> asinh(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> atanh(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> cosh(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> sinh(const VectorIndirectionSlice<T>& source); 
template<class T> VectorIndirectionSlice<T> tanh(const VectorIndirectionSlice<T>& source);
 
// complex vectors
template<class T> VectorIndirectionSlice<T> conj(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<typename DemoteComplex<T>::demote_t> imag(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<typename DemoteComplex<T>::demote_t> real(const VectorIndirectionSlice<T>& source);

// miscellaneous
template<class T> VectorIndirectionSlice<T> bound(const VectorIndirectionSlice<T>& x,const T& min_val,const T& max_val);
template<class T> VectorIndirectionSlice<Binary> even(const VectorIndirectionSlice<T>& x);
template<class T> VectorIndirectionSlice<Binary> odd(const VectorIndirectionSlice<T>& x);
template<class T> VectorIndirectionSlice<T> mod(const VectorIndirectionSlice<T>& x,const T& y);
template<class T> VectorIndirectionSlice<T> mod(const VectorIndirectionSlice<T>& x,const VectorIndirectionSlice<T>& y);
template<class T> VectorIndirectionSlice<T> quotient(const VectorIndirectionSlice<T>& x,const T& y);
template<class T> VectorIndirectionSlice<T> quotient(const VectorIndirectionSlice<T>& x,const VectorIndirectionSlice<T>& y);

// vector specific
template<class T> bool all(const VectorIndirectionSlice<T>&);
template<class T> bool any(const VectorIndirectionSlice<T>&);

template<class T> T dot(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> T mag(const VectorIndirectionSlice<T>& source);
template<class T> T norm(const VectorIndirectionSlice<T>& source);	
template<class T> T prod(const VectorIndirectionSlice<T>& source);		
template<class T> T sum(const VectorIndirectionSlice<T>& source);

// in place
// arithmetic functions
template<class T> const VectorIndirectionSlice<T>& Abs(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::abs(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Ceil(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::ceil(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Exp(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::exp(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Floor(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::floor(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Log(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::log(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Pow(VectorIndirectionSlice<T>& source,const T& expon)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::pow(source[i],expon);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Round(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::round(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Sqr(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] *= source[i];
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Sqrt(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sqrt(source[i]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> const VectorIndirectionSlice<T>& Conj(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::conj(source[i]);
		
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Imag(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::imag(source[i]);
		
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Real(VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::real(source[i]);
		
	return source;
}

// trigonometric functions
template<class T> const VectorIndirectionSlice<T>& Sin(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sin(source[i]);

	return source;
}

template<class T> const VectorIndirectionSlice<T>& Cos(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cos(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Tan(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tan(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Asin(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asin(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Acos(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acos(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Atan(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atan(source[i]);
	
	return source;
}

// hyperbolic functions
template<class T> const VectorIndirectionSlice<T>& Sinh(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sinh(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Cosh(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cosh(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Tanh(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tanh(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Asinh(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asinh(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Acosh(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acosh(source[i]);
	
	return source;
}

template<class T> const VectorIndirectionSlice<T>& Atanh(VectorIndirectionSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atanh(source[i]);

	return source;
}

// not in place
// arithmetic functions
template<class T> inline VectorIndirectionSlice<T> abs(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Abs(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> ceil(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Ceil(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> exp(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Exp(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> floor(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Floor(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> log(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Log(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> pow(const VectorIndirectionSlice<T>& source,const T& expon)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Pow(tmp,expon);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> round(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Round(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> sqr(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Sqr(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> sqrt(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Sqrt(tmp);
	
	return tmp;
}

// complex vectors
template<class T> inline VectorIndirectionSlice<T> conj(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Conj(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<typename DemoteComplex<T>::demote_t> imag(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Imag(tmp);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> imag(const VectorIndirectionSlice< std::complex<T> >& source)
{
	VectorIndirectionSlice<T>	tmp(source.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = source[i].imag();
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<typename DemoteComplex<T>::demote_t> real(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Real(tmp);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> real(const VectorIndirectionSlice< std::complex<T> >& source)
{
	VectorIndirectionSlice<T>	tmp(source.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = source[i].real();
	
	return tmp;
}

// trigonometric functions
template<class T> inline VectorIndirectionSlice<T> sin(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Sin(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> cos(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Cos(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> tan(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Tan(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> asin(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Asin(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> acos(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Acos(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> atan(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Atan(tmp);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> atan2(const VectorIndirectionSlice<T>& y,const VectorIndirectionSlice<T>& x)
{
	static const EnsureFloat<T>	dummy_;
	
	if (x.range() != y.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();

	VectorIndirectionSlice<T>	tmp(y.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::atan2(y[i],x[i]);
		
	return tmp;
}

// hyperbolic functions
template<class T> inline VectorIndirectionSlice<T> sinh(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Sinh(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> cosh(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Cosh(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> tanh(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Tanh(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> asinh(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Asinh(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> acosh(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Acosh(tmp);
	
	return tmp;
}

template<class T> inline VectorIndirectionSlice<T> atanh(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T>	tmp(source);
	
	Atanh(tmp);
	
	return tmp;
}

// miscellaneous
template<class T> VectorIndirectionSlice<T> bound(const VectorIndirectionSlice<T>& x,const T& min_val,const T& max_val)
{
	VectorIndirectionSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::bound(x[i],min_val,max_val);

	return tmp;
}
/*
template<class T> VectorIndirectionSlice<Binary> even(const VectorIndirectionSlice<T>& x)
{
	VectorIndirectionSlice<Binary>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::even(x[i]);

	return tmp;
}

template<class T> VectorIndirectionSlice<Binary> odd(const VectorIndirectionSlice<T>& x)
{
	VectorIndirectionSlice<Binary>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::odd(x[i]);

	return tmp;
}
*/
template<class T> VectorIndirectionSlice<T> mod(const VectorIndirectionSlice<T>& x,const T& y)
{
	VectorIndirectionSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::mod(x[i],y);

	return tmp;
}

template<class T> VectorIndirectionSlice<T> mod(const VectorIndirectionSlice<T>& x,const VectorIndirectionSlice<T>& y)
{
	if (x.range() != y.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorIndirectionSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::mod(x[i],y[i]);

	return tmp;
}

template<class T> VectorIndirectionSlice<T> quotient(const VectorIndirectionSlice<T>& x,const T& y)
{
	VectorIndirectionSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::quotient(x[i],y);

	return tmp;
}

template<class T> VectorIndirectionSlice<T> quotient(const VectorIndirectionSlice<T>& x,const VectorIndirectionSlice<T>& y)
{
	if (x.range() != y.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorIndirectionSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::quotient(x[i],y[i]);

	return tmp;
}

// vector specific
template<class T> bool all(const VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (!source[i]) return false;
		
	return true;
}

template<class T> bool any(const VectorIndirectionSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (source[i]) return true;
		
	return false;
}

// dot product
template<class T> inline T dot(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{	
	return sum(lhs*rhs);
}

// vector magnitude	
template<class T> inline T mag(const VectorIndirectionSlice<T>& source)
{
	return std::sqrt(norm(source));
}

// vector norm
template<class T> inline T norm(const VectorIndirectionSlice<T>& source)
{
	return sum(sqr(source));
}

// product of all elements
template<class T> T product(const VectorIndirectionSlice<T>& source)
{
	T tprod(1);
	
	for (Int32 i=source.first();i<=source.last();tprod *= source[i++]) { };
	
	return tprod;
}

// sum of all elements
template<class T> T sum(const VectorIndirectionSlice<T>& source)
{
	T tsum(0);
	
	for (Int32 i=source.first();i<=source.last();tsum += source[i++]) { };
	
	return tsum;
}

} // namespace mcs

#endif // MCS_VECTORINDIRECTIONSLICEFUNCTIONS_H
