#ifndef MCS_VECTORINDIRECTIONSLICEOPERATORS_H
#define MCS_VECTORINDIRECTIONSLICEOPERATORS_H

#include "MCS_VectorIndirectionSlice.h"

namespace mcs {

// vector non-member operators

// logical operations		
template<class T> const VectorMask operator==(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator!=(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator<(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator<=(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator>(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator>=(const VectorIndirectionSlice<T>& lvec,const T& rscal);

template<class T> const VectorMask operator==(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorMask operator!=(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorMask operator<(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorMask operator<=(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorMask operator>(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorMask operator>=(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec);

// arithmetic operators
template<class T> const VectorIndirectionSlice<T> operator+(const T& lscal,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorIndirectionSlice<T> operator-(const T& lscal,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorIndirectionSlice<T> operator*(const T& lscal,const VectorIndirectionSlice<T>& rvec);
template<class T> const VectorIndirectionSlice<T> operator/(const T& lscal,const VectorIndirectionSlice<T>& rvec);

template<class T> const VectorIndirectionSlice<T> operator+(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorIndirectionSlice<T> operator-(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorIndirectionSlice<T> operator*(const VectorIndirectionSlice<T>& lvec,const T& rscal);
template<class T> const VectorIndirectionSlice<T> operator/(const VectorIndirectionSlice<T>& lvec,const T& rscal);

template<class T> const VectorIndirectionSlice<T> operator+(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> const VectorIndirectionSlice<T> operator-(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> const VectorIndirectionSlice<T> operator*(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> const VectorIndirectionSlice<T> operator/(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);

template<class T> const VectorMask operator&&(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> const VectorMask operator||(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);

template<class T> const VectorIndirectionSlice<T> operator+(const VectorIndirectionSlice<T>& source);
template<class T> const VectorIndirectionSlice<T> operator-(const VectorIndirectionSlice<T>& source);

template<class T> const VectorIndirectionSlice<T> operator!(const VectorIndirectionSlice<T>& source);

// logical operations		

// VectorIndirectionSlice op scalar
template<class T> const VectorMask operator==(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] == rscal);
		
	return tmp;
}

template<class T> const VectorMask operator!=(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] != rscal);
		
	return tmp;
}

template<class T> const VectorMask operator<(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] < rscal);
		
	return tmp;
}

template<class T> const VectorMask operator<=(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] <= rscal);
		
	return tmp;
}

template<class T> const VectorMask operator>(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] > rscal);
		
	return tmp;
}

template<class T> const VectorMask operator>=(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] >= rscal);
		
	return tmp;
}

// VectorIndirectionSlice op VectorIndirectionSlice
template<class T> const VectorMask operator==(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] == rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator!=(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] != rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator<(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] < rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator<=(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] <= rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator>(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] > rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator>=(const VectorIndirectionSlice<T>& lvec,const VectorIndirectionSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] >= rvec[i]);
		
	return tmp;
}

// arithmetic operators
// scalar op VectorIndirectionSlice
template<class T> inline const VectorIndirectionSlice<T> operator+(const T& lscal,const VectorIndirectionSlice<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (VectorIndirectionSlice<T>(rvec.range(),lscal) += rvec);
}

template<class T> inline const VectorIndirectionSlice<T> operator-(const T& lscal,const VectorIndirectionSlice<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return -rvec;
		
	return (VectorIndirectionSlice<T>(rvec.range(),lscal) -= rvec);
}

template<class T> inline const VectorIndirectionSlice<T> operator*(const T& lscal,const VectorIndirectionSlice<T>& rvec)
{
	if (lscal == static_cast<T>(1)) return rvec;
	if (lscal == static_cast<T>(0)) return VectorIndirectionSlice<T>(rvec.range(),static_cast<T>(0));
	
	return (VectorIndirectionSlice<T>(rvec.range(),lscal) *= rvec);
}

template<class T> inline const VectorIndirectionSlice<T> operator/(const T& lscal,const VectorIndirectionSlice<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (VectorIndirectionSlice<T>(rvec.range(),lscal) /= rvec);
}

// VectorIndirectionSlice op scalar
template<class T> inline const VectorIndirectionSlice<T> operator+(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (VectorIndirectionSlice<T>(lvec) += rscal);
}

template<class T> inline const VectorIndirectionSlice<T> operator-(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (VectorIndirectionSlice<T>(lvec) -= rscal);
}

template<class T> inline const VectorIndirectionSlice<T> operator*(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
	if (rscal == static_cast<T>(0)) return VectorIndirectionSlice<T>(lvec.range(),static_cast<T>(0));	

	return (VectorIndirectionSlice<T>(lvec) *= rscal);
}

template<class T> inline const VectorIndirectionSlice<T> operator/(const VectorIndirectionSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
		
	return (VectorIndirectionSlice<T>(lvec) /= rscal);
}

// VectorIndirectionSlice op VectorIndirectionSlice
template<class T> inline const VectorIndirectionSlice<T> operator+(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	return (VectorIndirectionSlice<T>(lhs) += rhs);
}

template<class T> inline const VectorIndirectionSlice<T> operator-(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
		
	return (VectorIndirectionSlice<T>(lhs) -= rhs);
}


template<class T> inline const VectorIndirectionSlice<T> operator*(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	return (VectorIndirectionSlice<T>(lhs) *= rhs);
}

template<class T> inline const VectorIndirectionSlice<T> operator/(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	return (VectorIndirectionSlice<T>(lhs) /= rhs);
}

template<class T> const VectorMask operator&&(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lhs.range());
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		tmp[i] = (lhs[i] && rhs[i]);
		
	return tmp;
}

template<class T> const VectorMask operator||(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lhs.range());
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		tmp[i] = (lhs[i] || rhs[i]);
		
	return tmp;
}

// unary operators
template<class T> inline const VectorIndirectionSlice<T> operator+(const VectorIndirectionSlice<T>& source)
{
	return source;
}

template<class T> inline const VectorIndirectionSlice<T> operator-(const VectorIndirectionSlice<T>& source)
{
	return (VectorIndirectionSlice<T>(source.range(),static_cast<T>(0)) -= source);
}

template<class T> const VectorIndirectionSlice<T> operator!(const VectorIndirectionSlice<T>& source)
{
	VectorIndirectionSlice<T> tmp(source);
	
	for (l i=source.first();i<=source.last();i++)
		tmp[i] = !source[i];
		
	return tmp;
}

} // namespace mcs

#endif // MCS_VECTORINDIRECTIONSLICEOPERATORS_H