#ifndef MCS_VECTORINDIRECTIONSLICESTATISTICS_H
#define MCS_VECTORINDIRECTIONSLICESTATISTICS_H

#include "MCS_Promote.h"
#include "MCS_VectorIndirectionSlice.h"

namespace mcs {

namespace statistics {

template<class T> T 							max(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> 	max(const VectorIndirectionSlice<T>& lhs,const T& rhs);
template<class T> VectorIndirectionSlice<T> 	max(const T& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> VectorIndirectionSlice<T> 	max(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);

template<class T> T								min(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> 	min(const VectorIndirectionSlice<T>& lhs,const T& rhs);
template<class T> VectorIndirectionSlice<T> 	min(const T& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> VectorIndirectionSlice<T> 	min(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);

template<class T> T								product(const VectorIndirectionSlice<T>& source);
template<class T> T								sum(const VectorIndirectionSlice<T>& source); 		

template<class T> typename PromoteFloat<T>::promote_t	mean(const VectorIndirectionSlice<T>& source);		
template<class T> typename PromoteFloat<T>::promote_t	standardDeviation(const VectorIndirectionSlice<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	variance(const VectorIndirectionSlice<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	weightedMean(const VectorIndirectionSlice<T>& source,const VectorIndirectionSlice<T>& weight);

template<class T> VectorIndirectionSlice<typename PromoteFloat<T>::promote_t>	moments(const VectorIndirectionSlice<T>& source,Int32 num_moments);

// min/max
template<class T> T max(const VectorIndirectionSlice<T>& source)
{
	T max_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		max_val = std::max(source[i],max_val);
		
	return max_val;
}

template<class T> VectorIndirectionSlice<T> max(const VectorIndirectionSlice<T>& lhs,const T& rhs)
{
	VectorIndirectionSlice<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(lhs[i],rhs);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> max(const T& lhs,const VectorIndirectionSlice<T>& rhs)
{
	VectorIndirectionSlice<T> tmp(rhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(rhs[i],lhs);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> max(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::RangeErr();
	
	VectorIndirectionSlice<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::max(lhs[i],rhs[i]);
	
	return tmp;
}

template<class T> T min(const VectorIndirectionSlice<T>& source)
{
	T min_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		min_val = std::min(source[i],min_val);
		
	return min_val;
}

template<class T> VectorIndirectionSlice<T> min(const VectorIndirectionSlice<T>& lhs,const T& rhs)
{
	VectorIndirectionSlice<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs[i],rhs);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> min(const T& lhs,const VectorIndirectionSlice<T>& rhs)
{
	VectorIndirectionSlice<T> tmp(rhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs,rhs[i]);
	
	return tmp;
}

template<class T> VectorIndirectionSlice<T> min(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorIndirectionSlice<T>::RangeErr();
		
	VectorIndirectionSlice<T> tmp(lhs);
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::min(lhs[i],rhs[i]);
	
	return tmp;
}

// product of all elements
template<class T> T product(const VectorIndirectionSlice<T>& source)
{
	return mcs::prod(source);
}

// sum of all elements
template<class T> T sum(const VectorIndirectionSlice<T>& source)
{
	return mcs::sum(source);
}

// other statistics
template<class T> typename PromoteFloat<T>::promote_t mean(const VectorIndirectionSlice<T>& source)
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);
	
	for (Int32 i=source.first();i<=source.last();tsum += source[i++]) { };
			
	return tsum/source.dim();
}

// standard deviation
template<class T> typename PromoteFloat<T>::promote_t standardDeviation(const VectorIndirectionSlice<T>& source)
{ 
	return std::sqrt(variance(source)); 
}

// corrected two-pass calculation (NR pp. 613)
template<class T> typename PromoteFloat<T>::promote_t variance(const VectorIndirectionSlice<T>& source)
{ 
	if (source.dim() < 2) throw VectorIndirectionSlice<T>::IllegalOperationErr();
		
	typename PromoteFloat<T>::promote_t xbar = mean(source),
										sum1(0),
										sum2(0);
	
	for (Int32 i=source.first();i<=source.last();i++)
	{
		const typename PromoteFloat<T>::promote_t	tmp(source[i]-xbar);
		
		sum2 += tmp*tmp;
		sum1 += tmp;
	}
		
	return (sum2 - sum1*sum1/source.dim())/(source.dim()-1); 
}

template<class T> typename PromoteFloat<T>::promote_t weightedMean(const VectorIndirectionSlice<T>& source,const VectorIndirectionSlice<T>& weight) 
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		tsum += source[i]*weight[i];
		norm += weight[i];
	}
				
	return tsum/norm;
}
/*
template<class T> VectorIndirectionSlice<typename PromoteFloat<T>::promote_t> moments(const VectorIndirectionSlice<T>& source,Int32 num_moments)
{
	VectorIndirectionSlice<typename PromoteFloat<T>::promote_t>	mom(IndexRange<1>(num_moments));

	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 m=mom.first();m<=mom.last();m++)
			mom[m] += std::pow(source[i],m);
			
	return mom;
}
*/
namespace ignoreNaN {

template<class T> T 			max(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> 	max(const VectorIndirectionSlice<T>& lhs,const T& rhs);
template<class T> VectorIndirectionSlice<T> 	max(const T& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> VectorIndirectionSlice<T> 	max(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> T 			min(const VectorIndirectionSlice<T>& source);
template<class T> VectorIndirectionSlice<T> 	min(const VectorIndirectionSlice<T>& lhs,const T& rhs);
template<class T> VectorIndirectionSlice<T> 	min(const T& lhs,const VectorIndirectionSlice<T>& rhs);
template<class T> VectorIndirectionSlice<T> 	min(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs);

template<class T> T				product(const VectorIndirectionSlice<T>& source);
template<class T> T 			sum(const VectorIndirectionSlice<T>& source); 
		
template<class T> typename PromoteFloat<T>::promote_t	mean(const VectorIndirectionSlice<T>& source);		
template<class T> typename PromoteFloat<T>::promote_t	standardDeviation(const VectorIndirectionSlice<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	variance(const VectorIndirectionSlice<T>& source);
template<class T> typename PromoteFloat<T>::promote_t	weightedMean(const VectorIndirectionSlice<T>& source,const VectorIndirectionSlice<T>& weight);

template<class T> VectorIndirectionSlice<typename PromoteFloat<T>::promote_t>	moments(const VectorIndirectionSlice<T>& source,Int32 num_moments);

// min/max
template<class T> T max(const VectorIndirectionSlice<T>& source)
{
	T max_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		if (isfinite(source[i]))
			max_val = std::max(source[i],max_val);
		
	return max_val;
}

template<class T> VectorIndirectionSlice<T> max(const VectorIndirectionSlice<T>& lhs,const T& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T> VectorIndirectionSlice<T> max(const T& lhs,const VectorIndirectionSlice<T>& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T> VectorIndirectionSlice<T> max(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	return normal::max(lhs,rhs);
}

template<class T> T min(const VectorIndirectionSlice<T>& source)
{
	T min_val(source[source.first()]);
	
	for (Int32 i=source.first()+1;i<=source.last();i++)
		if (isfinite(source[i]))
			min_val = std::min(source[i],min_val);
		
	return min_val;
}

template<class T> VectorIndirectionSlice<T> min(const VectorIndirectionSlice<T>& lhs,const T& rhs)
{
	return normal::min(lhs,rhs);
}

template<class T> VectorIndirectionSlice<T> min(const T& lhs,const VectorIndirectionSlice<T>& rhs)
{
	return normal::min(lhs,rhs);
}

template<class T> VectorIndirectionSlice<T> min(const VectorIndirectionSlice<T>& lhs,const VectorIndirectionSlice<T>& rhs)
{
	return normal::min(lhs,rhs);
}

// product of all elements
template<class T> T product(const VectorIndirectionSlice<T>& source)
{
	T tprod(1);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i])) 
			tprod *= source[i];
				
	return tprod;
}

// sum of all elements
template<class T> T sum(const VectorIndirectionSlice<T>& source)
{
	T tsum(0);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i])) 
			tsum += source[i];
				
	return tsum;
}

// other statistics
template<class T> typename PromoteFloat<T>::promote_t mean(const VectorIndirectionSlice<T>& source)
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		if (isfinite(source[i]))
		{
			tsum += source[i];
			norm += 1;
		}
	}
		
	return tsum/norm;
}

// standard deviation
template<class T> typename PromoteFloat<T>::promote_t standardDeviation(const VectorIndirectionSlice<T>& source)
{ 
	return std::sqrt(variance(source)); 
}

// corrected two-pass calculation (NR pp. 613)
template<class T> typename PromoteFloat<T>::promote_t variance(const VectorIndirectionSlice<T>& source)
{ 
	if (source.dim() < 2) throw VectorIndirectionSlice<T>::IllegalOperationErr();
		
	typename PromoteFloat<T>::promote_t xbar(mean(source)),
										sum1(0),
										sum2(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
	{
		const typename PromoteFloat<T>::promote_t tmp(source[i]-xbar);
		
		if (isfinite(tmp))
		{
			sum2 += tmp*tmp;
			sum1 += tmp;
			norm += 1;
		}
	}
	
	return (sum2 - sum1*sum1/norm)/(norm-1); 
}

template<class T> typename PromoteFloat<T>::promote_t weightedMean(const VectorIndirectionSlice<T>& source,const VectorIndirectionSlice<T>& weight) 
{ 
	typename PromoteFloat<T>::promote_t	tsum(0),
										norm(0);

	for (Int32 i=source.first();i<=source.last();i++)
		if (isfinite(source[i]) && isfinite(weight[i]))
		{
			tsum += source[i]*weight[i];
			norm += weight[i];
		}

	return tsum/norm;
}
/*
template<class T> VectorIndirectionSlice<typename PromoteFloat<T>::promote_t> moments(const VectorIndirectionSlice<T>& source,Int32 num_moments)
{
	VectorIndirectionSlice<typename PromoteFloat<T>::promote_t>	mom(IndexRange<1>(num_moments));

	for (Int32 i=source.first();i<=source.last();i++)
		for (Int32 m=mom.first();m<=mom.last();m++)
			if (isfinite(source[i])) 
				mom[m] += std::pow(source[i],m);
			
	return mom;
}
*/
} // namespace ignoreNaN

} // namespace statistics

} // namespace mcs

#endif // MCS_VECTORINDIRECTIONSLICESTATISTICS_H
