#ifndef MCS_VECTORMASK_H
#define MCS_VECTORMASK_H

#include "MCS_Typedefs.h"
#include "MCS_Vector.h"

namespace mcs {

// vector mask class
class VectorMask : public Vector<Binary>
{
	public:
		VectorMask(const IndexRange<1>&);
		VectorMask(const VectorMask&);
		VectorMask(const IndexRange<1>&,const Binary&);
		VectorMask(const IndexRange<1>&,const std::string&);
		~VectorMask();
		
		VectorMask& operator=(const VectorMask&);
		
		Int32 trueElements() const		{ return countElements(); }
		Int32 falseElements() const		{ return elem()-countElements(); }
		
	private:
		Int32	countElements() const;
};

inline VectorMask::VectorMask(const IndexRange<1>& rng) : 
	Vector<Binary>(rng,false)
{ }

inline VectorMask::VectorMask(const IndexRange<1>& rng,const Binary& val) :
	Vector<Binary>(rng,val)
{ }

inline VectorMask::VectorMask(const IndexRange<1>& rng,const std::string& str) :
	Vector<Binary>(rng,str)
{ }

inline VectorMask::VectorMask(const VectorMask& source) :
	Vector<Binary>(source)
{ }

inline VectorMask::~VectorMask()
{ }

inline VectorMask& VectorMask::operator=(const VectorMask& source)
{
	if (this == &source) return *this;

	Vector<Binary>::operator=(source);
		
	return *this;
}

inline Int32 VectorMask::countElements() const
{
	Int32	sum=0;
	
	for (Int32 i=first();i<=last();++i)
		if ((*this)[i]) sum++;
		
	return sum;
}

inline bool all(const VectorMask& source)
{ return (source.trueElements() == source.dim()); }

inline bool any(const VectorMask& source)
{ return (source.trueElements() >= 1); }

} // namespace mcs

#endif // MCS_VECTORMASK_H