#ifndef MCS_VECTOROPERATORS_H
#define MCS_VECTOROPERATORS_H

#include "MCS_Vector.h"

namespace mcs {

// vector non-member operators

// logical operations		
template<class T> const VectorMask operator==(const Vector<T>& lvec,const T& rscal);
template<class T> const VectorMask operator!=(const Vector<T>& lvec,const T& rscal);
template<class T> const VectorMask operator<(const Vector<T>& lvec,const T& rscal);
template<class T> const VectorMask operator<=(const Vector<T>& lvec,const T& rscal);
template<class T> const VectorMask operator>(const Vector<T>& lvec,const T& rscal);
template<class T> const VectorMask operator>=(const Vector<T>& lvec,const T& rscal);

template<class T> const VectorMask operator==(const Vector<T>& lvec,const Vector<T>& rvec);
template<class T> const VectorMask operator!=(const Vector<T>& lvec,const Vector<T>& rvec);
template<class T> const VectorMask operator<(const Vector<T>& lvec,const Vector<T>& rvec);
template<class T> const VectorMask operator<=(const Vector<T>& lvec,const Vector<T>& rvec);
template<class T> const VectorMask operator>(const Vector<T>& lvec,const Vector<T>& rvec);
template<class T> const VectorMask operator>=(const Vector<T>& lvec,const Vector<T>& rvec);

// arithmetic operators
template<class T> const Vector<T> operator+(const T& lscal,const Vector<T>& rvec);
template<class T> const Vector<T> operator-(const T& lscal,const Vector<T>& rvec);
template<class T> const Vector<T> operator*(const T& lscal,const Vector<T>& rvec);
template<class T> const Vector<T> operator/(const T& lscal,const Vector<T>& rvec);

template<class T> const Vector<T> operator+(const Vector<T>& lvec,const T& rscal);
template<class T> const Vector<T> operator-(const Vector<T>& lvec,const T& rscal);
template<class T> const Vector<T> operator*(const Vector<T>& lvec,const T& rscal);
template<class T> const Vector<T> operator/(const Vector<T>& lvec,const T& rscal);

template<class T> const Vector<T> operator+(const Vector<T>& lhs,const Vector<T>& rhs);
template<class T> const Vector<T> operator-(const Vector<T>& lhs,const Vector<T>& rhs);
template<class T> const Vector<T> operator*(const Vector<T>& lhs,const Vector<T>& rhs);
template<class T> const Vector<T> operator/(const Vector<T>& lhs,const Vector<T>& rhs);

template<class T> const VectorMask operator&&(const Vector<T>& lhs,const Vector<T>& rhs);
template<class T> const VectorMask operator||(const Vector<T>& lhs,const Vector<T>& rhs);

template<class T> const Vector<T> operator+(const Vector<T>& source);
template<class T> const Vector<T> operator-(const Vector<T>& source);

template<class T> const Vector<T> operator!(const Vector<T>& source);

// logical operations		

// Vector op scalar
template<class T> const VectorMask operator==(const Vector<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] == rscal);
		
	return tmp;
}

template<class T> const VectorMask operator!=(const Vector<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] != rscal);
		
	return tmp;
}

template<class T> const VectorMask operator<(const Vector<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] < rscal);
		
	return tmp;
}

template<class T> const VectorMask operator<=(const Vector<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] <= rscal);
		
	return tmp;
}

template<class T> const VectorMask operator>(const Vector<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] > rscal);
		
	return tmp;
}

template<class T> const VectorMask operator>=(const Vector<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] >= rscal);
		
	return tmp;
}

// Vector op Vector
template<class T> const VectorMask operator==(const Vector<T>& lvec,const Vector<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] == rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator!=(const Vector<T>& lvec,const Vector<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] != rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator<(const Vector<T>& lvec,const Vector<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] < rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator<=(const Vector<T>& lvec,const Vector<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] <= rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator>(const Vector<T>& lvec,const Vector<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] > rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator>=(const Vector<T>& lvec,const Vector<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] >= rvec[i]);
		
	return tmp;
}

// arithmetic operators
// scalar op Vector
template<class T> inline const Vector<T> operator+(const T& lscal,const Vector<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (Vector<T>(rvec.range(),lscal) += rvec);
}

template<class T> inline const Vector<T> operator-(const T& lscal,const Vector<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return -rvec;
		
	return (Vector<T>(rvec.range(),lscal) -= rvec);
}

template<class T> inline const Vector<T> operator*(const T& lscal,const Vector<T>& rvec)
{
	if (lscal == static_cast<T>(1)) return rvec;
	if (lscal == static_cast<T>(0)) return Vector<T>(rvec.range(),static_cast<T>(0));
	
	return (Vector<T>(rvec.range(),lscal) *= rvec);
}

template<class T> inline const Vector<T> operator/(const T& lscal,const Vector<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (Vector<T>(rvec.range(),lscal) /= rvec);
}

// Vector op scalar
template<class T> inline const Vector<T> operator+(const Vector<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (Vector<T>(lvec) += rscal);
}

template<class T> inline const Vector<T> operator-(const Vector<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (Vector<T>(lvec) -= rscal);
}

template<class T> inline const Vector<T> operator*(const Vector<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
	if (rscal == static_cast<T>(0)) return Vector<T>(lvec.range(),static_cast<T>(0));	

	return (Vector<T>(lvec) *= rscal);
}

template<class T> inline const Vector<T> operator/(const Vector<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
		
	return (Vector<T>(lvec) /= rscal);
}

// Vector op Vector
template<class T> inline const Vector<T> operator+(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<T>::SizeMismatchErr();
	
	return (Vector<T>(lhs) += rhs);
}

template<class T> inline const Vector<T> operator-(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<T>::SizeMismatchErr();
		
	return (Vector<T>(lhs) -= rhs);
}


template<class T> inline const Vector<T> operator*(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<T>::SizeMismatchErr();
	
	return (Vector<T>(lhs) *= rhs);
}

template<class T> inline const Vector<T> operator/(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<T>::SizeMismatchErr();
	
	return (Vector<T>(lhs) /= rhs);
}

template<class T> const VectorMask operator&&(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lhs.range());
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		tmp[i] = (lhs[i] && rhs[i]);
		
	return tmp;
}

template<class T> const VectorMask operator||(const Vector<T>& lhs,const Vector<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<T>::SizeMismatchErr();
	
	VectorMask tmp(lhs.range());
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		tmp[i] = (lhs[i] || rhs[i]);
		
	return tmp;
}

// unary operators
template<class T> inline const Vector<T> operator+(const Vector<T>& source)
{
	return source;
}

template<class T> inline const Vector<T> operator-(const Vector<T>& source)
{
	return (Vector<T>(source.range(),static_cast<T>(0)) -= source);
}

template<class T> const Vector<T> operator!(const Vector<T>& source)
{
	Vector<T> tmp(source);
	
	for (Int32 i=source.first();i<=source.last();i++)
		tmp[i] = !source[i];
		
	return tmp;
}

} // namespace mcs

#ifdef __INTEL__
	#if USE_INTEL_SPL
		#include "MCS_VectorOperators_IntelSpecialized.h"
	#endif
#endif

#endif // MCS_VECTOROPERATORS_H