#ifndef MCS_VECTOROPERATORSINTELSPECIALIZED_H
#define MCS_VECTOROPERATORSINTELSPECIALIZED_H

#include "MCS_Vector.h"

#include "MCS_IntelNSP.h"

#if USE_INTEL_SPL

namespace mcs {

template<> inline const Vector<Float32> operator+(const Float32& lscal,const Vector<Float32>& rvec)
{
	if (lscal == static_cast<Float32>(0)) return rvec;
	
	Vector<Float32> tmp(rvec);
	
	if (rvec.isValid()) 
		nspsbAdd1(lscal,tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float64> operator+(const Float64& lscal,const Vector<Float64>& rvec)
{
	if (lscal == static_cast<Float64>(0)) return rvec;
	
	Vector<Float64> tmp(rvec);
	
	if (rvec.isValid()) 
		nspdbAdd1(lscal,tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex32> operator+(const Complex32& lscal,const Vector<Complex32>& rvec)
{
	if (lscal == static_cast<Complex32>(0)) return rvec;
	
	Vector<Complex32> tmp(rvec);
	
	if (rvec.isValid()) 
		nspcbAdd1(scplx(lscal),reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex64> operator+(const Complex64& lscal,const Vector<Complex64>& rvec)
{
	if (lscal == static_cast<Complex64>(0)) return rvec;
	
	Vector<Complex64> tmp(rvec);
	
	if (rvec.isValid()) 
		nspzbAdd1(dcplx(lscal),reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float32> operator*(const Float32& lscal,const Vector<Float32>& rvec)
{
	if (lscal == static_cast<Float32>(1)) return rvec;
	if (lscal == static_cast<Float32>(0)) return Vector<Float32>(rvec.range());
	
	Vector<Float32> tmp(rvec);
	
	if (rvec.isValid()) 
		nspsbMpy1(lscal,tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float64> operator*(const Float64& lscal,const Vector<Float64>& rvec)
{
	if (lscal == static_cast<Float64>(1)) return rvec;
	if (lscal == static_cast<Float64>(0)) return Vector<Float64>(rvec.range());
	
	Vector<Float64> tmp(rvec);
	
	if (rvec.isValid()) 
		nspdbMpy1(lscal,tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex32> operator*(const Complex32& lscal,const Vector<Complex32>& rvec)
{
	if (lscal == static_cast<Complex32>(1)) return rvec;
	if (lscal == static_cast<Complex32>(0)) return Vector<Complex32>(rvec.range());
	
	Vector<Complex32> tmp(rvec);
	
	if (rvec.isValid()) 
		nspcbMpy1(scplx(lscal),reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex64> operator*(const Complex64& lscal,const Vector<Complex64>& rvec)
{
	if (lscal == static_cast<Complex64>(1)) return rvec;
	if (lscal == static_cast<Complex64>(0)) return Vector<Complex64>(rvec.range());
	
	Vector<Complex64> tmp(rvec);
	
	if (rvec.isValid()) 
		nspzbMpy1(dcplx(lscal),reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float32> operator+(const Vector<Float32>& lvec,const Float32& rscal)
{ 
	return (rscal+lvec); 
}

template<> inline const Vector<Float64> operator+(const Vector<Float64>& lvec,const Float64& rscal)
{ 
	return (rscal+lvec);
}

template<> inline const Vector<Complex32> operator+(const Vector<Complex32>& lvec,const Complex32& rscal)
{ 
	return (rscal+lvec);
}

template<> inline const Vector<Complex64> operator+(const Vector<Complex64>& lvec,const Complex64& rscal)
{ 
	return (rscal+lvec); 
}

template<> inline const Vector<Float32> operator-(const Vector<Float32>& lvec,const Float32& rscal)
{
	if (rscal == static_cast<Float32>(0)) return lvec;
	
	Vector<Float32> tmp(lvec);
	
	if (lvec.isValid()) 
		nspsbSub1(rscal,tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float64> operator-(const Vector<Float64>& lvec,const Float64& rscal)
{
	if (rscal == static_cast<Float64>(0)) return lvec;
	
	Vector<Float64> tmp(lvec);
	
	if (lvec.isValid()) 
		nspdbSub1(rscal,tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex32> operator-(const Vector<Complex32>& lvec,const Complex32& rscal)
{
	if (rscal == static_cast<Complex32>(0)) return lvec;
	
	Vector<Complex32> tmp(lvec);
	
	if (lvec.isValid()) 
		nspcbSub1(scplx(rscal),reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex64> operator-(const Vector<Complex64>& lvec,const Complex64& rscal)
{
	if (rscal == static_cast<Complex64>(0)) return lvec;
	
	Vector<Complex64> tmp(lvec);
	
	if (lvec.isValid()) 
		nspzbSub1(dcplx(rscal),reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float32> operator*(const Vector<Float32>& lvec,const Float32& rscal)
{ 
	return (rscal*lvec); 
}

template<> inline const Vector<Float64> operator*(const Vector<Float64>& lvec,const Float64& rscal)
{ 
	return (rscal*lvec); 
}

template<> inline const Vector<Complex32> operator*(const Vector<Complex32>& lvec,const Complex32& rscal)
{ 
	return (rscal*lvec); 
}

template<> inline const Vector<Complex64> operator*(const Vector<Complex64>& lvec,const Complex64& rscal)
{ 
	return (rscal*lvec); 
}

template<> inline const Vector<Float32> operator+(const Vector<Float32>& lhs,const Vector<Float32>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Float32>::SizeMismatchErr();
	
	Vector<Float32> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspsbAdd3(lhs.dataPointer(),rhs.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float64> operator+(const Vector<Float64>& lhs,const Vector<Float64>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Float64>::SizeMismatchErr();
	
	Vector<Float64> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspdbAdd3(lhs.dataPointer(),rhs.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex32> operator+(const Vector<Complex32>& lhs,const Vector<Complex32>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Complex32>::SizeMismatchErr();
	
	Vector<Complex32> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspcbAdd3(reinterpret_cast<SCplx*>(lhs.dataPointer()),
				  reinterpret_cast<SCplx*>(rhs.dataPointer()),
				  reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex64> operator+(const Vector<Complex64>& lhs,const Vector<Complex64>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Complex64>::SizeMismatchErr();
	
	Vector<Complex64> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspzbAdd3(reinterpret_cast<DCplx*>(lhs.dataPointer()),
				  reinterpret_cast<DCplx*>(rhs.dataPointer()),
				  reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float32> operator-(const Vector<Float32>& lhs,const Vector<Float32>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Float32>::SizeMismatchErr();
	
	Vector<Float32> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspsbSub3(lhs.dataPointer(),rhs.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float64> operator-(const Vector<Float64>& lhs,const Vector<Float64>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Float64>::SizeMismatchErr();
	
	Vector<Float64> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspdbSub3(lhs.dataPointer(),rhs.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex32> operator-(const Vector<Complex32>& lhs,const Vector<Complex32>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Complex32>::SizeMismatchErr();
	
	Vector<Complex32> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspcbSub3(reinterpret_cast<SCplx*>(lhs.dataPointer()),
				  reinterpret_cast<SCplx*>(rhs.dataPointer()),
				  reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex64> operator-(const Vector<Complex64>& lhs,const Vector<Complex64>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Complex64>::SizeMismatchErr();
	
	Vector<Complex64> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspzbSub3(reinterpret_cast<DCplx*>(lhs.dataPointer()),
				  reinterpret_cast<DCplx*>(rhs.dataPointer()),
				  reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float32> operator*(const Vector<Float32>& lhs,const Vector<Float32>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Float32>::SizeMismatchErr();
	
	Vector<Float32> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspsbMpy3(lhs.dataPointer(),rhs.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Float64> operator*(const Vector<Float64>& lhs,const Vector<Float64>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Float64>::SizeMismatchErr();
	
	Vector<Float64> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspdbMpy3(lhs.dataPointer(),rhs.dataPointer(),tmp.dataPointer(),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex32> operator*(const Vector<Complex32>& lhs,const Vector<Complex32>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Complex32>::SizeMismatchErr();
	
	Vector<Complex32> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspcbMpy3(reinterpret_cast<SCplx*>(lhs.dataPointer()),
				  reinterpret_cast<SCplx*>(rhs.dataPointer()),
				  reinterpret_cast<SCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

template<> inline const Vector<Complex64> operator*(const Vector<Complex64>& lhs,const Vector<Complex64>& rhs)
{
	if (lhs.range() != rhs.range()) throw typename Vector<Complex64>::SizeMismatchErr();
	
	Vector<Complex64> tmp(lhs.range());
	
	if (lhs.isValid() && rhs.isValid()) 
		nspzbMpy3(reinterpret_cast<DCplx*>(lhs.dataPointer()),
				  reinterpret_cast<DCplx*>(rhs.dataPointer()),
				  reinterpret_cast<DCplx*>(tmp.dataPointer()),static_cast<int>(tmp.dim()));
		
	return tmp;
}

} // namespace mcs

#endif // USE_INTEL_SPL

#endif // MCS_VECTOROPERATORSINTELSPECIALIZED_H
