#ifndef MCS_VECTORRANGESLICEFUNCTIONS_H
#define MCS_VECTORRANGESLICEFUNCTIONS_H

#include "MCS_Ensure.h"
#include "MCS_MathExtensions.h"
#include "MCS_Promote.h"
#include "MCS_VectorRangeSlice.h"

namespace mcs {

// in place
// arithmetic functions
template<class T> const VectorRangeSlice<T>& Abs(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Ceil(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Exp(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Floor(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Log(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Pow(VectorRangeSlice<T>& source,const T& expon);
template<class T> const VectorRangeSlice<T>& Round(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Sqr(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Sqrt(VectorRangeSlice<T>& source);

// trigonometric functions
template<class T> const VectorRangeSlice<T>& Acos(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Asin(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Atan(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Cos(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Sin(VectorRangeSlice<T>& source); 
template<class T> const VectorRangeSlice<T>& Tan(VectorRangeSlice<T>& source);

// hyperbolic functions
template<class T> const VectorRangeSlice<T>& Acosh(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Asinh(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Atanh(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Cosh(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Sinh(VectorRangeSlice<T>& source); 
template<class T> const VectorRangeSlice<T>& Tanh(VectorRangeSlice<T>& source);

// complex vectors
template<class T> const VectorRangeSlice<T>& Conj(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Imag(VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T>& Real(VectorRangeSlice<T>& source);

// not in place 
// arithmetic functions
template<class T> VectorRangeSlice<T> abs(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> ceil(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> exp(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> floor(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> log(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> pow(const VectorRangeSlice<T>& source,const T& expon);
template<class T> VectorRangeSlice<T> round(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> sqr(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> sqrt(const VectorRangeSlice<T>& source);

// trigonometric functions
template<class T> VectorRangeSlice<T> acos(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> asin(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> atan(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> atan2(const VectorRangeSlice<T>& y,const VectorRangeSlice<T>& x);
template<class T> VectorRangeSlice<T> cos(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> sin(const VectorRangeSlice<T>& source); 
template<class T> VectorRangeSlice<T> tan(const VectorRangeSlice<T>& source);

// hyperbolic functions
template<class T> VectorRangeSlice<T> acosh(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> asinh(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> atanh(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> cosh(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<T> sinh(const VectorRangeSlice<T>& source); 
template<class T> VectorRangeSlice<T> tanh(const VectorRangeSlice<T>& source);
 
// complex vectors
template<class T> VectorRangeSlice<T> conj(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<typename DemoteComplex<T>::demote_t> imag(const VectorRangeSlice<T>& source);
template<class T> VectorRangeSlice<typename DemoteComplex<T>::demote_t> real(const VectorRangeSlice<T>& source);

// miscellaneous
template<class T> VectorRangeSlice<T> bound(const VectorRangeSlice<T>& x,const T& min_val,const T& max_val);
template<class T> VectorRangeSlice<Binary> even(const VectorRangeSlice<T>& x);
template<class T> VectorRangeSlice<Binary> odd(const VectorRangeSlice<T>& x);
template<class T> VectorRangeSlice<T> mod(const VectorRangeSlice<T>& x,const T& y);
template<class T> VectorRangeSlice<T> mod(const VectorRangeSlice<T>& x,const VectorRangeSlice<T>& y);
template<class T> VectorRangeSlice<T> quotient(const VectorRangeSlice<T>& x,const T& y);
template<class T> VectorRangeSlice<T> quotient(const VectorRangeSlice<T>& x,const VectorRangeSlice<T>& y);

// vector specific
template<class T> bool all(const VectorRangeSlice<T>&);
template<class T> bool any(const VectorRangeSlice<T>&);

template<class T> T dot(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);
template<class T> T mag(const VectorRangeSlice<T>& source);
template<class T> T norm(const VectorRangeSlice<T>& source);	
template<class T> T prod(const VectorRangeSlice<T>& source);		
template<class T> T sum(const VectorRangeSlice<T>& source);

// in place
// arithmetic functions
template<class T> const VectorRangeSlice<T>& Abs(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::abs(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Ceil(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::ceil(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Exp(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::exp(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Floor(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::floor(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Log(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::log(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Pow(VectorRangeSlice<T>& source,const T& expon)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::pow(source[i],expon);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Round(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::round(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Sqr(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] *= source[i];
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Sqrt(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sqrt(source[i]);
	
	return source;
}

// complex functions
// conjugate complex vector
template<class T> const VectorRangeSlice<T>& Conj(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::conj(source[i]);
		
	return source;
}

template<class T> const VectorRangeSlice<T>& Imag(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::imag(source[i]);
		
	return source;
}

template<class T> const VectorRangeSlice<T>& Real(VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = mcs::real(source[i]);
		
	return source;
}

// trigonometric functions
template<class T> const VectorRangeSlice<T>& Sin(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sin(source[i]);

	return source;
}

template<class T> const VectorRangeSlice<T>& Cos(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cos(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Tan(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tan(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Asin(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asin(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Acos(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acos(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Atan(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atan(source[i]);
	
	return source;
}

// hyperbolic functions
template<class T> const VectorRangeSlice<T>& Sinh(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::sinh(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Cosh(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::cosh(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Tanh(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::tanh(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Asinh(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::asinh(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Acosh(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::acosh(source[i]);
	
	return source;
}

template<class T> const VectorRangeSlice<T>& Atanh(VectorRangeSlice<T>& source)
{
	static const EnsureFloat<T>	dummy_;
	
	for (Int32 i=source.first();i<=source.last();i++)
		source[i] = std::atanh(source[i]);

	return source;
}

// not in place
// arithmetic functions
template<class T> inline VectorRangeSlice<T> abs(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Abs(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> ceil(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Ceil(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> exp(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Exp(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> floor(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Floor(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> log(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Log(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> pow(const VectorRangeSlice<T>& source,const T& expon)
{
	VectorRangeSlice<T>	tmp(source);
	
	Pow(tmp,expon);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> round(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Round(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> sqr(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Sqr(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> sqrt(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Sqrt(tmp);
	
	return tmp;
}

// complex vectors
template<class T> inline VectorRangeSlice<T> conj(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Conj(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<typename DemoteComplex<T>::demote_t> imag(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Imag(tmp);
	
	return tmp;
}

template<class T> VectorRangeSlice<T> imag(const VectorRangeSlice< std::complex<T> >& source)
{
	VectorRangeSlice<T>	tmp(source.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = source[i].imag();
	
	return tmp;
}

template<class T> inline VectorRangeSlice<typename DemoteComplex<T>::demote_t> real(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<typename DemoteComplex<T>::demote_t>	tmp(source);
	
	Real(tmp);
	
	return tmp;
}

template<class T> VectorRangeSlice<T> real(const VectorRangeSlice< std::complex<T> >& source)
{
	VectorRangeSlice<T>	tmp(source.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = source[i].real();
	
	return tmp;
}

// trigonometric functions
template<class T> inline VectorRangeSlice<T> sin(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Sin(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> cos(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Cos(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> tan(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Tan(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> asin(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Asin(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> acos(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Acos(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> atan(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Atan(tmp);
	
	return tmp;
}

template<class T> VectorRangeSlice<T> atan2(const VectorRangeSlice<T>& y,const VectorRangeSlice<T>& x)
{
	static const EnsureFloat<T>	dummy_;
	
	if (x.range() != y.range()) throw VectorRangeSlice<T>::SizeMismatchErr();

	VectorRangeSlice<T>	tmp(y.range());
	
	for (Int32 i=tmp.first();i<=tmp.last();i++)
		tmp[i] = std::atan2(y[i],x[i]);
		
	return tmp;
}

// hyperbolic functions
template<class T> inline VectorRangeSlice<T> sinh(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Sinh(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> cosh(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Cosh(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> tanh(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Tanh(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> asinh(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Asinh(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> acosh(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Acosh(tmp);
	
	return tmp;
}

template<class T> inline VectorRangeSlice<T> atanh(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T>	tmp(source);
	
	Atanh(tmp);
	
	return tmp;
}

// miscellaneous
template<class T> VectorRangeSlice<T> bound(const VectorRangeSlice<T>& x,const T& min_val,const T& max_val)
{
	VectorRangeSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::bound(x[i],min_val,max_val);

	return tmp;
}
/*
template<class T> VectorRangeSlice<Binary> even(const VectorRangeSlice<T>& x)
{
	VectorRangeSlice<Binary>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::even(x[i]);

	return tmp;
}

template<class T> VectorRangeSlice<Binary> odd(const VectorRangeSlice<T>& x)
{
	VectorRangeSlice<Binary>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::odd(x[i]);

	return tmp;
}
*/
template<class T> VectorRangeSlice<T> mod(const VectorRangeSlice<T>& x,const T& y)
{
	VectorRangeSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::mod(x[i],y);

	return tmp;
}

template<class T> VectorRangeSlice<T> mod(const VectorRangeSlice<T>& x,const VectorRangeSlice<T>& y)
{
	if (x.range() != y.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorRangeSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::mod(x[i],y[i]);

	return tmp;
}

template<class T> VectorRangeSlice<T> quotient(const VectorRangeSlice<T>& x,const T& y)
{
	VectorRangeSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::quotient(x[i],y);

	return tmp;
}

template<class T> VectorRangeSlice<T> quotient(const VectorRangeSlice<T>& x,const VectorRangeSlice<T>& y)
{
	if (x.range() != y.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorRangeSlice<T>	tmp(x.range());
	
	for (Int32 i=x.first();i<=x.last();i++)
		tmp[i] = mcs::quotient(x[i],y[i]);

	return tmp;
}

// vector specific
template<class T> bool all(const VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (!source[i]) return false;
		
	return true;
}

template<class T> bool any(const VectorRangeSlice<T>& source)
{
	for (Int32 i=source.first();i<=source.last();i++)
		if (source[i]) return true;
		
	return false;
}

// dot product
template<class T> inline T dot(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{	
	return sum(lhs*rhs);
}

// vector magnitude	
template<class T> inline T mag(const VectorRangeSlice<T>& source)
{
	return std::sqrt(norm(source));
}

// vector norm
template<class T> inline T norm(const VectorRangeSlice<T>& source)
{
	return sum(sqr(source));
}

// product of all elements
template<class T> T product(const VectorRangeSlice<T>& source)
{
	T tprod(1);
	
	for (Int32 i=source.first();i<=source.last();tprod *= source[i++]) { };
	
	return tprod;
}

// sum of all elements
template<class T> T sum(const VectorRangeSlice<T>& source)
{
	T tsum(0);
	
	for (Int32 i=source.first();i<=source.last();tsum += source[i++]) { };
	
	return tsum;
}

} // namespace mcs

#endif // MCS_VECTORRANGESLICEFUNCTIONS_H
