#ifndef MCS_VECTORRANGESLICEOPERATORS_H
#define MCS_VECTORRANGESLICEOPERATORS_H

#include "MCS_VectorRangeSlice.h"

namespace mcs {

// vector non-member operators

// logical operations		
template<class T> const VectorMask operator==(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator!=(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator<(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator<=(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator>(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorMask operator>=(const VectorRangeSlice<T>& lvec,const T& rscal);

template<class T> const VectorMask operator==(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec);
template<class T> const VectorMask operator!=(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec);
template<class T> const VectorMask operator<(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec);
template<class T> const VectorMask operator<=(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec);
template<class T> const VectorMask operator>(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec);
template<class T> const VectorMask operator>=(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec);

// arithmetic operators
template<class T> const VectorRangeSlice<T> operator+(const T& lscal,const VectorRangeSlice<T>& rvec);
template<class T> const VectorRangeSlice<T> operator-(const T& lscal,const VectorRangeSlice<T>& rvec);
template<class T> const VectorRangeSlice<T> operator*(const T& lscal,const VectorRangeSlice<T>& rvec);
template<class T> const VectorRangeSlice<T> operator/(const T& lscal,const VectorRangeSlice<T>& rvec);

template<class T> const VectorRangeSlice<T> operator+(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorRangeSlice<T> operator-(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorRangeSlice<T> operator*(const VectorRangeSlice<T>& lvec,const T& rscal);
template<class T> const VectorRangeSlice<T> operator/(const VectorRangeSlice<T>& lvec,const T& rscal);

template<class T> const VectorRangeSlice<T> operator+(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);
template<class T> const VectorRangeSlice<T> operator-(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);
template<class T> const VectorRangeSlice<T> operator*(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);
template<class T> const VectorRangeSlice<T> operator/(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);

template<class T> const VectorMask operator&&(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);
template<class T> const VectorMask operator||(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs);

template<class T> const VectorRangeSlice<T> operator+(const VectorRangeSlice<T>& source);
template<class T> const VectorRangeSlice<T> operator-(const VectorRangeSlice<T>& source);

template<class T> const VectorRangeSlice<T> operator!(const VectorRangeSlice<T>& source);

// logical operations		

// VectorRangeSlice op scalar
template<class T> const VectorMask operator==(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] == rscal);
		
	return tmp;
}

template<class T> const VectorMask operator!=(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] != rscal);
		
	return tmp;
}

template<class T> const VectorMask operator<(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] < rscal);
		
	return tmp;
}

template<class T> const VectorMask operator<=(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] <= rscal);
		
	return tmp;
}

template<class T> const VectorMask operator>(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] > rscal);
		
	return tmp;
}

template<class T> const VectorMask operator>=(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] >= rscal);
		
	return tmp;
}

// VectorRangeSlice op VectorRangeSlice
template<class T> const VectorMask operator==(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] == rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator!=(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] != rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator<(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] < rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator<=(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] <= rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator>(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] > rvec[i]);
		
	return tmp;
}

template<class T> const VectorMask operator>=(const VectorRangeSlice<T>& lvec,const VectorRangeSlice<T>& rvec)
{
	if (lvec.range() != rvec.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lvec.range());
	
	for (Int32 i=lvec.first();i<=lvec.last();i++)
		tmp[i] = (lvec[i] >= rvec[i]);
		
	return tmp;
}

// arithmetic operators
// scalar op VectorRangeSlice
template<class T> inline const VectorRangeSlice<T> operator+(const T& lscal,const VectorRangeSlice<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (VectorRangeSlice<T>(rvec.range(),lscal) += rvec);
}

template<class T> inline const VectorRangeSlice<T> operator-(const T& lscal,const VectorRangeSlice<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return -rvec;
		
	return (VectorRangeSlice<T>(rvec.range(),lscal) -= rvec);
}

template<class T> inline const VectorRangeSlice<T> operator*(const T& lscal,const VectorRangeSlice<T>& rvec)
{
	if (lscal == static_cast<T>(1)) return rvec;
	if (lscal == static_cast<T>(0)) return VectorRangeSlice<T>(rvec.range(),static_cast<T>(0));
	
	return (VectorRangeSlice<T>(rvec.range(),lscal) *= rvec);
}

template<class T> inline const VectorRangeSlice<T> operator/(const T& lscal,const VectorRangeSlice<T>& rvec)
{
	if (lscal == static_cast<T>(0)) return rvec;
		
	return (VectorRangeSlice<T>(rvec.range(),lscal) /= rvec);
}

// VectorRangeSlice op scalar
template<class T> inline const VectorRangeSlice<T> operator+(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (VectorRangeSlice<T>(lvec) += rscal);
}

template<class T> inline const VectorRangeSlice<T> operator-(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(0)) return lvec;
		
	return (VectorRangeSlice<T>(lvec) -= rscal);
}

template<class T> inline const VectorRangeSlice<T> operator*(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
	if (rscal == static_cast<T>(0)) return VectorRangeSlice<T>(lvec.range(),static_cast<T>(0));	

	return (VectorRangeSlice<T>(lvec) *= rscal);
}

template<class T> inline const VectorRangeSlice<T> operator/(const VectorRangeSlice<T>& lvec,const T& rscal)
{
	if (rscal == static_cast<T>(1)) return lvec;
		
	return (VectorRangeSlice<T>(lvec) /= rscal);
}

// VectorRangeSlice op VectorRangeSlice
template<class T> inline const VectorRangeSlice<T> operator+(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	return (VectorRangeSlice<T>(lhs) += rhs);
}

template<class T> inline const VectorRangeSlice<T> operator-(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
		
	return (VectorRangeSlice<T>(lhs) -= rhs);
}


template<class T> inline const VectorRangeSlice<T> operator*(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	return (VectorRangeSlice<T>(lhs) *= rhs);
}

template<class T> inline const VectorRangeSlice<T> operator/(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	return (VectorRangeSlice<T>(lhs) /= rhs);
}

template<class T> const VectorMask operator&&(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lhs.range());
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		tmp[i] = (lhs[i] && rhs[i]);
		
	return tmp;
}

template<class T> const VectorMask operator||(const VectorRangeSlice<T>& lhs,const VectorRangeSlice<T>& rhs)
{
	if (lhs.range() != rhs.range()) throw VectorRangeSlice<T>::SizeMismatchErr();
	
	VectorMask tmp(lhs.range());
	
	for (Int32 i=lhs.first();i<=lhs.last();i++)
		tmp[i] = (lhs[i] || rhs[i]);
		
	return tmp;
}

// unary operators
template<class T> inline const VectorRangeSlice<T> operator+(const VectorRangeSlice<T>& source)
{
	return source;
}

template<class T> inline const VectorRangeSlice<T> operator-(const VectorRangeSlice<T>& source)
{
	return (VectorRangeSlice<T>(source.range(),static_cast<T>(0)) -= source);
}

template<class T> const VectorRangeSlice<T> operator!(const VectorRangeSlice<T>& source)
{
	VectorRangeSlice<T> tmp(source);
	
	for (Int32 i=source.first();i<=source.last();i++)
		tmp[i] = !source[i];
		
	return tmp;
}

} // namespace mcs

#endif // MCS_VECTORRANGESLICEOPERATORS_H