#ifndef MCS_VECTORINTELSPECIALIZED_H
#define MCS_VECTORINTELSPECIALIZED_H

#include "MCS_Vector.h"

#if USE_INTEL_SPL
#include "MCS_IntelNSP.h"

namespace mcs {

// specializations for types without constructors
template<> inline void Vector<Int8>::set(const Int8& val)
{
	if (isValid())
		std::memset(reinterpret_cast<char*>(dataPointer()),static_cast<char>(val),elem());
}

template<> inline void Vector<Uint8>::set(const Uint8& val)
{
	if (isValid())
		std::memset(reinterpret_cast<char*>(dataPointer()),static_cast<char>(val),elem());
}

template<> inline void Vector<Int16>::set(const Int16& val)
{ 
	if (isValid()) 
		nspwbSet(val,dataPointer(),elem()); 
}

template<> inline void Vector<Uint16>::set(const Uint16& val)
{
	Int16	nval(reinterpret_cast<const Int16&>(val));
	
	if (isValid()) 
		nspwbSet(nval,reinterpret_cast<Int16*>(dataPointer()),elem()); 
}

template<> inline void Vector<Int32>::set(const Int32& val)
{ 
	Float32	nval(reinterpret_cast<const Float32&>(val));
	
	if (isValid()) 
		nspsbSet(nval,reinterpret_cast<Float32*>(dataPointer()),elem()); 
}

template<> inline void Vector<Uint32>::set(const Uint32& val)
{ 
	Float32	nval(reinterpret_cast<const Float32&>(val));
	
	if (isValid()) 
		nspsbSet(nval,reinterpret_cast<Float32*>(dataPointer()),elem()); 
}

template<> inline void Vector<Float32>::set(const Float32& val)
{ 
	if (isValid()) 
		nspsbSet(val,dataPointer(),elem()); 
}

template<> inline void Vector<Float64>::set(const Float64& val)
{ 
	if (isValid()) 
		nspdbSet(val,dataPointer(),elem()); 
}

template<> inline void Vector<Complex32>::set(const Complex32& val)
{ 
	if (isValid()) 
		nspcbSet(val.real(),val.imag(),reinterpret_cast<SCplx*>(dataPointer()),elem()); 
}

template<> inline void Vector<Complex64>::set(const Complex64& val)
{ 
	if (isValid()) 
		nspzbSet(val.real(),val.imag(),reinterpret_cast<DCplx*>(dataPointer()),elem()); 
}

template<> inline void Vector<Int8>::set(const Int8* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(vec_ptr_),reinterpret_cast<const void*>(data),elem()*sizeof(Int8)); 
}

template<> inline void Vector<unsigned char>::set(const Uint8* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Uint8)); 
}

template<> inline void Vector<short>::set(const Int16* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Int16)); 
}

template<> inline void Vector<unsigned short>::set(const Uint16* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Uint16)); 
}

template<> inline void Vector<int>::set(const Int32* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Int32)); 
}

template<> inline void Vector<unsigned int>::set(const Uint32* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Uint32));
}

template<> inline void Vector<Float32>::set(const Float32* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Float32)); 
}

template<> inline void Vector<Float64>::set(const Float64* data)
{ 
	if (isValid() && data != NULL) 
		std::memcpy(reinterpret_cast<void*>(dataPointer()),reinterpret_cast<const void*>(data),elem()*sizeof(Float64)); 
}

template<> inline Vector<Float32>& Vector<Float32>::operator+=(const Float32& val)
{
	if (val == static_cast<Float32>(0)) return *this;
		
	if (isValid()) 
		nspsbAdd1(val,reinterpret_cast<Float32*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float64>& Vector<Float64>::operator+=(const Float64& val)
{
	if (val == static_cast<Float64>(0)) return *this;
		
	if (isValid()) 
		nspdbAdd1(val,reinterpret_cast<Float64*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex32>& Vector<Complex32>::operator+=(const Complex32& val)
{
	if (val == static_cast<Complex32>(0)) return *this;
		
	if (isValid()) 
		nspcbAdd1(scplx(val),reinterpret_cast<SCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex64>& Vector<Complex64>::operator+=(const Complex64& val)
{
	if (val == static_cast<Complex64>(0)) return *this;
		
	if (isValid()) 
		nspzbAdd1(dcplx(val),reinterpret_cast<DCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float32>& Vector<Float32>::operator-=(const Float32& val)
{
	if (val == static_cast<Float32>(0)) return *this;
		
	if (isValid()) 
		nspsbSub1(val,reinterpret_cast<Float32*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float64>& Vector<Float64>::operator-=(const Float64& val)
{
	if (val == static_cast<Float64>(0)) return *this;
		
	if (isValid()) 
		nspdbSub1(val,reinterpret_cast<Float64*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex32>& Vector<Complex32>::operator-=(const Complex32& val)
{
	if (val == static_cast<Complex32>(0)) return *this;
		
	if (isValid()) 
		nspcbSub1(scplx(val),reinterpret_cast<SCplx*>(dataPointer()),int(dim()));
		
	return *this;
}

template<> inline Vector<Complex64>& Vector<Complex64>::operator-=(const Complex64& val)
{
	if (val == static_cast<Complex64>(0)) return *this;
		
	if (isValid()) 
		nspzbSub1(dcplx(val),reinterpret_cast<DCplx*>(dataPointer()),int(dim()));
		
	return *this;
}

template<> inline Vector<Float32>& Vector<Float32>::operator*=(const Float32& val)
{
	if (val == static_cast<Float32>(1)) return *this;
		
	if (isValid()) 
		nspsbMpy1(val,reinterpret_cast<Float32*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float64>& Vector<Float64>::operator*=(const Float64& val)
{
	if (val == static_cast<Float64>(1)) return *this;
		
	if (isValid()) 
		nspdbMpy1(val,reinterpret_cast<Float64*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex32>& Vector<Complex32>::operator*=(const Complex32& val)
{
	if (val == static_cast<Complex32>(1)) return *this;
		
	if (isValid()) 
		nspcbMpy1(scplx(val),reinterpret_cast<SCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex64>& Vector<Complex64>::operator*=(const Complex64& val)
{
	if (val == static_cast<Complex64>(1)) return *this;
		
	if (isValid()) 
		nspzbMpy1(dcplx(val),reinterpret_cast<DCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float32>& Vector<Float32>::operator+=(const Vector<Float32>& source)
{
	if (source.range() != range()) throw typename Vector<Float32>::RangeErr();
			
	if (isValid()) 
		nspsbAdd2(source.dataPointer(),dataPointer(),elem());
		
	return *this;
}

template<> inline Vector<Float64>& Vector<Float64>::operator+=(const Vector<Float64>& source)
{
	if (source.range() != range()) throw typename Vector<Float64>::RangeErr();
	
	if (isValid()) 
		nspdbAdd2(source.dataPointer(),dataPointer(),elem());
		
	return *this;
}

template<> inline Vector<Complex32>& Vector<Complex32>::operator+=(const Vector<Complex32>& source)
{
	if (source.range() != range()) throw typename Vector<Complex32>::RangeErr();
	
	if (isValid()) 
		nspcbAdd2(reinterpret_cast<SCplx*>(source.dataPointer()),reinterpret_cast<SCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex64>& Vector<Complex64>::operator+=(const Vector<Complex64>& source)
{
	if (source.range() != range()) throw typename Vector<Complex64>::RangeErr();
	
	if (isValid()) 
		nspzbAdd2(reinterpret_cast<DCplx*>(source.dataPointer()),reinterpret_cast<DCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float32>& Vector<Float32>::operator-=(const Vector<Float32>& source)
{
	if (source.range() != range()) throw typename Vector<Float32>::RangeErr();
			
	if (isValid()) 
		nspsbSub2(source.dataPointer(),dataPointer(),elem());
		
	return *this;
}

template<> inline Vector<Float64>& Vector<Float64>::operator-=(const Vector<Float64>& source)
{
	if (source.range() != range()) throw typename Vector<Float64>::RangeErr();
	
	if (isValid()) 
		nspdbSub2(source.dataPointer(),dataPointer(),elem());
		
	return *this;
}

template<> inline Vector<Complex32>& Vector<Complex32>::operator-=(const Vector<Complex32>& source)
{
	if (source.range() != range()) throw typename Vector<Complex32>::RangeErr();
	
	if (isValid()) 
		nspcbSub2(reinterpret_cast<SCplx*>(source.dataPointer()),reinterpret_cast<SCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex64>& Vector<Complex64>::operator-=(const Vector<Complex64>& source)
{
	if (source.range() != range()) throw typename Vector<Complex64>::RangeErr();
	
	if (isValid()) 
		nspzbSub2(reinterpret_cast<DCplx*>(source.dataPointer()),reinterpret_cast<DCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Float32>& Vector<Float32>::operator*=(const Vector<Float32>& source)
{
	if (source.range() != range()) throw typename Vector<Float32>::RangeErr();
			
	if (isValid()) 
		nspsbMpy2(source.dataPointer(),dataPointer(),elem());
		
	return *this;
}

template<> inline Vector<Float64>& Vector<Float64>::operator*=(const Vector<Float64>& source)
{
	if (source.range() != range()) throw typename Vector<Float64>::RangeErr();
	
	if (isValid()) 
		nspdbMpy2(source.dataPointer(),dataPointer(),elem());
		
	return *this;
}

template<> inline Vector<Complex32>& Vector<Complex32>::operator*=(const Vector<Complex32>& source)
{
	if (source.range() != range()) throw typename Vector<Complex32>::RangeErr();
	
	if (isValid()) 
		nspcbMpy2(reinterpret_cast<SCplx*>(source.dataPointer()),reinterpret_cast<SCplx*>(dataPointer()),elem());
		
	return *this;
}

template<> inline Vector<Complex64>& Vector<Complex64>::operator*=(const Vector<Complex64>& source)
{
	if (source.range() != range()) throw typename Vector<Complex64>::RangeErr();
	
	if (isValid()) 
		nspzbMpy2(reinterpret_cast<DCplx*>(source.dataPointer()),reinterpret_cast<DCplx*>(dataPointer()),elem());
		
	return *this;
}

} // namespace mcs

#endif // USE_INTEL_SPL

#endif // MCS_VECTORINTELSPECIALIZED_H