$freeform
!@***h* MLT_RSS/AMSU_L1B_to_L2A_Driver.f90
!  NAME
!    AMSU_L1B_to_L2A_Driver
! 
!  PURPOSE
!    Top-level driver program for converting L1B data to L2A data
! 
!  DESCRIPTION
!    Top-level driver program for converting L1B data to L2A data.  Interprets command line paramters and 
!    makes a call to convert_L1B_orbits_to_L2A_orbits
!  ARGUMENTS
!    first argument is the satellite number
!    second argument is the channel
!    if a third argument is present, and equal to 'all', then all orbits are reprocessed.
!       otherwise, the files Channel_05_number_to_process.txt' and Channel_05_revs_to_process.txt'
!       are read to determine which orbits are processed.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE AMSU_L1B_to_L2A_Driver.exe 10 5 
!    
!@*****
program AMSU_L1B_to_L2A_Driver

	use NAN_support
	use date_and_time
	use AMSU_Constants
	use AMSU_Types
	use AMSU_inventory
	use AMSU_equator_crossing_times
	use AMSU_File_Locations
	use AMSU_Level_1B
	use AMSU_Level_2A
	use AMSU_L1B_to_L2A
	use AMSU_SAA

	use DFLIB  ! built in module, enables use of command line arguments

	integer(4)							:: sat_num
	integer(4)							:: channel
	integer(4),dimension(MAX_ORBITS_TO_PROCESS)			:: orbits_to_process
	integer(4)							:: number_of_orbits_to_process
	integer(4)							:: error

	character(len=7)					:: satellite_name
	character(len=120)					:: buffer
	character(len=120)					:: orbit_list_path
	character(len=120)					:: num_orbit_path

	logical								:: wrt,all



	integer(4)	:: i



	call init_NAN

	error = 0
	buffer = ''

	call getarg(1,buffer,error)
	read(buffer,111)sat_num
111 format(i3)	
	if (error .ne. 0) stop

	satellite_name = amsu_sat_names_2(sat_num)
	
	call getarg(2,buffer,error)
	read(buffer,111)channel
	if (error .ne. 0) stop

	all = .false.
	call getarg(3,buffer,error)
	if (error .eq. 0) then
		if (buffer(1:3) .eq. 'all') all = .true.
	endif

	if (.not. all) then
		write(num_orbit_path,222)trim(AMSU_L1B_PATH),satellite_name,channel	
		222 format(a,a,'\Channel_',i2.2,'_number_to_process.txt')
		write(orbit_list_path,333)trim(AMSU_L1B_PATH),satellite_name,channel	
		333 format(a,a,'\Channel_',i2.2,'_revs_to_process.txt')

		open(unit=3,file=num_orbit_path)
		read(3,444)number_of_orbits_to_process
		444 format(i7)
		close(3)
		
		if (number_of_orbits_to_process .gt. 0) then

			open(unit=3,file=orbit_list_path)
			do index = 1,number_of_orbits_to_process
				read(3,555)orbits_to_process(index)
			enddo
	555     format(i7)

			call convert_L1B_orbits_to_L2A_orbits(sat_num,channel,number_of_orbits_to_process,orbits_to_process)
		endif
	else
		number_of_orbits_to_process = MAX_ORBITS_TO_PROCESS
		do i = 1,MAX_ORBITS_TO_PROCESS 
			orbits_to_process(i) = i
		enddo
		
		call convert_L1B_orbits_to_L2A_orbits(sat_num,channel,number_of_orbits_to_process,orbits_to_process)
	endif


end program AMSU_L1B_to_L2A_Driver









