	program update_driver
!@***h* MLT_RSS/Process_AMSU_Monthly_Driver.f90
!  NAME
!    Process_AMSU_Monthly_Driver.f90
! 
!  PURPOSE
!    Top level program for assembling AMSU L2A data into monthly grid point maps
!    This routine is for channels TLT, TMT, TTS, and TLS.
! 
!  DESCRIPTION
!    Top level program for assembling L2A data into monthly grid point maps.  
!
!  INPUT PARAMETERS
!    satnum     satellite number to process (10 (NOAA-15) to 15 (NOAA-19)
!    channel    AMSU channel number to process
!    year       Year to process
!
!  OUTPUT FILES WRITTEN
!    
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    top level program called from command line:
!    Process_AMSU_Monthly_Driver 10 TMT 2007 1 12
!
!@*****
	use date_and_time
	use nan_support
	use AMSU_constants

    use AMSU_L2C_Update_Monthlies_v3_2_CD
	use AMSU_L2C_Update_Monthlies_v3_2_TLT_CD

	use DFLIB

	integer(4)				:: sat_num
	integer(4)				:: channel
	integer(4)				:: error
	character(len=20)		:: buffer
	character(len=3)		:: view_str
	character(len=3)		:: channel_str
	character(len=4)		:: diurnal_str


	integer(4)			:: year,start_month,end_month
	type(date_Str)		:: first_date_to_process
	type(date_Str)		:: last_date_to_process


	call init_nan

	! read in satellite number
	! 10 = NOAA-15, 11 = NOAA-16, 12 = NOAA-17, 13 = NOAA-18, 14 = METOP-A, 15 = AQUA, 16 = NOAA-19
	buffer = ''
	call getarg(1,buffer,error)
	read(buffer,111)sat_num
	111 format(i3)
	if (error .ne. 0) stop
	

	! read in the channel string
	! should be one of the following
	! TLT, TMT, TTS, TLS, C01, C02, C03, C04, C06, C08, C10, C11, C12, C13, C14, C15
	buffer = ''
	call getarg(2,buffer,error)
	channel_str = buffer(1:3)
	if (error .ne. 0) stop

    ! read in the year to process
	buffer = ''
	call getarg(3,buffer,error)
	read(buffer,112)year
	112 format(i4)

	buffer = ''
	call getarg(4,buffer,error)
	read(buffer,112)start_month

	buffer = ''
	call getarg(5,buffer,error)
	read(buffer,112)end_month
	
	if (start_month .lt. 1) start_month = 1
	if (start_month .gt. 12) start_month = 12
	if (end_month .lt. 1) end_month = 1
	if (end_month .gt. 12) end_month = 12

	! select view set based on channe name

	select case (channel_str)
			case('TLT')
				view_str = 'TLT'
				channel = 5
			case('TMT')
				view_str = 'N12'
				channel = 5
			case('TTS')
				view_str = 'N12'
				channel = 7
			case('TLS')
				view_str = 'L8'
				channel = 9
			case('C01')
				view_str = 'N12'
				channel  = 1
			case('C02')
				view_str = 'N12'
				channel  = 2
			case('C03')
				view_str = 'N12'
				channel  = 3
			case('C04')
				view_str = 'N12'
				channel  = 4
			case('C06')
				view_str = 'N12'
				channel  = 6
			case('C08')
				view_str = 'N12'
				channel  = 8
			case('C10')
				view_str = 'N12'
				channel = 10
			case('C11')
				view_str = 'N12'
				channel = 11
			case('C12')
				view_str = 'N12'
				channel = 12
			case('C13')
				view_str = 'N12'
				channel = 13
			case('C14')
				view_str = 'N12'
				channel = 14
			case('C15')
				view_str = 'N12'
				channel = 15
			case default
				print *,'Channel Name Does Not Match'
				print *,channel_str
				stop
		endselect

	first_Date_to_process = set_date(start_month,1,year,error)
	last_Date_to_process = set_date(end_month,31,year,error)

	if (error .ne. 0) stop
    diurnal_str = 'CCM3'
	print *, 'Doing Monthly Update for', year,' Channel ',channel_str
	print *, 'Start_month = ',start_month
	print *, 'End_month = ',end_month
    print *, 'diurnal = ',diurnal_str
    print *, 'sat_num = ',sat_num

	if ((diurnal_str .eq. 'CMAM') .and. ((channel_str .eq. 'TLT') .or. (channel_str .eq. 'TMT'))) then
		print *,'CMAM diurnal adjustment not valid for ',channel_str
		stop
	endif

	print *, 'Using ',diurnal_str,' diurnal cycle'
	if (channel_str .eq. 'TLT') then
	    print *,'Calling TLT Version'
		call update_all_amsu_L2C_monthlies_v3_2_TLT_CD(sat_num,channel,first_date_to_process,last_date_to_process,view_str,diurnal_str,error)	
	else
		call update_all_amsu_L2C_monthlies_v3_2_CD(sat_num,channel,first_date_to_process,last_date_to_process,view_str,diurnal_str,error)	
	endif


end program	update_driver 