$freeform
!@***m* MLT_RSS/AMSU_orbitify_driver.f90
!  NAME
!    AMSU_orbity_driver.f90
!
!  PURPOSE
!    Top level program for ingesting data from CLASS into RSS L1B format.
!
!  DESCRIPTION
!    Top level program for ingesting data from CLASS into RSS L1B format.
!    Data is assembled into complete orbits, reformatted, and redudant data
!    discarded.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
!
!  USAGE
!    From command line:
!       AMSU_Orbitify_Driver 10 
!       10 is the satellite number (10 = NOAA-15, 11 = NOAA-16, etc.)
!@*****
program AMSU_Orbitify_driver

	use AMSU_L1A_ingest
	use AMSU_File_Locations
	use DFLIB
	use logging

	integer(4)	:: sat_num
	integer(4)	:: error
	character(len=120) buffer
	character(len=120) :: file

	character(len=120) :: incoming_path


	logical				:: wrt

	error = 0
	buffer = ''
	call getarg(1,buffer,error)
	read(buffer,111)sat_num
	111 format(i3)
	
	buffer = ''
	incoming_path = ' '
	call getarg(2,buffer,error)

	if (error .eq. 0) then
		file = trim(buffer)
		call init_log_file(file,error)
		print *,'Init_log_file ',file
	endif

	if ((sat_num .ge. 10) .and. (sat_num .le. 16)) then
		if (sat_num .eq. 10) then
			incoming_path = trim(AMSU_L1A_Path)//'NOAA-15\'
		endif
	
		if (sat_num .eq. 11) then
			incoming_path = trim(AMSU_L1A_Path)//'NOAA-16\'
		endif			
		
		if (sat_num .eq. 12) then
			incoming_path = trim(AMSU_L1A_Path)//'NOAA-17\'
		endif			
		
		if (sat_num .eq. 13) then
			incoming_path = trim(AMSU_L1A_Path)//'NOAA-18\'
		endif

		if (sat_num .eq. 14) then
			incoming_path = trim(AMSU_L1A_Path)//'METOP-A\'
		endif

		if (sat_num .eq. 15) then
			write(log_str,*)'Fatal Error: Satellite Number out of range ',sat_num
			call log(log_str,log_err)
			write(log_str,*)'AQUA data is not processed using this code'
			call log(log_str,log_err)
			stop
		endif		
		
		if (sat_num .eq. 16) then
			incoming_path = trim(AMSU_L1A_Path)//'NOAA-19\'
		endif

	else
		write(log_str,*)'Fatal Error: Satellite Number out of range ',sat_num
		call log(log_str,log_err)
		stop
	endif


		
	write(log_str,115) sat_num,incoming_path
115 format(1x,'Orbitifying data for satellite # ',i2,' from ',a)
	call log(log_str,log_err)

	wrt = .true.  ! set to true to write out results
	call ingest_AMSU_L1A(sat_num,incoming_path,wrt)

end program AMSU_Orbitify_driver
