$freeform
!@***s* MLT_RSS/AMSU_Level_2A.f90/AQUA_AMSU_L2A_add_corrections_driver.f90
!  NAME
!    AQUA_AMSU_L2A_add_corrections_driver
!
!  PURPOSE
!    To level driver routine for adding corrections to AMSU-AQUA L2A orbit files.
!
!  DESCRIPTION
!   Adds corrections to AMSU-AQUA L2A orbit files.  AQUA files arrive as brightness temperatures, so L1B to L2A
!   is not needed.  For NOAA and EUMETSAT satellites, diurnal and angle adjustments are performed as this conversion
!   is done.  So we need a separate routine for AQUA to calculate adjustments.
!
!  INPUTS
!    integer(4)                     :: channel                  ! AMSU channel, command line argument
!  INPUT FILE
!    file of robit numbers to process:  orbits_to_adjust.txt
!    integer(4)                     :: num_orbits               ! number of orbits to process
!    integer(4),dimension(MAX_ORBITS_TO_PROCESS) :: orbit_list   ! list of orbits to process
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    10/21/2012 Initial Version prepared for NCDC
!  USAGE
!    AQUA_AMSU_L2A_add_corrections_driver 5   (at command line)
!@*****
program AMSU_L2A_Add_Corrections_Driver

	use NAN_support
	use date_and_time
	use AMSU_Constants
	use AMSU_Types
	use AMSU_equator_crossing_times
	use AMSU_File_Locations
	use AMSU_Level_2A
	use AQUA_AMSU_L2A_Corrections

	use DFLIB  ! enables use of command line arguments


	integer(4)							:: sat_num
	integer(4)							:: channel
	integer(4)							:: orbit_num
	integer(4)							:: j
	integer(4),dimension(MAX_ORBITS_TO_PROCESS)			:: orbits_to_process
	integer(4)							:: number_of_orbits_to_process
	integer(4)							:: error

	character(len=7)					:: satellite_name
	character(len=120)					:: buffer
	character(len=120)					:: orbit_list_file

	logical								:: wrt,all
	logical(4)							:: reprocess

	integer(4)	:: i

	error = 0
	buffer = ''
	sat_num = 15
	satellite_name = 'AQUA   '  !don't leave out spaces!

	call getarg(1,buffer,error)
	111 format(i3)
	read(buffer,111)channel
	if (error .ne. 0) stop
	
	call init_NAN


	 write(orbit_list_file,901)trim(AMSU_L2A_PATH),trim(satellite_name),channel
901  format(a,a,'\Channel_',i2.2,'\orbits_to_adjust.txt')

	 open the file that contains the list of orbits
    print *,'Opening ',orbit_list_file
	open(unit = 3, file = orbit_list_file,iostat = io_error)
	
	if(io_error .ne. 0) then
		write(*,*),'Fatal Error:  Could not open orbit file list: '//orbit_list_file
		error = -1
		stop
	endif

	do j = 1,MAX_ORBITS_TO_PROCESS
		read(3,888,end=999) orbit_num
		orbits_to_process(j) = orbit_num
	enddo

888 format(i6)

999 number_of_orbits_to_process = 55529-255
    print *,'Fortran: Processing ',number_of_orbits_to_process,' Orbits'
	call add_corrections_to_AQUA_L2A_orbits(sat_num,channel,number_of_orbits_to_process,orbits_to_process,reprocess)


end program AMSU_L2A_Add_Corrections_Driver









