$freeform
	! reads incoming MSU L1B files and cuts them up into orbits



	module MSU_L1B_Ingest

		use msu_level_1a
		use MSU_Level_1B
		use nan_support
		use date_and_time
		use msu_types
		use msu_constants
		use msu_date
		use interpolate
		use get_file_list
		use numerical_recipes_sorting
		use msu_l1b_inventory
		use msu_L2A_inventory
		use MSU_L1B_Quality


		implicit none


		integer(4),parameter  :: NRECMAX=2000000
		integer(4),parameter  :: NUM_CROSS_MAX=6000
		integer(4),parameter  :: MAX_FILES = 7000
		integer(4),parameter  :: NUM_ORBITS_IN_STACK = 3

		character(len = file_len),parameter		:: LOG_FILE_PATH = '\\dell-p1700d\e\MSU_processing\logs\orbit_logs\'


	contains

		subroutine ingest_MSU_l1b(msu_incoming_path,wrt)

!@***h* MLT_RSS/MSU_L1B_ingest.f90/ingest_MSU_l1b
!  NAME
!    ingest_MSU_l1b.f90
!
!  PURPOSE
!    Ingests MSU L1B data from NOAA.
!
!  DESCRIPTION
!    Ingests MSU L1B data from NOAA.  Reads partial orbit files from NOAA.  Tests
!    for duplicate records.  Removed unneeded data, and writes data back in RSS 
!    L1B format as complete orbits.
!
!  INPUTS
!    character(len = FILE_LEN)		::	msu_incoming_path ! location of MSU CLASS L1B files
!    logical						::  wrt   ! set true to write out files.
!
!  FILES READ
!    MSU L1B files from CLASS
!
!  FILES WRITTEN
!    RSS format L1B files
!    RSS orbit inventory files
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    10/15/2012 Initial Version prepared for NCDC
!
!  USAGE
!    call ingest_MSU_l1b(msu_incoming_path,wrt)
!@*****

		character(len = FILE_LEN)							::	msu_incoming_path	! outgoing file name
		logical												::  wrt
		character(len = FILE_LEN),dimension(MAX_FILES)		::	raw_l1b_file		! Incoming file name
		character(len = FILE_LEN)							::	file				! outgoing file name

		type(msu_raw)						:: msu_l1b_scan	! structure containing the scan data

		! stuff for the orbit data stack

		type(msu_L1B_raw_orbit_data),dimension(NUM_ORBITS_IN_STACK)		:: L1B_orbit_data
		integer(4),dimension(NUM_ORBITS_IN_STACK)						:: stack_orbit_num
		integer(4),dimension(NUM_ORBITS_IN_STACK)						:: stack_last_accessed
		integer(4)														:: orbit_index
		logical															:: orbit_found
		integer(4)														:: min_stack_last_accessed
		integer(4)														:: stack_pos_to_change
		

		! end stuff for the orbit data stack

		character(len = 20) :: dt_String

		character(len = 20) :: start_gap_date_String

		character(len = 20) :: end_gap_date_string





		! errors

		integer(4)							:: read_Error
		integer(4)							:: interp_error
		integer(4)							:: inv_error
		integer(4)							:: err

		integer(4)							:: sat_num
			

		real(8),dimension(NRECMAX)			:: scan_r_d	        ! r_d for this scan
		integer(4),dimension(NRECMAX)		:: scan_num_index   ! estimated scan number for each scan_index, with gaps accounted for -- set to minus 1 to ignore scan
																! i.e. the r_d of the ith actual scan lives in scan_r_d(scan_num_index(i))
		integer(2),dimension(NRECMAX)		:: scan_asc_desc    ! node for this scan
		real(4),dimension(NRECMAX)			:: scan_lat			! nadir latitude for this scan
		real(4),dimension(NRECMAX)			:: scan_lon	        ! nadir longitude for this scan
		logical,dimension(NRECMAX)			:: scan_gap_exists  ! gap exists between this record and the next record
		real(4),dimension(NRECMAX)			:: scan_gap_size    ! size of gap in units of typical scan difference

		integer(4)							:: scan_index
		integer(4)							:: int_gap_size

									 
		! crossing time stuff
		
		real(8),dimension(NUM_CROSS_MAX)		:: crossing_time
		real(4),dimension(NUM_CROSS_MAX)		:: crossing_long
		integer(4),dimension(NUM_CROSS_MAX)		:: orbit_num
		logical,dimension(NUM_CROSS_MAX)		:: duplicate
		integer(4)								:: num_cross
		integer(4)								:: cross_num
		real(8)									:: r_d_offset

		integer(4)								:: scan_num
		integer(4)								:: num_scan
		integer(4)								:: num_scan_this_file

		type(date_str)							:: cross_date


		! inventory file stuff

		type(msu_inventory_str)				:: msu_inventory
		real(8)								:: max_inv_rd    !rd_2000 of last record in inventory
		type(date_str)						:: max_inv_date,sc_Date


		! orbit assignment stuff

		integer(4)								:: 	delta_orbit
		real(4)								    :: 	delta_orbit_real
		integer(4)								::  start_orbit
		logical									::  check_for_dupl
		real(8)									::  time_err
		real(8)									::  guess_time
		integer(4)								::  guess_orbit_num
		integer(4)								::  j
		real(8)									::  time_offset
		integer(4),dimension(NRECMAX)			::  scan_orbit_num	! orbit number for this scan
		integer(4),dimension(NRECMAX)			::  orbit_scan_num  ! scan number in orbit
		integer(4)								::  orbit_offset
		real(8)									::  cross_time_guess
		real(8)									::  scan_guess_real
		integer(4)								::  scan_guess

		integer(4),dimension(10)				:: tot_err									
		integer(4),dimension(10,NUM_ORBITS_IN_STACK)	:: tot_err_this_output_file
		integer(4),dimension(NUM_ORBITS_IN_STACK)		:: num_scans_this_output_file									

		integer(4)								:: file_num
		integer(4)								:: num_files
		integer(4)								:: num_files_to_process !debugging

		real(8)									:: delta_time
		real(4)									:: delta_lat

		INTEGER(2) KQUAL
		integer(4),parameter					:: ILU = 20

		type(msu_L1B_orbit_header)				:: msu_orbit_header
		!type(msu_L1B_orbit_header)				:: msu_orbit_header_test

		!type(msu_L1B_raw_orbit_data)			:: 	msu_orbit_scan_data
		!type(msu_L1B_raw_orbit_data)			:: 	msu_orbit_scan_data_test

		integer(4),parameter					:: log_unit = 24
		integer(4),parameter					:: gap_unit = 23

		character(len = FILE_LEN)					::	log_file	
		character(len = FILE_LEN)					::	gap_file
		
		type(date_str)	  :: cur_date
		character(len=20) :: cur_date_string
		





			call init_NAN
			sat_num = 9
			call copy_msu_l1b_inventory_file_text(sat_num,err)

			! open the log files

			cur_date		= current_date(err)
			cur_date_string = date_string_for_file(cur_date)

			log_file = trim(LOG_FILE_PATH)//'msu_orbitification_log_file_'//trim(cur_date_String)//'.txt'
			open(unit=LOG_unit,file = log_file)

			gap_file = trim(LOG_FILE_PATH)//'msu_gaps_'//trim(cur_date_String)//'.txt'
			open(unit=GAP_unit,file = gap_file)

			! read the directory in the incoming file list

			call get_files_in_path(MSU_INCOMING_PATH,MAX_FILES,num_files,raw_l1b_file,err)

			write(log_unit,101)num_files,trim(MSU_INCOMING_PATH)
	101		format(' Found ',i6,' files in ',a)

			! sort the files according to date using the appropriate part of the file name

			call sort_msu_l1b_file_list(num_files,raw_l1b_file)		
				
	!---------------------------------------------------------------------------
	! read through all the files, and keep scan times and crossing times
	!---------------------------------------------------------------------------

			num_cross = 1
			num_Scan  = 0
			num_scan_this_file = 0

		!	num_files_to_process = 70
			num_files_to_process = num_files

			do file_num = 1,num_files_to_process
				file  = trim(MSU_INCOMING_PATH) // raw_l1b_file(file_num)
				open(unit = ILU,file = file,status= 'OLD',form = 'binary')
	!			write(21,110)file
				write(6,110)trim(file(20:))
	110			format(1x,a)

				read_error = 0
				num_Scan_this_file = 0
				do while (read_Error .le. 0)
					call read_L1A_msu_scan_saa(ILU,msu_l1b_scan,read_error)
					if (read_error .eq. 0) then
						num_scan = num_Scan + 1
						num_scan_this_file = num_Scan_this_file + 1

						!call evaluate_l1b_quality_control_initial(msu_L1B_scan,kqual)

						scan_r_d(num_scan) = msu_l1b_scan%date%rd_2000
						scan_lat(num_scan) = msu_L1b_scan%fov_latitude(6)  !nadir latitude
						scan_lon(num_scan) = msu_L1B_scan%fov_longitude(6) !nadir longitude
					endif
				enddo
				write(log_unit,1212)file_num,raw_l1b_file(file_num),num_Scan_this_file,num_scan
	1212		format(1x,i10,2x,a,2x,i5,1x,i10)
				close(ILU)
			enddo

			scan_asc_desc		= UNKNOWN
			scan_gap_exists		= .false.
			scan_gap_size		= 1

			! first, do the first one using the next scan

			delta_time = scan_r_d(2) - scan_r_d(1)
			delta_lat  = scan_lat(2) - scan_lat(1)

			! set asc/desc flag for first scan using next scan
			
			if ((delta_time .gt. 0.5*MSU_SCAN_TIME) .and. (delta_time .lt. 1.5*MSU_SCAN_TIME)) then ! single forward time step on this side
				if (delta_lat .gt. 0.001) scan_Asc_desc(1)  = ASCENDING
				if (delta_lat .lt. -0.001) scan_Asc_desc(1) = DESCENDING
			endif 
			 
			! loop through rest of scans

			do scan_num = 2,num_scan-1
				delta_time = scan_r_d(scan_num) - scan_r_d(scan_num-1)
				if ((delta_time .gt. 0.5*MSU_SCAN_TIME) .and. (delta_time .lt. 1.5*MSU_SCAN_TIME)) then 
					! single forward time step to get to this scan from previous one -- normal situation
					delta_lat  = scan_lat(scan_num) - scan_lat(scan_num-1)
					if (delta_lat .gt. 0.001) scan_Asc_desc(scan_num)  = ASCENDING
					if (delta_lat .lt. -0.001) scan_Asc_desc(scan_num) = DESCENDING
				else 
					scan_gap_exists(scan_num-1) = .true.  ! set for either forward or backward gaps
					scan_gap_size(scan_num-1)   = delta_time/MSU_SCAN_TIME
					start_gap_date_string		= date_String(set_date_2000(scan_r_d(scan_num-1),err))
					end_gap_date_string			= date_String(set_date_2000(scan_r_d(scan_num),err))

					write(6,212) start_gap_date_string,end_gap_date_string,idnint(delta_time/MSU_SCAN_TIME)

					! write the big ones out to unit 23

					if (delta_time .ge. 100.0*MSU_SCAN_TIME) then
						write(gap_unit,212) start_gap_date_string,end_gap_date_string,idnint(delta_time/MSU_SCAN_TIME)
					endif

					! look at the next scan scan --- if there no a measurement on 
					! either side, this is a single isolated scan, and
					! asc_desc remains unknown (or 0)

					delta_time = scan_r_d(scan_num+1) - scan_r_d(scan_num)
					if ((delta_time .gt. 0.5*MSU_SCAN_TIME) .and. (delta_time .lt. 1.5*MSU_SCAN_TIME)) then
						delta_lat  = scan_lat(scan_num+1) - scan_lat(scan_num)
						if (delta_lat .gt. 0.001) scan_Asc_desc(scan_num)  = ASCENDING
						if (delta_lat .lt. -0.001) scan_Asc_desc(scan_num) = DESCENDING
					endif
				endif
			enddo

			! do the special case of the last scan

			delta_time = scan_r_d(num_scan) - scan_r_d(num_Scan-1)
			delta_lat  = scan_lat(num_scan) - scan_lat(num_Scan-1)
			
			if ((delta_time .gt. 0.5*MSU_SCAN_TIME) .and. (delta_time .lt. 1.5*MSU_SCAN_TIME)) then
				if (delta_lat .gt. 0.001) then
					scan_asc_desc(num_scan) = ASCENDING
				else if (delta_lat .lt. -0.001) then
					scan_asc_desc(num_scan) = DESCENDING
				endif
			else
				scan_gap_exists(num_Scan-1) = .true.
				scan_gap_size(num_scan-1)   = delta_time/MSU_SCAN_TIME
				start_gap_date_string = date_String(set_date_2000(scan_r_d(scan_num-1),err))
				end_gap_date_string   = date_String(set_date_2000(scan_r_d(scan_num),err))

				write(6,212) start_gap_date_string,end_gap_date_string,idnint(delta_time/MSU_SCAN_TIME)

				if (delta_time .ge. 100.0*MSU_SCAN_TIME) then
					write(gap_unit,212) start_gap_date_string,end_gap_date_string,idnint(delta_time/MSU_SCAN_TIME)
				endif

	212			format(1x,a,1x,a,1x,i9)
 
			endif 

	!-----------------------------------------------
	! end try to figure out ascending/descending
	!-----------------------------------------------

	close(23)


	!-----------------------------------------------
	! begin assign scan numbers
	!-----------------------------------------------
	scan_index = 1
	scan_num_index = -1  ! set all initially to -1  -- no scan pointed at
	scan_num_index(scan_index) = 1

	do scan_num = 2,num_scan
		if (.not. scan_gap_exists(scan_num-1)) then
			! no gap, so just increment scan index and have it point to scan num
			scan_index = scan_index+1
			scan_num_index(scan_index) = scan_num
		else
			int_gap_size = nint(scan_gap_size(scan_num -1))
			if (abs(int_gap_size - scan_gap_size(scan_num -1)) .gt. 0.1) then
				print *,'Warning -- non integer scan gap size,scan index = ',scan_index,scan_gap_size(scan_num -1)
			endif
			!print *,int_gap_size
			scan_index = scan_index + int_gap_size
			scan_num_index(scan_index) = scan_num
		endif
	enddo

	!-----------------------------------------------
	! end assign scan numbers
	!-----------------------------------------------


	!-----------------------------------------------
	! begin look for ascending equator crossings
	!-----------------------------------------------


		do scan_num = 2,scan_index

			if ((scan_asc_desc(scan_num_index(scan_num)) .eq. ASCENDING) .and. (scan_num_index(scan_num-1) .gt. 0)) then
									 
				if ((scan_lat(scan_num_index(scan_num)) .ge. 0.0) .and. (scan_lat(scan_num_index(scan_num-1)) .lt. 0.0)) then
					! we just crossed the equator going up
				
					r_d_offset = floor(scan_r_d(scan_num_index(scan_num-1)))
									
					crossing_time(num_cross) =  lin_interpolate_r4_r8( &
												0.0,  &
												scan_lat(scan_num_index(scan_num-1)), &
												scan_lat(scan_num_index(scan_num)) ,  &
												scan_r_d(scan_num_index(scan_num-1)) - r_d_offset, &
												scan_r_d(scan_num_index(scan_num))   - r_d_offset, &
												interp_error)        + r_d_offset 

					

					crossing_long(num_cross) =  lin_interpolate_directions(0.0,  &
												scan_lat(scan_num_index(scan_num-1)), &
												scan_lat(scan_num_index(scan_num)) ,  &
												scan_lon(scan_num_index(scan_num-1)), &
												scan_lon(scan_num_index(scan_num)) ,  &
												interp_error)


	!				write(21,111)num_Cross,scan_num,crossing_long(num_cross),crossing_time(num_cross) 
	!	111			format(1x,i4,1x,i3,1x,f8.1,1x,f8.3,1x,i10,1x,i10)

					num_cross = num_cross + 1

				endif
			endif
		enddo

	num_cross = num_cross-1
		
	!-----------------------------------------------
	! end look for ascending equator crossings
	!-----------------------------------------------


	!-----------------------------------------------
	! begin assign orbit numbers
	!-----------------------------------------------

	! read in existing inventory list of orbit numbers/ascending node times
		sat_num = 9
		call read_msu_L1B_inventory_file_text(sat_num,msu_inventory,inv_Error)

		max_inv_rd = msu_inventory%start_rd_2000(msu_inventory%max_orbit) - &
					(msu_inventory%start_swath(msu_inventory%max_orbit) - 1) * MSU_SCAN_TIME

		max_inv_Date = set_date_2000(max_inv_rd,err)

		! crossing time for last orbit...
		
	! check to see if the earliest new orbit is before or after the last old orbit
		delta_time = (crossing_time(1) - max_inv_rd) 
		if (delta_time .gt. (MSU_ORBIT_TIME*0.5)) then
			! first new orbit is after last old orbit -- estimate new orbit number
			delta_orbit = idnint(delta_time/MSU_ORBIT_TIME) 
			start_orbit = msu_inventory%max_orbit+delta_orbit	
			check_for_dupl = .false.
		else
			! there is some overlap
			delta_orbit_real = delta_time/MSU_ORBIT_TIME
			delta_orbit = nint(delta_orbit_real)
			 
			check_for_dupl = .true.
			start_orbit = msu_inventory%max_orbit+delta_orbit
		endif


	! go through list of new ascending node times and assign orbit numbers to crossings

		time_offset = 0.0d0
		duplicate = .false.
		do j = 1,num_cross
			delta_time			= crossing_time(j) - (crossing_time(1)+time_offset)
			delta_orbit			= idnint(delta_time/MSU_ORBIT_TIME)
			guess_orbit_num		= start_orbit + delta_orbit
			guess_time			= crossing_time(1) + time_offset + delta_orbit*MSU_ORBIT_TIME
			time_err			= guess_time - crossing_time(j)
			time_offset			= time_offset-time_err

			cross_date = set_date_2000(crossing_time(j),err)
			dt_string = date_String(cross_date)


			if (msu_inventory%orbit_valid(guess_orbit_num)) then  ! we think this orbit number is already in inventory
				duplicate(j) = .true.
				if (abs(86400.0*(crossing_time(j)-msu_inventory%crossing_rd_2000(guess_orbit_num))) .lt. 2000) then
					orbit_num(j)		= guess_orbit_num
				else
					orbit_num(j)		= guess_orbit_num
					write(log_unit,*)'warning -- large difference in crossing times'
					write(log_unit,*)'orbit num = ',guess_orbit_num,'diff(sec) = ',86400.0*(crossing_time(j)-msu_inventory%crossing_rd_2000(guess_orbit_num))
				endif
			else
				orbit_num(j) = guess_orbit_num
			endif


			! update the inventory data with these new numbers if not a duplicate of whats already there

			if (.not. duplicate(j)) then
				msu_inventory%crossing_rd_2000(guess_orbit_num) = crossing_time(j)
				msu_inventory%orbit_valid(guess_orbit_num) = .true.
				if (msu_inventory%max_orbit .lt. guess_orbit_num) then
					msu_inventory%max_orbit = guess_orbit_num
				endif
			endif

		enddo
		
	! go through scans and assign orbit number and scan number

		do scan_num = 1,num_scan
			do cross_num = num_cross,1,-1
				if (scan_r_d(scan_num) .gt. (crossing_time(cross_num)- MSU_ORBIT_TIME)) then
					orbit_offset = floor((scan_r_d(scan_num) - crossing_time(cross_num))/MSU_ORBIT_TIME)
					scan_orbit_num(scan_num) = orbit_num(cross_num) + orbit_offset
					cross_time_guess = crossing_time(cross_num) + MSU_ORBIT_TIME*orbit_offset
					scan_guess_real = (scan_r_d(scan_num)-cross_time_guess)/MSU_SCAN_TIME
					scan_guess = floor(scan_guess_real)+1
					orbit_scan_num(scan_num) = scan_guess

					if ((scan_guess .ge. 248) .and.  (scan_lat(scan_num) .gt. 0.0)) then
						! oops need to adjust the next crossing time just a little
						! and put this scan in the next orbit
						crossing_time(cross_num+1) = crossing_time(cross_num+1) - 0.05*MSU_SCAN_TIME
						orbit_scan_num(scan_num) = 1
						scan_orbit_num(scan_num) = scan_orbit_num(scan_num) + 1
	!					write(21,889)scan_num,scan_orbit_num(scan_num),orbit_scan_num(scan_num),cross_time_guess,scan_guess_Real,scan_lat(scan_num)
						write(6,889)scan_num,scan_orbit_num(scan_num),orbit_scan_num(scan_num),cross_time_guess,scan_guess_Real,scan_lat(scan_num)
	889					format(3(1x,i10),4(1x,f11.5))
					endif

					if ((scan_guess .le. -5) .and.  (scan_lat(scan_num) .lt. 0.0)) then
						! oops need to adjust the this crossing time just a little
						! and put this scan in the previous orbit
			
						crossing_time(cross_num) = crossing_time(cross_num) + 0.05*MSU_SCAN_TIME
						scan_guess = 1
						scan_orbit_num(scan_num) = scan_orbit_num(scan_num) - 1
						orbit_scan_num(scan_num) = orbit_scan_num(scan_num-1)+1
						write(6,889)scan_num,scan_orbit_num(scan_num),orbit_scan_num(scan_num),cross_time_guess,scan_guess_Real,scan_lat(scan_num)
					endif
						sc_Date = set_date_2000(scan_r_d(scan_num),err)
					exit
				endif
			enddo
		enddo
		
	!   




	!-----------------------------------------------
	! end assign orbit numbers
	!-----------------------------------------------



	if (Wrt) then 

	!-----------------------------------------------
	! begin write out orbit files
	!-----------------------------------------------

		! we read all the files a second time, this time assigning the results
		! to the appropriate places in the orbit file
		!
		! orbit data is held in a not recnetly accessed first out pseaudo stack
		! consisting of NUM_ORBITS_IN_STACK orbits of MAX_SCANS_IN_ORBIT scans

			num_Scan						= 0
			num_scans_this_output_file		= 0
			tot_err							= 0
			tot_err_this_output_file		= 0

			do file_num = 1,num_files_to_process
				file  = trim(MSU_INCOMING_PATH) // raw_l1b_file(file_num)
				open(unit = ILU,file = file,status= 'OLD',form = 'binary')
				write(6,117)trim(file)
	117			format(1x,a)

				num_scan_this_file = 0
				read_error = 0
		
				do while (read_Error .le. 0)
					call read_L1A_msu_scan_saa(ILU,msu_l1b_scan,read_error)
					if (read_error .ne. 0) cycle
					num_scan = num_Scan + 1
					num_scan_this_file = num_Scan_this_file + 1
					call check_l1b_quality(msu_L1B_scan)
					call evaluate_l1b_quality_control_initial(msu_L1B_scan,kqual)
					if (kqual .ne. 0) cycle
					if(.not. almost_equal(scan_r_d(num_scan),msu_l1b_scan%date%rd_2000)) then
						! we have a problem.....  something changed in the files.
						print *, 'time match problem, scan ',num_scan
						stop
					endif
					if (scan_orbit_num(num_scan) .gt. 0) then !(we know where this scan goes, otherwise ignore it
						orbit_found = .false.
						do orbit_index = 1,NUM_ORBITS_IN_STACK
							if (scan_orbit_num(num_scan) .eq. stack_orbit_num(orbit_index)) then
								! we have a match to an existing orbit stack member

								! add the scan to the data for this orbit

								call add_scan_to_orbit(msu_l1b_scan,L1B_orbit_data(orbit_index),orbit_scan_num(num_scan),err)
								
								tot_err(err) = tot_err(err) + 1
								 
								num_scans_this_output_file(orbit_index) = num_scans_this_output_file(orbit_index) + 1 
								tot_err_this_output_file(err,orbit_index) = tot_err_this_output_file(err,orbit_index) + 1 
 
								orbit_found = .true.

								stack_last_accessed(orbit_index) = num_scan
								exit
							endif
						enddo
						if (.not. orbit_found) then
							! didn't find a matching orbit -- we need to write the least most recently used
							! orbit and assign its place to a new orbit, check for an existing orbit file
							! and read it in if necessary.
							min_stack_last_accessed = minval(stack_last_accessed)
							stack_pos_to_change = 0
							do orbit_index = 1,NUM_ORBITS_IN_STACK
								if (min_stack_last_accessed .eq. stack_last_accessed(orbit_index)) then
									stack_pos_to_change = orbit_index
									exit
								endif
							enddo
							!print *, stack_last_accessed
							!print *, stack_orbit_num
							if (stack_pos_to_change .le. 0) then
								! problem
								print *,'Bizarre Stack Error'
								stop
							endif

							! write out orbit, if it has any data

							if (stack_orbit_num(stack_pos_to_change) .gt. 0) then
								if (new_data_present(L1B_orbit_data(stack_pos_to_change))) then
									call write_L1B_orbit_file(	sat_names_2(sat_num),					&
																stack_orbit_num(stack_pos_to_change),	&
																L1B_orbit_data(stack_pos_to_change),	&
																num_Scans_this_output_file(stack_pos_to_change),  &
																tot_err_this_output_file(:,stack_pos_to_change),	&
																log_unit,								&
																err)
									endif
									!
									! Update the inventory data
									
									call update_inventory_data(	msu_inventory,							&
																L1B_orbit_data(stack_pos_to_change),	&
																stack_orbit_num(stack_pos_to_change))
										
								!endif
							endif

							! zero the orbit data

							do scan_index = 1,MAX_SCANS_PER_ORBIT
								call zero_msu_l1b_orbit_file_scan(L1B_orbit_data(stack_pos_to_change)%scan_data(scan_index))
								L1B_orbit_data(stack_pos_to_change)%scan_valid(scan_index) = 0
							enddo

							! zero scan and error counters

							num_scans_this_output_file(stack_pos_to_change) = 0 
							tot_err_this_output_file(:,stack_pos_to_change) = 0 
 

							! read in orbit file, if it exists

							call read_L1B_orbit_file(	sat_names_2(sat_num),			&
														scan_orbit_num(num_scan),		&
														msu_orbit_header,				&
														L1B_orbit_data(stack_pos_to_change),	&
														0,								&	 ! read from test directory
														err)

							! set orbit number in the stack

							stack_orbit_num(stack_pos_to_change) = scan_orbit_num(num_scan) 

							! add the current scan to this new stack pos

							call add_scan_to_orbit(msu_l1b_scan,L1B_orbit_data(stack_pos_to_change),orbit_scan_num(num_scan),err)
							tot_err(err) = tot_err(err) + 1 
							num_scans_this_output_file(orbit_index) = num_scans_this_output_file(orbit_index) + 1 
							tot_err_this_output_file(err,orbit_index) = tot_err_this_output_file(err,orbit_index) + 1 
							orbit_found = .true.
							stack_last_accessed(stack_pos_to_change) = num_scan
						endif
					endif

					
				enddo
				close(ILU)
			enddo

			! all files processed -- flush the orbit data stack

			do orbit_index = 1,NUM_ORBITS_IN_STACK
				if (stack_orbit_num(stack_pos_to_change) .gt. 0) then
					if (new_data_present(L1B_orbit_data(stack_pos_to_change))) then
						call write_L1B_orbit_file(	sat_names_2(sat_num),					&
													stack_orbit_num(orbit_index),			&
													L1B_orbit_data(orbit_index),			&
													num_Scans_this_output_file(orbit_index),&
													tot_err_this_output_file(:,orbit_index),&
													log_unit,								&
													err  )
											
						call update_inventory_data(	msu_inventory,							&
													L1B_orbit_data(orbit_index),			&
													stack_orbit_num(orbit_index))

					endif
				endif
			enddo

			! update the inventory files

			call copy_msu_l1b_inventory_file(sat_num,err)
			call write_msu_l1b_inventory_file(sat_num,msu_inventory,err)
			call write_msu_l1b_inventory_file_text(sat_num,msu_inventory,err)
	endif


	!-----------------------------------------------
	! end write out orbit files
	!-----------------------------------------------


			close(22)
			close(23)
			print *,num_scan
			print *,tot_err(1:6)
			write(log_unit,2323)num_scan,tot_err(1:6)
	2323	format(1x,10i12)
			close(log_unit)


	end subroutine ingest_MSU_l1b

    function almost_equal(t1,t2)
!@***s* MLT_RSS/MSU_L1B_ingest.f90/almost_equal
!  NAME
!    almost_equal
!
!  PURPOSE
!    tests two real numbers to see if they are equal with 2.0e-5
!
!  DESCRIPTION
!     tests two real numbers to see if they are equal with 2.0e-5
!
!  INPUTS
!     real(4)   :: t1,t2   ! numbers to test 
!
!  Return Value
!    logical   ! set to true if absolute difference between two numbers is less than 2.0e-5   
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!    equal = almost_equal(x1,x1)
!
!@*****

			! compares two real*8's -- true is their absolute difference is less than 2.0*10-5 (about 2 seconds)

			real(8)		:: t1
			real(8)		:: t2
			logical     :: almost_equal

			almost_equal = .false.
			if (abs(t1 - t2) .lt. 2.0e-5) then
				almost_equal	= .true.
			endif

			return
		end function almost_equal
 

		subroutine sort_msu_l1b_file_list(num_files,raw_l1b_file)
!@***s* MLT_RSS/MSU_L1B_ingest.f90/sort_msu_l1b_file_list
!  NAME
!    sort_msu_l1b_file_list
!
!  PURPOSE
!    sorts list of CLASS L1B files
!
!  DESCRIPTION
!     sorts list of CLASS L1B files
!
!  INPUTS
!    integer(4)											::  num_files     !number of file in list
!    character(len = FILE_LEN),dimension(MAX_FILES)		::	raw_l1b_file  ! list of files to sort.  List is returned sorted.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!   call sort_msu_l1b_file_list(num_files,raw_l1b_file)
!
!@*****
			integer(4)											::  num_files
			character(len = FILE_LEN),dimension(MAX_FILES)		::	raw_l1b_file

			character(len = FILE_LEN),dimension(MAX_FILES)		::	temp_file_list


			character(len = 5)				:: temp1
			character(len = 3)				:: temp2
			character(len = 9)				:: temp3

			real(4),dimension(num_files) :: file_index
			integer(4),dimension(num_files) :: indx
			integer(4)						:: i
			
			
			! first extract integer related to date and time

			do i = 1,num_files
				temp1 = raw_l1b_file(i)(14:18)
				temp2 = raw_l1b_File(i)(21:23)
				temp3 = temp1 // temp2
				read(temp3,111) file_index(i)
	111			format(f9.0)

			enddo

			! sort this integer array

			call indexx(num_files,file_index,indx)

			! put sorted file names in temp variable

			do i = 1, num_files
				temp_file_list(i) = raw_l1b_file(indx(i))
			enddo

			! put back into original array

			raw_l1b_file = temp_file_list

			return

		end subroutine sort_msu_l1b_file_list

		subroutine add_scan_to_orbit(	scan_to_add,					&
										L1B_orbit_data,					&
										scan_num,						&
										error)
!@***s* MLT_RSS/MSU_L1B_ingest.f90/add_scan_to_orbit
!  NAME
!    add_scan_to_orbit
!
!  PURPOSE
!    adds a single MSU scan to MSU L1B orbit structure
!
!  DESCRIPTION
!     adds a single MSU scan to MSU L1B orbit structure
!
!  INPUTS
!    type(msu_raw)						:: scan_to_add       ! scan to be added
!    type(msu_L1B_raw_orbit_data)		:: L1B_orbit_data    ! orbit structure to add scan to
!    integer(4)							:: scan_num          ! scan number in orbit

!  OUTPUTS
!    integer(4)							:: error   ! error flag			! errors:
!           Error = 6 means something weird happened
!           Error = 1 means match found in correct place, no replacement made
!           Error = 2 means empty spot in correct place, replacement made
!           Error = 3 means match found, but to one side, no replacement made
!           Error = 4 means match found, but to one side, replacement made
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
!
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!
!  USAGE
!   call add_scan_to_orbit(	scan_to_add,L1B_orbit_data,scan_num,error)
!
!@*****
			type(msu_raw)						:: scan_to_add
			type(msu_L1B_raw_orbit_data)		:: L1B_orbit_data
			integer(4)							:: scan_num,scan_match
			integer(4)							:: error
			real(8)								:: delta_time,delta_time_earlier,delta_time_later
			integer(4)							:: fov
			integer(4)							:: tot_fov_qual_old
			integer(4)							:: tot_fov_qual_new

			! this routine puts the data in scan into the L1B_orbit_data array
			!
			! eventually, we need to make decisions about with version of the scan data is
			! better, if one already exists, and they are different

! errors:
!   Error = 6 means something weird happened
!	Error = 1 means match found in correct place, no replacement made
!	Error = 2 means empty spot in correct place, replacement made
!   Error = 3 means match found, but to one side, no replacement made
!   Error = 4 means match found, but to one side, replacement made


			! check to see if r_d_2000s match up -- if not, look on appropriate side

			error = 6
			
			if (L1B_orbit_data%scan_valid(scan_num) .gt. 0) then
				delta_time = (L1B_orbit_data%scan_data(scan_num)%date%r_d - scan_to_add%date%r_d)/MSU_SCAN_TIME
			else
				delta_time = -999.9
			endif

			scan_match = -1 ! assume no match  if still -1 then no match at location or at either side

			if (abs(delta_time) .gt. 0.2) then  ! look one scan earlier
				if (scan_num .gt. 1) then
					delta_time_earlier = (L1B_orbit_data%scan_data(scan_num-1)%date%r_d - scan_to_add%date%r_d)/MSU_SCAN_TIME
				else
					delta_time_earlier = -999.9
				endif

				if (abs(delta_time_earlier) .lt. 0.2) then  ! we have a match with this scan
					scan_match = scan_num-1
					error = 3
				endif
					
				! look one scan later

				delta_time_later = (L1B_orbit_data%scan_data(scan_num+1)%date%r_d - scan_to_add%date%r_d)/MSU_SCAN_TIME
  				if (abs(delta_time_later) .lt. 0.2) then  ! we have a match with this scan
					scan_match = scan_num+1
					error = 3
				endif
			else
				scan_match = scan_num   ! times match closely enough
				error = 1
			endif

			if (scan_match .eq. -1) then
				if (L1B_orbit_data%scan_valid(scan_num) .eq. 0) then  ! if the slot is empty, and
																	  ! there was not match on either side
																	  ! then we place it

 					scan_to_Add%scan_number = scan_num
					L1B_orbit_data%scan_data(scan_num) = scan_to_add
					L1B_orbit_data%scan_valid(scan_num) = 2
					error = 2
				else
					!no match, and no empty spot)
					error = 5
				endif
			else
				! try and figure out if new scan is better than the one we found
				tot_fov_qual_new = 0
				tot_fov_qual_old = 0

				! add up fov quality flags -- we'll overwrite if the new scan has fewer bad spots

				do fov = 1,11
					if(iand(scan_to_add%fov_quality(fov),4093) .gt. 0) tot_fov_qual_new = tot_fov_qual_new +1
					if(iand(L1B_orbit_data%scan_data(scan_match)%fov_quality(fov),4093) .gt. 0) tot_fov_qual_old = tot_fov_qual_old +1
				enddo
				if (tot_fov_qual_new .lt. tot_fov_qual_old) then
					print *,tot_fov_qual_new,tot_fov_qual_old
					scan_to_Add%scan_number = scan_match  ! set the scan num inthe scan!
					L1B_orbit_data%scan_data(scan_match) = scan_to_add
					L1B_orbit_data%scan_valid(scan_match) = 2
					if (scan_match .eq. scan_num) then
						error = 4
					else
						error = 4
					endif
				endif
					
			endif
		end subroutine add_scan_to_orbit
												 
	end module msu_l1b_ingest