	program update_driver
	
!@***p* MLT_RSS/Process_Monthly_Driver_V3_0.f90
!  NAME
!    update_driver
! 
!  PURPOSE
!    top level program to perform monthly averages for TMT, TTS, and TLS
! 
!  DESCRIPTION
!    top level program to perform monthly averages
!    
!  INPUTS
!    (passed as command line parameters from windows)
!    sat_num     satellite number to process.  Valid range 1..9
!    channel     MSU channel number to process.  Valid range 2..4
!    year        year to process.  Valid range depends on satellite and channel choosen
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE 
!    (at command line)
!    Process_Monthly_Driver_V3_0 7 2 1994
! 
!@*****


	use date_and_time
	use NAN_support
	use msu_constants
	use read_msu_swath_L1A 
	use update_monthly_msu_level2c_v3_0
	use msu_date
	use zonal
	use topography
	use portlib
	use write_heights
	use DFLIB
	use update_monthly_msu_level2c


	integer(4)				:: sat_num
	integer(4)				:: channel
	integer(4)				:: error
	integer(2)				:: error2
	character(len=20)		:: buffer


	integer(4)			:: month,day,year
	type(date_Str)		:: first_date_to_process
	type(date_Str)		:: last_date_to_process

	call init_nan

	buffer = ''
	call getarg(1,buffer,error2)
	read(buffer,111)sat_num
	111 format(i3)
	if (error .ne. 0) stop
	
	buffer = ''
	call getarg(2,buffer,error2)
	read(buffer,111)channel
	if (error .ne. 0) stop
	
	buffer = ''
	call getarg(3,buffer,error2)
	read(buffer,114)year
	114 format(i4)
	if (error .ne. 0) stop


	first_date_to_process = set_date(1,1,year,error)
	last_date_to_process = set_date(12,31,year,error)
	if (error .ne. 0) stop

	print *,'Doing Monthly Gridpoint Update for year = ',year
	print *,'Sat_num = ',sat_num
	print *,'Channel = ',channel
	print *,'call update_monthly_msulevel2c_V3_0(sat_num,channel,first_date_to_process,last_date_to_process,error)'
	call update_monthly_msulevel2c_V3_0(sat_num,channel,first_date_to_process,last_date_to_process,error)
	
end program	update_driver 