$freeform
!@***h* MLT_RSS/pdate_monthly_msu_level2c_v3_2.f90
!  NAME
!    module update_monthly_msu_level2c_v3_2
! 
!  PURPOSE
!    module that contains update_monthly_msu_level2c_v3_2
! 
!  DESCRIPTION
!    module that contains update_monthly_msu_level2c_v3_2
!    
!  ROUTINES
!   update_monthly_msu_level2c_v3_2
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use update_monthly_msu_level2c_v3_2
! 
!@*****
module update_monthly_msu_level2c_v3_2

use msu_constants
use msu_date
use date_and_time
use NAN_support
use portlib
use MSU_Level_2A
use MSU_L2A_inventory

implicit none

integer(4),parameter	:: GRPT_TB_REC_LENGTH     = 4 + (7*2*2*144*72*4)
integer(4),parameter	:: GRPT_TARGET_REC_LENGTH = 4 + (2*2*2*144*72*4)

character(len = 80)		:: monthly_map_path			= '\\Nasserver5\MSU_AMSU_Data\AMSU-A_L2C\Monthly_Maps\'
character(len = 80)		:: monthly_zonal_path		= '\\Nasserver5\MSU_AMSU_Data\AMSU-A_L2C\zonal_months\'


contains


subroutine update_monthly_msulevel2c_V3_2_TLT(sat_num,channel,first_date_to_process,last_date_to_process,error)
! @***h* MLT_RSS/update_monthly_msulevel2c_V3_2_TLT.f90
!  NAME
!    update_monthly_msulevel2c_V3_2_TLT
! 
!  PURPOSE
!    creates monthly 2.5 by 2.5 degree gridded maps from MSU L2A data.
! 
!  DESCRIPTION
!    creates monthly 2.5 by 2.5 degree gridded maps from MSU L2A data.
!    reads L2A data from the range of dates specified, and writes data to L2C file
!    summary of results is written to a log file
!    
!  INPUTS
!    satnum     integer(4)  satellite number to process (1 (TIROS-N) to 9 (NOAA-14)
!    channel    integer(4)  MSU channel number to process
!    first_date_to_process       type(date_str)  first date to process
!    last_date_to_process        type(date_str)  last date to process
! 
!  OUTPUTS
!    integer(4)                     :: error
!  
!  INPUT FILES
!    MSU L2A  orbit files        e.g. NOAA-06_r00040.L1A.gz
!    MSU L2A  correction files   e.g. NOAA-06_r00040_Ch2.dnl,NOAA-06_r00040_Ch2.inc,NOAA-06_r00040_Ch2.lmb
!    MSU L2A  inventory file     e.g. NOAA-06_L1A.inv.txt
!
!  OUTPUT FILES
!    MSU L2C monthly map files   e.g. MSU_02_Chan_3_Monthly_Hist_144_72_N5_V3_0.dat
!    log file                    e.g. log_ch2_sat_06_2006_05_05_20_56_32.txt
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    update_monthly_msulevel2c_V3_2_TLT(sat_num,channel,first_date_to_process,last_date_to_process,error)
! 
! @*****

	integer(4)				:: sat_num
	integer(4)				:: channel
	type(date_str)			:: first_date_to_process
	type(date_str)			:: last_date_to_process
	integer(4)				:: error

	logical,dimension(NUM_FOVS)	:: fov_ok
	real(4),dimension(NUM_CHANNELS,NUM_FOVS)	:: Tbs			


	! declaration of input data structures

	type(msu_L2A_orbit_header)							:: L2A_header
	type(msu_L2A_orbit_data)							:: L2A_scan
	type(msu_inventory_str)								:: L2A_inventory

	type(date_str)				 :: swath_date
	type(date_str)				 :: strt_date

	integer(4)					 :: orbit_num
	character(len = 8)           :: sat_name
	integer(4)					 :: n_swath
	integer(4)					 :: orbit_error
	integer(4)				     :: diur_error
	integer(4)				     :: inc_error
	integer(4)				     :: limb_error

	integer(4)					 :: surf
	real(4)						 :: alt
	real(4)						 :: remainder

	real(4),dimension(2)		 :: target_temp
	real(4),dimension(NUM_FOVS)  :: longitude
	real(4),dimension(NUM_FOVS)  :: latitude
	real(4),dimension(NUM_FOVS)	 :: diurnal_correction
	real(4),dimension(NUM_FOVS)	 :: incidence_correction
	real(4),dimension(NUM_FOVS)	 :: limb_correction
	real(4),dimension(NUM_FOVS)  :: deriv_2
	real(4),parameter            :: precip_thres = 1.5

	type(Date_Str)				 :: curr_Date


	real(4),dimension(7,2,2,144,72)	    :: hist	 ! (num/tot_tb/tot_diur/tot_inc/tot_limb/tot_tar/tot_samp,asc/desc,left/right,ilon,ilat)

	integer(4)			:: ilon
	integer(4)			:: ilat

	! Set logical unit numbers for the various files

	integer(4),parameter			:: lu_Tbs_grpt		= 50	! Tbs
	integer(4),parameter			:: lu_target_1_grpt	= 51	! target_1_temp
	integer(4),parameter			:: lu_target_2_grpt	= 52	! target_2_temp
	integer(4),parameter			:: lu_Sampling_grpt = 53

	integer(4),parameter			:: lu_log			= 70	! log 
	integer(4),parameter			:: lu_sample_list   = 71    ! list of samples
	integer(4),parameter			:: lu_inventory     = 100	! orbit inventory file

	! various local variables

	integer(4)			:: i
	integer(4)			:: fov
	integer(4)			:: asc_desc
	integer(4)			:: asc_desc2
	real(4)				:: delta_lat1
	real(4)				:: delta_lat2
	logical				:: data_ok
	logical(4)			:: scan_ok

	integer(4)			:: zone
	integer(4)			:: land_sea
	integer(4)			:: month_num
	integer(4)          :: month_num_last
	integer(4)			:: grpt_rec_num
	real(4)				:: tot_tb 
	real(4)				:: tot_diur 
	real(4)				:: tot_incidence 
	real(4)				:: tot_limb
	real(4)				:: mn_tb 
	real(4)				:: mn_diur 
	real(4)				:: mn_incidence 
	real(4)				:: mn_limb

	real(4)				:: tot_wt 

	! input_file names

	character(len=100)	:: inventory_file_name

	! log file name

	character(len=100)  :: log_file_name
	character(20) :: ds


	! output file names

	character(len=140)  :: tb_grpt_name

	! declare variables related to inventory file

	type(msu_inventory_str) :: inv

	integer(4)						:: max_recs
	integer(4),dimension(:),pointer	:: orbit_nums
	real(8),dimension(:),pointer	:: start_rd_2000
	real(8),dimension(:),pointer    :: end_rd_2000

	integer(4)						:: start_month
	integer(4)						:: strt_orbit
	integer(4)						:: last_month
	integer(4)						:: last_orbit
	integer(4)						:: rec_month
	type(date_str)					:: rec_date
	integer(4)						:: inv_rec_num
	integer(4)						:: last_valid_rec_num
	logical							:: start_found
	logical							:: last_found
	integer(4)						:: num_ok = 0


	integer(4),parameter			:: NUM_WT = 2
	integer(4)						:: wt_num
	real(4),dimension(11,NUM_WT)	:: weights = RESHAPE((/-1.5,-1.5,2.0,2.0,0.0,0.0,0.0,0.0,0.0, 0.0, 0.0,  &
	                                                               0.0, 0.0,0.0,0.0,0.0,0.0,0.0,2.0,2.0,-1.5,-1.5/), &
												         (/11,2/))
	integer(4),dimension(11,NUM_WT)	:: onto_map = RESHAPE((/  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  &
	                                                                      0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1/), &
															              (/11,2/))


	logical				:: in_cell
	integer(4)			:: ilon_Cell = 10
	integer(4)			:: ilat_cell = 7
	character(len=120)  :: sample_list_file_name

	!----------------------------------------------




		!call init_NAN

		! open log file
		ds = curr_date_string(error)
		write(log_file_name,110) channel,sat_num,trim(ds)
110     format('\\dell-p866a\e\documents\msu\logs\log_ch',i1,'_sat_',i2.2,'_',a,'.txt')
		open(unit = lu_log,file = log_file_name)

		write(sample_list_file_name,115) sat_num,ilon_cell,ilat_cell
115     format('\\dell-p866a\e\documents\msu\logs\sample_list_sat_',i2.2,'_',i3.3,'_',i3.3,'.txt')
		open(unit = lu_sample_list,file = sample_list_file_name)

		error = 0

		sat_name = sat_names_2(sat_num)


	    ! read in inventory file

		call read_msu_L2A_inventory_file(sat_num,'Mo1995_V02_1',inv,error)
		max_recs = inv%max_orbit

		! determine the first orbit and month to use
		! we use the first month that contains the start date, and the last pentad that contains the
		! end date.  Thus the region of computation is expanded somewhat.

		start_month = calc_month(first_date_to_process,remainder)
		start_found = .false.
		last_valid_rec_num = -1
		do inv_rec_num = 1,max_recs
			if (inv%orbit_valid(inv_rec_num) .and. finite(inv%start_rd_2000(inv_rec_num))) then
				rec_date = set_date_2000(inv%start_rd_2000(inv_rec_num),error)
				rec_month = calc_month(rec_date,remainder)
				
				if (rec_month .ge. start_month) then
					start_found = .true.
					if (last_valid_rec_num .gt. 0) then
						strt_orbit = last_valid_rec_num
					else
						strt_orbit = inv_rec_num
					endif
					strt_date = set_date_2000(inv%start_rd_2000(strt_orbit),error)
					exit
				endif
				last_valid_rec_num = inv_rec_num
			endif
		enddo
			 
		if (.not.(start_found)) then 
			print *, 'start orbit not found for sat num ',sat_num
			return
		endif

		! determine the last orbit to use

		last_month= calc_month(last_date_to_process,remainder)
		last_found = .false.
		last_valid_rec_num = -1
		do inv_rec_num = 1,max_recs

			if (inv%orbit_valid(inv_rec_num) .and. finite(inv%start_rd_2000(inv_rec_num))) then
				!print *,inv_rec_num,inv%start_rd_2000(inv_rec_num)
 				rec_date = set_date_2000(inv%start_rd_2000(inv_rec_num),error)
				rec_month = calc_month(rec_date,remainder)
				!print *,inv_rec_num,rec_month
				if (rec_month .gt. last_month) then
					last_found = .true.
					last_orbit = inv_rec_num+2
					if (last_valid_rec_num .gt. 0) then
						last_month = calc_month(set_date_2000(inv%start_rd_2000(last_valid_rec_num),error),remainder)
					else
						last_month  = rec_month
					endif
					exit
				endif
				last_valid_rec_num = inv_rec_num
			endif
		enddo

		if (.not.(last_found)) then 
			last_orbit = max_recs
			last_month = calc_month(set_date_2000(inv%start_rd_2000(max_recs),error),remainder)
		endif

		print *,strt_orbit,start_month,' ',date_string(strt_Date)
		if (finite(inv%start_rd_2000(last_orbit))) then
			print *,last_orbit,last_month,' ',date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))
		else 
			print *,last_orbit,last_month
		endif

		! open all the output files

			write(lu_log,3351)
	3351	format(' Computing Tb GrPt Maps')
			write(tb_grpt_name,351)trim(monthly_map_path),sat_num
    351		format(a,'MSU_Channel_TLT\MSU_',i2.2,'_Chan_TLT_Monthly_Hist_144_72_TLT_V3_2.dat')
			open(unit = lu_tbs_grpt,file = tb_grpt_name,form='binary',access = 'direct',recl = GRPT_TB_REC_LENGTH)


			! Initialize a few variables before beginning to read

			i = 1
			error = 0
			hist = 0.0	

			do orbit_num = strt_orbit,last_orbit
                if (orbit_num .eq. 100*(orbit_num/100)) then
					write(6,222)sat_num,month_num,orbit_num,num_ok
222					format('TLT, Sat:',i2.2,' Month: ',i3.3,' Orbit: ',i6,' OK ',i6)
					num_ok = 0
				endif
							
				diur_error = 0
				orbit_error = 0

				call zero_L2A_orbit(L2A_scan)

			! this sets corrections to zero -- they are set to NAN if a read of them is attempted
			! so that if they are not wanted, they pass quality control, but if they are wanted, but 
			! not there, they fail QC


				call read_L2A_orbit_file(sat_names_2(sat_num),	&
										orbit_num,			&
										'Mo1995_V02_1',		&
										L2A_header,			&
										L2A_scan,			&
										orbit_error)

				call  read_L2A_Corr_file(	sat_names_2(sat_num),			&
									orbit_num,			&
									L2A_scan,				&
									DIURNAL,				&
									diur_error)

				call  read_L2A_Corr_file(	sat_names_2(sat_num),			&
									orbit_num,			&
									L2A_scan,				&
									INCIDENCE,				&
									inc_error)

				call  read_L2A_Corr_file(	sat_names_2(sat_num),			&
									orbit_num,			&
									L2A_scan,				&
									LIMB,					&
									limb_error)

				if ((orbit_error .eq. 0)) then 
					num_ok = num_ok + 1
					do n_swath = L2A_header%start_swath_num,L2A_header%end_swath_num

							if (L2A_scan%scan_valid(n_swath) .le. 0) cycle  ! skip rest if scan not valid

							call eval_swath_qual(	L2A_scan%scan_Data(n_swath)%scan_qual,		&
													L2A_scan%scan_Data(n_swath)%fov_qual,		&
													fov_ok)

							if (btest(L2A_scan%scan_Data(n_swath)%scan_qual,8)) then
								asc_desc = ASCENDING
							else
								asc_desc = DESCENDING
							endif

							! check asc_desc to see it makes sense, if possible

							delta_lat1 = NAN
							delta_lat2 = NAN
							if (n_swath .gt. 1) then
								delta_lat1 = L2a_scan%scan_data(n_swath)%fov_latitude(6) - &
											 L2a_scan%scan_data(n_swath-1)%fov_latitude(6)
							endif
							if (n_swath .lt. L2A_header%end_swath_num) then
								delta_lat2 = L2a_scan%scan_data(n_swath+1)%fov_latitude(6) - &
											 L2a_scan%scan_data(n_swath)%fov_latitude(6)
							endif

							if (.not. finite(delta_lat1)) delta_lat1 = 0.0
							if (.not. finite(delta_lat2)) delta_lat2 = 0.0

							asc_desc2 = asc_desc
							if ((delta_lat1 + delta_lat2) .gt. 0.0) then
								asc_Desc2 = ASCENDING
							endif

							if ((delta_lat1 + delta_lat2) .lt. 0.0) then
								asc_Desc2 = DESCENDING
							endif

							if (asc_desc .ne. asc_desc2) then
							!	print *,orbit_num,n_swath,'asc_desc problem'
								asc_desc = asc_desc2
							endif

							! set date
							 
							swath_date = L2A_scan%scan_Data(n_swath)%date
						
							if (.not. finite(swath_Date%rd_2000)) cycle	 ! skip rest if bad time
                            
							month_num = calc_month(swath_date,remainder)
							if (month_num .lt. start_month) cycle
							!if (month_num .gt. last_month) cycle

				
							if (i .gt. 2) then
								if ((month_num .ne. month_num_last)) then !write out record for this pentad
								!	call write_error_report(lu_log,2)
									call zero_error_report(2)

									curr_date = current_date(error)

									write(6,987) channel,sat_name,orbit_num,swath_date%month,swath_date%day,  &
										 swath_date%year,curr_date%time%hour,curr_date%time%minute,&
										 floor(curr_date%time%sec),orbit_num,month_num_last

									write(lu_log,987) channel,sat_name,orbit_num,swath_date%month,swath_date%day,  &
										 swath_date%year,curr_date%time%hour,curr_date%time%minute,&
										 floor(curr_date%time%sec),orbit_num,month_num_last
									call flush(lu_log)

			987						format(' Processing Channel ',i1,1x,a,1x,i7,2x,i2.2,'/',i2.2,'/',i4.4,'     ',i2.2,':',i2.2,':',i2.2,2x,i6,2x,i6)
							

									grpt_rec_num = month_num_last
									print *,'writing daily record = ',grpt_rec_num
									write(lu_Tbs_grpt,rec = grpt_rec_num) month_num_last,hist
									hist          = 0.0
								endif

							endif ! i gt 2
						

							month_num_last = month_num
							i = i+1
							scan_ok = .true.

							Tbs						= L2A_scan%scan_data(n_swath)%MSU_TBS_RAW
							longitude				= L2A_scan%scan_data(n_swath)%fov_longitude
							latitude				= L2A_scan%scan_data(n_swath)%fov_latitude
							diurnal_correction		= L2A_scan%scan_data(n_swath)%MSU_TBS_DIURNAL_CORR(channel,:) 
							incidence_correction	= L2A_scan%scan_data(n_swath)%MSU_TBS_NOM_EIA_CORR(channel,:) 
							limb_correction			= L2A_scan%scan_data(n_swath)%MSU_TBS_NADIR_CORR(channel,:) 
							target_temp(1)			= L2A_scan%scan_data(n_swath)%T%Target1_temp
							target_temp(2)			= L2A_scan%scan_data(n_swath)%T%Target2_temp

							if (scan_ok) then

								do wt_num = 1,NUM_WT

									tot_tb = 0.0
									tot_diur = 0.0
									tot_incidence = 0.0
									tot_limb = 0.0
									tot_wt = 0.0

									in_cell = .false.

									do fov = 1,11
										if (abs(weights(fov,wt_num)) .lt. 0.001) cycle
										if ((finite(Tbs(channel,fov)))          .and.  &
											(fov_ok(fov))                       .and.  &
											(finite(diurnal_correction(fov)))   .and.  &
											(finite(incidence_correction(fov)))	.and.  &
											(finite(limb_correction(fov)))      .and.  &
											(finite(longitude(fov)))            .and.  &
											(finite(latitude(fov)))) then !
											
											tot_tb =		tot_tb + weights(fov,wt_num)*tbs(channel,fov)
											tot_diur =		tot_diur + weights(fov,wt_num)*diurnal_correction(fov)
											tot_incidence = tot_incidence + weights(fov,wt_num)*incidence_correction(fov)
											tot_limb =		tot_limb + weights(fov,wt_num)*limb_correction(fov)
											tot_wt =		tot_wt + weights(fov,wt_num)

											ilon = floor(longitude(fov)/2.5) + 1
											ilat = floor((latitude(fov)+90.0)/2.5) + 1
											
											if ((ilon .eq. ilon_cell) .and. (ilat .eq. ilat_cell)) in_cell = .true.		
										endif
									enddo	! fov

									if (in_cell) then
										
										do fov = 1,11
											if (abs(weights(fov,wt_num)) .lt. 0.001) cycle
											write(lu_sample_list,444) longitude(fov),latitude(fov),fov,asc_desc,n_swath,orbit_num,swath_Date%rd_2000
	444										format(f8.3,1x,f8.3,1x,i2,1x,i2,1x,i4,1x,i6,1x,f12.5)
										enddo	! fov
									endif
									if (tot_wt .gt. 0.0) then
										mn_tb			= tot_tb/tot_wt
										mn_diur			= tot_diur/tot_wt
										mn_incidence	= tot_incidence/tot_wt
										mn_limb			= tot_limb/tot_wt
                                        
										do fov = 1,11
											if ((onto_map(fov,wt_num) .gt. 0)     .and.  &
												(finite(longitude(fov)))          .and.  &
												(finite(latitude(fov)))) then ! 
												ilon = floor(longitude(fov)/2.5) + 1
												ilat = floor((latitude(fov)+90.0)/2.5) + 1	

												hist(1,asc_desc,wt_num,ilon,ilat) =  hist(1,asc_desc,wt_num,ilon,ilat) + 1
												hist(2,asc_desc,wt_num,ilon,ilat) =  hist(2,asc_desc,wt_num,ilon,ilat) + mn_tb
												hist(3,asc_desc,wt_num,ilon,ilat) =  hist(3,asc_desc,wt_num,ilon,ilat) + mn_diur
												hist(4,asc_desc,wt_num,ilon,ilat) =  hist(4,asc_desc,wt_num,ilon,ilat) + mn_incidence
												hist(5,asc_desc,wt_num,ilon,ilat) =  hist(5,asc_desc,wt_num,ilon,ilat) + mn_limb
												hist(6,asc_desc,wt_num,ilon,ilat) =  hist(6,asc_desc,wt_num,ilon,ilat) + target_temp(1+(channel-1)/2)
												hist(7,asc_desc,wt_num,ilon,ilat) =  hist(7,asc_desc,wt_num,ilon,ilat) + swath_date%day
											endif
										enddo
									endif
								enddo ! wt_num
							endif   ! scan_ok
						enddo		! n_swath
					endif			! orbit error & diurnal error
				enddo				! orbit num
		
		close(lu_tbs_grpt)
	    close(lu_log)
end subroutine update_monthly_msulevel2c_V3_2_TLT

end module update_monthly_msu_level2c_V3_2
