$freeform

! @***m* MLT_RSS/AMSU_types.f90
!  NAME
!    Module AMSU_types
! 
!  PURPOSE
!    Module containing structured types for AMSU processing
! 
!  DESCRIPTION
!    Module containing structured types for AMSU processing
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    8/15/2012 Initial Version prepared for NCDC
! 
!  USAGE
!    use AMSU_Types
! @*****
module AMSU_types

use AMSU_constants

integer(4),parameter :: AMSU_LEVEL_1_RECORD_LENGTH = 2560

type AMSU_A_L1A_File_Header

	sequence

	! general information

	character(len = 3)			::	data_set_creation_site_ID
	character(len = 1)			::  fill_1
	integer(2)					::	version_number
	integer(2)					::	version_year
	integer(2)					::	version_day_of_year
	integer(2)					::	logical_record_length
	integer(2)					::	block_size
	integer(2)					::	count_of_header_records
	integer(2)					::	fill_2(3)
	character(len=42)			::	data_set_name
	character(len=8)			::	processing_block_identification
	integer(2)					::  NOAA_spacecraft_identification_code
	integer(2)					::  instrument_ID
	integer(2)					::  data_type_code
	integer(2)					::  TIP_source_code
	integer(4)					::  start_of_data_set_day_count
	integer(2)					::  start_of_data_set_year
	integer(2)					::  start_of_data_set_day_of_year
	integer(4)					::  start_of_data_set_UTC_time_of_day
	integer(4)					::  end_of_data_set_day_count
	integer(2)					::  end_of_data_set_year
	integer(2)					::  end_of_data_set_day_of_year
	integer(4)					::  end_of_data_set_UTC_time_of_day
	integer(2)					::  year_of_last_CPIDS_update
	integer(2)					::  day_of_year_of_last_CPIDS_update
	integer(2)					::  fill_3(6)

	! data set quality indicators

	integer(4)					::  instrument_status_A2
	integer(2)					::  fill_4
	integer(2)					::  record_number_of_status_change_of_A2
	integer(4)					::  second_instrument_status_A2
	integer(4)					::  instrument_status_A1
	integer(2)					::  fill_5
	integer(2)					::  record_number_of_status_change_of_A1
	integer(4)					::  second_instrument_status_A1
	integer(2)					::  count_of_data_records
	integer(2)					::  count_of_scan_lines
	integer(2)					::  count_of_missing_scan_lines
	integer(2)					::  count_of_data_gaps
	integer(2)					::  count_of_AMSU_frames
	integer(2)					::  count_of_PACS_detected_TIP_errors
	integer(2)					::  sum_of_all_auxiliary_sync_errors
	integer(2)					::  time_sequence_error
	integer(2)					::  time_sequence_error_code
	integer(2)					::  SOCC_clock_update_indicator
	integer(2)					::  earth_location_error_indicator
	integer(2)					::  earth_location_error_code
	integer(2)					::  PACS_status_bit_field
	integer(2)					::  PACS_data_source
	integer(4)					::  fill_6
	character(len=8)			::	reserved_for_ingester
	character(len=8)			::	reserved_for_docommutation
	integer(4)					::  fill_7(4)

	! calibration
	integer(2)					::	fill_8
	integer(2)					::	instrument_temperature_sensor_ID(3)
	integer(2)					::	RF_shelf_A1_1_reference_temperature_1(3)		! SF 2 minimum nominal maximum
	integer(2)					::	RF_shelf_A1_2_reference_temperature(3)			! SF 2 minimum nominal maximum
	integer(2)					::	RF_shelf_A2_reference_temperature(3)			! SF 2 minimum nominal maximum
	integer(2)					::	RF_shelf_A1_1_reference_temperature_2(3)		! SF 2 minimum nominal maximum
	integer(2)					::	RF_Mux_A1_1_reference_temperature_1(3)			! SF 2 minimum nominal maximum
	integer(2)					::	RF_Mux_A1_2_reference_temperature(3)			! SF 2 minimum nominal maximum
	integer(2)					::	RF_Mux_A2_reference_temperature(3)				! SF 2 minimum nominal maximum
	integer(2)					::	RF_Mux_A1_1_reference_temperature_2(3)			! SF 2 minimum nominal maximum
	integer(2)					::	fixed_bias_corr(15,4)							! SF 3 Ch 1-15 	
																					! warm target at minimum
																					! warm target at nominal
																					! warm target at maximum
																					! cold space
	integer(2)					::	warm_target_bias_corr(6,3)						! SF 3 Ch 9-14 minimum nominal maximum
	integer(4)					::	nonlinearity_coef(15,3)							! SF 6 Ch 1-15 ref 1 2 3
	integer(4)					::	nonlinearity_coef_2(6,3)						! SF 6 Ch 9-14 ref 1 2 3
	integer(4)					::	fill_9(4)										! changed : cm

	! temperature to radiance conversion		
	integer(4)					::	temperature_radiance(15,3)					! SF 6 Ch 1-15 central wavenumber constant 1 constant 2 slope
	integer(4)					::	fill_10(3)

	! navigation
	character(len = 8)			::	reference_ellipsoid_model_ID
	integer(2)					::	nadir_earth_location_tolerance					! SF 1 in km
	integer(2)					::	earth_location_bit_field
	integer(2)					::	fill_11
	integer(2)					::	constant_roll_attitude_error					! SF 3 in deg
	integer(2)					::	constant_pitch_attitude_error					! SF 3 in deg
	integer(2)					::	constant_yaw_attitude_error						! SF 3 in deg
	integer(2)					::	epoch_year_for_orbit_vector
	integer(2)					::	day_of_epoch_year_for_orbit_vector
	integer(4)					::	epoch_UTC_time_of_day_for_orbit_vector			! in msec
	integer(4)					::	semi_major_axis									! SF 5 in km
	integer(4)					::	eccentricity									! SF 8
	integer(4)					::	inclination										! SF 5 in deg
	integer(4)					::	argument_of_perigee								! SF 5 in deg
	integer(4)					::	right_ascension_of_the_ascending_node			! SF 5 in deg
	integer(4)					::	mean_anomaly									! SF 5 in deg
	integer(4)					::	position(3)										! SF 5 in km
	integer(4)					::	velocity(3)										! SF 8 in km/sec
	integer(4)					::	earth_sun_distance_ratio						! SF 6
	integer(4)					::	fill_12(4)

	! A1 D/A conversion
	integer(4)					::	A1_1_scan_motor_temperature_coefficient(4)					! SF 4/9/16/20  Offset 0
	integer(4)					::	A1_2_scan_motor_temperature_coefficient(4)					! SF 4/9/16/20	Offset 16
	integer(4)					::	A1_1_feed_horn_temperature_coefficient(4)					! SF 4/9/16/20	Offset 32
	integer(4)					::	A1_2_feed_horn_temperature_coefficient(4)					! SF 4/9/16/20	Offset 48
	integer(4)					::	A1_1_RF_Mux_temperature_coefficient(4)						! SF 4/9/16/20	Offset 64
	integer(4)					::	A1_2_RF_Mux_temperature_coefficient(4)						! SF 4/9/16/20	Offset 80
	integer(4)					::	A1_local_oscillator_ch_3_temperature_coefficient(4)			! SF 4/9/16/20	Offset 96
	integer(4)					::	A1_local_oscillator_ch_4_temperature_coefficient(4)			! SF 4/9/16/20	Offset 112
	integer(4)					::	A1_local_oscillator_ch_5_temperature_coefficient(4)			! SF 4/9/16/20	Offset 128
	integer(4)					::	A1_local_oscillator_ch_6_temperature_coefficient(4)			! SF 4/9/16/20	Offset 144
	integer(4)					::	A1_local_oscillator_ch_7_temperature_coefficient(4)			! SF 4/9/16/20	Offset 160
	integer(4)					::	A1_local_oscillator_ch_8_temperature_coefficient(4)			! SF 4/9/16/20	Offset 176
	integer(4)					::	A1_local_oscillator_ch_15_temperature_coefficient(4)		! SF 4/9/16/20	Offset 192
	integer(4)					::	A1_PLLO_2_temperature_coefficient(4)						! SF 4/9/16/20	Offset 208
	integer(4)					::	A1_PLLO_1_temperature_coefficient(4)						! SF 4/9/16/20	Offset 224
	integer(4)					::	A1_PLLO_reference_oscillator_temperature_coefficient(4)	! SF 4/9/16/20		Offset 240
	integer(4)					::	A1_mixer_IF_amplifier_ch_3_temperature_coefficient(4)		! SF 4/9/16/20	Offset 256
	integer(4)					::	A1_mixer_IF_amplifier_ch_4_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_mixer_IF_amplifier_ch_5_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_mixer_IF_amplifier_ch_6_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_mixer_IF_amplifier_ch_7_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_mixer_IF_amplifier_ch_8_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_mixer_IF_amplifier_ch_9_14_temperature_coefficient(4)	! SF 4/9/16/20
	integer(4)					::	A1_mixer_IF_amplifier_ch_15_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_11_14_temperature_coefficient(4)	! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_9_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_10_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_11_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_DC_converter_temperature_coefficient(4)					! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_13_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_14_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_IF_amplifier_channel_12_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_1_RF_shelf_temperature_coefficient(4)					! SF 4/9/16/20
	integer(4)					::	A1_2_RF_shelf_temperature_coefficient(4)					! SF 4/9/16/20
	integer(4)					::	A1_detector_preamp_assembly_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A1_1_warm_load_1_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_1_warm_load_2_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_1_warm_load_3_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_1_warm_load_4_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_1_warm_load_center_temperature_coefficient(4)			! SF 4/9/16/20
	integer(4)					::	A1_2_warm_load_1_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_2_warm_load_2_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_2_warm_load_3_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_2_warm_load_4_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A1_2_warm_load_center_temperature_coefficient(4)			! SF 4/9/16/20
	integer(4)					::	fill_13
	! A1 analog telemetry conversion
	integer(4)					::	A1_1_scan_motor_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A1_2_scan_motor_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A1_1_RF_shelf_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A1_2_RF_shelf_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A1_1_warm_load_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A1_2_warm_load_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A1_1_antenna_motor_current(2)			! SF 3 slope/intercept
	integer(4)					::	A1_2_antenna_motor_current(2)			! SF 3 slope/intercept
	integer(4)					::	A1_plus_15v_signal_processing(2)		! SF 3 slope/intercept
	integer(4)					::	A1_plus_15v_antenna_drive(2)			! SF 3 slope/intercept
	integer(4)					::	A1_minus_15v_signal_processing(2)		! SF 3 slope/intercept
	integer(4)					::	A1_minus_15v_antenna_drive(2)			! SF 3 slope/intercept
	integer(4)					::	A1_plus_8v_receiver_amps(2)			! SF 3 slope/intercept
	integer(4)					::	A1_plus_5v_signal_processing(2)		! SF 3 slope/intercept
	integer(4)					::	A1_plus_5v_antenna_drive(2)			! SF 3 slope/intercept
	integer(4)					::	A1_plus_8_5v_phase_lock(2)				! SF 3 slope/intercept
	integer(4)					::	A1_plus_15v_phase_lock(2)				! SF 3 slope/intercept
	integer(4)					::	A1_minus_15v_phase_lock(2)				! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_3(2)					! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_4(2)					! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_5(2)					! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_6(2)					! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_7(2)					! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_8(2)					! SF 3 slope/intercept
	integer(4)					::	A1_PLLO_primary_lock_detect(2)			! SF 3 slope/intercept
	integer(4)					::	A1_PLLO_redundant_lock_detect(2)		! SF 3 slope/intercept
	integer(4)					::	A1_GDO_voltage_ch_15(2)				! SF 3 slope/intercept
	integer(4)					::	fill_14

	! A2 D/A conversion

	integer(4)					::	A2_scan_motor_temperature_coefficient(4)					! SF 4/9/16/20
	integer(4)					::	A2_feed_horn_temperature_coefficient(4)					! SF 4/9/16/20
	integer(4)					::	A2_RF_Mux_temperature_coefficient(4)						! SF 4/9/16/20
	integer(4)					::	A2_Mixer_IF_ch_1_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A2_Mixer_IF_ch_2_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A2_local_oscillator_ch_1_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A2_local_oscillator_ch_2_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A2_compensation_motor_temperature_coefficient(4)			! SF 4/9/16/20
	integer(4)					::	A2_subreflector_temperature_coefficient(4)					! SF 4/9/16/20
	integer(4)					::	A2_DC_DC_converter_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A2_RF_shelf_temperature_coefficient(4)						! SF 4/9/16/20
	integer(4)					::	A2_detector_preamp_assembly_temperature_coefficient(4)		! SF 4/9/16/20
	integer(4)					::	A2_warm_load_center_temperature_coefficient(4)				! SF 4/9/16/20
	integer(4)					::	A2_warm_load_temperature_coefficient(6,4)					! SF 4/9/16/20
	integer(4)					::	fill_15

	! A2 analog telemetry conversion
	integer(4)					::	A2_scan_motor_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A2_compensator_motor_temperature(2)	! SF 3 slope/intercept
	integer(4)					::	A2_RF_shelf_temperature(2)				! SF 3 slope/intercept
	integer(4)					::	A2_warm_load_temperature(2)			! SF 3 slope/intercept
	integer(4)					::	A2_compensator_motor_current(2)		! SF 3 slope/intercept
	integer(4)					::	A2_antenna_motor_current(2)			! SF 3 slope/intercept
	integer(4)					::	A2_plus_15v_signal_processing(2)		! SF 3 slope/intercept
	integer(4)					::	A2_plus_15v_antenna_drive(2)			! SF 3 slope/intercept
	integer(4)					::	A2_minus_15v_signal_processing(2)		! SF 3 slope/intercept
	integer(4)					::	A2_minus_15v_antenna_drive(2)			! SF 3 slope/intercept
	integer(4)					::	A2_plus_8v_receiver_amps(2)			! SF 3 slope/intercept
	integer(4)					::	A2_plus_5v_signal_processing(2)		! SF 3 slope/intercept
	integer(4)					::	A2_plus_5v_antenna_drive(2)			! SF 3 slope/intercept
	integer(4)					::	A2_GDO_voltage_ch_1(2)					! SF 3 slope/intercept
	integer(4)					::	A2_GDO_voltage_ch_2(2)					! SF 3 slope/intercept
	integer(4)					::	fill_16(53)
			
end type AMSU_A_L1A_File_Header

type AMSU_A_Data_to_Keep_from_Header
	character(len = 462)		:: Calibration
	character(len = 180)        :: Temp_to_Rad 
	character(len = 720)		:: A1_Digital_Conv
	character(len = 304)		:: A2_Digital_Conv
end type AMSU_A_Data_to_Keep_from_Header

type AMSU_A_Raw_Swath_Data

	! this is an undecode version of the L1B swath data, with just
	! enough data decoded to perform the orbitification step

	character*1,dimension(2560) :: raw_scan
	real(8)						:: rd_2000
	real(4)						:: nadir_latitude
	real(4)						:: nadir_longitude

end type AMSU_A_Raw_Swath_Data

type AMSU_A_L1B_Calibration_Data

	! this is the fully decoded versions of the Calibration data from the
	! L1B orbit header

	sequence

	integer(2)					::	instrument_temperature_sensor_ID(3)
	real(4)						::	RF_shelf_A1_1_reference_temperature_1(3)		! SF 2 minimum nominal maximum :: offset = 6
	real(4)						::	RF_shelf_A1_2_reference_temperature(3)			! SF 2 minimum nominal maximum :: offset = 12
	real(4)						::	RF_shelf_A2_reference_temperature(3)			! SF 2 minimum nominal maximum :: offset = 18
	real(4)						::	RF_shelf_A1_1_reference_temperature_2(3)		! SF 2 minimum nominal maximum :: offset = 24
	real(4)						::	RF_Mux_A1_1_reference_temperature_1(3)			! SF 2 minimum nominal maximum :: offset = 30
	real(4)						::	RF_Mux_A1_2_reference_temperature(3)			! SF 2 minimum nominal maximum :: offset = 36
	real(4)						::	RF_Mux_A2_reference_temperature(3)				! SF 2 minimum nominal maximum :: offset = 42
	real(4)						::	RF_Mux_A1_1_reference_temperature_2(3)			! SF 2 minimum nominal maximum :: offset = 48
	real(4)						::	fixed_bias_corr(15,4)							! SF 3 Ch 1-15 	               :: offset = 54
																					! warm target at minimum
																					! warm target at nominal
																					! warm target at maximum
																					! cold space
	real(4)						::	warm_target_bias_corr(6,3)						! SF 3 Ch 9-14 minimum nominal maximum  :: offset = 54+120 = 174
	real(4)						::	nonlinearity_coef(15,3)							! SF 6 Ch 1-15 ref 1 2 3                :: offset = 174+36 = 210
	real(4)						::	nonlinearity_coef_2(6,3)						! SF 6 Ch 9-14 ref 1 2 3                :: offset = 210 + 45*4 = 390


end type AMSU_A_L1B_Calibration_Data

type AMSU_A_L1B_Swath_Data

	! This is the fully decoded version of the L1B swath data

	sequence  ! this ensures that the structure is stored in sequence and that the
	          ! the equivalencing step will work

	! scan line information
	integer(2)					::	scan_line_number
	integer(2)					::	scan_line_year
	integer(2)					::	scan_line_day_of_year
	integer(2)					::	satellite_clock_drift_delta			    ! in msec
	integer(4)					::	scan_line_UTC_time_of_day				! in msec
	integer(2)					::	scan_line_bit_field
	integer(2)					::	major_frame_count
	integer(4)					::	fill_1(2)

	! quality indicators
	integer(4)					::	quality_indicator_bit_field
	integer(4)					::	scan_line_quality_flags
	integer(2)					::	calibration_quality_flags(16)
	integer(4)					::	fill_2(4)

	! calibration		
	integer(4)					::	primary_calibration(3,15)				! SF 19/13/9 Ch 1-15 a2/a1/a0
	integer(4)					::	secondary_calibration(3,15)			    ! SF 19/13/9 Ch 1-15 a2/a1/a0
	integer(4)					::	fill_3(4)

	! navigation
	integer(4)					::	navigation_status_bit_field
	integer(4)					::	time_associated_with_TIP_euler_angles	! in sec
	integer(2)					::	TIP_euler_angles(3)					! SF 3 in deg
	integer(2)					::	spacecraft_altitude					! SF 1 in km
	integer(2)					::	angular_relationships(3,30)			! SF 2 in deg	-> scan position 1-30 solar zenith/satellite zenith/local azimuth
	integer(4)					::	earth_location(2,30)				! SF 4 in deg lat (-90..90) / lon (-180..180)
	integer(4)					::	fill_4

	! A1 digital A telemetry
	integer(1)					::	A1_synchronization_sequence(3)
	integer(1)					::	A1_unit_identification
	integer(1)					::	A1_digital_housekeeping(4)
	integer(2)					::	A1_scene_telemetry(17,30)		! scan position 1-30
																	! reflector A1-1 counts reading 1
																	! reflector A1-2 counts reading 1
																	! reflector A1-1 counts reading 2
																	! reflector A1-2 counts reading 2
																	! Ch 3-15 scene counts

	integer(2)					::	A1_cold_calibration_telemetry(30)	! reflector A1-1 cold counts reading 1
																		! reflector A1-2 cold counts reading 1
																		! reflector A1-1 cold counts reading 2
																		! reflector A1-2 cold counts reading 2
																		! Ch 3-15 cold count 1
																		! Ch 3-15 cold count 2
	integer(2)					::	A1_temperature_sensor_telemetry(46)
	integer(2)					::	A1_warm_calibration_telemetry(30)		! reflector A1-1 warm counts reading 1
													! reflector A1-2 warm counts reading 1
													! reflector A1-1 warm counts reading 2
													! reflector A1-2 warm counts reading 2
													! Ch 3-15 warm count 1
													! Ch 3-15 warm count 2
	integer(4)					::	fill_5

	! A1 digital B telemetry
	integer(2)					::	A1_invalid_word_bit_flags
	integer(2)					::	A1_digital_B_telemetry
	integer(4)					::	fill_6
	! A1 analog housekeeping
	integer(4)					::	A1_invalid_data_bit_flags
	integer(1)					::	A1_housekeeping(28)
	integer(4)					::	fill_7
	! A2 digital A telemetry
	integer(1)					::	A2_synchronization_sequence(3)
	integer(1)					::	A2_unit_identification
	integer(1)					::	A2_digital_housekeeping(4)
	integer(2)					::	A2_scene_telemetry(4,30)		! scan position 1-30
																	! reflector counts reading 1
																	! reflector counts reading 2
													! Ch 1-2 scene counts
	integer(2)					::	A2_cold_calibration_telemetry(6)! reflector cold counts reading 1
																	! reflector cold counts reading 2
																	! Ch 1-2 cold count
	integer(2)					::	A2_temperature_sensor_telemetry(20)	
	integer(2)					::	A2_warm_calibration_telemetry(6)	! reflector warm counts reading 1
																		! reflector warm counts reading 2
																		! Ch 1-2 warm count
	integer(4)					::	fill_8
	! A2 digital B telemetry
	integer(2)					::	A2_invalid_word_bit_flags
	integer(2)					::	A2_digital_B_telemetry
	integer(4)					::	fill_9
	! A2 analog housekeeping
	integer(4)					::	A2_invalid_data_bit_flags
	integer(1)					::	A2_housekeeping(16)
	integer(4)					::	fill_10(8)
			
	end type AMSU_A_L1B_Swath_Data

	type AMSU_L1B_raw_orbit_data
		type(AMSU_A_L1B_Swath_Data),dimension(AMSU_MAX_SCANS_PER_ORBIT)		:: scan_data
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)						:: scan_valid
	end type AMSU_L1B_raw_orbit_data 

	type amsu_L1B_orbit_header

	    sequence

		character(len=7)	:: satellite_name		! byte 1:7
		character(len=1)    :: pad1					! byte 8
		integer(4)			:: orbit_number			! byte 9:12
		integer(4)			:: number_of_swaths		! byte 13:16
		integer(4)			:: start_swath_num		! byte 17:20
		integer(4)			:: end_swath_num		! byte 21:24
		real(8)				:: start_rd_2000		! byte 25:32
		real(8)				:: end_rd_2000			! byte 33:40
		real(8)				:: eqx_rd_2000			! byte 41:48
		character(len=52)   :: pad2					! byte 49:100
		character(len = 462)		:: Calibration	! byte 101:562
		character(len = 180)        :: Temp_to_Rad 	! byte 563:742
		character(len = 720)		:: A1_Digital_Conv !byte 743:1462
		character(len = 304)		:: A2_Digital_Conv  !byte 1463:1766	
		integer(1),dimension(AMSU_MAX_SCANS_PER_ORBIT)   :: scan_valid !byte 1767:2556
		character(len = 4)	:: pad3					

	end type amsu_L1B_orbit_header

	integer(4),parameter :: AMSU_LEVEL_2A_RECORD_LENGTH = 784

	type AMSU_A_L2A_Swath_Data
		
		sequence

		character(len = 7)		:: satellite_name		! byte 1:7
		integer(1)				:: channel_num			! byte 8:8
		integer(4)				:: orbit_number			! byte 9:12
		integer(4)				:: scan_line_in_orbit	! byte 13:16
		real(8)					:: scan_rd_2000			! byte 16:24
		real(4),dimension(30)	:: fov_latitude			! byte 25:144
		real(4),dimension(30)	:: fov_longitude		! byte 145:264
		integer(1),dimension(30):: fov_quality			! byte 265:294
		real(4)					:: warm_load_temp       ! byte 295:298
		integer(2)				:: asc_desc				! byte 299:300  (1 = asc, 2 = desc, 0 = undetermined)
		real(4),dimension(30)   :: Tbs					! byte 301:420
		real(4),dimension(30)	:: Diurnal_Corr			! byte 421:540
		real(4),dimension(30)	:: Nadir_Corr			! byte 541:660
		real(4),dimension(30)	:: Incidence_Corr		! byte 661:780
		real(4)					:: spacecraft_altitude  ! byte 781:784
	end type AMSU_A_L2A_Swath_Data 



	type AMSU_L2A_orbit_header

		sequence

		character(len=7)	:: satellite_name		! byte 1:7
		character(len=1)    :: pad1					! byte 8
		integer(4)			:: orbit_number			! byte 9:12
		integer(4)			:: number_of_swaths		! byte 13:16
		integer(4)			:: start_swath_num		! byte 17:20
		integer(4)			:: end_swath_num		! byte 21:24
		real(8)				:: start_rd_2000		! byte 25:32
		real(8)				:: end_rd_2000			! byte 33:40
		real(8)				:: eqx_rd_2000			! byte 41:48
		integer(4)			:: channel				! byte 49:52
		character(len = 48) :: pad					! byte 53:100
		integer(1),dimension(AMSU_MAX_SCANS_PER_ORBIT/2)   :: scan_valid !byte 101:495  packed 2 per byte
		real(4),dimension(30)::  cross_track_correction ! byte 496:615
		character(len= 20)  :: L2A_processor_ID      ! byte 616:635
		character(len = 20) :: L1B_processor_ID      ! byte 636:655

	end type AMSU_L2A_orbit_header

	integer(4),parameter :: AMSU_LEVEL_2B_RECORD_LENGTH = 420


	type AMSU_A_L2B_Swath_Data
		
		sequence

		character(len = 7)		:: satellite_name		! byte 1:7
		integer(1)				:: pad1					! byte 8:8
		integer(4)				:: orbit_number			! byte 9:12
		integer(4)				:: scan_line_in_orbit	! byte 13:16
		real(8)					:: scan_rd_2000			! byte 16:24
		real(4),dimension(30)	:: fov_latitude			! byte 25:144
		real(4),dimension(30)	:: fov_longitude		! byte 145:264
		integer(1),dimension(30):: fov_quality			! byte 265:294
		integer(2)				:: asc_desc				! byte 295:296  (1 = asc, 2 = desc, 0 = undetermined)
		real(4),dimension(30)   :: geophysical_par		! byte 297:416
		real(4)					:: spacecraft_altitude  ! byte 417:420


	end type AMSU_A_L2B_Swath_Data 

	type AMSU_L2B_orbit_header

		sequence

		character(len=7)	:: satellite_name		! byte 1:7
		character(len=1)    :: pad1					! byte 8
		integer(4)			:: orbit_number			! byte 9:12
		integer(4)			:: number_of_swaths		! byte 13:16
		integer(4)			:: start_swath_num		! byte 17:20
		integer(4)			:: end_swath_num		! byte 21:24
		real(8)				:: start_rd_2000		! byte 25:32
		real(8)				:: end_rd_2000			! byte 33:40
		real(8)				:: eqx_rd_2000			! byte 41:48
		character(len=10)	:: geophysical_par_name	! byte 49:68
		character(len = 48) :: pad					! byte 53:100
		integer(1),dimension(AMSU_MAX_SCANS_PER_ORBIT/4)   :: scan_valid !byte 101:495  packed 2 per byte


	end type AMSU_L2B_orbit_header
			


	type AMSU_L2A_orbit_data
		type(AMSU_A_L2A_Swath_Data),dimension(AMSU_MAX_SCANS_PER_ORBIT)		:: scan_data
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)						:: scan_valid
	end type AMSU_L2A_orbit_data 

	type amsu_L1B_orbit_data
		type(AMSU_A_L1B_Swath_Data),											&
			dimension(AMSU_MAX_SCANS_PER_ORBIT)				:: scan_data
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)		:: scan_valid
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)		:: header_num_used	
	end type amsu_L1B_orbit_data	 

	type amsu_L1A_orbit_data
		type(AMSU_A_Data_to_Keep_from_Header)				:: header_data
		character(len = AMSU_LEVEL_1_RECORD_LENGTH),	&
			dimension(AMSU_MAX_SCANS_PER_ORBIT)				:: scan_data
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)		:: scan_valid
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)		:: header_num_used	
	end type amsu_L1A_orbit_data 

    integer(4),parameter :: AMSU_LEVEL_2A_NAV_RECORD_LENGTH = 656

    type AMSU_L2A_nav_header       
                   
        sequence

		character(len=7)	:: satellite_name		! byte 1:7
		character(len=1)    :: pad1					! byte 8
		integer(4)			:: orbit_number			! byte 9:12
		integer(4)			:: number_of_swaths		! byte 13:16
		integer(4)			:: start_swath_num		! byte 17:20
		integer(4)			:: end_swath_num		! byte 21:24
		real(8)				:: start_rd_2000		! byte 25:32
		real(8)				:: end_rd_2000			! byte 33:40
		real(8)				:: eqx_rd_2000			! byte 41:48
		integer(4)			:: channel				! byte 49:52
		character(len = 48) :: pad					! byte 53:100
		integer(1),dimension(AMSU_MAX_SCANS_PER_ORBIT/2)   :: scan_valid !byte 101:495  packed 2 per byte
		real(4),dimension(30)::  cross_track_correction ! byte 496:615
		character(len= 20)  :: L2A_processor_ID      ! byte 616:635
		character(len = 20) :: L1B_processor_ID      ! byte 636:655
        character(len=1)    :: pad2					 ! byte 656
	end type AMSU_L2A_nav_header

    type AMSU_A_L2A_nav_Swath_Data
    
        sequence

		character(len = 7)		:: satellite_name		! byte 1:7
		integer(1)				:: channel_num			! byte 8:8
		integer(4)				:: orbit_number			! byte 9:12
		integer(4)				:: scan_line_in_orbit	! byte 13:16
		real(8)					:: scan_rd_2000			! byte 17:24
		real(4)                 :: satheight            ! byte 25:28
		real(4)                 :: satroll              ! byte 29:32
		real(4)                 :: satpitch             ! byte 33:36
		real(4)                 :: satyaw               ! byte 37:40
		real(8)                 :: sat_lat              ! byte 41:48
		real(8)                 :: sat_lon              ! byte 49:56
        real(4),dimension(30)   :: satzen               ! byte 57:176
        real(4),dimension(30)   :: satazi               ! byte 177:296
        real(4),dimension(30)   :: solzen               ! byte 297:416
        real(4),dimension(30)   :: sol_azi              ! byte 417:536
        real(4),dimension(30)   :: landfrac             ! byte 537:656          
     
     end type AMSU_A_L2A_nav_Swath_Data   
 
	 type AMSU_L2A_nav_data
		type(AMSU_A_L2A_nav_Swath_Data),dimension(AMSU_MAX_SCANS_PER_ORBIT)	:: scan_data
		integer(4),dimension(AMSU_MAX_SCANS_PER_ORBIT)						:: scan_valid
	end type AMSU_L2A_nav_data 

end module AMSU_Types