$freeform
!@***m* MLT_RSS/NAN_support_intel.f90
!  NAME
!    module NAN_Support
! 
!  PURPOSE
!    module containing Not-A-Number wrapper routines for intel compiler and
!    constant parameter NAN valued set equal to 1.0/0.0
!    emulates behavior of early module for a compiler that did not explicitly
!    support ieee not-a-numbers
! 
!  DESCRIPTION
!     module Not-A-Numbers Wrapper routines for intel compiler 
!    
!  ROUTINES
!      logical function finite(x), overloaded to accept both real(4) and real(8) arguaments
!
!  INPUTS
!       x    real   number to be tested
!
!  OUTPUTS
!       returns logical true if number is finite, false if number in any type of non-finite number
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use NAN_Support
! 
!@*****
	module NAN_support
        USE, INTRINSIC :: IEEE_ARITHMETIC
		implicit none
		
		real(4),parameter  :: NAN = 1.0/0.0

		interface finite
			module procedure finite4
			module procedure finite8
		end interface

	contains

		logical function finite4(x)

		! This function determines if x is a finite number
		! Returns 1 if X is finite, and 0 if x is infinite, or a non-zero denormal

			real(4),intent(in)			::	x
		    finite4 = IEEE_IS_FINITE(x)
			

			return

		end function finite4



		logical function finite8(x)

		! This function determines if x is a finite number
		! Returns 1 if X is finite, and 0 if x is infinite, or a non-zero denormal

			real(8),intent(in)			::	x

			finite8 = IEEE_IS_FINITE(x)
			return

		end function finite8
	end module NAN_support 


