$freeform
module byte_swapping
!@***m* MLT_RSS/byte_swapping.f90
!  NAME
!    byte_swapping
! 
!  PURPOSE
!    module containing a number of routines that perform byte swapping
! 
!  DESCRIPTION
!    module containing a number of routines that perform byte swapping.  The function "byte_swap" is overloaded
!    and calls the appropriate of the 4 routines below to perfrom a big endian to little endian byte swap.
!    
!  ROUTINES
!      real(4) function real_4_byte_swap(x)
!      real(8) function real_8_byte_swap(x)
!      integer(4) function int_4_byte_swap(x)
!      integer(2) function int_2_byte_swap(x)
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use byte_swap
!    j = byte_swap(i)
! 
!@*****

   interface byte_swap
		module procedure real_4_byte_swap		!(x)    real(4)
		module procedure real_8_byte_swap		!(x)    real(8)
		module procedure int_2_byte_swap		!(x)	int(2)
		module procedure int_4_byte_swap		!(x)    int(4)
   end interface

contains

	real(4) function real_4_byte_swap(x)

		implicit none

		real(4) x,xx
		character*1 a(4),aa1,aa2
		equivalence(xx,a(1))

		xx   = x
		aa1  = a(1)
		aa2  = a(2)
		a(1) = a(4)
		a(2) = a(3)
		a(3) = aa2
		a(4) = aa1
		real_4_byte_swap = xx

	end function real_4_byte_swap

	
	real(8) function real_8_byte_swap(x)

		implicit none
	
		real(8) x,xx
		character*1 a(8),aa1,aa2,aa3,aa4
		equivalence(xx,a(1))

		xx   = x
		aa1  = a(1)
		aa2  = a(2)
		aa3  = a(3)
		aa4  = a(4)
		a(1) = a(8)
		a(2) = a(7)
		a(3) = a(6)
		a(4) = a(5)
		a(5) = aa4
		a(6) = aa3
		a(7) = aa2
		a(8) = aa1
		real_8_byte_swap = xx

	end function real_8_byte_swap


	integer(4) function int_4_byte_swap(x)

		implicit none
	
		integer(4) x,xx
		character(len=1) a(4),aa1,aa2
		equivalence(xx,a(1))

		xx   = x
		aa1  = a(1)
		aa2  = a(2)
		a(1) = a(4)
		a(2) = a(3)
		a(3) = aa2
		a(4) = aa1
		int_4_byte_swap = xx
		return

	end function int_4_byte_swap

	integer(2) function int_2_byte_swap(x)

		implicit none
	
		integer(2),intent(IN)		:: x
		integer(2)					:: xx
		character(1),dimension(2)	::  a
		character(1)				:: aa1
		equivalence(xx,a(1))

		xx   = x
		aa1  = a(1)
		a(1) = a(2)
		a(2) = aa1
		int_2_byte_swap = xx

	end function int_2_byte_swap
end module byte_swapping
