$freeform
! @***h* MLT_RSS/error_statistics.f90
!  NAME
!    module error_statistics
! 
!  PURPOSE
!    module for accumulating error statistics
! 
!  DESCRIPTION
!    module for accumulating error statistics
!    
!    Components:
!    the module has eight integers defined for each MSU channel
!
!	integer(4),dimension(NUM_CHANNELS)			:: Num_Tot_Scans
!   integer(4),dimension(NUM_CHANNELS)			:: Num_Bad_Qual_Flags
!	integer(4),dimension(NUM_CHANNELS)			:: Num_bad_geolocation
!	integer(4),dimension(NUM_CHANNELS)			:: Num_height_out_of_range
!	integer(4),dimension(NUM_CHANNELS)			:: Num_Bad_Target_Temp
!	integer(4),dimension(NUM_CHANNELS)			:: Num_counts_at_Limit
!	integer(4),dimension(NUM_CHANNELS)			:: Num_Tb_out_of_range
!	integer(4),dimension(NUM_CHANNELS)			:: Num_Hydrometeors
! 
!   ROUTINES
!    subroutine zero_error_report(chan)
!       sets the above constants to zero for the given channel (1..4)
!       INPUTS
!           chan         MSU channel number

!    write_error_report(lu,chan)
!       writes a text error report to the given logical unit lu, using data for the
!       given channel
!       INPUTS
!           lu          logical unit to write to
!           chan        MSU channel number

! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use error_statistics
! 
! @*****
module error_statistics

use msu_constants

implicit none

! This module contains globals definitions and 
! procedures to accumulate and report error statistics

	integer(4),dimension(NUM_CHANNELS)			:: Num_Tot_Scans
	integer(4),dimension(NUM_CHANNELS)			:: Num_Bad_Qual_Flags
	integer(4),dimension(NUM_CHANNELS)			:: Num_bad_geolocation
	integer(4),dimension(NUM_CHANNELS)			:: Num_height_out_of_range
	integer(4),dimension(NUM_CHANNELS)			:: Num_Bad_Target_Temp
	integer(4),dimension(NUM_CHANNELS)			:: Num_counts_at_Limit
	integer(4),dimension(NUM_CHANNELS)			:: Num_Tb_out_of_range
	integer(4),dimension(NUM_CHANNELS)			:: Num_Hydrometeors

contains

	subroutine zero_error_report(chan)
		
		integer(4),intent(IN)		:: chan

		Num_Tot_Scans(chan)				= 0
		Num_Bad_Qual_Flags(chan)		= 0
		Num_bad_geolocation(chan)		= 0
		Num_height_out_of_range(chan)	= 0
		Num_Bad_Target_Temp(chan)		= 0
		Num_counts_at_Limit(chan)		= 0
		Num_Tb_out_of_range(chan)		= 0
		Num_Hydrometeors(chan)			= 0

	end subroutine zero_error_report


	subroutine write_error_report(lu,chan)

		integer(4),intent(IN)		::	lu
		integer(4),intent(IN)		::	chan

		write(lu,*)
		write(lu,*)
		write(lu,8)chan
 8      format(1x,'Error Statistics Report for Channel ',i2)
		write(lu,*)
		
		write(lu,10)num_tot_scans(chan)					    
10		format(1x,'Total Number of Scans Processed          : ',i8)

		write(lu,11)Num_Bad_Qual_Flags(chan)
11      format(1x,'Num Already flagged as bad               : ',i8)
															
		write(lu,12)Num_bad_geolocation(chan)
12		format(1x,'Number excluded by bad geolocation       : ',i8)

		write(lu,14)Num_height_out_of_range(chan)
14		format(1x,'Number excluded by height out of range   : ',i8)
															
		write(lu,16)Num_bad_target_temp(chan)
16		format(1x,'Number excluded by bad target temp       : ',i8)

		write(lu,18)Num_Counts_at_limit(chan)
18		format(1x,'Number excluded by counts at limit       : ',i8)

		write(lu,20)Num_Tb_out_of_range(chan)
20		format(1x,'Number excluded Tb out of range          : ',i8)

		write(lu,22)Num_hydrometeors(chan)
22		format(1x,'Number excluded by possible hydrometeors : ',i8)

	end subroutine write_error_report
end module error_statistics