
! @***h* MLT_RSS/gzip3264.f
!  NAME
!    module gzip3264
! 
!  PURPOSE
!    module for interfacing to standard gzip library, which is in C
! 
!  DESCRIPTION
!    module for interfacing to standard gzip library, which is in C
! 
!  ROUTINES
!       FASTUNZIP(FILENAME, ISIZE,ABUF,IERR)
!           unzips data in filename into buffer ABUF
!       
!       INPUTS
!           FILENAME      filename to unzip
!       OUTPUTS
!           ISIZE         size of resulting data, in bytes
!           ABUF          buffer containing the inflated data
!           IERR          error flag -- problem if not zero

!       FASTGZIP(FILENAME,ISIZE,ABUF, IERR)
!          writes data in abuf to a gzipped file
!       INPUTS
!           FILENAME      filename to unzip
!           ISIZE         size of resulting data, in bytes
!           ABUF          buffer containing the inflated data
!       OUTPUTS
!           IERR          error flag -- problem if not zero
!       
!       FIND_UNZIP_SIZE(FILENAME, ISIZE)
!           finds size of unzipped data in file
!       
!       INPUTS
!           FILENAME      filename to unzip
!       OUTPUTS
!           ISIZE         size of resulting data, in bytes
! 
!  AUTHOR
!    Frank Wentz, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
!   TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use gzip3264
! 
!  COMMENTS
!     This version is for the new intel fortran compiler.  It works for for 32 and 64 bit applications
!     The ALIAS convention has been removed and actual c names are used for the subroutines
!	LOADS AND DECOMPRESSES GZIPPED DATA FROM FILE  
!     GIVEN BY FILENAME INTO DATARRY, 
!	AND RETURNS NUMBER OF BYTES DECOMPRESSED IN DATACNT
!	THIS SHOULD BE THE SAME AS RETURNED BY UNCMPSIZ
! @*****

      Module gzip 
      
      contains
      
	SUBROUTINE FASTUNZIP(FILENAME, ISIZE,ABUF,IERR)

	INTERFACE
	SUBROUTINE gzip_inflate [C] (FILENAME, DATARRY,DATACNT,ERR)
	CHARACTER*(*) FILENAME [REFERENCE]
	CHARACTER(1),DIMENSION(*)::DATARRY [REFERENCE]
	INTEGER(4) DATACNT [REFERENCE]
	INTEGER(4) ERR [REFERENCE]
	END	SUBROUTINE
	END INTERFACE

	CHARACTER(1) CHAR
	CHARACTER*(*) FILENAME
	CHARACTER(101) FILENAMEC
	CHARACTER(1) ABUF(*)

      NN=LEN_TRIM(FILENAME)
	FILENAMEC=FILENAME(1:NN) // CHAR(0)
	CALL gzip_inflate(FILENAMEC, ABUF,ISIZE,IERR)
	RETURN
	END SUBROUTINE FASTUNZIP



C	COMPRESSES DATA IN DATARRY IN GZIP FORMAT AND
C     SAVES IN FILE GIVEN BY FILENAME, 
C	AND RETURNS NUMBER OF BYTES DECOMPRESSED IN DATACNT
C	THIS SHOULD BE THE SAME AS RETURNED BY UNCMPSIZ

	SUBROUTINE FASTGZIP(FILENAME,ISIZE,ABUF, IERR)

	INTERFACE
	SUBROUTINE gzip_deflate [C] (FILENAME, DATARRY,DATACNT,ERR)
	CHARACTER*(*) FILENAME [REFERENCE]
	CHARACTER(1),DIMENSION(*)::DATARRY [REFERENCE]
	INTEGER(4) DATACNT [REFERENCE]
	INTEGER(4) ERR [REFERENCE]
	END	SUBROUTINE
	END INTERFACE

	CHARACTER(1) CHAR
	CHARACTER*(*) FILENAME
	CHARACTER(101) FILENAMEC
	CHARACTER(1) ABUF(*)

      NN=LEN_TRIM(FILENAME)
	FILENAMEC=FILENAME(1:NN) // CHAR(0)
	CALL gzip_deflate(FILENAMEC,ABUF,ISIZE, IERR)

	RETURN
	END SUBROUTINE FASTGZIP




C     DETERMINES UNCOMPRESS SIZE

	SUBROUTINE FIND_UNZIP_SIZE(FILENAME, ISIZE)

	INTERFACE
	SUBROUTINE uncompressed_size [C] (FILENAME, DATASIZE)
	CHARACTER*(*) FILENAME [REFERENCE]
	INTEGER(4) DATASIZE [REFERENCE]
	END	SUBROUTINE
	END INTERFACE

	CHARACTER(1) CHAR
	CHARACTER*(*) FILENAME
	CHARACTER(101) FILENAMEC
      NN=LEN_TRIM(FILENAME)
	FILENAMEC=FILENAME(1:NN) // CHAR(0)
	CALL uncompressed_size(FILENAMEC, ISIZE)
	RETURN
	END SUBROUTINE FIND_UNZIP_SIZE
      end module gzip