$freeform
!@***h* MLT_RSS/logging.f90
!  NAME
!    logging
! 
!  PURPOSE
!    pseudoclass for producing log files
! 
!  DESCRIPTION
!    pseudoclass for producing log files.
!    
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use logging
! 
!@*****
module logging
	
		use date_and_time

		integer(4),private				:: log_unit = 4
		logical(4),private				:: log_open = .false.
		character(len = 120),private	:: log_file
		character(len = 300)			:: log_str
		integer(4)						:: log_err



	contains

		subroutine init_log_file(file,error)

!@***s* MLT_RSS/logging.f90/init_log_file
!  NAME
!    init_log_file
! 
!  PURPOSE
!    Opens log file with name file
!  DESCRIPTION
!    Opens log file with name "file".  This file is accessible to any routine that includes logging, simplifying
!    log file writing. 
! 
!  INPUTS
!    character(len = 120)    :: file    ! file name to open as log file
!  OUTPUTS
!    integer(4)		     :: error   ! error flag.  0 mean everything OK
!  USAGE
!     call init_log_file(file,error)
!@*****

			character(len = 120)	               :: file
			integer(4)				:: error
			integer(4)				:: ioerror


			open(unit = log_unit, file = file, iostat = io_error,ACCESS = 'APPEND')
			
			if (io_error .eq. 0) then
				log_open = .true.
				log_file = file
				print *,'Opened Log File In Fortran: '//file
				error = 0
			else
				log_open = .false.
				log_file = 'Log File not Open'
				print *,'Could not open log file in Fortran: '//file
				error = -1
			endif

		end subroutine init_log_file

		subroutine log(a,error)
!@***s* MLT_RSS/logging.f90/log
!  NAME
!    log
!  PURPOSE
!    write a message to the open log file.
!  INPUTS
!    character*(*)    :: a       ! message to write to log file
!  OUTPUTS
!    integer(4)		     :: error   ! error flag.  0 mean everything OK
!  USAGE
!     call log(file,error)
!@*****
			character*(*)			:: a
			integer(4)			:: error



	10		format(a)
			write(6,10)curr_date_string(error)//': '//a
			if (log_open) then
				write(log_unit,10)curr_date_string(error)//': '//a
			endif

		end subroutine log
	end module logging