$freeform
!@***m* MLT_RSS/mcs.f90
!  NAME
!    mcs
! 
!  PURPOSE
!    psuedoclass that reads and write "mcs" self-describing array type
!  DESCRIPTION
!    psuedoclass that reads and write "mcs" self-describing array type
!
!  ROUTINES
!    subroutine read_real_4_mcs(lu,mat,error)
!    subroutine write_real_4_mcs(lu,mat,error)
!    deallocate_mcs(mat,error)
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use mcs
! 
!@*****
module mcs

	! Module contains routines to read and write matthias's matrix type
	! file structures	 

	use NAN_support

	implicit none

	type real_4_matrix_str
		integer(4)						:: lo_x
		integer(4)						:: hi_x
		integer(4)						:: lo_y
		integer(4)						:: hi_y
		real(4),dimension(:,:),pointer	:: dat		  
	end type real_4_matrix_str


contains

	subroutine read_real_4_mcs(lu,mat,error)

		use NAN_support

		implicit none

		integer(4),intent(IN)				:: lu	! logical unit number of file to be read
		type(real_4_matrix_str),intent(OUT)	:: mat  ! matric to be read
		integer(4),intent(OUT)				:: error

		integer(4)							:: lo_x
		integer(4)							:: hi_x
		integer(4)							:: lo_y
		integer(4)							:: hi_y
		byte,dimension(4)					:: flags


		read(lu)lo_x,hi_x,lo_y,hi_y,flags

		if (flags(2) .ne. 8) then
			error = 1
			return
		endif

		allocate(mat%dat(lo_x:hi_x,lo_y:hi_y),STAT = error)
		
		if (error .ne. 0) then
			print *,'Allocation Error'
			return
		endif
		
		mat%lo_x = lo_x
		mat%hi_x = hi_x
		mat%lo_y = lo_y
		mat%hi_y = hi_y
		
		read(lu)mat%dat
		
		return
		
	end subroutine read_real_4_mcs

	subroutine write_real_4_mcs(lu,mat,error)

		use NAN_support

		implicit none

		integer(4),intent(IN)				:: lu	! logical unit number of file to be read
		type(real_4_matrix_str),intent(IN)	:: mat  ! matric to be written
		integer(4),intent(OUT)				:: error

		byte,dimension(4)					:: flags

		flags = 0
		flags(2) = 8
		write(lu)mat%lo_x,mat%hi_x,mat%lo_y,mat%hi_y,flags	
		write(lu)mat%dat
		error = 0
		
		return
		
	end subroutine write_real_4_mcs


	subroutine deallocate_mcs(mat,error)

		use NAN_support

		implicit none

		type(real_4_matrix_str),intent(INOUT)	:: mat  ! matrix to be deallocated
		integer(4),intent(OUT)					:: error

	
		deallocate(mat%dat,STAT = error)

		mat%lo_x = 0
		mat%hi_x = 0
		mat%lo_y = 0
		mat%hi_y = 0

		return

	end subroutine deallocate_mcs
	
end module mcs 











