$freeform
! @***h* MLT_RSS/msu_L2A_inventory.f90
!  NAME
!    msu_L2A_inventory
! 
!  PURPOSE
!    msu_L2A_inventory psuedoclass
! 
!  DESCRIPTION
!    pseudoclass for reading and writing MSU L2A orbit inventories
!    
!  ROUTINES
!      read_text_msu_L2A_inventory_file
!      write_msu_L2A_inventory_file_text
!      update_L2A_inventory_data
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use msu_L2A_inventory
! 
! @*****

module msu_L2A_inventory

	use nan_support
	use date_and_time
	use msu_constants
	use msu_types

	implicit none

contains

	subroutine write_msu_L2A_inventory_file_text(sat_num,inv,error)
	
	! @***h* write_msu_L2A_inventory_file_text
    !  NAME
    !    write_msu_L2A_inventory_file_text
    ! 
    !  PURPOSE
    !    writes msu L2A inventory file to a text file
    ! 
    !  DESCRIPTION
    !    writes msu L2A inventory file to a text file
    !    
    !  INPUTS
    !    	integer(4)					:: satnum		    ! satellite number (1 = TIROS-N, 2 = NOAA-06, etc)
    !    	type(msu_inventory_str)		:: msu_inventory     ! MSU inventory structure (type defined in msu_L1B_inventory)
    !  OUTPUTS
    !    error          integer(4)                  :: error, set to 0 if read successful,1 if day_num_1978 = 0 (empty file) or -1 if there is read error. 

    ! 
    !  AUTHOR
    !    Carl Mears, Remote Sensing Systems
    ! 
    ! COPYRIGHT
    !   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
    !   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
    !   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
    !   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
    !   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
    !   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
    !   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
    !   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
    !   TO USERS.
    ! 
    !  HISTORY
    !    2/21/2012 Initial Version prepared for NCDC
    !  
    !  USAGE
    !    call write_msu_L2A_inventory_file_text(sat_num,inv,error)
    !
    ! @*****

		integer(4)				:: sat_num
		type(msu_inventory_str) :: inv
		integer(4)				:: error

		character(len = FILE_LEN) :: inv_file
		integer(4)				  :: io_error
		integer(4)				  :: j

		error = 0

		if ((sat_num .lt. 1) .or. (sat_num .gt. 9)) then
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(MSU_L2A_PATH),trim(sat_names_2(sat_num)),trim(sat_names_2(sat_num))
111		format(a,a,'\Mo1995\',a,'_L1A.inv.txt')

		! open the file

		open(unit = 3, file = inv_file,iostat = io_error)
		
		if(io_error .ne. 0) then
			print *,'Fatal Error -- could not open inventory file'
			print *,inv_file
			error = -1
			stop
		endif

		do j = 1,inv%max_orbit
			if (inv%orbit_valid(j)) then
			write(3,888) j,inv%start_swath(j),	&
					inv%start_rd_2000(j),inv%end_swath(j),	&
					inv%end_rd_2000(j),inv%crossing_rd_2000(j)
888					format(1x,i7,1x,i3,1x,f14.7,1x,i3,1x,f14.7,1x,f14.7)
			endif
		enddo
		
		close(3)

		return
		
	end subroutine write_msu_L2A_inventory_file_text

	subroutine read_text_msu_L2A_inventory_file(sat_num,inv,error)
		! @***h* read_txt_msu_L2A_inventory_file
        !  NAME
        !    read_text_msu_L2A_inventory_file
        ! 
        !  PURPOSE
        !    reads msu L2A inventory file to a text file
        ! 
        !  DESCRIPTION
        !    reads msu L2A inventory file to a text file
        !    
        !  INPUTS
        !    	integer(4)					:: sat_num		    ! satellite number (1 = TIROS-N, 2 = NOAA-06, etc)
        !  OUTPUTS
        !    type(msu_inventory_str)		:: msu_inventory     ! MSU inventory structure (type defined in msu_L1B_inventory)
        !    error          integer(4)      :: error, set to 0 if read successful,1 if day_num_1978 = 0 (empty file) or -1 if there is read error. 

        ! 
        !  AUTHOR
        !    Carl Mears, Remote Sensing Systems
        ! 
        ! COPYRIGHT
        !   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
        !   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
        !   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
        !   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
        !   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
        !   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
        !   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
        !   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
        !   TO USERS.
        ! 
        !  HISTORY
        !    2/21/2012 Initial Version prepared for NCDC
        !  
        !  USAGE
        !    call read_text_msu_L2A_inventory_file(sat_num,inv,error)
        !
        ! @*****

		integer(4)				:: sat_num

		type(msu_inventory_str) :: inv
		integer(4)				:: error

		character(len = FILE_LEN) :: inv_file
		integer(4)				  :: io_error
		integer(4)				  :: j

		integer(4)				  :: orbit_num

		integer(4)				  :: strt_sw,end_sw
		real(8)					  :: strt_rd,end_rd,eqx_rd


		error = 0

		if ((sat_num .lt. 1) .or. (sat_num .gt. 9)) then
			error = -1
			return
		endif

		! make the file name

		write(inv_file,111) trim(MSU_L2A_PATH),trim(sat_names_2(sat_num)),trim(cal_string),trim(sat_names_2(sat_num))
111		format(a,a,'\Mo1995\',a,'_L1A.inv.txt')
		! open the file
		print *,'read_text_msu_L2A_inventory_file'
		print *,inv_file

		open(unit = 3, file = inv_file,iostat = io_error,status='old')
		
		if(io_error .ne. 0) then
			print *,'Fatal Error -- could not open inventory file'
			print *,inv_file
			error = -1
			stop
		endif

		do j = 1,MX_ORBITS
			read(3,888,end=999) orbit_num,strt_sw,strt_rd,end_sw,end_rd,eqx_rd
888					format(1x,i7,1x,i3,1x,f14.7,1x,i3,1x,f14.7,1x,f14.7)
			if ((orbit_num .gt. 0) .and. (orbit_num .le. MX_ORBITS)) then
					inv%start_swath(orbit_num)		= strt_sw
					inv%start_rd_2000(orbit_num)	= strt_rd
					inv%end_swath(orbit_num)		= end_sw
					inv%end_rd_2000(orbit_num)		= end_rd
					inv%orbit_valid(orbit_num)		= .true.
					inv%max_orbit = orbit_num
			endif
		enddo
		
999		close(3)

			print *,'Read Inventory File: ',inv_file     
	        print *,'Max Orbit = ',inv%max_orbit


		do orbit_num = 1,inv%max_orbit
			if (inv%orbit_valid(orbit_num)) then
				inv%crossing_rd_2000(orbit_num) = inv%start_rd_2000(orbit_num) - ((inv%start_swath(orbit_num)-0.5)*MSU_SCAN_TIME)
			endif
		enddo


		return
		
	end subroutine read_text_msu_L2A_inventory_file

	subroutine update_L2A_inventory_data(msu_inventory,				&
									 L2A_orbit_data,				&
									 orbit_num)
									 
		! @***h* update_L2A_inventory_data
        !  NAME
        !    update_L2A_inventory_data
        ! 
        !  PURPOSE
        !    updates a single orbit record in the inventory structure
        ! 
        !  DESCRIPTION
        !    updates a single orbit record in the inventory structure by evaluating the data from that orbit
        !    
        !  INPUTS
        !    type(msu_inventory_str)		:: msu_inventory     ! MSU inventory structure (type defined in msu_L1B_inventory)
        !    type(msu_L2A_orbit_data)	    :: L2A_orbit_data    ! L2A data from orbit
		!    integer(4)					    :: orbit_num         ! orbit number to be updated
        !  OUTPUTS
        !    type(msu_inventory_str)		:: msu_inventory     ! MSU inventory structure, updated with new info
        !
        ! 
        !  AUTHOR
        !    Carl Mears, Remote Sensing Systems
        ! 
        ! COPYRIGHT
        !   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
        !   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
        !   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
        !   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
        !   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
        !   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
        !   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
        !   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
        !   TO USERS.
        ! 
        !  HISTORY
        !    2/21/2012 Initial Version prepared for NCDC
        !  
        !  USAGE
        !    call update_L2A_inventory_data(msu_inventory,L2A_orbit%data,orbit_num)
        !
        ! @*****

		type(msu_inventory_str)				:: msu_inventory
		type(msu_L2A_orbit_data)			:: L2A_orbit_data
		integer(4)							:: orbit_num

		integer(4)			:: scan
		integer(4)			:: num_scans_valid

		integer(4)			:: first_scan
		integer(4)			:: last_scan

		real(8)				:: first_rd_2000
		real(8)				:: last_rd_2000

		num_scans_valid = 0

		do scan = 1,MAX_SCANS_PER_ORBIT
			if (L2A_orbit_data%scan_valid(scan)) then
				num_scans_valid = num_Scans_valid + 1
				if (num_scans_valid .eq. 1) first_scan = scan
				last_scan = scan
			endif
		enddo

		first_rd_2000 = L2A_orbit_data%scan_data(first_scan)%date%rd_2000
		last_rd_2000 =  L2A_orbit_data%scan_data(last_scan)%date%rd_2000
		

		if (msu_inventory%orbit_valid(orbit_num)) then 
			msu_inventory%start_swath(orbit_num)	= first_Scan
			msu_inventory%end_swath(orbit_num)		= last_Scan
			msu_inventory%start_rd_2000(orbit_num)	= first_rd_2000
			msu_inventory%end_rd_2000(orbit_num)	= last_rd_2000
		else
			write(6,*)'inventory error -- attemp to assign scan data to invalid orbit =',orbit_num
			write(21,*)'inventory error -- attemp to assign scan data to invalid orbit =',orbit_num
			return
		endif
			
	end subroutine update_L2A_inventory_data
		
end module msu_L2A_inventory        	 







		