$freeform
module MSU_calibration
!@***s* MLT_RSS/MSU_calibration.f90
!  NAME
!    module MSU_calibration
!
!  PURPOSE
!    psuedoclass for laoding and applying NOAA-supplied MSU calibration parameters
! 
!  DESCRIPTION
!    psuedoclass for laoding and applying NOAA-supplied MSU calibration parameters
!    performs MSU counts to antenna temperature conversion step.
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use MSU_calibration
!
!@*****
!
!@***s* MLT_RSS/MSU_calibration.f90/get_Mo_1995_NL_Coeff   
!  NAME
!       get_Mo_1995_NL_Coeff
!  DESCRIPTION    
!       This reads in the Mo 1995 version of the NOAA operational
!       non-linearity corrections.  Values are stored in module variables.
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_Mo_1995_NL_Coeff
!@*****

!@***s* MLT_RSS/MSU_calibration.f90/get_Mo_2001_NL_Coeff   
!  NAME
!       get_Mo_2001_NL_Coeff
!  DESCRIPTION    
!       This reads in the Mo 2001 version of the NOAA operational
!       non-linearity corrections.  Values are stored in module variables.
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call get_Mo_2001_NL_Coeff
!@*****

!@***s* MLT_RSS/MSU_calibration.f90/msu_non_linearity_correction
!
!       NAME msu_non_linearity_correction(channel,sat_num,version,counts_raw,counts_corr,error)
!
!       DESCRIPTION
!           calculates corrected instrument counts after applying NOAA operational non-linearity correction
!       INPUTS
!           integer(4) :: channel     ! msu channel number
!           integer(4) :: sat_num     ! satellite index (TIROS-N = 1, NOO-06 = 1, etc.)
!           integer(4) :: version     ! flag -- which version of NL correction to use.
!           integer(2) :: counts_raw  ! raw instrument counts
!       OUTPUTS
!           real(8) :: counts_corr    ! adjusted counts
!           integer(4) :: error       ! error flag == 0 means all OK
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!       USAGE
!           call msu_non_linearity_correction(channel,sat_num,version,counts_raw,counts_corr,error)
!@*****

!@***s* MLT_RSS/MSU_calibration.f90/calc_resistances
!       NAME 
!           calc_resistances
!
!       DESCRIPTION
!           calculates MSU hot load thermistor resistances from instrument counts
!       INPUTS
!           type(msu_raw)		:: L1b_scan
!		
!       OUTPUTS
!           type(msu_L2A_scan)	:: L2A_scan
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!
!       USAGE
!           calc_resistances(L1B_scan,L2A_scan)
!@*****

!@***s* MLT_RSS/MSU_calibration.f90/calc_target_temps
!       
!       subroutine calc_target_temps(L2A_scan)
!
!       DESCRIPTION
!           calculates MSU hot load thermistor temperatures from thermistor resistance
!       INPUTS
!           type(msu_L2A_scan)	:: L2A_scan
!		
!       OUTPUTS
!           type(msu_L2A_scan)	:: L2A_scan
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!       USAGE
!           call calc_target_temps(L2A_scan)
!

!@*****

!@***s* MLT_RSS/MSU_calibration.f90/
!       NAME
!           calc_raw_TBs
!       DESCRIPTION
!           calculates MSU Raw brightness temperatures from thermistor temperature and
!           corrected radiometer counts
!       INPUTS
!           type(msu_L2A_scan)	:: L2A_scan
!		
!       OUTPUTS
!           type(msu_L2A_scan)	:: L2A_scan
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!    THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!    THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!    AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!    AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!    SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!    THE USE OF THE SOFTWARE AND DOCUMENTATION!  OR (2) TO PROVIDE TECHNICAL SUPPORT 
!    TO USERS.
! 
!  HISTORY
!    9/21/2012 Initial Version prepared for NCDC
!       USAGE
!           call calc_raw_TBs(L2A_scan)
!@*****!

	use NAN_support
	use msu_constants
	use msu_types

	implicit none


	integer(4),parameter		:: MO_1995 = 1
	integer(4),parameter		:: MO_2001 = 2
	integer(4),parameter		:: LINEAR  = 3

	integer(4),parameter			:: NUM_MSU_NL_COEFFS = 3

	real(8),dimension(NUM_SATS,NUM_CHANNELS,NUM_NL_COEFFS)	:: MO_1995_NL_CFS
	logical													:: MO_1995_Valid = .false.
	character(len = 100),parameter :: MO_1995_File = &
		'\\dell-p866a\e\module_data\msu_utilities\msu_calibration\Mo_1995_NL_Coeff.txt'
	real(8),dimension(NUM_SATS,NUM_CHANNELS,NUM_NL_COEFFS)	:: MO_2001_NL_CFS
	logical													:: MO_2001_Valid = .false.
	character(len = 100),parameter :: MO_2001_File = &
		'\\dell-p866a\e\module_data\msu_utilities\msu_calibration\Mo_2001_NL_Coeff.txt'


contains

	subroutine get_Mo_1995_NL_Coeff

		integer(4)			:: sat_num
		integer(4)			:: chan
		character(len = 5)	:: junk

		open(unit=3,file=MO_1995_File,status='old')
		do sat_num = 1,9
			read(3,5)junk
5			format(a5)
			do chan = 1,4
				read(3,10)	MO_1995_NL_CFS(sat_num,chan,1),  &
							MO_1995_NL_CFS(sat_num,chan,2),	 &
							MO_1995_NL_CFS(sat_num,chan,3)
10							format(2x,3f20.0)
			enddo
		enddo

		MO_1995_Valid = .true.


	end subroutine get_Mo_1995_NL_Coeff

	subroutine get_Mo_2001_NL_Coeff

		implicit none

		integer(4)			:: sat_num
		integer(4)			:: chan
		character(len = 5)	:: junk

		open(unit=3,file=MO_2001_File,status='old')
		do sat_num = 1,9
			read(3,5)junk
5			format(a5)
			do chan = 1,4
				read(3,10)	MO_2001_NL_CFS(sat_num,chan,1),  &
							MO_2001_NL_CFS(sat_num,chan,2),	 &
							MO_2001_NL_CFS(sat_num,chan,3)
10							format(2x,3f20.0)
			enddo
		enddo

		MO_2001_Valid = .true.


	end subroutine get_Mo_2001_NL_Coeff




	
	subroutine msu_non_linearity_correction(channel,sat_num,version,counts_raw,counts_corr,error)

		use msu_constants
		use NAN_support

		implicit none

		integer(4)			:: channel
		integer(4)			:: sat_num
		integer(4)			:: version
		integer(2)			:: counts_raw
		real(8)				:: counts_corr
		integer(4)			:: error

		! this routine performs the correction to account for the non-linear
		! behavior of the radiometer
		!
		error = 0
		
		select case (version)
			case(MO_1995)
				if ((counts_raw .eq. 0) .or. (counts_raw .eq. MAX_COUNTS)) then
					counts_corr = NAN
					error = 1
				else
					if (.not. MO_1995_valid) call get_Mo_1995_NL_Coeff
					
					counts_corr =  &
							MO_1995_NL_CFS(sat_num,channel,1) + &
							counts_raw * (MO_1995_NL_CFS(sat_num,channel,2) + &
   					     	(MO_1995_NL_CFS(sat_num,channel,3) * counts_raw))
							
					error = 0
				endif
			case(MO_2001)
				if ((counts_raw .eq. 0) .or. (counts_raw .eq. MAX_COUNTS)) then
					counts_corr = NAN
					error = 1
				else

					if (.not. MO_2001_valid) call get_Mo_2001_NL_Coeff

					counts_corr =  &
							MO_2001_NL_CFS(sat_num,channel,1) + &
							counts_raw * (MO_2001_NL_CFS(sat_num,channel,2) + &
   					     	(MO_2001_NL_CFS(sat_num,channel,3) * counts_raw))
							
					error = 0
				endif
			case(LINEAR)
				if ((counts_raw .eq. 0) .or. (counts_raw .eq. MAX_COUNTS)) then
					counts_corr = NAN
					error = 1
				else
					counts_corr =  dble(counts_raw)
					error = 0
				endif
			case DEFAULT
				counts_corr = NAN
				error = -1
		end select

	end subroutine MSU_non_linearity_correction

	subroutine calc_non_linearity_correction(L1B_scan,L2A_scan)

		use msu_constants
		use msu_types
		use NAN_support
		

		type(msu_raw)		:: L1b_scan
		type(msu_L2A_scan)	:: L2A_scan

		integer(4)			:: chan
		integer(4)			:: fov
		integer(4)			:: error

		
		do chan = 1,4
			do fov = 1,NUM_SCAN_POSITIONS
				call msu_non_linearity_correction(chan,L2A_scan%sat_num,MO_1995,  &
				               L1B_scan%MSU_counts(chan+3,fov),					  &
							   L2A_scan%MSU_counts_corr(chan,fov),error)
			enddo
		enddo
	end subroutine calc_non_linearity_correction
				


	subroutine calc_resistances(L1B_scan,L2A_scan)

		use msu_constants
		use msu_types
		use NAN_support
		

		type(msu_raw)		:: L1b_scan
		type(msu_L2A_scan)	:: L2A_scan

		integer(4)	cal_lo_1		! # of counts for T1 cal lo
		integer(4)  cal_hi_1		! # of counts for T1 cal hi
		integer(4)  cal_lo_2		! # of counts for T2 cal lo
		integer(4)  cal_hi_2		! # of counts for T2 cal hi

		integer(4)  cts_a1			! # of counts for T1A
		integer(4)  cts_b1			! # of counts for T1B
		integer(4)  cts_a2			! # of counts for T2A
		integer(4)  cts_b2			! # of counts for T2B

		cal_lo_1 = L1B_scan%MSU_counts(CAL_LO_1_CHAN_POS,CAL_LO_1_SCAN_POS)
		cal_hi_1 = L1B_scan%MSU_counts(CAL_HI_1_CHAN_POS,CAL_HI_1_SCAN_POS)!+1
		cal_lo_2 = L1B_scan%MSU_counts(CAL_LO_2_CHAN_POS,CAL_LO_2_SCAN_POS)
		cal_hi_2 = L1B_scan%MSU_counts(CAL_HI_2_CHAN_POS,CAL_HI_2_SCAN_POS)!+1

		cts_a1   = L1B_scan%MSU_counts(A1_CHAN_POS,A1_SCAN_POS)!-1
		cts_b1   = L1B_scan%MSU_counts(B1_CHAN_POS,B1_SCAN_POS)
		cts_a2   = L1B_scan%MSU_counts(A2_CHAN_POS,A2_SCAN_POS)
		cts_b2   = L1B_scan%MSU_counts(B2_CHAN_POS,B2_SCAN_POS)



   		L2A_scan%T%R_PRT_A1 = PRT_k0(L1B_scan%sat_num,A1) +		&
					  		  PRT_k1(L1B_scan%sat_num,A1)	*	&
							  dfloat(cts_a1 - cal_lo_1)/dfloat(cal_hi_1 - cal_lo_1)

   		L2A_scan%T%R_PRT_B1 = PRT_k0(L1B_scan%sat_num,B1) +		&
							  PRT_k1(L1B_scan%sat_num,B1)	*	&
							  dfloat(cts_b1 - cal_lo_2)/dfloat(cal_hi_2 - cal_lo_2)  

   		L2A_scan%T%R_PRT_A2 = PRT_k0(L1B_scan%sat_num,A2) +		&
							  PRT_k1(L1B_scan%sat_num,A2)	*	&
							  dfloat(cts_a2 - cal_lo_1)/dfloat(cal_hi_1 - cal_lo_1)	 

   		L2A_scan%T%R_PRT_B2 = PRT_k0(L1B_scan%sat_num,B2) +		&
							  PRT_k1(L1B_scan%sat_num,B2)	*	&
							  dfloat(cts_b2 - cal_lo_2)/dfloat(cal_hi_2 - cal_lo_2)

	end subroutine calc_resistances

	subroutine calc_target_temps(L2A_scan)

		use msu_constants
		use msu_types
		use NAN_support

		type(msu_L2A_scan)	:: L2A_scan

		real(4)			:: Temp_A1
		real(4)			:: Temp_A2
		real(4)			:: Temp_B1
		real(4)			:: Temp_B2

		integer(4)		:: chan		! channel index

		logical			:: A1_ok
		logical			:: B1_ok
		logical			:: diff_ok
		logical			:: A2_ok
		logical			:: B2_ok

		! This routine calculate the temperature of the black-body calibration targets
		! on the MSUs

		! The calibration coefficients both for retrieving the resistances of the 
		! PRTs and converting these resistances to degrees K are contained in the 
		! module MSU constants


		! calculate temperatures for all 4 PRTs

		Temp_A1 =  PRT_e0(L2A_scan%sat_num,A1) + L2A_scan%T%R_PRT_A1* &
				  (PRT_e1(L2A_scan%sat_num,A1) + (PRT_e2(L2A_scan%sat_num,A1)*L2A_scan%T%R_PRT_A1)) - &
				   PRT_error(L2A_scan%sat_num,A1)

		Temp_A2 = PRT_e0(L2A_scan%sat_num,A2) + L2A_scan%T%R_PRT_A2* &
		          (PRT_e1(L2A_scan%sat_num,A2) + (PRT_e2(L2A_scan%sat_num,A2)*L2A_scan%T%R_PRT_A2)) - &
				   PRT_error(L2A_scan%sat_num,A2)
	  
		Temp_B1 = PRT_e0(L2A_scan%sat_num,B1) + L2A_scan%T%R_PRT_B1* &
		          (PRT_e1(L2A_scan%sat_num,B1) + (PRT_e2(L2A_scan%sat_num,B1)*L2A_scan%T%R_PRT_B1)) - &
				   PRT_error(L2A_scan%sat_num,B1)
	  
		Temp_B2 = PRT_e0(L2A_scan%sat_num,B2) + L2A_scan%T%R_PRT_B2* &
		          (PRT_e1(L2A_scan%sat_num,B2) + (PRT_e2(L2A_scan%sat_num,B2)*L2A_scan%T%R_PRT_B2)) - &
				   PRT_error(L2A_scan%sat_num,B2)

		L2A_scan%T%Target1_Temp_A = Temp_A1
		L2A_scan%T%Target1_Temp_B = Temp_B1
		L2A_scan%T%Target2_Temp_A = Temp_A2
		L2A_scan%T%Target2_Temp_B = Temp_B2
 
		! compare temperature against resonableness bounds, and to the other temperature on the
		! same Target

		A1_ok = ((Temp_A1 .le. UPPER_TARGET_TEMP_LIMIT) .and. &
				 (Temp_A1 .ge. LOWER_TARGET_TEMP_LIMIT)) 

		B1_ok =  ((Temp_B1 .le. UPPER_TARGET_TEMP_LIMIT) .and. &
				  (Temp_B1 .ge. LOWER_TARGET_TEMP_LIMIT)) 

		diff_ok = (abs(Temp_A1 - Temp_B1) .le. TARGET_TEMP_DIFF_LIMIT)
 
		L2A_scan%T%Target1_temp = NAN

		if ((A1_ok) .and. (B1_ok) .and. (diff_ok)) then
			L2A_scan%T%Target1_temp = (Temp_A1 + Temp_B1)/2.0
		endif

		if ((A1_ok) .and. (.not. B1_ok)) then
			L2A_scan%T%Target1_temp = Temp_A1
		endif 

		if((.not. A1_ok) .and. (B1_ok))then
			L2A_scan%T%target1_temp = Temp_B1
		endif

		if (.not. finite(L2A_scan%T%target1_temp)) then
			do chan = 1,2
!				L2A_scan%scan_qual(chan) =  ibset(L2A_scan%scan_qual(chan),3)
!				Num_Bad_Target_Temp(chan) = Num_Bad_Target_Temp(chan) + 1
			enddo
		endif					

		A2_ok = ((Temp_A2 .le. UPPER_TARGET_TEMP_LIMIT) .and. &
				 (Temp_A2 .ge. LOWER_TARGET_TEMP_LIMIT)) 

		B2_ok =  ((Temp_B2 .le. UPPER_TARGET_TEMP_LIMIT) .and. &
				  (Temp_B2 .ge. LOWER_TARGET_TEMP_LIMIT)) 

		diff_ok = (abs(Temp_A2 - Temp_B2) .le. TARGET_TEMP_DIFF_LIMIT)
 
		L2A_scan%T%Target2_temp = NAN

		if ((A2_ok) .and. (B2_ok) .and. (diff_ok)) then
			L2A_scan%T%Target2_temp = (Temp_A2 + Temp_B2)/2.0
		endif

		if ((A2_ok) .and. (.not. B2_ok)) then
			L2A_scan%T%Target2_temp = Temp_A2
		endif 

		if((.not. A2_ok) .and. (B2_ok))then
			L2A_scan%T%target2_temp = B2
		endif

		if (.not. finite(L2A_scan%T%target2_temp)) then
			do chan = 3,4

!			L2A_scan%scan_qual(chan) =  ibset(L2A_scan%scan_qual(chan),3)
!				Num_Bad_Target_Temp(chan) = Num_Bad_Target_Temp(chan) + 1
			enddo
		endif

		return

	end subroutine calc_target_temps

	subroutine calc_raw_TBs(L2A_scan)

		use msu_constants
		use msu_types
		use NAN_support
		use error_statistics

		implicit none

		type(msu_L2A_Scan)		:: L2A_scan

		integer(4)				:: fov		! field of view index
		integer(4)				:: chan		! channel index

		real(8)				:: T_Target
		real(8)				:: space_cts	! # of counts when viewing deep space
		real(8)				:: target_cts	! # of counts when viewing warm target
		real(8)				:: fov_cts		! # of counts when viewing earth
		real(8)				:: Tb_temp      ! brightness temperature 
		logical				:: all_Tbs_ok   ! flag set true if all 11 brightness temperature
											! are within bounds


		! This routine used the radiometer counts, the calibration Coeffs,
		! and the load temperatures to find the raw brightness temperatures

		! At this point, the correction for non-linearity has been performed,
		! but not the orbital decay correction, the diurnal correction, or the
		! corrections due to satellite body temperature

		! Calculate Tbs

		do chan = 1,4

			if (chan .le. 2) then
				T_target = L2A_scan%T%Target1_temp
			else
				T_target = L2A_scan%T%Target2_temp
			endif


			all_Tbs_ok = .true.

			target_cts = L2A_scan%MSU_counts_corr(chan,BLACK_BODY)
			space_cts  = L2A_scan%MSU_counts_corr(chan,SPACE)

			do fov = 1,NUM_FOVS 
				fov_cts		= L2A_scan%MSU_counts_corr(chan,fov)
				Tb_temp		= T_SPACE + (T_Target - T_SPACE)* &
							  dble(fov_cts - space_cts)/dble(target_cts - space_cts)

				if ((Tb_temp .le. MAX_REASONABLE_TB(chan)) .and. (Tb_temp .ge. MIN_REASONABLE_TB(chan))) then
					L2A_scan%MSU_Tbs_raw(chan,fov) = Tb_temp
				else
					L2A_scan%MSU_Tbs_raw(chan,fov) = NAN
					all_Tbs_OK = .false.			
				endif		
			enddo
		enddo
		return

	end subroutine calc_raw_TBs


end module MSU_calibration