$freeform
! @***h* MLT_RSS/msu_date.f90
!  NAME
!    module msu_date
! 
!  PURPOSE
!    module containing MSU-specific date routines 
! 
!  DESCRIPTION
!     module containing MSU-specific date routines 
!    
!  ROUTINES
!      convert_date(scan)
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use msu_date
! 
! @*****
module msu_date

use date_and_time
use msu_types

implicit none

contains

subroutine convert_date(scan)
!@***h* MLT_RSS/msu_date.f90/convert_date
!  NAME
!    module convert_date
! 
!  PURPOSE
!    converts raw MSU data to type date_str
! 
!  DESCRIPTION
!     converts raw MSU data to type date_str
!    
!  INPUT
!     scan    = type(msu_scan)
!
!  OUTPUT
!    scan    = type(msu_scan)
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    call convert_date(scan)
! 
!@*****
	
	implicit none

	type(msu_scan),intent(INOUT)		:: scan
	integer(4)							:: error
	integer(4)							:: chan

		scan%der%date = set_date(dble(scan%raw%secdy),int(scan%raw%jday),int(scan%raw%lyear),error)

		if (error .ne. 0) then 
			print *,'Error converting date to date type'
			print *,dble(scan%raw%secdy),scan%raw%jday,scan%raw%lyear
			do chan = 1,NUM_CHANNELS
				scan%der%scan_qual(chan) = ibset(scan%der%scan_qual(chan),0)
			enddo
		endif
	end subroutine convert_date
end module msu_date
