$freeform
module type_conversion
!@***m* MLT_RSS/type_conversion.f90
!  NAME
!    module NAN_Support
! 
!  PURPOSE
!    module containing a number of routines that convert between various fortran types.
! 
!  DESCRIPTION
!    module containing a number of routines that convert between various fortran types.
!    it is somewhat mind-boggling that these are not intrinsic functions, at least for
!    our compiler.  Used to read and write flat binary files.
!    
!  ROUTINES
!      integer(1) function char_to_int_1(a)
!      character(len = 1) function int_1_to_char(i)
!      integer(2) function	char_arr_to_int_2(a)
!      integer(2) function string_to_int_2(a)
!      integer(2) function string_to_int_2(a)
!      character(len=2) function int_2_to_string(i)
!      integer(4) function string_to_int_4(a)
!      integer(4) function string_to_int_4(a)
!      real(4) function char_arr_to_real_4(a)
!      real(4) function string_to_real_4(a)
!      character(len=4) function real_4_to_string(x)
!      real(8) function string_to_real_8(a)
!      real(8) function char_arr_to_real_8(a)
!      character(len=8) function real_8_to_string(x)
! 
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    2/21/2012 Initial Version prepared for NCDC
!  
!  USAGE
!    use type_conversion
! 
!@*****
	 implicit none

contains

	integer(1) function char_to_int_1(a)
		
		character(len=1)	:: a
		character(len=1)	:: a_temp
		integer(1)						:: i

		equivalence(a_temp,i)

		a_temp = a
		char_to_int_1 = i

	end function char_to_int_1

	character(len = 1) function int_1_to_char(i)

		integer(1)			:: i
		
		character(len=1)	:: a
		integer(1)			:: i_temp

		equivalence(i_temp,a)
        i_temp = i
		int_1_to_char = a

	end function int_1_to_char


	
	integer(2) function	char_arr_to_int_2(a)
		
		character(len=1),dimension(2)	:: a
		character(len=1),dimension(2)	:: a_temp
		integer(2)						:: i

		equivalence(a_temp(1),i)

		a_temp = a
		char_arr_to_int_2 = i

	end function char_arr_to_int_2

	integer(2) function string_to_int_2(a)
		
		character(len=2)	:: a
		character(len=2)	:: a_temp
		integer(2)						:: i

		equivalence(a_temp,i)

		a_temp = a
		string_to_int_2 = i

	end function string_to_int_2

	character(len=2) function int_2_to_string(i)
		
		character(len=2)	:: a
		integer(2)		    :: i,i_temp

		equivalence(i_temp,a)

		i_temp = i
		int_2_to_string = a

	end function int_2_to_string

	integer(4) function	char_arr_to_int_4(a)
		
		character(len=1),dimension(4)	:: a
		character(len=1),dimension(4)	:: a_temp
		integer(4)						:: i

		equivalence(a_temp(1),i)

		a_temp = a
		char_arr_to_int_4 = i

	end function char_arr_to_int_4

	integer(4) function string_to_int_4(a)
		
		character(len=4)	:: a
		character(len=4)	:: a_temp
		integer(4)						:: i

		equivalence(a_temp,i)

		a_temp = a
		string_to_int_4 = i

	end function string_to_int_4

	character(len=4) function int_4_to_string(i)
		
		character(len=4)	:: a
		integer(4)		    :: i,i_temp

		equivalence(i_temp,a)

		i_temp = i
		int_4_to_string = a

	end function int_4_to_string


	real(4) function char_arr_to_real_4(a)
		
		character(len=1),dimension(4)	:: a
		character(len=1),dimension(4)	:: a_temp
		real(4)							:: x

		equivalence(a_temp(1),x)

		a_temp = a
		char_arr_to_real_4 = x

	end function char_arr_to_real_4

	real(4) function string_to_real_4(a)
		
		character(len=4)	:: a
		character(len=4)	:: a_temp
		real(4)				:: x

		equivalence(a_temp,x)

		a_temp = a
		string_to_real_4 = x

	end function string_to_real_4

	character(len=4) function real_4_to_string(x)
		
		character(len=4)	:: a
		real(4)				:: x,x_temp

		equivalence(x_temp,a)

		x_temp = x
		real_4_to_string = a

	end function real_4_to_string



	real(8) function char_arr_to_real_8(a)
		
		character(len=1),dimension(8)	:: a
		character(len=1),dimension(8)	:: a_temp
		real(8)							:: x

		equivalence(a_temp(1),x)

		a_temp = a
		char_arr_to_real_8 = x

	end function char_arr_to_real_8

	real(8) function string_to_real_8(a)
		
		character(len=8)	:: a
		character(len=8)	:: a_temp
		real(8)				:: x

		equivalence(a_temp,x)

		a_temp = a
		string_to_real_8 = x

	end function string_to_real_8

	character(len=8) function real_8_to_string(x)
		
		character(len=8)	:: a
		real(8)				:: x,x_temp

		equivalence(x_temp,a)

		x_temp = x
		real_8_to_string = a

	end function real_8_to_string
end module type_conversion
