function aqua_amsu_file,year,month,day,granule_num,error
;@***f* MLT_RSS/aqua_amsu_file.pro
; NAME
;   aqua_amsu_file
;
; PURPOSE
;   returns the name of the file corresponding to the date and granule number
;
; DESCRIPTION
;   returns the name of the file corresponding to the date and granule number
;
; INPUTS:
;   year      ; year to read
;   month     ; month to read
;   day       ; day in month to read
;   granule_num   granaule number to read
;   
; OUTPUTS
;   error   ; error flag, 0 if everything OK
;   
; RETURN VALUE:
;   NASA L1B aqua_AMSU file name
;
; AUTHOR:
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT:
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   11/10/2012 Initial Version prepared for NCDC
;
; USAGE
;   file = aqua_amsu_file(year,month,day,granule_num,error)
;
;@*****
error = 0
; returns the name of the file corresponding to the date and granule number

jan1 = JULDAY(1, 1, year)
this_day = julday(month,day,year)
day_of_year = 1+this_day-jan1


aqua_l1a_path = 'AQUA_V05\'
search_path = aqua_l1a_path+string(year,format='(i4,"\")')+ $
							string(day_of_year,format='(i3.3,"\")') + $
							string(year,month,day,granule_num, $
							format='("AIRS.",i4.4,".",i2.2,".",i2.2,".",i3.3,".L1B.AMSU_Rad*.hdf")')
file = file_search(search_path)
if ((n_elements(file) eq 1) and (strlen(file[0]) gt 10)) then begin
	error = 0
	return,(file[0])
endif else begin
	error = -1
	return,' '
endelse

end