pro orbitify_aqua_v5,days_before=days_before

;@***h* MLT_RSS/orbitify_aqua_v5.pro
;
; NAME
;   orbitify_aqua_v5.pro
;
; PURPOSE
;   Top level driver program for ingesting AMSU on AQUA data
;
; DESCRIPTION
;   Top level driver program for ingesting AMSU on AQUA data.  Reads AQUA files (in hdf_eos)
;   and combines data into complete orbits
;
; FILE READ:
;   AQUA L2A files in hdf_eos format
;   e.g. AIRS.2011.12.31.001.L1B.AMSU_Rad.v5.0.0.0.G11365123057.hdf
;
; FILE WRITTEN:
;   AQUA L2A files in RSS format, as complete orbits.
;   e.g. NOAA-18_r30000.L2A.gz
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/24/2012 Initial Version prepared for NCDC
;
; USAGE
;   orbitify_aqua_v5,days_before=5
;
;
;@*****

  compile_opt IDL2
  
	status_root = '\\Athena\MSU\Status\'
	log_root = 'M:\msu_amsu_data\AMSU-A_L1A\AQUA_AMSU_orbit_logs\'

	max_orbits = 60000L

	common aqua1,scan_last,active_orbit
	common aqua2,time_scan1
	
	if keyword_set(days_before) eq 0 then days_before = 3

	time_scan1 = replicate(!values.d_nan,max_orbits)

	defsysv,'!log_lun',4s
	close,!log_lun
	defsysv,'!stat_lun',5s
	close,!stat_lun

	for channel_index = 0,10 do begin
		channel = 4+channel_index
		log_fyl = string(log_root,channel,format='(a,"channel_",i2.2,"_log.txt")')
		openw,!log_lun,log_fyl
		read_orbitify_status_file,status_root,channel,status
		ok = convert_date(date_in = status.last_rd_2000_processed,from = 'rd_2000',to= 'julday',date_out = last_julday_processed)
		current_julday = systime(/julian,/UTC)
    
		first_day = floor(current_julday) - days_before
		last_day = floor(current_julday) 

    ;first_day = julday(1,1,2011,0,0,0) + 327
    ;last_day =  julday(1,1,2011,0,0,0) + 327
		mode = 'APPEND'
		for jday = first_day,last_day do begin
			caldat,jday,month,day,year
			print,string(month,day,year,format='("Processing ",i2.2,"/",i2.2,"/",i4)')
			active_orbit = -999
			scan_last = -999
			orbitify_aqua_data,year,month,day,channel,mode,num_missing_files,num_read_errors,last_rd_2000 = last_rd_2000,last_orbit=last_orbit
			log,'Day Summary ',year,month,day,channel,num_missing_files,num_read_errors,last_rd_2000,last_orbit
		endfor

		current_julday = systime(/julian,/UTC)
		status.last_time_run = date_str_from_julday(current_julday)
		status.last_rd_2000_processed = last_rd_2000
		status.last_date_processed = date_str_from_rd_2000(last_rd_2000)
		status.last_orbit_processed = last_orbit

		write_orbitify_status_file,status_root,channel,status
    write_orbitify_status_file,'M:\MSU_AMSU_Data\AMSU_Status\',channel,status
		close,!log_lun
	endfor

end