pro read_AIRS_AQUA_L1B, file, x, error
;@***h* MLT_RSS/read_AIRS_AQUA_L1B.pro

; NAME
;   read_AIRS_AQUA_L1B.pro
;
; PURPOSE
;   Reads NASA AQUA AIRS L1B file in hdfeos format
;
; DESCRIPTION
;   Reads NASA AQUA AIRS L1B file in hdfeos format.  Data is returned in an IDL structure.
;
; INPUTS
;   file  ; file to read
;
; OUTPUTS
;   x     ; L1B data in an IDL structure
;   error ; error flag.  0 is all is OK
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_AIRS_AQUA_L1B, file, x, error
;
;@*****
		error = 0
		fileID = EOS_SW_OPEN(file, /READ)
		swathID = EOS_SW_ATTACH(fileID, 'L1B_AMSU')
		Result = 0
		Result = Result + EOS_SW_READATTR(swathID,'SWATH_TITLE= ',SWATH_TITLE__)
		Result = Result + EOS_SW_READATTR(swathID,'processing_level',processing_level)
		Result = Result + EOS_SW_READATTR(swathID,'instrument',instrument)
		Result = Result + EOS_SW_READATTR(swathID,'DayNightFlag',DayNightFlag)
		Result = Result + EOS_SW_READATTR(swathID,'AutomaticQAFlag',AutomaticQAFlag)
		Result = Result + EOS_SW_READATTR(swathID,'NumTotalData',NumTotalData)
		Result = Result + EOS_SW_READATTR(swathID,'NumProcessData',NumProcessData)
		Result = Result + EOS_SW_READATTR(swathID,'NumSpecialData',NumSpecialData)
		Result = Result + EOS_SW_READATTR(swathID,'NumBadData',NumBadData)
		Result = Result + EOS_SW_READATTR(swathID,'NumMissingData',NumMissingData)
		Result = Result + EOS_SW_READATTR(swathID,'NumLandSurface',NumLandSurface)
		Result = Result + EOS_SW_READATTR(swathID,'NumOceanSurface',NumOceanSurface)
		Result = Result + EOS_SW_READATTR(swathID,'node_type',node_type)
		Result = Result + EOS_SW_READATTR(swathID,'start_year',start_year)
		Result = Result + EOS_SW_READATTR(swathID,'start_month',start_month)
		Result = Result + EOS_SW_READATTR(swathID,'start_day',start_day)
		Result = Result + EOS_SW_READATTR(swathID,'start_hour',start_hour)
		Result = Result + EOS_SW_READATTR(swathID,'start_minute',start_minute)
		Result = Result + EOS_SW_READATTR(swathID,'start_sec',start_sec)
		Result = Result + EOS_SW_READATTR(swathID,'start_orbit',start_orbit)
		Result = Result + EOS_SW_READATTR(swathID,'end_orbit',end_orbit)
		Result = Result + EOS_SW_READATTR(swathID,'orbit_path',orbit_path)
		Result = Result + EOS_SW_READATTR(swathID,'start_orbit_row',start_orbit_row)
		Result = Result + EOS_SW_READATTR(swathID,'end_orbit_row',end_orbit_row)
		Result = Result + EOS_SW_READATTR(swathID,'granule_number',granule_number)
		Result = Result + EOS_SW_READATTR(swathID,'num_scansets',num_scansets)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines',num_scanlines)
		Result = Result + EOS_SW_READATTR(swathID,'start_Latitude',start_Latitude)
		Result = Result + EOS_SW_READATTR(swathID,'start_Longitude',start_Longitude)
		Result = Result + EOS_SW_READATTR(swathID,'start_Time',start_Time)
		Result = Result + EOS_SW_READATTR(swathID,'end_Latitude',end_Latitude)
		Result = Result + EOS_SW_READATTR(swathID,'end_Longitude',end_Longitude)
		Result = Result + EOS_SW_READATTR(swathID,'end_Time',end_Time)
		Result = Result + EOS_SW_READATTR(swathID,'eq_x_longitude',eq_x_longitude)
		Result = Result + EOS_SW_READATTR(swathID,'eq_x_tai',eq_x_tai)
		Result = Result + EOS_SW_READATTR(swathID,'orbitgeoqa',orbitgeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_satgeoqa',num_satgeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_glintgeoqa',num_glintgeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_moongeoqa',num_moongeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_ftptgeoqa',num_ftptgeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_zengeoqa',num_zengeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_demgeoqa',num_demgeoqa)
		Result = Result + EOS_SW_READATTR(swathID,'num_fpe',num_fpe)
		Result = Result + EOS_SW_READATTR(swathID,'LonGranuleCen',LonGranuleCen)
		Result = Result + EOS_SW_READATTR(swathID,'LatGranuleCen',LatGranuleCen)
		Result = Result + EOS_SW_READATTR(swathID,'LocTimeGranuleCen',LocTimeGranuleCen)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_not_norm_mode_a1',num_scanlines_not_norm_mode_a1)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_not_norm_mode_a2',num_scanlines_not_norm_mode_a2)
		Result = Result + EOS_SW_READATTR(swathID,'num_missing_scanlines_a1',num_missing_scanlines_a1)
		Result = Result + EOS_SW_READATTR(swathID,'num_missing_scanlines_a2',num_missing_scanlines_a2)
		Result = Result + EOS_SW_READATTR(swathID,'num_data_gaps_a1',num_data_gaps_a1)
		Result = Result + EOS_SW_READATTR(swathID,'num_data_gaps_a2',num_data_gaps_a2)
		Result = Result + EOS_SW_READATTR(swathID,'num_instr_mode_changes_a1',num_instr_mode_changes_a1)
		Result = Result + EOS_SW_READATTR(swathID,'num_instr_mode_changes_a2',num_instr_mode_changes_a2)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_rec_cal_prob_a11',num_scanlines_rec_cal_prob_a11)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_rec_cal_prob_a12',num_scanlines_rec_cal_prob_a12)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_rec_cal_prob_a2',num_scanlines_rec_cal_prob_a2)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_sig_coast_xing',num_scanlines_sig_coast_xing)
		Result = Result + EOS_SW_READATTR(swathID,'num_scanlines_sig_sun_glint',num_scanlines_sig_sun_glint)
		Result = Result + EOS_SW_READATTR(swathID,'MoonInViewMWCount',MoonInViewMWCount)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.min',QA_bb_PRT_a11_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.max',QA_bb_PRT_a11_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.mean',QA_bb_PRT_a11_mean)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.dev',QA_bb_PRT_a11_dev)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.num_in',QA_bb_PRT_a11_num_in)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.num_lo',QA_bb_PRT_a11_num_lo)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.num_hi',QA_bb_PRT_a11_num_hi)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.num_bad',QA_bb_PRT_a11_num_bad)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.range_min',QA_bb_PRT_a11_range_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.range_max',QA_bb_PRT_a11_range_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.missing',QA_bb_PRT_a11_missing)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.max_track',QA_bb_PRT_a11_max_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.max_xtrack',QA_bb_PRT_a11_max_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.min_track',QA_bb_PRT_a11_min_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a11.min_xtrack',QA_bb_PRT_a11_min_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.min',QA_bb_PRT_a12_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.max',QA_bb_PRT_a12_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.mean',QA_bb_PRT_a12_mean)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.dev',QA_bb_PRT_a12_dev)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.num_in',QA_bb_PRT_a12_num_in)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.num_lo',QA_bb_PRT_a12_num_lo)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.num_hi',QA_bb_PRT_a12_num_hi)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.num_bad',QA_bb_PRT_a12_num_bad)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.range_min',QA_bb_PRT_a12_range_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.range_max',QA_bb_PRT_a12_range_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.missing',QA_bb_PRT_a12_missing)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.max_track',QA_bb_PRT_a12_max_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.max_xtrack',QA_bb_PRT_a12_max_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.min_track',QA_bb_PRT_a12_min_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a12.min_xtrack',QA_bb_PRT_a12_min_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.min',QA_bb_PRT_a2_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.max',QA_bb_PRT_a2_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.mean',QA_bb_PRT_a2_mean)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.dev',QA_bb_PRT_a2_dev)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.num_in',QA_bb_PRT_a2_num_in)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.num_lo',QA_bb_PRT_a2_num_lo)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.num_hi',QA_bb_PRT_a2_num_hi)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.num_bad',QA_bb_PRT_a2_num_bad)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.range_min',QA_bb_PRT_a2_range_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.range_max',QA_bb_PRT_a2_range_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.missing',QA_bb_PRT_a2_missing)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.max_track',QA_bb_PRT_a2_max_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.max_xtrack',QA_bb_PRT_a2_max_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.min_track',QA_bb_PRT_a2_min_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_bb_PRT_a2.min_xtrack',QA_bb_PRT_a2_min_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.min',QA_rec_PRT_a11_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.max',QA_rec_PRT_a11_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.mean',QA_rec_PRT_a11_mean)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.dev',QA_rec_PRT_a11_dev)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.num_in',QA_rec_PRT_a11_num_in)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.num_lo',QA_rec_PRT_a11_num_lo)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.num_hi',QA_rec_PRT_a11_num_hi)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.num_bad',QA_rec_PRT_a11_num_bad)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.range_min',QA_rec_PRT_a11_range_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.range_max',QA_rec_PRT_a11_range_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.missing',QA_rec_PRT_a11_missing)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.max_track',QA_rec_PRT_a11_max_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.max_xtrack',QA_rec_PRT_a11_max_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.min_track',QA_rec_PRT_a11_min_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a11.min_xtrack',QA_rec_PRT_a11_min_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.min',QA_rec_PRT_a12_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.max',QA_rec_PRT_a12_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.mean',QA_rec_PRT_a12_mean)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.dev',QA_rec_PRT_a12_dev)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.num_in',QA_rec_PRT_a12_num_in)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.num_lo',QA_rec_PRT_a12_num_lo)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.num_hi',QA_rec_PRT_a12_num_hi)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.num_bad',QA_rec_PRT_a12_num_bad)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.range_min',QA_rec_PRT_a12_range_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.range_max',QA_rec_PRT_a12_range_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.missing',QA_rec_PRT_a12_missing)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.max_track',QA_rec_PRT_a12_max_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.max_xtrack',QA_rec_PRT_a12_max_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.min_track',QA_rec_PRT_a12_min_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a12.min_xtrack',QA_rec_PRT_a12_min_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.min',QA_rec_PRT_a2_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.max',QA_rec_PRT_a2_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.mean',QA_rec_PRT_a2_mean)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.dev',QA_rec_PRT_a2_dev)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.num_in',QA_rec_PRT_a2_num_in)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.num_lo',QA_rec_PRT_a2_num_lo)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.num_hi',QA_rec_PRT_a2_num_hi)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.num_bad',QA_rec_PRT_a2_num_bad)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.range_min',QA_rec_PRT_a2_range_min)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.range_max',QA_rec_PRT_a2_range_max)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.missing',QA_rec_PRT_a2_missing)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.max_track',QA_rec_PRT_a2_max_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.max_xtrack',QA_rec_PRT_a2_max_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.min_track',QA_rec_PRT_a2_min_track)
		Result = Result + EOS_SW_READATTR(swathID,'QA_rec_PRT_a2.min_xtrack',QA_rec_PRT_a2_min_xtrack)
		Result = Result + EOS_SW_READATTR(swathID,'granules_present',granules_present)
		Result = Result + EOS_SW_READFIELD(swathID,'scanang',scanang)
		Result = Result + EOS_SW_READFIELD(swathID,'satheight',satheight)
		Result = Result + EOS_SW_READFIELD(swathID,'satroll',satroll)
		Result = Result + EOS_SW_READFIELD(swathID,'satpitch',satpitch)
		Result = Result + EOS_SW_READFIELD(swathID,'satyaw',satyaw)
		Result = Result + EOS_SW_READFIELD(swathID,'satgeoqa',satgeoqa)
		Result = Result + EOS_SW_READFIELD(swathID,'glintgeoqa',glintgeoqa)
		Result = Result + EOS_SW_READFIELD(swathID,'moongeoqa',moongeoqa)
		Result = Result + EOS_SW_READFIELD(swathID,'ftptgeoqa',ftptgeoqa)
		Result = Result + EOS_SW_READFIELD(swathID,'zengeoqa',zengeoqa)
		Result = Result + EOS_SW_READFIELD(swathID,'demgeoqa',demgeoqa)
		Result = Result + EOS_SW_READFIELD(swathID,'nadirTAI',nadirTAI)
		Result = Result + EOS_SW_READFIELD(swathID,'sat_lat',sat_lat)
		Result = Result + EOS_SW_READFIELD(swathID,'sat_lon',sat_lon)
		Result = Result + EOS_SW_READFIELD(swathID,'scan_node_type',scan_node_type)
		Result = Result + EOS_SW_READFIELD(swathID,'satzen',satzen)
		Result = Result + EOS_SW_READFIELD(swathID,'satazi',satazi)
		Result = Result + EOS_SW_READFIELD(swathID,'solzen',solzen)
		Result = Result + EOS_SW_READFIELD(swathID,'solazi',solazi)
		Result = Result + EOS_SW_READFIELD(swathID,'glintlat',glintlat)
		Result = Result + EOS_SW_READFIELD(swathID,'glintlon',glintlon)
		Result = Result + EOS_SW_READFIELD(swathID,'sun_glint_distance',sun_glint_distance)
		Result = Result + EOS_SW_READFIELD(swathID,'topog',topog)
		Result = Result + EOS_SW_READFIELD(swathID,'topog_err',topog_err)
		Result = Result + EOS_SW_READFIELD(swathID,'landFrac',landFrac)
		Result = Result + EOS_SW_READFIELD(swathID,'landFrac_err',landFrac_err)
		Result = Result + EOS_SW_READFIELD(swathID,'antenna_temp',antenna_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'brightness_temp',brightness_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'brightness_temp_err',brightness_temp_err)
		Result = Result + EOS_SW_READFIELD(swathID,'center_freq',center_freq)
		Result = Result + EOS_SW_READFIELD(swathID,'IF_offset_1',IF_offset_1)
		Result = Result + EOS_SW_READFIELD(swathID,'IF_offset_2',IF_offset_2)
		Result = Result + EOS_SW_READFIELD(swathID,'bandwidth',bandwidth)
		Result = Result + EOS_SW_READFIELD(swathID,'num_calibrated_scanlines',num_calibrated_scanlines)
		Result = Result + EOS_SW_READFIELD(swathID,'num_scanlines_ch_cal_problems',num_scanlines_ch_cal_problems)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.min',QA_unfiltered_scene_count_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.max',QA_unfiltered_scene_count_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.mean',QA_unfiltered_scene_count_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.dev',QA_unfiltered_scene_count_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.num',QA_unfiltered_scene_count_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.num_bad',QA_unfiltered_scene_count_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.max_track',QA_unfiltered_scene_count_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.max_xtrack',QA_unfiltered_scene_count_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.min_track',QA_unfiltered_scene_count_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_scene_count.min_xtrack',QA_unfiltered_scene_count_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.min',QA_unfiltered_BB_count_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.max',QA_unfiltered_BB_count_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.mean',QA_unfiltered_BB_count_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.dev',QA_unfiltered_BB_count_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.num',QA_unfiltered_BB_count_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.num_bad',QA_unfiltered_BB_count_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.max_track',QA_unfiltered_BB_count_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.max_xtrack',QA_unfiltered_BB_count_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.min_track',QA_unfiltered_BB_count_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_BB_count.min_xtrack',QA_unfiltered_BB_count_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.min',QA_unfiltered_space_count_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.max',QA_unfiltered_space_count_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.mean',QA_unfiltered_space_count_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.dev',QA_unfiltered_space_count_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.num',QA_unfiltered_space_count_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.num_bad',QA_unfiltered_space_count_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.max_track',QA_unfiltered_space_count_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.max_xtrack',QA_unfiltered_space_count_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.min_track',QA_unfiltered_space_count_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_unfiltered_space_count.min_xtrack',QA_unfiltered_space_count_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.min',QA_cal_coef_a0_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.max',QA_cal_coef_a0_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.mean',QA_cal_coef_a0_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.dev',QA_cal_coef_a0_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.num',QA_cal_coef_a0_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.num_bad',QA_cal_coef_a0_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.max_track',QA_cal_coef_a0_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.max_xtrack',QA_cal_coef_a0_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.min_track',QA_cal_coef_a0_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a0.min_xtrack',QA_cal_coef_a0_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.min',QA_cal_coef_a1_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.max',QA_cal_coef_a1_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.mean',QA_cal_coef_a1_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.dev',QA_cal_coef_a1_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.num',QA_cal_coef_a1_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.num_bad',QA_cal_coef_a1_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.max_track',QA_cal_coef_a1_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.max_xtrack',QA_cal_coef_a1_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.min_track',QA_cal_coef_a1_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a1.min_xtrack',QA_cal_coef_a1_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.min',QA_cal_coef_a2_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.max',QA_cal_coef_a2_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.mean',QA_cal_coef_a2_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.dev',QA_cal_coef_a2_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.num',QA_cal_coef_a2_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.num_bad',QA_cal_coef_a2_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.max_track',QA_cal_coef_a2_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.max_xtrack',QA_cal_coef_a2_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.min_track',QA_cal_coef_a2_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_cal_coef_a2.min_xtrack',QA_cal_coef_a2_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.min',QA_bb_raw_noise_counts_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.max',QA_bb_raw_noise_counts_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.mean',QA_bb_raw_noise_counts_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.dev',QA_bb_raw_noise_counts_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.num',QA_bb_raw_noise_counts_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.num_bad',QA_bb_raw_noise_counts_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.max_track',QA_bb_raw_noise_counts_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.max_xtrack',QA_bb_raw_noise_counts_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.min_track',QA_bb_raw_noise_counts_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_bb_raw_noise_counts.min_xtrack',QA_bb_raw_noise_counts_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.min',QA_sv_raw_noise_counts_min)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.max',QA_sv_raw_noise_counts_max)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.mean',QA_sv_raw_noise_counts_mean)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.dev',QA_sv_raw_noise_counts_dev)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.num',QA_sv_raw_noise_counts_num)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.num_bad',QA_sv_raw_noise_counts_num_bad)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.max_track',QA_sv_raw_noise_counts_max_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.max_xtrack',QA_sv_raw_noise_counts_max_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.min_track',QA_sv_raw_noise_counts_min_track)
		Result = Result + EOS_SW_READFIELD(swathID,'QA_sv_raw_noise_counts.min_xtrack',QA_sv_raw_noise_counts_min_xtrack)
		Result = Result + EOS_SW_READFIELD(swathID,'state1',state1)
		Result = Result + EOS_SW_READFIELD(swathID,'state2',state2)
		Result = Result + EOS_SW_READFIELD(swathID,'cal_coef_a0',cal_coef_a0)
		Result = Result + EOS_SW_READFIELD(swathID,'cal_coef_a1',cal_coef_a1)
		Result = Result + EOS_SW_READFIELD(swathID,'cal_coef_a2',cal_coef_a2)
		Result = Result + EOS_SW_READFIELD(swathID,'cal_coef_a0_err',cal_coef_a0_err)
		Result = Result + EOS_SW_READFIELD(swathID,'cal_coef_a1_err',cal_coef_a1_err)
		Result = Result + EOS_SW_READFIELD(swathID,'cal_coef_a2_err',cal_coef_a2_err)
		Result = Result + EOS_SW_READFIELD(swathID,'a1_ColdCalPstion',a1_ColdCalPstion)
		Result = Result + EOS_SW_READFIELD(swathID,'a2_ColdCalPstion',a2_ColdCalPstion)
		Result = Result + EOS_SW_READFIELD(swathID,'a1_PLO_Redundncy',a1_PLO_Redundncy)
		Result = Result + EOS_SW_READFIELD(swathID,'a11_mux_temp_used',a11_mux_temp_used)
		Result = Result + EOS_SW_READFIELD(swathID,'a11_receiver_temp',a11_receiver_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'a11_target_temp',a11_target_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'a12_mux_temp_used',a12_mux_temp_used)
		Result = Result + EOS_SW_READFIELD(swathID,'a12_receiver_temp',a12_receiver_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'a12_target_temp',a12_target_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'a2_diplexer_temp_used',a2_diplexer_temp_used)
		Result = Result + EOS_SW_READFIELD(swathID,'a2_receiver_temp',a2_receiver_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'a2_target_temp',a2_target_temp)
		Result = Result + EOS_SW_READFIELD(swathID,'qa_scanline',qa_scanline)
		Result = Result + EOS_SW_READFIELD(swathID,'qa_receiver_a11',qa_receiver_a11)
		Result = Result + EOS_SW_READFIELD(swathID,'qa_receiver_a12',qa_receiver_a12)
		Result = Result + EOS_SW_READFIELD(swathID,'qa_receiver_a2',qa_receiver_a2)
		Result = Result + EOS_SW_READFIELD(swathID,'qa_channel',qa_channel)
		Result = Result + EOS_SW_READFIELD(swathID,'Latitude',Latitude)
		Result = Result + EOS_SW_READFIELD(swathID,'Longitude',Longitude)
		Result = Result + EOS_SW_READFIELD(swathID,'Time',Time)

		if result ne 0 then error = -1

		Detach_result = EOS_SW_DETACH(swathID)
		Close_result  = EOS_SW_CLOSE(fileID)

		if error eq 0 then begin

			attr = {$
				SWATH_TITLE__:SWATH_TITLE__,  $
				processing_level:processing_level,  $
				instrument:instrument,  $
				DayNightFlag:DayNightFlag,  $
				AutomaticQAFlag:AutomaticQAFlag,  $
				NumTotalData:NumTotalData,  $
				NumProcessData:NumProcessData,  $
				NumSpecialData:NumSpecialData,  $
				NumBadData:NumBadData,  $
				NumMissingData:NumMissingData,  $
				NumLandSurface:NumLandSurface,  $
				NumOceanSurface:NumOceanSurface,  $
				node_type:node_type,  $
				start_year:start_year,  $
				start_month:start_month,  $
				start_day:start_day,  $
				start_hour:start_hour,  $
				start_minute:start_minute,  $
				start_sec:start_sec,  $
				start_orbit:start_orbit,  $
				end_orbit:end_orbit,  $
				orbit_path:orbit_path,  $
				start_orbit_row:start_orbit_row,  $
				end_orbit_row:end_orbit_row,  $
				granule_number:granule_number,  $
				num_scansets:num_scansets,  $
				num_scanlines:num_scanlines,  $
				start_Latitude:start_Latitude,  $
				start_Longitude:start_Longitude,  $
				start_Time:start_Time,  $
				end_Latitude:end_Latitude,  $
				end_Longitude:end_Longitude,  $
				end_Time:end_Time,  $
				eq_x_longitude:eq_x_longitude,  $
				eq_x_tai:eq_x_tai,  $
				orbitgeoqa:orbitgeoqa,  $
				num_satgeoqa:num_satgeoqa,  $
				num_glintgeoqa:num_glintgeoqa,  $
				num_moongeoqa:num_moongeoqa,  $
				num_ftptgeoqa:num_ftptgeoqa,  $
				num_zengeoqa:num_zengeoqa,  $
				num_demgeoqa:num_demgeoqa,  $
				num_fpe:num_fpe,  $
				LonGranuleCen:LonGranuleCen,  $
				LatGranuleCen:LatGranuleCen,  $
				LocTimeGranuleCen:LocTimeGranuleCen,  $
				num_scanlines_not_norm_mode_a1:num_scanlines_not_norm_mode_a1,  $
				num_scanlines_not_norm_mode_a2:num_scanlines_not_norm_mode_a2,  $
				num_missing_scanlines_a1:num_missing_scanlines_a1,  $
				num_missing_scanlines_a2:num_missing_scanlines_a2,  $
				num_data_gaps_a1:num_data_gaps_a1,  $
				num_data_gaps_a2:num_data_gaps_a2,  $
				num_instr_mode_changes_a1:num_instr_mode_changes_a1,  $
				num_instr_mode_changes_a2:num_instr_mode_changes_a2,  $
				num_scanlines_rec_cal_prob_a11:num_scanlines_rec_cal_prob_a11,  $
				num_scanlines_rec_cal_prob_a12:num_scanlines_rec_cal_prob_a12,  $
				num_scanlines_rec_cal_prob_a2:num_scanlines_rec_cal_prob_a2,  $
				num_scanlines_sig_coast_xing:num_scanlines_sig_coast_xing,  $
				num_scanlines_sig_sun_glint:num_scanlines_sig_sun_glint,  $
				MoonInViewMWCount:MoonInViewMWCount,  $
				QA_bb_PRT_a11_min:QA_bb_PRT_a11_min,  $
				QA_bb_PRT_a11_max:QA_bb_PRT_a11_max,  $
				QA_bb_PRT_a11_mean:QA_bb_PRT_a11_mean,  $
				QA_bb_PRT_a11_dev:QA_bb_PRT_a11_dev,  $
				QA_bb_PRT_a11_num_in:QA_bb_PRT_a11_num_in,  $
				QA_bb_PRT_a11_num_lo:QA_bb_PRT_a11_num_lo,  $
				QA_bb_PRT_a11_num_hi:QA_bb_PRT_a11_num_hi,  $
				QA_bb_PRT_a11_num_bad:QA_bb_PRT_a11_num_bad,  $
				QA_bb_PRT_a11_range_min:QA_bb_PRT_a11_range_min,  $
				QA_bb_PRT_a11_range_max:QA_bb_PRT_a11_range_max,  $
				QA_bb_PRT_a11_missing:QA_bb_PRT_a11_missing,  $
				QA_bb_PRT_a11_max_track:QA_bb_PRT_a11_max_track,  $
				QA_bb_PRT_a11_max_xtrack:QA_bb_PRT_a11_max_xtrack,  $
				QA_bb_PRT_a11_min_track:QA_bb_PRT_a11_min_track,  $
				QA_bb_PRT_a11_min_xtrack:QA_bb_PRT_a11_min_xtrack,  $
				QA_bb_PRT_a12_min:QA_bb_PRT_a12_min,  $
				QA_bb_PRT_a12_max:QA_bb_PRT_a12_max,  $
				QA_bb_PRT_a12_mean:QA_bb_PRT_a12_mean,  $
				QA_bb_PRT_a12_dev:QA_bb_PRT_a12_dev,  $
				QA_bb_PRT_a12_num_in:QA_bb_PRT_a12_num_in,  $
				QA_bb_PRT_a12_num_lo:QA_bb_PRT_a12_num_lo,  $
				QA_bb_PRT_a12_num_hi:QA_bb_PRT_a12_num_hi,  $
				QA_bb_PRT_a12_num_bad:QA_bb_PRT_a12_num_bad,  $
				QA_bb_PRT_a12_range_min:QA_bb_PRT_a12_range_min,  $
				QA_bb_PRT_a12_range_max:QA_bb_PRT_a12_range_max,  $
				QA_bb_PRT_a12_missing:QA_bb_PRT_a12_missing,  $
				QA_bb_PRT_a12_max_track:QA_bb_PRT_a12_max_track,  $
				QA_bb_PRT_a12_max_xtrack:QA_bb_PRT_a12_max_xtrack,  $
				QA_bb_PRT_a12_min_track:QA_bb_PRT_a12_min_track,  $
				QA_bb_PRT_a12_min_xtrack:QA_bb_PRT_a12_min_xtrack,  $
				QA_bb_PRT_a2_min:QA_bb_PRT_a2_min,  $
				QA_bb_PRT_a2_max:QA_bb_PRT_a2_max,  $
				QA_bb_PRT_a2_mean:QA_bb_PRT_a2_mean,  $
				QA_bb_PRT_a2_dev:QA_bb_PRT_a2_dev,  $
				QA_bb_PRT_a2_num_in:QA_bb_PRT_a2_num_in,  $
				QA_bb_PRT_a2_num_lo:QA_bb_PRT_a2_num_lo,  $
				QA_bb_PRT_a2_num_hi:QA_bb_PRT_a2_num_hi,  $
				QA_bb_PRT_a2_num_bad:QA_bb_PRT_a2_num_bad,  $
				QA_bb_PRT_a2_range_min:QA_bb_PRT_a2_range_min,  $
				QA_bb_PRT_a2_range_max:QA_bb_PRT_a2_range_max,  $
				QA_bb_PRT_a2_missing:QA_bb_PRT_a2_missing,  $
				QA_bb_PRT_a2_max_track:QA_bb_PRT_a2_max_track,  $
				QA_bb_PRT_a2_max_xtrack:QA_bb_PRT_a2_max_xtrack,  $
				QA_bb_PRT_a2_min_track:QA_bb_PRT_a2_min_track,  $
				QA_bb_PRT_a2_min_xtrack:QA_bb_PRT_a2_min_xtrack,  $
				QA_rec_PRT_a11_min:QA_rec_PRT_a11_min,  $
				QA_rec_PRT_a11_max:QA_rec_PRT_a11_max,  $
				QA_rec_PRT_a11_mean:QA_rec_PRT_a11_mean,  $
				QA_rec_PRT_a11_dev:QA_rec_PRT_a11_dev,  $
				QA_rec_PRT_a11_num_in:QA_rec_PRT_a11_num_in,  $
				QA_rec_PRT_a11_num_lo:QA_rec_PRT_a11_num_lo,  $
				QA_rec_PRT_a11_num_hi:QA_rec_PRT_a11_num_hi,  $
				QA_rec_PRT_a11_num_bad:QA_rec_PRT_a11_num_bad,  $
				QA_rec_PRT_a11_range_min:QA_rec_PRT_a11_range_min,  $
				QA_rec_PRT_a11_range_max:QA_rec_PRT_a11_range_max,  $
				QA_rec_PRT_a11_missing:QA_rec_PRT_a11_missing,  $
				QA_rec_PRT_a11_max_track:QA_rec_PRT_a11_max_track,  $
				QA_rec_PRT_a11_max_xtrack:QA_rec_PRT_a11_max_xtrack,  $
				QA_rec_PRT_a11_min_track:QA_rec_PRT_a11_min_track,  $
				QA_rec_PRT_a11_min_xtrack:QA_rec_PRT_a11_min_xtrack,  $
				QA_rec_PRT_a12_min:QA_rec_PRT_a12_min,  $
				QA_rec_PRT_a12_max:QA_rec_PRT_a12_max,  $
				QA_rec_PRT_a12_mean:QA_rec_PRT_a12_mean,  $
				QA_rec_PRT_a12_dev:QA_rec_PRT_a12_dev,  $
				QA_rec_PRT_a12_num_in:QA_rec_PRT_a12_num_in,  $
				QA_rec_PRT_a12_num_lo:QA_rec_PRT_a12_num_lo,  $
				QA_rec_PRT_a12_num_hi:QA_rec_PRT_a12_num_hi,  $
				QA_rec_PRT_a12_num_bad:QA_rec_PRT_a12_num_bad,  $
				QA_rec_PRT_a12_range_min:QA_rec_PRT_a12_range_min,  $
				QA_rec_PRT_a12_range_max:QA_rec_PRT_a12_range_max,  $
				QA_rec_PRT_a12_missing:QA_rec_PRT_a12_missing,  $
				QA_rec_PRT_a12_max_track:QA_rec_PRT_a12_max_track,  $
				QA_rec_PRT_a12_max_xtrack:QA_rec_PRT_a12_max_xtrack,  $
				QA_rec_PRT_a12_min_track:QA_rec_PRT_a12_min_track,  $
				QA_rec_PRT_a12_min_xtrack:QA_rec_PRT_a12_min_xtrack,  $
				QA_rec_PRT_a2_min:QA_rec_PRT_a2_min,  $
				QA_rec_PRT_a2_max:QA_rec_PRT_a2_max,  $
				QA_rec_PRT_a2_mean:QA_rec_PRT_a2_mean,  $
				QA_rec_PRT_a2_dev:QA_rec_PRT_a2_dev,  $
				QA_rec_PRT_a2_num_in:QA_rec_PRT_a2_num_in,  $
				QA_rec_PRT_a2_num_lo:QA_rec_PRT_a2_num_lo,  $
				QA_rec_PRT_a2_num_hi:QA_rec_PRT_a2_num_hi,  $
				QA_rec_PRT_a2_num_bad:QA_rec_PRT_a2_num_bad,  $
				QA_rec_PRT_a2_range_min:QA_rec_PRT_a2_range_min,  $
				QA_rec_PRT_a2_range_max:QA_rec_PRT_a2_range_max,  $
				QA_rec_PRT_a2_missing:QA_rec_PRT_a2_missing,  $
				QA_rec_PRT_a2_max_track:QA_rec_PRT_a2_max_track,  $
				QA_rec_PRT_a2_max_xtrack:QA_rec_PRT_a2_max_xtrack,  $
				QA_rec_PRT_a2_min_track:QA_rec_PRT_a2_min_track,  $
				QA_rec_PRT_a2_min_xtrack:QA_rec_PRT_a2_min_xtrack,  $
				granules_present:granules_present  $
			}

			field  = {$
				scanang:scanang,  $
				satheight:satheight,  $
				satroll:satroll,  $
				satpitch:satpitch,  $
				satyaw:satyaw,  $
				satgeoqa:satgeoqa,  $
				glintgeoqa:glintgeoqa,  $
				moongeoqa:moongeoqa,  $
				ftptgeoqa:ftptgeoqa,  $
				zengeoqa:zengeoqa,  $
				demgeoqa:demgeoqa,  $
				nadirTAI:nadirTAI,  $
				sat_lat:sat_lat,  $
				sat_lon:sat_lon,  $
				scan_node_type:scan_node_type,  $
				satzen:satzen,  $
				satazi:satazi,  $
				solzen:solzen,  $
				solazi:solazi,  $
				glintlat:glintlat,  $
				glintlon:glintlon,  $
				sun_glint_distance:sun_glint_distance,  $
				topog:topog,  $
				topog_err:topog_err,  $
				landFrac:landFrac,  $
				landFrac_err:landFrac_err,  $
				antenna_temp:antenna_temp,  $
				brightness_temp:brightness_temp,  $
				brightness_temp_err:brightness_temp_err,  $
				center_freq:center_freq,  $
				IF_offset_1:IF_offset_1,  $
				IF_offset_2:IF_offset_2,  $
				bandwidth:bandwidth,  $
				num_calibrated_scanlines:num_calibrated_scanlines,  $
				num_scanlines_ch_cal_problems:num_scanlines_ch_cal_problems,  $
				QA_unfiltered_scene_count_min:QA_unfiltered_scene_count_min,  $
				QA_unfiltered_scene_count_max:QA_unfiltered_scene_count_max,  $
				QA_unfiltered_scene_count_mean:QA_unfiltered_scene_count_mean,  $
				QA_unfiltered_scene_count_dev:QA_unfiltered_scene_count_dev,  $
				QA_unfiltered_scene_count_num:QA_unfiltered_scene_count_num,  $
				QA_unfiltered_scene_count_num_bad:QA_unfiltered_scene_count_num_bad,  $
				QA_unfiltered_scene_count_max_track:QA_unfiltered_scene_count_max_track,  $
				QA_unfiltered_scene_count_max_xtrack:QA_unfiltered_scene_count_max_xtrack,  $
				QA_unfiltered_scene_count_min_track:QA_unfiltered_scene_count_min_track,  $
				QA_unfiltered_scene_count_min_xtrack:QA_unfiltered_scene_count_min_xtrack,  $
				QA_unfiltered_BB_count_min:QA_unfiltered_BB_count_min,  $
				QA_unfiltered_BB_count_max:QA_unfiltered_BB_count_max,  $
				QA_unfiltered_BB_count_mean:QA_unfiltered_BB_count_mean,  $
				QA_unfiltered_BB_count_dev:QA_unfiltered_BB_count_dev,  $
				QA_unfiltered_BB_count_num:QA_unfiltered_BB_count_num,  $
				QA_unfiltered_BB_count_num_bad:QA_unfiltered_BB_count_num_bad,  $
				QA_unfiltered_BB_count_max_track:QA_unfiltered_BB_count_max_track,  $
				QA_unfiltered_BB_count_max_xtrack:QA_unfiltered_BB_count_max_xtrack,  $
				QA_unfiltered_BB_count_min_track:QA_unfiltered_BB_count_min_track,  $
				QA_unfiltered_BB_count_min_xtrack:QA_unfiltered_BB_count_min_xtrack,  $
				QA_unfiltered_space_count_min:QA_unfiltered_space_count_min,  $
				QA_unfiltered_space_count_max:QA_unfiltered_space_count_max,  $
				QA_unfiltered_space_count_mean:QA_unfiltered_space_count_mean,  $
				QA_unfiltered_space_count_dev:QA_unfiltered_space_count_dev,  $
				QA_unfiltered_space_count_num:QA_unfiltered_space_count_num,  $
				QA_unfiltered_space_count_num_bad:QA_unfiltered_space_count_num_bad,  $
				QA_unfiltered_space_count_max_track:QA_unfiltered_space_count_max_track,  $
				QA_unfiltered_space_count_max_xtrack:QA_unfiltered_space_count_max_xtrack,  $
				QA_unfiltered_space_count_min_track:QA_unfiltered_space_count_min_track,  $
				QA_unfiltered_space_count_min_xtrack:QA_unfiltered_space_count_min_xtrack,  $
				QA_cal_coef_a0_min:QA_cal_coef_a0_min,  $
				QA_cal_coef_a0_max:QA_cal_coef_a0_max,  $
				QA_cal_coef_a0_mean:QA_cal_coef_a0_mean,  $
				QA_cal_coef_a0_dev:QA_cal_coef_a0_dev,  $
				QA_cal_coef_a0_num:QA_cal_coef_a0_num,  $
				QA_cal_coef_a0_num_bad:QA_cal_coef_a0_num_bad,  $
				QA_cal_coef_a0_max_track:QA_cal_coef_a0_max_track,  $
				QA_cal_coef_a0_max_xtrack:QA_cal_coef_a0_max_xtrack,  $
				QA_cal_coef_a0_min_track:QA_cal_coef_a0_min_track,  $
				QA_cal_coef_a0_min_xtrack:QA_cal_coef_a0_min_xtrack,  $
				QA_cal_coef_a1_min:QA_cal_coef_a1_min,  $
				QA_cal_coef_a1_max:QA_cal_coef_a1_max,  $
				QA_cal_coef_a1_mean:QA_cal_coef_a1_mean,  $
				QA_cal_coef_a1_dev:QA_cal_coef_a1_dev,  $
				QA_cal_coef_a1_num:QA_cal_coef_a1_num,  $
				QA_cal_coef_a1_num_bad:QA_cal_coef_a1_num_bad,  $
				QA_cal_coef_a1_max_track:QA_cal_coef_a1_max_track,  $
				QA_cal_coef_a1_max_xtrack:QA_cal_coef_a1_max_xtrack,  $
				QA_cal_coef_a1_min_track:QA_cal_coef_a1_min_track,  $
				QA_cal_coef_a1_min_xtrack:QA_cal_coef_a1_min_xtrack,  $
				QA_cal_coef_a2_min:QA_cal_coef_a2_min,  $
				QA_cal_coef_a2_max:QA_cal_coef_a2_max,  $
				QA_cal_coef_a2_mean:QA_cal_coef_a2_mean,  $
				QA_cal_coef_a2_dev:QA_cal_coef_a2_dev,  $
				QA_cal_coef_a2_num:QA_cal_coef_a2_num,  $
				QA_cal_coef_a2_num_bad:QA_cal_coef_a2_num_bad,  $
				QA_cal_coef_a2_max_track:QA_cal_coef_a2_max_track,  $
				QA_cal_coef_a2_max_xtrack:QA_cal_coef_a2_max_xtrack,  $
				QA_cal_coef_a2_min_track:QA_cal_coef_a2_min_track,  $
				QA_cal_coef_a2_min_xtrack:QA_cal_coef_a2_min_xtrack,  $
				QA_bb_raw_noise_counts_min:QA_bb_raw_noise_counts_min,  $
				QA_bb_raw_noise_counts_max:QA_bb_raw_noise_counts_max,  $
				QA_bb_raw_noise_counts_mean:QA_bb_raw_noise_counts_mean,  $
				QA_bb_raw_noise_counts_dev:QA_bb_raw_noise_counts_dev,  $
				QA_bb_raw_noise_counts_num:QA_bb_raw_noise_counts_num,  $
				QA_bb_raw_noise_counts_num_bad:QA_bb_raw_noise_counts_num_bad,  $
				QA_bb_raw_noise_counts_max_track:QA_bb_raw_noise_counts_max_track,  $
				QA_bb_raw_noise_counts_max_xtrack:QA_bb_raw_noise_counts_max_xtrack,  $
				QA_bb_raw_noise_counts_min_track:QA_bb_raw_noise_counts_min_track,  $
				QA_bb_raw_noise_counts_min_xtrack:QA_bb_raw_noise_counts_min_xtrack,  $
				QA_sv_raw_noise_counts_min:QA_sv_raw_noise_counts_min,  $
				QA_sv_raw_noise_counts_max:QA_sv_raw_noise_counts_max,  $
				QA_sv_raw_noise_counts_mean:QA_sv_raw_noise_counts_mean,  $
				QA_sv_raw_noise_counts_dev:QA_sv_raw_noise_counts_dev,  $
				QA_sv_raw_noise_counts_num:QA_sv_raw_noise_counts_num,  $
				QA_sv_raw_noise_counts_num_bad:QA_sv_raw_noise_counts_num_bad,  $
				QA_sv_raw_noise_counts_max_track:QA_sv_raw_noise_counts_max_track,  $
				QA_sv_raw_noise_counts_max_xtrack:QA_sv_raw_noise_counts_max_xtrack,  $
				QA_sv_raw_noise_counts_min_track:QA_sv_raw_noise_counts_min_track,  $
				QA_sv_raw_noise_counts_min_xtrack:QA_sv_raw_noise_counts_min_xtrack,  $
				state1:state1,  $
				state2:state2,  $
				cal_coef_a0:cal_coef_a0,  $
				cal_coef_a1:cal_coef_a1,  $
				cal_coef_a2:cal_coef_a2,  $
				cal_coef_a0_err:cal_coef_a0_err,  $
				cal_coef_a1_err:cal_coef_a1_err,  $
				cal_coef_a2_err:cal_coef_a2_err,  $
				a1_ColdCalPstion:a1_ColdCalPstion,  $
				a2_ColdCalPstion:a2_ColdCalPstion,  $
				a1_PLO_Redundncy:a1_PLO_Redundncy,  $
				a11_mux_temp_used:a11_mux_temp_used,  $
				a11_receiver_temp:a11_receiver_temp,  $
				a11_target_temp:a11_target_temp,  $
				a12_mux_temp_used:a12_mux_temp_used,  $
				a12_receiver_temp:a12_receiver_temp,  $
				a12_target_temp:a12_target_temp,  $
				a2_diplexer_temp_used:a2_diplexer_temp_used,  $
				a2_receiver_temp:a2_receiver_temp,  $
				a2_target_temp:a2_target_temp,  $
				qa_scanline:qa_scanline,  $
				qa_receiver_a11:qa_receiver_a11,  $
				qa_receiver_a12:qa_receiver_a12,  $
				qa_receiver_a2:qa_receiver_a2,  $
				qa_channel:qa_channel  $
			}

			geo_field  = {$
				Latitude:Latitude,  $
				Longitude:Longitude,  $
				Time:Time  $
			}

			L1B_AMSU = {attr:attr,field:field,geo_field:geo_field}
			x = { $
			L1B_AMSU:L1B_AMSU  $
			}

		return
	endif else begin
		x = !values.f_nan
		return
	endelse
end
