pro read_AMSU_L2A_orbit,satellite_name,				$
							 orbit_number_in,					$
							 channel,						$
							 L2A_header,					$
							 L2A_orbit,						$
							 L2A_scan_valid,				$
							 AMSU_L2A_Path = AMSU_L2A_Path, $
							 error = error
;@***h* MLT_RSS/read_AMSU_L2A_orbit.pro

; NAME
;   read_AMSU_L2A_orbit.pro
;
; PURPOSE
;   Reads RSS L2A file with adjusted AMSU on AQUA data.
;
; DESCRIPTION
;   Reads RSS L2A file with adjusted AMSU on AQUA data.
;
; INPUTS:
;   satellite_name,  ; satellite name ('AQUA  ')
;   orbit_number,    ; orbit number        
;   channel,         ; AMSU channel  
;   AMSU_L2A_Path = AMSU_L2A_Path  ; location of AMSU_L2A_files
;   
; OUTPUTS:
;   L2A_header,         ;   L2A header data
;   L2A_orbit,          ;   L2A scan data
;   L2A_scan_valid,     ;   L2A array of integers -- set to one if scan is valid, 0 otherwise
;   error ; error flag. ;   0 if read went OK, -1 otherwise
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/28/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_AMSU_L2A_orbit,satellite_name,orbit_number,channel,           $
;                                  L2A_header,L2A_orbit,L2A_scan_valid,           $
;                                  AMSU_L2A_Path = AMSU_L2A_Path, error = error
;
;@*****

	;	character(len = 7)					:: satellite_name
	;	integer(4)							:: orbit_number
	;	integer(4)							:: channel
	;	type(AMSU_L2A_orbit_header)			:: L2A_orbit_header
	;	type(AMSU_L2A_orbit_data)			:: L2A_orbit
	;	integer(4)							:: error

	orbit_number = orbit_number_in+0L

	buffer = bytarr(660,790)
	line   = bytarr(660)
	amsu_sat_names_2 = ['      ','TIROS-N','NOAA-06', 'NOAA-07', 'NOAA-08','NOAA-09', $
	                    'NOAA-10','NOAA-11','NOAA-12','NOAA-14','NOAA-15', $
	                    'NOAA-16','NOAA-17','NOAA-18','METOP-A','AQUA   ']


		error = 0


		; determine the file name

		range_num = orbit_number/5000
		orbit_Range_lower = (range_num*5000)+1
		orbit_range_upper = orbit_range_lower + 4999

		filename=string(amsu_L2A_path,satellite_name,channel,orbit_range_lower, $
		                orbit_range_upper,strtrim(satellite_name,2),orbit_number, $
		format='(a,a,"\Channel_",i2.2,"\r",i5.5,"_",i5.5,"\",a,"_r",i5.5,".L2A.gz")')
		log,'Reading.... ',filename
		CLOSE,3
		on_ioerror,no_file
		openr,3,filename,/compress
		fuz_err = 0
		on_ioerror,file_end

		L2A_header = {satellite_name:'       ', 		$
		              fill0:' ',						$
		              orbit_number:0L, 					$
		              number_of_swaths:0L,				$
		              start_swath_num:0L,				$
		              end_swath_num:0L,					$
		              start_rd_2000:0.0D0,				$
   		              end_rd_2000:0.0D0,				$
   		              eqx_rd_2000:0.0D0,				$
   		              channel:0L,						$
   		              fill:bytarr(48),					$
   		              packed_scan_valid:bytarr(395), 	$
   		              residual_cross_track_Corrections:fltarr(30), $
   		              L2A_processor_ID:bytArr(20),    $
   		              L1B_processor_ID:bytarr(20),      $
   		              fill2:bytarr(129)}

			scan_str_templ = {satellite_name:'       ',	$
						channel_num:0B,					$
						orbit_number:0L,				$
						scan_line_in_orbit:0L,			$
						scan_rd_2000:0.0D0,				$
						fov_latitude:fltarr(30),		$
						fov_longitude:fltarr(30),		$
						fov_quality:bytarr(30),			$
						warm_load_temp:0.0, 			$
						asc_desc:0,						$	;(1 = asc, 2 = desc, 0 = undetermined)
						Tbs:fltarr(30),					$
						Diurnal_Corr:fltarr(30),		$
						Nadir_Corr:fltarr(30),			$
						Incidence_Corr:fltarr(30),		$
						spacecraft_altitude:0.0}

			L2A_orbit = replicate(scan_str_templ,790)

  		readu,3,L2A_header
  	    L2A_scan_valid = intarr(790)
		for i = 0,394 do begin
			L2A_scan_valid[i*2] 	= L2A_header.packed_scan_valid[i] and 15
			L2A_scan_valid[(i*2)+1] = L2A_header.packed_scan_valid[i]/16
		endfor

		on_ioerror,file_end
		for line_num = 1,800 do begin
			scan_str = scan_str_templ
			readu,3,scan_str
			L2A_orbit[line_num-1] = scan_str
		endfor

		file_end:
			return

		no_header:
			log,'Error Reading Header'
			error = -1
			return

		no_file:
			log,'File does not exist'
			error = -1
			return

	end
