pro read_orbitify_status_file,status_root,channel,status
;@***h* MLT_RSS/read_orbitify_status_file.pro

; NAME
;   read_orbitify_status_file
;
; PURPOSE
;   Reads RSS L2A Status File
;
; DESCRIPTION
;   Reads RSS L2A Status File.  File is used both for web based tools, and to keep 
;   track of progress of the orbitification process.
;
; INPUTS
;   status_root   ; location of status files
;   channel       ; AMSU channel number
;   
; OUTPUTS
;   status        ; status structure
;   
; FILE READ
;   RSS AMSU status file
;   e.g.  AQUA_Orbitify_Channel_05.stat.js
;   status is stored as a javascript file so it can be executed by web servers.
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT:
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   11/04/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_orbitify_status_file,status_root,channel,status
;
;@*****
	; file contains
	; 	last_time_run
	;   last_date_processed
	;   last_rd_2000_processed

	orbitify_stat_file = string(status_root,channel,format = '(a,"Aqua_Orbitify_Channel_",i2.2,".stat.js")')
	close,!stat_lun
	openr,!stat_lun,orbitify_stat_file

	line = ' '

	readf,!stat_lun,line
	chunks = strsplit(line,'"'+"'",/extract)
	last_time_run = chunks[1]

	readf,!stat_lun,line
	chunks = strsplit(line,'"'+"'",/extract)
	last_date_processed = chunks[1]

	readf,!stat_lun,line
	chunks = strsplit(line,'"'+"'",/extract)
	last_rd2000_processed_str = chunks[1]
	last_rd2000_processed = 0.0D0
	reads,last_rd2000_processed_str,last_rd2000_processed

	readf,!stat_lun,line
	chunks = strsplit(line,'"'+"'",/extract)
	last_orbit_processed_str = chunks[1]
	last_orbit_processed = 0L
	reads,last_orbit_processed_str,last_orbit_processed


	status = {last_time_run:last_time_run,  				$
			  last_date_processed:last_date_processed,		$
			  last_rd_2000_processed:last_rd2000_processed,	$
			  last_orbit_processed:last_orbit_processed}


end