
pro write_AMSU_L2A_orbit_unadjusted,	 						$
							 satellite_name,				$
							 orbit_number,					$
							 channel,						$
							 L2A_header,					$
							 L2A_orbit,						$
							 L2A_scan_valid,				$
							 AMSU_L2A_Path = AMSU_L2A_Path, $
							 error = error
;@***h* MLT_RSS/write_AMSU_L2A_orbit_unadjusted.pro

; NAME
;   write_AMSU_L2A_orbit_unadjusted
;
; PURPOSE
;   Writes RSS L2A file with unadjusted AMSU on AQUA data.
;
; DESCRIPTION
;   Writes RSS L2A file with unadjusted AMSU on AQUA data.
;
; INPUTS:
;   satellite_name,  ; satellite name ('AQUA  ')
;   orbit_number,    ; orbit number        
;   channel,         ; AMSU channel  
;   AMSU_L2A_Path = AMSU_L2A_Path  ; location of AMSU_L2A_files
;   L2A_header,         ;   L2A header data
;   L2A_orbit,          ;   L2A scan data
;   L2A_scan_valid,     ;   L2A array of integers -- set to one if scan is valid, 0 otherwise
;   
; OUTPUTS:
;   error ; error flag. ;   0 if read went OK, -1 otherwise
;   
; FILE WRITTEN:
;   
; AUTHOR:
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT:
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   write_AMSU_L2A_orbit_unadjusted,satellite_name,orbit_number,channel,           $
;                                  L2A_header,L2A_orbit,L2A_scan_valid,           $
;                                  AMSU_L2A_Path = AMSU_L2A_Path, error = error
;
;@*****

	error = 0


	; determine the file name

	range_num = orbit_number/5000
	orbit_Range_lower = (range_num*5000)+1
	orbit_range_upper = orbit_range_lower + 4999

	filename=string(amsu_L2A_path,satellite_name,channel,strtrim(satellite_name,2),orbit_number, $
		format='(a,a,"\Channel_",i2.2,"\unadjusted\",a,"_r",i5.5,".L2A.unadj.gz")')
	print,'Writing.... ',filename

	; open the file, if possible
	close,3
	on_ioerror,bad_path
	openw,3,filename,/compress
	; pack the scan valid data
	for i = 0,394 do begin
		L2A_header.packed_scan_valid[i] = L2A_scan_valid[i*2]+16*L2A_scan_valid[(i*2)+1]
	endfor
	;write the header
	writeu,3,L2A_header

	for scan_num = 0,789 do begin
		if L2A_scan_valid[scan_num] gt 0 then begin
			writeu,3,L2A_orbit[scan_num]
		endif
	endfor

	close,3

	return

	bad_path:
		error = -1
		return

	end
