pro write_L2A_inventory,sat_num,channel,exists,start_scan,end_scan,start_rd_2000,end_rd_2000,eqx_rd_2000
;@***h* MLT_RSS/write_L2A_inventory.pro

; NAME
;   write_L2A_inventory
;
; PURPOSE
;   Writes RSS L2A inventory file.
;
; DESCRIPTION
;   Writes RSS L2A inventory file.
;
; INPUTS
;   sat_num
;   channel
;   exists          ;intarr(80000)  1 if orbit exists, 0 if does not exist
;   start_scan      ;intarr(80000)  first scan number in oribt 
;   end_scan        ;intarr(80000)  last scan number in orbit
;   start_rd_2000   ;dlbarr(80000)  begin date, in rd_2000 format
;   end_rd_2000     ;dblarr(80000)  end date, in rd_2000 format
;   eqx_rd_2000     ;dblarr(80000)  scending node crossing time, in rd_2000 format
;   
; FILE WRITTEN
;   RSS L2a inventory file in text form
;   e.g.  AQUA_Channel_05_L2A.inv.txt
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT:
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   10/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   write_L2A_inventory,sat_num,channel,exists,start_scan,end_scan,start_rd_2000,end_rd_2000,eqx_rd_2000
;
;@*****
	name = sat_name(sat_num,/fill)
	chan_str = string(channel,format='(i2.2)')
	get_AMSU_File_Locations,AMSU_L2A_Path = AMSU_L2A_Path
	inv_file = AMSU_L2A_Path+name+'\'+name+'_channel_'+chan_str+'_L2A.inv.txt'
	print,'Writing ',inv_file
	close,3
	openw,3,inv_file

	num_orbits = n_elements(exists)

	orbit = 0L
	st_scan = 0
	ed_scan = 0
	st_rd_2000 = 0.0D0
	ed_rd_2000 = 0.0D0
	for orbit = 0L,num_orbits-1 do begin

		if exists(orbit) ge 1 then begin
			printf,3,	orbit,					$
						start_scan[orbit],		$
						start_rd_2000[orbit],	$
						end_scan[orbit],		$
						end_rd_2000[orbit],		$
						eqx_rd_2000[orbit], $
						format = '(i8,i4,f15.7,i4,f15.7,f15.7)'
		endif
	endfor
	close,3
end
