pro write_orbitify_status_file,status_root,channel,status
;@***h* MLT_RSS/write_orbitify_status_file.pro

; NAME
;   write_orbitify_status_file
;
; PURPOSE
;   Writes RSS L2A Status File
;
; DESCRIPTION
;   Writes RSS L2A Status File.  File is used both for web based tools, and to keep 
;   track of progress of the orbitification process.
;
; INPUTS
;   status_root   ; location of status files
;   channel       ; AMSU channel number
;   status        ; status structure
;   
; FILE WRITTEN
;   RSS AMSU status file
;   e.g.  AQUA_Orbitify_Channel_05.stat.js
;   status is stored as a javascript file so it can be executed by web servers.
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT:
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   11/04/2012 Initial Version prepared for NCDC
;
; USAGE
;   write_orbitify_status_file,status_root,channel,status
;
;@*****
;
;
	; file contains
	; 	last_time_run
	;   last_date_processed
	;   last_rd_2000_processed

	orbitify_stat_file = string(status_root,channel,format = '(a,"AQUA_Orbitify_Channel_",i2.2,".stat.js")')
	close,!stat_lun
	openw,!stat_lun,orbitify_stat_file
	printf,!stat_lun,"var last_time_run = '"+status.last_time_run+"'"
	printf,!stat_lun,"var last_date_processed = '"+status.last_date_processed+"'"
	printf,!stat_lun,"var last_rd2000_processed = '"+string(status.last_rd_2000_processed)+"'"
	printf,!stat_lun,"var last_orbit_processed = '"+string(status.last_orbit_processed)+"'"

	close,!stat_lun

end