pro apply_TF_to_grpt_maps_AMSU,tb_arr,tar_arr,channel,target_factors,target_means,months_to_use,tb_arr_adj

;@***h* MLT_RSS/apply_TF_to_grpt_maps_AMSU.pro
; NAME
;   apply_TF_to_grpt_maps_AMSU.pro
;
; PURPOSE
;   applies AMSU target factors to grid point maps to produce an adjusted dataset. 
;
; DESCRIPTION
;   Applies the target factors using the method described in 
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the Remote Sensing Systems V3.2 atmospheric temperature records from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1040-1056.
;   
;   and 
;   
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the RSS V3.2 lower tropospheric dataset from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1493-1509.
;
; INPUTS
;   tb_arr          (144,72,num_months,num_sats) real(4) array of brightness temperatures
;   tar_arr         (144,72,num_months,num_sats) real(4) array of target temperatures
;   channel          3 character channel string (TLT,TMT, etc)  not used in this version
;   target_factors  (num_sats) float 32 array of target_factors
;   target_means    (num_sats) float32 array of target_factors
;   months_to_use   (num_sats,num_months) array of integers -- 1 means use this month, 0 mean ignore)
;
; OUTPUTS
;   tb_arr_adj      (144,72,num_months,num_sats) real(4) array of adjusted brightness temperatures
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;   
; USAGE
;   apply_TF_to_grpt_maps_AMSU,tb_arr,tar_arr,channel,target_factors,target_means,months_to_use,tb_arr_adj
;
;@*****
	sz = size(tb_arr,/dimensions)
	num_months = sz[2]
	num_sats = sz[3]

	tb_arr_adj = tb_arr/0.0
	for month = 0,num_months-1 do begin
		for satnum = 0,num_sats-1 do begin
			if months_to_use[month,satnum] lt 1 then continue
			tb_arr_adj[*,*,month,satnum] = tb_arr[*,*,month,satnum] - target_factors[satnum]*(tar_arr[*,*,month,satnum]-target_means[satnum])
		endfor
	endfor
end
