pro check_grpt_maps_AMSU_v3_3,num_arr,sats_to_use,num_thres,months_to_use_mask,months_to_use

;@***h* MLT_RSS/check_grpt_maps_AMSU_v3_3.pro

; NAME
;   check_grpt_maps_AMSU_v3_3.pro
;
; PURPOSE
;   Check AMSU montly gridded data for months with too little data
;   Determines which months to use in merge and returns and array, months_to_use,
;   that is used in subsequent steps to choose which satellite months to include

;
; DESCRIPTION
; This routine checks the AMSU GRPT data for months with too little data
; 
; Inputs:
;   num_arr  (should be a (144,72,num_months,num_sats) array of number of obs per grid cell
;   sats_to_use (num_AMSUs) array of integers  0 to ignore this satellite, 1, to use it
;   num_thres is the threshold for the mean number of observations to be good data.
;   months_to_use_mask is a (num_months, num_sats) array of integers:  
;               -1 to exclude
;                0 to use threshold to determine use (the usual case)
;                1 to use even if threshold fails

; Output:
;   months_to_use  (num_months,num_sats) array of months to use in the merge.  1 means use,0 means don't use
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2019 Initial Version prepared for NCDC
;
; USAGE
;   check_grpt_maps_AMSU_v3_3,num_arr,sats_to_use,num_thres,months_to_use_mask,months_to_use


;@*****

  sz = size(num_arr,/dimensions)
  num_lons    = sz[0]
  num_lats    = sz[1]
  num_months  = sz[2]
  num_sats    = sz[3]

  forward_function latitude_wts
	lat_range=[-50,50]
	lat_wts = latitude_wts(num_lats,lat_range)
	
	;calculate tropical/global averages for the tb_arr,tar_arr, and the num_arr

	mean_num_arr = fltarr(num_months,num_sats)
	months_to_use = intarr(num_months,num_sats)

	surf_str = 'Land_and_Sea'

	for satnum = 0,num_sats-1 do begin
		if sats_to_use[satnum] lt 1 then continue
		for month = 0,num_months-1 do begin
			map = num_arr[*,*,month,satnum]
			mean_num_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine)
		endfor
	endfor

	;start by including all months, then remove points with not enough data
  
  ; set all months to use
	months_to_use = replicate(1,num_months,num_sats)
   
  bad = where(mean_num_arr lt num_thres,ct)
  if ct gt 0 then begin
     months_to_use[bad] = 0
  endif
  
  ; apply months_to_use_mask to force results for certain months and satellites
  
  bad_anyway = where(months_to_use_mask eq -1,ct2)  ; -1 means don't use
  if ct2 gt 0 then begin
      months_to_use[bad_anyway] = 0
  endif
  
  good_anyway = where(months_to_use_mask eq 1,ct3)  ; 1 means use even if fails num_thres test.
  if ct3 gt 0 then begin
      months_to_use[good_anyway] = 1
  endif
  return
end
