pro write_amsu_only_merged_Tbs_144_72_netcdf_3_3,	Tbs_144_72 = Tbs_144_72 ,		$
													channel_str=channel_str,		$
													version_num=version_num,		$
													source_files = source_files,	$
													offsets = offsets,				$
													target_factors = target_factors,$
													target_means = target_means, $
													tb_Factors = tb_factors,		$
													months_used = months_used,      $
													date_L2C_processed = date_L2C_processed,  $
                          output_path = output_path
                          
;@***h* MLT_RSS/write_amsu_only_merged_Tbs_144_72_netcdf_3_3.pro
; NAME
;   write_amsu_only_merged_Tbs_144_72_netcdf_3_3.pro
;
; PURPOSE
;   writes intermediate results from the AMSU-only merge step to a netcdf file 
;
; DESCRIPTION
;   writes intermediate results from the AMSU-only merge step to a netcdf file.  The netcdf file contain
;   both the merged tbs, the values of the various merging parameters, and the months for each satellite that
;   were used in the merge.
;
; INPUTS
;   Tbs_144_72                Float32 array[num_lons,num_lats,num_amsu_months] of merged intercalibrated brightness temperatures
;   channel_str               Channel (TLT, TMT, TTS, TLS)
;   version_num               Version Number
;   source_files              List of Source Files
;   offsets = offsets         Array[!Num_GrPt_Lats,!num_AMSUs] of offsets applied
;   target_factors            Array[!Num_GrPt_Lats,!num_AMSUs] of target factors
;   target_means              Array[!num_AMSUs] of mean target temperatures
;   tb_Factors                Array[!Num_GrPt_Lats,!num_AMSUs] of tb factors applied
;   months_used               Array[!Num_AMSU_months,!num_AMSUs] of offsets applied
;   date_L2C_processed        Array[!Num_AMSU_months,!num_AMSUs] of the date each level 2c file used was processed
;   output_path               Optional output path parameter.  If not supplied, set to !AMSU_L3_Path+'AMSU_Only\'
; OUTPUTS
;   none
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
;
;@*****                 
  
  if keyword_set(output_path) eq 0 then output_path =     !AMSU_L3_Path+'AMSU_Only\'
 
	fill_value = -9999.0
	sz = size(Tbs_144_72,/dimensions)
	num_lons = sz[0]
	num_lats = sz[1]
	num_months = sz[2]
	num_satellites = n_elements(source_files)
	num_satellites = !num_AMSUs

	Tbs = Tbs_144_72
	bad = where(finite(Tbs) eq 0,ct)
	if ct gt 0 then begin
		Tbs[bad] = fill_value
	endif

	todays_date = current_date_string()

	nc_file = output_path + 'RSS_Tb_Maps_ch_'+channel_str+'_V3_3_'+string(version_num,format='(i3.3)')+'.nc'
	log,'Info: netcdf output file = ',nc_file

	id = NCDF_CREATE(nc_file, /CLOBBER) ; Create the netCDF file.
	NCDF_ATTPUT, id, 'title', 'RSS Version 3.3 AMSU brightness temperatures', /GLOBAL
	NCDF_ATTPUT, id, 'institution', 'Remote Sensing Systems, Santa Rosa, CA, USA', /GLOBAL

	source_str = 'Advanced Microwave Sounding Unit (AMSU) satellite-borne microwave radiometer, channel '+channel_str
	n = n_elements(source_files)
	if n ge 1 then begin
	  for i = 0,n-1 do begin
			source_str = source_str + '  ' +source_files[i]+string(13B)
	  endfor
	endif

	a = SCOPE_TRACEBACK()
	main_program_name = a[0]

	NCDF_ATTPUT, id, 'source',  source_str, /GLOBAL
	NCDF_ATTPUT, id, 'history', todays_date + ' produced from version 3.2 RSS monthly maps for each AMSU instrument using the idl routine ' +main_program_name, /GLOBAL
	NCDF_ATTPUT, id, 'references', 'C. A. Mears, M. Schabel, and F. J. Wentz: A Reanalysis of the MSU Channel 2 Tropospheric Temperature Record, Journal of Climate 16, pp. 3650-3664', /GLOBAL

	comment_str =  $
				'This is AMSU only data, produced as an intermediate step toward producing a combined MSU/AMSU product  ' + string(13B) + $
				'Data is from a restricted set of fields of view'+ string(13B)

	NCDF_ATTPUT, id, 'comment',comment_str , /GLOBAL

	x_dim      = NCDF_DIMDEF(id, 'longitude', num_lons) 		; Define the X dimension.
	x_edge_dim = NCDF_DIMDEF(id, 'longitude_bounds', num_lons+1) 	    ; Define the X dimension cell edges.
	y_dim      = NCDF_DIMDEF(id, 'latitude', num_lats) 		    ; Define the Y dimension.
	y_edge_dim = NCDF_DIMDEF(id, 'latitude_bounds', num_lats+1) 		; Define the Y dimension cell edges.
	time_dim   = NCDF_DIMDEF(id, 'months', num_months) 			; Define the month dimension.
	sat_dim    = NCDF_DIMDEF(id, 'satellites',num_satellites)   ; Define the satellite number dimension

	lon_id   			= NCDF_VARDEF(id, 'longitude', 			[x_dim], /FLOAT)
		NCDF_ATTPUT,id,lon_id,'long_name','longitude'
		NCDF_ATTPUT,id,lon_id,'units','degrees east'
		NCDF_ATTPUT,id,lon_id,'bounds','longitude_bounds'
	lat_id   			= NCDF_VARDEF(id, 'latitude', 			[y_dim], /FLOAT)
		NCDF_ATTPUT,id,lat_id,'long_name','latitude'
		NCDF_ATTPUT,id,lat_id,'units','degrees north'
		NCDF_ATTPUT,id,lat_id,'bounds','latitude_bounds'
	lon_boundary_id   	= NCDF_VARDEF(id, 'longitude_bounds', 	[x_edge_dim], /FLOAT)
		NCDF_ATTPUT,id,lon_boundary_id,'long_name','longitude_bounds'
		NCDF_ATTPUT,id,lon_boundary_id,'units','degrees east'
	lat_boundary_id   	= NCDF_VARDEF(id, 'latitude_bounds', 	[y_edge_dim], /FLOAT)
		NCDF_ATTPUT,id,lat_boundary_id,'long_name','latitude_bounds'
		NCDF_ATTPUT,id,lat_boundary_id,'units','degrees north'
	time_id 			= NCDF_VARDEF(id, 'months', 			[time_dim], /LONG)
		NCDF_ATTPUT,id,time_id,'long_name','time'
		NCDF_ATTPUT,id,time_id,'units','months since 1978-1-1 0:0:0'
	temperature_id  = NCDF_VARDEF(id, 'brightness_temperature',[x_dim,y_dim,time_dim], /FLOAT)
		NCDF_ATTPUT,id,temperature_id,'long_name','monthly averaged single channel brightness temperature'
		NCDF_ATTPUT,id,temperature_id,'units','degrees K'
		NCDF_ATTPUT,id,temperature_id,'_FillValue',fill_value
	months_used_id  = NCDF_VARDEF(id, 'satellites_used',[sat_dim,time_dim], /BYTE)
	    NCDF_ATTPUT,id,months_used_id,'long_name','satellites used by month'
		NCDF_ATTPUT,id,months_used_id,'units','none'
		NCDF_ATTPUT,id,months_used_id,'comment','1 when satellite used, 0 otherwise'
	date_L2C_processed_id = NCDF_VARDEF(id,'date_L2C_processed',[sat_dim,time_dim],/DOUBLE)
		NCDF_ATTPUT,id,date_L2C_processed_id,'long_name','date satellite processed in IDL julday format'
		NCDF_ATTPUT,id,date_L2C_processed_id,'units','julian day number'
	offset_id		= NCDF_VARDEF(id, 'offset_values',[y_dim,sat_dim],/FLOAT)
		NCDF_ATTPUT,id,offset_id,'long_name','satellite calibration offset for each latitude'
		NCDF_ATTPUT,id,offset_id,'units','degrees K'
	target_factor_id= NCDF_VARDEF(id,'target_factor_values',[y_dim,sat_dim],/FLOAT)
		NCDF_ATTPUT,id,target_factor_id,'long_name','satellite calibration target factor for each latitude'
		NCDF_ATTPUT,id,target_factor_id,'units','dimensionless'
	tb_factor_id= NCDF_VARDEF(id,'tb_factor_values',[y_dim,sat_dim],/FLOAT)
		NCDF_ATTPUT,id,target_factor_id,'long_name','satellite calibration tb factor for each latitude'
		NCDF_ATTPUT,id,target_factor_id,'units','dimensionless'
	target_mean_id = NCDF_VARDEF(id,'target_means',[sat_dim],/FLOAT)
	  NCDF_ATTPUT,id,target_factor_id,'long_name','satellite mean target temperature'
    NCDF_ATTPUT,id,target_factor_id,'units','K'

	; switch to data mode and write data

	NCDF_CONTROL, id, /ENDEF ; Put the file into data mode.

	latitude   		= (indgen(num_lats)*2.5) -90.0+1.25
	longitude  		= 1.25 +(indgen(num_lons) * 2.5)
	latitude_bnd   	= (indgen(num_lats+1)*2.5) -90.0
	longitude_bnd  	= indgen(num_lons+1) * 2.5
	months      	= indgen(num_months)+240

	NCDF_VARPUT, id, lon_id, longitude
	NCDF_VARPUT, id, lat_id, latitude
	NCDF_VARPUT, id, lon_boundary_id, longitude_bnd
	NCDF_VARPUT, id, lat_boundary_id, latitude_bnd
	NCDF_VARPUT, id, time_id, months
	NCDF_VARPUT, id, temperature_id, Tbs
	NCDF_VARPUT, id, months_used_id, transpose(months_used)
	NCDF_VARPUT, id, date_L2C_processed_id,transpose(date_L2C_processed)
	NCDF_VARPUT, id, offset_id,offsets
	NCDF_VARPUT, id, target_factor_id,target_factors
	NCDF_VARPUT, id, tb_factor_id,tb_factors
	NCDF_VARPUT, id, target_mean_id,target_means


	NCDF_CLOSE, id ; Close the NetCDF file.

end
