pro chng_to_midnight_TLT,mean_tb_map,mean_tb_map_midnight,noon_midnight_diff=noon_midnight_diff

;@***h* MLT_RSS/grpt_merge_AMSU_only_3_3.pro

; NAME
;   chng_to_midnight_tlt.pro
;
; PURPOSE
;   uses diurnal climatology to change reference time for a gridded TLT dataset from noon to midnight
;
; DESCRIPTION
;   Uses the CCM3 diurnal cycle to adjust gridded data from noon to midnight, local time.
;
; INPUTS:
;   mean_tb_map  gridded monthly Tb dataset, local reference time set to noon.  Channel is assumed to be TLT
;
; OUTPUTS:
;   mean_tb_map_midnight  gridded monthly Tb dataset, local reference time adjusted to midnight
;   noon_midnight_diff    optional:  gridded monthly dataset of noon minus midnight differences

; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;   
; HISTORY
;   6/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   chng_to_midnight_tlt,mean_tb_map,mean_tb_map_midnight,noon_midnight_diff = noon_midnight_diff
;
;
;@*****
	ccm3_file =string(2, '("M:\MSU_AMSU_Data\MSU_Adjustments\MSU_Diurnal\CCM3_Diurnal_Climatology\mn_diur_cycles_chan_",i1,"_ccm3_144x72.dat")')
	sz = size(mean_tb_map,/dimensions)

	mean_tb_map_noon = mean_tb_map
	mean_tb_map_midnight = fltarr(144,72,sz[2])
    num_years = sz[2]/12

	for month = 1,12 do begin
		openr,lun,ccm3_file,/binary,/get_lun
		read_ccm3_diurnal_cycle_map,lun,0,month-1,d_cycle_map_0  ; this is a 144 x 72 x 24  array
		read_ccm3_diurnal_cycle_map,lun,1,month-1,d_cycle_map_1  ; this is a 144 x 72 x 24  array
		read_ccm3_diurnal_cycle_map,lun,2,month-1,d_cycle_map_2  ; this is a 144 x 72 x 24  array
		read_ccm3_diurnal_cycle_map,lun,3,month-1,d_cycle_map_3  ; this is a 144 x 72 x 24  array
		close,lun
		free_lun,lun

		d_cycle_map_TLT = -1.5*(d_cycle_map_0+d_cycle_map_1) + 2.0*(d_cycle_map_2+d_cycle_map_3)
		noon_midnight_diff = d_cycle_map_TLT[*,*,0] - d_cycle_map_TLT[*,*,11]

		for year = 0,num_years-1 do begin
			mean_tb_map_midnight(*,*,(year*12) + month -1) =  mean_tb_map_noon(*,*,(year*12) + month -1) + noon_midnight_diff
		endfor
	endfor

end

