;@***h* MLT_RSS/combine_msu_amsu_merged_maps_3_3.pro
;
; NAME
;   combine_msu_amsu_merged_maps_3_3.pro
;
; PURPOSE
;   Top level driver program for merging msu and amsu data together
;
; DESCRIPTION
;   Top level driver program for merging msu and amsu data together.  Reads in merged MSU and AMSU
;   data from netcdf files, computes merging parameters, calculates merged data, and writes a new 
;   netcdf file containing the merged data.
;
; FILE READ:
;   merged msu data file in netcdf format
;   e.g. RSS_Tb_Maps_ch_TMT_V3_3_011.nc
;   merged amsu data file in netcdf format
;   e.g. RSS_Tb_Maps_ch_05_V3_3_011.nc
; FILES WRITTEN
;   netcdf file containing msu amsu merged data
;   e.g. RSS_Tb_Maps_ch_TMT_V3_3_001.nc

; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   10/24/2012 Initial Version prepared for NCDC
;
; USAGE
;   top level program invoked by .r command in IDL environment
;
;
;@*****


; set some basic constants
defsysv,'!year_range',[1978,2012]
defsysv,'!num_AMSUs',7
defsysv,'!num_months', 12*(1+!year_Range[1]-!year_range[0])
defsysv,'!num_years',!num_months/12
defsysv,'!Num_GrPt_Lats',72
defsysv,'!Num_GrPt_Lons',144
defsysv,'!Num_Nodes',2
defsysv,'!Log_lun',9
host_set  ; sets !host to the local computer name
;period to use to calculate merge parameters
start_month 	= 252
end_month 		= 311


;period to use in merge
last_msu_month = 311
first_amsu_month = 249

;AMSU_L3_Path = 'C:\IDLWorkspace\Merge_Monthly_AMSU_Data_v3_3_op\results\'
AMSU_L3_Path = 'M:\MSU_AMSU_Data\AMSU-A_L3\'
;MSU_L3_Path =  'C:\IDLWorkspace\Merge_Monthly_MSU_Data_3_3\results\'
MSU_L3_Path =  'M:\MSU_AMSU_Data\AMSU-A_L3\'

AMSU_L3_Plot_Path='C:\IDLWorkspace\Combine_MSU_AMSU_v3_3_op\plots\'
 

channels = ['TLT','TMT','TTS','TLS']
process  = [ 1,    1,    1,   1]

for channel_index = 0,3 do begin
	channel = channels[channel_index]
	if process[channel_index] gt 0 then begin
		case channel of
			'TLT': begin
			  msu_version_num = 1
			  amsu_version_num = 11
				amsu_file = AMSU_L3_Path+'AMSU_only\RSS_Tb_Maps_ch_TLT_V3_3_'+string(amsu_version_num,format='(i3.3)')+'.nc'
				msu_file  = MSU_L3_Path+'MSU_only\RSS_Tb_Maps_ch_TLT_V3_3_'+string(msu_version_num,format='(i3.3)')+'.nc'
				version_num = 1
				end
			'TMT': begin
			  msu_version_num = 1
        amsu_version_num = 11
				msu_file  = MSU_L3_Path+'MSU_only\RSS_Tb_Maps_ch_TMT_V3_3_'+string(msu_version_num,format='(i3.3)')+'.nc'
				amsu_file = AMSU_L3_Path+'AMSU_only\RSS_Tb_Maps_ch_05_V3_3_'+string(amsu_version_num,format='(i3.3)')+'.nc'
				msu_channel = 2
				version_num=1
				end
			'TTS': begin
			  msu_version_num = 1
			  amsu_version_num = 11
				amsu_file = AMSU_L3_Path+'AMSU_only\RSS_Tb_Maps_ch_07_V3_3_'+string(amsu_version_num,format='(i3.3)')+'.nc'
				msu_file  = MSU_L3_Path+'MSU_only\RSS_Tb_Maps_ch_TTS_V3_3_'+string(msu_version_num,format='(i3.3)')+'.nc'
				msu_channel =3
				version_num = 1
				end
			'TLS': begin
			  msu_version_num = 1
        amsu_version_num = 11
				amsu_file = AMSU_L3_Path+'AMSU_only\RSS_Tb_Maps_ch_09_V3_3_'+string(amsu_version_num,format='(i3.3)')+'.nc'
				msu_file  = MSU_L3_Path+'MSU_only\RSS_Tb_Maps_ch_TLS_V3_3_'+string(msu_version_num,format='(i3.3)')+'.nc'
				msu_channel = 4
				version_num = 1
				end
		endcase

    defsysv,'!log_lun',2
    log_file = string(channel,version_num,format='("C:\IDLWorkspace\Combine_MSU_AMSU_v3_3_OP\logs\merge_log_amsu_channel_",a,"_v3.3.",i3.3,".log.txt")')
    close,!log_lun
    openw,!log_lun,log_file
    print,log_file
    
    log,'Info: MSU AMSU Combine V3.3 log file"
    log,'Info:','Channel = ',channel
    log,'Info:','Version Num = 3.3.'+string(version_num,format = '(i3.3)')
		log,'Info:  reading '+amsu_file

		read_monthly_msu_tbs_netcdf,file = amsu_file,								$
										longitude = longitude,						$
										latitude  = latitude,						$
										long_bounds = long_bounds,					$
										lat_bounds  = lat_bounds,					$
										months = months,							$
										mean_tb_maps = amsu_tbs_temp,					$
										offsets=amsu_offsets,						$
										target_factors=amsu_target_factors,			$
										tb_Factors=amsu_tb_factors,					$
										satellites_used = amsu_satellites_used_temp,		$
										date_L2C_processed = amsu_date_L2C_processed,    $
										global_Attributes = amsu_global_Attributes

    ; extend the AMSU arrays back in time to 1978 -- fill with NAN
    sz = size(amsu_tbs_temp,/dimensions)
    num_months = sz[2]+240
    amsu_tbs = replicate(!values.f_nan,144,72,num_months)
    amsu_satellites_used = bytarr(7,num_months)
    amsu_satellites_used[*,240:num_months-1] = amsu_satellites_used_temp
    amsu_tbs[*,*,240:num_months-1] = amsu_tbs_temp
		sz_amsu = size(amsu_tbs,/dimensions)
		amsu_tbs[*,*,247] = !values.f_nan

		log,'Info: reading '+msu_file

		read_monthly_msu_tbs_netcdf,file = msu_file,									$
										longitude = longitude,						$
										latitude  = latitude,						$
										long_bounds = long_bounds,					$
										lat_bounds  = lat_bounds,					$
										months = months,							$
										mean_tb_maps = msu_tbs_temp,					$
										offsets=msu_offsets,							$
										target_factors=msu_target_factors,				$
										tb_Factors=msu_tb_factors,						$
										satellites_used = msu_satellites_used_temp,			$
										global_Attributes = msu_global_Attributes


    ; extend the msu array so that it is the same length as the AMSU array
		sz_msu = size(msu_tbs_temp,/dimensions)
    msu_satellites_used = bytarr(9,sz_amsu[2])
    msu_satellites_used[*,0:sz_msu[2]-1] = msu_satellites_used_temp
		msu_tbs = replicate(!values.f_nan,144,72,sz_amsu[2])
		msu_tbs[*,*,0:sz_msu[2]-1] = msu_tbs_temp
		msu_tbs[*,*,last_msu_month+1:sz_amsu[2]-1] = !values.f_nan
		

		; fit annual harmonic fit to the difference time series at each point

		length = end_month-start_month+1
		create_harmonic_basis,2,length,12,basis,linear = 1,constant=1
		sigma_in = replicate(1.0,length)

		msu_amsu_diff = replicate(!values.f_nan,144,72,12)
		msu_amsu_diff_smooth = replicate(!values.f_nan,144,72,12)
		msu_amsu_diff_coeff = replicate(!values.f_nan,144,72,6)
		msu_amsu_lin_error = replicate(!values.f_nan,144,72)
		msu_amsu_mean_diff = replicate(!values.f_nan,144,72)
		msu_amsu_harm_diff = replicate(!values.f_nan,144,72)


    ; fit the differences at each point to a harmonic series.
		for ilat = 0,71 do begin
			for ilon = 0,143 do begin
				msu_ts = reform(msu_tbs[ilon,ilat,*])
				amsu_ts = reform(amsu_tbs[ilon,ilat,*])
				diff_ts = msu_ts - amsu_ts
				ok = where(finite(diff_ts),ct)

				if ct gt 12 then begin

					diff_ts2 = diff_ts[start_month:end_month]

					ok = where(finite(diff_ts2) eq 1,ct)
					if ct ge 12 then begin

						lin_regress_SVD,basis[*,ok],diff_ts2[ok],sigma_in[ok],A,sigma_out,fit,error
						a = real_part(a)
						fit_no_linear = a[0]*basis[0,*] + a[1]*basis[1,*] + a[2]*basis[2,*] + a[3]*basis[3,*] + a[5]*basis[5,*]
						msu_amsu_diff[ilon,ilat,*] = fit_no_linear[0:11]
						msu_amsu_diff_coeff[ilon,ilat,*] = a
						msu_amsu_lin_error[ilon,ilat] = a[4]*2.0  ; convert to K/decade
						msu_amsu_mean_diff[ilon,ilat] = a[5]
						msu_amsu_harm_diff[ilon,ilat] = sqrt(a[0]^2 + a[1]^2 + a[2]^2 + a[3]^2)

					endif
				endif
			endfor
		endfor

		; smooth the fitting parameters using a spherical harmonic fit
		; the degree of the fit is controlled by the L_max parameter, currently 9
		L_max = 9
    msu_amsu_diff_to_use = msu_amsu_diff
		if ((channel ne 'TMT') and (channel ne 'TLT')) then begin
			sigma_in = replicate(1.0,144,72)
			for i = 0,5 do begin
				y_lm_fit,msu_amsu_diff_coeff[*,*,i],sigma_in,L_max,fitted_map,coeff
				msu_amsu_diff_coeff[*,*,i] = fitted_map
			endfor

      ;reconstruct monthly differences from the smoothed fitting parameters
      
			for ilat = 0,71 do begin
				for ilon = 0,143 do begin
					a = reform(msu_amsu_diff_coeff[ilon,ilat,*])
					fit_no_linear = a[0]*basis[0,*] + a[1]*basis[1,*] + a[2]*basis[2,*] + a[3]*basis[3,*] + a[5]*basis[5,*]
					msu_amsu_diff_smooth[ilon,ilat,*] = fit_no_linear[0:11]
					msu_amsu_mean_diff[ilon,ilat] = a[5]
				endfor
			endfor
			msu_amsu_diff_to_use = msu_amsu_diff_smooth
		endif

		; adjust the AMSU data to match the MSU data using this harmonic fit

		amsu_tbs_adj = replicate(!values.f_nan,144,72,sz_amsu[2])
		diff_maps = replicate(!values.f_nan,144,72,sz_amsu[2])

		for m = first_amsu_month,sz_amsu[2]-1 do begin
			amsu_tbs_adj[*,*,m] = amsu_tbs[*,*,m] + msu_amsu_diff_to_use[*,*,(m mod 12)]
		endfor
		
		yrs = 1978.0 + 1.0/24.0 + findgen(!num_months)/12
		trend_diff_map = replicate(!values.f_nan,144,72)
		for ilon = 0,143 do begin
		  for ilat = 0,71 do begin
		    diff_ts_1999_2003 = reform(msu_tbs[ilon,ilat,252:252+59] - amsu_tbs_Adj[ilon,ilat,252:252+59])
		    ok = where(finite(diff_ts_1999_2003) eq 1,ct)
		    if ct gt 30 then begin
		      fit = linfit(yrs[0:59],diff_ts_1999_2003)
		      trend_diff_map[ilon,ilat] = 10.0*fit[1]
		    endif
		  endfor
		endfor
		      
		; combine the two products with equal wts equal to the number of satellites averaged

		combined_tbs = fltarr(sz_amsu[0],sz_amsu[1],sz_amsu[2])
		mn_diff = replicate(!values.f_nan,sz_amsu[2])
    mn_diff2 = replicate(!values.f_nan,sz_amsu[2])
    mn_diff3 = replicate(!values.f_nan,sz_amsu[2])
    mn_diff4 = replicate(!values.f_nan,sz_amsu[2])
		stddev_diff = replicate(!values.f_nan,sz_amsu[2])
		
		num_msus_used = total(msu_satellites_used,1)
		num_amsus_used = total(amsu_satellites_used,1)

		for m = 0,sz_amsu[2]-1 do begin

			ok_map 	= intarr(sz_amsu[0],sz_amsu[1])
			map 	= fltarr(sz_amsu[0],sz_amsu[1])

			mp_msu = msu_tbs[*,*,m]
			ok_msu = where(finite(mp_msu),ct_msu)
			if ct_msu gt 0 then begin
				ok_map[ok_msu] = ok_map[ok_msu] + num_msus_used[m]
				map[ok_msu] = map[ok_msu] + num_msus_used[m]*mp_msu[ok_msu]
			endif

			mp_amsu = amsu_tbs_adj[*,*,m]
			ok_amsu = where(finite(mp_amsu),ct_amsu)
			if ((ct_amsu gt 0) and (m ge start_month)) then begin
				ok_map[ok_amsu] = ok_map[ok_amsu] + num_amsus_used[m]
				map[ok_amsu] = map[ok_amsu] + num_amsus_used[m]*mp_amsu[ok_amsu]
			endif

			if ((ct_amsu gt 0) and (ct_msu gt 0)) then begin
				diff_map = mp_msu - mp_amsu
				diff_maps[*,*,m] = diff_map
				ok = where(finite(diff_map),ct)
				if ct gt 0 then begin
				  lat_wts = fltarr(72)
				  lat_wts[10:61]=1.0
					mn_diff[m] = mean(diff_map[ok])
          mn_diff2[m] = compute_global_average_from_144_72_map(diff_map,lat_wts); surf_str = surf_str,no_cosine = no_cosine,max_alt=max_alt
          mn_diff3[m] = compute_global_average_from_144_72_map(diff_map,lat_wts,surf_str = 'Sea');,no_cosine = no_cosine,max_alt=max_alt
          mn_diff4[m] = compute_global_average_from_144_72_map(diff_map,lat_wts,surf_str = 'Land');,no_cosine = no_cosine,max_alt=max_alt
					stddev_diff[m] = stddev(diff_map[ok])
				endif
			endif
			map = map/ok_map
			combined_tbs[*,*,m] = map
		endfor

		if channel eq 'TLT' then begin
			chng_to_midnight_TLT,combined_tbs,mean_tb_map_midnight,noon_midnight_diff = noon_midnight_diff
		endif else begin
			chng_to_midnight,msu_channel,combined_tbs,mean_tb_map_midnight,noon_midnight_diff = noon_midnight_diff
		endelse

		;add 0.29 offset into channel TMT for legacy agreement
		;this will end in version 4.0
		if channel eq 'TMT' then begin
			mean_tb_map_midnight = mean_tb_map_midnight+0.29
			noon_midnight_diff = noon_midnight_diff + 0.29
		endif
				
		write_adj_arrays_AMSU_nc,AMSU_L3_Path,channel,msu_amsu_diff_to_use,noon_midnight_diff
		
		; merge together offset, target factor, and Tb factor arrays
		offsets 		= transpose([transpose(msu_offsets),transpose(amsu_offsets)])
		target_factors 	= transpose([transpose(msu_target_factors),transpose(amsu_target_Factors)])
		tb_factors 	= transpose([transpose(msu_tb_factors),transpose(amsu_tb_Factors)])

		; merge together the satellites used variables

			sz_msu  = size(msu_satellites_used, /dimensions)
			sz_amsu = size(amsu_satellites_used,/dimensions)
			max_months = max([sz_msu[1],sz_amsu[1]])
			total_num_sat = sz_msu[0]+sz_amsu[0]
			satellites_used = intarr(total_num_sat,max_months)
			satellites_used[0:sz_msu[0]-1,0:sz_msu[1]-1] = msu_satellites_used
			satellites_used[sz_msu[0]:sz_msu[0]+sz_amsu[0]-1,0:sz_amsu[1]-1] = amsu_satellites_used

			date_L2C_processed = replicate(!values.d_nan,total_num_sat,max_months)
			date_L2C_processed[9:15,240:max_months-1] = amsu_date_L2C_processed

			write_msu_amsu_merged_Tbs_144_72_netcdf_3_3,				$
					Tbs_144_72 = mean_tb_map_midnight ,					$
					channel_str=channel,								$
					version_num=version_num,								$
					offsets = offsets,									$
					target_factors = target_factors,					$
					tb_Factors = tb_factors,							$
					satellites_used = satellites_used,					$
					date_L2C_processed = date_L2C_processed,			$
					msu_global_Attributes=msu_global_Attributes,		$
					amsu_global_Attributes=amsu_global_Attributes,		$
					msu_amsu_diff=msu_amsu_diff_to_use,  $
					output_path = AMSU_L3_Path+'MSU_AMSU\

	
	endif
endfor
end
