pro read_ccm3_diurnal_cycle_map,lun,fov,month,d_cycle_map

;@***h* MLT_RSS/read_ccm3_diurnal_cycle_map.pro

; NAME
;   read_ccm3_diurnal_cycle_map.pro
;
; PURPOSE
;   Reads ccm3 diurnal climatology map for a given month and fov 
;
; DESCRIPTION
;   Reads ccm3 diurnal climatology map for a given month and fov from an
;   already open file.
;   
;   FILES READ
;   ccm3 diurnal climatology file
;   e.g. mn_ccm3_diur_cycles_amsu_chan_09.dat
;
; INPUTS:
;   lun logical unit number of diurnal climatology file
;   fov  field of view number
;   month month of the diurnal cycle wanted
;
; OUTPUTS:
;  d_cycle_map 144 (longitude) by 72 (latitude) x 24(hours) array of 
;  climatological diurnal anomalies.

; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;  
; HISTORY
;   6/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_ccm3_diurnal_cycle_map,lun,fov,month,d_cycle_map;
;
;@*****

		start_pos = (4L*144L*72L*24L)*((fov*12) + month)
		point_lun,lun,start_pos
		d_cycle_map = fltarr(144,72,24)
		readu,lun,d_cycle_map
		return

end

