pro write_msu_amsu_merged_Tbs_144_72_netcdf_3_3,	Tbs_144_72 = Tbs_144_72 ,			$
													channel_str=channel_str,			$
													version_num=version_num,			$
													offsets = offsets,					$
													target_factors = target_factors,	$
													tb_Factors = tb_factors,			$
													satellites_used = satellites_used,	$
													date_L2C_processed = date_L2C_processed,	$
													global_Attributes = global_attributes,		$
													msu_global_Attributes=msu_global_Attributes,   $
													amsu_global_Attributes=amsu_global_Attributes, $
													msu_amsu_diff=msu_amsu_diff, $
													nc_file = nc_file,  $
													output_path = output_path
													
;@***h* MLT_RSS/write_msu_amsu_merged_Tbs_144_72_netcdf_3_3.pro

; NAME
;   write_msu_amsu_merged_Tbs_144_72_netcdf_3_3.pro
;
; PURPOSE
;   writes merged results to netcdf file
;
; DESCRIPTION
;   writes merged results to netcdf file
;
; INPUTS:
;   Tbs_144_72 = Tbs_144_72 ,     $           ; brightness temperatures
;   channel_str=channel_str,      $           ; string containing channel name ('TLT, etc)
;   version_num=version_num,      $           ; version number
;   offsets = offsets,          $             ; offsets used in intratype merge
;   target_factors = target_factors,  $       ; target factors used in intratype merge
;   tb_Factors = tb_factors,      $           ; scene temepratuer factors used in intratype merge
;   satellites_used = satellites_used,  $     ; array of satellites used
;   date_L2C_processed = date_L2C_processed,  $    ; array of processing dates for L2C files
;   global_Attributes = global_attributes,    $    ; new global arrtibutes
;   msu_global_Attributes=msu_global_Attributes,   $  ; msu-only global atrributes
;   amsu_global_Attributes=amsu_global_Attributes, $  ; amsu_only global attributes
;   msu_amsu_diff=msu_amsu_diff, $                    ; 144x72x12 array of msu amsu differences -- adjustments have been applied to AMSU data
;   nc_file = nc_file,  $                             ; file name for netcdf file
;   output_path = output_path                         ; location to write netcdf file
;
; FILES WRITTEN:
;   netcdf file containing msu amsu merged data

; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   6/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   write_msu_amsu_merged_Tbs_144_72_netcdf_3_3,  Tbs_144_72 = Tbs_144_72 ,     $
;                          channel_str=channel_str,      $
;                          version_num=version_num,      $
;                          offsets = offsets,          $
;                          target_factors = target_factors,  $
;                          tb_Factors = tb_factors,      $
;                          satellites_used = satellites_used,  $
;                          date_L2C_processed = date_L2C_processed,  $
;                          global_Attributes = global_attributes,    $
;                          msu_global_Attributes=msu_global_Attributes,   $
;                          amsu_global_Attributes=amsu_global_Attributes, $
;                          msu_amsu_diff=msu_amsu_diff, $
;                          nc_file = nc_file,  $
;                          output_path = output_path
;
;
;@*****

	fill_value = -9999.0
	sz = size(Tbs_144_72,/dimensions)
	num_lons = sz[0]
	num_lats = sz[1]
	num_months = sz[2]
	sz2 = size(satellites_used,/dimensions)
	num_sats = sz2[0]


	Tbs = Tbs_144_72
	bad = where(finite(Tbs) eq 0,ct)
	if ct gt 0 then begin
		Tbs[bad] = fill_value
	endif

	todays_date = current_date_string()

  if keyword_set(output_path) eq 0 then begin
	  get_AMSU_File_Locations,AMSU_L3_Path = AMSU_L3_Path
    output_path = 		AMSU_L3_Path+'MSU_AMSU\'
  endif
	if keyword_set(nc_file) eq 0 then begin
		nc_file = output_path + 'RSS_Tb_Maps_ch_'+channel_str+'_V3_3_'+string(version_num,format='(i3.3)')+'.nc'
	endif

	print,nc_file


	id = NCDF_CREATE(nc_file, /CLOBBER) ; Create the netCDF file.
	NCDF_ATTPUT, id, 'title', 'RSS Version 3.3 AMSU brightness temperatures, channel '+channel_str, /GLOBAL
	NCDF_ATTPUT, id, 'institution', 'Remote Sensing Systems, Santa Rosa, CA, USA', /GLOBAL

	source_str = 'Microwave Sounding Unit (MSU) and Advanced Microwave Sounding Unit (AMSU) satellite-borne microwave radiometer, channel '+channel_str

	a = SCOPE_TRACEBACK()
	main_program_name = a[0]

	NCDF_ATTPUT, id, 'history', todays_date + ' produced from version 3.2 RSS monthly merged monthly maps for MSU and AMSU using the idl routine '+main_program_name, /GLOBAL
	NCDF_ATTPUT, id, 'references', 'C. A. Mears, M. Schabel, and F. J. Wentz: A Reanalysis of the MSU Channel 2 Tropospheric Temperature Record, Journal of Climate 16, pp. 3650-3664', /GLOBAL

	comment_str =  $
				'This is MSU / AMSU combined data'

	NCDF_ATTPUT, id, 'comment',comment_str , /GLOBAL

	; unpack the  global, msu and amsu attribute structures and put them in this file with appropriate prefix
    if keyword_set(global_attributes) then begin
		attr_names = tag_names(global_Attributes)
		num_attr = n_elements(attr_names)
		for i = 0,num_attr-1 do begin
			name = attr_names(i)
			value = global_attributes.(i)
			NCDF_ATTPUT,id,name,value,/GLOBAL
		endfor
	endif


	msu_attr_names = tag_names(msu_global_Attributes)
	num_msu_attr = n_elements(msu_attr_names)
	for i = 0,num_msu_attr-1 do begin
		name = msu_attr_names(i)
		value = msu_global_attributes.(i)
		name = 'msu_'+name
		NCDF_ATTPUT,id,name,value,/GLOBAL
	endfor

	amsu_attr_names = tag_names(amsu_global_Attributes)
	num_amsu_attr = n_elements(amsu_attr_names)
	for i = 0,num_amsu_attr-1 do begin
		name = amsu_attr_names(i)
		value = amsu_global_attributes.(i)
		name = 'amsu_'+name
		NCDF_ATTPUT,id,name,value,/GLOBAL
	endfor

	; define dimensions for variables

	x_dim      = NCDF_DIMDEF(id, 'longitude', num_lons) 		; Define the X dimension.
	x_edge_dim = NCDF_DIMDEF(id, 'longitude_bounds', num_lons+1) 	    ; Define the X dimension cell edges.
	y_dim      = NCDF_DIMDEF(id, 'latitude', num_lats) 		    ; Define the Y dimension.
	y_edge_dim = NCDF_DIMDEF(id, 'latitude_bounds', num_lats+1) 		; Define the Y dimension cell edges.
	time_dim   = NCDF_DIMDEF(id, 'months', num_months) 			; Define the month dimension.
	month_dim  = NCDF_DIMDEF(id, 'month_of_year', 12)					; Define the month dimension for one year
	sat_dim    = NCDF_DIMDEF(id, 'satnum', num_sats) 			; Define the satellite dimension.


	; define the data variables

	lon_id   			= NCDF_VARDEF(id, 'longitude', 			[x_dim], /FLOAT)
		NCDF_ATTPUT,id,lon_id,'long_name','longitude'
		NCDF_ATTPUT,id,lon_id,'units','degrees east'
		NCDF_ATTPUT,id,lon_id,'bounds','longitude_bounds'
	lat_id   			= NCDF_VARDEF(id, 'latitude', 			[y_dim], /FLOAT)
		NCDF_ATTPUT,id,lat_id,'long_name','latitude'
		NCDF_ATTPUT,id,lat_id,'units','degrees north'
		NCDF_ATTPUT,id,lat_id,'bounds','latitude_bounds'
	lon_boundary_id   	= NCDF_VARDEF(id, 'longitude_bounds', 	[x_edge_dim], /FLOAT)
		NCDF_ATTPUT,id,lon_boundary_id,'long_name','longitude_bounds'
		NCDF_ATTPUT,id,lon_boundary_id,'units','degrees east'
	lat_boundary_id   	= NCDF_VARDEF(id, 'latitude_bounds', 	[y_edge_dim], /FLOAT)
		NCDF_ATTPUT,id,lat_boundary_id,'long_name','latitude_bounds'
		NCDF_ATTPUT,id,lat_boundary_id,'units','degrees north'
	time_id 			= NCDF_VARDEF(id, 'months', 			[time_dim], /LONG)
		NCDF_ATTPUT,id,time_id,'long_name','time'
		NCDF_ATTPUT,id,time_id,'units','months since 1978-1-1 0:0:0'
	temperature_id  = NCDF_VARDEF(id, 'brightness_temperature',[x_dim,y_dim,time_dim], /FLOAT)
		NCDF_ATTPUT,id,temperature_id,'long_name','monthly averaged single channel brightness temperature'
		NCDF_ATTPUT,id,temperature_id,'units','degrees K'
		NCDF_ATTPUT,id,temperature_id,'_FillValue',fill_value
	months_used_id  = NCDF_VARDEF(id, 'satellites_used',[sat_dim,time_dim], /BYTE)
	    NCDF_ATTPUT,id,months_used_id,'long_name','satellites used by month'
		NCDF_ATTPUT,id,months_used_id,'units','none'
		NCDF_ATTPUT,id,months_used_id,'comment','1 when satellite used, 0 otherwise'
	date_L2C_processed_id = NCDF_VARDEF(id,'date_L2C_processed',[sat_dim,time_dim],/DOUBLE)
		NCDF_ATTPUT,id,date_L2C_processed_id,'long_name','date satellite processed in IDL julday format'
		NCDF_ATTPUT,id,date_L2C_processed_id,'units','julian day number'
	offset_id		= NCDF_VARDEF(id, 'offset_values',[y_dim,sat_dim],/FLOAT)
		NCDF_ATTPUT,id,offset_id,'long_name','satellite calibration offset for each latitude'
		NCDF_ATTPUT,id,offset_id,'units','degrees K'
	target_factor_id= NCDF_VARDEF(id,'target_factor_values',[y_dim,sat_dim],/FLOAT)
		NCDF_ATTPUT,id,target_factor_id,'long_name','satellite calibration target factor for each latitude'
		NCDF_ATTPUT,id,target_factor_id,'units','dimensionless'
	tb_factor_id= NCDF_VARDEF(id,'tb_factor_values',[y_dim,sat_dim],/FLOAT)
		NCDF_ATTPUT,id,target_factor_id,'long_name','satellite calibration tb factor for each latitude'
		NCDF_ATTPUT,id,target_factor_id,'units','dimensionless'
	msu_amsu_diff_id = NCDF_VARDEF(id,'msu_amsu_differences',[x_dim,y_dim,month_dim], /FLOAT)
		NCDF_ATTPUT,id,msu_amsu_diff_id,'long_name','adjustments applied to amsu to match msu for each location and month of year'
		NCDF_ATTPUT,id,msu_amsu_diff_id,'units','degrees K'

	; switch to data mode and write data

	NCDF_CONTROL, id, /ENDEF ; Put the file into data mode.

	latitude   		= (indgen(num_lats)*2.5) -90.0+1.25
	longitude  		= 1.25 +(indgen(num_lons) * 2.5)
	latitude_bnd   	= (indgen(num_lats+1)*2.5) -90.0
	longitude_bnd  	= indgen(num_lons+1) * 2.5
	months      	= indgen(num_months)

	NCDF_VARPUT, id, lon_id, longitude
	NCDF_VARPUT, id, lat_id, latitude
	NCDF_VARPUT, id, lon_boundary_id, longitude_bnd
	NCDF_VARPUT, id, lat_boundary_id, latitude_bnd
	NCDF_VARPUT, id, time_id, months
	NCDF_VARPUT, id, temperature_id, Tbs
	NCDF_VARPUT, id, months_used_id, satellites_used
	NCDF_VARPUT, id, date_L2C_processed_id,date_L2C_processed
	NCDF_VARPUT, id, offset_id,offsets
	NCDF_VARPUT, id, target_factor_id,target_factors
	NCDF_VARPUT, id, tb_factor_id,tb_factors
	NCDF_VARPUT, id, msu_amsu_diff_id,msu_amsu_diff


	NCDF_CLOSE, id ; Close the NetCDF file.

end

