pro y_lm_fit,map,sigma_in,l_max,fitted_map,coeff,trim = trim,init=init

;@***h* MLT_RSS/y_lm_fit.pro

; NAME
;   y_lm_fit.pro
;
; PURPOSE
;   Fits a 144 x 72 gridded map using a series of spherical harmonics
;
; DESCRIPTION
;   Fits a 144 x 72 gridded map using a series of spherical harmonics
;
; INPUTS:
;   map         ; 144 x 72 array : map to be fitted
;   sigma_in,   ; 144 x 72 array of estimated errors
;   l_max       ; largest l coefficient to be used -- maximum is L = 14
;   trim = trim ; if set, outliers are trimmed.  e.g. if set to 0.8, 80% of the values are kept
;   init=init   ; set if need to recompute YLM basis
;
; OUTPUTS:
;   fitted_map  ; spherical harmonic fit
;   coeff       ; coefficients of fit
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   6/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   y_lm_fit,map,sigma_in,l_max,fitted_map,coeff,trim = 0.8,init=1
;
;@*****
	common ylm_map,y_lm_map,done
    ;done = 0
	if ((keyword_set(done) eq 0) or (keyword_set(init) eq 1)) then begin
		y_lm_map = fltarr(144,72,225)  ; contains y_lm_values for each lat/lon up to L = 14
		for lat_i = 0,71 do begin
			for lon_i = 0,143 do begin

				n_col = 0

				lat = -90.0 + 1.25 + lat_i * 2.5
				lon = 1.25 + lon_i * 2.5

			     for l = 0,14 do begin
			          for m = 0,l do begin
	     				if (m eq 0) then begin
	     					y_lm_map(lon_i,lat_i,n_col) = y_lm_cosd(l,m,90.0D0-lat,lon)
	     					n_col = n_col+1
	     				endif else begin
	     					y_lm_map(lon_i,lat_i,n_col) = y_lm_cosd(l,m,90.0D0-lat,lon)
	     					n_col = n_col+1
	     					y_lm_map(lon_i,lat_i,n_col) = y_lm_sind(l,m,90.0D0-lat,lon)
	     					n_col = n_col+1
	     				endelse
	     			endfor
	     		endfor
	     	endfor
	     endfor

		;print,'Finished Calculating Y_lms'
		done = 1
	endif else begin

		;print,'Using Precalculated Y_lms'
	endelse




	; fits map data to a series of real Ylms
	mp = map
	sz = size(mp)

	num_x = sz[1]
	num_y = sz[2]
	dx = 360.0/float(num_x)
	dy = 180.0/float(num_y)

	tot_num = num_x*num_y

	num_col = (l_max+1)^2
	y_lm    = fltarr(num_col,tot_num)
	y    = fltarr(tot_num)
	sig  = fltarr(tot_num)
	ix_ok   = intarr(tot_num)
	iy_ok   = intarr(tot_num)

	if keyword_set(trim) then begin
		if (trim gt 0.0) and (trim lt 1.0) then begin
			ok = where(finite(mp) eq 1,ct)
			if ct gt 0 then begin
				map_ok = mp[ok]
				s = sort(map_ok)
				lower_limit = floor(((1.0 - trim)/2.0)*ct)
				upper_limit = ct-1-lower_limit
				lower_trim_value = map_ok[s[lower_limit]]
				upper_trim_value = map_ok[s[upper_limit]]
				not_ok = where((mp lt lower_trim_value) or (mp gt upper_trim_value))
				mp[not_ok] = !values.f_nan
			endif
		endif
	endif


	n = 0
     for ix = 0,num_x-1 do begin
          for iy = 0,num_y-1 do begin
          	if finite(mp[ix,iy]) then begin
          		ix_ok[n] = ix
          		iy_ok[n] = iy
				y_lm[*,n] = y_lm_map[ix,iy,0:num_col -1]
	     		y[n] = mp[ix,iy]
	     		sig[n] = sigma_in[ix,iy]
	     		n = n+1
	     	endif
     	endfor
     endfor

	if n gt l_max then begin
		y_lm = y_lm[*,0:n-1]
		y    = y[0:n-1]
		sig  = sig[0:n-1]

		lin_regress,y_lm,y,sig,coeff,sigma_out,fit,error

		fitted_map = fltarr(num_x,num_y)

	   	for n_col = 0,num_col - 1 do begin

			fitted_map = fitted_map + coeff[n_col]*y_lm_map[*,*,n_col]

		endfor
	endif else begin
		fitted_map = replicate(!values.f_nan,num_X,num_y)
	endelse

	;map = mp

	return
end




