pro apply_offsets_to_grpt_maps,tb_arr_adj,offsets

;@***h* MLT_RSS/apply_TF_to_grpt_maps.pro
; NAME
;   apply_TF_to_grpt_maps.pro
;
; PURPOSE
;   applies MSU offset to grid point maps to produce an adjusted dataset. 
;
; DESCRIPTION
;   Applies the target factors using the method described in 
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the Remote Sensing Systems V3.2 atmospheric temperature records from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1040-1056.
;   
;   and 
;   
;   Mears, CA, FJ Wentz, 2009, 
;   Construction of the RSS V3.2 lower tropospheric dataset from the MSU and AMSU microwave sounders 
;   Journal of Atmospheric and Oceanic Technology, 26, 1493-1509.
;
; INPUTS
;   tb_arr_adj          (144,72,num_months,num_sats) real(4) array of brightness temperatures
;   offsets  (num_lats,num_sats) float 32 array of latitude-dependent offsets
;
; OUTPUTS
;   tb_arr_adj      (144,72,num_months,num_sats) real(4) array of adjusted brightness temperatures (tb_arr_adj is modified in place)
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   3/02/2012 Initial Version prepared for NCDC
;   
; USAGE
;   apply_offsets_to_grpt_maps,tb_arr_adj,offsets
;
;@*****

	; compute the offsets separately for each latitude
	sz = size(tb_arr_adj,/dimensions)

	
	num_lats 	= sz[1]
	num_sats 	= sz[3]

	for ilat = 0,num_lats-1 do begin
		for satnum = 0,num_sats-1 do begin
			tb_arr_adj[*,ilat,*,satnum] = tb_arr_adj[*,ilat,*,satnum]-offsets[ilat,satnum]
		endfor
	endfor
end