;@***h* MLT_RSS/grpt_merge_AMSU_only_3_3.pro

; NAME
;   grpt_merge_MSU_only_3_3.pro
;
; PURPOSE
;   Top-level program that performs adjustments to MSU monthly gridded data, and then 
;   merges the data together into a single MSU derived brightness temperature dataset
;
; DESCRIPTION
;   Reads gridded monthly averages from individual files.  We call these L2C files.
;   Calculates and applies target factors using singular value decomposition
;   Calculates and applies latitude dependent offsets using singular value decomposition
;   Scene temperature factors are set to zero for AMSU
;   Merges data from the chosen set of satellite together using equal weighting
;   Writes the results to a netcdf file.  We call these files L3.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   grpt_merge_MSU_only_3_3
;@*****
print_global_pars = 1
; basic constants
  defsysv,'!year_range',[1978,2005]
  defsysv,'!num_MSUs' ,9
  defsysv,'!num_months', 12*(1+!year_Range[1]-1978)
  defsysv,'!num_years',!num_months/12
  defsysv,'!Num_GrPt_Lats',72
  defsysv,'!Num_GrPt_Lons',144
  defsysv,'!Num_Nodes',2
  defsysv,'!Num_Hist_Datatypes',7
  defsysv,'!Log_lun',9

; Paths imported from Fortran

  load_AMSU_file_locations
  ; fortran_file = '\\dell-p1700d\e\AMSU_processing\fortran_dev\Source\AMSU_File_Locations.f90'

; Format strings for constructing L2B file names

;  defsysv,'!format_Str_Tb','("'+!AMSU_L2C_path+'zonal_pentads\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_raw_mo_1995_direct.dat")'
;  defsysv,'!msu_format_Str_Tb','("'+!AMSU_L2C_path+'zonal_pentads\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_raw_mo_1995_direct.dat")'
;  defsysv,'!format_Str_Tb_v02_1','("'+!AMSU_L2C_path+'zonal_pentads_v02_1\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_raw_mo_1995_direct.dat")'
;  defsysv,'!format_Str_inc','("'+!AMSU_L2C_path+'zonal_pentads\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_inc_direct.dat")'
;  defsysv,'!format_Str_limb','("'+!AMSU_L2C_path+'zonal_pentads\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_limb_direct.dat")'
;  defsysv,'!format_Str_diur','("'+!AMSU_L2C_path+'zonal_pentads\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_diurnal_direct.dat")'
;  defsysv,'!format_Str_climate','("'+!AMSU_L2C_path+'zonal_pentads\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Pentad_36_",a,"_clim_direct.dat")'
;  defsysv,'!format_str_target','("'+!AMSU_L2C_path+'zonal_pentads\MSU_target\L1A_",i2.2,"_target_",i1,"_Zonal_Pentad_36_",a,"_direct.dat")'
;  defsysv,'!format_str_target_v4','("'+!AMSU_L2C_path+'zonal_pentads_v4\MSU_target\L1A_",i2.2,"_target_",i1,"_Zonal_Pentad_36_",a,"_direct.dat")'
;  defsysv,'!format_str_target_v02_1','("'+!AMSU_L2C_path+'zonal_pentads_v02_1\MSU_target\L1A_",i2.2,"_target_",i1,"_Zonal_Pentad_36_",a,"_direct.dat")'
;  defsysv,'!format_Str_zonal_monthly_Tb',    '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_mo_1995.dat")'
;  defsysv,'!format_Str_zonal_monthly_limb',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_limb.dat")'
;  defsysv,'!format_Str_zonal_monthly_diur',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_diurnal.dat")'
;  defsysv,'!format_str_zonal_monthly_target','("'+!AMSU_L2C_path+'zonal_months\AMSU_target\AMSU_",i2.2,"_channel_",i1,"_Target_Zonal_Month_36.dat")'
;  defsysv,'!format_Str_zonal_monthly_Tb_v3',    '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i2.2,"_Zonal_Month_36_v3_01.dat")'
;  defsysv,'!format_Str_zonal_monthly_inc_v3',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i2.2,"_Zonal_Month_36_inc_v3_01.dat")'
;  defsysv,'!format_Str_zonal_monthly_limb_v3',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i2.2,"_Zonal_Month_36_nadir_v3_01.dat")'
;  defsysv,'!format_Str_zonal_monthly_diur_v3',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i2.2,"_Zonal_Month_36_diurnal_v3_01.dat")'
;  defsysv,'!format_str_zonal_monthly_target_v3','("'+!AMSU_L2C_path+'zonal_months\AMSU_target\AMSU_",i2.2,"_channel_",i2.2,"_Target_Zonal_Month_36_v3_01.dat")'
;  defsysv,'!format_Str_zonal_monthly_Tb_v3_02',    '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_v3_02.dat")'
;  defsysv,'!format_Str_zonal_monthly_inc_v3_02',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_inc_v3_02.dat")'
;  defsysv,'!format_Str_zonal_monthly_limb_v3_02',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_nadir_v3_02.dat")'
;  defsysv,'!format_Str_zonal_monthly_diur_v3_02',  '("'+!AMSU_L2C_path+'zonal_months\AMSU_channel_",i2.2,"\AMSU_",i2.2,"_chan_",i1,"_Zonal_Month_36_diurnal_v3_02.dat")'
;  defsysv,'!format_str_zonal_monthly_target_v3_02','("'+!AMSU_L2C_path+'zonal_months\AMSU_target\AMSU_",i2.2,"_channel_",i2.2,"_Target_Zonal_Month_36_v3_02.dat")'
;  defsysv,'!format_Str_msu_zonal_monthly_Tb',    '("'+!AMSU_L2C_path+'zonal_months\MSU_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Month.dat")'
;  defsysv,'!format_Str_msu_zonal_monthly_inc',  '("'+!AMSU_L2C_path+'zonal_months\msu_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Month_inc.dat")'
;  defsysv,'!format_Str_msu_zonal_monthly_limb',  '("'+!AMSU_L2C_path+'zonal_months\msu_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Month_limb.dat")'
;  defsysv,'!format_Str_msu_zonal_monthly_diur',  '("'+!AMSU_L2C_path+'zonal_months\msu_channel_",i1,"\L1A_",i2.2,"_chan_",i1,"_Zonal_Month_diurnal.dat")'
;  defsysv,'!format_str_msu_zonal_monthly_target','("'+!AMSU_L2C_path+'zonal_months\msu_target\L1A_",i2.2,"_Target_",i1,"_Zonal_Month.dat")'


  defsysv,'!AMSU_L3_Path','C:\MSU_AMSU\IDL_MSU_AMSU\Merge_Monthly_AMSU_Data_3_3\results\'
  defsysv,'!AMSU_L3_Plot_Path','C:\MSU_AMSU\IDL_MSU_AMSU\Merge_Monthly_AMSU_Data_3_3\plots\'


 read_land_fraction_144_72,lf
 land_mask = intarr(144,72)
 land = where(lf gt 0.5)
 land_mask[land] = 1.0

 channels = ['TLT','TMT','TTS','TLS']
 process  = [ 1,   1,    1,    1]

for channel_index = 0,3 do begin
	if process[channel_index] gt 0 then begin
		channel = channels[channel_index]
		case channel of
			'TLT': begin	
			  	channel_str='TLT'
        			num_thres = 60
				end
			'TMT': begin
				channel_str='2'
				num_thres = 40
				end
			'TTS': begin
				channel_str='3'
				num_Thres = 40
				end
			'TLS': begin
				channel_str='4'
				num_thres = 40
				end
		endcase

		sats_to_use = intarr(9)
		
		if channel eq 'TTS' then begin
			sats_to_use[5:8] = 1
		endif else begin
			sats_to_use[0:8] = 1
		endelse

		;read in the monthly map data
		ocean_only = 0
		tf_surf = 'Land_and_Sea'
    		version_num = 011 & diurnal_model = 'CCM3' & preprocess_rss = 1  ; this is the version used for operational V3.3
    
    		if channel ne 'TLT' then begin
     	           read_all_monthly_maps_msu_only_v3_3, channel,          $
                        tb_arr,              $
                        tar_arr,             $
                        mn_day_arr,          $
                        tb_clim_arr,         $
                        num_arr,             $
                        file_list=file_list, $
                        diurnal_model = diurnal_model
    endif else begin
      read_all_monthly_maps_msu_only_tlt_V3_3, channel,          $
                        tb_arr,           $
                        tar_arr,          $
                        mn_day_arr,         $
                        tb_clim_arr,        $
                        num_arr,          $
                        file_list=file_list, $
                        diurnal_model = diurnal_model
    endelse
    	
		sz = size(tb_arr,/dimensions)
		num_months = sz[2]

	  if channel eq 'TLT' then begin
			read_topography_144x72,hgt
			bad = where(hgt gt 3000.0)
			for m = 0,num_months-1 do begin
				for sat = 0,8 do begin
					map = tb_arr[*,*,m,sat]
					map[bad] = !values.f_nan
					map[*,0:7] = !values.f_nan
					map[*,69:71] = !values.f_nan
					tb_arr[*,*,m,sat] = map
				endfor
			endfor
		endif
		
		if preprocess_rss then begin  ; convert antenna temperature to approximate brightness temperature 
		                              ; by assuming T cold space is 4.77 K
		                              ; initial processing assumed cold space was 2.73 K
                            tb_arr = 4.77 + (tar_arr-4.77)*(tb_arr-2.73)/(tar_arr-2.73)
                endif

		; find the target factors from grid_point data.  The adjusted data is returned in tb_arr_adj.
		; the target factors are contained in the merge_parameters variable..

		calc_TF_from_grpt_maps,	tb_arr,					$
								tar_arr,			$
								num_arr,			$
								channel,			$
								sats_to_use,			$
								target_factors,			$
								target_means,			$
								offsets,			$
								months_to_use,			$
								num_thres=num_thres, $
								trop_tb_arr = trop_tb_arr,  $
								trop_tar_arr = trop_tar_arr, $
								surf_str = tf_surf

		if channel eq 'TTS' then target_factors[0:4] = !values.f_nan
 
		if keyword_Set(print_global_pars) then begin
			print,target_factors,format = '("target_Factors = [",8(f8.4,","),f8.4,"]")'
			print,target_means,format = '("target_means = [",8(f8.4,","),f8.4,"]")'
			print,offsets,format = '("offsets = [",8(f8.4,","),f8.4,"]")'
		endif

    ; apply the target factors to the grpt maps
		apply_TF_to_grpt_maps,tb_arr,tar_arr,channel,target_factors,target_means,months_to_use,tb_arr_adj
  
    ;calculate offsets as a function of latitude
    calc_offsets_from_grpt_maps,tb_arr_adj,months_to_use,offsets
    
    ;smooth offsets
    smooth_offsets,offsets,offsets_smooth
    
    ;apply offsets to grpt maps
    apply_offsets_to_grpt_maps,tb_arr_adj,offsets_smooth
    
    ;calculate single target factors for each satellite
		calc_single_TbF_from_grpt_map,tb_arr_adj,tb_clim_arr,months_to_use,tb_factors		
		
		; merge the adjusted data together
		tb_arr_merged = replicate(!values.f_nan,144,72,336)
		print,'Merging'
		pixel_thres = 10
		for i = 0,num_months-1 do begin
			tot_tb = fltarr(144,72)
			tot_num = fltarr(144,72)
			for satnum = 0,8 do begin
				if months_to_use[i,satnum] lt 1 then continue
				tb = tb_arr_adj[*,*,i,satnum]
				num = num_arr[*,*,i,satnum]
				ok = where((finite(tb) gt 0) and (num gt pixel_thres),ct)
				if ct gt 5000 then begin
					tot_tb[ok] = tot_tb[ok] + tb[ok]*num[ok]
					tot_num[ok] = tot_num[ok] + num[ok]
				endif
			endfor ; satnum
			tb_merged = tot_tb/tot_num
			tb_arr_merged[*,*,i] = tb_merged
		endfor
		
		; remove data before november 1978 and after Dec 2004
		tb_arr_merged[*,*,324:335] = !values.f_nan
		tb_arr_merged[*,*,0:9] = !values.f_nan
		
		; expand target factor and tbf into arrays by latititude for writing to netcdf
		tf_array = fltarr(72,9)
		tbf_Array = fltarr(72,9)
		for ilat = 0,71 do begin
			tf_array[ilat,*]  = target_factors
			tbf_array[ilat,*] = tb_factors
		endfor

		; write the netcdf file

		write_msu_only_merged_Tbs_144_72_netcdf_3_3,					$
										Tbs_144_72 = tb_arr_merged ,  	$
										channel_str=channel,			$
										version_num=version_num, 		$
										source_files = file_list,		$
										offsets = offsets,				$
										target_factors = tf_array,		$
										tb_factors = tbf_array, 		$
										months_used = months_to_use
									
	endif
endfor
end