pro read_msu_monthly_histograms_one_sat,channel,satnum,monthly_histograms_one_sat,error,file=file

;@***h* MLT_RSS/read_msu_monthly_histograms_one_sat.pro
;
; NAME
;   read_msu_monthly_histograms_one_sat.pro
;
; PURPOSE
;   finds and reads file containing monthly gridded averages for a 
;   single msu satellite and channel
;
; DESCRIPTION
;   Reads a number of flat binary file with the monthly averaged data.  Each file is
;   real(4) little endian.
;   
;   The global variables the describe the size of this file and its general location must already be set.  These are:
;     !AMSU_L2C_Path
;     !Num_months
;     !Num_hist_datatypes
;     !Num_Nodes
;     !Num_GrPt_Lons
;     !Num_GrPt_Lats
;     !Num_months
;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   3/02/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_msu_monthly_histograms_one_sat,channel,satnum,monthly_histograms_one_sat,error,file=file
;   
;     channel          is the MSU channel to be read (2,3,4)
;     satnum           is the satellite number of the AMSU satellite to be read (1 = TIROS-N, 2 = NOAA-06, 3 = NOAA-07, etc)
;     monthly_histgrams_one_sat[!Num_hist_datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats] is a float(4) array
;     error            is set to 0 if the read was successful, -1 otherwise
;     file             is the file name that was read
;
;@*****

monthly_histograms_one_sat = fltarr(!Num_Hist_Datatypes,!Num_Nodes,!Num_GrPt_Lons,!Num_GrPt_Lats,!Num_months)
file = string(!AMSU_L2C_Path,channel,satnum,channel,format='(a,"Monthly_Maps\MSU_Channel_",i1,"\MSU_",i2.2,"_Chan_",i1,"_Monthly_Hist_144_72_N5_V3_0.dat")')
print,'Reading ',file

close,3
openr,3,file
for j = 0,335 do begin
	read_monthly_histograms_V3_0,3,month_num,monthly_hist,error
	if error eq 0 then begin
		if month_num gt 0 then monthly_histograms_one_sat[*,*,*,*,month_num-1] = monthly_hist
	endif else begin
		break
	endelse
endfor
close,3
end

