function check_path,file,create=create
;@***h* MLT_RSS/check_path.pro

; NAME
;   check_path.pro
;
; PURPOSE
;   Makes sure the path to the filename is valid
;
; DESCRIPTION
;   Check to see if the path to a given filename is valid.  If the create keyword is
;   set, creates the path if possible
; 
; INPUTS:
;   file:   filename of file to check
;   create: Keyword.  If set to 1, then directory is created if it does not exist
;   
; RETURN VALUE:
;     0 is path is now valid
;     -1 if path is not valid
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
; HISTORY
;   2/29/2019 Initial Version prepared for NCDC
;
; USAGE
;   check_path,path,/create
;   
;@*****

  ; look for last slash
  slash_pos = 1+strpos(file,'\',/reverse_search)
  if slash_pos gt 0 then begin
    path = strmid(file,0,slash_pos)
    path_info = file_info(path)
    if path_info.exists eq 0 then begin
      if keyword_set(create) then begin
        file_mkdir,path
        return,1
      endif else begin
        return,0
      endelse
    endif else begin
      return,1
    endelse
  endif
  return,1
  
  end