pro create_harmonic_basis,num_harmonics,length,period,basis,linear = linear,constant=constant,delta_t = delta_t
;@***h* MLT_RSS/create_harmonic_basis.pro

; NAME
;   create_harmonic_basis.pro
;
; PURPOSE
;   creates a basis of harmonic functions for fitting purposes
;   
; DESCRIPTION
;   creates a basis of harmonic functions for fitting purposes
;
; INPUTS:
;   num_harmonics         ; number of harmonics in basis
;   length                ; length of basis in time steps
;   period                ; period of fundamental in time steps
;   linear = linear       ; set to include a linear trend term basis
;   constant=constant     ; set to include a constant term in basis
;   delta_t = delta_t     ; time shift in basis -- default = 0.0
;
; OUTPUTS:
;   basis
;   coeff       ; coefficients of fit
;   
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;   
; HISTORY
;   6/18/2012 Initial Version prepared for NCDC
;
; USAGE
;   create_harmonic_basis,4,300,12,basis,linear = 1,constant=1,delta_t = 0.0
;
;@*****

	num_vectors = num_harmonics*2

	if keyword_set(linear) then begin
		num_vectors = num_vectors+1
	endif

	if keyword_set(constant) then begin
		num_vectors = num_vectors+1
	endif

  if keyword_set(delta_t) eq 0 then begin
    delta_t = 0.0
  endif

	basis = fltarr(num_vectors,length)


	t = findgen(length)
	for harmonic = 1,num_harmonics do begin
		omega = harmonic * !pi*2.0/(period)
		basis[(harmonic-1)*2,*] = cos((t+delta_t)*omega)
		basis[1+(harmonic-1)*2,*] = sin((t+delta_t)*omega)
	endfor

	vec_num = 2*num_harmonics

	if keyword_set(linear) then begin
		basis[vec_num,*] = findgen(length)/120.0  ; results end up in K/decade
		basis[vec_num,*] = basis[vec_num,*]-mean(basis[vec_num,*])
		vec_num = vec_num+1
	endif
	if keyword_set(constant) then begin
		basis[vec_num,*] = replicate(1.0,length)
		vec_num = vec_num+1
	endif
end

