function latitude_wts,num_lats,lat_range,no_cos=no_cos
;
;@***h* MLT_RSS/latitude_wts.pro
;
; NAME
;   latitude_wts.pro
;
; PURPOSE
;   computes latitude weights for data gridded on an equally spaced latitude grid for
;   a given latitude range.  Grids are assumed to start at 90S and continue to 90N. 
;
; DESCRIPTION
;   It is assumed that the latitude grid contains num_lats grid cells of equal width in
;   latitude.  A fine grid with 100*num_lats points is constructed.
;   This fine grid is set to 1.0 between lat_range[0] and lat_range[1] and used to calculate
;   the weight in the original grid.  By default, each latitude grid cell is futher weighted by
;   cosine(latitude) to account for the different areas of different latitude bands.  Setting the
;   no_cos flag to one removes the cosine(latitude) weighting.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   lat_wts = latitude_weights(72,[-60.0,60.0],/no_cos)
;
;@*****
 
	lat_wts = fltarr(num_lats)
	lat_wts_temp = fltarr(num_lats * 100)
	lats_temp = (0.5+indgen(num_lats*100))*(180/(num_lats*100.0)) - 90.0
	lat_wts_temp[where((lats_temp ge lat_range[0]) and (lats_temp le lat_range[1]))] = 1.0
	lat_wts = rebin(lat_wts_temp,num_lats)

	if keyword_set(no_cos) then begin
		return,lat_wts
	endif else begin
		lat_centers =(indgen(num_lats)+0.5) * (180.0/num_lats) - 90.0
		cos_wt = cos(!pi*lat_centers/180.0)
		lat_wts = lat_wts*cos_wt
		return,lat_wts
	endelse
end