pro log,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,lun = lun, quiet = quiet

;@***h* MLT_RSS/log.pro
;
; NAME
;   log.pro
;
; PURPOSE
;   writes up to 12 strings to a line in a log file, separated by spaces 
;
; DESCRIPTION
;   Test to see if there is a log file open. If lun is passed, use that unit,
;   otherwise look for a unit number in !log_lun.  If neither exists, then we assume log file
;   is not open (set lu = -99).  If lu is greater than zero, test to make sure the 
;   file is open for writing.
;
;   the message is written to the screen whether or not the log file is open, unless quiet is set.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2012 Initial Version prepared for NCDC
;
; USAGE
;   log,'Info: AMSU Only V3.3 log file',/quiet,lun=23
;
;@***** MLT_RSS/log.pro



	n = n_elements(lun)
	if n gt 0 then begin
		lu = fix(lun[0])
	endif else begin
		defsysv,'!log_lun',exists = exists
		if exists eq 1 then begin
			lu = !log_lun
		endif else begin
			lu = -99
		endelse
	endelse

	if lu ge 0 then begin
		stat = fstat(lu)
		if ((stat.open ne 0) and (stat.write ne 0)) then begin
				log_open = 1
		endif else begin
				log_open = 0
		endelse
	endif else begin
		log_open = 0
	endelse

	; assemble a string to write
	str = ''
	if n_elements(a1) ge 1 then str 	= str + strjoin(string(a1),' ',/single)+' '
	if n_elements(a2) ge 1 then str 	= str + strjoin(string(a2),' ',/single)+' '
	if n_elements(a3) ge 1 then str 	= str + strjoin(string(a3),' ',/single)+' '
	if n_elements(a4) ge 1 then str 	= str + strjoin(string(a4),' ',/single)+' '
	if n_elements(a5) ge 1 then str 	= str + strjoin(string(a5),' ',/single)+' '
	if n_elements(a6) ge 1 then str 	= str + strjoin(string(a6),' ',/single)+' '
	if n_elements(a7) ge 1 then str 	= str + strjoin(string(a7),' ',/single)+' '
	if n_elements(a8) ge 1 then str 	= str + strjoin(string(a8),' ',/single)+' '
	if n_elements(a9) ge 1 then str 	= str + strjoin(string(a9),' ',/single)+' '
	if n_elements(a10) ge 1 then str 	= str + strjoin(string(a10),' ',/single)+' '
	if n_elements(a11) ge 1 then str 	= str + strjoin(string(a11),' ',/single)+' '
	if n_elements(a12) ge 1 then str 	= str + strjoin(string(a12),' ',/single)+' '

	if keyword_set(quiet) eq 0 then print,str
	if log_open then printf,!log_lun,str
end