pro read_L2A_inventory,sat_num,channel,exists,start_scan,end_scan,start_rd_2000,end_rd_2000,eqx_rd_2000
;@***h* MLT_RSS/read_L2A_inventory.pro

; NAME
;   read_L2A_inventory.pro
;
; PURPOSE
;   Reads RSS L2A inventory file.
;
; DESCRIPTION
;   Reads RSS L2A inventory file.
;   
; INPUTS
;   sat_num
;   channel
;   
; OUTPUTS:
;   exists          ;intarr(80000)  1 if orbit exists, 0 if does not exist
;   start_scan      ;intarr(80000)  first scan number in oribt 
;   end_scan        ;intarr(80000)  last scan number in orbit
;   start_rd_2000   ;dlbarr(80000)  begin date, in rd_2000 format
;   end_rd_2000     ;dblarr(80000)  end date, in rd_2000 format
;   eqx_rd_2000     ;dblarr(80000)  scending node crossing time, in rd_2000 format
;   
; FILE READ
;   L2A inventory file in text form
;   e.g. AQUA_Channel_05_L2A.inv.txt
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;   ;
; HISTORY
;   10/28/2012 Initial Version prepared for NCDC
;
; USAGE
;   read_L2A_inventory,10,5,exists,start_scan,end_scan,start_rd_2000,end_rd_2000,eqx_rd_2000
;
;@*****

	exists = intarr(80000)
	start_scan = intarr(80000)
	end_scan = intarr(80000)
	start_rd_2000 = replicate(!values.d_nan,80000)
	end_rd_2000 = replicate(!values.d_nan,80000)
	eqx_rd_2000 = replicate(!values.d_nan,80000)

	name = sat_name(sat_num,/fill)
	chan_str = string(channel,format='(i2.2)')
	get_AMSU_File_Locations,AMSU_L2A_Path = AMSU_L2A_Path
	get_AMSU_File_Locations,MSU_L2A_Path = MSU_L2A_Path
	if sat_num gt 9 then begin
		inv_file = AMSU_L2A_Path+name+'\'+name+'_channel_'+chan_str+'_L2A.inv.txt'
		print,'Reading ',inv_file
		on_ioerror,no_file
		close,3
		openr,3,inv_file


		orbit = 0L
		st_scan = 0
		ed_scan = 0
		st_rd_2000 = 0.0D0
		ed_rd_2000 = 0.0D0
		while not(eof(3)) do begin
			readf,3,orbit,st_scan,st_rd_2000,ed_scan,ed_rd_2000,eq_rd_2000, $
				format = '(i8,i4,f15.7,i4,f15.7,f15.7)'

			exists(orbit) = 1
			start_scan(orbit) = st_scan
			end_scan(orbit) = ed_scan
			start_Rd_2000(orbit) = st_Rd_2000
			end_rd_2000(orbit) = ed_rd_2000
			eqx_rd_2000(orbit) = eq_rd_2000
		endwhile
        close,3
		no_file:
	endif

	if sat_num le 9 then begin
		inv_file = MSU_L2A_Path+name+'\Mo1995_V02_1\'+name+'_L1a.inv.txt'

		print,'Reading ',inv_file
		on_ioerror,no_file2
		close,3
		openr,3,inv_file


		orbit = 0L
		st_scan = 0
		ed_scan = 0
		st_rd_2000 = 0.0D0
		ed_rd_2000 = 0.0D0
		while not(eof(3)) do begin
			readf,3,orbit,st_scan,st_rd_2000,ed_scan,ed_rd_2000,eq_rd_2000, $
				format = '(i8,i4,f15.7,i4,f15.7,f15.7)'

			exists(orbit) = 1
			start_scan(orbit) = st_scan
			end_scan(orbit) = ed_scan
			start_Rd_2000(orbit) = st_Rd_2000
			end_rd_2000(orbit) = ed_rd_2000
			eqx_rd_2000(orbit) = eq_rd_2000
		endwhile
        close,3
		no_file2:
	endif
    close,3
end
