
pro set_up_ps,file=file,x_size=x_size,y_size=y_size,xsize=xsize,ysize=ysize,cmyk=cmyk

;@***h* MLT_RSS/set_up_ps.pro

; NAME
;   set_up_ps.pro
;
; PURPOSE
;   Sets up a postscript file for output from IDL
;
; DESCRIPTION
;    Sets up a postscript file for output from IDL.  Default size (7.0 by 3.5 inches) can
;    be overridden by passing keywords (with two alternate spellings!)
; 
; INPUTS
;   file    filename of file to open.  Should end in .ps
;   x_size  size of ps plot in inches in x direction
;   xsize   size of ps plot in inches in x direction
;   y_size  size of ps plot in inches in y direction
;   ysize   size of ps plot in inches in y direction   
;   cmyk    flag.  set this flag to force output to be in cmyk colorspace
;   
;   note:  two spellings of xsize(x_size) and ysize(y_size) are allowed to reduce stupid coding errors.
;          if both are set (a REALLY stupid coding error), then the one with the underscore (eg x_size) takes precedent.
;          
; OUTPUT
;   none
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/21/2019 Initial Version prepared for NCDC
;
; USAGE
;   set_up_ps,file=file,x_size=x_size,y_size=y_size,xsize=xsize,ysize=ysize,cmyk=cmyk
;   
;@*****
	if not keyword_Set(x_size) then begin
		if not keyword_set(xsize) then begin
			size_x = 7.0
		endif else begin
			size_x = xsize
		endelse
	endif else begin
		size_x = x_size
	endelse

	if not keyword_Set(y_size) then begin
		if not keyword_set(ysize) then begin
			size_y = 10.5
		endif else begin
			size_y = ysize
		endelse

	endif else begin
		size_y = y_size
	endelse

  ;make sure they are floats
  
	size_x = size_x*1.0
	size_y = size_y*1.0

	set_plot,'ps'
	if size_x gt 8.0 then begin
		device,/landscape
	endif else begin
		device,/portrait
	endelse
	
	; convert to centimeters
	
	device,xsize= size_x*2.54
	device,ysize = size_y*2.54
	device,xoffset = 0.5*2.54
	
	;locate plot on page
	if size_x le 8.0 then begin
		device,yoffset = (10.5 - size_y)*2.54
	endif else begin
		device,yoffset = 10.5*2.54
	endelse
	device,/color
	if keyword_set(cmyk) then device,/cmyk
	device,bits_per_pixel = 8
	ok = check_path(file,/create)  ; create the entire path of necessary
	device,file = file
end


