pro smooth_offsets,offsets,offsets_smooth

;@***h* MLT_RSS/smooth_offsets.pro
;
; NAME
;   smooth_offsets.pro
;
; PURPOSE
;   smooths AMSU zonal intersatellite offsets in the north/south direction. 
;
; DESCRIPTION
;   smooths AMSU zonal intersatellite offsets in the north/south direction.  A simple, "boxcar" smooth
;   is used of width 5 2.5 degree bins
; INPUTS
;   offsets         (num_lats,num_sats) float32 array of offsets
; OUTPUTS
;   offsets_smooth  (num_lats,num_sats) float32 array of smoothed offsets
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   2/29/2012 Initial Version prepared for NCDC
; 
; USAGE
;   smooth_offsets,offsets,offsets_smooth
;
;@*****
sz = size(offsets,/dimensions)
num_sats = sz[1]

offsets_smooth = replicate(!values.f_nan,sz[0],sz[1])

for sat = 0,num_sats-1 do begin
  y = offsets[*,sat]
  y_smooth = smooth(y,5,/nan,/edge_truncate)
  bad = where(finite(y) eq 0,ct)
  if ct gt 0 then y_smooth[bad] = !values.f_nan
  offsets_smooth[*,sat] = y_smooth
endfor

end