;@***h* MLT_RSS/write_msu_amsu_merged_Tbs_144_72_netcdf4_for_NCDC.pro
;
; NAME
;   write_msu_amsu_merged_Tbs_144_72_netcdf4_for_NCDC.pro
;
; PURPOSE
;   Writes MSU/AMSU data to netcdf4 file with NCDC-approved metadata.
;
; DESCRIPTION
;   Writes MSU/AMSU data to netcdf4 file with NCDC-approved metadata.
;   
; INPUTS:
;   Tbs,                          $           ; gridded brightness temperatures
;   channel_str = channel_str,    $           ; string containing channel name ('TLT, etc)
;   offsets = offsets,            $           ; offsets used in intratype merge
;   target_factors = target_factors,  $       ; target factors used in intratype merge
;   tb_Factors = tb_factors,      $           ; scene temepratuer factors used in intratype merge
;   satellites_used = satellites_used,  $     ; array of satellites used
;   date_L2C_processed = date_L2C_processed,  $    ; array of processing dates for L2C files
;   global_Attributes = global_attributes,    $    ; new global arrtibutes
;   msu_global_Attributes=msu_global_Attributes,   $  ; msu-only global atrributes
;   amsu_global_Attributes=amsu_global_Attributes, $  ; amsu_only global attributes
;   msu_amsu_diff=msu_amsu_diff, $                    ; 144x72x12 array of msu amsu differences -- adjustments have been applied to AMSU data
;   nc_file = nc_file,                        $    ; name of netdcf4 file to write
;   data_Type = data_type,  ;anom or avrg     $    ; string containing the data type ("anom" for anomaly.  "avrg" from mean
;   start_year=start_year                     $    ; first year of dataset
;   start_month=start_month                   $    ; first month of dataset
;   end_year = end_year                       $    ; last year of dataset
;   end_month=end_month,                      $    ; last month of dataset
;   first_valid_month=first_valid_month       $    ; index of first month with valid data (1 means Jan 1978, etc
;   last_valid_month = last_valid_month       $    ; index of last month with valid data (1 means Jan 1978, etc
;
; FILES WRITTEN
;   netcdf4 file containing msu amsu merged data
;   e.g. uat4_tb_v03r03_anom_chTLT_197812_201212.nc

; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.;
; HISTORY
;   1/14/2013 Initial Version prepared for NCDC
;
; USAGE
; 
; write_msu_amsu_merged_Tbs_144_72_netcdf4_for_NCDC,  Tbs = Tbsx,Clim_Tbs=Clim_Tbsx,      $
;                          channel_str=channel_str,      $
;                          offsets = offsets,          $
;                          target_factors = target_factors,  $
;                          tb_Factors = tb_factors,      $
;                          satellites_used = satellites_used,  $
;                          date_L2C_processed = date_L2C_processed,  $
;                          global_Attributes = global_attributes,    $
;                          msu_global_Attributes=msu_global_Attributes,   $
;                          amsu_global_Attributes=amsu_global_Attributes, $
;                          msu_amsu_diff=msu_amsu_diff, $
;                          nc_file = nc_file,nc4=nc4, data_Type = data_type,  $ ;anom or avrg
;                          start_year=start_year,start_month=start_month,end_year = end_year,end_month=end_month, $
;                          first_valid_month=first_valid_month,last_valid_month = last_valid_month
;   
;
;
;@*****


pro write_msu_amsu_merged_Tbs_144_72_netcdf4_for_NCDC,	Tbs = Tbsx,Clim_Tbs=Clim_Tbsx,			$
													channel_str=channel_str,			$
													offsets = offsets,					$
													target_factors = target_factors,	$
													tb_Factors = tb_factors,			$
													satellites_used = satellites_used,	$
													date_L2C_processed = date_L2C_processed,	$
													global_Attributes = global_attributes,		$
													msu_global_Attributes=msu_global_Attributes,   $
													amsu_global_Attributes=amsu_global_Attributes, $
													msu_amsu_diff=msu_amsu_diff, $
													nc_file = nc_file,nc4=nc4, data_Type = data_type,  $ ;anom or avrg
													start_year=start_year,start_month=start_month,end_year = end_year,end_month=end_month, $
												  first_valid_month=first_valid_month,last_valid_month = last_valid_month
  target_factors[0:2,*] = !values.f_nan
  target_factors[69:71,*] = !values.f_nan
  tb_factors[0:2,*] = !values.f_nan
  tb_factors[69:71,*] = !values.f_nan
  
  
  case channel_str of
    'TLT':begin
            location = 'Lower Troposphere'
            location_ = 'lower_troposphere
            adjective = 'Lower Tropospheric'
            vertical_location = 'Troposphere'
            offsets[0:7,*] = !values.f_nan
            target_factors[0:7,*] = !values.f_nan
            tb_factors[0:7,*] = !values.f_nan
            
          end
    'TMT':begin
            location = 'Middle Troposphere'
            location_ = 'middle_troposphere'
            adjective = 'Middle Tropospheric'
            vertical_location = 'Troposphere'
            
          end
    'TTS':begin
            location = 'Near Tropopause'
            location_ = 'near_tropopause'
            adjective = 'Atmospheric'
            vertical_location = 'Troposphere, Stratosphere'
           
          end
    'TLS':begin
            location = 'Lower Stratosphere'
            location_ = 'lower_stratosphere'
            adjective = 'Lower Stratospheric'
            vertical_location = 'Stratosphere'
            
          end
  endcase
  
  start_year_str = string(start_year,format='(i4.4)')
  start_month_str = string(start_month,format='(i2.2)')
  end_year_str = string(end_year,format='(i4.4)')
  end_month_str = string(end_month,format='(i2.2)')
  
  days_in_month_arr = [['31','28','31','30','31','30','31','31','30','31','30','31'],['31','29','31','30','31','30','31','31','30','31','30','31']]
  leap = 0
  if end_year mod 4 eq 0 then leap = 1
  days_in_month_str = days_in_month_arr[end_month-1,leap]
  Tbs = Tbsx  ; no side effects
  Clim_Tbs = Clim_Tbsx
  
  ; rotate by 180 degrees
  
  Tbs[0:71,*,*]   = Tbsx[72:143,*,*]
  Tbs[72:143,*,*] = Tbsx[0:71,*,*]
  
  Clim_Tbs[0:71,*,*]   = Clim_Tbsx[72:143,*,*]
  Clim_Tbs[72:143,*,*] = Clim_Tbsx[0:71,*,*]
  
	fill_value = -9999.0
	sz = size(Tbs,/dimensions)
	num_lons = sz[0]
	num_lats = sz[1]
	num_months = sz[2]
	sz2 = size(satellites_used,/dimensions)
	num_sats = sz2[0]



	bad = where(finite(Tbs) eq 0,ct)
	if ct gt 0 then begin
		Tbs[bad] = fill_value
	endif
	Tbs = Tbs[*,*,0:last_valid_month]
	satellites_used = satellites_used[*,0:last_valid_month]
	bad = where(Clim_Tbs lt 100,ct)
	if ct gt 0 then begin
	 Clim_Tbs[bad] = fill_value
	endif

	todays_date = systime_iso8601_utc()

	get_AMSU_File_Locations,AMSU_L3_Path = AMSU_L3_Path
	;output_path = 		AMSU_L3_Path+'MSU_AMSU\'
	output_path = 'M:\MSU_TB_CDR\'

	if keyword_set(nc_file) eq 0 then begin
	  if keyword_set(nc4) then begin
	    nc_file_only = 'uat4_tb_v03r03_'+data_type+'_ch'+channel_str+'_'+start_year_str+start_month_str+'_'+end_year_str+end_month_str+'.nc'
      nc_file = output_path + nc_file_only
    endif else begin
      nc_file_only = 'uat4_tb_v03r03_'+data_type+'_ch'+channel_str+'_'+start_year_str+start_month_str+'_'+end_year_str+end_month_str+'.nc3.nc'
      nc_file = output_path + nc_file_only
    endelse
	endif

  path_ok = check_path(nc_file,/create)
	print,nc_file
  if path_ok eq 1 then begin
    if keyword_set(nc4) then begin
     id = NCDF_CREATE(nc_file, /CLOBBER,/NETCDF4_FORMAT) ; Create the netCDF file.
    endif else begin
     id = NCDF_CREATE(nc_file, /CLOBBER) ; Create the netCDF file.
    endelse
  
    ;write global attibutes
    
    NCDF_ATTPUT,id,'Conventions','CF-1.6',/global
    NCDF_ATTPUT,id,'title','Monthly anomalies of brightness temperatures in the '+location+' on a 2.5 degree grid',/global
    NCDF_ATTPUT,id,'source','MSU and AMSU-A Level 1b',/global
    NCDF_ATTPUT,id,'references','doi:10.1175/2009JTECHA1237.1, doi:10.1175/2008JTECHA1176.1',/global
    NCDF_ATTPUT,id,'history','netcdf4 file converted from netcdf3 file written at Remote Sensing Systems',/global
    NCDF_ATTPUT,id,'comment','dataset padded with missing data from January 1978 to date of first valid data',/global  ; add something about missing data
    NCDF_ATTPUT,id,'Metadata_Conventions','CF-1.6, Unidata Dataset Discovery v1.0, NOAA CDR v1.0, GDS v2.0',/global
    NCDF_ATTPUT,id,'standard_name_vocabulary','CF Standard Name Table (v19, 22 March 2012)',/global
    
    NCDF_ATTPUT,id,'id',FILE_BASENAME(nc_file),/global
    NCDF_ATTPUT,id,'naming_authority','gov.noaa.ncdc',/global
    NCDF_ATTPUT,id,'date_created',todays_date,/global
    NCDF_ATTPUT,id,'license','No constraints on data access or use',/global
    
    if data_type eq 'anom' then NCDF_ATTPUT,id,'summary','Global monthly anomaly of '+adjective+' temperatures derived from microwave radiometers on NOAA and NASA polar orbiting satellites. The anomalies are deviations from 1979-1998 mean.  ',/global
    if data_type eq 'avrg' then NCDF_ATTPUT,id,'summary','Global monthly average of '+adjective+' temperatures derived from microwave radiometers on NOAA and NASA polar orbiting satellites. The anomalies are deviations from 1979-1998 mean.  ',/global

    if data_type eq 'avrg' then NCDF_ATTPUT,id,'keywords','Atmosphere > Atmospheric Temperature > Air Temperature, Vertical Location > '+vertical_location,/global
    if data_type eq 'anom' then NCDF_ATTPUT,id,'keywords','Atmosphere > Atmospheric Temperature > Temperature Anomalies, Vertical Location > '+vertical_location,/global
    
    NCDF_ATTPUT,id,'keywords_vocabulary','NASA Global Change Master Directory (GCMD) Earth Science keywords, Version 6.0',/global
    NCDF_ATTPUT,id,'cdm_data_type','Grid',/global
    NCDF_ATTPUT,id,'project','RSS MSU/AMSU brightness temperature',/global
    NCDF_ATTPUT,id,'processing_level','NASA Level 3',/global
    NCDF_ATTPUT,id,'creator_name','Carl A. Mears',/global
    NCDF_ATTPUT,id,'creator_url','http://www.remss.com/',/global
    NCDF_ATTPUT,id,'creator_email','mears@remss.com, support@remss.com',/global
    NCDF_ATTPUT,id,'institution','Remote Sensing Systems',/global
    NCDF_ATTPUT,id,'geospatial_lat_min',-90.0,/global,/float
    NCDF_ATTPUT,id,'geospatial_lat_max',90.0,/global,/float
    NCDF_ATTPUT,id,'geospatial_lon_min',-180.0,/global,/float
    NCDF_ATTPUT,id,'geospatial_lon_max',180.0,/global,/float
    NCDF_ATTPUT,id,'geospatial_lat_units','degrees_north',/global
    NCDF_ATTPUT,id,'geospatial_lon_units','degrees_east',/global
    NCDF_ATTPUT,id,'geospatial_lat_resolution',2.5,/global,/float
    NCDF_ATTPUT,id,'geospatial_lon_resolution',2.5,/global,/float
    time_coverage_start_str = start_year_str+'-'+start_month_str+'-01T00:00:00Z'
    time_coverage_end_str = end_year_str+'-'+end_month_str+'-'+days_in_month_str+'T23:59:59Z'
    
    
    NCDF_ATTPUT,id,'time_coverage_start',time_coverage_start_str,/global  ; real date
    NCDF_ATTPUT,id,'time_coverage_end',time_coverage_end_str,/global    ; 
    
    duration = 1 + last_valid_month-first_valid_month
    num_yrs_duration = duration/12
    num_months_duration = duration mod 12
    if num_months_duration eq 0 then begin
      duration_str = 'P'+string(num_yrs_duration,format='(i2.2,"Y")')
    endif
    if ((num_months_duration gt 0) and (num_months_duration lt 10)) then begin
      duration_str = 'P'+string(num_yrs_duration,num_months_duration,format='(i2.2,"Y",i1,"M")')
    endif
    if num_months_duration ge 10 then begin
       duration_str = 'P'+string(num_yrs_duration,num_months_duration,format='(i2.2,"Y",i2,"M")')
    endif
    NCDF_ATTPUT,id,'time_coverage_duration',duration_str,/global  ;P32Y6M 
    NCDF_ATTPUT,id,'time_coverage_resolution','P1M',/global
    NCDF_ATTPUT,id,'contributor_name','Carl A. Mears,Carl A. Mears,Marty Brewer,Carl A. Mears,Carl A. Mears,Carl A. Mears, Marty Brewer',/global
    NCDF_ATTPUT,id,'contributor_role','principalInvestigator, pointOfContact, pointOfContact, author, originator, processor, processor',/global
    NCDF_ATTPUT,id,'acknowledgement','This project was supported in part by a grant from the NOAA Scientific Data Stewardship (SDS) Program.  Previous work in this area was supported by the NOAA Office of Global Programs',/global
    NCDF_ATTPUT,id,'cdr_program','NOAA Climate Data Record Program for satellites',/global
    NCDF_ATTPUT,id,'cdr_variable','brightness_temperature_anomaly_'+location_,/global 
    NCDF_ATTPUT,id,'software_version_id','gov.noaa.ncdc:/svn-repos/rsad/CDR/RSS-MSU-MLT/tags/V03r03/',/global
    NCDF_ATTPUT,id,'Metadata_Link','gov.noaa.ncdc:C00761',/global
    NCDF_ATTPUT,id,'product_version','V3.3',/global
    NCDF_ATTPUT,id,'platform','TIROS-N,NOAA-06,NOAA-07,NOAA-08,NOAA-09,NOAA-10,NOAA-11,NOAA-12,NOAA-14,NOAA-15,NOAA-16,NOAA-18,METOP-A,AQUA',/global  
    NCDF_ATTPUT,id,'sensor','MSU > Microwave  Sounding Unit,AMSU-A > Advanced Microwave Sounding Unit-A',/global
    NCDF_ATTPUT,id,'spatial_resolution','2.5 degree',/global
    
    ; create satellite name string
    sat_name_arr = strarr(num_sats)
    for satnum = 1,num_sats do begin
      sat_name_arr[satnum-1] = sat_name(satnum)
    endfor
    
  	; define dimensions for variables
  
  	x_dim      = NCDF_DIMDEF(id, 'longitude', num_lons) 		; Define the X dimension.
  	y_dim      = NCDF_DIMDEF(id, 'latitude', num_lats) 		    ; Define the Y dimension.
  	time_dim   = NCDF_DIMDEF(id, 'time', last_valid_month+1) 			  ; Define the month dimension.
  	clim_time_dim  = NCDF_DIMDEF(id, 'climatology_time', 12)					; Define the month dimension for one year
  	sat_dim    = NCDF_DIMDEF(id, 'satnum', num_sats) 			    ; Define the satellite dimension.
  	bnds_dim   = NCDF_DIMDEF(id, 'bnds', 2)           ; Define the satellite dimension.
  
  	; define the data variables
  
  	lon_id   			= NCDF_VARDEF(id, 'longitude', 			[x_dim], /FLOAT)
  		NCDF_ATTPUT,id,lon_id,'valid_min',-180.0,/float
      NCDF_ATTPUT,id,lon_id,'valid_max',180.0,/float
      
      NCDF_ATTPUT,id,lon_id,'units','degrees_east'
      NCDF_ATTPUT,id,lon_id,'axis','X'    
      NCDF_ATTPUT,id,lon_id,'long_name','longitude'
      NCDF_ATTPUT,id,lon_id,'standard_name','longitude'
      NCDF_ATTPUT,id,lon_id,'bounds','longitude_bounds'
      NCDF_ATTPUT,id,lon_id,'_CoordinateAxisType','Lon'
      NCDF_ATTPUT,id,lon_id,'coordinate_defines','center'

  		
  	lat_id   			= NCDF_VARDEF(id, 'latitude', 			[y_dim], /FLOAT)
  	  NCDF_ATTPUT,id,lat_id,'units','degrees_north'
  	  NCDF_ATTPUT,id,lat_id,'axis','Y'
      NCDF_ATTPUT,id,lat_id,'long_name','latitude'
      NCDF_ATTPUT,id,lat_id,'standard_name','latitude'
      NCDF_ATTPUT,id,lat_id,'bounds','latitude_bounds' 
      NCDF_ATTPUT,id,lat_id,'valid_min',-90.0,/float
      NCDF_ATTPUT,id,lat_id,'valid_max', 90.0,/float
  		NCDF_ATTPUT,id,lat_id,'_CoordinateAxisType','Lat'
  		NCDF_ATTPUT,id,lat_id,'coordinate_defines','center'

  	time_id = NCDF_VARDEF(id, 'time', 			[time_dim], /FLOAT)
  	  NCDF_ATTPUT,id,time_id,'units','days since 1978-1-1 00:00:00'
  	  NCDF_ATTPUT,id,time_id,'axis','T'
  		NCDF_ATTPUT,id,time_id,'long_name','time'
      NCDF_ATTPUT,id,time_id,'standard_name','time'
      NCDF_ATTPUT,id,time_id,'bounds','time_bounds'
  		NCDF_ATTPUT,id,time_id,'_CoordinateAxisType','Time'
  		NCDF_ATTPUT,id,time_id,'valid_min',0.0, /float
      NCDF_ATTPUT,id,time_id,'valid_max',20000.0, /float
  		
  	clim_time_id = NCDF_VARDEF(id, 'climatology_time',       [clim_time_dim], /FLOAT)
  	  NCDF_ATTPUT,id,clim_time_id,'units','days since 1979-1-1'
      NCDF_ATTPUT,id,clim_time_id,'axis','T'
      NCDF_ATTPUT,id,clim_time_id,'long_name','climatology_time'
      NCDF_ATTPUT,id,clim_time_id,'standard_name','time'
      NCDF_ATTPUT,id,clim_time_id,'climatology','climatology_time_bounds'      
      ;NCDF_ATTPUT,id,clim_time_id,'bounds','climatology_time_bounds'
      NCDF_ATTPUT,id,clim_time_id,'_CoordinateAxisType','Time'
      NCDF_ATTPUT,id,clim_time_id,'valid_min',0.0, /float
      NCDF_ATTPUT,id,clim_time_id,'valid_max',370.0, /float
    
    if keyword_set(nc4) then begin
      sat_name_id = NCDF_VARDEF(id, 'platform_name',[sat_dim], /STRING)
      NCDF_ATTPUT,id,sat_name_id,'long_name','platform_name'
    endif
    
    lon_bnds_id = NCDF_VARDEF(id, 'longitude_bounds',[bnds_dim,x_dim], /FLOAT)
      NCDF_ATTPUT,id,lon_bnds_id,'units','degrees_east'  
      NCDF_ATTPUT,id,lon_bnds_id,'long_name','longitude_bounds'
      NCDF_ATTPUT,id,lon_bnds_id,'valid_min',-180.0,/float
      NCDF_ATTPUT,id,lon_bnds_id,'valid_max',180.0,/float
    
    lat_bnds_id = NCDF_VARDEF(id, 'latitude_bounds',[bnds_dim,y_dim], /FLOAT)
      NCDF_ATTPUT,id,lat_bnds_id,'units','degrees_north'
      NCDF_ATTPUT,id,lat_bnds_id,'long_name','latitude_bounds'
       NCDF_ATTPUT,id,lat_bnds_id,'valid_min',-90.0,/float
      NCDF_ATTPUT,id,lat_bnds_id,'valid_max', 90.0,/float
      
    time_bnds_id = NCDF_VARDEF(id, 'time_bounds',[bnds_dim,time_dim], /FLOAT)
      NCDF_ATTPUT,id,time_bnds_id,'units','days since 1978-1-1 00:00:00'
      NCDF_ATTPUT,id,time_bnds_id,'long_name','time_bounds
      NCDF_ATTPUT,id,time_bnds_id,'valid_min',0.0, /float
      NCDF_ATTPUT,id,time_bnds_id,'valid_max',20000.0, /float
      
    clim_time_bnds_id  = NCDF_VARDEF(id, 'climatology_time_bounds',[bnds_dim,clim_time_dim], /FLOAT)
      NCDF_ATTPUT,id,clim_time_bnds_id,'units','days since 1979-1-1 00:00:00'
      NCDF_ATTPUT,id,clim_time_bnds_id,'long_name','climatology_time_bounds
      NCDF_ATTPUT,id,clim_time_bnds_id,'valid_min',0.0, /float
      NCDF_ATTPUT,id,clim_time_bnds_id,'valid_max',20000.0, /float
    
    if data_type eq 'anom' then begin 
  	temperature_id  = NCDF_VARDEF(id, 'brightness_temperature_anomaly',[x_dim,y_dim,time_dim], /FLOAT)
  	  NCDF_ATTPUT,id,temperature_id,'_FillValue',fill_value
  	  NCDF_ATTPUT,id,temperature_id,'valid_min',-30.0,/float
      NCDF_ATTPUT,id,temperature_id,'valid_max',30.0,/float
      NCDF_ATTPUT,id,temperature_id,'coordinates','time latitude longitude'
      NCDF_ATTPUT,id,temperature_id,'units','K'
      NCDF_ATTPUT,id,temperature_id,'long_name','NOAA Climate Data Record of monthly anomaly for '+adjective+' microwave brightness temperature'
      NCDF_ATTPUT,id,temperature_id,'standard_name','brightness_temperature_anomaly'
  	  NCDF_ATTPUT,id,temperature_id,'cell_methods','time: mean within month'
  	  NCDF_ATTPUT,id,temperature_id,'comment','Reference Period is Jan 1979-December 2008'  
  	endif  
  	
  	if data_type eq 'avrg' then begin 
      temperature_id  = NCDF_VARDEF(id, 'brightness_temperature',[x_dim,y_dim,time_dim], /FLOAT)
      NCDF_ATTPUT,id,temperature_id,'_FillValue',fill_value
      NCDF_ATTPUT,id,temperature_id,'valid_min',150.0,/float
      NCDF_ATTPUT,id,temperature_id,'valid_max',350.0,/float
      NCDF_ATTPUT,id,temperature_id,'coordinates','time latitude longitude'
      NCDF_ATTPUT,id,temperature_id,'units','K'
      NCDF_ATTPUT,id,temperature_id,'long_name','NOAA Climate Data Record of monthly average for '+adjective+' microwave brightness temperature'
      NCDF_ATTPUT,id,temperature_id,'standard_name','brightness_temperature'
      NCDF_ATTPUT,id,temperature_id,'cell_methods','time: mean within month'
    endif 
    
    clim_id = NCDF_VARDEF(id, 'brightness_temperature_climatology',[x_dim,y_dim,clim_time_dim], /FLOAT)
      NCDF_ATTPUT,id,clim_id,'_FillValue',fill_value
      NCDF_ATTPUT,id,clim_id,'valid_min',170.0,/float
      NCDF_ATTPUT,id,clim_id,'valid_max',310.0,/float
      NCDF_ATTPUT,id,clim_id,'coordinates','climatology_time latitude longitude'
      NCDF_ATTPUT,id,clim_id,'units','K'
      NCDF_ATTPUT,id,clim_id,'long_name','NOAA Climate Data Record of monthly climatology for '+adjective+'  microwave brightness temperature'
      NCDF_ATTPUT,id,clim_id,'cell_methods','climatology_time: mean within month climatology_time: mean over years' 
      NCDF_ATTPUT,id,clim_id,'comment','Reference Period is Jan 1979-December 2008'	 
       
  	offset_id		= NCDF_VARDEF(id, 'offset_values',[y_dim,sat_dim],/FLOAT)
      NCDF_ATTPUT,id,offset_id,'long_name','satellite calibration offset for each satellite and latitude'
  		NCDF_ATTPUT,id,offset_id,'units','K'
  		NCDF_ATTPUT,id,offset_id,'valid_min',-10.0,/float
      NCDF_ATTPUT,id,offset_id,'valid_max',10.0,/float
  		NCDF_ATTPUT,id,offset_id,'comment','Variable (A) in referenced papers'
  		
  	target_factor_id= NCDF_VARDEF(id,'target_factor_values',[y_dim,sat_dim],/FLOAT)
   		NCDF_ATTPUT,id,target_factor_id,'long_name','satellite calibration target factor for each satellite and latitude'
      NCDF_ATTPUT,id,target_factor_id,'valid_min',-0.1,/float
      NCDF_ATTPUT,id,target_factor_id,'valid_max',0.1,/float
      NCDF_ATTPUT,id,target_factor_id,'comment','Variable (alpha) in referenced papers'
  		
  	tb_factor_id= NCDF_VARDEF(id,'tb_factor_values',[y_dim,sat_dim],/FLOAT)
      NCDF_ATTPUT,id,tb_factor_id,'long_name','satellite calibration tb factor for each satellite and latitude'
      NCDF_ATTPUT,id,tb_factor_id,'valid_min',-0.1,/float
      NCDF_ATTPUT,id,tb_factor_id,'valid_max',0.1,/float
      NCDF_ATTPUT,id,tb_factor_id,'comment','Variable (beta) in referenced papers'
  		
  	msu_amsu_diff_id = NCDF_VARDEF(id,'msu_amsu_offsets',[x_dim,y_dim,clim_time_dim], /FLOAT)
      NCDF_ATTPUT,id,msu_amsu_diff_id,'long_name','adjustments added to amsu brightness temperatures to match msu brightness temperatures for each location and month of year'
      NCDF_ATTPUT,id,msu_amsu_diff_id,'valid_min',-5.0,/float
      NCDF_ATTPUT,id,msu_amsu_diff_id,'valid_max',5.0,/float
  		NCDF_ATTPUT,id,msu_amsu_diff_id,'units','K'
  		
    months_used_id  = NCDF_VARDEF(id, 'satellites_used',[sat_dim,time_dim], /BYTE)
      NCDF_ATTPUT,id,months_used_id,'long_name','which satellites are used for merged product by month'
      NCDF_ATTPUT,id,months_used_id,'valid_min',0,/byte 
      NCDF_ATTPUT,id,months_used_id,'valid_max',1,/byte   
      NCDF_ATTPUT,id,months_used_id,'flag_values','0b,1b'
      NCDF_ATTPUT,id,months_used_id,'flag_meanings' ,'satellite_not_used, satellite_used'
   		
  
  	; switch to data mode and write data
  
  	NCDF_CONTROL, id, /ENDEF ; Put the file into data mode.
  
  	latitude   		= (indgen(num_lats)*2.5) -90.0+1.25
  	longitude  		= (1.25 +(indgen(num_lons) * 2.5)) - 180
  	
  	latitude_bnd  = fltarr(2,num_lats)
    latitude_bnd[0,*] = (indgen(num_lats)*2.5) -90.0
    latitude_bnd[1,*] = (indgen(num_lats)*2.5) -90.0+2.5
  	
  	longitude_bnd  	   = fltarr(2,num_lons)
  	longitude_bnd[0,*] = (indgen(num_lons) * 2.5) - 180
  	longitude_bnd[1,*] = (indgen(num_lons) * 2.5) - 180 + 2.5
  	
  	months      	= indgen(1+last_valid_month)
  	
  	days_1978 = fltarr(n_elements(months))
  	days_1978_bnds = fltarr(2,n_elements(months))
        jan_1_1978 = julday(1,1,1978,0,0,0.0)
        jan_1_1997 = julday(1,1,1978,0,0,0.0)
  
  	
  	for m = 0,n_elements(months) -1 do begin
  	  year = 1978 + m/12
  	  month = 1 + (m mod 12)
  	  year_next = year
  	  month_next = month + 1
  	  if month eq 13 then begin
  	   year_next = year_next+1
  	   month = 1
  	  endif
  	  jday1 = julday(month,1,year,0,0,0.0)
  	  jday2 = julday(month_next,1,year_next,0,0,0.0)
  	  days_1978[m] = 0.5*(jday1+jday2)- jan_1_1978
          days_1978_bnds[0,m] = jday1 - jan_1_1978
          days_1978_bnds[1,m] = jday2 - jan_1_1978 
      clim_time_1979 = days_1978[0:11]
      clim_1979_bnds = fltarr(2,12)
      clim_1979_bnds[0,*] = days_1978_bnds[0,0:11]
      clim_1979_bnds[1,*] = days_1978_bnds[1,228:239]
  	endfor
  	 
  	NCDF_VARPUT, id, lon_id, longitude
  	NCDF_VARPUT, id, lat_id, latitude
  	NCDF_VARPUT, id, lon_bnds_id, longitude_bnd
  	NCDF_VARPUT, id, lat_bnds_id, latitude_bnd
  	NCDF_VARPUT, id, time_id, days_1978
  	NCDF_VARPUT, id, time_bnds_id, days_1978_bnds
    NCDF_VARPUT, id, clim_time_id,clim_time_1979
    NCDF_VARPUT, id, clim_time_bnds_id,clim_1979_bnds
  	
  	if keyword_set(nc4) then NCDF_VARPUT, id, sat_name_id,sat_name_arr
  	NCDF_VARPUT, id, temperature_id, Tbs
  	NCDF_VARPUT, id, clim_id, Clim_Tbs
  	NCDF_VARPUT, id, months_used_id, satellites_used
  	NCDF_VARPUT, id, offset_id,offsets
  	NCDF_VARPUT, id, target_factor_id,target_factors
  	NCDF_VARPUT, id, tb_factor_id,tb_factors
  	NCDF_VARPUT, id, msu_amsu_diff_id,msu_amsu_diff
  
  
  	NCDF_CLOSE, id ; Close the NetCDF file.
 endif else begin
    log,'Error: Can not create path for netcdf file'
    log,'Info: file = ',nc_file
 endelse

end

